Package: memisc
Type: Package
Title: Management of Survey Data and Presentation of Analysis Results
Version: 0.99.31.8.3
Date: 2025-04-13
Authors@R: c(person(given = "Martin",
                    family = "Elff",
                    role = c("aut", "cre"),
                    email = "memisc@elff.eu"),
             person(given = "Christopher N.", family = "Lawrence", role = "ctb"),
             person(given = "Dave", family = "Atkins", role = "ctb"),
             person(given = "Jason W.", family = "Morgan", role = "ctb"),
             person(given = "Achim", family = "Zeileis", role = "ctb"),
             person(given = "Mael", family = "Astruc-Le Souder", role = "ctb"),
             person(given = "Kiril", family = "Mueller", role = "ctb"),
             person(given = "Pieter", family = "Schoonees", role = "ctb"))
Maintainer: Martin Elff <memisc@elff.eu>
Description: An infrastructure for the management of survey data including
        value labels, definable missing values, recoding of variables,
        production of code books, and import of (subsets of) 'SPSS' and
        'Stata' files is provided. Further, the package allows to produce
        tables and data frames of arbitrary descriptive statistics and
        (almost) publication-ready tables of regression model
        estimates, which can be exported to 'LaTeX' and HTML.
License: GPL-2 | GPL-3
LazyLoad: Yes
Depends: R (>= 3.3.0), lattice, stats, methods, utils, MASS
Suggests: splines, knitr, rmarkdown, sandwich
Enhances: AER, car, eha, lme4, ordinal, simex, tibble, haven
Imports: grid, data.table, yaml, jsonlite
VignetteBuilder: knitr
URL: https://melff.github.io/memisc/,https://github.com/melff/memisc/
BugReports: https://github.com/melff/memisc/issues
NeedsCompilation: yes
Packaged: 2025-04-13 19:03:31 UTC; elff
Author: Martin Elff [aut, cre],
  Christopher N. Lawrence [ctb],
  Dave Atkins [ctb],
  Jason W. Morgan [ctb],
  Achim Zeileis [ctb],
  Mael Astruc-Le Souder [ctb],
  Kiril Mueller [ctb],
  Pieter Schoonees [ctb]
Repository: CRAN
Date/Publication: 2025-04-14 07:50:10 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 15:47:10 UTC; unix
Archs: memisc.so.dSYM

Package: memisc
Type: Package
Title: Management of Survey Data and Presentation of Analysis Results
Version: 0.99.31.8
Date: 2024-09-28
Authors@R: c(person(given = "Martin",
                    family = "Elff",
                    role = c("aut", "cre"),
                    email = "memisc@elff.eu"),
             person(given = "Christopher N.", family = "Lawrence", role = "ctb"),
             person(given = "Dave", family = "Atkins", role = "ctb"),
             person(given = "Jason W.", family = "Morgan", role = "ctb"),
             person(given = "Achim", family = "Zeileis", role = "ctb"),
             person(given = "Mael", family = "Astruc-Le Souder", role = "ctb"),
             person(given = "Kiril", family = "Mueller", role = "ctb"),
             person(given = "Pieter", family = "Schoonees", role = "ctb"))
Maintainer: Martin Elff <memisc@elff.eu>
Description: An infrastructure for the management of survey data including
        value labels, definable missing values, recoding of variables,
        production of code books, and import of (subsets of) 'SPSS' and
        'Stata' files is provided. Further, the package allows to produce
        tables and data frames of arbitrary descriptive statistics and
        (almost) publication-ready tables of regression model
        estimates, which can be exported to 'LaTeX' and HTML.
License: GPL-2 | GPL-3
LazyLoad: Yes
Depends: R (>= 3.3.0), lattice, stats, methods, utils, MASS
Suggests: splines, knitr, rmarkdown, sandwich
Enhances: AER, car, eha, lme4, ordinal, simex, tibble, haven
Imports: grid, data.table, yaml, jsonlite
VignetteBuilder: knitr
URL: https://melff.github.io/memisc/,https://github.com/melff/memisc/
BugReports: https://github.com/melff/memisc/issues
NeedsCompilation: yes
Packaged: 2024-09-28 19:26:25 UTC; elff
Author: Martin Elff [aut, cre],
  Christopher N. Lawrence [ctb],
  Dave Atkins [ctb],
  Jason W. Morgan [ctb],
  Achim Zeileis [ctb],
  Mael Astruc-Le Souder [ctb],
  Kiril Mueller [ctb],
  Pieter Schoonees [ctb]
Repository: CRAN
Date/Publication: 2024-09-28 20:30:02 UTC

useDynLib(memisc)
import(stats,grid,lattice,methods,MASS)
importFrom(utils,
           ls.str,
           str,
           toLatex,
           head,
           tail,
           capture.output,
           getFromNamespace,
           View,
           browseURL,
           getS3method,
           localeToCharset)
export(
      contr,
      contr.treatment,
      contr.sum,
      cases,
      colrename,
      dimrename,
      mtable,
      write.mtable,
      getSummary,
      getSummary.glm,
      getSummary.lm,
      getSummary.polr,
      getSummary.clm,
      getSummary.simex,
      getSummary.coxph,
      getSummary.survreg,
      getSummary.aftreg,
      getSummary.phreg,
      getSummary.weibreg,
      getSummary.merMod,
      getSummary_expcoef,
      getSummary_expcoef.default,
      getSummary.ivreg,
      getSummary.tobit,
      getSummary.hurdle,
      getSummary.zeroinfl,
      getSummary.betareg,
      getSummary.multinom,
      applyTemplate,
      #get.substr,
      getSummaryTemplate,
      setSummaryTemplate,
      getCoefTemplate,
      setCoefTemplate,
      recode,
      relabel,
      relabel4,
      rename,
      rowrename,
      genTable,
      Aggregate,
      #wtable,
      Table,
      Descriptives,
      percent,
      percent.default,
      percent.logical,
      By,
      Sapply,
      Lapply,
      sort.data.frame,
      sort.data.set,
      to.data.frame,
      as.array,
      as.item,
      labels,
      "labels<-",
      value.filter,
      valid.values,
      missing.values,
      valid.range,
      "valid.values<-",
      "valid.range<-",
      "missing.values<-",
      measurement,
      "measurement<-",
      include.missings,
      annotation,
      "annotation<-",
      description,
      "description<-",
      wording,
      "wording<-",
      is.missing,
      is.valid,
      nvalid,
      is.nominal,
      is.ordinal,
      is.interval,
      is.ratio,
      as.nominal,
      as.ordinal,
      as.interval,
      as.ratio,
      contrasts,
      "contrasts<-",
      data.set,
      as.data.set,
      is.data.set,
      retain,
      foreach,
      xapply,
      syms,
      as.symbols,
      Substitute,
      collect,
      fapply,
      "%nin%",
      codebook,
      wild.codes,
      spss.fixed.file,
      spss.portable.file,
      spss.system.file,
      Stata.file,
      query,
      relabel4,
      Iconv,
      charTrans,
      Write,
      mtable_format_print,
      mtable_format_latex,
      mtable_format_html,
      mtable_format_delim,
      show_html,
      write_html,
      format_html,
      dsView,
      summaryTemplate,
      percentages
      )
S3method(getSummary,aftreg)
S3method(getSummary,betareg)
S3method(getSummary,clm)
S3method(getSummary,coxph)
S3method(getSummary,glm)
S3method(getSummary,hurdle)
S3method(getSummary,zeroinfl)
S3method(getSummary,ivreg)
S3method(getSummary,lm)
S3method(getSummary,merMod)
S3method(getSummary,multinom)
S3method(getSummary,phreg)
S3method(getSummary,polr)
S3method(getSummary,simex)
S3method(getSummary,survreg)
S3method(getSummary,tobit)
S3method(getSummary,weibreg)
S3method(getSummary_expcoef,default)
S3method(getSummary_expcoef,glm)
S3method(getSummary,clm)
S3method(getSummary,clmm)
S3method(sort,data.frame)
S3method(sort,data.set)
S3method(print,memisc_mtable)
S3method(format,memisc_mtable)
S3method(dim,memisc_mtable)
S3method(dimnames,memisc_mtable)
S3method("[",memisc_mtable)
S3method(c,memisc_mtable)
S3method(toLatex,default)
S3method(toLatex,matrix)
S3method(toLatex,memisc_mtable)
S3method(toLatex,ftable)

S3method(relabel,default)
S3method(relabel,factor)
S3method(relabel,memisc_mtable)
S3method(relabel,table)
S3method(relabel,ftable)

S3method(reorder,matrix)
S3method(reorder,array)
S3method(collect,default)
S3method(collect,matrix)
S3method(collect,array)
S3method(collect,table)
S3method(collect,data.frame)
S3method(collect,data.set)
S3method(collect,by)
S3method(fapply,default)
S3method(percent,default)
S3method(percent,logical)
S3method(By,default)
S3method(By,data.frame)

exportClasses(
  atomic,
  value.labels,
  value.filter,
  valid.values,
  valid.range,
  missing.values,
  annotation,
  item,
  integer.item,
  double.item,
  character.item,
  numeric.item,
  item.vector,
  named.list,
  item.list,
  data.set,
  spss.fixed.importer,
  spss.portable.importer,
  spss.system.importer,
  Stata.importer,
  importer,
  codebook,
  datetime.item,
  Date.item
  )

if(getRversion() < "3.6.0") exportClasses(double)

exportMethods(
    as.item,
    labels,
    "labels<-",
    value.filter,
    valid.values,
    missing.values,
    valid.range,
    "valid.values<-",
    "valid.range<-",
    "missing.values<-",
    measurement,
    "measurement<-",
    annotation,
    "annotation<-",
    Arith,
    Compare,
    Math,
    Math2,
    Summary,
    print,
    show,
    unique,
    "[",
    "[<-",
    "rep",
    row.names,
    dimnames,
    as.array,
    as.vector,
    as.character,
    as.numeric,
    as.integer,
    as.factor,
    as.ordered,
    contrasts,
    as.data.set,
    "contrasts<-",
    summary,
    within,
    merge,
    "%in%",
    recode,
    dim,
    initialize,
    coerce,
    Table,
    Descriptives,
    codebook,
    wild.codes,
    sample,
    query,
    "[[",
    "$",
    head,
    tail
    )
S3method(as.data.frame,character.item)
S3method(as.data.frame,double.item)
S3method(as.data.frame,integer.item)
S3method("[[<-",data.set)
S3method(str,data.set)
S3method(str,integer.item)
S3method(str,double.item)
S3method(str,character.item)
S3method(str,datetime.item)
S3method(str,Date.item)
S3method(as.list,data.set)
S3method(as.data.frame,data.set)
S3method(subset,data.set)
S3method(cbind,data.set)
S3method(fapply,data.set)
S3method(print,descriptions)
S3method(as.data.frame,descriptions)

S3method(Iconv,character)
S3method(Iconv,item)
S3method(Iconv,value.labels)
S3method(Iconv,annotation)
S3method(Iconv,data.set)
S3method(Iconv,importer)

S3method(charTrans,character)
S3method(charTrans,item)
S3method(charTrans,value.labels)
S3method(charTrans,annotation)
S3method(charTrans,data.set)
S3method(charTrans,importer)

S3method(Write,descriptions)
S3method(Write,codebook)

S3method(as.data.frame,spss.system.importer)
S3method(as.data.frame,spss.portable.importer)
S3method(as.data.frame,spss.fixed.importer)
S3method(as.data.frame,Stata.importer)
S3method(as.data.frame,Stata_new.importer)

S3method(subset,spss.system.importer)
S3method(subset,spss.portable.importer)
S3method(subset,spss.fixed.importer)
S3method(subset,Stata.importer)
S3method(subset,Stata_new.importer)

S3method(format_html,memisc_mtable)
S3method(format_html,ftable)
S3method(format_html,codebook)
S3method(format_html,codebookEntry)
S3method(format_html,data.frame)
S3method(format_html,matrix)

S3method(sort,data.set)
S3method(rbind,data.set)

S3method(as.POSIXct,datetime.item)
S3method(as.data.frame,datetime.item)
S3method(as.data.frame,Date.item)

S3method(xtfrm,integer.item)
S3method(xtfrm,double.item)
S3method(xtfrm,character.item)
S3method(xtfrm,datetime.item)
S3method(xtfrm,Date.item)

if (getRversion() >= "4.2.0") {
    
    S3method(mtfrm,integer.item)
    S3method(mtfrm,double.item)
    S3method(mtfrm,character.item)
    S3method(mtfrm,datetime.item)
    S3method(mtfrm,Date.item)
}


S3method(unique,integer.item)
S3method(unique,double.item)
S3method(unique,character.item)
S3method(unique,datetime.item)
S3method(unique,Date.item)

S3method(toLatex,data.frame)
S3method(toLatex,data.set)

S3method(percentages,table)
S3method(percentages,formula)
S3method(percentages,default)
S3method(percentages,list)
S3method(percentages,data.frame)
S3method(as.data.frame,percentage.table)
S3method(as.data.frame,xpercentage.table)

S3method(cbind,ftable)
S3method(rbind,ftable)
S3method(cbind,ftable_matrix)
S3method(rbind,ftable_matrix)
S3method(format,ftable_matrix)
S3method(print,ftable_matrix)
S3method(Write,ftable_matrix)

S3method(toLatex,ftable_matrix)
S3method(format_html,ftable_matrix)
S3method(relabel,ftable_matrix)

export(
    df_format_stdstyle,
    ftable_format_stdstyle,
    mat_format_stdstyle,
    mtable_format_stdstyle,
    codebook_format_stdstyle)

export(html,content,attribs,style,
  setContent,setStyle,setAttribs,
  html_group,as.html_group,
  css,as.css,
  "content<-","attribs<-","style<-")
  
S3method(as.character,html_elem)
S3method(print,html_elem)
S3method(c,html_elem)
S3method(format_html,html_elem)

S3method(as.character,html_group)
S3method(print,html_group)
S3method(c,html_group)
S3method("[",html_group)
S3method("[<-",html_group)
S3method(format_html,html_group)


S3method(setAttribs,character)
S3method(setAttribs,html_elem)
S3method(setAttribs,html_group)
S3method("[<-",html_attributes)

S3method(as.character,css)
S3method(print,css)
S3method(setStyle,character)
S3method(setStyle,html_elem)
S3method(setStyle,html_group)
S3method("[<-",css)

S3method(format_html,descriptions)

export(withSE,withVCov)
S3method(withVCov,lm)
S3method(vcov,withVCov)
S3method(summary,withVCov)
S3method(summary,withVCov.lm)

export(codeplan,
       setCodeplan,
       "codeplan<-")
S3method(print,codeplan)

exportMethods(codeplan,setCodeplan)

S3method(as.list,item.list)
export(duplicated_labels)
S3method(duplicated_labels,item)
S3method(duplicated_labels,item.list)

S3method(print,dupLabelsReport)
S3method(print,dupLabelsReport1)

export(
    "%#%",
    "%##%",
    "%@%",
    "%@%<-"
)
export(List)

export(Groups)
S3method(Groups,data.frame)
S3method(Groups,data.set)
S3method(Groups,grouped.data)

S3method(with,grouped.data)
S3method(within,grouped.data)

S3method(print,grouped.data.frame)
S3method(print,grouped.data.set)
export(recombine)
S3method(recombine,grouped.data.frame)
S3method(recombine,grouped.data.set)

S3method(as.data.frame,grouped.data)
# S3method(as.data.set,grouped.data.frame)
# S3method(as.data.set,grouped.data.set)


S3method(print,grouped.result)
export(withGroups,withinGroups)

export(Reshape)
export(spss.file)

export(as_tibble.data.set)
# S3method("as_tibble",data.set)
# exportMethods(as_tibble)
export(as_haven)
exportMethods(as_haven)

export(view)
export(viewPrep)
S3method(viewPrep,default)
S3method(viewPrep,data.set)
S3method(viewPrep,data.frame)
S3method(viewPrep,descriptions)
S3method(viewPrep,codeplan)
S3method(viewPrep,importer)
export(view_html)

importFrom(data.table,as.data.table)
S3method(as.data.table,data.set)
# export(as.data.table.data.set) 
# exportMethods(as.data.table) 

export(set_measurement)

export(Mean,Median,Min,Max,Weighted.Mean,
       Var,StdDev,Cov,Cor,Range)

export(deduplicate_labels)
S3method(deduplicate_labels,default)
S3method(deduplicate_labels,item)
S3method(deduplicate_labels,item.list)

export(trim_labels)
exportMethods(trim_labels)
export(reversed)

export(Means)
S3method(Means,formula)
S3method(Means,data.frame)
S3method(Means,data.set)
S3method(Means,numeric)
S3method(as.data.frame,means.table)
S3method(as.data.frame,xmeans.table)

export(contract)
S3method(contract,data.frame)
S3method(contract,data.set)

S3method(format,data.set)
S3method(format,codebookEntry)
S3method(format,Date.item)
S3method(format,datetime.item)
S3method(format,item.vector)

importFrom(yaml,as.yaml,read_yaml,write_yaml)
importFrom(jsonlite,read_json,write_json)
export(read_codeplan,write_codeplan)

export(format_md)
S3method(format_md, codebook)
S3method(format_md, codebookEntry)

export(coarsen)
S3method(coarsen,numeric)

export(measurement_autolevel)
exportMethods(measurement_autolevel)

export(
    "%if%",
    "%$%",
    "%$$%",
    Within
)
S3method(Within,data.frame)

export(neg2mis)

## ----echo=FALSE,message=FALSE-------------------------------------------------
knitr::opts_chunk$set(comment=NA,
               fig.align="center",
               results="markup")

## -----------------------------------------------------------------------------
tab.Class.Age <- xtabs(Freq~Class+Age,data=Titanic)
tab.Survived.Class.Age <- xtabs(Freq~Survived+Class+Age,data=Titanic)
tab.Survived.Class.Sex <- xtabs(Freq~Survived+Class+Sex,data=Titanic)
tab.Survived.Class <- xtabs(Freq~Survived+Class,data=Titanic)
tab.Survived.Sex <- xtabs(Freq~Survived+Sex,data=Titanic)
tab.Survived.Age <- xtabs(Freq~Survived+Age,data=Titanic)
tab.Survived <- xtabs(Freq~Survived,data=Titanic)

## ----message=FALSE------------------------------------------------------------
library(memisc)

## -----------------------------------------------------------------------------
(ftab.Survived.Age <- ftable(tab.Survived.Age))
(ftab.Survived.Sex <- ftable(tab.Survived.Sex))

## -----------------------------------------------------------------------------
cbind(ftab.Survived.Age,
      ftab.Survived.Sex)

## -----------------------------------------------------------------------------
cbind(ftab.Survived.Age,
      ftab.Survived.Sex,
      Total=tab.Survived)

## ----results='asis'-----------------------------------------------------------
show_html(ftab.Survived.Age)

## ----results='asis'-----------------------------------------------------------
show_html(ftab.Survived.Sex)

## ----results='asis'-----------------------------------------------------------
show_html(
  cbind(ftab.Survived.Age,
      ftab.Survived.Sex,
      Total=tab.Survived)
)

## -----------------------------------------------------------------------------
knit_print.ftable_matrix <-function(x,options,...)
  knitr::asis_output(
    format_html(x,
                digits=if(length(options$ftable.digits))
                          options$ftable.digits
                       else 0,
                ...))

## -----------------------------------------------------------------------------
cbind(ftab.Survived.Age,
      ftab.Survived.Sex,
      Total=tab.Survived)

## -----------------------------------------------------------------------------
rm(knit_print.ftable_matrix)

## ----results='asis'-----------------------------------------------------------
show_html(
  cbind(ftab.Survived.Age,
      ftab.Survived.Sex,
      Total=tab.Survived),
  varinfront=FALSE
)

## ----results='asis'-----------------------------------------------------------
show_html(
  cbind(ftab.Survived.Age,
      ftab.Survived.Sex,
      Total=tab.Survived),
  varontop=FALSE
)

## -----------------------------------------------------------------------------
ftab.Age.Survived <- ftable(tab.Survived.Age,col.vars=1)
ftab.Sex.Survived <- ftable(tab.Survived.Sex,col.vars=1)
ftab.Class.Survived <- ftable(tab.Survived.Class,col.vars=1)

rbind(
  ftab.Age.Survived,
  ftab.Sex.Survived,
  ftab.Class.Survived,
  Total=tab.Survived
)

## ----results='asis'-----------------------------------------------------------
show_html(
  rbind(
    ftab.Age.Survived,
    ftab.Sex.Survived,
    ftab.Class.Survived,
    Total=tab.Survived
  )
)

## ----results='asis'-----------------------------------------------------------
ptab.Survived.Age<-percentages(Survived~Age,data=Titanic)
ptab.Survived.Sex<-percentages(Survived~Sex,data=Titanic)
ptab.Survived.Class<-percentages(Survived~Class,data=Titanic)

fptab.Age.Survived <- ftable(ptab.Survived.Age,col.vars=1)
fptab.Sex.Survived <- ftable(ptab.Survived.Sex,col.vars=1)
fptab.Class.Survived <- ftable(ptab.Survived.Class,col.vars=1)

show_html(
  rbind(
    fptab.Age.Survived,
    fptab.Sex.Survived,
    fptab.Class.Survived
  ),
  digits=1
)

## ----results='asis'-----------------------------------------------------------
tab.Age <- xtabs(Freq~Age,data=Titanic)
tab.Sex <- xtabs(Freq~Sex,data=Titanic)
tab.Class <- xtabs(Freq~Class,data=Titanic)


show_html(
  rbind(
    cbind(fptab.Age.Survived,Total=tab.Age),
    cbind(fptab.Sex.Survived,Total=tab.Sex),
    cbind(fptab.Class.Survived,Total=tab.Class)
  ),
  digits=c(1,0) # One digit after dot for percentages 
                # no digits for total counts.
)

## -----------------------------------------------------------------------------
toLatex(
  rbind(
    cbind(fptab.Age.Survived,Total=tab.Age),
    cbind(fptab.Sex.Survived,Total=tab.Sex),
    cbind(fptab.Class.Survived,Total=tab.Class)
  ),
  digits=c(1,0) # One digit after dot for percentages 
                # no digits for total counts.
)

setMethod("codebook","data.set",function(x,weights,unweighted=TRUE,...){
  weights <- eval(substitute(weights),x,parent.frame())
  cb <- lapply(x,codebookEntry,weights=weights,unweighted=unweighted)
  new("codebook",cb)
})

setMethod("codebook","item",function(x,weights,unweighted=TRUE,...){
  xname <- paste(deparse(substitute(x)))
  cb <- list(codebookEntry(x,weights=weights,unweighted=unweighted))
  names(cb) <- xname
  new("codebook",cb)
})

setMethod("codebook","data.frame",function(x,weights,unweighted=TRUE,...){
  weights <- eval(substitute(weights),x,parent.frame())
  cb <- lapply(x,codebookEntry,weights=weights,unweighted=unweighted)
  new("codebook",cb)
})

setMethod("codebook","tbl_df",function(x,weights,unweighted=TRUE,...){
  weights <- eval(substitute(weights),x,parent.frame())
  cb <- lapply(x,codebookEntry,weights=weights,unweighted=unweighted)
  new("codebook",cb)
})

setMethod("codebook","atomic",function(x,weights,unweighted=TRUE,...){
  xname <- paste(deparse(substitute(x)))
  cb <- list(codebookEntry(x,weights=weights,unweighted=unweighted))
  names(cb) <- xname
  new("codebook",cb)
})

setMethod("codebook","factor",function(x,weights,unweighted=TRUE,...){
  xname <- paste(deparse(substitute(x)))
  cb <- list(codebookEntry(x,weights=weights,unweighted=unweighted))
  names(cb) <- xname
  new("codebook",cb)
})

setMethod("codebook","ANY",function(x,weights,unweighted=TRUE,...){
  xname <- paste(deparse(substitute(x)))
  cb <- list(codebookEntry(x,weights=weights,unweighted=unweighted))
  names(cb) <- xname
  new("codebook",cb)
})

setMethod("codebook","NULL",function(x,weights,unweighted=TRUE,...)NULL)

setMethod("codebookEntry","item",function(x,weights,unweighted=TRUE,...){
  annotation <- annotation(x)
  filter <- x@value.filter
  spec <- c(
            "Storage mode:"=storage.mode(x),
            "Measurement:"=measurement(x)
            )
  if(length(filter)) spec <- c(spec,
                              switch(class(filter),
                                        missing.values = c("Missing values:" = format(filter)),
                                        valid.values   = c("Valid values:"   = format(filter)),
                                        valid.range    = c("Valid range:"    = format(filter))
                                     ))
  stats <- switch(measurement(x),
            nominal=,ordinal=codebookStatsCateg(x,weights=weights,unweighted=unweighted),
            interval=,ratio=codebookStatsMetric(x,weights=weights,unweighted=unweighted)
            )
  cbe <- new("codebookEntry",
    spec = spec,
    stats = stats,
    annotation = annotation
    )
  return(cbe)
})

NAtab <- function(isna,weights=NULL){
  if(!length(weights))
    weights <- rep(1,length(isna))
  counts <- c(
    Valid = sum(weights*!isna),
    "Missing (NA)" = sum(weights*isna),
    Total = sum(weights)
  )
  perc <- 100*counts/counts[3]
  perc[3] <- NA
  cbind(N=counts,
        Percent=perc)
}


setMethod("codebookEntry","ANY",function(x,weights,unweighted=TRUE,...){

  if(length(attr(x,"label")))
      annotation <- c(description=attr(x,"label"))
  else
      annotation <- NULL

  spec <- c("Storage mode:"=storage.mode(x))

  isat <- is.atomic(x)
  if(isat)
    isna <- is.na(x)
  else
    isna <- FALSE

  if(mode(x) == "numeric"){
    descr <- Descriptives(x)
    if(length(descr) > 4)
        descr <- descr[1:4]
    if(length(weights) && length(descr) > 2){ # There is more than a range
      wdescr <- Descriptives(x,weights)
      if(unweighted)
          descr <- collect(Unweighted=format(descr),
                           Weighted=format(wdescr))
      else
          descr <- as.matrix(format(wdescr))
    }
    else 
      descr <- as.matrix(format(descr))
  }
  else
    descr <- NULL

  if(any(isna)){
    tab <- NAtab(isna)
    if(length(weights)){
      wtab <- NAtab(isna,weights)
      if(unweighted)
          tab <- collect(Unweighted=tab,
                         Weighted=wtab)
      else
          tab <- array(wtab,
                       dim=c(dim(tab),1),
                       dimnames=c(dimnames(tab),
                                  list(NULL)))
    }
    else
      tab <- array(tab,
                   dim=c(dim(tab),1),
                   dimnames=c(dimnames(tab),
                              list(NULL)))
    attr(tab,"title") <- "Valid and missing values"
  } else
    tab <- integer(0)

  stats <- list(tab=tab,
                descr=descr)
  
  new("codebookEntry",
    spec = spec,
    stats = stats,
    annotation = annotation
  )
})

rwnexp <- function(mat,nms){
  res <- array(0,c(length(nms),ncol(mat)),
               dimnames=list(nms,colnames(mat)))
  rn <- rownames(mat)
  res[rn,] <- mat
  return(res)
}

codebookTable_factor <- function(x,weights=NULL,...){

  if(!length(weights))
    weights <- rep(1,length(x))

  isna <- is.na(x)
  counts <- rowsum(weights[!isna],x[!isna])
  lev <- levels(x)
  counts <- rwnexp(counts,lev)
  
  NAs <- sum(weights*isna)

  tab <- cbind(counts,100*counts/sum(counts))
  if(any(isna)) {
      labs <- sQuote(levels(x))
      if(nlevels(x))
        labs <- paste(format(c(1:nlevels(x),NA),justify="right"),
                      format(c(labs,""),justify="left")
                      )
      else
        labs <- "NA"
      tab <- rbind(tab,c(NAs,NA))
      counts <- c(counts,NAs)
      tab <- cbind(tab,100*counts/sum(counts))
      colnames(tab) <- c("N","Valid","Total")
    }
  else {
      labs <- sQuote(levels(x))
      labs <- paste(format(1:nlevels(x),justify="right"),
                    format(labs,justify="left")
                    )
      colnames(tab) <- c("N","Valid")
  }
  rownames(tab) <- labs
  

  return(tab)
}

setMethod("codebookEntry","factor",function(x,weights=NULL,unweighted=TRUE,...){

  tab <- codebookTable_factor(x)
  if(length(weights)){
    wtab <- codebookTable_factor(x,weights=weights)
    if(unweighted)
        tab <- collect(Unweighted=tab,
                       Weighted=wtab)
    else {
        tab <- wtab
        tab.dn <- dimnames(tab)
        tab.d <- dim(tab)
        dim(tab) <- c(tab.d,1)
        dimnames(tab) <- c(tab.dn,list(NULL))
    }
  }
  else{
    tab.dn <- dimnames(tab)
    tab.d <- dim(tab)
    dim(tab) <- c(tab.d,1)
    dimnames(tab) <- c(tab.dn,list(NULL))
  }
  attr(tab,"title") <- "Levels and labels"
  
  if(length(attr(x,"label")))
      annotation <- c(description=attr(x,"label"))
  else
      annotation <- NULL
    
  spec <- c("Storage mode:"=storage.mode(x))
  spec <- if(is.ordered(x)) c(spec,
            "Ordered factor with"=paste(nlevels(x),"levels"))
          else c(spec,
            "Factor with"=paste(nlevels(x),"levels"))
            
  stats <- list(tab=tab)
  new("codebookEntry",
    spec = spec,
    stats = stats,
    annotation = annotation
  )
})

setMethod("codebookEntry","character",function(x,weights=NULL,unweighted=TRUE,...){
  spec <- c("Storage mode:"=storage.mode(x))
  isna <- is.na(x)
  descr <- codebookStatsChar(x)
  descr <- descr$descr

  if(any(isna)){
    tab <- NAtab(isna)
    if(length(weights)){
      wtab <- NAtab(isna,weights)
      if(unweighted)
          tab <- collect(Unweighted=tab,
                         Weighted=tab)
      else
          tab <- array(wtab,
                       dim=c(dim(tab),1),
                       dimnames=c(dimnames(tab),
                                  list(NULL)))
    }
    else
      tab <- array(tab,
                   dim=c(dim(tab),1),
                   dimnames=c(dimnames(tab),
                              list(NULL)))
    attr(tab,"title") <- "Valid and missing values"
  } else
    tab <- integer(0)

  stats <- list(tab=tab,
                descr=descr)
  
  new("codebookEntry",
    spec = spec,
    stats = stats,
    annotation = NULL
  )
})

codebookTable_logical <- function(x,weights=NULL,...){

  if(!length(weights))
    weights <- rep(1,length(x))

  isna <- is.na(x)
  counts <- rowsum(weights[!isna],x[!isna])
  NAs <- sum(weights*isna)

  tab <- cbind(counts,100*counts/sum(counts))
  if(any(isna)) {
      labs <- c("FALSE","TRUE","NA")
      tab <- rbind(tab,c(NAs,NA))
      counts <- c(counts,NAs)
      tab <- cbind(tab,100*counts/sum(counts))
      colnames(tab) <- c("N","Valid","Total")
    }
  else {
      labs <- c("FALSE","TRUE")
      colnames(tab) <- c("N","Valid")
  }
  rownames(tab) <- labs
  

  return(tab)
}

setMethod("codebookEntry","logical",function(x,weights=NULL,unweighted=TRUE,...){

  tab <- codebookTable_logical(x)
  if(length(weights)){
    wtab <- codebookTable_logical(x,weights=weights)
    if(unweighted)
        tab <- collect(Unweighted=tab,
                       Weighted=wtab)
    else {
        tab <- wtab
        tab.dn <- dimnames(tab)
        tab.d <- dim(tab)
        dim(tab) <- c(tab.d,1)
        dimnames(tab) <- c(tab.dn,list(NULL))
    }
  }
  else{
    tab.dn <- dimnames(tab)
    tab.d <- dim(tab)
    dim(tab) <- c(tab.d,1)
    dimnames(tab) <- c(tab.dn,list(NULL))
  }
  attr(tab,"title") <- "Logical values"
  
  if(length(attr(x,"label")))
      annotation <- c(description=attr(x,"label"))
  else
      annotation <- NULL
    
  spec <- c("Storage mode:"=storage.mode(x))
            
  stats <- list(tab=tab)
  new("codebookEntry",
    spec = spec,
    stats = stats,
    annotation = annotation
  )
})


codebookStatsCateg <- function(x,weights=NULL,unweighted=TRUE,...){
    vl <- labels(x)
    ic <- inherits(x,"character")
    if(length(vl) || !ic){
        tab <- codebookTable_item(x)
        tab.title <- attr(tab,"title")
        if(length(weights)){
            wtab <- codebookTable_item(x,weights)
            if(unweighted)
                tab <- collect(Unweighted=tab,
                               Weighted=wtab)
            else
                tab <- array(wtab,
                             dim=c(dim(tab),1),
                             dimnames=c(dimnames(tab),
                                        list(NULL)))
        }
        else
            tab <- array(tab,
                         dim=c(dim(tab),1),
                         dimnames=c(dimnames(tab),
                                    list(NULL)))
        attr(tab,"title") <- tab.title
        list(tab=tab)
    }
    else
        codebookStatsChar(x)
}

codebookStatsChar <- function(x,...){
  descr <- structure(dQuote(range(x,na.rm=TRUE)),
                     names=c("Min","Max"))
  descr <- as.matrix(descr)
  list(
    tab = NULL,
    descr = descr
  )
}

codebookStatsMetric <- function(x,weights=TRUE,unweighted=TRUE,...){

    tab <- codebookTable_item(x,drop.unlabelled=TRUE)
    tab.title <- attr(tab,"title")
    if(length(weights) && length(tab)){
        wtab <- codebookTable_item(x,weights=weights,
                                   drop.unlabelled=TRUE)
        if(unweighted)
            tab <- collect(Unweighted=tab,
                           Weighted=wtab)
        else
            tab <- array(wtab,
                         dim=c(dim(tab),1),
                         dimnames=c(dimnames(tab),
                                    list(NULL)))
        attr(tab,"title") <- tab.title
    }
    else if(length(tab)){
        tab <- array(tab,
                     dim=c(dim(tab),1),
                     dimnames=c(dimnames(tab),
                                list(NULL)))
        attr(tab,"title") <- tab.title
    }
  
    descr <- Descriptives(x)[1:4]
    if(length(weights)){
        wdescr <- Descriptives(x,weights)[1:4]
        if(unweighted)
            descr <- collect(Unweighted=descr,
                             Weighted=wdescr)
        else
            descr <- as.matrix(wdescr)
    }
    else 
        descr <- as.matrix(descr)
    list(
        tab=tab,
        descr=descr
    )
}

codebookTable_item <- function(x,weights=NULL,drop.unlabelled=FALSE,drop.empty=TRUE){

  is.m <- is.missing(x)
  isNA <- is.na(x)
  vl <- labels(x)
  if(!length(weights)){
    weights <- rep(1,length(x))
  }
  if(length(vl)){
    vvl <- vl@values
    lvl <- vl@.Data
    valid <- !is.missing2(vvl,x@value.filter)
    i <- match(x@.Data,vvl,nomatch=0L)
    tab <- drop(rowsum(weights,i))
    if("0" %in% names(tab)){
      ii <- match("0",names(tab))
      tab <- tab[-ii]
    }
    if(length(tab) < length(vvl)){
      tab0 <- numeric(length(vvl))
      ii <- as.integer(names(tab))
      tab0[ii] <- tab
      tab <- tab0
    }
    names(tab) <- as.character(vvl)
    lab <- sQuote(vl@.Data)
    tab.title <- "Values and labels"
  }
  else {
    valid <- logical(0)
    tab <- c()
    lab <- c()
    i <- logical(length(x))
    tab.title <- "Values"
  }

  ovld <- sum(weights*(!is.m & !i))
  omiss <- sum(weights*(is.m & !i & !isNA))
  NAs <- sum(weights*(isNA))

  if(ovld | !drop.empty){
    tab <- c(tab," "=ovld)
    lab <- c(lab,"(unlab.val.)")
    valid <- c(valid,TRUE)
  }

  if(omiss | !drop.empty){
    tab <- c(tab," "=omiss)
    lab <- c(lab,"(unlab.mss.)")
    valid <- c(valid,FALSE)
  }
  if(NAs | !drop.empty){
    tab <- c(tab,"NA"=NAs)
    lab <- c(lab,"")
    valid <- c(valid,FALSE)
  }

  if(any(!valid)){
    missing.marker <- "M"
    valid.marker <- paste(rep(" ",nchar(missing.marker)),collapse="")
    lab <- paste(ifelse(valid,valid.marker,missing.marker),lab)
  }
  tab.nonzero <- tab>0
  tab.keep <- tab.nonzero | !drop.empty
  tab <- tab[valid | tab.keep]
  lab <- lab[valid | tab.keep]
  valid <- valid[valid | tab.keep]
  if(any(!valid)){
    vperc <- rep(NA,length(tab))
    vtab <- tab[valid]
    Nvalid <- sum(vtab)
    if(Nvalid) vperc[valid] <- 100 * vtab/Nvalid
    else vperc[valid] <- 0
    tperc <- 100 * tab/sum(tab)
    tab <- cbind(N=tab,Valid=vperc,Total=tperc)
  } else {
    tperc <- 100 * tab/sum(tab)
    tab <- cbind(N=tab,Percent=tperc)
  }
  rownames(tab) <- names(tperc)
  if(drop.unlabelled){
      drp <- match("(unlab.val.)",trimws(lab),nomatch=0L)
      if(drp > 0){
          tab <- tab[-drp,,drop=FALSE]
          lab <- lab[-drp]
          
      }
    if(all(is.na(tab[,2])) && length(tab)){
        tab <- tab[,-2,drop=FALSE]
        colnames(tab) <- c("N","Percent")
    }
  }
  if(!length(tab))
    tab <- NULL
  else {
    rownames(tab) <- paste(format(rownames(tab),justify="right"),format(lab,justify="left"))
    attr(tab,"title") <- tab.title
  }
  return(tab)
}


setMethod("as.character","codebook",function(x,...){
  width <- getOption("width")
  toprule <- paste(rep("=",width),collapse="")
  midrule <- paste(rep("-",width),collapse="")
  out <- mapply(format,x=x@.Data,name=names(x),toprule=toprule,midrule=midrule)
  unlist(out)
})

setMethod("show","codebook",function(object){
  width <- getOption("width")
  toprule <- paste(rep("=",width),collapse="")
  midrule <- paste(rep("-",width),collapse="")
  out <- mapply(format,x=object@.Data,name=names(object),toprule=toprule,midrule=midrule)
  out <- unlist(out)
  writeLines(out)
})


Write.codebook <- function(x,file=stdout(),...){
  width <- getOption("width")
  toprule <- paste(rep("=",width),collapse="")
  midrule <- paste(rep("-",width),collapse="")
  out <- mapply(format,x=x@.Data,name=names(x),toprule=toprule,midrule=midrule)
  out <- unlist(out)
  writeLines(out,con=file)
}


format.codebookEntry <- function(x,name="",width=getOption("width"),
          toprule=paste(rep("=",width),collapse=""),
          midrule=paste(rep("-",width),collapse=""),
          ...
      ){
      
  annot <- x@annotation
  description <- annot["description"]
  wording <- annot["wording"]
  if(length(annot)) annot <- annot[names(annot) %nin% c("description","wording")]

  title <- strwrap(if(length(description) && !is.na(description))
                  paste(name[1],sQuote(description))
                  else name[1] ,width=width,prefix="   ")
  
  wording <- if(length(wording) && !is.na(wording))
                strwrap(dQuote(wording),width=width,prefix="   ")
             else NULL

  spec <- paste("  ",names(x@spec),x@spec)
  tab <- unclass(x@stats$tab)
  descr <- unclass(x@stats$descr)
  
  if(length(tab)){

    tab.title <- attr(tab,"title")
    tab.d <- dim(tab)
    tab.dn <- dimnames(tab)
    tab <- apply(tab,3,format_cb_table)
    #tab <- lapply(tab,unlist)
    #tab <- do.call(cbind,tab)
    rn <- rownames(tab)
    rn[1] <- tab.title
    rn <- format(rn)
    if(tab.d[3]>1){
       tab <- cbind(" "=rn,tab)
       tab <- rbind(colnames(tab),"",tab)
       tab <- apply(tab,2,format,justify="right")
    }
    else {
      tab <- cbind(rn,tab)
    }
    tab <- paste("  ",apply(tab,1,paste,collapse=" "))
  }
  if(!is.matrix(descr)) descr <- NULL
  if(length(descr)){
    descr.rn <- format(paste(rownames(descr),":",sep=""),justify="right")
    if(is.numeric(descr[]))  
       descr[] <- formatC(descr[],format="f",digits=3)
    descr[] <- gsub("NA","",descr[])
    if(!length(ncol(descr))) browser()
    if(ncol(descr) > 1){
      descr.rn <- c("","",descr.rn)
      descr <- rbind(colnames(descr),"",descr)
    }
    descr <- cbind(descr.rn,descr)
    descr <- apply(descr,2,format,justify="right")
    descr <- paste("  ",apply(descr,1,paste,collapse=" "))
  }
  if(length(tab) && length(descr)){
    statstab <- format(c(tab,"",descr),justify="left")
  }
  else if(length(tab)){
    statstab <- tab
  }
  else if(length(descr)){
    statstab <- descr
  }
  else
    statstab <- NULL
  
  annot.out <- character()
  if(length(annot)){
    for(i in seq_len(length(annot))){
      annot.i <- annot[i]
      nm.i <- trimws(names(annot.i))
      annot.i <- strwrap(annot.i,width=getOption("width")-8-4)
      annot.i <- c(paste("      ",annot.i),"")
      if(nzchar(nm.i)){
        annot.i <- c(
          paste("   ",nm.i,":",sep=""),
          annot.i
          )
      }
      annot.out <- c(annot.out,annot.i)
    }
  }
  c(
    toprule,
    "",
    title,
    if(length(wording)) c(
        "",
        wording
        ),
    "",
    midrule,
    "",
    spec,
    "",
    statstab,
    "",
    if(length(annot.out)) annot.out
    )
}

#setMethod("format","codebookEntry",format.codebookEntry)


format_cb_table <- function(tab){
    cn <- colnames(tab)
    if(ncol(tab)>2){
        if(all(trunc(tab[,1])==tab[,1])){
            tab <- cbind(
                formatC(tab[,1,drop=FALSE],format="d"),
                formatC(tab[,2],format="f",digits=1),
                formatC(tab[,3],format="f",digits=1)
            )
        } else {
            tab <- cbind(
                formatC(tab[,1,drop=FALSE],format="f",digits=1),
                formatC(tab[,2],format="f",digits=1),
                formatC(tab[,3],format="f",digits=1)
            )
        }
    }
    else {
        if(all(trunc(tab[,1])==tab[,1])){
            tab <- cbind(
                formatC(tab[,1,drop=FALSE],format="d"),
                formatC(tab[,2],format="f",digits=1)
            )
        } else {
            tab <- cbind(
                formatC(tab[,1,drop=FALSE],format="f",digits=1),
                formatC(tab[,2],format="f",digits=1)
            )
        }
        
    }
    tab[tab=="NA"] <- ""
    tab <- rbind(" "=cn,"",tab)
    tab <- format(tab,justify="right")
    tab <- apply(tab,1,paste,collapse=" ")
    tab
}

setMethod("[",signature(x="codebook",i="atomic",j="missing",drop="ANY"),
  function(x,i,j,...,drop=TRUE){
      if(is.character(i))
          i <- match(i,names(x))
      cb <- x@.Data[i]
      if(!length(cb)) return(NULL)
      if(is.numeric(i) || is.logical(i))
          names(cb) <- names(x)[i]
      else return(NULL)
      new("codebook",cb)
  })

setMethod("$",signature(x="codebook"),
function(x,name){
  i <- match(name,names(x))
  cb <- x@.Data[i]
  if(!length(cb)) return(NULL)
  names(cb) <- name
  new("codebook",cb)
})

setMethod("[[",signature(x="codebook"),
function(x,i,...){
      if(is.character(i))
          i <- match(i,names(x))
      cb <- x@.Data[i]
      if(!length(cb)) return(NULL)
      if(is.numeric(i) || is.logical(i))
          names(cb) <- names(x)[i]
      new("codebook",cb)
})

compress.storage.mode <- function(x) UseMethod("compress.storage.mode")
compress.storage.mode.default <- function(x) x
compress.storage.mode.double <- function(x){
  ok <- is.finite(x)
  #y <- as.integer(x)
  if(max(abs(x[ok]))>.Machine$integer.max || any (x[ok]!=trunc(x[ok]))) x else as.integer(x)
}

setMethod("wild.codes","item",function(x){
  vl <- labels(x)
  vvl <- vl@values
  use <- !(x %in% vvl)
  N <- length(x)
  x <- x@.Data[use]
   
  tab <- Table(x,counts=TRUE,percentage=TRUE)
  tab[,2] <- 100*tab[,1]/N
  tab
})

withSE <- function(object, vcov, ...){
    if(is.matrix(vcov)) UseMethod("withVCov")
    else if(is.function(vcov)) UseMethod("withVCov")
    else if(is.character(vcov)){
        if(exists(vcov))
            vcov <- get(vcov)
        else if(exists(vcov2 <- paste0("vcov",vcov)))
            vcov <- get(vcov2)
        else
            stop("neither '",vcov,"' nor '", vcov2, "' is a known function")
        withVCov(object, vcov=vcov, ...)
    }
    else stop("cannot handle 'vcov' argument of type",typeof(vcov))
}

withVCov <- function(object, vcov, ...) UseMethod("withVCov")

withVCov.lm <- function(object, vcov, ...){

    if(is.function(vcov))
        V <- vcov(object, ...)
    else if(is.matrix(vcov))
        V <- vcov
    else
        stop("argument 'vcov' should be a matrix")

    cls <- class(object)
    cls <- c(paste("withVCov",cls[1],sep="."),
             "withVCov",
             class(object))
    structure(object,
              .VCov=V,
              class=cls)
}

vcov.withVCov <- function(object, ...) attr(object,".VCov")

summary.withVCov <- function(object, ...){

    V <- attr(object,".VCov")
    
    res <- NextMethod()
    coefTab <- res$coefficients

    est <- coefTab[,1]
    se <- sqrt(diag(V))
    zval <- est/se
    pval <- 2 * pnorm(abs(zval),lower.tail=FALSE)
    coefTab[,2] <- se
    coefTab[,3] <- zval
    coefTab[,4] <- pval
    res$coefficients <- coefTab

    res
}

summary.withVCov.lm <- function(object, ...){

    V <- attr(object,".VCov")
    
    res <- NextMethod()
    coefTab <- res$coefficients
    rdf <- res$df[2]
    
    est <- coefTab[,1]
    se <- sqrt(diag(V))
    tval <- est/se
    pval <- 2 * pt(abs(tval),df=rdf,lower.tail=FALSE)
    coefTab[,2] <- se
    coefTab[,3] <- tval
    coefTab[,4] <- pval
    res$coefficients <- coefTab

    res
}

setMethod("codeplan",
          signature(x="ANY"),function(x) NULL)

setMethod("codeplan",
          signature(x="item.list"),
          function(x){
    dat <- x@.Data
    n <- x@names
    cp <- lapply(dat,codeplan1)
    names(cp) <- n
    structure(cp,
              class="codeplan")
})

setMethod("codeplan",
          signature(x="item"),
          function(x){
              cp <- codeplan1(x)
              structure(list(cp),class="codeplan")
})

codeplan1 <- function(x) {
    l <- list(
        annotation=depr_annot(x),
        labels=depr_labels(x),
        value.filter=depr_vfilter(x),
        mode=mode(x),
        measurement=measurement(x)
    )
    l[sapply(l,length)>0]
}

print.codeplan <- function(x,...){
    cat("\n",as.yaml(unclass(x)),"\n",sep="")
}

any_nzchar <- function(x) any(nzchar(x))


formatw <- function(x,width){
    l <- nchar(x)
    to.abbr <- l > width
    x[to.abbr] <- paste0(substr(x[to.abbr],1,width-3),"...")
    format(x)
}

depr_annot <- function(x){
    a <- annotation(x)
    a <- structure(a@.Data, names=names(a))
    if(length(a)) as.list(a)
    else NULL
}

depr_labels <- function(x){
    l <- labels(x)
    if(length(l))
        structure(as.list(l@values),names=l@.Data)
    else NULL
}

depr_vfilter <- function(x){
    v <- value.filter(x)
    if(length(v)){
        cl <- as.character(class(v))
        if(cl == "missing.values"){
            res <- list(class=cl,
                        range=v@range,
                        values=v@filter)
        }
        else if(cl == "valid.values"){
            res <- list(class=cl,
                        values=v@filter)
        }
        else if(cl == "valid.range"){
            res <- list(class=cl,
                        range=v@filter)
        }
        else res <- NULL
        res <- res[sapply(res,length)>0]
    }
    else NULL
}

setMethod("setCodeplan",signature(x="data.frame",value="codeplan"),
function(x,value){
    x <- as.data.set(x)
    setCodeplan(x,value)
})

setMethod("setCodeplan",signature(x="data.frame",value="NULL"),
function(x,value) x)

setMethod("setCodeplan",signature(x="data.set",value="codeplan"),
function(x,value){
    n <- intersect(names(x),names(value))
    if(length(n))
        for(nn in n){
            x[[nn]] <- setCodeplan1(x[[nn]],value[[nn]])
        }
    x
})

setMethod("setCodeplan",signature(x="data.set",value="NULL"),
function(x,value){
    y <- as.list(x)
    y <- lapply(y,setCodeplan,value=NULL)
    as.data.frame(y,row.names=row.names(x))
})

setMethod("setCodeplan",signature(x="item",value="codeplan"),
function(x,value){
    setCodeplan1(x,value[[1]])
})

setMethod("setCodeplan",signature(x="atomic",value="codeplan"),
function(x,value){
    setCodeplan1(x,value[[1]])
})

setMethod("setCodeplan",signature(x="item",value="NULL"),
function(x,value){
    x@.Data
})

setMethod("setCodeplan",signature(x="atomic",value="NULL"),
function(x,value){
    x@.Data
})



"codeplan<-" <- function(x,value) {
    x <- setCodeplan(x,value)
    invisible(x)
}

setCodeplan1 <- function(x,val){

    if(is.null(x)){
        x <- vector(mode=val$mode)
    }
    else if(mode(x) != val$mode)
        stop(sprintf("mode conflict: '%s' != '%s'",
                     mode(x),val$mode))
    l <- val$labels
    if(length(l)){
        l <- unlist(l)
        labels(x) <- l
    }
    m <- val$measurement
    measurement(x) <- m
    a <- val$annotation
    if(length(a)){
        annotation(x) <- unlist(a)
    }
    vf <- val$value.filter
    if(length(vf)){
        cl <- vf$class
        if(cl=="missing.values")
            vf <- new(cl,
                      filter=vf$values,
                      range=vf$range
                      )
        else if(cl=="valid.values")
            vf <- new(cl,filter=vf$values)
        else if(cl=="valid.range")
            vf <- new(cl,filter=vf$range)
        x@value.filter <- vf
    }
    x
}

write_codeplan <- function(x,filename,
                           type=NULL,
                           pretty=FALSE){
    if(!length(type)){
        if(endsWith(filename,".yaml") ||
           endsWith(filename,".yml"))
            type <- "yaml"
        else if(endsWith(filename,".json"))
            type <- "json"
    }

    if(type=="yaml"){
        write_yaml(unclass(x),file=filename)
    }
    else if(type=="json"){
        write_json(unclass(x),path=filename,
                   auto_unbox=TRUE,pretty=pretty)
    }
    else stop(sprintf("File type %s not supported (yet)",type))
}

read_codeplan <- function(filename,
                          type=NULL){
    if(!length(type)){
        if(endsWith(filename,".yaml") ||
           endsWith(filename,".yml"))
            type <- "yaml"
        else if(endsWith(filename,".json"))
            type <- "json"
    }

    if(type=="yaml"){
        structure(read_yaml(file=filename),class="codeplan")
    }
    else if(type=="json"){
        structure(read_json(path=filename, simplifyVector = TRUE),
                  class="codeplan")
    }
    else stop(sprintf("File type %s not supported (yet)",type))
}

"%#%" <- function(x,descr){
    description(x) <- descr
    return(x)
}


"%##%" <- function(x,annot){
    for(n in names(annot)){
        annotation(x)[n] <- as.character(annot[n])
    }
    return(x)
}


"%@%" <- function(x,nm){
    nm <- substitute(nm)
    if(typeof(nm)=="symbol"){
        nm <- deparse(nm)
    } else {
        nm <- eval.parent(nm)
    }
    attr(x,nm)
}

"%@%<-" <- function(x,nm,value){
    nm <- substitute(nm)
    if(typeof(nm)=="symbol"){
        nm <- deparse(nm)
    } else {
        nm <- eval.parent(nm)
    }
    attr(x,nm) <- value
    return(invisible(x))
}

## annotation ###########################################################################

setMethod("annotation","ANY",function(x)attr(x,"annotation"))
setMethod("annotation","item",function(x)x@annotation)

setMethod("annotation<-",signature(x="ANY",value="NULL"),function(x,value){
  attr(x,"annotation") <- NULL
  x
})


setMethod("annotation<-",signature(x="ANY",value="character"),function(x,value){
  value <- new("annotation",structure(as.character(value),names=names(value)))
  callGeneric(x,value)
})


setMethod("annotation<-",signature(x="item",value="annotation"),function(x,value){
  x@annotation <- value
  x
})

setMethod("annotation<-",signature(x="ANY",value="annotation"),function(x,value){
  attr(x,"annotation") <- value
  x
})

description <- function(x){
    d <- annotation(x)["description"]
    if(!length(d)) return(NULL)
    if(is.na(d)) NULL else unname(d)
}
setGeneric("description",function(x)standardGeneric("description"))

description.data.frame <- function(x){
    dict <- lapply(x,attr,"label")
    structure(dict,class="descriptions")
}

setMethod("description","data.frame",description.data.frame)
setMethod("description","tbl_df",description.data.frame)

"description<-" <- function(x,value){
  annotation(x)["description"] <- value
  x
}

wording <- function(x){
  wdng <- annotation(x)["wording"]
  if(is.na(wdng)) NULL else unname(wdng)
}

"wording<-" <- function(x,value){
  annotation(x)["wording"] <- value
  x
}

setMethod("show","annotation",function(object){
  if(length(object)){
    annot.out <- character()
    for(i in seq_len(length(object))){
      annot.i <- object[i]
      nm.i <- trimws(names(annot.i))
      annot.i <- strwrap(annot.i,width=getOption("width")-8)
      annot.i <- c(paste("    ",annot.i),"")
      if(nzchar(nm.i)){
        annot.i <- c(
          paste(nm.i,":",sep=""),
          annot.i
          )
      }
      annot.out <- c(annot.out,annot.i)
    }
    writeLines(annot.out)
    }
})

## The following two functions are contributed by
## Dave Atkins, PhD
## Research Associate Professor
## Center for the Study of Health and Risk Behaviors
## Department of  Psychiatry and Behavioral Science
## Unversity of Washington

getSummary_expcoef <- function(obj, alpha = 0.05, ...) UseMethod("getSummary_expcoef")

### This function is no longer exported, because there is a more general
### default method available now.
getSummary_expcoef.glm <- function (obj, alpha = 0.05, ...)
{
    smry <- summary(obj)
    N <- if(length(weights(obj)))
             sum(weights(obj),na.rm=TRUE)
         else sum(smry$df[1:2])
    
    coef <- cbind(exp(smry$coef[,1]), smry$coef[,2:4,drop=FALSE])
    # NOTE: exponentiated coefficients

    lower <- exp(qnorm(p = alpha/2, mean = smry$coef[, 1], sd = coef[,2]))
    upper <- exp(qnorm(p = 1 - alpha/2, mean = smry$coef[, 1], sd = coef[,2]))
    # NOTE: exponentiated CI

    coef <- cbind(coef, lower, upper)

    dn <- list(
        rownames(coef),
        c("est","se","stat","p","lwr","upr"),
        names(obj$model)[1]
    )
    dim(coef) <- c(dim(coef)[1],dim(coef)[2],1)
    dimnames(coef) <- dn

    phi <- smry$dispersion
    LR <- smry$null.deviance - smry$deviance
    df <- smry$df.null - smry$df.residual

    ll <- logLik(obj)
    deviance <- deviance(obj)

    if (df > 0) {
        p <- pchisq(LR, df, lower.tail = FALSE)
        L0.pwr <- exp(-smry$null.deviance/N)
        Aldrich.Nelson <- LR/(LR+N)
        McFadden <- 1 - smry$deviance/smry$null.deviance
        Cox.Snell <- 1 - exp(-LR/N)
        Nagelkerke <- Cox.Snell/(1 - L0.pwr)
    }
    else {
        LR <- NA
        df <- NA
        p <- NA
        Aldrich.Nelson <- NA
        McFadden <- NA
        Cox.Snell <- NA
        Nagelkerke <- NA
    }
    
    AIC <- AIC(obj)
    BIC <- AIC(obj,k=log(N))
    sumstat <- c(
        phi            = phi,
        LR             = LR,
        df             = df,
        p              = p,
        logLik         = ll,
        deviance       = deviance,
        Aldrich.Nelson = Aldrich.Nelson,
        McFadden       = McFadden,
        Cox.Snell      = Cox.Snell,
        Nagelkerke     = Nagelkerke,
        AIC            = AIC,
        BIC            = BIC,
        N              = N
    )

    list(coef=coef,
         sumstat=sumstat,
         contrasts=obj$contrasts,
         xlevels=obj$xlevels,
         call=obj$call)
}

view <- function(x,
                 title=deparse(substitute(x)),
                 vfunc=getOption("vfunc","View"),
                 ...)
{

    # cls <- class(x)
    # title <- paste0(cls,": ",title)

    prepd <- viewPrep(x,title,...)
    title <- attr(prepd,"title")
    
    View.call <- call(vfunc,x=prepd,title=title)
    eval(View.call,globalenv())
}

viewPrep <- function(x,title,...)UseMethod("viewPrep")
viewPrep.default <- function(x,title,...){
    return(structure(x,title=title))
}

viewPrep.data.set <- function(x,title,...){
    # title <- paste("Data set:",title)
    Data <- lapply(x@.Data,format,justify="left")
    frame <- structure(Data,
                       row.names=x@row_names,
                       names=x@names,
                       class="data.frame")
    for(n in names(frame)){
        d <- description(x[[n]])
        if(length(d))
            attr(frame[[n]],"label") <- d
    }
    return(structure(frame,title=title))
}

viewPrep.data.frame <- function(x,title,...){
    structure(x,title=title)
}

viewPrep.descriptions <- function(x,title,...){
  # title <- paste("Descriptions:",title)
  viewPrep.data.frame(as.data.frame(x),
                      title=title,
                      ...)
}

parse_string <- function(x)parse(file=NULL,text=x)
format_lab_view <- function(x){
  fmt <- paste(unname(x),sQuote(names(x)))
  fmt <- paste(fmt,collapse=", ")
  return(fmt)
}
in_range <- function(x,rng){
    x >= rng[1] & x <= rng[2]
}

view1cp <- function(x){
    n <- x["name"]
    d <- x["description"]
    l <- x["labels"]
    if(nzchar(l))
        l <- eval(parse(text=l))
    else
        l <- NULL
    ms <- x["measurement"]
    vf <- x["value.filter"]
    if(nzchar(vf)){
        vf <- paste0("list(",vf,")")
        vf <- eval(parse(text=vf))
        cl <- names(vf)
        vf <- vf[[1]]
        if(cl=="missing")
            vf <- new("missing.values",
                      filter=vf$values,
                      range=vf$range
                      )
        else if(cl=="valid"){
            if(length(vf$values))
                vf <- new("valid.values",filter=vf$values)
            else if(length(vf$range))
                vf <- new("valid.range",filter=vf$range)
        }
    } else vf <- NULL
    nl <- length(l)
    tab <- matrix("",ncol=4,nrow=max(1,nl))
    colnames(tab) <- c("variable","description","value","label")
    tab[1,1] <- n
    tab[1,2] <- d
    omtab <- NULL
    if(nl>0){
      if(length(vf))
          ism <- is.missing2(l,vf)
      else
          ism <- FALSE
      tab[1:nl,3] <- paste(unname(l),ifelse(ism,"M"," "))
      tab[1:nl,4] <- names(l)
      if(length(vf) && inherits(vf,"missing.values")){
          if(length(vf@filter) && any(vf@filter %nin% l)){
              omiss <- setdiff(vf@filter,l)
              omiss <- paste(omiss,"M")
              omtab <- rbind(omtab,cbind("","",omiss,"(unlabelled)"))
          }
      #     if(length(vf@range) && any(!in_range(l,vf@range))){
      #         omiss <- paste(vf@range[1],"--",vf@range[2],"M")
      #         omtab <- rbind(omtab,cbind("","",omiss,"(unlabelled)"))
      #     }
      } 
    } else if(length(vf)){
        if(inherits(vf,"missing.values")){
          if(length(vf@filter)){
              omiss <- setdiff(vf@filter,l)
              omiss <- paste(omiss,"M")
              omtab <- rbind(omtab,cbind("","",omiss,"(unlabelled)"))
          }
          if(length(vf@range)){
              omiss <- paste(vf@range[1],"--",vf@range[2],"M")
              omtab <- rbind(omtab,cbind("","",omiss,"(unlabelled)"))
          }
        } else if(inherits(vf,"valid.values")){
            nvd <- length(vf@filter)
            if(nvd > 1)
                tab <- rbind(tab,matrix("",ncol=ncol(tab),nrow=nvd-1))
            tab[1:nvd,3] <- paste(vf@filter," ")
            tab[1:nvd,4] <- "(unlabelled)"
        } else if(inherits(vf,"valid.range")){
            tab[1,3] <- paste(vf@filter[1],"--",vf@filter[2]," ")
            tab[1:nvd,4] <- "(unlabelled)"
        }
    }
    
    if(length(omtab)){
        tab <- rbind(tab,omtab)
    }
    return(tab)
}

viewPrep.codeplan <- function(x,title,compact=FALSE,...){
    # title <- paste("Code plan:",title)
    if(compact){
      labels <- x$labels
      nll <- nzchar(labels)
      labels <- as.list(labels)
      labels[nll] <- lapply(labels[nll],parse_string)
      labels[nll] <- lapply(labels[nll],eval)
      labels[nll] <- lapply(labels[nll],format_lab_view)
      labels <- unlist(labels)
      frame <- data.frame(variable=x$name,
                          description=x$description,
                          labels=labels,
                          value.filter=x$value.filter)
      viewPrep.data.frame(frame,
                          title=title,
                          ...)
    }
    else {
        if(nrow(x)>1){
            tabs <- apply(x,1,view1cp)
            tab <- do.call(rbind,tabs)
        }
        else
            tab <- view1cp(unlist(x))
            
      tab[,3] <- format(trimws(tab[,3]),justify="right",width=5)
      viewPrep.default(tab,
                       title=title,
                       ...)
    }      
}

viewPrep.importer <- function(x,title,compact=TRUE,...){
    cp <- codeplan(x)
    viewPrep.codeplan(x=cp,title=title,compact=compact,...)
}

coarsen <- function(x,...) UseMethod("coarsen")

coarsen.numeric <- function(x,
                n=5,
                pretty=TRUE,
                quantiles=!pretty,
                breaks=NULL,
                brackets=FALSE,
                sep=if(brackets)";"else if(quantiles) "-" else " - ",
                left="[",
                right="]",
                range=FALSE,
                labels=NULL,
                ...){
    if(quantiles && missing(pretty)) pretty <- FALSE
    if(pretty && quantiles) stop("Only one of 'pretty' or 'quantiles' may be TRUE.")
    if(!(pretty || quantiles)) stop("At least one of 'pretty' or 'quantiles' must be TRUE.")
    if(length(breaks)) { 
        pretty <- FALSE
        quantiles <- FALSE
    }
    lo <- min(x)
    hi <- max(x)

    if(pretty || quantiles){
        if(pretty){
            breaks <- pretty(x,n=n,...)
            lwr <- breaks[-length(breaks)]
            upr <- breaks[-1]
        }
        else if(quantiles){
            p <- seq(from=0,to=1,length=n+1)
            p <- p[!(p %in% c(0,1))]
            breaks <- quantile(x,probs=p,...)
            lab <- names(breaks)
            breaks <- c(lo,breaks,hi)
            lwr <- c("0%",lab)
            upr <- c(lab,"100%")
        }
    } else {
        if(range)
            breaks <- c(lo,breaks,hi)
        lwr <- breaks[-length(breaks)]
        upr <- breaks[-1]
    }
    if(missing(labels)){
        if(brackets)
            labels <- paste0(left,paste(lwr,upr,sep=sep),right)
        else 
            labels <- paste(lwr,upr,sep=sep)
    }
    f <- cut(x,breaks=breaks,include.lowest=TRUE)
    if(length(labels)!=nlevels(f))
        stop("Number of labels does not match the number of levels.")
    f <- factor(f,levels=levels(f),labels=labels)
    f
}

mtable_format_latex <- function(x,
          useDcolumn=getOption("useDcolumn",TRUE),
          colspec=if(useDcolumn)
                      paste("D{.}{",LaTeXdec,"}{",ddigits,"}",sep="")
                  else "l",
          LaTeXdec=".",
          ddigits=min(3,getOption("digits")),
          useBooktabs=getOption("useBooktabs",TRUE),
          toprule=if(useBooktabs) "\\toprule" else "\\hline\\hline",
          midrule=if(useBooktabs) "\\midrule" else "\\hline",
          cmidrule=if(useBooktabs) "\\cmidrule" else "\\cline",
          bottomrule=if(useBooktabs) "\\bottomrule" else "\\hline\\hline",
          interaction.sep = " $\\times$ ",
          sdigits=min(1,ddigits),
          compact=FALSE,
          sumry.multicol=FALSE,
          escape.tex=getOption("toLatex.escape.tex",FALSE),
          signif.notes.type=getOption("toLatex.signif.notes.type","include"),
          signif.notes.spec=getOption("toLatex.signif.notes.spec","p{.5\\linewidth}"),
          ...)
    pf_mtable_format_latex(preformat_mtable(x),
          useDcolumn=useDcolumn,
          colspec=colspec,
          LaTeXdec=LaTeXdec,
          ddigits=ddigits,
          useBooktabs=useBooktabs,
          toprule=toprule,
          midrule=midrule,
          cmidrule=cmidrule,
          bottomrule=bottomrule,
          interaction.sep = interaction.sep,
          sdigits=sdigits,
          compact=compact,
          sumry.multicol=sumry.multicol,
          escape.tex=escape.tex,
          signif.notes.type=signif.notes.type,
          signif.notes.spec=signif.notes.spec,
          ...)


pf_mtable_format_latex <- function(x,
          useDcolumn=getOption("useDcolumn",TRUE),
          colspec=if(useDcolumn)
                      paste("D{.}{",LaTeXdec,"}{",ddigits,"}",sep="")
                  else "l",
          LaTeXdec=".",
          ddigits=min(3,getOption("digits")),
          useBooktabs=getOption("useBooktabs",TRUE),
          toprule=if(useBooktabs) "\\toprule" else "\\hline\\hline",
          midrule=if(useBooktabs) "\\midrule" else "\\hline",
          cmidrule=if(useBooktabs) "\\cmidrule" else "\\cline",
          bottomrule=if(useBooktabs) "\\bottomrule" else "\\hline\\hline",
          interaction.sep = " $\\times$ ",
          sdigits=min(1,ddigits),
          compact=FALSE,
          sumry.multicol=FALSE,
          escape.tex=getOption("toLatex.escape.tex",FALSE),
          signif.notes.type=getOption("toLatex.signif.notes.type","include"),
          signif.notes.spec=getOption("toLatex.signif.notes.spec",
                                      paste0("p{",signif.notes.width,"\\linewidth}")),
          signif.notes.width=getOption("toLatex.signif.notes.width",".7"),
          show.parmtypes = nrow(x$parmtab) > 1,
          ...
          ){

    signif.notes.type <- match.arg(signif.notes.type,
                                           c("include","append","tnotes","drop"))
    
    colsep <- " & "
    rowsep <- "\n"
    tabcr <- "\\\\"

    pt <- x$parmtab
    sst <- x$summary.stats
    leaders <- x$leaders
    headers <- x$headers
    eq.headers <- x$eq.headers
    outtypes <- x$outtypes
    
    ncols <- sapply(pt[1,],ncol)
    if(missing(compact) && all(ncols==1))
        compact <- TRUE

    if(compact) modelsep <- " & "
    else modelsep <- " && "
    
    res <- NULL
    has.eq.headers <- length(eq.headers) > 0

    for(j in 1:ncol(pt)){
        
        name.j <- colnames(pt)[j]
        pt.j <- pt[,j]

        ncol.j <- unique(sapply(pt.j,ncol))
        stopifnot(length(ncol.j)==1)

        for(i in 1:length(pt.j)){
            ot.ij <- outtypes[i,j]
            pt.ij <- pt.j[[i]]
            if(ot.ij == "num"){
                pt.ij <- sub("(\\*+)","^{\\1}",pt.ij)
                pt.ij <- sub("([eE])([-+]?[0-9]+)","\\\\textrm{\\1}\\2",pt.ij)
                #pt.ij <- centerAt(pt.ij,at=".",integers="dot")
                if(!useDcolumn)
                    pt.ij[] <- paste0("$",pt.ij,"$")
            } else {
                if(useDcolumn)
                    pt.ij[] <- paste0("\\multicolumn{1}{c}{",pt.ij,"}")
            }
            if(show.parmtypes){
                pt.ij <- rbind(" ",pt.ij)
            }
            pt.ij <- apply(pt.ij,1,paste,collapse=colsep)
            pt.j[[i]] <- pt.ij
        }
        pt.j <- unlist(pt.j)
        if(has.eq.headers){
            eq.header.j <- eq.headers[[name.j]]
            n.eq.j <- length(eq.header.j)
            eq.span <- ncol.j/n.eq.j
            eq.header.j <- paste0("\\multicolumn{",eq.span,"}{c}{",eq.header.j,"}")
            eq.header.j <- paste(eq.header.j,collapse=" & ")
            pt.j <- c(eq.header.j,pt.j)
        }
        if(length(sst)){
            sst.j <- sst[[j]]
            if(!useDcolumn){
                sst.j <- centerAt(sst.j,at=".",integers="dot")
                sst.j <- paste0("$",sst.j,"$")
            }
            else if(sumry.multicol){
                has.dot <- grep(".",sst.j,fixed=TRUE)
                sumry.spec <- if(useDcolumn && has.dot)
                                  paste("D{.}{",LaTeXdec,"}{",sdigits,"}",sep="")
                              else "r"
                sst.j <- paste0("\\multicolumn{1}{",sumry.spec,"}{",sst.j,"}")
            }
            sst.j <- as.matrix(sst.j)
            sst.j <- centerAt(sst.j,at=".",integers="dot")
            sst.j <- colexpand(sst.j,ncol.j)
            sst.j <- apply(sst.j,1,paste,collapse=colsep)
            pt.j <- c(pt.j,sst.j)
        }

        res <- cbind(res,pt.j)
    }
    l.leaders <- length(leaders)
    if(l.leaders){
        leaders <- lapply(leaders,ldxp)
        if(show.parmtypes){
            parmtypes <- rownames(x$parmtab)
            for(p in parmtypes){
                leaders[[p]] <- rbind(p,leaders[[p]])
            }
        }
        leaders <- do.call(rbind,leaders)

        leaders <- gsub(" x ",interaction.sep,leaders,fixed=TRUE)
        if(escape.tex)
            leaders <- LaTeXcape(leaders)
        else
            checkLaTeXcape(leaders,"row leaders")
        if(has.eq.headers)
            leaders <- rbind("",leaders)
        leaders <- format(leaders,justify="left")
        res <- cbind(leaders,res)
    }
    res <- apply(res,1,paste,collapse=modelsep)

    wdld <- as.integer(nzchar(leaders[1,]))
    l.headers <- length(headers)
    hi.rules <- character()
    if(length(headers)){

        headers1 <- Map(structure,list(""),span=ncols) # To take care of multi-eqn models
        headers <- c(headers,list(headers1))
        headers <- rev(headers)
        hspan <- list()
        
        for(k in 1:length(headers)){
            hspan[[k]] <- sapply(headers[[k]],attr,"span")
        }
        ghspan <- mkGrpSpan(hspan)
        headers <- headers[-1]
        ghspan <- ghspan[-1]

        headers <- rev(headers)
        hspan <- rev(hspan)       
        ghspan <- rev(ghspan)

        if(!compact)
            wdld <- wdld + 1
        for(k in 1:l.headers){

            header.k <- unlist(headers[[k]])
            if(escape.tex)
                header.k <- LaTeXcape(header.k)
            else
                checkLaTeXcape(header.k,"column headings")
            hspan.k <- hspan[[k]]
            ghspan.k <- ghspan[[k]]
            header.k <- paste0("\n\\multicolumn{",hspan.k,"}{c}{",header.k,"}")
            if(l.leaders)
                header.k <- c("",header.k)
            headers[[k]] <- paste(header.k,collapse=modelsep)

            n.grps <- length(hspan.k)
            if(!compact){
                hspan.k <- hspan.k + ghspan.k - 1
                hi.rule.k.start <- cumsum(c(1,hspan.k[-n.grps] + 1)) + wdld
            }
            else
                hi.rule.k.start <- cumsum(c(1,hspan.k[-n.grps])) + wdld
            hi.rule.k.end <- hi.rule.k.start + hspan.k - 1
            hi.rules.k <- paste0(cmidrule,"{",hi.rule.k.start,"-",hi.rule.k.end,"}")
            hi.rules.k <- paste(hi.rules.k,collapse="")
            hi.rules[k] <- hi.rules.k
        }

        headers <- unlist(headers)
        res <- c(headers,res)
    }
    
    res <- paste0(res,tabcr)


    l.headers <- length(headers)

    csum <- l.headers

    sectsep.at <- integer()
    sectseps <- character()

    sec.hrules.at <- integer()
    sec.hrules <- character()

    sectionrule <- midrule

    for(i in 1:nrow(pt)){
        sectseps   <- c(sectseps,   sectionrule)
        sectsep.at <- c(sectsep.at, csum)
        csum <- csum + nrow(pt[[i,1]]) 
        if(nrow(pt) > 1)
            csum <- csum + 1
    }
    if(length(sst) && any(sapply(sst,length)>0)){
        sectseps   <- c(sectseps,   sectionrule)
        sectsep.at <- c(sectsep.at, csum)
    }
    if(has.eq.headers){
        sectsep.at <- sectsep.at + 1
    }
    

    res <- .insert(res,
                   c(sec.hrules.at,sectsep.at),
                   c(sec.hrules,   sectseps)
                   )
    
    if(l.headers > 1 || l.headers > 0 && has.eq.headers){
        n.rules <- l.headers - 1
        if(has.eq.headers)
            n.rules <- n.rules + 1
        hi.rules.at <- seq.int(n.rules)
        res <- .insert(res,hi.rules.at,hi.rules)
    }
    res <- c(toprule,res,bottomrule)
    
    tab.spec <- character()
    
    if(l.leaders)
        tab.spec <- c(tab.spec,"l")
    for(j in 1:ncol(pt)){
        if(!compact) tab.spec <- c(tab.spec,"c")
        ncol.j <- ncol(pt[[1,j]])
        tab.spec <- c(tab.spec,rep(colspec,ncol.j))
    }

    total.ncol <- length(tab.spec)
    tab.spec <- paste(tab.spec,collapse="")

    signif.symbols <- x$signif.symbols
    if(length(signif.symbols)){
        signif.template <- getOption("signif.symbol.toLatex.template",
                                     signif.symbol.toLatex.default.template)
        signif.symbols <- format_signif_latex(signif.symbols,
                                              signif.template,
                                              dec=LaTeXdec,
                                              width=nchar(bottomrule))
        if(signif.notes.type=="include"){
            signif.symbols <- paste0("\\multicolumn{",total.ncol,"}{",
                                     signif.notes.spec,"}{",
                                     paste0(signif.symbols,collapse="\n"),
                                 "}\\\\")
            res <- c(res, signif.symbols)
        }
    }
    
    tabbegin <- paste("\\begin{tabular}{",tab.spec,"}",sep="")
    tabend <- "\\end{tabular}"

    res <- c(tabbegin,res,tabend)
    if(length(signif.symbols)){
        if(signif.notes.type=="append"){
            signif.symbols <- paste0(signif.symbols,collapse="\n")
            res <- c(res,"\\par",signif.symbols,"\\par")
        }
        else if(signif.notes.type=="tnotes"){
            signif.symbols <- c(
                "\\begin{tablenotes}",
                paste0(c("\\item",signif.symbols),collapse="\n"),
                "\\end{tablenotes}"
            )
            res <- c(
                "\\begin{threeparttable}",
                res,
                signif.symbols,
                "\\end{threeparttable}"
                )
        }
    }
    
    return(res)
    
}


signif.symbol.toLatex.default.template <- c("Significance: ","$$sym \\equiv p < $val$","; ")

format_signif_latex <- function(syms,tmpl,width,dec="."){
    title <- tmpl[1]
    clps <- tmpl[3]
    tmpl <- tmpl[2]
    res <- title
    empty.title <- paste(rep(" ",nchar(title)),collapse="")
    
    ns <- length(syms)
    dotrepl <- paste0("{",dec,"}")
    for(i in 1:ns){
        sym <- names(syms)[i]
        thrsh <- unname(syms[i])
        thrsh <- gsub(".",dotrepl,thrsh,fixed=TRUE)
        res.i <- sub("$sym",sym,tmpl,fixed=TRUE)
        res.i <- sub("$val",thrsh,res.i,fixed=TRUE)
        if(i < ns)
            res.i <- paste0(res.i,clps)
        len <- length(res)
        res.l <- res[len]
        n.res.l <- nchar(res.l)
        n.res.i <- nchar(res.i)
        if(n.res.l+n.res.i <= width)
            res[len] <- paste0(res.l,res.i)
        else
            res <- c(res,paste0(empty.title,res.i))
    }
    res
}

dim.memisc_mtable <- function(x){

    sapply(dimnames(x),length)
}

dimnames.memisc_mtable <- function(x){

    modn <- names(x)

    allcompo <- unique(unlist(lapply(x,names)))
    nonparnames <- c("sumstat","contrasts","xlevels","call")
    partypes <- setdiff(allcompo,nonparnames)

    parms <- lapply(x,`[`,partypes)
    parms <- do.call(cbind,parms)

    rown <- lapply(1:nrow(parms),function(i)unique(unlist(lapply(parms[i,],rownames))))

    rown <- Map(
        function(x,n)
        {
            names(x) <- rep(n,length(x))
            x
        },rown,partypes)
    
    rown <- do.call(c,rown)

    parn <- attr(x,"parameter.names")
    
    resp <- unique(unlist(lapply(parms[1,],dimnames3)))
    
    list(
        parn,
        resp,
        modn
    )
}

safe.charidx <- function(nms,i){

    lnms <- length(nms)
    if(!length(i))
        i <- nms
    else if(is.character(i))
        i <- i[i %in% nms]
    else if(is.logical(i))
        i <- nms[i]
    else if(is.numeric(i)){
        i <- i[abs(i) <= lnms]
        i <- nms[i]
    }
    i
}

"[.memisc_mtable" <- function(x, i, j, k){

    if(nargs() == 2){
        k <- if(missing(i)) return(x) else i
        i <- logical(0) 
        j <- logical(0) 
    }
    else if(nargs() == 3){
        i <- if(missing(i)) logical(0) else i
        k <- if(missing(j)) logical(0) else j
        j <- logical(0)
    }
    else { #nargs() == 4   
        i <- if(missing(i)) logical(0) else i
        j <- if(missing(j)) logical(0) else j
        k <- if(missing(k)) logical(0) else k
    }
    
    dn.x <- dimnames(x)
    parn <- dn.x[[1]]
    resp <- dn.x[[2]]
    modn <- dn.x[[3]]
        
    allcompo <- unique(unlist(lapply(x,names)))
    nonparnames <- c("sumstat","contrasts","xlevels","call")
    partypes <- setdiff(allcompo,nonparnames)
    
    nparn <- length(parn)
    nresp <- length(resp)
    nmodn <- length(modn)

    i <- safe.charidx(parn,i)
    j <- safe.charidx(resp,j)
    k <- safe.charidx(modn,k)
    kk <- match(k,names(x))
    
    y <- unclass(x)[k]
    attr.y <- attributes(x)
    attr.y$names <- k
    attr.y$stemplates <- attr.y$stemplates[kk]
    attr.y$summary.stats <- attr.y$summary.stats[kk]
    attr.y$parameter.names <- i

    if(length(attr.y$controls)){
        attr.y$controls$coefs <- attr.y$controls$coefs[kk]
        attr.y$controls$terms <- attr.y$controls$terms[kk]
    }
    
    l.y <- length(y)
    l.pt <- length(partypes)

    lt <- matrix(0,l.pt,l.y)
    
    for(k in 1:l.pt){
        pt <- partypes[k]
        for(m in 1:l.y){
            tmp <- y[[m]][[pt]]
            i.tmp <- i[i %in% rownames(tmp)]
            j.tmp <- j[j %in% dimnames3(tmp)]
            if(length(i.tmp) && length(j.tmp))
                tmp <- tmp[i.tmp,,j.tmp,drop=FALSE]
            else if(length(i.tmp))
                tmp <- tmp[i.tmp,,0,drop=FALSE]
            else if(length(j.tmp))
                tmp <- tmp[0,,j.tmp,drop=FALSE]
            else
                tmp <- tmp[0,,0,drop=FALSE]
            y[[m]][[pt]] <- tmp
            lt[k,m] <- length(tmp)
        }
        if(!any(lt>0)){
            for(m in 1:l.y)
                y[[m]][[pt]] <- NULL
        }
    }
    if(all(lt==0))
        return(NULL)
    else{
        attributes(y) <- attr.y
        return(structure(y,class="memisc_mtable"))
    }
}



##
get.summary.stats <- function(x){
    y <- attr(x,"summary.stats")
    if(is.logical(y)){
        if(isTRUE(y)){
           names(unlist(attr(x,"stemplates")))
        }
        else character()
    }
    else
        y
}

combine_mtables <- function(...){
  
    args <- list(...)
    argnames <- names(args)
    names(args) <- NULL

    if(length(argnames)>1){
        
        mg <- sapply(args,length)   
        mg1 <- lapply(mg,seq.int)
        mg0 <- c(0,mg[-length(mg)])
        model.groups <- Map(`+`,mg0,mg1)
        names(model.groups) <- argnames
    }
    else
        model.groups <- NULL
    
    coef.style <- attr(args[[1]],"coef.style")

    parameter.names <- lapply(args,attr,"parameter.names")
    parameter.names <- unique(unlist(parameter.names))
    
    summary.stats <- lapply(args,get.summary.stats)
    summary.stats <- unlist(summary.stats,recursive=FALSE)

    signif.symbols <- attr(args[[1]],"signif.symbols")       
    factor.style <- attr(args[[1]],"factor.style")
    show.baselevel <- attr(args[[1]],"show.baselevel")
    baselevel.sep <- attr(args[[1]],"baselevel.sep")
    float.style <- attr(args[[1]],"float.style")
    digits <- attr(args[[1]],"digits")
    stemplates <- do.call(c,lapply(args,attr,"stemplates"))
    sdigits <- attr(args[[1]],"sdigits")
    show.eqnames <- attr(args[[1]],"show.eqnames")
    
    args <- lapply(args,unclass)

    res <- do.call(c,args)

    controls <- lapply(args,attr,"controls")
        controls.coefs <- lapply(controls,"[[","coefs")
    controls.terms <- lapply(controls,"[[","terms")
    controls.coefs <- unlist(controls.coefs,recursive=FALSE)
    controls.terms <- unlist(controls.terms,recursive=FALSE)
    if(length(controls.terms))
        controls <- list(coefs = controls.coefs,
                         terms = controls.terms)
    else controls <- NULL
    
    collapse.controls <- unlist(lapply(args,attr,"collapse.controls"))
    collapse.controls <- any(collapse.controls)

    control.var.indicator <- attr(args[[1]],"control.var.indicator")
    
    structure(res,
            class="memisc_mtable",
            parameter.names=parameter.names,
            coef.style=coef.style,
            summary.stats=summary.stats,
            signif.symbols=signif.symbols,
            factor.style=factor.style,
            show.baselevel=show.baselevel,
            baselevel.sep=baselevel.sep,
            float.style=float.style,
            digits=digits,
            stemplates=stemplates,
            sdigits=sdigits,
            show.eqnames=show.eqnames,
            model.groups=model.groups,
            controls=controls,
            collapse.controls=collapse.controls,
            control.var.indicator=control.var.indicator
            )
}

c.memisc_mtable <- function(...) combine_mtables(...)

format_html.ftable_matrix <- function(x,
                               show.titles=TRUE,
                               digits=0,
                               format="f",
                               toprule=2,midrule=1,bottomrule=2,
                               split.dec=TRUE,
                               style=ftable_format_stdstyle,
                               margin="2ex auto",
                               varontop,varinfront,
                               ...){
  
  first.col <- c("padding-left"="0.3em")
  toprule <- c("border-top"=paste0(midrule,"px solid"))
  bottomrule <- c("border-bottom"=paste0(midrule,"px solid"))
  midrule_above <- c("border-top"=paste0(midrule,"px solid"))
  midrule <- c("border-bottom"=paste0(midrule,"px solid"))
  align.right <- c("text-align"="right")  
  align.left <- c("text-align"="left")  
  align.center <- c("text-align"="center")
  lrpad <- c("padding-left"="0.3em","padding-right"="0.3em")
  row_style <- c("border-style"="none")
  table_style <- c("border-collapse"="collapse" ,"border-style"="none")
  
  row.vars <- attr(x,"row.vars")
  col.vars <- attr(x,"col.vars")

  N <- nrow(x)
  M <- ncol(x)
  
  n <- sapply(x[,1],nrow)
  m <- sapply(x[1,],ncol)
  
  l.cv <- sapply(col.vars,length)
  max.l.cv <- max(l.cv)
  
  l.rv <- sapply(row.vars,length)
  max.l.rv <- max(l.rv)
  
  if(missing(varontop)) varontop <- (max.l.cv <= 1)
  if(missing(varinfront)) varinfront <- (max.l.rv <= 1)
  if(varontop) max.l.cv <- max.l.cv + 1
  
  d <- digits
  digits <- integer(M)
  digits[] <- d
  
  fo <- format
  format <- integer(M)
  format[] <- fo
  
  
  headers <- mapply(htfm_mkHeader,col.vars,m,
                    MoreArgs=list(max.l.cv,varontop,split.dec,style,midrule),SIMPLIFY=FALSE)
  leaders <- mapply(htfm_mkLeader,row.vars,n,
                    MoreArgs=list(max.l.rv,varinfront,style),SIMPLIFY=FALSE)

  body <- array(list(),dim=dim(x))
  
  mm <- length(headers[[1]])
  nn <- ncol(leaders[[1]])
  
  lheaders <- matrix("",nrow=mm,ncol=nn)
  lstyle <- upd_vect(style,align.left)
  
  for(i in 1:N){
    rv.i <- row.vars[[i]]

    if(l.rv[i] > 1 || (!varinfront && nzchar(names(rv.i)))){
      
      n.rv.i <- names(rv.i)
      if(l.rv[i] > 1 && varinfront)
        n.rv.i[1] <- ""

      if(i == 1)
        lheaders[mm,] <- n.rv.i
      else {
        tmp.ldr <- leaders[[i]]
        tmp.ldr <- rbind(html_td(n.rv.i,vectorize=TRUE),tmp.ldr)
        leaders[[i]] <- tmp.ldr
      }
    }
    if(i < N) {
      tmp.ldr <- leaders[[i]]
      nnn <- nrow(tmp.ldr)
      tmp.ldr[nnn,] <- lapply(tmp.ldr[nnn,],setStyle,midrule)
      leaders[[i]] <- tmp.ldr
    }
    
    for(j in 1:M){
      cv.j <- col.vars[[j]]
      tmp.bdy <- htfm_mkBody(x[[i,j]],format[[j]],digits[[j]],split.dec,style)
      if(i > 1 && (l.rv[i] > 1 || (!varinfront && nzchar(names(rv.i))))){

        if(split.dec){
          tmp. <- spltDec(rep("",ncol(tmp.bdy)))
          tmp.bdy <- rbind(html_td_spltDec(tmp.),tmp.bdy)
        }        
        else
          tmp.bdy <- rbind(html_td(rep("",ncol(tmp.bdy)),vectorize=TRUE),tmp.bdy)
      }
      if(l.cv[j] > 1 || (!varontop && nzchar(names(cv.j)))){
        
        tmp.bdy <- cbind(html_td(rep("",nrow(tmp.bdy)),vectorize=TRUE),tmp.bdy)
      }
      if(i < N){
        nnn <- nrow(tmp.bdy)
        tmp.bdy[nnn,] <- lapply(tmp.bdy[nnn,],setStyle,midrule)
      }
      body[[i,j]] <- tmp.bdy
    }
  }

  lheaders <- array(html_td(lheaders,vectorize=TRUE,style=css(lstyle)),
          dim=dim(lheaders))

  leaders <- rbind(lheaders,do.call(rbind,leaders))
    
  ans <- list()
  for(j in 1:M){
    ans.j <- body[,j]
    ans.j <- do.call(rbind,ans.j)
    ans.j <- apply(ans.j,1,as.html_group)
    ans[[j]] <- as.html_group(c(headers[[j]],ans.j))
  }
  ans <- do.call(cbind,ans)
  ans <- cbind(leaders,ans)
  mmm <- nrow(ans)
  ans[1,] <- lapply(ans[1,],setStyle,toprule)
  ans[mm,] <- lapply(ans[mm,],setStyle,midrule)
  ans[mmm,] <- lapply(ans[mmm,],setStyle,bottomrule)
  ans <- apply(ans,1,as.html_group)
    
  ans <- html_tr(ans,vectorize=TRUE,style=as.css(row_style))
  
  if(length(margin))
    table_style <- c(table_style,margin=margin)
  ans <- html_table(ans,class="ftable",style=as.css(table_style))

  ans <- as.character(ans)
  return(ans)
  
}


htfm_mkHeader <- function(col.vars,m,max.l.cv,varontop,split.dec,style,midrule){
 
  align.right <- c("text-align"="right")  
  align.left <- c("text-align"="left")  
  align.center <- c("text-align"="center")
 
  nms.cv <- names(col.vars)
  n.col.vars <- length(col.vars)
  
  hstyle <- upd_vect(style,align.center)
  header <- vector("list",length=n.col.vars)
  mm <- 1
  for(i in rev(1:n.col.vars)){
    
    cv <- col.vars[[i]]
    ncv <- length(cv)
    if(split.dec)
      colspan <- mm*3
    else
      colspan <- mm
    mm <- mm*ncv
    cv <- rep(cv,m%/%mm)
    
    header.i <- html_td(cv,vectorize=TRUE,style=css(hstyle),
                        colspan=colspan) 
    header.i <- setStyle(header.i,midrule)
    
    if(n.col.vars > 1 || !varontop){
      
      if(i == 1 && varontop || !length(nms.cv)){
        htmp <- html_td("",
                        style=css(upd_vect(hstyle,align.left)))
      }
      else {
        if(!nzchar(nms.cv[i])){
          htmp <- NULL
        }
        else
          htmp <- html_td(paste0(nms.cv[i],":"),
                          style=css(upd_vect(hstyle,align.left)))
      }    
      if(length(htmp))
        header.i <- c(htmp,header.i)
    }
    header[[i]] <- header.i
  }
  
  if(varontop){

    if(split.dec)
      colspan <- mm*3
    else
      colspan <- mm
    
    if(n.col.vars > 1){
      htmp1 <- html_td("",
                       style=css(upd_vect(hstyle)))
      htmp2 <- html_td(nms.cv[1],
                      style=css(upd_vect(hstyle,align.center,midrule)),
                      colspan=colspan)
      htmp <- c(htmp1,htmp2)
    }
    else if(nzchar(nms.cv[1])){
      htmp <- html_td(names(col.vars)[1],
                      style=css(upd_vect(hstyle,align.center,midrule)),
                      colspan=colspan)
    }
    else {
      htmp <- html_td("",
                      style=css(upd_vect(hstyle,align.center)),
                      colspan=colspan)
    }
    header <- c(list(htmp),header)
  }
  
  tmp.header <- header

  if(split.dec)
    colspan <- m*3
  else
    colspan <- m
  if(n.col.vars > 1 || !varontop)
    colspan <- colspan + 1
  
  header <- rep(list(html_td("",colspan=colspan)),max.l.cv)
  
  i <- seq(to=max.l.cv,length=length(tmp.header))
  header[i] <- tmp.header
  header
}

htfm_mkLeader <- function(row.vars,n,max.l.rv,varinfront,style) {
  
  align.left <- c("text-align"="left")  
  n.row.vars <- length(row.vars)
  nms.row.vars <- names(row.vars)
  lstyle <- upd_vect(style,align.left)
  leader <- matrix("",nrow=n,ncol=n.row.vars)
  
  for(ii in 1:n.row.vars){
    
    rv <- row.vars[[ii]]
    rep.rv <- if(ii==1)1 else nn.rv
    rv <- rep(rv,rep.rv)
    nn.rv <- length(rv)
    rv.n <- n/nn.rv
    pos <- (1:nn.rv)*rv.n
    pos <- pos - rv.n + 1
    leader[pos,ii] <- rv
  }
  if(varinfront){
    lleader <- character(n)
    if(nzchar(nms.row.vars[1]))
      lleader[1] <- nms.row.vars[1]
    else if(n==1){
      lleader[1] <- leader[1,1]
      leader[1,1] <- ""
    }
      
    leader <- cbind(lleader,leader)
    if(n.row.vars > 1){
      hleader <- character(n.row.vars+1)
      ii <- 2:n.row.vars
      hleader[ii] <- nms.row.vars[ii]
      leader <- rbind(hleader,leader)
    }
  }
  
  tmp.leader <- leader
  leader <- matrix("",nrow=nrow(leader),ncol=max.l.rv+as.integer(varinfront))
  ii <- seq(to=ncol(leader),length=ncol(tmp.leader))
  leader[,ii] <- tmp.leader
  
  leader <- array(html_td(leader,vectorize=TRUE,style=css(lstyle)),
                  dim=dim(leader))
  leader
}

htfm_mkBody <- function(x,format,digits,split.dec,style){
  
  n <- nrow(x)
  m <- ncol(x)
  
  d <- digits
  digits <- integer(m)
  digits[] <- d
  
  fo <- format
  format <- integer(m)
  format[] <- fo
  
  body <- array(list(),dim=c(n,m))
  
  for(i in 1:m){
    tmp <- formatC(x[,i],digits=digits[i],format=format[i])
    if(getOption("html.use.ampersand",FALSE))
      tmp <- gsub("-","&minus;",tmp,fixed=TRUE)
    else
      tmp <- gsub("-","\u2212",tmp,fixed=TRUE)
    if(!split.dec)
      body[,i] <- html_td(tmp,vectorize=TRUE,style=css(style))
    else {
      tmp <- spltDec(tmp)
      tmp <- html_td_spltDec(tmp,style=css(style))
      body[,i] <- tmp
    }
  }
  body
}

## labels ################################################################################

setMethod("initialize","value.labels",function(.Object,...){
  args <- list(...)
  values <- args$values
  labels <- args[[1]]
  ii <- order(values)
  values <- values[ii]
  labels <- labels[ii]
  labels <- iconv(labels,from="",to="UTF-8")
  Encoding(labels) <- "UTF-8"
  .Object@.Data <- labels
  .Object@values <- values
  if(validObject(.Object)) return(.Object)
})

setValidity("value.labels",function(object){
    if(length(object@.Data)==length(object@values)){
        if(length(unique(object@.Data))<=length(unique(object@values))) TRUE
        else paste("More labels than labelled values:\n",
                 "   ",length(unique(object@.Data)),"unique labels\n",
                 "   ",length(unique(object@values)),"labelled values")
    }
    else paste("There are",length(object@values),"values, but",
               length(object@.Data),"labels")
})

setMethod("labels","NULL",function(object,...)NULL)

setMethod("labels","item",function(object,...)object@value.labels)
setMethod("labels<-",signature(x="item",value="ANY"),function(x,value){
  x@value.labels<-as(value,"value.labels")
  x
})

setMethod("labels<-",signature(x="ANY",value="NULL"),function(x,value) x)

setMethod("labels<-",signature(x="item",value="NULL"),function(x,value){
  x@value.labels<-NULL
  x
})

setMethod("labels<-",signature(x="vector",value="NULL"),function(x,value) x)

setMethod("labels<-",signature(x="vector",value="ANY"),function(x,value)as.item(x,
  labels=as(value,"value.labels")))

format.value.labels <- function(x,...){
  paste(format(x@values,justify="right"),format(sQuote(x@.Data),justify="left"))
}
print.value.labels <- function(x,...) writeLines(format.value.labels(x))
setMethod("show","value.labels",function(object){
  writeLines(c("",
               " Values and labels:",
               "",
                paste("  ",format.value.labels(object)),
                ""))
})

setMethod("[",signature(x="value.labels",i="numeric",j="missing",drop="missing"),
  function(x,i)  new("value.labels",x@.Data[i], values=x@values[i])
)

setMethod("[",signature(x="value.labels",i="logical",j="missing",drop="missing"),
  function(x,i)
  new("value.labels",x@.Data[i], values=x@values[i]))

setAs(from="numeric",to="value.labels",function(from,to){
    if(length(names(from))){
        if(any(dups <- duplicated(names(from))))
            warning("Duplicate labels ",
                    paste(sQuote(unique(names(from)[dups])),collapse=" "),
                    call.=FALSE)
        if(any(duplicated(from))) warning("Duplicate values",call.=FALSE)
        new("value.labels",names(from),values=unname(from))
    }
    else new("value.labels",character(0),values=logical(0))
})

setAs(from="character",to="value.labels",function(from,to){
    if(length(names(from))){
        if(any(dups <- duplicated(names(from))))
            warning("Duplicate labels",
                    paste(sQuote(unique(names(from)[dups])),collapse=" "),
                    call.=FALSE)
        if(any(duplicated(from))) warning("Duplicate values",call.=FALSE)
        new("value.labels",names(from),values=unname(from))
    }
    else new("value.labels",character(0),values=logical(0))
})


setAs(from="value.labels",to="numeric",function(from,to)
  structure(as.vector(from@values),names=from@.Data)
)

setAs(from="value.labels",to="character",function(from,to)
  from@.Data
)


setMethod("as.vector","value.labels",function(x,mode="any")
  structure(
    as.vector(x@values,mode=mode),
    names=x@.Data
    )
)


setMethod("Arith",signature(e1="value.labels",e2="ANY"),
  function(e1,e2){
    if(!is(e2,"value.labels"))
      e2 <- as(e2,"value.labels")
    vals <- e1@values
    labs <- e1@.Data
    newvals <- e2@values
    newlabs <- e2@.Data
    upd.vals <- intersect(vals,newvals)
    upd.labs <- newlabs[newvals %in% upd.vals]
    tmp <- setdiff(newvals,upd.vals)
    newlabs <- newlabs[match(tmp,newvals,nomatch=0L)]
    newvals <- tmp
    if(.Generic == "+"){
      labs <- c(labs[vals %nin% upd.vals],upd.labs,newlabs)
      vals <- c(vals[vals %nin% upd.vals],upd.vals,newvals)
      ii <- order(vals)
      vals <- vals[ii]
      labs <- labs[ii]
    }
    else if(.Generic == "-"){
      labs <- labs[vals %nin% upd.vals]
      vals <- vals[vals %nin% upd.vals]
      ii <- order(vals)
      vals <- vals[ii]
      labs <- labs[ii]
    }
    else stop("unsupported operator ",dQuote(.Generic))
    new("value.labels",labs,
      values = vals)
})



setMethod("has.value.labels","ANY",function(x)FALSE)
setMethod("has.value.labels","item",function(x)length(x@value.labels)>0)

setMethod("is.labelled",signature(x="item.vector"),function(x){
  if(!length(x@value.labels)) return(logical(length(x)))
  cl <- x@value.labels
  x %in% cl@values
})

add_value_labels <- function(x,v){
    if(length(labels(x)))
        labels(x) <- labels(x) + v
    else
        labels(x) <- v
    x
}

car_recode <- function (var, recodes, as.factor.result, levels)
  stop("package 'car' is not available")

memisc_env <- environment()


.onLoad <- function(lib,pkg){

  if(requireNamespace("car",quietly = TRUE)){
    assign("car_recode",car::recode,envir=memisc_env)
  }

  options(coef.style="default",
          show.baselevel=TRUE,
          baselevel.sep="/",
          factor.style="($f): ($l)",
          float.style="f",
          signif.symbols=c(
              "***"=.001,
              "**"=.01,
              "*"=.05
          ),
          labelled.factor.coerce.NA=FALSE,
          html.use.ampersand=FALSE,

          memisc.repr_html=TRUE,
          memisc.repr_latex=TRUE,
          mtable.show.eqnames=NA,
          codebook.chunk.size=1000,
          Simulation.chunk.size=1000,
          print.use.value.labels=TRUE,
          show.max.obs=25,
          measurement.adapt.threshold=.75,
          measurement.adapt.default="nominal"
  )

    names(.SummaryStats) <- paste0("summary.stats.",names(.SummaryStats))
    do.call(options,.SummaryStats)
}


.onUnload <- function(libpath)
{
    library.dynam.unload("memisc", libpath)
}

sQuote <- function (x)
{
    if (length(x) == 0)
        return(character())
    paste("'", x, "'", sep = "")
}

dQuote <- function (x)
{
    if (length(x) == 0)
        return(character())
    paste("\"", x, "\"", sep = "")
}

quickInteraction <- function(by){
  if(is.list(by)){
    n.arg <- length(by)
    f <- 0L
    uf <- 0L
    for(i in rev(1:n.arg)){
      y <- by[[i]]
      y <- as.numeric(y)
      uy <- unique(na.omit(y))
      y <- match(y,uy,NA)
      l <- length(uy)
      f <- f*l + y - 1
      uf <- unique(na.omit(f))
      f <- match(f,uf,NA)
      uf <- seq(length(uf))
    }
  }
  else {
    by <- as.numeric(by)
    uf <- unique(na.omit(by))
    f <- match(by,uf,NA)
    uf <- seq(length(uf))
  }
  return(structure(f,unique=uf))
}

fapply <- function(formula,data,...) {
    .Deprecated("withGroups")
    UseMethod("fapply",data)
}

fapply.default <- function (formula,
                        data,
                        subset=NULL,
                        names=NULL,
                        addFreq=TRUE,
                        ...)
{
    
    .Deprecated("withGroups")
    m <- match.call(expand.dots = FALSE)
    dots <- m$...
    if(attr(terms(formula,data=data),"response")){
      fcall <- formula[[2]]
      formula <- formula[-2]
      }
    else
      fcall <- NULL

    #names(m)[2] <- "formula"
    m$formula <- formula
    if (is.matrix(data))
        m$data <- data <- as.data.frame(data)

    m$... <- m$exclude <- m$drop.unused.levels <- m$names <- m$addFreq <- NULL
    #m <- m[c(1,3,2)]
    m[[1]] <- as.name("model.frame")
    if(!length(subset)) m$subset <- NULL
    else m$subset <- subset
    #m$na.action <- na.action

    m$data <- data
    by <- eval(m, parent.frame())

    omitted <- attr(by,"na.action")
    if(as.character(formula[[2]])[1]==".")
      by <- by[setdiff(names(by),all.vars(fcall))]

    if(length(fcall)){
      if(length(fcall)==1){
        makeTableCall <- FALSE
        fcall.c <- as.character(fcall)
        if(is.table(data)
            && fcall.c  %in% names(dimnames(data)))
            makeTableCall <- TRUE
        if(is.data.frame(data)
            && is.factor(data[[fcall.c]]))
            makeTableCall <- TRUE
        if(is.environment(data)
            && exists(fcall.c,envir=data)
            && is.factor(get(fcall.c,envir=data)))
            makeTableCall <- TRUE
        if(makeTableCall)
          fcall <- as.call(c(as.symbol("table"),fcall))
        else
          fcall <- as.call(c(as.symbol("sum"),fcall))
      }
      if(addFreq){
        if(length(fcall) > 1 &&
            as.character(fcall[[1]]) %in% c("table","Table","percent","nvalid") &&
            !("weights" %in% names(fcall))
          ){
          if(is.table(data) || (is.data.frame(data) && "Freq" %in% names(data))){
            fcall[[3]] <- as.symbol("Freq")
            if(as.character(fcall[[1]])=="table")
              fcall[[1]] <- as.symbol("Table")
            by <- by[setdiff(names(by),all.vars(fcall))]
            }
        }
      }
      if(length(dots)) fcall <- as.call(c(as.list(fcall),dots))
      resp.var.formula <- parse(text=paste("~",paste(all.vars(fcall),collapse="+")))[[1]]
      m$formula <- resp.var.formula
      m$na.action <- na.pass
      data <- eval(m, enclos=parent.frame())
      if(length(omitted))
        data <- data[-omitted,,drop=FALSE]
      rows <- seq(nrow(data))
    }
    else rows <- seq(nrow(by))

    BY <- quickInteraction(by)
    uBY <- attr(BY,"unique")
    fntBY <- is.finite(BY)
    BY <- BY[fntBY]
    by <- by[fntBY,,drop=FALSE]
    data <- data[fntBY,,drop=FALSE]

    if(length(fcall)>1)
      rows <- seq_len(nrow(data))
    else
      rows <- seq_len(length(BY))

    rows <- split.default(rows,BY)

    good <- TRUE
    data <- data[all.vars(fcall)]
    if(length(fcall)>1){
      res <- lapply(rows,function(i)
                    eval(fcall,
                    data[i,,drop=FALSE],
                    enclos=parent.frame()
                ))
      good <- sapply(res,length) > 0
      if(!all(good))
        res <- res[good]
      if(as.character(fcall[[1]]) %in% c("table","Table")){
        if(length(dim(res[[1]]))<2) res <- lapply(res,c)
      }
    } else
    if(length(fcall)==1){
      res <- c(rowsum(x=data,group=BY,reorder=FALSE,na.rm=FALSE))
      if(missing(names)) names <- "Freq"
    }
    else {
      res <- tabulate(BY,nbins=length(uBY))
      if(missing(names)) names <- "Freq"
    }
    urows <- sapply(rows,function(ix)ix[1])
    by <- by[urows,,drop=FALSE]
    if(!all(good))
      by <- by[good,,drop=FALSE]
    ii <- do.call("order",rev(by))

    structure(res[ii],
      by=by[ii,,drop=FALSE],
      formula=formula
      )
}

setMethod("Table",signature(x="atomic"),
      function(x,weights=NULL,counts=TRUE,percentage=FALSE,...) {
        if(!(counts || percentage)) stop("either counts or percentage must be TRUE")
          if(!length(weights)){
            tab <- drop(table(x))
          } else {
            good <- is.finite(weights) & is.finite(x)
            tmp <- rowsum(weights[good],x[good])
            tab <- drop(tmp)
          }
        if(percentage) {
          perc <- 100 * tab/sum(tab)
        }
        structure(if(counts && percentage)
            cbind(Counts=tab,Percent=perc)
          else if(percentage)
            perc
          else
            tab
            ,class="table")
      })

setMethod("Table",signature(x="factor"),
      function(x,weights=NULL,counts=TRUE,percentage=FALSE,...) {
        if(!(counts || percentage)) stop("either counts or percentage must be TRUE")
          if(!length(weights)){
            tab <- drop(table(x))
          } else {
            good <- is.finite(weights) & is.finite(x)
            tmp <- rowsum(weights[good],x[good])
            tab <- structure(rep(0,nlevels(x[good])),names=levels(x[good]))
            tab[rownames(tmp)] <- tmp[]
          }
        if(percentage) {
          perc <- 100 * tab/sum(tab)
        }
        structure(if(counts && percentage)
            cbind(Counts=tab,Percent=perc)
          else if(percentage)
            perc
          else
            tab
            ,class="table")
      })

setMethod("Table",signature(x="item.vector"),
    function(x,
            weights=NULL,
            counts=TRUE,
            percentage=(style=="codebook"),
            style=c("table","codebook","nolabels"),
            include.missings=(style=="codebook"),
            missing.marker=if(style=="codebook") "M" else "*",
            ...){
      if(!(counts || percentage)) stop("either counts or percentage must be TRUE")
      is.m <- is.missing(x)
      isNA <- is.na(x)
      style <- match.arg(style)
      if (style %in% c("table","codebook")) {
        vl <- labels(x)
        if(length(vl)){
          vvl <- vl@values
          lvl <- vl@.Data
          valid <- !is.missing2(vvl,x@value.filter)
          i <- match(x@.Data,vvl,nomatch=0L)
          if(!length(weights)){
            tab <- tabulate(i,nbins=length(vvl))
            names(tab) <- as.character(vl@values)
            }
          else {
            f <- factor(x@.Data,levels=vvl)
            good <- is.finite(weights) & is.finite(f)
            tmp <- rowsum(weights[good],f[good])
            tab <- structure(rep(0,nlevels(f[good])),names=levels(f))
            tab[rownames(tmp)] <- tmp[]
          }
          lab <- if(style=="codebook") sQuote(vl@.Data) else vl@.Data
        }
        else {
          valid <- logical(0)
          tab <- c()
          lab <- c()
          i <- logical(length(x))
        }
        if(!length(weights)){
          ovld <- sum(!is.m & !i)
          omiss <- sum(is.m & !i & !isNA)
          NAs <- sum(isNA)
        }
        else {
          good <- is.finite(weights)
          weights <- weights[good]
          is.m <- is.m[good]
          i <- i[good]
          isNA <- isNA[good]
          ovld <- sum(weights*(!is.m & !i))
          omiss <- sum(weights*(is.m & !i & !isNA))
          NAs <- sum(weights*(isNA))
        }
        #browser()
        if(ovld){
          tab <- c(tab," "=ovld)
          if(style=="codebook")
            lab <- c(lab,"(unlab.val.)")
          else {
            lab <- if(length(vl)) c(lab,"Other valid") else c(lab,"Valid")
          }
          valid <- c(valid,TRUE)
          }
        if(include.missings){
          if(omiss){
            tab <- c(tab," "=omiss)
            if(style == "codebook")
              lab <- c(lab,"(unlab.mss.)")
            else {
              if(length(vl)){
                lab <- c(lab,"Other missing")
              } else {
                lab <- c(lab,"Missing")
                missing.marker <- ""
              }
            }
            valid <- c(valid,FALSE)
            }
          if(NAs){
            tab <- c(tab,"NA"=NAs)
            if(style == "codebook")
              lab <- c(lab,"")
            else {
                lab <- c(lab,"NA")
              if(!length(vl)){
                missing.marker <- ""
              }
            }
            valid <- c(valid,FALSE)
            }
          if(length(missing.marker)){
            missing.marker <- missing.marker[1]
            if(style=="codebook"){
              valid.marker <- paste(rep(" ",nchar(missing.marker)),collapse="")
              lab <- paste(ifelse(valid,valid.marker,missing.marker),lab)
              names(tab) <- paste(format(names(tab),justify="right"),format(lab,justify="left"))
            }
            else {
              lab <- paste(ifelse(valid,"",missing.marker),lab,sep="")
              names(tab) <- lab
            }
          }
        } else {
          if(style=="codebook")
            names(tab) <- paste(format(names(tab),justify="right"),format(lab,justify="left"))
          else
            names(tab) <- lab
          tab <- tab[valid]
        }
      }
      else { # style == "nolabels"
        if(include.missings){
          if(!length(weights)){
            NAs <- sum(isNA)
            tab <- table(x@.Data)
          }
          else {
            good <- is.finite(weights)
            weights <- weights[good]
            NAs <- sum(weights*isNA)
            tab <- rowsum(weights,x@.Data[good])
          }
          if(NAs)
            tab <- c(tab,"NA"=NAs)
          if(length(missing.marker)){
            missing.marker <- missing.marker[1]
            valid <- !is.missing2(sort(unique(x@.Data)),x@value.filter)
            if(NAs)
              valid <- c(valid,FALSE)
            lab <- paste(ifelse(valid,"",missing.marker),names(tab),sep="")
            names(tab) <- lab
          }
        }
        else if(!length(weights)){
            tab <- table(x@.Data[!is.m])
          }
          else {
            good <- is.finite(weights) & !is.m
            weights <- weights[good]
            tab <- rowsum(weights,x@.Data[good])
          }
      }
      if(include.missings){
        if(percentage && counts) {
          vperc <- rep(NA,length(tab))
          vtab <- tab[valid]
          Nvalid <- sum(vtab)
          if(Nvalid) vperc[valid] <- 100 * vtab/Nvalid
          else vperc[valid] <- 0
          tperc <- 100 * tab/sum(tab)
          tab <- cbind(Counts=tab,Valid=vperc,Total=tperc)
          }
        else if(percentage) {
          vperc <- rep(NA,length(tab))
          vtab <- tab[valid]
          Nvalid <- sum(vtab)
          if(Nvalid) vperc[valid] <- 100 * vtab/Nvalid
          else vperc[valid] <- 0
          tperc <- 100 * tab/sum(tab)
          tab <- cbind(Valid=vperc,Total=tperc)
          rownames(tab) <- names(tperc)
        }
      }
      else {
        if(percentage && counts) {
          perc <- 100 * tab/sum(tab)
          tab <- cbind(Counts=tab,Percent=perc)
          }
        else if(percentage) {
          tab <- 100 * tab/sum(tab)
        }
      }
     structure(tab,class="table")
})


wmean <- function(x,w){
    if(length(w)>0) weighted.mean(x,w)
    else mean(x)
}


NULLfunc <- function(...) NULL

Descriptives_numeric <- function(x,weights=NULL,...){

     isna <- is.na(x)
     if(length(weights)>0)
         w <- weights[!isna]
     else
         w <- NULL
     x <- x[!isna]
     m.1 <- tryCatch(wmean(1L*x,w),error=NULLfunc)
     x.cent <- tryCatch(x-m.1,error=NULLfunc)
     m.2 <- tryCatch(wmean(x.cent^2,w),error=NULLfunc)
     m.3 <- tryCatch(wmean(x.cent^3,w),error=NULLfunc)
     m.4 <- tryCatch(wmean(x.cent^4,w),error=NULLfunc)
     c(
      Min=min(x,na.rm=TRUE),
      Max=max(x,na.rm=TRUE),
      Mean=m.1,
      "Std.Dev."=tryCatch(sqrt(m.2),error=NULLfunc),
      Skewness=tryCatch(m.3/m.2^(3/2),error=NULLfunc),
      Kurtosis=tryCatch(m.4/m.2^2-3,error=NULLfunc)
     )
}

setMethod("Descriptives",signature(x="atomic"),
          Descriptives_numeric)

setMethod("Descriptives",signature(x="ANY"),
function(x,weights=NULL,...){
    if(mode(x)=="numeric")
        Descriptives_numeric(x,weights,...)
    else NULL
})


setMethod("Descriptives",signature(x="item.vector"),
    function(x,weights=NULL,...){
     miss <- is.missing(x)
     if(length(weights)>0)
         w <- weights[!miss]
     else
         w <- NULL
     x <- x@.Data[!miss]
     m.1 <- wmean(x,w)
     x.cent <- x-m.1
     m.2 <- wmean(x.cent^2,w)
     m.3 <- wmean(x.cent^3,w)
     m.4 <- wmean(x.cent^4,w)
     c(
      Min=min(x,na.rm=TRUE),
      Max=max(x,na.rm=TRUE),
      Mean=m.1,
      "Std.Dev."=sqrt(m.2),
      Skewness=m.3/m.2^(3/2),
      Kurtosis=m.4/m.2^2-3
     )
})

Moments <- function(x)
  c(
    mm.1 = mean(x),
    mm.2 = mean(x^2),
    mm.3 = mean(x^3),
    mm.4 = mean(x^4),
    N = length(x)
)

ftab_mat_html_env <- new.env()
ftab_mat_html_env$counter <- 1

style_ftab_matrix_group_rule <- function(id=NULL,class=NULL,rulewidth=1){
    class_or_id <- class_id_selector(id=id,class=class)
    rulewidth <- paste0(rulewidth,"px")
    selector_tmpl <- "table<<class-or-id>> td:nth-child(n of .header)"
    selector <- fillin(selector_tmpl,c("class-or-id"=class_or_id))
    style_tmpl <- paste("<<selector>> {",
                        "     border-bottom: <<rulewidth>> solid;",
                        "}",sep="\n")
    fillin(style_tmpl,c(selector=selector,rulewidth=rulewidth))
}

format_html.ftable_matrix <- function(x,
                                      show.titles=TRUE,
                                      digits=0,
                                      format="f",
                                      toprule=2,midrule=1,bottomrule=2,
                                      split.dec=TRUE,
                                      style=ftable_format_stdstyle,
                                      margin="2ex auto",
                                      varontop,
                                      varinfront,
                                      grouprules=1,
                                      multi_digits=NULL,
                                      ...){
    
    row.vars <- attr(x,"row.vars")
    col.vars <- attr(x,"col.vars")

    nms.cv <- names(col.vars)
    n.col.vars <- length(col.vars)

    N <- nrow(x)
    M <- ncol(x)
    
    n <- sapply(x[,1],nrow)
    m <- sapply(x[1,],ncol)
    
    l.cv <- sapply(col.vars,length)
    max.l.cv <- max(l.cv)
    
    l.rv <- sapply(row.vars,length)
    max.l.rv <- max(l.rv)

    if(missing(varontop)) varontop <- (max.l.cv <= 1)
    if(missing(varinfront)) varinfront <- (max.l.rv <= 1)
    
    if(!missing(multi_digits)){
        if(is.list(multi_digits)){
            digits <- multi_digits
        }
        else {
            digits <- rep(list(multi_digits),M)
        }
    }
    else {
        d <- digits
        digits <- integer(M)
        digits[] <- d
    }
    
    fo <- format
    format <- integer(M)
    format[] <- fo
    
    
    headers <- mapply(htfm_mkHeader,col.vars,m,
                      MoreArgs=list(max.l.cv,varontop,split.dec),SIMPLIFY=FALSE)
    leaders <- mapply(htfm_mkLeader,row.vars,n,
                      MoreArgs=list(max.l.rv,varinfront),SIMPLIFY=FALSE)

    body <- array(list(),dim=dim(x))
    
    mm <- length(headers[[1]])
    nn <- ncol(leaders[[1]])
    
    lheaders <- matrix("",nrow=mm,ncol=nn)

    dot_cols <- integer(0)
    col_sums <- 0
    
    for(i in 1:N){
        rv.i <- row.vars[[i]]

        if(l.rv[i] > 1 || (!varinfront && nzchar(names(rv.i)))){
            
            n.rv.i <- names(rv.i)
            if(l.rv[i] > 1 && varinfront)
                n.rv.i[1] <- ""

            if(i == 1){
                lheaders[mm,] <- n.rv.i
            }
            else {
                tmp.ldr <- leaders[[i]]
                tmp.ldr <- rbind(html_td(n.rv.i,vectorize=TRUE),tmp.ldr)
                leaders[[i]] <- tmp.ldr
            }
        }
        if(i < N) {
            tmp.ldr <- leaders[[i]]
            leaders[[i]] <- tmp.ldr
        }
        
        for(j in 1:M){
            cv.j <- col.vars[[j]]
            tmp.bdy <- htfm_mkBody(x[[i,j]],format[[j]],digits[[j]],split.dec)
            if(i == 1 && split.dec) {
                nm.cv.j <- names(cv.j)
                nz.nm.cv.j <- length(nm.cv.j) && all(nzchar(nm.cv.j))
                m.j <- ncol(tmp.bdy)
                if((!varontop || max.l.cv > 1) && nz.nm.cv.j) {
                    col_sums <- col_sums + 1
                    # message(sprintf("Incremented col_sums j=%d varontop=%d max.l.cv=%d col_sums=%d",j,varontop,max.l.cv,col_sums))
                    # print(dot_cols)
                }
                dot_cols <- c(dot_cols, col_sums + (1:m.j-1)*3 + 2)
                col_sums <- col_sums + 3*m.j
            }
            if(i > 1 && (l.rv[i] > 1 || (!varinfront && nzchar(names(rv.i))))){

                if(split.dec){
                    tmp. <- spltDec(rep("",ncol(tmp.bdy)))
                    tmp.bdy <- rbind(html_td_spltDec(tmp.),tmp.bdy)
                }        
                else
                    tmp.bdy <- rbind(html_td(rep("",ncol(tmp.bdy)),vectorize=TRUE),tmp.bdy)
            }
            if(l.cv[j] > 1 || (!varontop && nzchar(names(cv.j)))){
                
                tmp.bdy <- cbind(html_td(rep("",nrow(tmp.bdy)),vectorize=TRUE),tmp.bdy)
            }
            if(i < N){
                nnn <- nrow(tmp.bdy)
            }
            body[[i,j]] <- tmp.bdy
        }
    }

    lheaders <- array(html_td(lheaders,vectorize=TRUE),
                      dim=dim(lheaders))

    leaders <- rbind(lheaders,do.call(rbind,leaders))
    
    res <- list()
    for(j in 1:M){
        res.j <- body[,j]
        res.j <- do.call(rbind,res.j)
        res.j <- apply(res.j,1,as.html_group)
        res[[j]] <- as.html_group(c(headers[[j]],res.j))
    }
    res <- do.call(cbind,res)
    res <- cbind(leaders,res)

    nh <- nrow(lheaders)
    nl <- ncol(leaders)
    ntot <- nrow(res)
    
    res <- apply(res,1,as.html_group)
    
    res <- html_tr(res,vectorize=TRUE)
    
    ftab_id <- paste0("ftab-matrix-",ftab_mat_html_env$counter)
    res <- html_table(res,class="ftable",id=ftab_id)

    ftab_mat_html_env$counter <- ftab_mat_html_env$counter + 1    

    style_global <- style_ftab_global(id=ftab_id,style=style,margin=margin)
    style_toprule <- style_ftab_rule(id=ftab_id,rulewidth=toprule,top=TRUE,
                                       rows=1)
    style_bottomrule <- style_ftab_rule(id=ftab_id,rulewidth=bottomrule,bottom=TRUE,
                                          rows=ntot)
    nrows <- sapply(x,nrow)
    dim(nrows) <- dim(x)
    nrows <- nrows[,1]

    if(length(nrows) > 1){
        if(varinfront){
            midrule_rows <- cumsum(c(nh,nrows))
        }
        else {
            nrows <- nrows[-1]
            l.nr <- length(nrows)
            if(grouprules > 1)
                midrule_rows <- c(nh,nrows + (rep(1:l.nr,each=2) - c(1,0)) + nh)
            else
                midrule_rows <- c(nh,nrows + 1:l.nr - 1 + nh)
        }
    }
    else midrule_rows <- nh

    style_midrule <- style_ftab_rule(id=ftab_id,rulewidth=midrule,bottom=TRUE,
                                          rows=midrule_rows)
    style_grouprule <- style_ftab_matrix_group_rule(id=ftab_id,rulewidth=midrule)

    style_content <- paste(
        style_global,
        style_toprule,
        style_midrule,
        style_bottomrule,
        style_grouprule,
        sep="\n"
    )

    if(split.dec){
        dot_style <- "
         padding-left: 0px;
         padding-right: 0px;
         text-align: center; 
         width: .2ex;
         /*background-color: blue;*/
        "
        before_dot_style <- "
         padding-left: 0.5ex;
         padding-right: 0px;
         text-align: right; 
         /*background-color: red;*/
        "
        behind_dot_style <- "
         padding-left: 0px;
         padding-right: 0.5ex;
         text-align: left; 
         /*background-color: green;*/
        "
        dot_cols <- dot_cols + nl
        style_dots <- style_df_cols(id=ftab_id,cols=dot_cols,style=dot_style)
        style_before_dots <- style_df_cols(id=ftab_id,cols=dot_cols-1,style=before_dot_style)
        style_behind_dots <- style_df_cols(id=ftab_id,cols=dot_cols+1,style=behind_dot_style)
        style_content <- paste(style_content,
                               style_dots,
                               style_before_dots,
                               style_behind_dots,sep="\n")
    }
    
    header_style <- "
         padding-left: 0.5ex;
         padding-right: 0.5ex;
         text-align: center; 
         /*background-color: gray;*/
        "
    style_header <- style_ftab_header(id=ftab_id,style=header_style)
    style_content <- paste(style_content,
                           style_header,
                           "\n")
    
    style_element <- html("style",style_content,linebreak=TRUE)
    res <- html_group(style_element,res)
    res <- as.character(res)
    return(res)
}


htfm_mkHeader <- function(col.vars,m,max.l.cv,varontop,split.dec){
 
  nms.cv <- names(col.vars)
  n.col.vars <- length(col.vars)
  
  header <- vector("list",length=n.col.vars)
  mm <- 1
  for(i in rev(1:n.col.vars)){
    
    cv <- col.vars[[i]]
    ncv <- length(cv)
    if(split.dec)
      colspan <- mm*3
    else
      colspan <- mm
    mm <- mm*ncv
    cv <- rep(cv,m%/%mm)
    
    header.i <- html_td(cv,vectorize=TRUE,colspan=colspan,class="header") 
    
    if(n.col.vars > 1 || !varontop){
      
      if(i == 1 && varontop || !length(nms.cv)){
        htmp <- html_td("")
      }
      else {
        if(!nzchar(nms.cv[i])){
          htmp <- NULL
        }
        else
          htmp <- html_td(paste0(nms.cv[i],":"),class="header")
      }    
      if(length(htmp))
        header.i <- c(htmp,header.i)
    }
    header[[i]] <- header.i
  }
  
  if(varontop){

    if(split.dec)
      colspan <- mm*3
    else
      colspan <- mm
    
    if(n.col.vars > 1){
      htmp1 <- html_td("")
      htmp2 <- html_td(nms.cv[1],colspan=colspan,class="header")
      htmp <- c(htmp1,htmp2)
    }
    else if(nzchar(nms.cv[1])){
      htmp <- html_td(names(col.vars)[1],colspan=colspan,class="header")
    }
    else {
      htmp <- html_td("",colspan=colspan)
    }
    header <- c(list(htmp),header)
  }
  
  tmp.header <- header

  if(split.dec)
    colspan <- m*3
  else
    colspan <- m
  if(n.col.vars > 1 || !varontop)
    colspan <- colspan + 1
  
  l.header <- max.l.cv + varontop

  header <- rep(list(html_td("",colspan=colspan)),l.header)
  
  i <- seq(to=l.header,length=length(tmp.header))
  header[i] <- tmp.header
  header
}

htfm_mkLeader <- function(row.vars,n,max.l.rv,varinfront) {
  
  align.left <- c("text-align"="left")  
  n.row.vars <- length(row.vars)
  nms.row.vars <- names(row.vars)
  leader <- matrix("",nrow=n,ncol=n.row.vars)
  
  for(ii in 1:n.row.vars){
    
    rv <- row.vars[[ii]]
    rep.rv <- if(ii==1)1 else nn.rv
    rv <- rep(rv,rep.rv)
    nn.rv <- length(rv)
    rv.n <- n/nn.rv
    pos <- (1:nn.rv)*rv.n
    pos <- pos - rv.n + 1
    leader[pos,ii] <- rv
  }
  if(varinfront){
    lleader <- character(n)
    if(nzchar(nms.row.vars[1]))
      lleader[1] <- nms.row.vars[1]
    else if(n==1){
      lleader[1] <- leader[1,1]
      leader[1,1] <- ""
    }
      
    leader <- cbind(lleader,leader)
    if(n.row.vars > 1){
      hleader <- character(n.row.vars+1)
      ii <- 2:n.row.vars
      hleader[ii] <- nms.row.vars[ii]
      leader <- rbind(hleader,leader)
    }
  }
  
  tmp.leader <- leader
  leader <- matrix("",nrow=nrow(leader),ncol=max.l.rv+as.integer(varinfront))
  ii <- seq(to=ncol(leader),length=ncol(tmp.leader))
  leader[,ii] <- tmp.leader
  
  leader <- array(html_td(leader,vectorize=TRUE),dim=dim(leader))
  leader
}

htfm_mkBody <- function(x,format,digits,split.dec){
  
  n <- nrow(x)
  m <- ncol(x)
  
  d <- digits
  digits <- integer(m)
  digits[] <- d
  
  fo <- format
  format <- integer(m)
  format[] <- fo
  
  body <- array(list(),dim=c(n,m))
  
  for(i in 1:m){
    tmp <- formatC(x[,i],digits=digits[i],format=format[i])
    if(getOption("html.use.ampersand",FALSE))
      tmp <- gsub("-","&minus;",tmp,fixed=TRUE)
    else
      tmp <- gsub("-","\u2212",tmp,fixed=TRUE)
    if(!split.dec)
      body[,i] <- html_td(tmp,vectorize=TRUE)
    else {
      tmp <- spltDec(tmp)
      tmp <- html_td_spltDec(tmp)
      body[,i] <- tmp
    }
  }
  body
}

## value filters #######################################################################

setAs(from="atomic",to="valid.values",function(from,to)new(to,filter=from))
setAs(from="atomic",to="valid.range",function(from,to)new(to,filter=range(from)))
setAs(from="atomic",to="missing.values",function(from,to)new(to,filter=from))
setAs(from="list",to="missing.values",function(from,to)new(to,filter=from$values,range=from$range))

is.valid2 <- function(x,filter){
  is.na.x <- is.na(x)
  x <- x@.Data
  if(!length(filter)) !is.na.x
  else switch(class(filter),
      valid.values = !is.na.x & (x %in% filter@filter),
      valid.range = !is.na.x & (x >= filter@filter[1] & x <= filter@filter[2]),
      missing.values = {
          vld <- !is.na.x
          if(length(filter@filter)) vld <- vld & (x %nin% filter@filter)
          if(length(filter@range)==2) vld <- vld & (x < filter@range[1] | x > filter@range[2])
          vld
        }
      )
}

is.missing2 <- function(x,filter){
  is.na.x <- is.na(x)
  x <- x@.Data
  if(!length(filter)) is.na.x
  else switch(class(filter),
      valid.values = is.na.x | x %nin% filter@filter,
      valid.range = is.na.x | (x < filter@filter[1] | x > filter@filter[2]),
      missing.values = {
          msng <- is.na.x 
          if(length(filter@filter)) msng <- msng | x %in% filter@filter
          if(length(filter@range)==2) msng <- msng | (x >= filter@range[1] & x <= filter@range[2])
          msng
        }
      )
}

setMethod("value.filter",signature(x="NULL"),function(x)NULL)

setMethod("value.filter",signature(x="item"),function(x)x@value.filter)

setMethod("value.filter<-",signature(x="ANY",value="NULL"),function(x,value){
  x
})

setMethod("value.filter<-",signature(x="item",value="NULL"),
  function(x,value){
    x@value.filter <- NULL
    x
})

setMethod("value.filter<-",signature(x="item",value="value.filter"),
  function(x,value){
    x@value.filter <- value
    x
})

setMethod("valid.values",signature(x="NULL"),function(x)NULL)

setMethod("valid.values",signature(x="item.vector"),function(x){
  filter <- x@value.filter
  if(!length(filter)) return(new("valid.values",filter=sort(unique(x@.Data))))
  vals <- switch(class(filter),
    valid.values=return(filter),
    valid.range=sort(unique(x@.Data[x@.Data >= filter@filter[1] & x@.Data <= filter@filter[2]])),
    missing.values=sort(unique(x@.Data[!is.missing2(x,filter)]))
    )
  new("valid.values",filter=vals)
})

setMethod("missing.values",signature(x="NULL"),function(x)NULL)

setMethod("missing.values",signature(x="item.vector"),function(x){
  filter <- x@value.filter
  if(!length(filter)) return(NULL)
  vals <- switch(class(filter),
    valid.values=sort(unique(x@.Data[x@.Data %nin% filter@filter])),
    valid.range=sort(unique(x@.Data[x@.Data < filter@filter[1] | x@.Data > filter@filter[2]])),
    missing.values=return(filter)
    )
  new("missing.values",filter=vals)
})

setMethod("valid.range",signature(x="NULL"),function(x)NULL)

setMethod("valid.range",signature(x="item.vector"),function(x){
  filter <- x@value.filter
  if(!length(filter)) return(new("valid.range",filter=range(range(x@.Data))))
  vals <- switch(class(filter),
    valid.values=range(x@.Data[x@.Data %in% filter@filter]),
    valid.range=return(filter),
    missing.values={
        rgn <- range(x@.Data[!is.missing2(x,filter)])
        mv <- filter@filter
        if(any(mv >= rgn[1] & mv <= rgn[2])) stop("cannot make a valid range from this missing values definition")
        rgn
      }
    )
  new("valid.range",filter=range(vals))
})

setReplaceMethod("missing.values",signature(x="ANY",value="NULL"),
  function(x,value){
    x
})


setReplaceMethod("missing.values",signature(x="item",value="NULL"),
  function(x,value){
    x@value.filter <- NULL
    x
})

setReplaceMethod("missing.values",signature(x="ANY",value="atomic"),
  function(x,value){
    missing.values(x) <- new("missing.values",filter=value)
    x
  })
setReplaceMethod("missing.values",signature(x="ANY",value="list"),
  function(x,value){
    missing.values(x) <- new("missing.values",filter=value$values,range=value$range)
    x
  })
setReplaceMethod("missing.values",signature(x="item",value="missing.values"),
  function(x,value){
    x@value.filter <- value
    x
})
setReplaceMethod("missing.values",signature(x="atomic",value="missing.values"),
  function(x,value){
    if(is(x,"item")){
      message("'missing.values<-' called with item as atomic")
      x@value.filter <- value
      x
    }
    else as.item(x,value.filter=value)
})

setReplaceMethod("valid.values",signature(x="ANY",value="NULL"),
  function(x,value){
    x
})

setReplaceMethod("valid.values",signature(x="ANY",value="atomic"),
  function(x,value){
    valid.values(x) <- new("valid.values",filter=value)
    x
  })
setReplaceMethod("valid.values",signature(x="item",value="valid.values"),
  function(x,value){
    x@value.filter <- value
    x
})
setReplaceMethod("valid.values",signature(x="atomic",value="valid.values"),
  function(x,value){
    if(is(x,"item")){
      message("'valid.values<-' called with item as atomic")
      x@value.filter <- value
      x
    }
    else as.item(x,value.filter=value)
})

setReplaceMethod("valid.range",signature(x="ANY",value="NULL"),
  function(x,value){
    x
})

setReplaceMethod("valid.range",signature(x="ANY",value="atomic"),
  function(x,value){
    valid.range(x) <- new("valid.range",filter=value)
    x
  })
setReplaceMethod("valid.range",signature(x="item",value="valid.range"),
  function(x,value){
    x@value.filter <- value
    x
})
setReplaceMethod("valid.range",signature(x="atomic",value="valid.range"),
  function(x,value){
    if(is(x,"item")){
      message("'valid.range<-' called with item as atomic")
      x@value.filter <- value
      x
    }
    else as.item(x,value.filter=value)
})



setMethod("Arith",signature(e1="valid.values",e2="valid.values"),
  function(e1,e2){
    new("valid.values",filter=switch(.Generic,
      "+" = union(e1@filter,e2@filter),
      "-" = setdiff(e1@filter,e2@filter),
      stop("unsupported operator ",dQuote(.Generic))
    ))
})

setMethod("Arith",signature(e1="missing.values",e2="missing.values"),
  function(e1,e2){
    new("missing.values",filter=switch(.Generic,
      "+" = union(e1@filter,e2@filter),
      "-" = setdiff(e1@filter,e2@filter),
      stop("unsupported operator ",dQuote(.Generic))
    ))
})

setMethod("Arith",signature(e1="valid.range",e2="valid.range"),
  function(e1,e2){
    r1 <- e1@filter
    r2 <- e2@filter
    if(r1[2] < r2[1] || r1[1] > r2[2]) warning("ranges leave a gap")
    new("valid.values",filter=switch(.Generic,
      "+" = range(r1,r2),
      stop("unsupported operator ",dQuote(.Generic))
    ))
})

setMethod("Arith",signature(e1="value.filter",e2="vector"),
  function(e1,e2) callGeneric(e1,as(e2,class(e1))))

setMethod("is.missing",signature(x="NULL"),function(x)NULL)

setMethod("is.missing",signature(x="atomic"),function(x){
  is.na(x)
})

setMethod("is.missing",signature(x="factor"),function(x){
  is.na(x)
})


setMethod("is.missing",signature(x="item.vector"),function(x){
  filter <- x@value.filter
  is.missing2(x,filter)
})

is.valid <- function(x) !is.missing(x) & !is.na(x)
nvalid <- function(x) sum(is.valid(x))


format.valid.values <- function(x,digits=3,...){
  paste(trimws(format(x@filter,digits=digits,...)),collapse=", ")
}
format.valid.range <- function(x,digits=3,...){
  paste(trimws(format(x@filter[1:2],digits=digits,...)),collapse=" - ")
}
format.missing.values <- function(x,digits=3,...){
  if(length(x@filter) && length(x@range))
    paste(
      paste(trimws(format(x@filter,digits=digits,...)),collapse=", "),
      paste(trimws(format(x@range[1:2],digits=digits,...)),collapse=" - "),
      sep=", ")
  else if(length(x@filter))
    paste(trimws(format(x@filter,digits=digits,...)),collapse=", ")
  else if(length(x@range))
    paste(trimws(format(x@range[1:2],digits=digits,...)),collapse=" - ")
}

# setMethod("format","valid.values",format.valid.values) 
# setMethod("format","valid.range",format.valid.range) 
# setMethod("format","missing.values",format.missing.values) 


setMethod("show","value.filter",function(object){
  writeLines(format(object))
})

setMethod("include.missings","item",function(x,mark="*"){
  if(length(vl <- x@value.labels)){
    vlv <- vl@values
    labs <- vl@.Data
    ism <- is.missing2(vlv,x@value.filter)
    labs[ism] <- paste(mark,labs[ism],sep="")
    vl@.Data <- labs
    x@value.labels <- vl
  }
  x@value.filter <- NULL
  x
})

## value filters #######################################################################

setAs(from="atomic",to="valid.values",function(from,to)new(to,filter=from))
setAs(from="atomic",to="valid.range",function(from,to)new(to,filter=range(from)))
setAs(from="atomic",to="missing.values",function(from,to)new(to,filter=from))
setAs(from="list",to="missing.values",function(from,to)new(to,filter=from$values,range=from$range))

is.valid2 <- function(x,filter){
  is.na.x <- is.na(x)
  x <- x@.Data
  if(!length(filter)) !is.na.x
  else switch(class(filter),
      valid.values = !is.na.x & (x %in% filter@filter),
      valid.range = !is.na.x & (x >= filter@filter[1] & x <= filter@filter[2]),
      missing.values = {
          vld <- !is.na.x
          if(length(filter@filter)) vld <- vld & (x %nin% filter@filter)
          if(length(filter@range)==2) vld <- vld & (x < filter@range[1] | x > filter@range[2])
          vld
        }
      )
}

is.missing2 <- function(x,filter){
  is.na.x <- is.na(x)
  x <- x@.Data
  if(!length(filter)) is.na.x
  else switch(class(filter),
      valid.values = is.na.x | x %nin% filter@filter,
      valid.range = is.na.x | (x < filter@filter[1] | x > filter@filter[2]),
      missing.values = {
          msng <- is.na.x 
          if(length(filter@filter)) msng <- msng | x %in% filter@filter
          if(length(filter@range)==2) msng <- msng | (x >= filter@range[1] & x <= filter@range[2])
          msng
        }
      )
}

setMethod("value.filter",signature(x="NULL"),function(x)NULL)

setMethod("value.filter",signature(x="item"),function(x)x@value.filter)

setMethod("value.filter<-",signature(x="ANY",value="NULL"),function(x,value){
  x
})

setMethod("value.filter<-",signature(x="item",value="NULL"),
  function(x,value){
    x@value.filter <- NULL
    x
})

setMethod("value.filter<-",signature(x="item",value="value.filter"),
  function(x,value){
    x@value.filter <- value
    x
})

setMethod("valid.values",signature(x="NULL"),function(x)NULL)

setMethod("valid.values",signature(x="item.vector"),function(x){
  filter <- x@value.filter
  if(!length(filter)) return(new("valid.values",filter=sort(unique(x@.Data))))
  vals <- switch(class(filter),
    valid.values=return(filter),
    valid.range=sort(unique(x@.Data[x@.Data >= filter@filter[1] & x@.Data <= filter@filter[2]])),
    missing.values=sort(unique(x@.Data[!is.missing2(x,filter)]))
    )
  new("valid.values",filter=vals)
})

setMethod("missing.values",signature(x="NULL"),function(x)NULL)

setMethod("missing.values",signature(x="item.vector"),function(x){
  filter <- x@value.filter
  if(!length(filter)) return(NULL)
  vals <- switch(class(filter),
    valid.values=sort(unique(x@.Data[x@.Data %nin% filter@filter])),
    valid.range=sort(unique(x@.Data[x@.Data < filter@filter[1] | x@.Data > filter@filter[2]])),
    missing.values=return(filter)
    )
  new("missing.values",filter=vals)
})

setMethod("valid.range",signature(x="NULL"),function(x)NULL)

setMethod("valid.range",signature(x="item.vector"),function(x){
  filter <- x@value.filter
  if(!length(filter)) return(new("valid.range",filter=range(range(x@.Data))))
  vals <- switch(class(filter),
    valid.values=range(x@.Data[x@.Data %in% filter@filter]),
    valid.range=return(filter),
    missing.values={
        rgn <- range(x@.Data[!is.missing2(x,filter)])
        mv <- filter@filter
        if(any(mv >= rgn[1] & mv <= rgn[2])) stop("cannot make a valid range from this missing values definition")
        rgn
      }
    )
  new("valid.range",filter=range(vals))
})

setReplaceMethod("missing.values",signature(x="ANY",value="NULL"),
  function(x,value){
    x
})


setReplaceMethod("missing.values",signature(x="item",value="NULL"),
  function(x,value){
    x@value.filter <- NULL
    x
})

setReplaceMethod("missing.values",signature(x="ANY",value="atomic"),
  function(x,value){
    missing.values(x) <- new("missing.values",filter=value)
    x
  })
setReplaceMethod("missing.values",signature(x="ANY",value="list"),
  function(x,value){
    missing.values(x) <- new("missing.values",filter=value$values,range=value$range)
    x
  })
setReplaceMethod("missing.values",signature(x="item",value="missing.values"),
  function(x,value){
    x@value.filter <- value
    x
})
setReplaceMethod("missing.values",signature(x="atomic",value="missing.values"),
  function(x,value){
    if(is(x,"item")){
      message("'missing.values<-' called with item as atomic")
      x@value.filter <- value
      x
    }
    else as.item(x,value.filter=value)
})

setReplaceMethod("valid.values",signature(x="ANY",value="NULL"),
  function(x,value){
    x
})

setReplaceMethod("valid.values",signature(x="ANY",value="atomic"),
  function(x,value){
    valid.values(x) <- new("valid.values",filter=value)
    x
  })
setReplaceMethod("valid.values",signature(x="item",value="valid.values"),
  function(x,value){
    x@value.filter <- value
    x
})
setReplaceMethod("valid.values",signature(x="atomic",value="valid.values"),
  function(x,value){
    if(is(x,"item")){
      message("'valid.values<-' called with item as atomic")
      x@value.filter <- value
      x
    }
    else as.item(x,value.filter=value)
})

setReplaceMethod("valid.range",signature(x="ANY",value="NULL"),
  function(x,value){
    x
})

setReplaceMethod("valid.range",signature(x="ANY",value="atomic"),
  function(x,value){
    valid.range(x) <- new("valid.range",filter=value)
    x
  })
setReplaceMethod("valid.range",signature(x="item",value="valid.range"),
  function(x,value){
    x@value.filter <- value
    x
})
setReplaceMethod("valid.range",signature(x="atomic",value="valid.range"),
  function(x,value){
    if(is(x,"item")){
      message("'valid.range<-' called with item as atomic")
      x@value.filter <- value
      x
    }
    else as.item(x,value.filter=value)
})



setMethod("Arith",signature(e1="valid.values",e2="valid.values"),
  function(e1,e2){
    new("valid.values",filter=switch(.Generic,
      "+" = union(e1@filter,e2@filter),
      "-" = setdiff(e1@filter,e2@filter),
      stop("unsupported operator ",dQuote(.Generic))
    ))
})

setMethod("Arith",signature(e1="missing.values",e2="missing.values"),
  function(e1,e2){
    new("missing.values",filter=switch(.Generic,
      "+" = union(e1@filter,e2@filter),
      "-" = setdiff(e1@filter,e2@filter),
      stop("unsupported operator ",dQuote(.Generic))
    ))
})

setMethod("Arith",signature(e1="valid.range",e2="valid.range"),
  function(e1,e2){
    r1 <- e1@filter
    r2 <- e2@filter
    if(r1[2] < r2[1] || r1[1] > r2[2]) warning("ranges leave a gap")
    new("valid.values",filter=switch(.Generic,
      "+" = range(r1,r2),
      stop("unsupported operator ",dQuote(.Generic))
    ))
})

setMethod("Arith",signature(e1="value.filter",e2="vector"),
  function(e1,e2) callGeneric(e1,as(e2,class(e1))))

setMethod("is.missing",signature(x="NULL"),function(x)NULL)

setMethod("is.missing",signature(x="atomic"),function(x){
  is.na(x)
})

setMethod("is.missing",signature(x="factor"),function(x){
  is.na(x)
})


setMethod("is.missing",signature(x="item.vector"),function(x){
  filter <- x@value.filter
  is.missing2(x,filter)
})

is.valid <- function(x) !is.missing(x) & !is.na(x)
nvalid <- function(x) sum(is.valid(x))


format.valid.values <- function(x,digits=3,...){
  paste(trimws(format(x@filter,digits=digits,...)),collapse=", ")
}
format.valid.range <- function(x,digits=3,...){
  paste(trimws(format(x@filter[1:2],digits=digits,...)),collapse=" - ")
}
format.missing.values <- function(x,digits=3,...){
  if(length(x@filter) && length(x@range))
    paste(
      paste(trimws(format(x@filter,digits=digits,...)),collapse=", "),
      paste(trimws(format(x@range[1:2],digits=digits,...)),collapse=" - "),
      sep=", ")
  else if(length(x@filter))
    paste(trimws(format(x@filter,digits=digits,...)),collapse=", ")
  else if(length(x@range))
    paste(trimws(format(x@range[1:2],digits=digits,...)),collapse=" - ")
}

# setMethod("format","valid.values",format.valid.values) 
# setMethod("format","valid.range",format.valid.range) 
# setMethod("format","missing.values",format.missing.values) 


setMethod("show","value.filter",function(object){
  writeLines(format(object))
})

setMethod("include.missings","ANY",function(x,mark="*") x)

setMethod("include.missings","item",function(x,mark="*"){
  if(length(vl <- x@value.labels)){
    vlv <- vl@values
    labs <- vl@.Data
    ism <- is.missing2(vlv,x@value.filter)
    labs[ism] <- paste(mark,labs[ism],sep="")
    vl@.Data <- labs
    x@value.labels <- vl
  }
  x@value.filter <- NULL
  x
})

Stata.file <- function(file,
                       iconv=TRUE,
                       encoded=if(new_format)getOption("Stata.new.encoding","utf-8")
                               else getOption("Stata.old.encoding","cp1252"),
                       negative2missing = FALSE
                       ){
    file <- path.expand(file)
    check.file(file,error=TRUE)

    ptr <- dta117_file_open(file)

    new_format <- dta117_check_magic(ptr)
    
    if(new_format){

        dta117_read_header(ptr)
        dta117_read_map(ptr)
        types <- dta117_read_vtypes(ptr)
        vnames <- dta117_read_vnames(ptr)
        dta117_read_sortlist(ptr)
        dta117_read_formats(ptr)
        vallab_names <- dta117_read_vlab_names(ptr)
        varlabs <- dta117_read_varlabs(ptr)
        vallabs <- dta117_read_vallabs(ptr)
        variables <- dta117_make_prototype(types)

        names(types) <- vnames
        names(variables) <- vnames
        names(varlabs) <- vnames
        names(vallab_names) <- vnames
        
        varlabs <- varlabs[nzchar(varlabs)]
        vallabs <- vallabs[nzchar(vallabs)]
        vallab_names <- vallab_names[nzchar(vallab_names)]

        vallabs <- vallabs[vallab_names]
        names(vallabs) <- names(vallab_names)
        
        if(length(varlabs))
            variables[names(varlabs)] <- mapply("description<-",variables[names(varlabs)],varlabs)
        if(length(vallabs))
            suppressWarnings(variables[names(vallabs)] <- mapply(add_value_labels,
                                                                 variables[names(vallabs)],
                                                                 vallabs))

        missings <- dta117_missing.values(types)
        missings <- missings[sapply(missings,length) > 0]
        if(length(missings))
            variables[names(missings)] <- mapply("missing.values<-",variables[names(missings)],missings)
        if(negative2missing)
            variables <- lapply(variables,`valid.range<-`,c(0,Inf))

        missval_labels <- dta117_missval_labels(types)
        missval_labels <- missval_labels[sapply(missval_labels,length) > 0]

        if(length(missval_labels)){
            missval_labels <- lapply(missval_labels,as,"value.labels")
            meas <- lapply(variables[names(missval_labels)],"measurement")
            suppressWarnings(variables[names(missval_labels)] <- mapply(add_value_labels,
                                                                 variables[names(missval_labels)],
                                                                 missval_labels))
            variables[names(missval_labels)] <- mapply("measurement<-",
                                                       variables[names(missval_labels)],
                                                       meas)
        }

        if(iconv)
            variables <- lapply(variables,Iconv,from=encoded,to="UTF-8")
        else
            encoded = ""
        
        
        new("Stata_new.importer",
            variables,
            ptr=ptr,
            types=types,
            encoded=encoded
            )
        
    }
    else {

        dta117_file_close(ptr)
        ptr <- new.dta(file)
        data.spec <- get.dictionary.dta(ptr)
        types <- data.spec$types
        variables <- .Call("dta_make_prototype",types)
        names(variables) <- data.spec$names
        varlabs <- data.spec$varlabs
        varlabs <- varlabs[nzchar(varlabs)]
        vallabs <- data.spec$value.labels
        missings <- data.spec$missing.values
        missings <- missings[sapply(missings,length) > 0]
        missval_labels <- data.spec$missval_labels
        missval_labels <- missval_labels[sapply(missval_labels,length) > 0]
        
        if(length(varlabs))
            variables[names(varlabs)] <- mapply("description<-",variables[names(varlabs)],varlabs)
        if(length(vallabs))
            suppressWarnings(variables[names(vallabs)] <- mapply("labels<-",variables[names(vallabs)],vallabs))
        if(length(missings))
            variables[names(missings)] <- mapply("missing.values<-",variables[names(missings)],missings)
        if(negative2missing)
            variables <- lapply(variables,`valid.range<-`,c(0,Inf))

        if(length(missval_labels)){
            missval_labels <- lapply(missval_labels,as,"value.labels")
            meas <- lapply(variables[names(missval_labels)],"measurement")
            suppressWarnings(variables[names(missval_labels)] <- mapply(add_value_labels,
                                                                 variables[names(missval_labels)],
                                                                 missval_labels))
            variables[names(missval_labels)] <- mapply("measurement<-",
                                                       variables[names(missval_labels)],
                                                       meas)
        }

        if(iconv)
            variables <- lapply(variables,Iconv,from=encoded,to="UTF-8")
        else
            encoded <- ""

        new("Stata.importer",
            variables,
            ptr=ptr,
            data.spec=data.spec,
            encoded=encoded
            )
    }
}

## New Stata formats from 117
setMethod("initialize","Stata_new.importer",function(.Object,
                                                     variables,
                                                     ptr,
                                                     types,
                                                     encoded
                                                     ){
     .Object@.Data <- variables
     .Object@ptr <- ptr
     .Object@types <- types
     .Object@encoded <- encoded
     .Object
})

setMethod("getNobs","Stata_new.importer",
          function(x) dta117_dim(x@ptr)[1])

setMethod("seekData","Stata_new.importer",function(x)
  .Call("dta117_seek_data",x@ptr)
)

setMethod("readData","Stata_new.importer",
  function(x,n)
    iconv_list(.Call("dta117_read_data",
      x@ptr,
      what=x,
      n=n,
      types=x@types),
      encoded=x@encoded))

setMethod("readSlice","Stata_new.importer",
  function(x,rows,cols)
    iconv_list(.Call("dta117_read_slice",x@ptr,
      what=x,
      j=cols,i=rows,
      types=x@types),
      encoded=x@encoded))

setMethod("readChunk","Stata_new.importer",
  function(x,nrows,cols)
    iconv_list(.Call("dta117_read_chunk",x@ptr,
      what=x,
      cols=cols,n=nrows,
      types=x@types),
      encoded=x@encoded))

setMethod("show","Stata_new.importer",
  function(object){
    file.name <- attr(object@ptr,"file.name")
    nobs <- nrow(object)
    nvar <- ncol(object)
    cat("\nStata file",sQuote(file.name),"\n\twith ")
    cat(nvar,"variables and ")
    cat(nobs,"observations\n")
})


## Old Stata formats before 117

setMethod("initialize","Stata.importer",function(.Object,
                                                 variables,
                                                 ptr,
                                                 data.spec,
                                                 encoded
                                                 ){
     .Object@.Data <- variables
     .Object@ptr <- ptr
     .Object@data.spec <- data.spec
     .Object@encoded <- encoded
     .Object
})

setMethod("getNobs","Stata.importer",function(x) x@data.spec$nobs)

setMethod("seekData","Stata.importer",function(x)
  .Call("dta_seek_data",x@ptr)
)

setMethod("readData","Stata.importer",
  function(x,n)
    iconv_list(.Call("dta_read_data",
      x@ptr,
      what=x,
      ncases=n,
      types=x@data.spec$types),
      encoded=x@encoded))

setMethod("readSlice","Stata.importer",
  function(x,rows,cols)
    iconv_list(.Call("dta_read_slice",x@ptr,
      what=x,
      j=cols,i=rows,
      types=x@data.spec$types),
      encoded=x@encoded))

setMethod("readChunk","Stata.importer",
  function(x,nrows,cols)
    iconv_list(.Call("dta_read_chunk",x@ptr,
      what=x,
      cols=cols,n=nrows,
      types=x@data.spec$types),
      encoded=x@encoded))

setMethod("show","Stata.importer",
  function(object){
    file.name <- attr(object@ptr,"file.name")
    nobs <- nrow(object)
    nvar <- ncol(object)
    cat("\nStata file",sQuote(file.name),"\n\twith ")
    cat(nvar,"variables and ")
    cat(nobs,"observations\n")
})

subset.Stata.importer <- subset.importer
subset.Stata_new.importer <- subset.importer

percent <- function(x,...) UseMethod("percent")

percent.default <- function(x,weights=NULL,total=!(se || ci),
                      se=FALSE,ci=FALSE,ci.level=.95,
                      total.name="N",perc.label="Percentage",...){
  subst <- substitute(x)
  x.label <- paste(deparse(subst))
  tab <- Table(x,weights=weights,total=FALSE,...)
  tabsum <- sum(tab)
  perc <- drop(100*tab/tabsum)
  #names(perc) <- rownames(tab)

  if(total.name %in% names(perc)) total.name <- paste("_",total.name,"_",sep="")

  if(!se && !ci){
    if(total)
      perc <- c(perc,structure(tabsum,names=total.name))
    perc <- as.matrix(perc)
    colnames(perc) <- x.label
    drop(perc)
  }
  else {
      perc <- t(as.matrix(perc))
      rownames(perc) <- perc.label
      prop <- tab/tabsum
      var.prop <- prop*(1-prop)/tabsum
      se.perc <- 100*sqrt(var.prop)
    if(se){
      perc <- rbind(perc,se=se.perc)
    }
    if(ci){
      alpha <- (1-ci.level)/2
      lower <- upper <- numeric(length(tab))
      isnull <- tab == 0
      isfull <- tab == tabsum
      lower[!isnull] <- qbeta(alpha,tab[!isnull],tabsum-tab[!isnull]+1)
      lower[isnull] <- 0
      upper[!isfull] <- qbeta(1-alpha,tab[!isfull]+1,tabsum-tab[!isfull])
      upper[isfull] <- 1
      #lower <- qnorm(alpha,mean=prop,sd=sqrt(var.prop))
      #upper <- qnorm(1-alpha,mean=prop,sd=sqrt(var.prop))
      perc <- rbind(perc,lower=100*lower,upper=100*upper)
    }
    if(total){
      total <- numeric(nrow(perc))
      total[] <- tabsum
      total <- t(as.matrix(total))
      rownames(total) <- total.name
      perc <- cbind(perc,t(total))
    }
    names(dimnames(perc))[2] <- x.label
    names(dimnames(perc))[is.na(names(dimnames(perc)))] <- ""
    drop(perc)
  }
}

percent.logical <- function(x,weights=NULL,total=!(se || ci),
                      se=FALSE,ci=FALSE,ci.level=.95,
                      total.name="N",perc.label="Percentage",...){
  subst <- substitute(x)
  x.label <- paste(deparse(subst))
  tab <- drop(Table(x,weights=weights,total=FALSE,...))
  N <- sum(tab)
  n <- tab["TRUE"]
  p <- n/N
  perc <- structure(c(100*p),names=perc.label)
  if(se){
    var <- p*(1-p)/N
    perc <- c(perc,se=100*sqrt(var))
  }
  if(ci){
    if(is.finite(p)){
      alpha <- (1-ci.level)/2
      lower <- if(p) qbeta(alpha,n,N-n+1) else 0
      upper <- if(p<1) qbeta(1-alpha,n+1,N-n) else 1
      perc <- c(perc,lower=100*lower,upper=100*upper)
    }
    else
      perc <- c(perc,lower=NA,upper=NA)
  }
  if(total){
    perc <- c(perc,structure(N,names=total.name))
  }
  perc <- as.matrix(perc)
  colnames(perc) <- x.label
  drop(perc)
}

setSummaryTemplate(lmerMod = c("Log-likelihood" = "($logLik:f#)",
                     "Deviance" = "($deviance:f#)",
                     "AIC" = "($AIC:f#)",
                     "BIC" = "($BIC:f#)",
                     N     = "($N:d)"))

setSummaryTemplate(glmerMod = c("Log-likelihood" = "($logLik:f#)",
                               "Deviance" = "($deviance:f#)",
                               "AIC" = "($AIC:f#)",
                               "BIC" = "($BIC:f#)",
                               N     = "($N:d)"))

getSummary.merMod <- function (obj, alpha = 0.05, ...) {

  smry <- summary(obj)
  coef <- smry$coefficients
  
  lower <- qnorm(p = alpha/2, mean = coef[, 1], sd = coef[,2])
  upper <- qnorm(p = 1 - alpha/2, mean = coef[, 1], sd = coef[,2])
  if (ncol(coef) == 3) {
    p <- (1 - pnorm(abs(coef[, 3]))) * 2
    coef <- cbind(coef, p, lower, upper)
  }
  else {
    coef <- cbind(coef, lower, upper)
  }

  dn.cf <- list(
      rownames(coef),
      c("est","se","stat","p","lwr","upr"),
      names(obj@frame)[1]
  )
  dim(coef) <- c(dim(coef)[1],dim(coef)[2],1)
  dimnames(coef) <- dn.cf
    
  varcor <- smry$varcor

  VarPar <- NULL

  if(smry$sigma!=1){
    vp.i <- matrix(NA,nrow=1,ncol=6)
    vp.i[1] <- smry$sigma^2
    dim(vp.i) <- c(dim(vp.i),1)
    dimnames(vp.i) <- list("Var(residual)",
                           c("est","se","stat","p","lwr","upr"),
                           names(obj@frame)[1])
    VarPar <- list(residual=vp.i)
  }

  for(i in seq_along(varcor)){
    vc.i <- varcor[[i]]
    lv.i <- names(varcor)[i]
    vr.i <- diag(vc.i)
    cv.i <- vc.i[lower.tri(vc.i)]
    nms.i <- rownames(vc.i)
    nms.i <- gsub("(Intercept)","1",nms.i,fixed=TRUE)
    vrnames.i <- paste0("Var(~",nms.i,"|",lv.i,")")
    cvnames.i <- t(outer(nms.i,nms.i,FUN=paste,sep=":"))
    cvnames.i <- cvnames.i[lower.tri(cvnames.i)]
    if(length(cvnames.i))
      cvnames.i <- paste0("Cov(~",cvnames.i,"|",lv.i,")")
    vp.i <- matrix(NA,nrow=length(vr.i)+length(cv.i),ncol=6)
    vp.i[,1] <- c(vr.i,cv.i)
    dim(vp.i) <- c(dim(vp.i),1)
    dimnames(vp.i) <- list(c(vrnames.i,cvnames.i),
                           c("est","se","stat","p","lwr","upr"),
                           names(obj@frame)[1])
    VarPar <- c(VarPar,structure(list(vp.i),names=lv.i))
  }
    
  ## Factor levels.
  xlevels <- list()
  Contr <- names(attr(model.matrix(obj), "contrasts"))
  for (c in Contr) xlevels[[c]] <- levels(obj@frame[,c])

  ## Model fit statistics.
  ll <- logLik(obj)[1]
  isREML <- inherits(obj@resp,"lmerResp") && obj@resp$REML > 0
  if(!isREML)
    deviance <- deviance(obj)
  else 
    deviance <- lme4::REMLcrit(obj)
  AIC <- AIC(obj)
  BIC <- BIC(obj)
  
  G <-as.integer(smry$ngrps)
  names(G) <- names(smry$ngrps)
    
  sumstat <- c(logLik = ll,
               deviance = deviance,
               AIC = AIC,
               BIC = BIC,
               N=nobs(obj))
  ## Return model summary.
  
  ans <- list(coef= coef)

  ans <- c(ans,VarPar)
    
  ans <- c(ans,       
           list(Groups = G,
                sumstat = sumstat,
                contrasts = Contr, ## Reuse 'Contr'
                xlevels = xlevels,
                call = obj@call))
  return(ans)
}

.last <- function(x) x[[length(x)]]
.getElts <- function(x,i) lapply(x,"[[",i)

toLatex.ftable_matrix <- function(object,
                                  show.titles=TRUE,
                                  digits=getOption("digits"),
                                  format="f",
                                  useDcolumn=getOption("useDcolumn",TRUE),
                                  colspec=if(useDcolumn)
                                              paste("D{.}{",LaTeXdec,"}{",ddigits,"}",sep="")
                                          else "r",
                                  LaTeXdec=".",
                                  ddigits=digits,
                                  useBooktabs=getOption("useBooktabs",TRUE),
                                  toprule=if(useBooktabs) "\\toprule" else "\\hline\\hline",
                                  midrule=if(useBooktabs) "\\midrule" else "\\hline",
                                  cmidrule=if(useBooktabs) "\\cmidrule" else "\\cline",
                                  bottomrule=if(useBooktabs) "\\bottomrule" else "\\hline\\hline",
                                  compact=FALSE,
                                  varontop,varinfront,
                                  groupsep="3pt",
                                  grouprule=midrule,
                                  toLatex.escape.tex=getOption("toLatex.escape.tex",FALSE),
                                  multi_digits=NULL,
                                  ...)
{
  
  row.vars <- attr(object,"row.vars")
  col.vars <- attr(object,"col.vars")
  
  N <- nrow(object)
  M <- ncol(object)
  
  n <- sapply(object[,1],nrow)
  m <- sapply(object[1,],ncol)
  n.row.vars <- sapply(row.vars,length)
  n.col.vars <- sapply(col.vars,length)
  max.n.row.vars <- max(n.row.vars)
  max.n.col.vars <- max(n.col.vars)
  
  if(missing(varontop)) varontop <- max(n.col.vars) <= 1
  if(missing(varinfront)) varinfront <- max(n.row.vars) <= 1
  
  if(!missing(multi_digits)){
    if(is.list(multi_digits)){
      digits <- multi_digits
    }
    else {
      digits <- rep(list(multi_digits),M)
    }
  }
  else {
    d <- digits
    digits <- integer(M)
    digits[] <- d
  }
    
  fo <- format
  format <- integer(M)
  format[] <- fo

  if(missing(ddigits) && missing(colspec) && useDcolumn && is.list(digits)){
    colspec <- lapply(digits,function(ddigits)paste0("D{.}{",LaTeXdec,"}{",ddigits,"}"))
  }
  else {
    csp <- colspec
    colspec <- character(length(m))
    colspec[] <- csp
  }
  
  
  cv.desc <- sapply(col.vars,cv_get_desc,compact=compact)
  start.g <- cv.desc["start.g",]
  end.g <- cv.desc["end.g",]
  nms.cv <- cv.desc["nms.cv",]
  mcols <- cv.desc["mcols",]
  width <- cv.desc["width",]
  ii <- cv.desc["ii",]
  cv.repg <- cv.desc["cv.repg",]
  
  headers <- mapply(ltfm_mkHeader,col.vars,n.col.vars,mcols,width,cv.repg,ii,varontop,compact,SIMPLIFY=FALSE)
  
  l.hd <- sapply(headers,length)
  max.l.hd <- max(l.hd)
  
  for(i in 1:M){
    
    jj <- seq.int(to=max.l.hd,length.out=l.hd[i])
    tmp.hdr1 <- paste0(rep("",width[[i]]),collapse="&")
    if(n.col.vars[i]>1||!varontop & any(nzchar(nms.cv[[i]])) )
      tmp.hdr1 <- paste0("&",tmp.hdr1)
    
    tmp.header <- vector("list",max.l.hd)
    tmp.header[] <- list(tmp.hdr1)
    tmp.header[jj] <- headers[[i]]
    headers[[i]] <- tmp.header
  }
  
  headers <- lapply(1:max.l.hd,.getElts,x=headers)
  headers <- lapply(headers,unlist)
  
  for(i in 1:M){
    
    if(i==1){
      start.g[[i]] <- mapply(`+`,start.g[[i]],max.n.row.vars,SIMPLIFY=FALSE)
      end.g[[i]] <- mapply(`+`,end.g[[i]],max.n.row.vars,SIMPLIFY=FALSE)
    }
    else if(compact){
      start.g[[i]] <- mapply(`+`,start.g[[i]],tmp.end.g,SIMPLIFY=FALSE)
      end.g[[i]] <- mapply(`+`,end.g[[i]],tmp.end.g,SIMPLIFY=FALSE)
    }
    else {
      start.g[[i]] <- mapply(`+`,start.g[[i]],tmp.end.g+1,SIMPLIFY=FALSE)
      end.g[[i]] <- mapply(`+`,end.g[[i]],tmp.end.g+1,SIMPLIFY=FALSE)
    }
    
    if(n.col.vars[[i]] > 1 | (!varontop & nzchar(nms.cv[[i]]))[1]){
      start.g[[i]] <- mapply(`+`,start.g[[i]],1,SIMPLIFY=FALSE)
      end.g[[i]] <- mapply(`+`,end.g[[i]],1,SIMPLIFY=FALSE)
    }
    tmp.end.g <- .last(end.g[[i]])
    tmp.end.g <- .last(tmp.end.g)
  }
  
  for(i in 1:M){
    
    jj <- seq.int(to=max.n.col.vars,length.out=n.col.vars[i])
    
    tmp.start.g <- vector("list",max.n.col.vars)
    if(varontop || n.col.vars[i]>1)
      tmp.start.g[jj] <- start.g[[i]]
    start.g[[i]] <- tmp.start.g
    tmp.end.g <- vector("list",max.n.col.vars)
    if(varontop || n.col.vars[i]>1)
      tmp.end.g[jj] <- end.g[[i]]
    end.g[[i]] <- tmp.end.g
  }
  
  start.g <- lapply(1:max.n.col.vars,.getElts,x=start.g)
  end.g <- lapply(1:max.n.col.vars,.getElts,x=end.g)
  start.g <- lapply(start.g,unlist)
  end.g <- lapply(end.g,unlist)
  
  if(varinfront) {
    
    start.g <- mapply(`+`,start.g,1,SIMPLIFY=FALSE)
    end.g <- mapply(`+`,end.g,1,SIMPLIFY=FALSE)
  }
  
  header <- unlist(lapply(headers,paste0,collapse=if(compact)"&" else "&&"))
  len.hdr <- length(header)
  lheader <- matrix("",ncol=max.n.row.vars,nrow=len.hdr)
  lheader[len.hdr,seq.int(to=max.n.row.vars,length.out=n.row.vars[1])] <- names(row.vars[[1]])
  if(varinfront) lheader[,1] <- ""
  lheader <- apply(lheader,1,paste0,collapse="&")
  header <- paste(lheader,header,sep="&")
  if(varinfront)
    header <- paste0("&",header)
  
  cln <- mapply(mkclines,cmidrule,start.g,end.g,SIMPLIFY=FALSE)
  cln <- sapply(cln,paste0,collapse="")
  
  header <- paste0(header,"\\\\")
  if(varontop)
    header[-1] <- paste0(cln,"\n",header[-1])
  else
    header[-1] <- paste0(cln[-1],"\n",header[-1])
  
  leaders <- mapply(ltfm_mkLeader,row.vars,n.row.vars,n,SIMPLIFY=FALSE)
  
  if(varinfront){
    
    for(i in 1:N){
      
      nm.row.vars.i <- names(row.vars[[i]])
      tmp.leaders <- matrix("",nrow=n[i],ncol=max.n.row.vars+1)
      jj <- seq.int(to=max.n.row.vars+1,length.out=n.row.vars[i])
      tmp.leaders[,jj] <- leaders[[i]]
      if(nzchar(nm.row.vars.i[1]))
        tmp.leaders[1,1] <- nm.row.vars.i[1]
      else if(varinfront && length(n[i] < 2)){
        tmp.leaders[1,1] <- tmp.leaders[1,max.n.row.vars+1]
        tmp.leaders[1,max.n.row.vars+1] <- ""
      }
      leaders[[i]] <- tmp.leaders
      if(i > 1 && n.row.vars[i] > 1 && any(nzchar(nm.row.vars.i[-n.row.vars[i]]))){
        
        tmp.leaders <- character(max.n.row.vars)
        tmp.leaders[jj] <- names(row.vars[[i]])
        leaders[[i]] <- rbind(tmp.leaders,leaders[[i]])
      }
    }
  }
  else {
    
    for(i in 1:N){
      
      tmp.leaders <- matrix("",nrow=n[i],ncol=max.n.row.vars)
      jj <- seq.int(to=max.n.row.vars,length.out=n.row.vars[i])
      tmp.leaders[,jj] <- leaders[[i]]
      leaders[[i]] <- tmp.leaders
      if(i > 1 && any(nzchar(names(row.vars[[i]])))
      ){
        tmp.leaders <- character(max.n.row.vars)
        tmp.leaders[jj] <- names(row.vars[[i]])
        leaders[[i]] <- rbind(tmp.leaders,leaders[[i]])
      }
    }
  }
  
  body <- array(list(),dim=dim(object))
  
  for(j in 1:M){
    for(i in 1:N){
      
      tmp.bdy <- ltfm_mkBody(object[[i,j]],ii[[j]],format[[j]],digits[[j]])
      if(n.col.vars[[j]] > 1 |  (!varontop & nzchar(nms.cv[[j]]))[1])
        tmp.bdy <- cbind("",tmp.bdy)
      if(j > 1 && !compact)
        tmp.bdy <- cbind("",tmp.bdy)
      if(varinfront){
        
        nm.row.vars.i <- names(row.vars[[i]])
        if(i > 1 && n.row.vars[i] > 1 && any(nzchar(nm.row.vars.i[-n.row.vars[i]])))
          tmp.bdy <- rbind("",tmp.bdy)
      }
      else{
        
        if(i > 1 && any(nzchar(names(row.vars[[i]]))))
          tmp.bdy <- rbind("",tmp.bdy)
      }
      body[[i,j]] <- tmp.bdy
    }
  }
  
  ans <- list()
  for(i in 1:N){
    
    tmp.bdy <- cbind(leaders[[i]],do.call(cbind,body[i,,drop=FALSE]))
    tmp.bdy <- apply(tmp.bdy,1,paste0,collapse=" & ")
    tmp.rowsep <- rep("\\\\",length(tmp.bdy))
    if(i<N && (!varinfront || max.n.row.vars > 1) && !compact)
      tmp.rowsep[length(tmp.bdy)] <- paste0(tmp.rowsep[length(tmp.bdy)],"[",groupsep,"]")
    if(i>1 && (!varinfront || max.n.row.vars > 1) && any(nzchar(names(row.vars[[i]])))){
      
      tmp.rowsep[1] <- paste0(tmp.rowsep[1],"\n",grouprule)
    }
    tmp.bdy <- paste0(tmp.bdy,tmp.rowsep)
    if(i > 1 && varinfront && max.n.row.vars == 1)
      tmp.bdy[1] <- paste(grouprule,tmp.bdy[1])
    ans[[i]] <- tmp.bdy
  }
  ans <- unlist(ans)
  
  ans <- c(toprule,header,midrule,ans,bottomrule)
  if(toLatex.escape.tex)
    ans <- LaTeXcape(ans)
  
  tabspec <- mapply(ltfm_mkTabspec,colspec,ii,m,USE.NAMES=FALSE)
  
  for(i in 1:M){
    
    if(n.col.vars[i] > 1 | (!varontop & nzchar(nms.cv[[i]]))[1])
      tabspec[i] <- paste0("l",tabspec[i])
    if(i>1 & !compact)
      tabspec[i] <- paste0("c",tabspec[i])
  }
  tabspec <- c(rep("l",max.n.row.vars),tabspec)
  if(varinfront) tabspec <- c("l",tabspec)
  tabspec <- paste0(tabspec,collapse="")
  
  tabbegin <- paste0("\\begin{tabular}{",tabspec,"}")
  tabend <- "\\end{tabular}"
  ans <- c(tabbegin,ans,tabend)
  
  structure(ans,class="Latex")
}

cv_get_desc <- function(col.vars,compact){
  
  m <- prod(sapply(col.vars,length))
  n.col.vars <- length(col.vars)
  nms.cv <- names(col.vars)
  l.cv <- sapply(col.vars,length)
  cv.sizeg <- rev(cumprod(rev(l.cv)))
  cv.repg <- m/cv.sizeg
  
  if(compact){
    cv.stepg <- m/cv.repg
    width <- m
  }
  else {
    
    width <- cv.repg[n.col.vars]*(cv.sizeg[n.col.vars]+1)
    cv.stepg <- width/cv.repg
    width <- width - 1
  }
  
  mcols <- c(if(compact) cv.sizeg else cv.stepg-1,1)

  if(compact){
    start.g <- lapply(1:n.col.vars,function(i){
      seq(from=1,length=cv.repg[i],by=cv.stepg[i])
    })
    end.g <- mapply(`+`,start.g,cv.stepg-1)
  }
  else{
    start.g <- lapply(1:n.col.vars,function(i){
      seq(from=1,length=cv.repg[i],by=cv.stepg[i])
    })
    end.g <- mapply(`+`,start.g,cv.stepg-2)
  }
  start.g.n <- start.g[[n.col.vars]]
  end.g.n <- end.g[[n.col.vars]]
  ii <- unlist(mapply(seq,from=start.g.n,to=end.g.n,SIMPLIFY=FALSE))
  
  list(
    nms.cv=nms.cv,
    l.cv=l.cv,
    cv.sizeg=cv.sizeg,
    cv.repg=cv.repg,
    cv.stepg=cv.stepg,
    width=width,
    mcols=mcols,
    start.g=start.g,
    end.g=end.g,
    start.g.n=start.g.n,
    end.g.n=end.g.n,
    ii=ii
  )
}

ltfm_mkHeader <- function(col.vars,n.col.vars,mcols,width,cv.repg,ii,varontop,compact){
  
  nms.cv <- names(col.vars)
  
  header <- character()
  for(i in 1:n.col.vars){
    
    if(i==1 && varontop){
      if(nzchar(nms.cv[i]))
        tmp.header <- paste0("\\multicolumn{",mcols[i],"}{c}{",nms.cv[i],"}")
      else
        tmp.header <- ""
      if(n.col.vars>1)
        tmp.header <- paste0("&",tmp.header)
      header <- append(header,list(tmp.header))
    }
    
    if(i==n.col.vars){
      tmp.header <- character(width)
      tmp.header[ii] <- paste0("\\multicolumn{",mcols[i+1],"}{c}{",rep(col.vars[[i]],cv.repg[i]),"}")
      tmp.header <- paste0(tmp.header,collapse="&")
    }
    else {
      tmp.header <- paste0("\\multicolumn{",mcols[i+1],"}{c}{",rep(col.vars[[i]],cv.repg[i]),"}")
      tmp.header <- paste0(tmp.header,collapse=if(compact)"&"else"&&")
    }
    if(n.col.vars==1){
      
      if(nzchar(nms.cv[i]) && !varontop)
        tmp.header <- paste0(nms.cv[i],":&",tmp.header)
      
    }
    else {
      
      if((i > 1 || !varontop) && nzchar(nms.cv[i]))
        tmp.header <- paste0(nms.cv[i],":&",tmp.header)
      else
        tmp.header <- paste0("&",tmp.header)
    }
    
    header <- append(header,list(tmp.header))
  }
  header
}

mkclines <- function(cmidrule,start.g,end.g){
  
  len <- end.g - start.g + 1
  start.g <- start.g[len>1]
  end.g <- end.g[len>1]
  paste0(cmidrule,"{",start.g,"-",end.g,"}")
}

ltfm_mkLeader <- function(row.vars,n.row.vars,n) {
  
  leader <- matrix("",nrow=n,ncol=n.row.vars)
  
  for(i in 1:n.row.vars){
    
    rv <- row.vars[[i]]
    rep.rv <- if(i==1)1 else nn.rv
    rv <- rep(rv,rep.rv)
    nn.rv <- length(rv)
    rv.n <- n/nn.rv
    pos <- (1:nn.rv)*rv.n
    pos <- pos - rv.n + 1
    leader[pos,i] <- rv
  }
  leader
}

ltfm_mkBody <- function(object,ii,format,digits){
  
  n <- nrow(object)
  m <- ncol(object)
  
  d <- digits
  digits <- integer(m)
  digits[] <- d
  
  fo <- format
  format <- integer(m)
  format[] <- fo
  
  total.width <- max(ii)
  
  body <- array("",dim=c(nrow(object),total.width))
  
  for(i in seq(along=digits))
    body[,ii[i]] <- formatC(object[,i],digits=digits[i],format=format[i])
  
  sub("([eE])([-+]?[0-9]+)","\\\\textrm{\\1}\\2",body)
}

ltfm_mkTabspec <- function(colspec,ii,m){
  
  csp <- character(m)
  csp[] <- colspec
  colspec <- csp
  
  total.width <- length(ii)
  tabspec <-rep("c",total.width)
  tabspec[ii] <- colspec
  paste(tabspec,collapse="")
}

"%if%" <- function(expr,condition){
    m <- match.call()
    expr <- m$expr
    expr.ok <- !is.symbol(expr)
    expr.ok <- expr.ok && as.character(expr[[1]]) == "("
    if(expr.ok){
        expr <- expr[[2]]
        expr.ok <- expr.ok && as.character(expr[[1]]) == "<-"
    }
    if(!expr.ok) stop(sprintf("'%s' is not an assignment operation",deparse(m$expr)))   
    if(!is.logical(condition)) stop(sprintf("'%s' is not a logical condition",
                                            deparse(m$condition)))
    lhs <- expr[[2]]
    rhs_val <- eval(expr[[3]],envir=parent.frame())
    lhs_val <- if(exists(lhs,envir=parent.frame())) eval(lhs,envir=parent.frame())
               else rep(NA,length(condition))
    if(length(lhs_val) > 1 && length(rhs_val) > 1 &&
       length(rhs_val) != sum(condition) &&
       length(rhs_val) != length(lhs_val))
        warning("Non-matching lengths in assignment")
    lhs_val[] <- ifelse(condition,rhs_val,lhs_val)
    assign(as.character(lhs),value=lhs_val,envir=parent.frame())
}

setGeneric("recode", function(x,...,
                              copy=getOption("recode_copy",identical(otherwise,"copy")),
                              otherwise=NA) standardGeneric("recode"))
setMethod("recode","item",function(x,...,
                                   copy=getOption("recode_copy",identical(otherwise,"copy")),
                                   otherwise=NA){
  recodings <- match.call(expand.dots=FALSE)$...
  recodings <- recodings[nzchar(sapply(recodings,paste,collapse=""))]
  if(length(otherwise)!=1) stop("otherwise= argument must have length 1")
  
  if(any(sapply(sapply(recodings,"[[",1),paste)!="<-"))
    stop("invalid recoding request")
  if(!length(recodings)) return(x)
  newcodes <- lapply(recodings,"[[",2)
  conditions <- lapply(recodings,"[[",3)
  has.range <- paste(lapply(conditions,"[[",1)) == "range"
  if(any(has.range)){
    oldcodes <- conditions
    has.min <- paste(lapply(conditions[has.range],"[[",2)) == "min"
    has.max <- paste(lapply(conditions[has.range],"[[",3)) == "max"
    if(any(has.min)){
      min.list <- list(min=min(x))
      conditions[has.range][has.min] <- lapply(
                conditions[has.range][has.min],
                function(x) do.call("substitute",list(x,min.list))
                )
    }
    if(any(has.max)){
      max.list <- list(max=max(x))
      conditions[has.range][has.max] <- lapply(
                conditions[has.range][has.max],
                function(x) do.call("substitute",list(x,max.list))
                )
    }
  }
  else
      oldcodes <- sapply(conditions,eval)
  NA_recoded <- any(unlist(suppressWarnings(sapply(oldcodes,is.na))))
      
  conditions[has.range] <- lapply(conditions[has.range],
                          function(x)
                          as.call(list(as.symbol("&"),
                            as.call(list(as.symbol("<="),x[[2]],quote(x@.Data))),
                            as.call(list(as.symbol("<="),quote(x@.Data),x[[3]]))
                            )
                          ))
  conditions[!has.range] <- lapply(conditions[!has.range],
                          function(x)
                          as.call(list(as.symbol("%in%"),quote(x@.Data),x))
                          )
  torecode <- sapply(conditions,eval,envir=environment(),enclos=parent.frame())
  if(!is.matrix(torecode)) torecode <- t(torecode)
  newcodes <- sapply(newcodes,eval,parent.frame())
  nevtrue <- colSums(torecode) == 0
  if(any(nevtrue)){
    nrecng <- paste(recodings[nevtrue])
    if(length(nrecng)>1)
      warning("recodings ",paste(nrecng,collapse=", ")," have no consequences")
    else
      warning("recoding ",nrecng," has no consequences")
  }
  ambiguous <- rowSums(torecode) > 1
  if(any(ambiguous))
    stop("recoding request is ambiguous")
  #y <- if(is.character(newcodes)) as.character(x) else x
  y <- x
  labels(y) <- NULL
  if(is.character(newcodes) && is.numeric(y)) {
    newcodes <- structure(
      seq_along(newcodes),
      names=newcodes
      )
  }
  #newcodes <- newcodes[!nevtrue]
  #torecode <- torecode[,!nevtrue,drop=FALSE]
  for(i in seq(along=newcodes)){
    if(!nevtrue[i])
      y[torecode[,i]] <- as.vector(newcodes[i],mode=storage.mode(y))
  }
  if(!copy){
    recoded <- as.logical(rowSums(torecode))
    tmp <- as.vector(otherwise,mode=storage.mode(y))
    length(otherwise) <- length(y)
    otherwise[] <- tmp
    y[!recoded] <- otherwise[!recoded]
    nNA <-sum(is.na(otherwise[!recoded])) - sum(is.na(x[!recoded]))
    if(nNA > 0)
      warning("recoding created ",nNA," NAs")
    if(!NA_recoded)
      y[is.na(x)] <- NA
}
  newvlab <- newcodes[nzchar(names(newcodes))]
  bijective <- FALSE
  lab.y <- lab.x <- labels(x)
  if(!is.list(oldcodes)){
      # This happens only in one-to-one recodings
      if(length(oldcodes) == length(newcodes) &&
          all(oldcodes %in% newcodes)){
          if(copy) bijective <- TRUE
          else if(length(lab.x) && all(lab.x@values %in% oldcodes))
              bijective <- TRUE
      }
  }
  if(length(lab.x) && bijective && !length(names(newcodes))){
      ii <- match(oldcodes,lab.x@values)
      jj <- match(newcodes,lab.y@values)
      lab.y@.Data[jj] <- lab.x@.Data[ii]
      labels(y) <- lab.y
  }
  else if(length(lab.x) && copy){
    lab.y.val <- lab.y@values
    lab.oldcodes <- lapply(conditions,Substitute,list(x=lab.y.val))
    lab.torecode <- sapply(lab.oldcodes,eval,envir=environment(),enclos=parent.frame())
    for(i in seq(along=newcodes)){
      lab.y.val[lab.torecode[,i]] <- as.vector(newcodes[i],mode=storage.mode(lab.y.val))
    }
    lab.y.val <- sort(unique(lab.y.val))
    ii <- match(lab.y.val,lab.y@values)
    lab.y.lab <- lab.y@.Data[ii]
    labels(y) <- new("value.labels",lab.y.lab,values=lab.y.val)
    if(length(newvlab))
      labels(y) <- labels(y) + newvlab
  }
  else if(length(newvlab)){
    if(length(names(otherwise)))
      newvlab <- new("value.labels",names(newvlab),values=newvlab) + otherwise
    labels(y) <- newvlab
    if(measurement(y) %in% c("interval","ratio"))
        measurement(y) <- "nominal"
  }
#   else if(length(newvlab)){
#     if(identical(otherwise,"copy"))
#       labels(y) <- labels(y) + newvlab
#     else
#       labels(y) <- newvlab
#     }

  return(y)
})

setMethod("recode","vector",function(x,...,
                                     copy=getOption("recode_copy",identical(otherwise,"copy")),
                                     otherwise=NA){
    call <- match.call()

    if(length(otherwise)!=1) stop("otherwise= argument must have length 1")

    if("recodes" %in% names(call)
       || is.character(call[[3]])
       ) {
        return(car_recode(var=x,...))
    }
    recodings <- match.call(expand.dots=FALSE)$...
    if("to.factor" %in% names(recodings)
       && is.logical(recodings[["to.factor"]])
       ) {
        warning("'to.factor' argument no longer in use")
        recodings["to.factor"] <- NULL
    }
    recodings <- recodings[nzchar(sapply(recodings,paste,collapse=""))]
    if(any(sapply(sapply(recodings,"[[",1),paste)!="<-"))
        stop("invalid recoding request")

    if(!length(recodings)) return(x)
    newcodes <- lapply(recodings,"[[",2)
    oldcodes <- lapply(recodings,"[[",3)
    NA_recoded <- any(unlist(suppressWarnings(sapply(oldcodes,is.na))))
    has.range <- paste(lapply(oldcodes,"[[",1)) == "range"
    if(any(has.range)){
        has.min <- paste(lapply(oldcodes[has.range],"[[",2)) == "min"
        has.max <- paste(lapply(oldcodes[has.range],"[[",3)) == "max"
        if(any(has.min)){
            min.list <- list(min=min(x, na.rm=TRUE))
            oldcodes[has.range][has.min] <- lapply(
                oldcodes[has.range][has.min],
                function(x) do.call("substitute",list(x,min.list))
            )
        }
        if(any(has.max)){
            max.list <- list(max=max(x, na.rm=TRUE))
            oldcodes[has.range][has.max] <- lapply(
                oldcodes[has.range][has.max],
                function(x) do.call("substitute",list(x,max.list))
            )
        }
    }
    oldcodes[has.range] <- lapply(oldcodes[has.range],
                                  function(x)
                                      as.call(list(as.symbol("&"),
                                                   as.call(list(as.symbol("<="),x[[2]],as.symbol("x"))),
                                                   as.call(list(as.symbol("<="),as.symbol("x"),x[[3]]))
                                                   )
                                              ))
    oldcodes[!has.range] <- lapply(oldcodes[!has.range],
                                   function(x)
                                       as.call(list(as.symbol("%in%"),as.symbol("x"),x))
                                   )
    torecode <- sapply(oldcodes,eval,envir=environment())
    if(!is.matrix(torecode)) torecode <- t(torecode)
    newcodes <- sapply(newcodes,eval,parent.frame())
    nevtrue <- colSums(torecode, na.rm=TRUE) == 0
    if(any(nevtrue)){
        nrecng <- paste(recodings[nevtrue])
        if(length(nrecng)>1)
            warning("recodings ",paste(nrecng,collapse=", ")," have no consequences")
        else
            warning("recoding ",nrecng," has no consequences")
    }
    ambiguous <- rowSums(torecode, na.rm=TRUE) > 1
    if(any(ambiguous))
        stop("recoding request is ambiguous")
    y <- if(is.character(newcodes)) as.character(x) else x
    newcodes <- newcodes[!nevtrue]
    torecode <- torecode[,!nevtrue,drop=FALSE]
    for(i in seq(along=newcodes)){
        y[torecode[,i]] <- newcodes[i]
    }
    if(!copy){
        if(is.character(otherwise)) newcodes <- c(newcodes, otherwise)
        recoded <- as.logical(rowSums(torecode))
        recoded[is.na(recoded)] <- TRUE
        tmp <- as.vector(otherwise,mode=storage.mode(y))
        length(otherwise) <- length(y)
        otherwise[] <- tmp
        y[!recoded] <- otherwise[!recoded]
        nNA <- sum(is.na(otherwise[!recoded])) - sum(is.na(x[!recoded]))
        if(nNA > 0)
            warning("recoding created ",nNA," NAs")
        if(!NA_recoded)
            y[is.na(x)] <- NA
    }
    if(is.character(y)) {
        levels <- sort(unique(y[y%nin%newcodes]))
        levels <- c(newcodes,levels)
        y <- factor(y,levels=levels)
    }
    return(y)
})

setMethod("recode","factor",function(x,...,copy=getOption("recode_copy",identical(otherwise,"copy")),
                                     otherwise=NA){
  recodings <- match.call(expand.dots=FALSE)$...

  if(length(otherwise)!=1) stop("otherwise= argument must have length 1")

  if("to.factor" %in% names(recodings)
    && is.logical(recodings[["to.factor"]])
    ) {
      warning("'to.factor' argument no longer in use")
      recodings["to.factor"] <- NULL
    }
  recodings <- recodings[nzchar(sapply(recodings,paste,collapse=""))]
  if(any(sapply(sapply(recodings,"[[",1),paste)!="<-"))
    stop("invalid recoding request")

  if(!length(recodings)) return(x)
  newcodes <- sapply(recodings,"[[",2)
  if(copy)
      newcodes <- as.character(newcodes) # 'copy=TRUE' requires the result also to be a factor ... 
  if(is.character(newcodes))
      to_factor <- TRUE
  else
      to_factor <- FALSE
  oldcodes <- lapply(recodings,"[[",3)
  oldcodes <- lapply(oldcodes,eval,envir=environment())
  torecode <- sapply(oldcodes,function(o) x %in% o)
  if(!is.matrix(torecode)) torecode <- t(torecode)
  nevtrue <- colSums(torecode) == 0
  if(any(nevtrue)){
    nrecng <- paste(recodings[nevtrue])
    if(length(nrecng)>1)
      warning("recodings ",paste(nrecng,collapse=", ")," have no consequences")
    else
      warning("recoding ",nrecng," has no consequences")
  }
  ambiguous <- rowSums(torecode) > 1
  if(any(ambiguous))
    stop("recoding request is ambiguous")

  if(to_factor){
      y <- integer(length=length(x))
      for(i in 1:ncol(torecode)){
          y[torecode[,i]] <- i
      }
      if(copy){
          olevels <- levels(unique(x[y==0,drop=TRUE]))
          max.y <- max(y)
          for(i in seq(along=olevels)){
              y[x == olevels[i]] <- max.y + i
          }
          y <- factor(y,levels=1:max(y),labels=c(newcodes,olevels))
      }
      else if(is.character(otherwise)){
          max.y <- max(y)
          y[y == 0] <- max.y + 1
          y <- factor(y,levels=1:max(y),labels=c(newcodes,otherwise[1]))
      }
      else {
          y <- factor(y,levels=1:max(y),labels=newcodes)
          recoded <- as.logical(rowSums(torecode))
          nNA <- sum(is.na(y[!recoded]) & !is.na(x[!recoded]))
          if(nNA > 0)
              warning("recoding created ",nNA," NAs")
      }
  }
  else {
      y <- vector(mode=storage.mode(newcodes),
                  length=length(x))
      for(i in 1:ncol(torecode))
          y[torecode[,i]] <- newcodes[i]
      recoded <- as.logical(rowSums(torecode))
      recoded[is.na(recoded)] <- TRUE
      tmp <- as.vector(otherwise,mode=storage.mode(y))
      length(otherwise) <- length(y)
      otherwise[] <- tmp
      y[!recoded] <- otherwise[!recoded]      
      nNA <- sum(is.na(y[!recoded]) & !is.na(x[!recoded]))
      if(nNA > 0)
          warning("recoding created ",nNA," NAs")
  }
  return(y)
})

spss.portable.file <- function(
    file,
    varlab.file=NULL,
    codes.file=NULL,
    missval.file=NULL,
    count.cases=TRUE,
    to.lower=getOption("spss.por.to.lower",FALSE),
    iconv=TRUE,
    encoded=getOption("spss.por.encoding","cp1252"),
    negative2missing = FALSE
    ){
    file <- path.expand(file)
    check.file(file,error=TRUE)
    ptr <- porStream(file)

    data.spec <- parseHeaderPorStream(ptr,iconv=iconv,encoded=encoded)
    types <- data.spec$types
    variables <- vector(length(types),mode="list")
    variables[types==0] <- list(new("double.item"))
    variables[types>0] <- list(new("character.item"))

    varprintfmt <- lapply(data.spec$dictionary,"[[",i="printformat")
    varprintfmt <- sapply(varprintfmt,"[",i=1)
    vardatetime <- varprintfmt %in% c(20,22:24,26:30,38,39)
    variables[vardatetime] <- list(new("datetime.item",
                                       tzone="GMT",
                                       origin="1582-10-14"))

    names(variables) <- names(types)
    
    varlabs <- lapply(data.spec$dictionary,"[[",i="label")
    varlabs <- unlist(varlabs)
    if(length(varlabs))
        varlabs <- varlabs[nzchar(varlabs)]

    vallabs <- data.spec$value.labels
    vallabs.vars <- lapply(vallabs,"[[",i="vars")
    vallab.tmp <- lapply(seq_along(vallabs),function(i){
                        ans <- list()
                        ans[vallabs.vars[[i]]] <- list(vallabs[[i]]$value.labels)
                        ans
                      })
    vallabs <- unlist(vallab.tmp,recursive=FALSE)
    
    missings <- lapply(data.spec$dictionary,"[[",i="missing")

    if(length(varlab.file) && check.file(varlab.file,error=TRUE)){
      message("using ",varlab.file)
      varlabs <- spss.parse.variable.labels(varlab.file,
                                            iconv=iconv,
                                            encoded=encoded)
      }
    if(length(codes.file) && check.file(codes.file,error=TRUE)){
      message("using ",codes.file)
      vallabs <- spss.parse.value.labels(codes.file,
                                         iconv=iconv,
                                         encoded=encoded)
      }
    if(length(missval.file) && check.file(missval.file,error=TRUE)){
      message("using ",missval.file)
      missings <- spss.parse.missing.values(missval.file)
      }

    if(length(varlabs))
      variables[names(varlabs)] <- mapply("description<-",variables[names(varlabs)],varlabs)
    if(length(vallabs))
      suppressWarnings(variables[names(vallabs)] <- mapply("labels<-",variables[names(vallabs)],vallabs))
    if(length(missings))
      variables[names(missings)] <- mapply("missing.values<-",variables[names(missings)],missings)
    if(negative2missing)
      variables <- lapply(variables,`valid.range<-`,c(0,Inf))
    
    if(count.cases){
        ncases <- .Call("countCasesPorStream",ptr,types)
        seekPorStream(ptr,data.spec$start.data)
    } else
        ncases <- NA
    attr(ptr,"ncases") <- ncases
    
    if(to.lower){
      names(variables) <- tolower(names(variables))
    }

    # if(iconv)
    #     variables <- lapply(variables,Iconv,from=encoded,to="UTF-8")
    # else
    #     encoded = ""

    warn_if_duplicate_labels(variables)
        
    document <- data.spec$document
    data.spec$document <- NULL

    new("spss.portable.importer",
      variables,
      ptr=ptr,
      varlab.file=varlab.file,
      codes.file=codes.file,
      missval.file=missval.file,
      document=document,
      data.spec=data.spec,
      encoded=encoded
      )
}
setMethod("initialize","spss.portable.importer",function(.Object,
                                                         variables,
                                                         ptr,
                                                         varlab.file=character(),
                                                         codes.file=character(),
                                                         missval.file=character(),
                                                         document=character(),
                                                         data.spec,
                                                         encoded
                                                         ){
     .Object@.Data <- variables
     .Object@ptr <- ptr
     .Object@varlab.file <- as.character(varlab.file)
     .Object@codes.file <- as.character(codes.file)
     .Object@missval.file <- as.character(missval.file)
     .Object@document <- as.character(document)
     .Object@data.spec <- data.spec
     .Object@encoded <- encoded
     .Object
})

setMethod("getNobs","spss.portable.importer",function(x){
  ncases <- attr(x@ptr,"ncases")
  if(!length(ncases)) {
    seekPorStream(x@ptr,x@data.spec$start.data)
    attr(x@ptr,"ncases") <- ncases <- .Call("countCasesPorStream",x@ptr,x@data.spec$types)
    seekPorStream(x@ptr,x@data.spec$start.data)
  }
  ncases
})

setMethod("seekData","spss.portable.importer",function(x)
  seekPorStream(x@ptr,x@data.spec$start.data)
)

setMethod("readData","spss.portable.importer",
  function(x,n)
    iconv_list(.Call("readDataPorStream",
           x@ptr,
           what=x,
           nlines=n,
           types=x@data.spec$types),
      encoded=x@encoded))

setMethod("readSlice","spss.portable.importer",
  function(x,rows,cols)
    iconv_list(.Call("readSlicePorStream",x@ptr,
           what=x,
           j=cols,i=rows,
           types=x@data.spec$types),
      encoded=x@encoded))

setMethod("readChunk","spss.portable.importer",
  function(x,nrows,cols)
    iconv_list(.Call("readChunkPorStream",x@ptr,
           what=x,
           vars=cols,n=nrows,
           types=x@data.spec$types),
      encoded=x@encoded))

setMethod("show","spss.portable.importer",
  function(object){
    file.name <- attr(object@ptr,"file.name")
    nobs <- nrow(object)
    nvar <- ncol(object)
    varlab.file <- object@varlab.file
    codes.file <- object@codes.file
    missval.file <- object@missval.file
    cat("\nSPSS portable file",sQuote(file.name),"\n\twith ")
    cat(nvar,"variables and ")
    cat(nobs,"observations\n")
    if(length(varlab.file)) cat("\twith variable labels from file",sQuote(varlab.file),"\n")
    if(length(codes.file)) cat("\twith value labels from file",sQuote(codes.file),"\n")
    if(length(missval.file)) cat("\twith missing value definitions from file",sQuote(missval.file),"\n")
})

subset.spss.portable.importer <- subset.importer

dta117_file_open <- function(file) .Call("dta117_file_open",file,"rb")
dta117_file_close <- function(file) .Call("dta117_file_close",file)
dta117_check_magic <- function(ptr) .Call("dta117_check_magic",ptr)
dta117_read_header <- function(ptr) .Call("dta117_read_header",ptr)
dta117_read_map <- function(ptr) .Call("dta117_read_map",ptr)
dta117_read_vtypes <- function(ptr) .Call("dta117_read_vtypes",ptr)
dta117_read_vnames <- function(ptr) .Call("dta117_read_vnames",ptr)
dta117_read_sortlist <- function(ptr) .Call("dta117_read_sortlist",ptr)
dta117_read_formats <- function(ptr) .Call("dta117_read_formats",ptr)
dta117_read_vlab_names <- function(ptr) .Call("dta117_read_vlab_names",ptr)
dta117_read_varlabs <- function(ptr) .Call("dta117_read_varlabs",ptr)
dta117_read_vallabs <- function(ptr) .Call("dta117_read_vallabs",ptr)
dta117_make_prototype <- function(types) .Call("dta117_make_prototype",types)
dta117_dim  <- function(types) .Call("dta117_dim",types)
dta117_seek_data <- function(ptr) .Call("dta117_seek_data",ptr)
dta117_read_data <- function(ptr,what,n,types) .Call("dta117_read_data",ptr,what,n,types)
dta117_read_slice <- function(ptr,what,vars,obs,types) .Call("dta117_read_slice",
                                                             ptr,what,vars,obs,types)
dta117_read_chunk <- function(ptr,what,vars,n,types) .Call("dta117_read_chunk",
                                                           ptr,what,vars,n,types)

dta117.byte   <- 65530
dta117.short  <- 65529
dta117.long   <- 65528
dta117.float  <- 65527
dta117.double <- 65526

dta117.byte.missrange   <- c(0x64,0x7f)
dta117.short.missrange  <- c(0x7fe5,0x7fff)
dta117.long.missrange   <- c(0x7fffffe5L,0x7fffffffL)
dta117.float.missrange  <- c(2.0^0x7f,Inf)
dta117.double.missrange <- c(2.0^0x3ff,Inf)

dta117.missnames <- paste(".",c("",letters),sep="")
dta117.byte.misslab <- structure(0x64 + 1:27,names=dta117.missnames)
dta117.short.misslab <- structure(0x7fe4 + 1:27,names=dta117.missnames)
dta117.long.misslab <- structure(0x7fffffe4L + 1:27,names=dta117.missnames)
dta117.float.misslab <- structure((1+(0:26)/16^3)*2.0^0x7f,names=dta117.missnames)
dta117.double.misslab <- structure((1+(0:26)/16^3)*2.0^0x3ff,names=dta117.missnames)

dta117_missing.values <- function(types){
    nvar <- length(length)
    missing.values <- vector(nvar,mode="list")
    missing.values[types==dta117.byte]   <- list(list(range=dta117.byte.missrange))
    missing.values[types==dta117.short]  <- list(list(range=dta117.short.missrange)) 
    missing.values[types==dta117.long]   <- list(list(range=dta117.long.missrange))
    missing.values[types==dta117.float]  <- list(list(range=dta117.float.missrange)) 
    missing.values[types==dta117.double] <- list(list(range=dta117.double.missrange))
    names(missing.values) <- names(types)
    missing.values
}

dta117_missval_labels <- function(types){
    nvar <- length(length)
    missval_labels <- vector(nvar,mode="list")
    missval_labels[types==dta117.byte]   <- list(dta117.byte.misslab)
    missval_labels[types==dta117.short]  <- list(dta117.short.misslab) 
    missval_labels[types==dta117.long]   <- list(dta117.long.misslab)
    missval_labels[types==dta117.float]  <- list(dta117.float.misslab)
    missval_labels[types==dta117.double] <- list(dta117.double.misslab)
    names(missval_labels) <- names(types)
    missval_labels
}

setGeneric("measurement_autolevel",function(x,...)standardGeneric("measurement_autolevel"))

setMethod("measurement_autolevel",
          signature(x="ANY"),function(x,...)x)

# If the most values are labelled, set the measurement level to "nominal" or "ordinal".
setMethod("measurement_autolevel",
          signature(x="item.vector"),
          function(x,
                   to=getOption("measurement.adapt.default","nominal"),
                   threshold=getOption("measurement.adapt.threshold",.75),
                   ...){
    mnt <- measurement(x)
    lab <- labels(x)
    if(!length(lab) || mnt%in%c("nominal","ordinal"))
        return(x)
    else {
        is_labd <- x %in% lab@values
        is_vald <- is.valid(x)
        is_labd_n_vald <- is_labd & is_vald
        prop_labd <- sum(is_labd_n_vald)/sum(is_vald)
        if(prop_labd > threshold)
            measurement(x) <- to
        return(x)
    }
})

setMethod("measurement_autolevel",
          signature(x="data.set"),
          function(x,
                   to=getOption("measurement.adapt.default","nominal"),
                   threshold=getOption("measurement.adapt.threshold",.75),
                   except=NULL,
                   only=NULL,
                   ...){
  nms <- names(x)
  if(!missing(except)){
      except <- substitute(except)
      if(length(except)>1)
          except <- sapply(except[-1],as.character)
      else
          except <- as.character(except)
      nms <- setdiff(nms,except)
  }
  if(!missing(only)){
      only <- substitute(only)
      if(length(only)>1)
          only <- sapply(only[-1],as.character)
      else
          only <- as.character(only)
      nms <- intersect(nms,only)
  }
  for(n in nms){
      x[[n]] <- measurement_autolevel(x[[n]])
  }
  return(x)
})

mkRule <- function(x,length) paste(rep_len(x,length),collapse="")

set_length <- function(x,n,fill=""){

    l <- length(x)
    nl <- min(n,l)

    y <- rep(fill,length=n)
    y[1:nl] <- x[1:nl]
    y
}

cols_folded <- function(x,n,sep){

    nc <- ncol(x)
    nr <- nrow(x)

    m <- nc/n
    dim(x) <- c(nr,n,m)
    y <- apply(x,c(1,3),paste,collapse=sep)
    y
}


mkGrpSpan <- function(hspan){

    hspan.out <- hspan
    l.hspan <- length(hspan)
    for(k in 2:l.hspan){
        hspan1 <- hspan[[k-1]]
        hspan2 <- hspan[[k]]
        hspan.out.k <- hspan2
        j1 <- 1
        s2 <- 0
        for(j2 in 1:length(hspan2)){
            s1 <- s2
            s2 <- s2 + hspan2[j2]
            c1 <- 0
            while(s1 < s2){
                s1 <- s1 + hspan1[j1]
                j1 <- j1 + 1
                c1 <- c1 + 1
            }
            if(s1 > s2) stop("misaligned spans")
            hspan.out.k[[j2]] <- c1
        }
        hspan.out[[k]] <- hspan.out.k
    }
    hspan.out
}

mtable_format_print <- function(x,
                                topsep="=",
                                bottomsep="=",
                                sectionsep="-",
                                interaction.sep = " x ",
                                center.at=getOption("OutDec"),
                                align.integers=c("dot","right","left"),
                                padding="  ",
                                ...)
    pf_mtable_format_print(preformat_mtable(x),
                                   topsep=topsep,
                                   bottomsep=bottomsep,
                                   sectionsep=sectionsep,
                                   interaction.sep =interaction.sep,
                                   center.at=center.at,
                                   align.integers=align.integers,
                                   padding=padding,
                                   ...)

pf_mtable_format_print <- function(x,
                                   topsep="=",
                                   bottomsep="=",
                                   sectionsep="-",
                                   interaction.sep = " x ",
                                   center.at=getOption("OutDec"),
                                   align.integers=c("dot","right","left"),
                                   padding="  ",
                                   show.parmtypes = nrow(x$parmtab) > 1,
                                   ...
                                   ){

    colsep <- " "
    rowsep <- "\n"

    pt <- x$parmtab
    sst <- x$summary.stats
    leaders <- x$leaders
    headers <- x$headers
    eq.headers <- x$eq.headers

    l.headers <- length(headers)
    
    res <- NULL
    ncols <- NULL

    has.eq.headers <- length(eq.headers) > 0
    
    for(j in 1:ncol(pt)){

        name.j <- colnames(pt)[j]
        pt.j <- pt[,j]
        l.pt.j <- length(pt.j)
        
        ncol.j <- unique(sapply(pt.j,ncol))
        stopifnot(length(ncol.j)==1)
        ncols <- c(ncols,ncol.j)

        skip.j <- logical(0)
        max.width <- 0

        maxncols.j <- max(unlist(lapply(pt.j,ncol)))
        
        nr.j <- numeric(l.pt.j)
        for(i in 1:l.pt.j){
            pt.ij <- pt.j[[i]]
            if(show.parmtypes){
                pt.ij <- rbind(" ",pt.ij)
            }
            nr.j[i] <- nrow(pt.ij)
            skip.ij <- rep(FALSE,nrow(pt.ij))
            tmp <- matrix("",nrow=nrow(pt.ij),ncol=maxncols.j)
            ii <- 1:ncol(pt.ij)
            tmp[,ii] <- pt.ij
            pt.j[[i]] <- pt.ij
            skip.j <- c(skip.j,skip.ij)
        }
        pt.j <- do.call(rbind,pt.j)
        
        if(length(sst)){
            tmp <- sst[[j]]
            sst.j <- matrix("",nrow=length(tmp),ncol=maxncols.j)
            sst.j[,1] <- tmp
            pt.j <- rbind(pt.j,sst.j)
            skip.j <- c(skip.j,rep(FALSE,length(tmp)))
        }

        pt.j <- apply(pt.j,2,centerAt,skip=skip.j)

        if(has.eq.headers){
            eq.header.j <- eq.headers[[name.j]]
            n.eq.j <- length(eq.header.j)
            eq.span <- ncol(pt.j)/n.eq.j
            dim(pt.j) <- c(nrow(pt.j),eq.span,n.eq.j)
            pt.j <- apply(pt.j,c(1,3),paste,collapse=colsep)
            pt.j <- rbind(eq.header.j,pt.j)
            pt.j <- apply(pt.j,2,format,justify="centre")
            pt.j <- apply(pt.j,1,paste,collapse=colsep)
        }
        else
            pt.j <- apply(pt.j,1,paste,collapse=colsep)
        res <- cbind(res,pt.j)
    }

    hlines <- character()
    if(l.headers){
        headers1 <- Map(structure,list(""),span=ncols) # To take care of multi-eqn models
        headers <- c(headers,list(headers1))
        headers <- rev(headers)
        hspan <- list()
        
        for(k in 1:length(headers)){
            hspan[[k]] <- sapply(headers[[k]],attr,"span")
        }
        hspan <- mkGrpSpan(hspan)
        headers <- headers[-1]
        hspan <- hspan[-1]

        for(k in 1:l.headers){

            headers.k <- headers[[k]]
            hspan.k <- hspan[[k]]
            res.grpd <- NULL
            cumsum <- 0
            
            for(j in 1:length(headers.k)){

                header.jk <- headers.k[[j]]
                hspan.jk <- hspan.k[[j]]
                res.cur.grp <- res[,cumsum + 1:hspan.jk,drop=FALSE]
                res.cur.grp <- apply(res.cur.grp,1,paste,collapse=colsep)
                nchar.jk <- nchar(res.cur.grp[1])
                if(has.eq.headers && k==1 || k > 1 && l.headers > 1){
                    header.sep <- mkRule(sectionsep,nchar.jk)
                    res.cur.grp <- c(header.sep,res.cur.grp)
                }
                res.cur.grp <- c(header.jk,res.cur.grp)
                res.cur.grp <- format(res.cur.grp,justify="centre")
                res.grpd <- cbind(res.grpd,res.cur.grp)

                cumsum <- cumsum + hspan.jk

            }
            res <- res.grpd
        }
    }
    res <- apply(res,2,function(x)paste0(" ",x))

    ld.headlines <- 0
    if(l.headers)
        ld.headlines <- ld.headlines + l.headers*2 - 1
    if(has.eq.headers){
        if(l.headers)
            ld.headlines <- ld.headlines + 2
        else
            ld.headlines <- ld.headlines + 1
    }
    l.leaders <- length(leaders)
    if(l.leaders){
        leaders <- c(list(headers=rep(list(structure("",span=1)),
                                          ld.headlines)),leaders)
        leaders <- lapply(leaders,ldxp)
        if(show.parmtypes){
            parmtypes <- rownames(x$parmtab)
            for(p in parmtypes){
                leaders[[p]] <- rbind(p,leaders[[p]])
            }
        }
        leaders <- do.call(rbind,leaders)
        leaders <- format(leaders,justify="left")
        res <- cbind(leaders,res)
    }

    res <- apply(res,1,paste,collapse=colsep)
    res <- paste0(padding,res,padding)
    width <- nchar(res[1])
    toprule <- mkRule(topsep,width)
    sectionrule <- mkRule(sectionsep,width)
    bottomrule <- mkRule(topsep,width)

    csum <- ld.headlines

    sectsep.at <- integer()
    sectseps <- character()

    sec.hrules.at <- integer()
    sec.hrules <- character()
    
    leaders.gap <- mkRule(" ",nchar(leaders[i]))
    for(i in 1:nrow(pt)){
        sectseps   <- c(sectseps,   sectionrule)
        sectsep.at <- c(sectsep.at, csum)
        csum <- csum + nrow(pt[[i,1]])
        if(show.parmtypes)
            csum <- csum + 1
    }
    if(length(sst) && any(sapply(sst,length)>0)){
        sectseps   <- c(sectseps,   sectionrule)
        sectsep.at <- c(sectsep.at, csum)
    }
    
    res <- .insert(res,
                   c(sec.hrules.at,sectsep.at),
                   c(sec.hrules,   sectseps)
                   )
   
    res <- c(toprule,res,bottomrule)

    signif.symbols <- x$signif.symbols
    if(length(signif.symbols)){
        signif.template <- getOption("signif.symbol.print.template",
                                     signif.symbol.print.default.template)
        signif.symbols <- format_signif_print(signif.symbols,
                                              signif.template,
                                              width=nchar(bottomrule)-2*nchar(padding))
        signif.symbols <- paste0(padding,signif.symbols,padding)
        res <- c(res,signif.symbols)
    }
    res <- paste0(res,rowsep)
    return(res)
}

.insert <- function(x,where,what){

    l <- rep(list(NULL),length(x))

    what <- rep_len(what,length(where))

    use <- where > 0
    where <- where[use]
    what <- what[use]

    l[where] <- what
    y <- Map(c,x,l)
    unlist(y,recursive=FALSE)
}

.last <- function(x){
    l <- length(x)
    x[[l]]
}

format_signif_print <- function(syms,tmpl,width,dec="."){
    title <- tmpl[1]
    clps <- tmpl[3]
    tmpl <- tmpl[2]
    res <- title
    empty.title <- paste(rep(" ",nchar(title)),collapse="")
    
    ns <- length(syms)
    for(i in 1:ns){
        sym <- names(syms)[i]
        thrsh <- unname(syms[i])
        thrsh <- gsub(".",dec,thrsh,fixed=TRUE)
        res.i <- sub("$sym",sym,tmpl,fixed=TRUE)
        res.i <- sub("$val",thrsh,res.i,fixed=TRUE)
        if(i < ns)
            res.i <- paste0(res.i,clps)
        len <- length(res)
        res.l <- res[len]
        n.res.l <- nchar(res.l)
        n.res.i <- nchar(res.i)
        if(n.res.l+n.res.i <= width)
            res[len] <- paste0(res.l,res.i)
        else
            res <- c(res,paste0(empty.title,res.i))
    }
    res
}
signif.symbol.print.default.template <- c("Significance: ","$sym = p < $val","; ")

.row_prepend <- function(x,where,value){
    l <- length(where)
    stopifnot(l == length(value))
    where_ <- where + 1:l - 1
    y <- matrix("",nrow(x)+l,ncol(x))
    nr.y <- nrow(y)
    rows.y <- 1:nr.y
    other <- rows.y[!(rows.y %in% where_)]
    y[other,] <- x
    for(i in 1:l){
        wh.i <- where_[l]
        y[wh.i,] <- value[[i]]
    }
    y
}
.drop_last <- function(x) x[-length(x)]

format_html.descriptions <- function(x,
                                     toprule=2,midrule=1,bottomrule=2,
                                     margin="2ex auto",
                                     ...){

  padleft <- c("padding-left"="0.3em")
  padright <- c("padding-right"="0.3em")
  toprule <- c("border-top"=paste0(midrule,"px solid"))
  bottomrule <- c("border-bottom"=paste0(midrule,"px solid"))
  midrule_above <- c("border-top"=paste0(midrule,"px solid"))
  midrule <- c("border-bottom"=paste0(midrule,"px solid"))
  align.right <- c("text-align"="right")  
  align.left <- c("text-align"="left")  
  align.center <- c("text-align"="center")
  row_style <- c("border-style"="none")
  table_style <- c("border-collapse"="collapse" ,"border-style"="none")


  tab <- cbind(
          html_td(names(x),style=css(align.left,padleft,padright),vectorize=TRUE),
          html_td(x,style=css(align.left,padright),vectorize=TRUE)
          )
  n <- nrow(tab)
  tab[n,] <- lapply(tab[n,],setStyle,bottomrule)
  
  header <- c(html_td("Variable",style=css(align.left,padleft,padright,toprule,midrule)),
              html_td("Description",style=css(align.center,padright,toprule,midrule)))
                    
  tab <- rbind(header,tab)
  ans <- apply(tab,1,html_tr,style=as.css(row_style))
  
  ans <- html_table(ans)
  
  if(length(margin))
    table_style <- c(table_style,margin=margin)
  style(ans) <- as.css(table_style)

  ans <- as.character(ans)
  return(ans)
}

retain <- function (..., list = character(0), envir = parent.frame(),force=FALSE)
{
    dots <- match.call(expand.dots = FALSE)$...
    if(environmentName(envir)==environmentName(globalenv()) && !force) {
      warning("retain will remove objects from global environment only if force=TRUE")
      return(invisible(NULL))
    }
    if(!length(dots)) stop("at least one object has to be retained")
    if (!all(sapply(dots, function(x) is.symbol(x) ||
        is.character(x))))
        stop("... must contain names or character strings")
    names <- sapply(dots, as.character)
    if (length(names) == 0)
        names <- character(0)
    list <- .Primitive("c")(list, names)
    obs <- ls(envir=envir)
    if(!all(list %in% obs)) {
        mis <- setdiff(list,obs)
        mis <- paste(mis,collapse=", ")
        stop("Undefined object(s) ",mis)
    }
    to.remove <- obs[!(obs %in% list)]
    remove(list=to.remove, envir=envir, inherits=FALSE)
}

combine_duplicated_labels <- function(x){
    dl <- duplicated_labels(x)
    l <- labels(x)
    for(i in seq_along(dl)){
        old_val <- dl[[i]]
        new_val <- old_val[1]
        drop_val <- old_val[-1]
        x[x %in% old_val] <- new_val
        drop <- l@values %in% drop_val
        l@.Data <- l@.Data[!drop]
        l@values <- l@values[!drop]
        labels(x) <- l
    }
    x
}


prefix_duplicated_labels <- function(x,
                                      pattern="%d. %s",
                                      ...){
    dl <- duplicated_labels(x)
    if(length(dl)){
        l <- labels(x)
        dedup_lab <- sprintf(pattern,l@values,l@.Data)
        l@.Data <- dedup_lab
        labels(x) <- l
    }
    x
}


postfix_duplicated_labels <- function(x,
                                      pattern="%s (%d)",
                                      ...){
    dl <- duplicated_labels(x)
    l <- labels(x)
    for(i in seq_along(dl)){
        dup_lab <- names(dl)[i]
        dup_val <- dl[[i]]
        ii <- match(dup_val,l@values)
        dedup_lab <- sprintf(pattern,dup_lab,dup_val)
        l@.Data[ii] <- dedup_lab
    }
    labels(x) <- l
    x
}

deduplicate_labels <- function(x,...) UseMethod("deduplicate_labels")
deduplicate_labels.default <- function(x,...) return(x)

deduplicate_labels.item <- function(x,method=c("combine codes",
                                               "prefix values",
                                               "postfix values"),...){
    method <- match.arg(method)
    # browser()
    switch(method,
           "combine codes"=combine_duplicated_labels(x),
           "prefix values"=prefix_duplicated_labels(x,...),
           "postfix values"=postfix_duplicated_labels(x,...))
}

deduplicate_labels.item.list <- function(x,...){
    n <- ncol(x)
    for(i in 1:n){
        x.i <- x[[i]]
        if(inherits(x.i,"item") &&
           length(labels(x.i)) &&
           length(duplicated_labels(x.i))){
            x.i <- deduplicate_labels.item(x.i,...)
            x@.Data[[i]] <- x.i
        }
    }
    x
}

setMethod("as.item","Date",function(x,...){

   annotation <-new("annotation",NULL)
   new("Date.item",
       as.numeric(x),
       measurement="date",
       annotation=annotation)
})

setMethod("as.item","Date.item",as_item_item)

as.Date.numeric.item <- function(x,...)
                          as.Date(as.numeric(x),
                                  origin=structure(0,class="Date"))

as.data.frame.Date.item <- function (x, row.names = NULL, optional = FALSE, ...)
{
    nrows <- length(x)
    nm <- paste(deparse(substitute(x), width.cutoff = 500), collapse = " ")
    if (is.null(row.names)) {
        if (nrows == 0L)
            row.names <- character(0L)
        else row.names <- .set_row_names(nrows)
    }

    x <- as.Date.numeric.item(x)

    value <- list(x)
    if (!optional)
        names(value) <- nm
    attr(value, "row.names") <- row.names
    class(value) <- "data.frame"
    value
}

str.Date.item <- function(object,give.head=TRUE,width=getOption("width"),...){
  if(give.head){
    hdr <- " Date item"
    cat(hdr,"")
  }
  str(as.Date.numeric.item(object),give.head=FALSE,width=width,...)
}


format.Date.item <- function(x,...){
  format(as.Date.numeric.item(x),...)
}
#setMethod("format","Date.item",format.Date.item)

print.Date.item <- function(x,
    width=getOption("width"),
    compress=FALSE,
    ...,print.gap=NULL){
    if(length(x)){
      mkdots <- function(n) paste(rep(".",n),collapse="")
      pg <- if(is.null(print.gap) || compress) 1 else print.gap
      l <- length(x)
      if(compress) {
        x <- x[seq_len(min(width,l))]
        x <- format(x,...)
        x <- trimws(x)
        xw <- cumsum(nchar(x,"w")+1)
        hdr <- paste(" [","1:",length(x),"]",sep="")
        width <- width - nchar(hdr)
        if(any(xw > width)){
          dots <- mkdots(3)
          width <- width - nchar(dots,"w") - 1
          upto <- max(which(xw <= width))
          x <- trimws(format(c(x[seq_len(upto)],dots)))
        }
        cat(hdr,x)
        cat("\n")
      }
      else
        print.default(
              format(x,...),
              quote=FALSE,print.gap=print.gap)
    }
    else
      print(as.Date.numeric.item(x),print.gap=print.gap)
}

setMethod("show","Date.item",function(object){
  cat("\nDate item",
    if(length(description(object))) sQuote(description(object)) else NULL,
    paste("(",
          "length = ",length(object),
          ")",sep=""),
    "\n\n")
  print.Date.item(object,width=getOption("width"),compress=TRUE)
})
setMethod("print","Date.item",print.Date.item)

setMethod("summary","Date.item",function(object,...,maxsum=100,digits=3){
  ism <- is.missing(object)
  xvalid <- as.Date.numeric.item(object[!ism])
  summary(xvalid)
})

setMethod("codebookEntry","Date.item",function(x){
  annotation <- annotation(x)
  filter <- x@value.filter
  spec <- c(
            "Storage mode:"=storage.mode(x),
            "Measurement:"="Date"
            )
  if(length(filter)) spec <- c(spec,
                              switch(class(filter),
                                        missing.values = c("Missing values:" = format(filter)),
                                        valid.values   = c("Valid values:"   = format(filter)),
                                        valid.range    = c("Valid range:"    = format(filter))
                                        ))
  ism <- is.missing(x)
  isna <- is.na(x)

  if(any(ism | isna)){
      tab <- missNAtab(ism,isna)
    if(length(weights)){
      wtab <- missNAtab(ism,isna,weights)
      tab <- collect(unweighted=tab,
                     weighted=tab)
    }
    else
      tab <- array(tab,
                   dim=c(dim(tab),1),
                   dimnames=c(dimnames(tab),
                              list(NULL)))
    attr(tab,"title") <- "Valid and missing values"
  } else
    tab <- integer(0)

  x <- as.Date(x)
  descr <- Descriptives(x)
  # if(length(weights)){
  #     wdescr <- Descriptives(x,weights)
  #     descr <- collect(unweighted=format(descr),
  #                      weighted=format(wdescr))
  # }
  # else 
  descr <- as.matrix(format(descr))

  stats <- list(tab=tab,
                descr=descr)
  new("codebookEntry",
    spec = spec,
    stats = stats,
    annotation = annotation
  )
})

## Methods for auxiliar functions

xtfrm.Date.item <- function(x) x@.Data
mtfrm.Date.item <- function(x) x@.Data

setMethod("as.character","Date.item",function(x,...)
   format(as.Date.numeric.item(x))
)

unique.Date.item <- unique.item.vector

setGeneric("getNobs",function(x)standardGeneric("getNobs"))
setGeneric("seekData",function(x)standardGeneric("seekData"))
setGeneric("readData",function(x,n)standardGeneric("readData"))
setGeneric("readSlice",function(x,rows,cols)standardGeneric("readSlice"))
setGeneric("readChunk",function(x,nrows,cols)standardGeneric("readChunk"))
setGeneric("codebookEntry",function(x,weights=NULL,unweighted=TRUE,...) standardGeneric("codebookEntry"))
# setGeneric("codebookStatsCateg",function(x,by=NULL) standardGeneric("codebookStatsCateg"))
# setGeneric("codebookStatsMetric",function(x,by=NULL) standardGeneric("codebookStatsMetric"))
setGeneric("relabel4",function(x,...) standardGeneric("relabel4"))

descr_format_stdstyle <- df_format_stdstyle

format_html.descriptions <- function(x,
                                     toprule=2,midrule=1,bottomrule=2,
                                     style=descr_format_stdstyle,
                                     margin="2ex auto",
                                     ...){

    tab <- cbind(
        html_td(names(x),vectorize=TRUE),
        html_td(x,vectorize=TRUE)
    )
    n <- nrow(tab)
    
    header <- c(html_td("Variable",class="header"),
                html_td("Description",class="header"))
    
    tab <- rbind(header,tab)
    res <- apply(tab,1,html_tr)
    
    res <- html_table(res,class="descriptions")

    style_global <- style_df_global(class="descriptions",style=style,margin=margin)

    style_toprule <- style_df_rule(class="descriptions",rulewidth=toprule,top=TRUE,
                                       rows=1)
    style_bottomrule <- style_df_rule(class="descriptions",rulewidth=bottomrule,bottom=TRUE,
                                          rows=n+1)
    style_midrule <- style_df_rule(class="descriptions",rulewidth=midrule,bottom=TRUE,
                                          rows=1)
    style_content <- paste(
        style_global,
        style_toprule,
        style_midrule,
        style_bottomrule,
        sep="\n"
    )
    style_element <- html("style",style_content,linebreak=TRUE)
    res <- html_group(style_element,res)
    res <- as.character(res)
    return(res)
}

trimws_drpz <- function(x){
  x <- trimws(x)
  x[nzchar(x)]
}

format_cb_table_ht <- function(tab){
    cn <- colnames(tab)
    if(ncol(tab)>2){
        tab <- cbind(
            formatC(tab[,1,drop=FALSE],format="d"),
            formatC(tab[,2],format="f",digits=1),
            formatC(tab[,3],format="f",digits=1)
        )
    }
    else {
        tab <- cbind(
            formatC(tab[,1,drop=FALSE],format="d"),
            formatC(tab[,2],format="f",digits=1)
        )
    }
    tab[tab=="NA"] <- ""
    tab <- rbind(" "=cn,"",tab)
    tab
}

format_html.codebookEntry <- function(x,name="",
                                      var_tag="code",
                                      varid_prefix="",
                                      title_tag="p",
                                      ...
){
  
  annot <- x@annotation
  description <- annot["description"]
  wording <- annot["wording"]
  if(length(annot)) annot <- annot[names(annot) %nin% c("description","wording")]
  
  title_html <- html(var_tag,.content=name[1],class="cbe-name")
  if(length(description) && !is.na(description))
    title_html <- html_group(title_html," &mdash; ",html("span",.content=sQuote(description),class="cbe-description"))
  
  title_html <- html(title_tag,.content=title_html,class="cbe-title",id=paste0(varid_prefix,name[1]))
  title_html <- html_group(html_p(""),title_html)
  
  wording_html <- if(length(wording) && !is.na(wording)) 
    html_p(dQuote(wording), class="cbe-wording") 
  else NULL
  x.spec <- gsub("--","&ndash;",x@spec)
  spec_html <- cbind(html_td(names(x.spec),vectorize=TRUE),
                     html_td(x.spec,vectorize=TRUE))
  spec_html <- apply(spec_html,1,html_tr)
  spec_html <- as.html_group(spec_html)
  
  spec_html <- html_table(spec_html,class="cbe-spec")
  spec_html <- html_group(html_p(""),spec_html)
  
  tab <- x@stats$tab
  descr <- x@stats$descr
  
  if(length(tab)){
    counts_html <- formatC(tab[,1,1],format="d")
    counts_html <- html_td(counts_html,vectorize=TRUE)

    perc_html <- formatC(tab[,-1,1,drop=FALSE],
                         format="f",digits=1)
    perc_html[perc_html=="NA"] <- ""
    perc_html <- spltDec(perc_html)
    perc_html <- html_td_spltDec(perc_html)

    tab_html <- c(
      counts_html,
      perc_html
    )

    if(dim(tab)[3] > 1){
      counts_html <- formatC(tab[,1,1],
                             format="f",digits=1)
      counts_html <- spltDec(counts_html)
      counts_html <- html_td_spltDec(counts_html)

      perc_html <- formatC(tab[,-1,2,drop=FALSE],
                            format="f",digits=1)
      perc_html[perc_html=="NA"] <- ""
      perc_html <- spltDec(perc_html)
      perc_html <- html_td_spltDec(perc_html)
      wtab_html <- c(
        counts_html,
        perc_html
      )
      tab_html <- c(tab_html,wtab_html)
    }  
    tab_html <- matrix(tab_html, nrow = nrow(tab))

    tab_rown <- trimws(rownames(tab))
    
    tab_lab <- strsplit(tab_rown," ")

    tab_lab <- lapply(tab_lab,trimws_drpz)
    
    tab_has_lab <- sapply(tab_lab,length) > 1
    tab_has_m <- sapply(sapply(tab_lab,"[",2) == "M",isTRUE)
    
    tab_val <- ifelse(tab_has_lab,sapply(tab_lab,"[",1),"")
    tab_lab <- ifelse(tab_has_lab,lapply(tab_lab,"[",-1,drop=FALSE),tab_lab)
    
    tab_m <- ifelse(tab_has_m,"M","")
    tab_lab <- ifelse(tab_has_m,lapply(tab_lab,"[",-1,drop=FALSE),tab_lab)
    
    tab_lab <- sapply(tab_lab,paste,collapse=" ")
    
    tab_lab_html <- cbind(html_td(tab_val,vectorize=TRUE),
                          html_td(tab_m,vectorize=TRUE),
                          html_td(tab_lab,vectorize=TRUE))
    
    tab_html <- cbind(tab_lab_html,tab_html)
    tab_html <- as.html_group(apply(tab_html,1,html_tr))

    if(dim(tab)[3] == 1){
      if(dim(tab)[2] == 3){
        tabh_html <- html_group(html_td("Values and labels",colspan=3,class="header"),
                                html_td("N",class="header"),
                                html_td("Valid",colspan=3,class="header"),
                                html_td("Total",colspan=3,class="header"))
      }
      else{
        tabh_html <- html_group(html_td("Values and labels",colspan=3,class="header"),
                                html_td("N",class="header"),
                                html_td("Percent",colspan=3,class="header"))
      }
      tabh_html <- html_tr(tabh_html)
    } else {
      if(dim(tab)[2] == 3){
        tabh_html2 <- html_group(html_td("Values and labels",colspan=3,class="header"),
                                html_td("N",class="header"),
                                html_td("Valid",colspan=3,class="header"),
                                html_td("Total",colspan=3,class="header"),
                                html_td("N",colspan=3,class="header"),
                                html_td("Valid",colspan=3,class="header"),
                                html_td("Total",colspan=3,class="header"))
        tabh_html1 <- html_group(html_td("",colspan=3),
                               html_td("Unweighted",colspan=7,class="header"),
                               html_td("Weighted",colspan=9,class="header"))
      }
      else{
        tabh_html2 <- html_group(html_td("Values and labels",colspan=3,class="header"),
                                html_td("N",class="header"),
                                html_td("Percent",colspan=3,class="header"),
                                html_td("N",colspan=3,class="header"),
                                html_td("Percent",colspan=3,class="header"))
        tabh_html1 <- html_group(html_td("",colspan=3),
                               html_td("Unweighted",colspan=4,class="header"),
                               html_td("Weighted",colspan=6,class="header"))
      }
      
      tabh_html1 <- html_tr(tabh_html1)
      tabh_html2 <- html_tr(tabh_html2)
      tabh_html <- c(tabh_html1,tabh_html2)
    }
    
    tab_html <- html_table(html_group(tabh_html,tab_html),class="cbe-table cbe-vl-table")
    tab_html <- html_group(html_p(""),tab_html)
  }
  else tab_html <- NULL
  
  if(length(descr)){
    descr_ldr_html <- trimws(paste(rownames(descr),": ",sep=""))
    descr_ldr_html <- html_td(descr_ldr_html,vectorize=TRUE)
    descr_html <- formatC(descr,format="f",digits=3)
    
    descr_html <- spltDec(descr_html)
    descr_html <- html_td_spltDec(descr_html)

    dim(descr_html) <- dim(descr)

    descr_html <- cbind(descr_ldr_html,descr_html)
    descr_html <- apply(descr_html,1,html_tr)

    if(ncol(descr)>1){
      descrh_html <- html_group(html_td(""),
                                html_td("Unweighted",colspan=3,class="header"),
                                html_td("&nbsp;Weighted",colspan=3,class="header"))
      descr_html <- c(list(html_tr(descrh_html)),descr_html)
    }
    
    descr_html <- html_table(descr_html,class="cbe-table cbe-d-table")
    descr_html <- html_group(html_p(""),descr_html)
  } else descr_html <- NULL
  
  if(length(annot)){
    annot_html <- cbind(html_dt(paste0(names(annot),":"),vectorize=TRUE),
                        html_dd(annot,vectorize=TRUE))
    annot_html <- as.html_group(t(annot_html))
    annot_html <- html("dl",.content=annot_html,class="cbe-annotations")
    annot_html <- html_group(html_p(""),annot_html)
  } else annot_html <- NULL
  
  header_html <- html_div(html_group(title_html,wording_html),class="cbe-header")
  body_html <- html_div(html_group(spec_html,tab_html,descr_html,annot_html,html_p("")),class="cbe-body")

  cbe_html <- html_div(
      html_group(header_html,body_html),class="codebook-entry")
  cbe_html <- as.character(cbe_html)
  return(cbe_html)
}

codebook_format_stdstyle <- c(
    "padding-left"=".3em",
    "padding-right"=".3em"
)

format_html.codebook <- function(x,
                                 toprule=2,
                                 midrule=1,
                                 indent="3ex",
                                 style=codebook_format_stdstyle,
                                 var_tag="code",
                                 varid_prefix="",
                                 title_tag="p",
                                 ...)
{
  out <- mapply(format_html,x=x@.Data,name=names(x),
                MoreArgs = list(
                    var_tag=var_tag,
                    varid_prefix=varid_prefix,
                    title_tag=title_tag
                  ))
  res <- html_div(unlist(out),class="codebook")

  cb_format_tmpl <- "
    .cbe-body table {
         margin: unset;
    }
    .cbe-body table,
    .cbe-body table tr,
    .cbe-body table td
    {
         border-style: none;
         padding: unset;
    }
    .cbe-header {
         border-top: solid 2px;
         border-bottom: solid 1px;
         padding-left: <<indent>>;
    }
    .cbe-body {
         padding-left: <<indent>>;
    }
    .cbe-table {
         border-collapse: collapse;
    }
    .cbe-table td:nth-child(n of .header) {
         text-align: center;
         padding-left: <<padding-left>>;
         padding-right: <<padding-right>>;
    }
    .cbe-vl-table td:nth-child(1) {
         text-align: right;
         padding-left: <<padding-left>>;
         padding-right: <<padding-right>>;
    }
    .cbe-vl-table td:nth-child(2) {
         text-align: center;
         padding-left: <<padding-left>>;
         padding-right: <<padding-right>>;
    }
    .cbe-vl-table td:nth-child(3) {
         text-align: left;
         padding-left: <<padding-left>>;
         padding-right: <<padding-right>>;
    }
    .cbe-vl-table td:nth-child(4) {
         text-align: right;
         padding-left: <<padding-left>>;
         padding-right: <<padding-right>>;
    }
    .cbe-vl-table td:nth-child(5),
    .cbe-vl-table td:nth-child(8),
    .cbe-vl-table td:nth-child(11),
    .cbe-vl-table td:nth-child(14),
    .cbe-vl-table td:nth-child(17),
    .cbe-d-table td:nth-child(2),
    .cbe-d-table td:nth-child(5){
         text-align: right;
         padding-left: <<padding-left>>;
         padding-right: 0px;
    }
    .cbe-vl-table td:nth-child(6),
    .cbe-vl-table td:nth-child(9),
    .cbe-vl-table td:nth-child(12),
    .cbe-vl-table td:nth-child(15),
    .cbe-vl-table td:nth-child(18),
    .cbe-d-table td:nth-child(3),
    .cbe-d-table td:nth-child(6) {
         text-align: center;
         padding-left: 0px;
         padding-right: 0px;
         width: 1px;
    }
    .cbe-vl-table td:nth-child(7),
    .cbe-vl-table td:nth-child(10),
    .cbe-vl-table td:nth-child(13),
    .cbe-vl-table td:nth-child(16),
    .cbe-vl-table td:nth-child(19),
    .cbe-d-table td:nth-child(4),
    .cbe-d-table td:nth-child(7) {
         text-align: left;
         padding-right: <<padding-left>>;
         padding-left: 0px;
    }
    .cbe-d-table td:nth-child(1){
         text-align: left;
         padding-left: <<padding-left>>;
         padding-right: <<padding-right>>;
    }
"
  toprule <- paste0(toprule,"px")
  midrule <- paste0(midrule,"px")
  style_content <- fillin(cb_format_tmpl,
                          c(codebook_format_stdstyle,
                            indent=indent,toprule=toprule,midrule=midrule))
  style_element <- html("style",style_content,linebreak=TRUE)
  res <- html_group(style_element,res)
  res <- as.character(res)
  return(res)
}

show_html <- function(x,output=NULL,...){
  
  ht <- format_html(x,...)
  ht <- as.character(ht)
  
  if(interactive()){
    # Test whether running under RStudio 
    isRStudio <- Sys.getenv("RSTUDIO") == "1"
    if(isRStudio)
      deflt.output <- "file-show"
    else
      deflt.output <- "browser"
  }
  else
    deflt.output <- "stdout"
  
  if(missing(output))
    output <- getOption("html_viewer",deflt.output)
  
  if(mode(output)=="character")
      output <- match.arg(output,c("stdout","browser","file-show"))
  else if(!is.function(output))
      stop("'output' should be either a character string of a function")
  
  if(is.function(output)){
    
    tf <- tempfile()
    tf <- paste0(tf,".html")
    ht <- paste0("<!DOCTYPE html>\n",ht)  
    cat(ht,file=tf)
    
    output(tf)
  }
  else if(nzchar(Sys.getenv("JPY_PARENT_PID"))){
      ## Inside Jupyter 
      return(html_div(ht))
  }     
  else if(output=="stdout") cat(ht)
  else {
    
    tf <- tempfile()
    tf <- paste0(tf,".html")
    ht <- paste0("<!DOCTYPE html>\n",ht)  
    cat(ht,file=tf)
    
    if(output=="file-show")
      file.show(tf,title=deparse(substitute(x)))
    else 
      browseURL(tf)
  }
  
}

write_html <- function(x,file,...,standalone=TRUE){
    ht <- format_html(x,...)
    ht <- as.character(ht)
    if(standalone)
        ht <- paste0("<!DOCTYPE html>\n",ht)  
    cat(ht,file=file)
}

format_html <- function(x,...)
  UseMethod("format_html")



spltDec <- function(x,at="."){
  y <- strsplit(x,at,fixed=TRUE)
  y1 <- sapply(y,"[",1)
  y3 <- sapply(y,"[",2)
  y1[is.na(y1)] <- ""
  y3[is.na(y3)] <- ""
  y2 <- ifelse(grepl("[[:digit:]]+",y3),at,"")
  y <- rbind(y1,y2,y3)
  as.vector(y)
}



upd_vect <- function(x,...){
  y <- c(...)
  n <- names(y)
  x[n] <- y
  x
}

cbind.ftable <- function(..., deparse.level=1){
  
  args <- list(...)
  argnames <- names(args)
  rv <- lapply(args,attr,"row.vars")
  rv.ok <- TRUE
  narg <- length(args)
  if(length(args)>1){
    for(i in 1:narg){
      if(!length(rv[[i]])){
        if(i==1) stop("first argument must by an ftable")
        if(is.matrix(args[[i]])){
          if(nrow(args[[i]])!=nrow(args[[1]])) stop("row vars must be identical")
          tmp <- args[[i]]
        }
        else{
          tmp <- matrix(NA,nrow=nrow(args[[1]]),ncol=1)
          tmp[] <- as.numeric(args[[i]])
          colnames(tmp) <- if(length(argnames)&&length(argnames[[i]]))argnames[[i]] else ""
          names(dimnames(tmp))[2] <- ""
        }
        attr(tmp,"row.vars") <- rv[[1]]
        attr(tmp,"col.vars") <- dimnames(tmp)[2]
        dimnames(tmp) <- NULL
        class(tmp) <- "ftable"
        args[[i]] <- tmp
      }
      else if(!identical(rv[[1]],rv[[i]])) {
        rv.ok <- FALSE
        break
      }
    }
  }

  if(!(rv.ok)) stop("row vars must be identical")

  row.vars <- list(attr(args[[1]],"row.vars"))
  col.vars <- lapply(args,attr,"col.vars")
  names(row.vars) <- NULL
  names(col.vars) <- NULL
  args <- lapply(args,function(arg)array(arg,dim=dim(arg)))
  res <- matrix(args,nrow=1,ncol=length(args))
  structure(res,
    row.vars=row.vars,
    col.vars=col.vars,
    class="ftable_matrix")
}

rbind.ftable <- function(..., deparse.level=1){

  args <- list(...)
  argnames <- names(args)
  cv <- lapply(args,attr,"col.vars")
  cv.ok <- TRUE
  narg <- length(args)
  if(length(args)>1){
    for(i in 1:narg){
      if(!length(cv[[i]])){
        if(i==1) stop("first argument must by an ftable")
        if(is.matrix(args[[i]])){
          if(ncol(args[[i]])!=ncol(args[[1]])) stop("col vars must be identical")
          tmp <- args[[i]]
        }
        else{
          tmp <- matrix(NA,ncol=ncol(args[[1]]),nrow=1)
          tmp[] <- as.numeric(args[[i]])
          rownames(tmp) <- if(length(argnames)&&length(argnames[[i]]))argnames[[i]] else ""
          names(dimnames(tmp))[1] <- ""
#           browser()
        }
        attr(tmp,"row.vars") <- dimnames(tmp)[1]
        attr(tmp,"col.vars") <- cv[[1]]
        dimnames(tmp) <- NULL
        class(tmp) <- "ftable"
        args[[i]] <- tmp
      }
      else if(!identical(cv[[1]],cv[[i]])) {
        cv.ok <- FALSE
        break
      }
    }
  }

  if(!(cv.ok)) stop("column vars must be identical")

  col.vars <- list(attr(args[[1]],"col.vars"))
  row.vars <- lapply(args,attr,"row.vars")
  names(row.vars) <- NULL
  names(col.vars) <- NULL
  args <- lapply(args,function(arg)array(arg,dim=dim(arg)))
  res <- matrix(args,ncol=1,nrow=length(args))
  structure(res,
    row.vars=row.vars,
    col.vars=col.vars,
    class="ftable_matrix")
}

cbind.ftable_matrix <- function(..., deparse.level=1){

  args <- list(...)
  inh.fm <- sapply(args,inherits,"ftable_matrix")
  classes.ok <- all(inh.fm)
  if(!classes.ok) stop("all arguments must be 'ftable' or 'ftable_matrix' objects")

  rv <- lapply(args,attr,"row.vars")
  rv.ok <- TRUE
  narg <- length(args)
  if(length(args)>1){
    for(i in 1:narg){
      if(!identical(rv[[1]],rv[[i]])) {
        rv.ok <- FALSE
        break
      }
    }
  }

  if(!(rv.ok)) stop("row vars must be identical")

  row.vars <- rv[[1]]
  col.vars <- do.call(c,lapply(args,attr,"col.vars"))
  res <- lapply(args,function(x){
                attributes(x)[c("col.vars","row.vars","class")] <- NULL
                x
              })
  res <- do.call(cbind,res)
  structure(res,
    row.vars=row.vars,
    col.vars=col.vars,
    class="ftable_matrix")
}

rbind.ftable_matrix <- function(..., deparse.level=1){

  args <- list(...)
  inh.fm <- sapply(args,inherits,"ftable_matrix")
  classes.ok <- all(inh.fm)
  if(!classes.ok) stop("all arguments must be 'ftable' or 'ftable_matrix' objects")

  cv <- lapply(args,attr,"col.vars")
  cv.ok <- TRUE
  narg <- length(args)
  if(length(args)>1){
    for(i in 1:narg){
      if(!identical(cv[[1]],cv[[i]])) {
        cv.ok <- FALSE
        break
      }
    }
  }

  if(!(cv.ok)) stop("column vars must be identical")

  row.vars <- do.call(c,lapply(args,attr,"row.vars"))
  col.vars <- cv[[1]]
  res <- lapply(args,function(x){
                attributes(x)[c("col.vars","row.vars","class")] <- NULL
                x
              })
  res <- do.call(rbind,res)
  structure(res,
    row.vars=row.vars,
    col.vars=col.vars,
    class="ftable_matrix")
}

fm_mkhdr <- function(cv,quote){

  nms <- names(cv)
  l <- sapply(cv,length)
  prod.l <- cumprod(l)
  m <- length(cv)
  n <- prod.l[length(prod.l)]
  
  header <- matrix("",nrow=m,ncol=n)

  repn <- c(1,prod.l[-length(prod.l)])
  for(i in 1:m){

    ll <- prod.l[i]
    ii <- seq(from=1,length=ll,by=n%/%ll)
    header[i,ii] <- if(quote) paste0("\"",cv[[i]],"\"") else cv[[i]]
  }
  
  cbind(format(nms,justify="l"),header,"")
}

fm_mkldr <- function(rv,quote){

  nms <- names(rv)
  l <- sapply(rv,length)
  prod.l <- cumprod(l)
  m <- length(rv)
  n <- prod.l[length(prod.l)]

  header <- matrix("",nrow=n,ncol=m)

  repn <- c(1,prod.l[-length(prod.l)])
  for(i in 1:m){

    ll <- prod.l[i]
    ii <- seq(from=1,length=ll,by=n%/%ll)
    header[ii,i] <- if(quote) paste0("\"",rv[[i]],"\"") else rv[[i]]
  }

  structure(rbind(nms,header,""),dimnames=NULL)
}

to.nrow <- function(x,nrow,from=c("top","bottom")){

  from <- match.arg(from)
  res <- matrix(vector(mode=mode(x),length=length(x)),nrow=nrow,ncol=ncol(x))

  if(from=="top") res[1:nrow(x),] <- x
  else res[1:nrow(x) + nrow-nrow(x),] <- x
  res
}

to.ncol <- function(x,ncol,from=c("left","right")){

  from <- match.arg(from)
  res <- matrix(vector(mode=mode(x),length=length(x)),nrow=nrow(x),ncol=ncol)
  if(from=="left") res[,1:ncol(x)] <- x
  else res[,1:ncol(x) + ncol-ncol(x)] <- x
  res
}


format.ftable_matrix <- function(x,quote=TRUE,digits=0,format="f",...){

  d <- digits
  digits <- integer(ncol(x))
  digits[] <- d

  f <- format
  format <- character(ncol(x))
  format[] <- f

  fmt <- array(list(),dim=dim(x))
  for(i in 1:nrow(x)){
    for(j in 1:ncol(x)){

      tmp <- formatC(x[[i,j]],digits=digits[j],format=format[j])
      tmp <- cbind("",tmp,"")
      tmp <- rbind("",tmp,"")
      fmt[[i,j]] <- tmp
    }
  }

  col.vars <- attr(x,"col.vars")
  row.vars <- attr(x,"row.vars")

  header <- lapply(col.vars,fm_mkhdr,quote=quote)
  max.hh <- max(sapply(header,nrow))

  header <- lapply(header,to.nrow,nrow=max.hh,from="bottom")
  header <- do.call(cbind,header)

  leader <- lapply(row.vars,fm_mkldr,quote=quote)
  max.lh <- max(sapply(leader,ncol))
  leader <- lapply(leader,to.ncol,ncol=max.lh,from="right")
  leader <- do.call(rbind,leader)

  leader <- rbind(matrix("",nrow=nrow(header),ncol=ncol(leader)),leader)
  leader <- apply(leader,2,format.default,justify="l")
  
  body <- list()
  for(i in 1:nrow(fmt))
    body[[i]] <- do.call(cbind,fmt[i,])
  body <- do.call(rbind,body)

  #body <- apply(body,2,format.default,justify="r")
  res <- rbind(header,body)
  res <- apply(res,2,format.default,justify="r")
  res <- cbind(leader,res)
  res
}

Write.ftable_matrix <- function(x,
                            file = "",
                            quote = TRUE,
                            append = FALSE,
                            digits = 0,
                            ...){
  r <- format.ftable_matrix(x,quote=quote,digits=digits)
  r <- apply(r,1,paste,collapse=" ")
  cat(r, file = file, append = append, sep = "\n")
  invisible(x)
}

print.ftable_matrix <- function(x,quote=FALSE,...)
  Write.ftable_matrix(x,file="",quote=quote,...)

unlist1 <- function(x)unlist(as.list(x[-1]))

get_vnames_or_eval <- function(x,envir=NULL,enclos=parent.frame()){
    if(inherits(x,"call") && deparse(x[[1]]) == "c"){
        return(sapply(x[-1],as.character))
    }
    else {
        res <- try(eval(x,envir=envir,enclos=enclos),silent=TRUE)
        if(inherits(res,"try-error"))
            return(NULL)
        else return(res)
    }
}

get_vnames <- function(x,envir=NULL,enclos=parent.frame()){
    if(length(x) == 1) return(as.character(x))
    else if(inherits(x,"call") && deparse(x[[1]]) == "c"){
        return(sapply(x[-1],as.character))
    }
    else {
        return(NULL)
    }
}


all_in_names <- function(x,names){
    all(!nzchar(x) | x %in% names)
}


Reshape <- function(data,...,id,within_id,drop,keep,direction){

    if(inherits(data,"data.set")){
        cdpl <- codeplan(data)
        reshape_attr <- attributes(data)[c("reshapeLong","reshapeWide")]
        codeplan(data) <- NULL
        attributes(data)[c("reshapeLong","reshapeWide")] <- reshape_attr
    }
    else cdpl <- NULL

    mycall <- match.call(expand.dots=FALSE)

    spec <- mycall$...
    m <- length(spec)

    if(m > 1 && missing(within_id))
        vspec <- spec[-m]
    else
        vspec <- spec
    if(length(vspec)==1 &&
       deparse(vspec[[1]][[1]]) == "list"){
       vspec <- as.list(vspec[[1]][-1]) 
    }
    vspec <- lapply(vspec,get_vnames_or_eval,envir=data,enclos=parent.frame())
    v.names <- names(vspec)
    varying <- vspec
    varying <- lapply(varying,function(x){
        x[!nzchar(x)] <- "__tmp__na__"
        return(x)
    })

    if(!missing(drop))
        drop <- intersect(get_vnames(mycall$drop,envir=data),
                          names(data))
    else
        drop <- NULL
    if(!missing(within_id)){
            within_id <- intersect(get_vnames(mycall$within_id,envir=data),
                                   names(data))
            timevar <- within_id
            times <- unique(data[[timevar]])
    }
    else if(m > 1){
        times <- eval(spec[[m]],envir=data,enclos=parent.frame())
        timevar <- names(spec)[m]
    }
    else {
        n <- length(varying[[1]])
        times <- 1:n
        timevar <- "times"
    }

    if(direction=="long"){
        data[["__tmp__na__"]] <- NA
        cls <- class(data)
        class(data) <- "data.frame"
        res <- reshape(data,
                       varying=varying,
                       v.names=v.names,
                       times=times,
                       timevar=timevar,
                       drop=drop,
                       direction=direction)
        res["__tmp__na__"] <- NULL
        if(is.character(times))
            res[[timevar]] <- factor(res[[timevar]],
                                      levels=times)
        idvar <- "id"
        ii <- order(res[[idvar]],res[[timevar]])
        res <- res[ii,]
        if(!missing(keep)){
            keep <- intersect(get_vnames(mycall$keep,envir=data),names(data))
            keep <- union(keep,c(v.names,timevar))
            keep <- intersect(keep,names(res))
            res <- res[keep]
        }
        class(res) <- cls

    } else {
        rlattr <- attr(data,"reshapeLong")
        drop_id <- FALSE
        if(missing(id)){
            if(length(rlattr) && length(rlattr$idvar))
                idvar <- rlattr$idvar
            else {
                if("id" %in% names(data)) idvar <- "id"
                else stop("missing id variable, please provide one with idvar=...")
            }
            idvars <- idvar
        }
        else {
            idvars <- mycall$id
            idvars <- get_vnames(idvars,envir=data)
            if(length(idvars)>1){
                ids <- data[,idvars,drop=FALSE]
                data[["id"]] <- as.integer(do.call(interaction,ids))
                idvar <- "id"
                drop_id <- TRUE
            }
            else idvar <- idvars
        }
        if(timevar %nin% names(data)){
            stop("missing within_id variable, please provide one.")
        }
        cls <- class(data)
        class(data) <- "data.frame"
        res <- reshape(as.data.frame(data),
                       varying=varying,
                       v.names=v.names,
                       times=times,
                       timevar=timevar,
                       idvar=idvar,
                       drop=drop,
                       direction=direction)
        varying_ <- unname(unlist(varying))
        nonvar_ <- setdiff(names(res),varying_) 
        ii <- c(nonvar_,varying_)
        attr_reshape <- attr(res,"reshapeWide")
        res <- res[ii]
        if(!missing(keep)){
            keep <- intersect(get_vnames(mycall$keep,envir=data),names(data))
            keep <- union(keep,c(unlist(varying),times))
            keep <- intersect(keep,names(res))
            res <- res[keep]
        }
        if("__tmp__na__" %in% names(res))
            res[["__tmp__na__"]] <- NULL
        else
            attr(res,"reshapeWide") <- attr_reshape
        if(drop_id)
            res[[idvar]] <- NULL
        class(res) <- cls
    }
    
    if(length(cdpl)){
        reshape_attr <- attributes(res)[c("reshapeLong","reshapeWide")]
        codeplan(res) <- cdpl
        attributes(res)[c("reshapeLong","reshapeWide")] <- reshape_attr
    }

    res
}



    # if(direction=="wide"){
    #     reshapeAttr <- attr(res,"reshapeLong")
    #     varying <- unlist(varying)
    #     othervars <- !(names(res) %in% varying)
    #     othervars <- names(res)[othervars]
    #     varying <- varying[varying %in% names(res)]
    #     res <- res[c(othervars,varying)]
    #     rownames(res) <- NULL
    # }

## levels of measurement ################################################################

setMethod("measurement","ANY",function(x)NULL)
setMethod("measurement","item",function(x){
  if(length(x@measurement)) return(x@measurement)
  else if(length(x@value.labels) || is.character(x)) return("nominal")
  else return("interval")
})

setReplaceMethod("measurement","item",function(x,value){
  if(length(value)) value <- as.measurement.level(value)
  x@measurement <- value
  invisible(x)
})

setMethod("as.measurement.level","character",function(x=c("interval","nominal","ordinal","ratio")) match.arg(x))
setMethod("as.measurement.level","NULL",function(x) "interval")

is.nominal <- function(x) measurement(x) == "nominal"
is.ordinal <- function(x) measurement(x) == "ordinal"
is.interval <- function(x) measurement(x) == "interval"
is.ratio <- function(x) measurement(x) == "ratio"

setReplaceMethod("measurement","data.set",function(x,value){
    mslevels <- names(value)
    for(mlv in mslevels){
        mlv <- as.measurement.level(mlv)
        vars <- value[[mlv]]
        for(var in vars){
            measurement(x[[var]]) <- mlv
        }
    }
    invisible(x)
})

set_measurement <- function(x,...){
    mycall <- match.call(expand.dots=FALSE)
    lst <- mycall$...
    mslevels <- names(lst)
    for(mlv in mslevels){
        mlv <- as.measurement.level(mlv)
        vars <- lst[[mlv]]
        if(inherits(vars,"call")){
            vars <- sapply(vars[-1],as.character)
        }
        else
            vars <- as.character(vars)
        for(var in vars){
            measurement(x[[var]]) <- mlv
        }
    }
    return(x)
}

as.nominal <- function(x){
    measurement(x) <- "nominal"
    invisible(x)
}

as.ordinal <- function(x){
    measurement(x) <- "ordinal"
    invisible(x)
}

as.interval <- function(x){
    measurement(x) <- "interval"
    invisible(x)
}

as.ratio <- function(x){
    measurement(x) <- "ratio"
    invisible(x)
}

Within <- function(data, expr, ...){
    if(inherits(data,"data.frame"))
        UseMethod("Within")
    else UseMethod("within")
}

Within.data.frame <- function (data, expr, ...) 
{
    parent <- parent.frame()
    e <- evalq(environment(), data, parent)
    eval(substitute(expr), e)
    l <- rev(as.list(e, all.names = TRUE))
    l <- l[!vapply(l, is.null, NA, USE.NAMES = FALSE)]
    nl <- names(l)
    del <- setdiff(names(data), nl)
    data[nl] <- l
    data[del] <- NULL
    data
}

"%$$%" <- function(data,expr){
    # parf <- parent.frame()
    # nm <- deparse(x)
    # res <- within(x,expr)
    # res
    res <- if(inherits(data,"data.frame"))
               eval.parent(substitute(Within(data,expr)))
           else
               eval.parent(substitute(within(data,expr))) 
    nm <- deparse(substitute(data))
    assign(nm,res,envir=parent.frame())
}

"%$%" <- function(data,expr){
    eval.parent(substitute(with(data,expr)))
}

toLatex.data.frame <- function(object,
                           digits=getOption("digits"),
                           format="f",
                           useDcolumn=getOption("useDcolumn",TRUE),
                           numeric.colspec=if(useDcolumn)
                                               paste("D{.}{",LaTeXdec,"}{",ddigits,"}",sep="")
                                           else "r",
                           factor.colspec="l",
                           LaTeXdec=".",
                           ddigits=digits,
                           useBooktabs=getOption("useBooktabs",TRUE),
                           toprule=if(useBooktabs) "\\toprule" else "\\hline\\hline",
                           midrule=if(useBooktabs) "\\midrule" else "\\hline",
                           cmidrule=if(useBooktabs) "\\cmidrule" else "\\cline",
                           bottomrule=if(useBooktabs) "\\bottomrule" else "\\hline\\hline",
                           row.names=is.character(attr(object,"row.names")),
                           NAas="",
                           toLatex.escape.tex=getOption("toLatex.escape.tex",FALSE),
                           ...){
  n <- nrow(object)
  m <- ncol(object)
  d <- digits
  is.num <- sapply(object,is.numeric)
  is.mat <- sapply(object,is.matrix)
  m.num <- sum(is.num)
  digits <- integer(m.num)
  digits[] <- d
  fdigits <- integer(m)
  fdigits[is.num] <- digits
  fo <- format
  format <- character(m)
  format[is.num] <- fo

  body <- list()
  for(i in 1:m) {
      object.i <- object[,i]
      is.na.object.i <- is.na(object.i)
      if(is.numeric(object.i))
          body.i <- formatC(object.i,digits=fdigits[i],format=format[i])
      else
        body.i <- as.character(object.i)
      body.i[is.na.object.i] <- NAas
      body[[i]] <- format(body.i,justify="right")
  }
  body <- do.call(cbind,body)
  ans <- gsub("([eE])([-+]?[0-9]+)","\\\\textrm{\\1}\\2",body)
  
  if(row.names){
    ans <- cbind(format(rownames(object),justify="right"),ans)
  }

  colspan <- sapply(object,NCOL)
  header <- paste0("\\multicolumn{",colspan,"}{c}{",colnames(object),"}")
  
  header <- paste(header,collapse=" & ")
  if(row.names) header <- paste("&",header)
  header <- paste(header,"\\\\")

  ans <- apply(ans,1,paste,collapse=" & ")
  ans <- paste(ans,"\\\\")
  ans <- c(
      toprule,
      header,
      midrule,
      ans,
      bottomrule
  )
  
  if(toLatex.escape.tex)
      ans <- LaTeXcape(ans)

  body.spec <- rep(factor.colspec,m)
  dd <- integer(m.num)
  dd[] <- ddigits
  ddigits <- dd
  body.spec[is.num] <- numeric.colspec
  body.spec <- rep(body.spec,colspan)

  if(row.names)
    tabspec <- c("l",body.spec)
  else
    tabspec <- body.spec
  tabspec <- paste(tabspec,collapse="")
  tabbegin <- paste("\\begin{tabular}{",tabspec,"}",sep="")
  tabend <- "\\end{tabular}"
  ans <- c(tabbegin,ans,tabend)
  structure(ans,class="Latex")
}

toLatex.data.set <- function(object,...){
  frame <- structure(object@.Data,row.names=object@row_names,names=object@names,class="data.frame")
  for(i in 1:ncol(frame))
    frame[[i]] <- format(frame[[i]])
  toLatex(frame,...)
}

By <- function(formula,expr,data=parent.frame()) UseMethod("By",data)

By.default <- function(formula,expr,data=parent.frame()){
  m <- match.call()
  parent <- parent.frame()
  expr <- m$expr
  formula <- m$formula
  if(is.environment(data)){
    allVars <- union(all.vars(formula),all.vars(expr))
    data <- lapply(allVars,function(v)get(v,envir=data))
    names(data) <- allVars
  }
  data <- as.data.frame(data)
  fun <- function(x) eval(expr,data[x,],parent)
  factors <- all.vars(formula)
  factors <- data[factors]
  nd <- nrow(data)
  ans <- eval(substitute(tapply(1:nd, factors, fun)), data)
  ans.is.nonnull <- !sapply(ans,is.null)
  if(!all(ans.is.nonnull)){
    ans <- if(is.matrix(ans)) ans[,ans.is.nonnull]
           else ans[ans.is.nonnull]
  }
  attr(ans, "call") <- match.call()
  class(ans) <- "by"
  ans
}

By.data.frame <- function(formula,expr,data){
  m <- match.call()
  parent <- parent.frame()
  expr <- m$expr
  formula <- m$formula
  if(is.environment(data)){
    allVars <- union(all.vars(formula),all.vars(expr))
    data <- lapply(allVars,function(v)get(v,envir=data))
    names(data) <- allVars
  }
  fun <- function(x) {
    xdata <- data[x,,drop=FALSE]
    res <- eval(expr,xdata,parent)
    if(is.list(res) && length(res[["data"]]) && is.environment(res[["data"]]))
      res[["data"]] <- xdata
    res
    }
  factors <- all.vars(formula)
  factors <- data[factors]
  nd <- nrow(data)
  ans <- eval(substitute(tapply(1:nd, factors, fun)), data)
  ans.is.nonnull <- !sapply(ans,is.null)
  if(!all(ans.is.nonnull)){
    ans <- if(is.matrix(ans)) ans[,ans.is.nonnull]
           else ans[ans.is.nonnull]
  }
  attr(ans, "call") <- match.call()
  class(ans) <- "by"
  ans
}

rabind2 <- function(x,y){
    if(!length(x)) return(y)
    else if(!length(y)) return(x)
    else {
        rnx <- rownames(x)
        rny <- rownames(y)
        nrx <- nrow(x)
        nry <- nrow(y)
        dimx <- dim(x)[-1]
        dimy <- dim(y)[-1]
        stopifnot(all(dimx==dimy))
        dimnx <- dimnames(x)[-1]
        dimny <- dimnames(y)[-1]
        x <- array(x,dim=c(nrx,prod(dimx)))
        y <- array(y,dim=c(nry,prod(dimy)))
        z <- rbind(x,y)
        dim(z) <- c(nrx+nry,dimx)
        dimnames(z) <- c(list(c(rnx,rny)),
                         dimnx)
        return(z)
    }
}

spss.readheader <- function(f)
 .Call("read_sysfile_header",f)

spss.readvars <- function(f,n){
  ans <- list()

  if(n > 0){
      for(i in 1:n){
          currvar <- .Call("read_sysfile_var",f)
          ans <- c(ans,list(currvar))
      }
  }
  else {
      repeat {
          currvar <- .Call("read_sysfile_var",f)
          if(!is.null(currvar))
              ans <- c(ans,list(currvar))
          else break
      }
  }
  names(ans) <- sapply(ans,function(x)x$name)
  ans
}

spss.testcode <- function(f)
  .Call("test_sysfile_int32",f)

spss.readlabels <- function(f){
  ans <- list()
  while(spss.testcode(f)==3){
    ans <- c(ans,list(.Call("read_sysfile_value_labels",f)))
  }
  ans
}

spss.readdocument <- function(f)
  .Call("read_sysfile_document",f)

spss.readaux <- function(f)
  .Call("read_sysfile_aux",f)

spss.dictterm <- function(f)
  .Call("read_sysfile_dict_term",f)

parseSysHeader <- function(file){
  header <- spss.readheader(file)
  swapcode <- header$swap_code
  nvars <- header$case_size
  attr(file,"swap_code") <- swapcode
  attr(file,"case_size") <- nvars
  attr(file,"bias") <- header$bias
  attr(file,"compressed") <- header$compressed
  variables <- spss.readvars(file,nvars)
  if(nvars == 0){
      warning("File header does not contain information on the number of variables.
  This is a bug in the software (ReadStat/haven?) with which the data file was created.
  Guessing that number.",
              call.=FALSE,immediate.=TRUE)
      nvars <- length(variables)
      attr(file,"case_size") <- nvars
  }
  
  value.labels <- NULL
  document <- NULL
  if(spss.testcode(file)==3)
    value.labels <- spss.readlabels(file)
  if(spss.testcode(file)==6)
    document <- spss.readdocument(file)
    
  auxiliaries <- list()
  while(spss.testcode(file)==7){
    aux <- spss.readaux(file)
    auxtype <- aux$type
    aux <- list(aux$data)
    names(aux) <- auxtype
    auxiliaries <- c(auxiliaries,aux)
  }
  if(length(auxiliaries$info_flt64)){

    sysmis  <- auxiliaries$info_flt64["sysmis"]
    highest <- auxiliaries$info_flt64["highest"]
    lowest  <- auxiliaries$info_flt64["lowest"]
  }
  else{

    warning("file lacks info_flt64 record, using defaults")

    info_flt64 <- .Call("dflt_info_flt64",file)
    sysmis  <- info_flt64["sysmis"]
    highest <- info_flt64["highest"]
    lowest  <- info_flt64["lowest"]
  }
  attr(file,"sysmis") <- sysmis
  attr(file,"highest") <- highest
  attr(file,"lowest") <- lowest
  if(spss.testcode(file)==999) start.data <- spss.dictterm(file)
  else stop("did not find dictionary termination code")
  #message("\nstart of data:",p$start.data)
  attr(file,"sysmis") <- sysmis
  attr(file,"data_pos") <- start.data

  if(length(auxiliaries$longVariableNames)){
    longVariableNames <- auxiliaries$longVariableNames
    longVariableNames <- strsplit(longVariableNames,"\t")[[1]]
    longVariableNames <- strsplit(longVariableNames,"=")
    longVariableNames <- sapply(longVariableNames,function(lvn)
        structure(lvn[2],names=lvn[1]))
    ii <- match(names(longVariableNames),names(variables))
    names(variables)[ii] <- unname(longVariableNames)
  }
  
  variables <- lapply(variables,function(x)x[-1])
  
  missings <- list()
  
  missings$values <- lapply(variables,"[[","missings")
  missings$ranges <- vector(length(variables),mode="list")
  names(missings$ranges) <- names(missings$values)
  n_missings <- sapply(variables,"[[","n_missing_values")
  
  mrang <- missings$values[n_missings == -2]
  if(length(mrang)){
    mrang <- do.call(rbind,mrang)
    is.lo <- mrang[,1] == lowest
    is.hi <- mrang[,2] == highest
    if(length(is.lo))
      mrang[is.lo,1] <- -Inf
    if(length(is.hi))
      mrang[is.hi,2] <- Inf
    missings$ranges[n_missings == -2] <- split(mrang,row(mrang))
  }
  mrang_val <- missings$values[n_missings == -3]
  if(length(mrang_val)){
    mrang_val <- do.call(rbind,mrang_val)
    
    is.lo <- mrang_val[,1] == lowest
    is.hi <- mrang_val[,2] == highest
    if(length(is.lo))
      mrang_val[is.lo,1] <- -Inf
    if(length(is.hi))
      mrang_val[is.hi,2] <- Inf
    missings$values[n_missings == -3] <- split(unname(mrang_val[,3,drop=FALSE]),seq_len(nrow(mrang_val)))
    missings$ranges[n_missings == -3] <- split(mrang_val[,1:2,drop=FALSE],row(mrang_val[,1:2,drop=FALSE]))
  }
  missings$values[n_missings == -2] <- list(NULL)
  
  types <- sapply(variables,function(x)x$type)
  digits <- sapply(variables,function(x)x$print[4])
  list(
    header = header,
    auxiliaries = auxiliaries,
    variables = variables[types>=0],
    value.labels = value.labels,
    missings = missings,
    types = types,
    document = document
    )
}

# seekSysData <- function(p)
#   .Call("rewind_sysfile",p$file)

# spss_numeric_to_POSIXct <- function(x){
#     x <- as.numeric(x)
#     as.POSIXct(x, origin = "1582-10-14", tz = "GMT")
# }

spss.string.vallabs <- function(vallab)
    structure(
        trimws(.Call("num_to_string8",vallab)),
        names=names(vallab)
    )

insert_rows <- function(x,i,val){
    l <- length(i)
    n <- nrow(x)
    y <- matrix(nrow=n+l,ncol=ncol(x))
    ii1 <- i + 1:l - 1
    ii2 <- seq.int(n+l)
    ii2 <- ii2[!(ii2%in%ii1)]
    y[ii2,] <- x
    y[ii1,] <- val
    return(y)
}

insert <- function(x,i,val){
    l <- length(i)
    n <- length(x)
    y <- vector(length=n+l,mode=mode(x))
    ii1 <- i + 1:l - 1
    ii2 <- seq.int(n+l)
    ii2 <- ii2[!(ii2%in%ii1)]
    y[ii2] <- x
    y[ii1] <- val
    return(y)
}


toLatex.ftable <- function(object,
          show.titles=TRUE,
          digits=if(is.integer(object)) 0 else getOption("digits"),
          format=if(is.integer(object)) "d" else "f",
          useDcolumn=getOption("useDcolumn",TRUE),
          colspec=if(useDcolumn) paste("D{.}{",LaTeXdec,"}{",ddigits,"}",sep="") else "r",
          LaTeXdec=".",
          ddigits=digits,
          useBooktabs=getOption("useBooktabs",TRUE),
          toprule=if(useBooktabs) "\\toprule" else "\\hline\\hline",
          midrule=if(useBooktabs) "\\midrule" else "\\hline\n",
          cmidrule=if(useBooktabs) "\\cmidrule" else "\\cline",
          bottomrule=if(useBooktabs) "\\bottomrule" else "\\hline\\hline",
          extrarowsep = NULL,
          toLatex.escape.tex=getOption("toLatex.escape.tex",FALSE),
          fold.leaders=FALSE,
          ...){
  row.vars <- attr(object,"row.vars")
  col.vars <- attr(object,"col.vars")
  n.row.vars <- length(row.vars)
  n.col.vars <- length(col.vars)
  n <- nrow(object)
  m <- ncol(object)
  d <- digits
  digits <- integer(m)
  digits[] <- d
  #print(digits)
  fo <- format
  format <- integer(m)
  format[] <- fo
  body <- array("",dim=dim(object))
  for(i in seq(along=digits)) {
    #print(digits[i])
    body[,i] <- formatC(object[,i],digits=digits[i],format=format[i])
    }
  body <- sub("([eE])([-+]?[0-9]+)","\\\\textrm{\\1}\\2",body)
  #str(digits)
  #str(body)
  header <- character(n.col.vars)
  clines <- character(n.col.vars)
  ncols <- ncol(body)
  n.col.grps <- length(col.vars[[1]])
  header <- character(n.col.vars)
  clns <- character(n.col.vars)
  c.starts <- list()
  c.ends <- list()
  mcols <- integer(n.col.vars)
  ng <- integer(n.col.vars)
  mcols <- integer(n.col.vars)
  for(i in n.col.vars:1){
    if(i == n.col.vars){
      mcols[i] <- 1
      ng[i] <- ncols
    }
    else {
      mcols[i] <- length(col.vars[[i+1]])*mcols[i+1]
      ng[i] <- ncols/mcols[i]
    }
  }
  for(i in 1:n.col.vars){
    
    cv <- col.vars[[i]]
    lcv <- length(cv)
    tmp.header <- character(ng[i])
    tmp.header[] <- cv
    tmp.header <- paste("\\multicolumn{",mcols[i],"}{c}{",tmp.header,"}",sep="")
    if(i == n.col.vars && n.col.vars > 1){
      dim(tmp.header) <- c(mcols[i-1],ng[i-1])
      tmp.header <- apply(tmp.header,2,paste,collapse="&")
      header[i] <- paste(tmp.header,collapse="&&")
      }
    else if (n.col.vars > 1)
      header[i] <- paste(tmp.header,collapse="&&")
    else
      header[i] <- paste(tmp.header,collapse="&")
    if(length(cmidrule)){
      if(i == n.col.vars)
        c.starts[[i]] <- length(row.vars) + 2 + (seq(lcv) - 1)*mcols[i]
      else
        c.starts[[i]] <- length(row.vars) + 2 + (seq(lcv) - 1)*(mcols[i]+1)
      c.ends[[i]] <- c.starts[[i]] + (mcols[i]-1)
      tmp.cln <- paste(cmidrule,"{",c.starts[[i]],"-",c.ends[[i]],"}",sep="")
      clns[i] <- paste(tmp.cln,collapse="")
    }
  }
  for(i in 1:n.col.vars){
    if(i == n.col.vars)
      header[i] <- paste(header[i],collapse=" & ")
    else
      header[i] <- paste(header[i],collapse=" && ")
  }


  if(!fold.leaders){
      show.titles <- show.titles && length(names(row.vars)) && length(names(col.vars))
      if(show.titles){
          if(length(names(col.vars))==1){
              
              hleaders <- matrix("",nrow=2,ncol=n.row.vars)
              hleaders[2,] <- names(row.vars)
              hleaders <- apply(hleaders,1,paste,collapse="&")
              
              header <- c(paste("\\multicolumn{",m,"}{c}{",names(col.vars),"}",sep=""),
                          header)
          }
          else {
              hleaders <- matrix("",nrow=n.col.vars,ncol=n.row.vars+1)
              hleaders[,n.row.vars+1] <- paste0(names(col.vars),":")
              hleaders[n.col.vars,1:n.row.vars] <- names(row.vars)
              hleaders <- apply(hleaders,1,paste,collapse="&")
          }
      } else {
          hleaders <- matrix("",nrow=n.col.vars,ncol=n.row.vars)
          hleaders <- apply(hleaders,1,paste,collapse="&")
      }
      
      leaders <- matrix("",nrow=nrow(body),ncol=n.row.vars)
      lrv <- integer(n.row.vars)
      for(i in 1:n.row.vars){
          rv <- row.vars[[i]]
          if(i == 1)
              lrv[i] <- length(rv)
          else
              lrv[i] <- length(rv) * lrv[i-1]
          tmp.leaders <- matrix("",ncol=lrv[i],nrow=n/lrv[i])
          tmp.leaders[1,] <- rv
          leaders[,i] <- c(tmp.leaders)
      }
  }
  else{
      leaders <- matrix("",nrow=n,ncol=1)
      rown <- row.vars[[n.row.vars]]
      leaders[] <- rown
      if(n.row.vars > 1){
          mg <- length(rown)
          ng <- n/mg
          jj <- rev(seq(n.row.vars - 1))
          for(j in jj){
              i <- seq(by=mg,length=ng)
              body <- insert_rows(body,i,"")
              rn.j <- row.vars[[j]]
              leaders <- insert_rows(leaders,i,rn.j)
              mg <- mg + 1
          }
      }
      n <- nrow(body)
      hleaders <- matrix("",nrow=n.col.vars,ncol=1)
  }
  
  leaders <- format(leaders)
  leaders <- apply(leaders,1,paste,collapse="&")


  lcv <- length(col.vars[[n.col.vars]])
  dim(body) <- c(n,lcv,m/lcv)
  body <- format(body)
  body <- apply(body,c(1,3),paste,collapse=" & ")
  body <- apply(body,1,paste,collapse=" && ")

  if(show.titles && length(names(col.vars))>1)
    header <- paste(hleaders,header,sep=" & ")
  else
    header <- paste(hleaders,header,sep=" && ")
  
  header <- paste(header,"\\\\",sep="")
  if(length(cmidrule)){
    header <- c(rbind(header,clns))
    header <- header[-length(header)]
  }
  body <- paste(leaders,body,sep=" && ")
  
  if(!length(extrarowsep))
    body <- paste(body,"\\\\",sep="")
  else {
    rowsep <- rep("\\\\",NROW(body))
    .extrarowsep <- rep("",NROW(body))
    if(!fold.leaders){
        lrv <- length(row.vars[[n.row.vars]])
        ii <- seq(from=lrv,by=lrv,to=n-lrv)
    }
    else {
        ii <- seq(from=mg,by=mg,to=n-mg)
    }
    .extrarowsep[ii] <- paste("[",extrarowsep,"]",sep="")
    rowsep <- paste(rowsep,.extrarowsep,sep="")
    body <- paste(body,rowsep,sep="")
  }
 
  ans <- c(toprule,header,body,bottomrule)
  mr_pos <- length(toprule) + length(header) + 1
  if(fold.leaders){
      mr_pos <- seq(from=mr_pos,by=mg,length=ng)
  }
  ans <- insert(ans,mr_pos,midrule)
  if(toLatex.escape.tex)
    ans <- LaTeXcape(ans)
  
  leader.spec <- paste(rep("l",length(row.vars)+1),collapse="")
  body.spec <- character(m)
  body.spec[] <- colspec
  i <- rep(seq_len(m/lcv),each=lcv)
  body.spec <- split(body.spec,i)
  body.spec <- sapply(body.spec,paste,collapse="")
  body.spec <- paste(body.spec,collapse="c")
  tabspec <- paste(leader.spec,body.spec,sep="")
  
  tabbegin <- paste("\\begin{tabular}{",tabspec,"}",sep="")
  tabend <- "\\end{tabular}"
  ans <- c(tabbegin,ans,tabend)
  structure(ans,class="Latex")
}

trim_labels.item <- function(x,...){
    l <- labels(x)
    if(!length(l)) return(x)
    lData <- l@.Data
    l@.Data <- gsub("[-+]?[0-9]+[.] ","",lData)
    labels(x) <- l
    x
}

trim_labels.data.set <- function(x,...){
    for(i in seq_along(x)){
        x@.Data[[i]] <- trim_labels.item(x@.Data[[i]])
    }
    x
}

setGeneric("trim_labels",
           function(x,...)standardGeneric("trim_labels"))

setMethod("trim_labels","item.vector",trim_labels.item)
setMethod("trim_labels","data.set",trim_labels.data.set)

dta.byte   <- 251
dta.short  <- 252
dta.long   <- 253
dta.float  <- 254
dta.double <- 255

dta.byte.missrange   <- c(0x64,0x7f)
dta.short.missrange  <- c(0x7fe5,0x7fff)
dta.long.missrange   <- c(0x7fffffe5L,0x7fffffffL)
dta.float.missrange  <- c(2.0^0x7f,Inf)
dta.double.missrange <- c(2.0^0x3ff,Inf)

missnames <- paste(".",c("",letters),sep="")
dta.byte.misslab <- structure(0x64 + 1:27,names=missnames)
dta.short.misslab <- structure(0x7fe4 + 1:27,names=missnames)
dta.long.misslab <- structure(0x7fffffe4L + 1:27,names=missnames)
dta.float.misslab <- structure((1+(0:26)/16^3)*2.0^0x7f,names=missnames)
dta.double.misslab <- structure((1+(0:26)/16^3)*2.0^0x3ff,names=missnames)

new.dta <- function(file) .Call("dta_file_open",file,"rb")
dta.read.version <- function(bf) .Call("dta_read_version",bf)
dta.read.header <- function(bf,lablen).Call("dta_read_header",bf,lablen)
dta.read.descriptors <- function(bf,nvar,len.varn,len.fmt,len.lbl)
        .Call("dta_read_descriptors",bf,nvar,len.varn,len.fmt,len.lbl)
dta.read.varlabs <- function(bf,nvar,len.varlab)
        .Call("dta_read_varlabs",bf,nvar,len.varlab)
dta.read.expansion.fields <- function(bf,shortext)
        .Call("dta_read_expansion_fields",bf,shortext)
# dta.read.data <- function(bf,nobs,types)
#         .Call("dta_read_data",bf,nobs,types)        
dta.read.labels <- function(bf,lbllen,padding)
        .Call("dta_read_labels",bf,lbllen,padding)
# dta.read.chunk <- function(bf,vars,obs,types)
#         .Call("dta_read_chunk",bf,vars,obs,types)
dta.trans.types <- function(types)
        .Call("dta_trans_types",types)
dta.calc.obssize <- function(bf,types)
        .Call("dta_calc_obssize",bf,types)
dta.tell <- function(bf) .Call("dta_ftell",bf)
dta.seek <- function(bf,pos,whence) .Call("dta_fseek",bf,pos,whence)
dta.feof <- function(bf) .Call("dta_feof",bf)
dta.skip.records <- function(bf,n) .Call("dta_skip_records",bf,n)
dta.seek.data <- function(bf) .Call("dta_seek_data",bf)
get.dictionary.dta <- function(bf){
  version <- dta.read.version(bf)
  if(version==105){ ## Stata 5
    version.string <- "Stata 5"
    len.varn <- 8
    len.fmt <- 11
    len.lbl <- 8
    len.varlab <- 31
    ext.short <- TRUE
    conv.types <- FALSE
  }
  else if(version==108){ ## Stata 6
    version.string <- "Stata 6"
    len.varn <- 8
    len.fmt <- 11
    len.lbl <- 8
    len.varlab <- 80
    ext.short <- TRUE
    conv.types <- FALSE
  }
  else if(version==110){ ## Stata 7
    version.string <- "Stata 7"
    len.varn <- 32
    len.fmt <- 11
    len.lbl <- 32
    len.varlab <- 80
    ext.short <- FALSE
    conv.types <- TRUE
  }
  else if(version==111){ ## Stata 7 SE
    version.string <- "Stata 7 SE"
    len.varn <- 32
    len.fmt <- 11
    len.lbl <- 32
    len.varlab <- 80
    ext.short <- FALSE
    conv.types <- FALSE
  }
  else if(version==113){ ## Stata 8
    version.string <- "Stata 8"
    len.varn <- 32
    len.fmt <- 11
    len.lbl <- 32
    len.varlab <- 80
    ext.short <- FALSE
    conv.types <- FALSE
  }
  else if(version %in% c(114,115)){ ## Newer
    version.string <- if(version==114) "Stata 9" else "Stata 10"
    len.varn <- 32
    len.fmt <- 48
    len.lbl <- 32
    len.varlab <- 80
    ext.short <- FALSE
    conv.types <- FALSE
  } else {
    stop("version ",version," not yet supported")
  }
  hdr <- dta.read.header(bf,len.varlab)
  nobs <- hdr$nobs
  nvar <- hdr$nvar
  descriptors <- dta.read.descriptors(bf,
          nvar=nvar,
          len.varn=len.varn,
          len.fmt=len.fmt,
          len.lbl=len.lbl)
  varnames <- descriptors$varlist
  types <- if(conv.types) dta.trans.types(descriptors$typelist)
           else descriptors$typelist
  names(types) <- varnames
  obs_size <- dta.calc.obssize(bf,types)
  varlab <- dta.read.varlabs(bf,
          nvar=nvar,
          len.varlab=len.varlab)
  names(varlab) <- varnames
  dta.read.expansion.fields(bf,ext.short)
  vallabs <- descriptors$lbllist
  names(vallabs) <- varnames
  vallabs <- vallabs[nzchar(vallabs)]
  if(version>105){
    dta.skip.records(bf,nobs)
    vallab.patterns <- list()
    while(!dta.feof(bf))
      vallab.patterns <-  c(vallab.patterns,dta.read.labels(bf,len.lbl,3))
    if(!any(sapply(vallab.patterns,length)>0))
      vallab.patterns <- NULL
  }
  else
    vallab.patterns <- NULL
  if(length(vallab.patterns)){
    vallabs[] <- vallab.patterns[vallabs]
  }
  if(version >= 113){
      missing.values <- dta_missing.values(types)
      missval_labels <- dta_missval_labels(types)
  }
  else {
      missing.values <- NULL
      missval_labels <- NULL
  }
    
  list(names=varnames,
       types=types,
       nobs=nobs,
       nvar=nvar,
       varlabs=varlab,
       value.labels=vallabs,
       missing.values=missing.values,
       missval_labels=missval_labels,
       version.string=version.string
      )
}

dta_missing.values <- function(types){
    nvar <- length(length)
    missing.values <- vector(nvar,mode="list")
    missing.values[types==dta.byte]   <- list(list(range=dta.byte.missrange))
    missing.values[types==dta.short]  <- list(list(range=dta.short.missrange)) 
    missing.values[types==dta.long]   <- list(list(range=dta.long.missrange))
    missing.values[types==dta.float]  <- list(list(range=dta.float.missrange)) 
    missing.values[types==dta.double] <- list(list(range=dta.double.missrange))
    names(missing.values) <- names(types)
    missing.values
}

dta_missval_labels <- function(types){
    nvar <- length(length)
    missval_labels <- vector(nvar,mode="list")
    missval_labels[types==dta.byte]   <- list(dta.byte.misslab)
    missval_labels[types==dta.short]  <- list(dta.short.misslab) 
    missval_labels[types==dta.long]   <- list(dta.long.misslab)
    missval_labels[types==dta.float]  <- list(dta.float.misslab)
    missval_labels[types==dta.double] <- list(dta.double.misslab)
    names(missval_labels) <- names(types)
    missval_labels
}

centerAt <- function(x,at=getOption("OutDec"),integers=c("dot","right","left"),skip=0){
  has.dot <- setdiff(grep(at,x,fixed=TRUE),skip)
  if(!any(has.dot>0)) return(x)
  x <- trimws(x)
  is.int <- setdiff(seq(x),union(has.dot,skip))
  splitted <- strsplit(x[has.dot],at,fixed=TRUE)
  left <- sapply(splitted,function(x)x[1])
  maxleft <- max(nchar(left))
  right <- sapply(splitted,function(x)paste(x[-1],collapse="."))
  maxright <- max(nchar(right))
  maxcentered <- maxleft+maxright+1
  
  if(any(is.int>0)){
    integers <- match.arg(integers)
    if(integers=="right"){
      left <- format(left,justify="right",width=maxleft)
      right <- format(right,justify="left",width=maxright)
      fintegers <- format(x[is.int],
                          justify="right",
                          width=maxcentered)
    }
    if(integers=="left"){
      left <- format(left,justify="right",width=maxleft)
      right <- format(right,justify="left",width=maxright)
      fintegers <- format(x[is.int],
                          justify="left",
                          width=maxcentered)
    }
    if(integers=="dot"){
      maxleft <- max(maxleft,max(nchar(as.character(x[is.int]))))
      left <- format(left,justify="right",width=maxleft)
      right <- format(right,justify="left",width=maxright)
      fintegers <- format(x[is.int],
                          justify="right",
                          width=maxleft)
      fintegers <- paste(fintegers,format(" ",width=maxright))
    }
    centered <- paste(left,right,sep=".")
    maxcentered <- max(nchar(centered))
    x[has.dot] <- centered
    x[is.int] <- fintegers
  } else {
    left <- format(left,justify="right",width=maxleft)
    right <- format(right,justify="left",width=maxright)
    centered <- paste(left,right,sep=".")
    x[has.dot] <- centered
  }
  if(any(as.logical(skip)))
    x[skip] <- format(x[skip],width=maxcentered,justify="centre")
  
  return(x)
}

coefxpand <- function(x,names){
    if(length(x)){
        d <- dx <- dim(x)
        dd <- ddx <- dimnames(x)
        ddNULL <- sapply(dd,is.null)
        d[3] <- length(names)
        dd[[3]] <- names
        dd_ <- dd
        dd[ddNULL] <- lapply(d[ddNULL],seq,from=1)
        ddx[ddNULL] <- dd[ddNULL]
        res <- array("",dim=d,dimnames=dd)
        call.arg <- list(res)
        call.arg <- c(call.arg,ddx)
        call.arg <- c(call.arg,list(value=as.vector(x)))
        res <- do.call("[<-",call.arg)
        dimnames(res) <- dd_
        res
    }
    else if(length(dim(x))){
        d <- dim(x)
        dd <- dimnames(x)
        d[3] <- length(names)
        dd[[3]] <- names
        array("",dim=d,dimnames=dd)
    } else {
        r <- length(names)
        array("",dim=c(r,1,1),
              dimnames=list(names,NULL,NULL))
    }
}

smryxpand <- function(x,names){
  
    res <- matrix(rep("",length(names)),ncol=1)
    rownames(res) <- names              
    if(length(x)) {
        nms.x <- rownames(x)
        res[nms.x,] <- x
    }
    return(res)
}

html <- function(tag,...,.content=NULL,linebreak=FALSE){
  stopifnot(length(tag)==1)
  args <- list(...)
  content <- NULL
  attribs <- NULL
  if(length(names(args))){
    has.name <- nzchar(names(args))
    attribs <- args[has.name]
    if(any(!has.name) && length(.content))
      stop("use either unnamed arguments or the .content argument")
    if(any(!has.name))
      content <- args[!has.name]
    if(length(.content))
      content <- .content
  }
  else{
    if(length(args)&&length(.content))
      stop("use either unnamed arguments or the .content argument")
    if(length(args))
      content <- args
    else
      content <- .content
  }
  structure(
    list(tag=tag,
         attributes=attribs,
         content=content,
         linebreak=linebreak),
    class="html_elem")
}

print.html_elem <- function(x,...) 
  cat(as.character(x),"\n",...)

as.character.html_elem <- function(x,...){
  out <- paste0("<",x$tag)
  if(length(names(x$attributes))){
    for(n in names(x$attributes)){
      a <- x$attributes[[n]]
      a <- as.character(a)
      a <- paste(a,collapse=" ")
      out <- paste(out,paste0(n,"=",shQuote(a,"cmd")))
    }
  }
  out <- paste0(out,">")
  if(length(x$linebreak)>1 && x$linebreak[1]){
    out <- paste0(out,"\n")
  }
  if(length(x$content)){
    if(is.list(x$content)){
      if(inherits(x$content,"html_elem"))
        content <- as.character(x$content)
      else{
        content <- lapply(x$content,as.character)
        content <- unlist(content,as.character)
      }
    }
    else
      content <- as.character(x$content)

    out <- paste0(out,paste(content,collapse=""))
    out <- paste0(out,"</",x$tag,">")
  }
  if(length(x$linebreak)>1){
    if(x$linebreak[2])
      out <- paste0(out,"\n")
  }
  else if(x$linebreak)
    out <- paste0(out,"\n")
  out
}

content <- function(x) x$content
"content<-" <- setContent <- function(x,value){
  x$content <- value
  x
}

attribs <- function(x){
    a <- x$attributes
    if(!length(a))
        a <- list()
    structure(a,class="html_attributes")
}

"attribs<-" <- function(x,value){
  x$attributes <- value
  x
}
"[<-.html_attributes" <- function(x,i,...,value){
  x <- unclass(x)
  x[i] <- list(value)
  structure(x,class="html_attributes")
}

setAttribs <- function(x,...)UseMethod("setAttribs")
setAttribs.character <- function(x,...)return(x)
setAttribs.html_elem <- function(x,...){
  value <- c(...)
  n <- names(value)
  x$attributes[n] <- value
  x
}

c.html_elem <- function(...) reduce(list(...),join_html)



css <- function(...) as.css(c(...))

as.css <- function(x){
  if(inherits(x,"css")) x
  else structure(as.character(unclass(x)),
                 names=names(x),
                 class="css")
}

"[<-.css" <- function(x,i,...,value){
  x <- unclass(x)
  x[i] <- as.character(value)
  structure(x,class="css")
}
as.character.css <- function(x,...){
  n <- names(x)
  paste0(n,": ",x,";")
}
print.css <- function(x,...)
  cat(as.character(x),"\n",...)


style <- function(x){
  x$attributes$style
}
"style<-" <- function(x,value){
  x$attributes$style <- as.css(value)
  x
}

setStyle <- function(x,...)UseMethod("setStyle")
setStyle.character <- function(x,...)return(x)
# setStyle.list <- function(x,...)lapply(x,setStyle,...)
setStyle.html_elem <- function(x,...){
  value <- c(...)
  s <- x$attributes$style
  if(length(s)){
    n <- names(value)
    s[n] <- value
    x$attributes$style <- s
  }
  else {
    x$attributes$style <- as.css(value)
  }
  x
}


.html <- function(x,tag,...,linebreak=FALSE) html(tag=tag,...,.content=x,linebreak=linebreak)
.html_group <- function(x,tag,...,vectorize=FALSE,linebreak=FALSE){
  if(vectorize)
    structure(lapply(x,.html,tag=tag,...,linebreak=linebreak),class="html_group")
  else
    html(tag=tag,...,.content=x,linebreak=linebreak)
}

as.html_group <- function(x) structure(check_html_classes(x),class="html_group")
html_group <- function(...){ 
  x <- list(...)
  x <- x[sapply(x,length)>0]
  as.html_group(x)
  }
check_html_classes <- function(x){
  if(inherits(x,"html_elem")) return(x)
  if(inherits(x,"html_group")) return(x)
  if(is.character(x)) return(x)
  if(!is.list(x)) stop("check failed")
  if(all(sapply(x,.check_html_classes))) return(x)
  stop("check failed")
}
.check_html_classes <- function(x){
  if(inherits(x,"html_elem")) return(TRUE)
  if(inherits(x,"html_group")) return(TRUE)
  if(is.character(x)) return(TRUE)
  if(is.list(x)) return(all(sapply(x,.check_html_classes)))
  else return(FALSE)
}

as.character.html_group <- function(x,...)
  paste(unlist(lapply(x,as.character)),collapse="")
print.html_group <- function(x,...) 
  cat(as.character(x),...)

"[.html_group" <- function(x,i,....){
  x <- NextMethod()
  structure(x,class="html_group")
}
"[<-.html_group" <- function(x,i,....,value){
  x <- NextMethod()
  structure(x,class="html_group")
}

c.html_group <- function(...) reduce(list(...),join_html)

setAttribs.html_group <- function(x,...){
  x <- lapply(x,setAttribs,...)
  structure(x,class="html_group")
}
setStyle.html_group <- function(x,...){
  x <- lapply(x,setStyle,...)
  structure(x,class="html_group")
}

join_html <- function(x,y){

  if(inherits(x,"html_elem")&&inherits(y,"html_elem"))
    return(structure(list(x,y),class="html_group"))
  else if(inherits(x,"html_group")&&inherits(y,"html_elem"))
    return(structure(c(unclass(x),list(y)),class="html_group"))
  else if(inherits(x,"html_elem")&&inherits(y,"html_group"))
    return(structure(c(list(x),unclass(y)),class="html_group"))
  else if(inherits(x,"html_group")&&inherits(y,"html_group"))
    return(structure(c(unclass(x),unclass(y)),class="html_group"))
  else stop("cannot handle these arguments.")
}



html_td <- function(x,...,linebreak=FALSE).html_group(x,tag="td",...,linebreak=linebreak)
html_tr <- function(x,...,linebreak=TRUE).html_group(x,tag="tr",...,linebreak=linebreak)

html_dt <- function(x,...,linebreak=TRUE).html_group(x,tag="dt",...,linebreak=linebreak)
html_dd <- function(x,...,linebreak=TRUE).html_group(x,tag="dd",...,linebreak=linebreak)


html_beforeDec <- css("text-align"="right",
                             "margin-right"="0px",
                             "padding-right"="0px",
                             "padding-left"="0.3em")

html_dotDec <- css("text-align"="center",
                          "margin-left"="0px",
                          "margin-right"="0px",
                          "padding-right"="0px",
                          "padding-left"="0px",
                          width="1px")

html_afterDec <- css("text-align"="left",
                            "margin-left"="0px",
                            "padding-left"="0px",
                            "padding-right"="0.3em")

html_td_spltDec <- function(x,style=character(),...,linebreak=FALSE,add_CSS=FALSE){
  y <- matrix(x,nrow=3)
  if(add_CSS){
      html_beforeDec <- css(style,html_beforeDec)
      html_dotDec <- css(style,html_dotDec)
      html_afterDec <- css(style,html_afterDec)
      y1 <- html_td(y[1,],style=html_beforeDec,...,vectorize=TRUE)
      y2 <- html_td(y[2,],style=html_dotDec,...,vectorize=TRUE)
      y3 <- html_td(y[3,],style=html_afterDec,...,vectorize=TRUE,linebreak=linebreak)
  }
  else {
      y1 <- html_td(y[1,],...,vectorize=TRUE)
      y2 <- html_td(y[2,],...,vectorize=TRUE)
      y3 <- html_td(y[3,],...,vectorize=TRUE,linebreak=linebreak)
  }
  y <- mapply(list,y1,y2,y3,SIMPLIFY=FALSE)
  y <- lapply(y,as.html_group)
  structure(y,class="html_group")
}

html_table <- function(x,...,linebreak=c(TRUE,TRUE)) html(tag="table",...,.content=x,linebreak=linebreak)
html_p <- function(x,...,linebreak=TRUE) html(tag="p",...,.content=x,linebreak=linebreak)
html_div <- function(x,...,linebreak=c(TRUE,TRUE)) html(tag="div",...,.content=x,linebreak=linebreak)
html_pre <- function(x,...,linebreak=c(TRUE,TRUE)) html(tag="pre",...,.content=x,linebreak=linebreak)


format_html.html_elem <- function(x,...) as.character.html_elem(x)
format_html.html_group <- function(x,...) as.character.html_group(x)

html <- function(tag,...,.content=NULL,linebreak=FALSE){
  stopifnot(length(tag)==1)
  args <- list(...)
  content <- NULL
  attribs <- NULL
  if(length(names(args))){
    has.name <- nzchar(names(args))
    attribs <- args[has.name]
    if(any(!has.name) && length(.content))
      stop("use either unnamed arguments or the .content argument")
    if(any(!has.name))
      content <- args[!has.name]
    if(length(.content))
      content <- .content
  }
  else{
    if(length(args)&&length(.content))
      stop("use either unnamed arguments or the .content argument")
    if(length(args))
      content <- args
    else
      content <- .content
  }
  structure(
    list(tag=tag,
         attributes=attribs,
         content=content,
         linebreak=linebreak),
    class="html_elem")
}

print.html_elem <- function(x,...) 
  cat(as.character(x),"\n",...)

as.character.html_elem <- function(x,...){
  out <- paste0("<",x$tag)
  if(length(names(x$attributes))){
    for(n in names(x$attributes)){
      a <- x$attributes[[n]]
      a <- as.character(a)
      a <- paste(a,collapse=" ")
      out <- paste(out,paste0(n,"=",shQuote(a,"cmd")))
    }
  }
  out <- paste0(out,">")
  if(length(x$linebreak)>1 && x$linebreak[1]){
    out <- paste0(out,"\n")
  }
  if(length(x$content)){
    if(is.list(x$content)){
      if(inherits(x$content,"html_elem"))
        content <- as.character(x$content)
      else{
        content <- lapply(x$content,as.character)
        content <- unlist(content)
      }
    }
    else
      content <- as.character(x$content)

    out <- paste0(out,paste(content,collapse=""))
    out <- paste0(out,"</",x$tag,">")
  }
  if(length(x$linebreak)>1){
    if(x$linebreak[2])
      out <- paste0(out,"\n")
  }
  else if(x$linebreak)
    out <- paste0(out,"\n")
  out
}

content <- function(x) x$content
"content<-" <- setContent <- function(x,value){
  x$content <- value
  x
}

attribs <- function(x){
    a <- x$attributes
    if(!length(a))
        a <- list()
    structure(a,class="html_attributes")
}

"attribs<-" <- function(x,value){
  x$attributes <- value
  x
}
"[<-.html_attributes" <- function(x,i,...,value){
  x <- unclass(x)
  x[i] <- list(value)
  structure(x,class="html_attributes")
}

setAttribs <- function(x,...)UseMethod("setAttribs")
setAttribs.character <- function(x,...)return(x)
setAttribs.html_elem <- function(x,...){
  value <- c(...)
  n <- names(value)
  x$attributes[n] <- value
  x
}

c.html_elem <- function(...) reduce(list(...),join_html)



css <- function(...) as.css(c(...))

as.css <- function(x){
  if(inherits(x,"css")) x
  else structure(as.character(unclass(x)),
                 names=names(x),
                 class="css")
}

"[<-.css" <- function(x,i,...,value){
  x <- unclass(x)
  x[i] <- as.character(value)
  structure(x,class="css")
}
as.character.css <- function(x,...){
  n <- names(x)
  paste0(n,": ",x,";")
}
print.css <- function(x,...)
  cat(as.character(x),"\n",...)


style <- function(x){
  x$attributes$style
}
"style<-" <- function(x,value){
  x$attributes$style <- as.css(value)
  x
}

setStyle <- function(x,...)UseMethod("setStyle")
setStyle.character <- function(x,...)return(x)
# setStyle.list <- function(x,...)lapply(x,setStyle,...)
setStyle.html_elem <- function(x,...){
  value <- c(...)
  s <- x$attributes$style
  if(length(s)){
    n <- names(value)
    s[n] <- value
    x$attributes$style <- s
  }
  else {
    x$attributes$style <- as.css(value)
  }
  x
}


.html <- function(x,tag,...,linebreak=FALSE) html(tag=tag,...,.content=x,linebreak=linebreak)
.html_group <- function(x,tag,...,vectorize=FALSE,linebreak=FALSE){
  if(vectorize)
    structure(lapply(x,.html,tag=tag,...,linebreak=linebreak),class="html_group")
  else
    html(tag=tag,...,.content=x,linebreak=linebreak)
}

as.html_group <- function(x) structure(check_html_classes(x),class="html_group")
html_group <- function(...){ 
  x <- list(...)
  x <- x[sapply(x,length)>0]
  as.html_group(x)
  }
check_html_classes <- function(x){
  if(inherits(x,"html_elem")) return(x)
  if(inherits(x,"html_group")) return(x)
  if(is.character(x)) return(x)
  if(!is.list(x)) stop("check failed")
  if(all(sapply(x,.check_html_classes))) return(x)
  stop("check failed")
}
.check_html_classes <- function(x){
  if(inherits(x,"html_elem")) return(TRUE)
  if(inherits(x,"html_group")) return(TRUE)
  if(is.character(x)) return(TRUE)
  if(is.list(x)) return(all(sapply(x,.check_html_classes)))
  else return(FALSE)
}

as.character.html_group <- function(x,...)
  paste(unlist(lapply(x,as.character)),collapse="")
print.html_group <- function(x,...) 
  cat(as.character(x),...)

"[.html_group" <- function(x,i,....){
  x <- NextMethod()
  structure(x,class="html_group")
}
"[<-.html_group" <- function(x,i,....,value){
  x <- NextMethod()
  structure(x,class="html_group")
}

c.html_group <- function(...) reduce(list(...),join_html)

setAttribs.html_group <- function(x,...){
  x <- lapply(x,setAttribs,...)
  structure(x,class="html_group")
}
setStyle.html_group <- function(x,...){
  x <- lapply(x,setStyle,...)
  structure(x,class="html_group")
}

join_html <- function(x,y){

  if(inherits(x,"html_elem")&&inherits(y,"html_elem"))
    return(structure(list(x,y),class="html_group"))
  else if(inherits(x,"html_group")&&inherits(y,"html_elem"))
    return(structure(c(unclass(x),list(y)),class="html_group"))
  else if(inherits(x,"html_elem")&&inherits(y,"html_group"))
    return(structure(c(list(x),unclass(y)),class="html_group"))
  else if(inherits(x,"html_group")&&inherits(y,"html_group"))
    return(structure(c(unclass(x),unclass(y)),class="html_group"))
  else stop("cannot handle these arguments.")
}



html_td <- function(x,...,linebreak=FALSE).html_group(x,tag="td",...,linebreak=linebreak)
html_tr <- function(x,...,linebreak=TRUE).html_group(x,tag="tr",...,linebreak=linebreak)

html_dt <- function(x,...,linebreak=TRUE).html_group(x,tag="dt",...,linebreak=linebreak)
html_dd <- function(x,...,linebreak=TRUE).html_group(x,tag="dd",...,linebreak=linebreak)


html_beforeDec <- css("text-align"="right",
                             "margin-right"="0px",
                             "padding-right"="0px",
                             "padding-left"="0.3em")

html_dotDec <- css("text-align"="center",
                          "margin-left"="0px",
                          "margin-right"="0px",
                          "padding-right"="0px",
                          "padding-left"="0px",
                          width="1px")

html_afterDec <- css("text-align"="left",
                            "margin-left"="0px",
                            "padding-left"="0px",
                            "padding-right"="0.3em")

html_td_spltDec <- function(x,style=character(),...,linebreak=FALSE,add_CSS=FALSE){
  y <- matrix(x,nrow=3)
  if(add_CSS){
      html_beforeDec <- css(style,html_beforeDec)
      html_dotDec <- css(style,html_dotDec)
      html_afterDec <- css(style,html_afterDec)
      y1 <- html_td(y[1,],style=html_beforeDec,...,vectorize=TRUE)
      y2 <- html_td(y[2,],style=html_dotDec,...,vectorize=TRUE)
      y3 <- html_td(y[3,],style=html_afterDec,...,vectorize=TRUE,linebreak=linebreak)
  }
  else {
      y1 <- html_td(y[1,],...,vectorize=TRUE)
      y2 <- html_td(y[2,],...,vectorize=TRUE)
      y3 <- html_td(y[3,],...,vectorize=TRUE,linebreak=linebreak)
  }
  y <- mapply(list,y1,y2,y3,SIMPLIFY=FALSE)
  y <- lapply(y,as.html_group)
  structure(y,class="html_group")
}

html_table <- function(x,...,linebreak=c(TRUE,TRUE)) html(tag="table",...,.content=x,linebreak=linebreak)
html_p <- function(x,...,linebreak=TRUE) html(tag="p",...,.content=x,linebreak=linebreak)
html_div <- function(x,...,linebreak=c(TRUE,TRUE)) html(tag="div",...,.content=x,linebreak=linebreak)
html_pre <- function(x,...,linebreak=c(TRUE,TRUE)) html(tag="pre",...,.content=x,linebreak=linebreak)


format_html.html_elem <- function(x,...) as.character.html_elem(x)
format_html.html_group <- function(x,...) as.character.html_group(x)

genTable <- function (formula,
                      data=parent.frame(),
                      subset=NULL,
                      names=NULL,
                      addFreq=TRUE,
                      ...){

    m <- match.call(expand.dots = FALSE)
    dots <- m$...
    if(length(formula) < 3){
        m[[1]] <- as.name("xtabs")
        m[c("names","addFreq")] <- NULL
        m$... <- NULL
        return(eval(substitute(m),enclos=parent.frame()))
    }

    parent <- parent.frame()

    if(is.environment(data)){
        data <- mget(all.vars(formula),
                     envir=data,
                     inherits=TRUE)
    }
    data <- as.data.frame(data)

    if(!missing(subset)){
        subset <- eval(substitute(subset),data,parent)
        if(is.logical(subset))
            subset <- subset & !is.na(subset)
        else stop("'subset' arg must be logical")
        data <- data[subset,,drop=FALSE]
    }
    

    by <- formula[-2]
    expr <- formula[[2]]

    if(length(expr)==1){
        expr.c <- as.character(expr)
        if(is.factor(data[[expr.c]]))
            expr <- as.call(c(as.symbol("table"),expr))
        else
            expr <- as.call(c(as.symbol("sum"),expr))
    }
    if(addFreq){
        if("Freq" %in% names(data) &&length(expr) > 1 && 
           as.character(expr[[1]]) %in% c("table","Table","percent") &&
           !("weights" %in% names(expr))
           ){
            if(as.character(expr[[1]])=="table")
                expr[[1]] <- as.symbol("Table")
            if(length(expr) > 2){
                if(!length(names(expr))) addFreq <- FALSE
                else if("weights" %in% names(expr))
                     addFreq <- FALSE
            }
            if(addFreq)
                expr$weights <- as.symbol("Freq")
        }
    }

    
    if(deparse(formula[[3]])=="."){
        vars <- setdiff(names(data),all.vars(expr))
        by <- as.formula(paste("~",paste(vars,collapse="+")))
    }
    if(length(dots)) expr <- as.call(c(as.list(expr),dots))
    
    gdata <- Groups(data=data,by=by)
    
    wcall <- call("with",data=gdata,expr=expr)
    res <- eval(wcall,enclos=parent.frame())
    spec <- attr(res,"spec")
    if(length(dim(res)) > length(spec) && !missing(names))
        dimnames(res)[[1]] <- names
    res
}

Aggregate <- function (formula,
                       data=parent.frame(),
                       subset=NULL,
                       names=NULL,
                       addFreq=TRUE,
                       drop=TRUE,
                       as.vars=1,
                       ...){

    m <- match.call()

    m[[1]] <- as.name("genTable")
    m$data <- force(data)
    res <- eval(m,enclos=parent.frame())

    empty <- attr(res,"empty")
    spec <- attr(res,"spec")
    if(length(dim(res)) > length(spec)){
        res <- to.data.frame(res,as.vars=as.vars)
    }
    else {
        res <- as.data.frame(res)
        if(!missing(names))
            names(res)[3L] <- names
        else
            names(res)[3L] <- deparse(formula[[2]])
    }
    if(drop && any(empty)){
        nr <- nrow(res)
        ne <- length(empty)
        emtpy <- rep(as.vector(empty),each=nr/ne)
        res <- res[!empty,,drop=FALSE]
    }
    res
}

## ----------------------------------------------------------------------------
## Author: Jason W. Morgan
## URL: http://leftcensored.skepsi.net
## Date: 2011.06.21
##
## Description: This file contains functions for extracting model summaries
##   according to the requirements of mtable() and toLatex() available in the
##   memisc packages.
##
## Note: This code is based heavily on the examples and functions provided by
##   Martin Elff in his memisc package. If you find it useful, you should cite
##   *his* work. You can find his website here: http://www.martin-elff.net
##
## Copyright 2011 Jason W. Morgan
## 
## This program is free software: you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation, either version 3 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along with
## this program.  If not, see <http://www.gnu.org/licenses/>.
## ----------------------------------------------------------------------------

## ----------------------------------------------------------------------------
## Supported model types
## ----------------------------------------------------------------------------
##
## coxph, survreg - Cox proportional hazards models and parametric survival
##   models from the survival library.
##
## aftreg, phreg, weibreg - parametric AFT, proportional hazards, and Weibull
##    models available from the eha library.
##
## mer - multilevel mixed effects models as provided by the lme4 package. The
##    model objects produced in the beta lme4a package are not supported.
##
## ----------------------------------------------------------------------------

## ----------------------------------------------------------------------------
## Summary functions for survival objects.
## ----------------------------------------------------------------------------

setSummaryTemplate(coxph = c("Log-likelihood" = "($logLik:f#)",
                             "AIC" = "($AIC:f#)",
                             "BIC" = "($BIC:f#)",
                             "N" = "($N:d)"))

setSummaryTemplate(survreg = c("Log-likelihood" = "($logLik:f#)",
                               "AIC" = "($AIC:f#)",
                               "BIC" = "($BIC:f#)",
                               "N" = "($N:d)"))

setSummaryTemplate(phreg = c("Log-likelihood" = "($logLik:f#)",
                             "AIC" = "($AIC:f#)",
                             "BIC" = "($BIC:f#)",
                             "N" = "($N:d)"))

getSummary.coxph <- function(obj, alpha = 0.05, ...) {
    N <- obj$n

    ## Take the info right out of the object to guarantee that I get the right
    ## standard errors.
    b <- coef(obj)
    se <- sqrt(diag(vcov(obj)))
    coef <- cbind(b, se, b / se, 1-pnorm(abs(b/se)))

    lower <- qnorm(p = alpha/2, mean = coef[, 1], sd = coef[,2])
    upper <- qnorm(p = 1 - alpha/2, mean = coef[, 1], sd = coef[,2])

    coef <- cbind(coef, lower, upper)

# Patched by ME 2017-09-02
    dn <- list(
        rownames(coef),
        c("est","se","stat","p","lwr","upr"),
        rownames(attr(obj$terms,"factors"))[1]
    )
    dim(coef) <- c(dim(coef)[1],dim(coef)[2],1)
    dimnames(coef) <- dn

    ll <- obj$loglik[2]
    K <- length(b)
    
    AIC <- -2*ll + 2*K
    BIC <- -2*ll + K*log(N) 

    sumstat <- c(logLik = ll, AIC = AIC, BIC = BIC, N = N)
    
    list(coef = coef,
         sumstat = sumstat,
         contrasts = obj$contrasts, 
         xlevels = obj$xlevels,
         call = obj$call)
}

getSummary.survreg <- function(obj, alpha = 0.05, ...) {

    coef <- cbind(c(coef(obj), obj$scale), c(sqrt(diag(obj$var)),NA))
    K <- length(coef) - 1
    N <- obj$df.residual + K
    stat  <- coef[, 1] / coef[, 2]
    pval  <- 1-pnorm(abs(stat))

    lower <- qnorm(p = alpha/2, mean = coef[, 1], sd = coef[,2])
    upper <- qnorm(p = 1 - alpha/2, mean = coef[, 1], sd = coef[,2])
    coef  <- cbind(coef, stat, pval, lower, upper)

# Patched by ME 2017-09-02
    dn <- list(
        rownames(coef),
        c("est","se","stat","p","lwr","upr"),
        rownames(attr(obj$terms,"factors"))[1]
    )
    dim(coef) <- c(dim(coef)[1],dim(coef)[2],1)
    dimnames(coef) <- dn
    rownames(coef)[nrow(coef)] <- "scale"

    pv <- c("(Intercept)",colnames(attr(obj$terms,"factors")))
    pv <- match(pv,rownames(coef),nomatch=0)
    distparms <- coef[-pv,,,drop=FALSE]
    coef <- coef[pv,,,drop=FALSE]
    
    ll <- obj$loglik[2]
    AIC <- -2*ll + 2*K
    BIC <- -2*ll + K*log(N) 
    sumstat <- c(logLik = ll, AIC = AIC, BIC = BIC, N = N)

    list(coef = coef,
         distparms = distparms,
         sumstat = sumstat,
         contrasts = obj$contrasts, 
         xlevels = obj$xlevels,
         call = obj$call)
}

## aftreg(), phreg(), weibreg() are part of the eha package.
getSummary.aftreg <- function(obj, alpha = 0.05, ...) {

    N <- obj$n

    coef  <- cbind(coef(obj), sqrt(diag(obj$var)))
    stat  <- coef[, 1] / coef[, 2]
    pval  <- 1-pnorm(abs(stat))
    lower <- qnorm(p = alpha/2, mean = coef[, 1], sd = coef[, 2])
    upper <- qnorm(p = 1 - alpha/2, mean = coef[, 1], sd = coef[, 2])

    coef  <- cbind(coef, stat, pval, lower, upper)

# Patched by ME 2017-09-02
    dn <- list(
        rownames(coef),
        c("est","se","stat","p","lwr","upr"),
        rownames(attr(obj$terms,"factors"))[1]
    )
    dim(coef) <- c(dim(coef)[1],dim(coef)[2],1)
    dimnames(coef) <- dn

    pv <- colnames(attr(obj$terms,"factors"))
    pv <- match(pv,rownames(coef))
    distparms <- coef[-pv,,,drop=FALSE]
    coef <- coef[pv,,,drop=FALSE]
    
    ll <- obj$loglik[2]
    K <- length(coef[,1,])
    AIC <- -2*ll + 2*K
    BIC <- -2*ll + K*log(N) 
    sumstat <- c(logLik = ll, AIC = AIC, BIC = BIC, N = N)

    list(coef = coef,
         distparms = distparms,
         sumstat = sumstat,
         contrasts = obj$contrasts, 
         xlevels = obj$xlevels,
         call = obj$call)
}

getSummary.phreg <- function(obj, alpha = 0.05, ...) {
  getSummary.aftreg(obj, alpha = alpha, ...)
}

getSummary.weibreg <- function(obj, alpha = 0.05, ...) {
  getSummary.aftreg(obj, alpha = alpha, ...)
}

sort.data.frame <- function(x,decreasing=FALSE,by=NULL,na.last=NA,...){
  if(!length(by))
    ii <- do.call(order,c(x,list(decreasing=decreasing,na.last=na.last)))
  else if(is.character(by))
    ii <- do.call(order,c(x[by],list(decreasing=decreasing,na.last=na.last)))
  else if(inherits(by,"formula")){
    bydata <- model.frame(formula=by,data=x,na.action=na.pass)
    bydata <- bydata[rev(all.vars(by))]
    ii <- do.call(order,c(bydata,list(decreasing=decreasing,na.last=na.last)))
  }
  x[ii,]
}

sort.data.set <- function(x,decreasing=FALSE,by=NULL,na.last=NA,...){
  if(!length(by))
    ii <- do.call(order,c(x,list(decreasing=decreasing,na.last=na.last)))
  else if(is.character(by))
    ii <- do.call(order,c(x[by],list(decreasing=decreasing,na.last=na.last)))
  else if(inherits(by,"formula")){
    bydata <- model.frame(formula=by,data=x,na.action=na.pass)
    bydata <- bydata[rev(all.vars(by))]
    ii <- do.call(order,c(bydata,list(decreasing=decreasing,na.last=na.last)))
  }
  x[ii,]
}

df_format_stdstyle <- c(
  "padding-top"="3px",
  "padding-bottom"="3px",
  "padding-left"="0.5ex",
  "padding-right"="0.5ex",
  "margin-top"="0px",
  "margin-bottom"="0px",
  "border-style"="none",
  "border-width"="0px"
)



format_html.data.frame <- function(x,
                               toprule=2,midrule=1,bottomrule=2,
                               split.dec=TRUE,
                               row.names=TRUE,
                               digits=getOption("digits"),
                               format="f",
                               style=df_format_stdstyle,
                               margin="2ex auto",
                               ...){

  
  firstcol <- c("padding-left"="0.3em")
  lastcol <- c("padding-right"="0.3em")
  toprule <- c("border-top"=paste0(midrule,"px solid"))
  bottomrule <- c("border-bottom"=paste0(midrule,"px solid"))
  midrule_above <- c("border-top"=paste0(midrule,"px solid"))
  midrule <- c("border-bottom"=paste0(midrule,"px solid"))
  align.right <- c("text-align"="right")  
  align.left <- c("text-align"="left")  
  align.center <- c("text-align"="center")
  row_style <- c("border-style"="none")
  table_style <- c("border-collapse"="collapse" ,"border-style"="none")
  
  colsep <- ""
  rowsep <- "\n"
  
  n <- nrow(x)
  m <- ncol(x)
  d <- digits
  is.int <- sapply(x,is.integer)
  is.num <- sapply(x,is.numeric) & !is.int
  m.num <- sum(is.num)
  digits <- integer(m.num)
  digits[] <- d
  fdigits <- integer(m)
  fdigits[is.num] <- digits
  fo <- format
  format <- character(m)
  format[is.num] <- fo  
  
  colspan <- integer(0)
  body <- matrix(nrow=nrow(x),ncol=0)
  for(i in 1:m) {
      tmp <- x[[i]]
      dim.x.i <- dim(tmp)
      ncol.tmp <- if(length(dim.x.i)) ncol(tmp) else 1
      if(is.int[i]){
          tmp <- formatC(tmp,format="d")
          col <- html_td(tmp,vectorize=TRUE,style=css(style))
          colspan <- c(colspan,ncol.tmp)
      }
      else if(is.num[i]){
          tmp <- formatC(tmp,digits=fdigits[i],format=format[i])
          if(split.dec){
              tmp <- spltDec(tmp)
              col <- html_td_spltDec(tmp,style=css(style))
              colspan <- c(colspan,3L*ncol.tmp)
          }
          else{
              col <- html_td(tmp,vectorize=TRUE,style=css(style))
              colspan <- c(colspan,ncol.tmp)
          }
      }
      else {
          tmp <- as.character(tmp)
          col <- html_td(tmp,vectorize=TRUE,style=css(style))
          col <- setStyle(col,align.left)
          colspan <- c(colspan,ncol.tmp)
      }
      dim(col) <- dim.x.i
      body <- cbind(body,col)
  }
    #browser()
  if(row.names){
    tmp <- rownames(x)
    ldr <- html_td(tmp,vectorize=TRUE,style=css(c(style,firstcol,align.right)))
    body <- cbind(ldr,body)
  }
   
  body[1,] <- lapply(body[1,],setStyle,toprule)
  body[n,] <- lapply(body[n,],setStyle,bottomrule)
  body <- apply(body,1,html_tr)
  
  hdr <- colnames(x)
  if(row.names) {
    hdr <- c("",hdr)
    colspan <- c(1L,colspan)
  }
  
  hdr <- html_td(hdr,vectorize=TRUE,style=css(style))
  hdr[] <- mapply(setAttribs,hdr,colspan=colspan,SIMPLIFY=FALSE)
  hdr <- lapply(hdr,setStyle,df_format_stdstyle)
  hdr <- lapply(hdr,setStyle,align.center)
  hdr <- lapply(hdr,setStyle,toprule)
  hdr[[1]] <- setStyle(hdr[[1]],lastcol)
  hdr[[length(hdr)]] <- setStyle(hdr[[length(hdr)]],lastcol)
  hdr <- html_tr(hdr)
  
  if(length(margin))
    table_style <- c(table_style,margin=margin)
  ans <- html_table(c(list(hdr),body),style=as.css(table_style))

  ans <- as.character(ans)
  return(ans)
}

"%nin%" <- function(x, table) !(x %in% table)

reorder.array <- function(x,dim=1,names=NULL,indices=NULL,FUN=mean,...){
  if(length(dim)>1) stop("'dim' has to be scalar")
  if(as.numeric(!missing(names)) + as.numeric(!missing(indices)) + as.numeric(!missing(FUN))>2) 
    stop("too many arguments, use *either* 'names', 'indices', or 'FUN'")
  if(!missing(names)){
    indices <- match(names,dimnames(x)[[dim]])
    if(any(is.na(indices))) stop("unused dinnames in 'names' argument")
    other.indices <- setdiff(1:dim(x)[dim],indices)
    sort.indices <- c(indices,other.indices)
  }
  else if(!missing(indices)){
    other.indices <- setdiff(1:dim(x)[dim],indices)
    sort.indices <- c(indices,other.indices)
  } else {
    results <- apply(x,dim,FUN)
    sort.indices <- order(results)
  } 
  all.indices <- lapply(dim(x),function(d)1:d)
  all.indices[[dim]] <- sort.indices
  do.call("[",c(list(x),all.indices))
}

reorder.matrix <- reorder.array

lsl <- function() ls.str(envir=parent.frame())

split.matrix <- function (x, f, drop = FALSE, dim.drop=TRUE, ...){
  index <- seq(nrow(x))
  if(length(index)==length(f)){
    index <- split.default(index,f,drop=drop)
    lapply(index,function(i)x[i,,drop=dim.drop])
  } else {
    split(c(x),f,drop=drop,...)
  }
}

"split<-.matrix" <- function(x, f, drop = FALSE, ..., value)
{
    ix <- split(seq_along(x), f, drop = drop, ...)
    n <- length(value)
    j <- 0
    for (i in ix) {
        j <- j%%n + 1
        x[i, ] <- value[[j]]
    }
    x
}

setClassUnion("atomic",c("raw","logical","integer","numeric","complex","character"))
setClassUnion("atomic or NULL",c("atomic","NULL"))
setClassUnion("atomic or factor",c("atomic","factor"))
setClassUnion("character or NULL",c("character","NULL"))
setClassUnion("character or integer",c("character","integer"))
# setClassUnion("index",c("logical","numeric","character"))
# setClassUnion("strict.index",c("logical","numeric"))

setClass("value.labels",
  contains = "character",
  representation(
    values = "atomic"
  )
)

setClass("value.filter",
  representation(
    filter="atomic or NULL"
  )
)
setClass("valid.values",contains="value.filter")
setClass("valid.range",contains="value.filter")
setClass("missing.values",contains="value.filter",representation(range="atomic or NULL"))

setClassUnion("value.labels or NULL",c("value.labels","NULL"))
setClassUnion("value.filter or NULL",c("value.filter","NULL"))

setClass("annotation",contains="character")

setClass("item",
  representation(
    value.labels="value.labels or NULL",
    value.filter="value.filter or NULL",
    measurement="character or NULL",
    annotation="annotation"
  )
)

if(getRversion() < "3.6.0")
    setClass("double",
             contains="numeric",
             prototype=double()
             )
# methods package in R version >= 3.6.0 defines 'double'

setClassUnion("numeric.item")
setClassUnion("item.vector","numeric.item")

setClass("integer.item",
  contains=c("item","numeric.item","item.vector","integer")
)

setClass("double.item",
  contains=c("item","numeric.item","item.vector","double","numeric")
)
setClass("character.item",
  contains=c("item","item.vector","character")
)

setClass("Date.item",
  contains=c("item","numeric.item","item.vector","double","numeric")
)

setClass("datetime.item",
   representation(tzone="character or NULL",origin="character or NULL"),
   contains=c("item","numeric.item","item.vector","double","numeric")
 )

setClass("named.list",
  contains="list",
  representation(
  names = "character")
)

setClass("item.list",contains="named.list")

setClass("data.set",
  contains=c("item.list","named.list","list"),
  representation(
    row_names = "character or integer",
    document = "character"
  )
)

setClass("importer",
  contains=c("item.list","named.list","list"),
  representation(
    ptr="externalptr",
    document = "character",
    encoded = "character"
  )
)


setClass("spss.fixed.importer",
  contains="importer",
  representation(
    columns.file="character",
    varlab.file="character or NULL",
    codes.file="character or NULL",
    missval.file="character or NULL",
    data.spec="list"
  )
)

setClass("spss.portable.importer",
  contains="importer",
  representation(
    varlab.file="character or NULL",
    codes.file="character or NULL",
    missval.file="character or NULL",
    data.spec="list"
  )
)

setClass("spss.system.importer",
  contains="importer",
  representation(
    varlab.file="character or NULL",
    codes.file="character or NULL",
    missval.file="character or NULL",
    data.spec="list"
  )
)

setClass("Stata.importer",
  contains="importer",
  representation(
    data.spec = "list"
  )
)

setClass("Stata_new.importer",
         contains="importer",
         representation(
             types = "integer"
         )
)

setClass("codebook",
  contains="list"
)

setClass("codebookEntry",
  representation(
    spec = "character",
    stats = "list",
    annotation = "character or NULL"
  )
)

setOldClass("codeplan")

# setOldClass("grouped.data")
# 
# setClass("grouped.data.set",
#          contains=c("grouped.data","data.set"),
#          representation(
#              groups="list",
#              spec="data.set"
#          ))

## The following three functions are contributed by
## Christopher N. Lawrence, Ph.D. <c.n.lawrence@gmail.com>
## Assistant Professor of Political Science
## Texas A&M International University
## 313 LBVSC, 5201 University Blvd
## Laredo, Texas 78041-1920

## Some patches were applied by ME in July 2012, August 2017

getSummary.polr <- function(obj,
            alpha=.05,
            ...){

  smry <- summary(obj)
  N <- if(length(weights(obj))) sum(weights(obj))
    else nobs(obj)

  coef <- smry$coef

  if (smry$df.residual) {
      pvals <- 2*pt(-abs(coef[,3]), smry$df.residual)
  } else {
      pvals <- 2*pnorm(-abs(coef[,3]))
  }

  lower <- qnorm(p=alpha/2,mean=coef[,1],sd=coef[,2])
  upper <- qnorm(p=1-alpha/2,mean=coef[,1],sd=coef[,2])

  coef <- cbind(coef,pvals,lower,upper)

  colnames(coef) <- c("est","se","stat","p","lwr","upr")
  null.model <- update(obj, .~1)

  LR <- deviance(null.model) - deviance(obj)
  df <- null.model$df.residual - smry$df.resid

  ll <- logLik(obj)
  dev <- deviance(obj)

  if(df > 0){
    p <- pchisq(LR,df,lower.tail=FALSE)
    L0.pwr <- exp(-deviance(null.model)/N)
    #LM.pwr <- exp(-smry$deviance/N)

    Aldrich.Nelson <- LR/(LR+N)
    McFadden <- 1 - dev/deviance(null.model)
    Cox.Snell <- 1 - exp(-LR/N)
    Nagelkerke <- Cox.Snell/(1-L0.pwr)
    }
  else {
    LR <- NA
    df <- NA
    p <- NA
    Aldrich.Nelson <- NA
    McFadden <- NA
    Cox.Snell <- NA
    Nagelkerke <- NA
    }

  AIC <- AIC(obj)
  BIC <- AIC(obj,k=log(N))
  sumstat <- c(
          LR             = LR,
          df         = df,
          p             = p,
          logLik        = ll,
          deviance      = dev,
          Aldrich.Nelson = Aldrich.Nelson,
          McFadden      = McFadden,
          Cox.Snell       = Cox.Snell,
          Nagelkerke    = Nagelkerke,
          AIC           = AIC,
          BIC           = BIC,
          N             = N
          )

  #coef <- apply(coef,1,applyTemplate,template=coef.template)

  #sumstat <- drop(applyTemplate(sumstat,template=sumstat.template))
  list(coef=coef,sumstat=sumstat,contrasts=obj$contrasts,xlevels=smry$xlevels,call=obj$call)
}


getSummary.simex <- function(obj,
            alpha=.05,
            ...){

  smry <- summary(obj)
  modsmry <- summary(obj$model)
  N <- if(length(weights(obj$model))) sum(weights(obj$model))
    else {
        if(length(modsmry$df) > 1) sum(modsmry$df[1:2])
        else obj$model$nobs
    }

  ## Asymptotics not guaranteed to be here.
  coef <- smry$coef$jackknife

  lower <- qnorm(p=alpha/2,mean=coef[,1],sd=coef[,2])
  upper <- qnorm(p=1-alpha/2,mean=coef[,1],sd=coef[,2])

  coef <- cbind(coef,lower,upper)

  colnames(coef) <- c("est","se","stat","p","lwr","upr")
  sumstat <- c(N=N)

  list(coef=coef,sumstat=sumstat,contrasts=obj$contrasts,xlevels=smry$xlevels,call=obj$call)
}

setSummaryTemplate("simex" = c(
  "N" = "($N:d)"
))

neg2mis <- function(x,all=FALSE,
                    exclude=NULL,
                    select=NULL,
                    zero=FALSE){
    if(!inherits(x,"item.list"))
        stop("Argument 'x' must inherit from 'item.list'.")
    nms <- names(x)
    Dat <- structure(x@.Data,names=nms)
    if(all){
        use <- nms
    }
    else if(!missing(exclude)){
        nl <- as.list(seq_along(nms))
        names(nl) <- nms
        dont.use <- eval(substitute(exclude), nl, parent.frame())
        use <- unlist(nl[-dont.use])
    }
    else if(!missing(select)){
        nl <- as.list(seq_along(nms))
        names(nl) <- nms
        use <- eval(substitute(select), nl, parent.frame())
        use <- unlist(nl[use])
    }
    else {
        stop("one of the arguments 'all', 'exclude', or 'select' shoud be supplied")
    }
    Dat[use] <- lapply(Dat[use],neg2mis_1,zero=zero)
    x@.Data <- unname(Dat)
    x
}

neg2mis_1 <- function(x,zero=zero){
    if(length(value.filter(x))) return(x)
    else {
        if(zero)
            missing.values(x) <- list(range=c(-Inf,0))
        else
            valid.range(x) <- c(0,Inf)
        return(x)
    }
}

to.data.frame <- function(X,as.vars=1,name="Freq"){
  if(is.character(as.vars)){
      nms <- names(dimnames(X))
      if(!(as.vars %in% nms)) stop("'",as.vars,"' is not a known dim extent")
      as.vars <- match(as.vars,nms)
  }
  if(is.atomic(X)){
    as.vars <- as.vars[1]
    if(as.vars==0){
        Z <- dimnames(X)
        if(!length(Z)){
          Z <- lapply(dim(X),seq_len)
        }
        else {
          Znull <- sapply(Z,length) == 0
          Zdims <- dim(X)
          Z[Znull] <- lapply(Zdims[Znull],seq_len)
        }
        Z <- numericIfPossible(expand.grid(Z))
        X <- as.data.frame(structure(list(c(X)),names=name))
    }
    else {
        ncols <- dim(X)[as.vars]
        nrows <- prod(dim(X)[-as.vars])
        coln <- dimnames(X)[[as.vars]]
        Z <- dimnames(X)[-as.vars]
        if(!length(Z)){
          Z <- lapply(dim(X)[-as.vars],seq_len)
        }
        else {
          Znull <- sapply(Z,length) == 0
          Zdims <- dim(X)[-as.vars]
          Z[Znull] <- lapply(Zdims[Znull],seq_len)
        }
        Z <- numericIfPossible(expand.grid(Z))
        ii <- seq(length(dim(X)))
        X <- aperm(X,c(ii[-as.vars],ii[as.vars]))
        dim(X) <- c(nrows,ncols)
        X <- as.data.frame.matrix(X)
        rownames(X) <- rownames(Z) <- 1:nrows
        names(X) <- coln
        }
    }
  else {
    nrows <- prod(dim(X))
    Z <- dimnames(X)
    if(!length(Z)){
      Z <- lapply(dim(X),seq_len)
    }
    else {
      Znull <- sapply(Z,length) == 0
      Zdims <- dim(X)
      Z[Znull] <- lapply(Zdims[Znull],seq_len)
    }
    X <- lapply(X,as.data.frame)
    Z <- numericIfPossible(expand.grid(Z))
    lnrows <- sapply(X,nrow)
    lncols <- sapply(X,ncol)
    if(!allequal(lncols)) stop("array elements do not match")
    ncols <- lncols[1]
    X <- do.call("rbind",X)
    i <- rep(1:nrows,lnrows)
    Z <- Z[i,,drop=FALSE]
    rownames(X) <- rownames(Z) <- 1:nrow(X)
    }
  cbind(Z,X)
}

# to.array <- function(x,...) UseMethod("to.array")
# to.array.default <- function(x,...)base::as.array(x)

# to.array.data.frame <- function(x,data.name=NULL,...){
#   s <- paste(deparse(substitute(x)),collapse="")
#   fcts <- sapply(x,is.factor)
#   if(any(fcts)){
#     nofcts <- !fcts
#     fcts <- x[fcts]
#     d <- c(sum(nofcts),sapply(fcts,nlevels))
#     m <- nrow(x) %/% prod(d[-1])
#     r <- nrow(x) %% prod(d[-1])
#     if(r!=0)stop(gettextf("cannot transform '%s' into an array",s))
#     rn <- rownames(x)
#     x <- x[nofcts]
#     ii <- c(list(rn),lapply(fcts,function(x)match(x,levels(x))))
#     x <- as.matrix(x)
#     x <- t(x)[,do.call("order",rev(ii))]
#     dn <- c(structure(list(rownames(x)),names=data.name),lapply(fcts,levels))
#     if(m>1){
#       dim(x) <- c(m,d)
#       dimnames(x) <- c(list(rn[1:m]),dn)
#     }
#     else {
#       dim(x) <- d
#       dimnames(x) <- dn
#     }
#     return(x)
#   }
#   else return(as.matrix(x))
# }

setMethod("as.array","data.frame",
  function(x,data.name=NULL,...){
    s <- paste(deparse(substitute(x)),collapse="")
    fcts <- sapply(x,is.factor)
    if(any(fcts)){
      nofcts <- !fcts
      fcts <- x[fcts]
      d <- c(sum(nofcts),sapply(fcts,nlevels))
      m <- nrow(x) %/% prod(d[-1])
      r <- nrow(x) %% prod(d[-1])
      if(r!=0)stop(gettextf("cannot transform '%s' into an array",s),call.=FALSE)
      rn <- rownames(x)
      x <- x[nofcts]
      ii <- c(list(rn),lapply(fcts,function(x)match(x,levels(x))))
      x <- as.matrix(x)
      x <- t(x)[,do.call("order",rev(ii))]
      dn <- c(structure(list(rownames(x)),names=data.name),lapply(fcts,levels))
      if(m>1){
        dim(x) <- c(m,d)
        dimnames(x) <- c(list(rn[1:m]),dn)
      }
      else {
        dim(x) <- d
        dimnames(x) <- dn
      }
      return(x)
    }
    else return(as.matrix(x))
})

check.file <- function(filename,error=FALSE,warn=FALSE){
    if(!length(filename) || !nchar(filename)){
        if(error) stop("empty filename",call.=FALSE)
        else if(warn) warning("empty filename",call.=FALSE)
        return(FALSE)
    }
    if(file.access(filename,mode=0)== -1){
        if(warn || error){
            msg <- paste("cannot find",sQuote(basename(filename)),"in",sQuote(dirname(filename)))
            if(error) stop(msg,call.=FALSE)
            if(warn) warning(msg,call.=FALSE)
        }
        return(FALSE)
    }
    else return(TRUE)
}

prettifyLab <- function(x){
      if(!is.character(x)) return(x)
      nx <- names(x)
      x <- gsub("^[0-9]+[.][0-9]*\\s+","",x)
      x <- paste(
                  toupper(substring(x,1,1)),
                  tolower(substring(x,2)),
                  sep=""
                  )
      names(x) <- nx
         return(x)
    }

prettifyNames <- function(x){
      if(!length(x) || !is.atomic(x) && !is.list(x)) return(x)
      nx <- names(x)
      nx <- gsub("^[0-9]+[.][0-9]*\\s+","",nx)
      nx <- paste(
                  toupper(substring(nx,1,1)),
                  tolower(substring(nx,2)),
                  sep=""
                  )
      names(x) <- nx
      return(x)
    }

iconv_list <- function(x,encoded){
    iconv <- nzchar(encoded)
    if(iconv){
        ic <- sapply(x,is.character)
        if(any(ic))
            x[ic] <- lapply(x[ic],iconv,from=encoded)
    }
    return(x)
}

prep.vars <- function(values,parent,sorted){

    colon <- function(x) {
        if(is.numeric(x[[2]]) && is.numeric(x[[3]]))
            return(seq.int(from=x[[2]],to=x[[3]]))
        from.name <- as.character(x[[2]])
        to.name <- as.character(x[[3]])
        nms <- ls(parent,sorted=sorted)
        from <- match(from.name,nms,nomatch=0L)
        to <- match(to.name,nms,nomatch=0L)
        if(any(c(from,to)==0))stop("Unknown variable referenced")
        values <- nms[from:to]
        lapply(values,as.symbol)
    }
    tosymbols <- function(x){
        x <- as.list(x)
        isc <- sapply(x,is.call)
        if(!any(isc))return(x)
        else {
            xisc <- x[isc]
            x[isc] <- lapply(xisc,expcall)   
        }
        x
    }
    expcall <- function(x){
        if(as.character(x[[1]]) %in% c("c","list")) tosymbols(x[-1])
        else if(as.character(x[[1]])=="rx") {
            rxcall <- x
            rxcall[[1]] <- as.name("rx2symbols")
            eval(rxcall)
        }
        else if(as.character(x[[1]])==":"){
            colon(x)
        }
        else eval(x,parent)
    }
    firstchar <- function(x)substr(x,1,1)
    lastchar <- function(x){
        n <- nchar(x)
        substr(x,n,n)
    }
    rx2symbols <- function(pattern){
        nms <- ls(parent)
        pattern <- pattern[1]
        if(firstchar(pattern)!="^") pattern <- paste0("^",pattern)
        if(lastchar(pattern)!="$") pattern <- paste0(pattern,"$")
        nms <- grep(pattern=pattern,x=nms,value=TRUE)
        as.symbols(nms)
    }

    values <- if(is.call(values)) {
                  if(as.character(values[[1]]) %in% c("c","list")) tosymbols(values[-1])
                  else if(as.character(values[[1]])=="rx") {
                      rxcall <- values
                      rxcall[[1]] <- as.name("rx2symbols")
                      eval(rxcall)
                  }
                  else if(as.character(values[[1]])==":"){
                      colon(values)
                  }
                  else eval(values,parent)
              }
              else
                  eval(values)

    values <- unlist(values)
    valchars <- lapply(values,as.character)
    if(!all(nzchar(valchars))) stop("empty element in substitution list")
    values
}


foreach <- function(...,.sorted,.outer=FALSE){
  args <- match.call(expand.dots=FALSE)$...
  tags <- names(args)
  parent <- parent.frame()
  vars <- args[nzchar(tags)]
  expr <- args[!nzchar(tags)]
  if(length(expr)) expr <- expr[[1]]
  else return()
  tags <- tags[nzchar(tags)]
  if(!length(expr) || !length(tags))return(invisible(NULL))
  if(missing(.sorted)){
      if(identical(parent,.GlobalEnv))
          .sorted <- TRUE
      else
          .sorted <- FALSE
  }
  vars <- lapply(vars,prep.vars,parent=parent,sorted=.sorted)
  if(length(vars)==1) {
    vars <- as.matrix(vars[[1]])
    colnames(vars) <- tags
  } else {
      l <- sapply(vars,length)
      if(.outer) {
          i <- lapply(l,seq.int)
          i <- do.call('expand.grid',i)
          vars <- mapply('[',vars,i)
      } else {
          if(length(unique(l)) > 1) stop("variables have unequal length")
          vars <- do.call('cbind',vars)
      }
  }

  for(i in seq_len(nrow(vars))){
    subst <- vars[i,]
    if(!is.list(subst)) 
        subst <- structure(as.list(subst),
                           names=colnames(vars))
    res <- do.call("substitute",list(expr,subst))
    eval(res,parent.frame())
    }
}

xapply <- function(...,.sorted,simplify=TRUE,USE.NAMES=TRUE,.outer=FALSE){
  args <- match.call(expand.dots=FALSE)$...
  tags <- names(args)
  parent <- parent.frame()
  vars <- args[nzchar(tags)]
  expr <- args[!nzchar(tags)]
  if(length(expr)) expr <- expr[[1]]
  else return()
  tags <- tags[nzchar(tags)]
  if(!length(expr) || !length(tags))return(invisible(NULL))
  parent <- parent.frame()
  e <- evalq(environment(), list(), parent)
  if(missing(.sorted)){
      if(identical(parent,.GlobalEnv))
          .sorted <- TRUE
      else
          .sorted <- FALSE
  }
  vars <- lapply(vars,prep.vars,parent=parent,sorted=.sorted)
  vars. <- lapply(vars,as.character)
  if(length(vars)==1) {
    vars <- as.matrix(vars[[1]])
    colnames(vars) <- tags
  } else {
      l <- sapply(vars,length)
      if(.outer) {
          i <- lapply(l,seq.int)
          i <- do.call('expand.grid',i)
          vars <- mapply('[',vars,i)
      } else {
          if(length(unique(l)) > 1) stop("variables have unequal length")
          vars <- do.call('cbind',vars)
      }
  }

  res <- Sapply(seq_len(nrow(vars)),function(i){
    subst <- vars[i,]
    if(!is.list(subst)) 
        subst <- structure(as.list(subst),
                           names=colnames(vars))
    res <- do.call("substitute",list(expr,subst))
    eval(res,e)
    },simplify=simplify)
  if(USE.NAMES){
      if(isTRUE(USE.NAMES)) USE.NAMES <- 1
      else USE.NAMES <- as.integer(USE.NAMES)
      nms <- as.character(vars[,USE.NAMES])
  }
  if(simplify && is.array(res)){
      ndim <- length(dim(res))
      dmn <- dimnames(res)
      if(.outer){
          dim(res) <- c(dim(res)[-ndim],l)
          ll <- length(l)
          if(USE.NAMES){
              dimnames(res) <- c(dmn[-ndim],vars.)
          }
          else {
              dimnames(res) <- c(dmn[-ndim],rep(list(NULL),ll))
          }
      }
      else if(USE.NAMES) 
          dimnames(res)[[ndim]] <- nms
  } else if(USE.NAMES){
      names(res) <- nms
  }
  res
}

syms <- function(...,paste=FALSE,sep=""){
  sep <- as.character(sep)
  args <- match.call(expand.dots=FALSE)$...
  parent <- parent.frame()
  args <- if(!length(args)) {
        tmp <- ls(parent.frame())
        i <- grep("^_",tmp)
        if(length(i)) tmp[-i] else tmp
      }
      else 
        lapply(args,function(arg){
          if(length(arg)>1) {
            if(is.call(arg)){
              if(as.character(arg[[1]]) %in% c("c","list")) as.character(arg[-1])
              else as.character(eval(arg,parent))
            }
            else as.character(arg)
          }
          else as.character(arg)
          })
  if(paste){
    res <- do.call("paste",c(args,list(sep=sep)))
    lapply(res,as.symbol)
  }
  else 
    lapply(args,as.symbol)
}

as.symbols <- function(x) lapply(x,as.symbol)


Pairs <- function(x,y=x){
  res <- outer(x,y,function(x,y)mapply(c,x,y,SIMPLIFY=FALSE))
  t(matrix(unlist(res[lower.tri(res)],recursive=FALSE),nrow=2)[2:1,])
}

toLatex.matrix <- function(object,
          show.titles=TRUE,
          show.vars=FALSE,
          show.xvar=show.vars,
          show.yvar=show.vars,
          digits=if(is.table(object)) 0 else getOption("digits"),
          format="f",
          useDcolumn=getOption("useDcolumn",TRUE),
          colspec=if(useDcolumn) paste("D{.}{",LaTeXdec,"}{",ddigits,"}",sep="") else "r",
          LaTeXdec=".",
          ddigits=digits,
          useBooktabs=getOption("useBooktabs",TRUE),
          toprule=if(useBooktabs) "\\toprule" else "\\hline\\hline",
          midrule=if(useBooktabs) "\\midrule" else "\\hline",
          cmidrule=if(useBooktabs) "\\cmidrule" else "\\cline",
          bottomrule=if(useBooktabs) "\\bottomrule" else "\\hline\\hline",
          toLatex.escape.tex=getOption("toLatex.escape.tex",FALSE),
          ...){
  n <- nrow(object)
  m <- ncol(object)
  d <- digits
  digits <- integer(m)
  digits[] <- d
  fo <- format
  format <- integer(m)
  format[] <- fo
  #print(digits)
  body <- array("",dim=dim(object))
  for(i in seq(along=digits)) {
    #print(digits[i])
    body[,i] <- formatC(object[,i],digits=digits[i],format=format[i])
    body[is.na(object)] <- ""
    }
  ans <- sub("([eE])([-+]?[0-9]+)","\\\\textrm{\\1}\\2",body)
  if(show.titles){
    if(length(rownames(object))){
      ans <- cbind(rownames(object),ans)
    }
    if(length(colnames(object))){
      header <- sapply(colnames(object),function(x)paste("\\multicolumn{1}{c}{",x,"}",sep=""))
      if(!show.yvar || !length(names(dimnames(object)))){
        if(length(rownames(object))){
          if(show.xvar && length(names(dimnames(object))))
              header <- c(names(dimnames(object))[1],header)
          else header <- c("",header)
        }
        header <- paste(header,collapse=" & ")
        header <- paste(header,"\\\\")
      }
      else {
        super.header <- paste("\\multicolumn{",m,"}{c}{",names(dimnames(object))[2],"}",sep="")
        if(length(rownames(object))){
          if(show.xvar && length(names(dimnames(object))))
              super.header <- c(names(dimnames(object))[1],super.header)
          else super.header <- c("",super.header)
          header <- c("",header)
          if(length(cmidrule))
            cmidrule <- paste(cmidrule,"{",2,"-",m+1,"}",sep="")
        }
        else if(length(cmidrule))
          cmidrule <- paste(cmidrule,"{",1,"-",m,"}",sep="")
        header <- paste(header,collapse=" & ")
        header <- paste(header,"\\\\")
        super.header <- paste(super.header,collapse=" & ")
        super.header <- paste(super.header,"\\\\")
        header <- c(super.header,cmidrule,header)
      }
    }
  }
  ans <- apply(ans,1,paste,collapse=" & ")
  ans <- paste(ans,"\\\\")
  if(show.titles && length(colnames(object)))
    ans <- c(
        toprule,
        header,
        midrule,
        ans,
        bottomrule
        )
  else {
    ans <- c(
        toprule,
        ans,
        bottomrule
        )
  }
  if(toLatex.escape.tex)
    ans <- LaTeXcape(ans)
  body.spec <- character(ncol(object))
  body.spec[] <- colspec
  if(show.titles && length(rownames(object)))
    tabspec <- c("l",body.spec)
  else
    tabspec <- body.spec
  tabspec <- paste(tabspec,collapse="")
  tabbegin <- paste("\\begin{tabular}{",tabspec,"}",sep="")
  tabend <- "\\end{tabular}"
  ans <- c(tabbegin,ans,tabend)
  structure(ans,class="Latex")
}

toLatex.default <- function(object,...) toLatex.matrix(as.matrix(object),...)

# Some idea, that I did not follow through with ...

eval.by <- function(f,expr) UseMethod("eval.by")
eval.by.default <- function(f,expr){
  parent <- parent.frame()
  expr <- substitute(expr)
  vars <- all.vars(expr)
  vars <- data.frame(sapply(vars,get,parent))
  vars <- split(vars,f)
  res <- lapply(vars,function(v)eval(expr,v,parent))
  unsplit(res,f)
}

# New stuff

getRows_ <- function(i,data) {
    data[i,,drop=FALSE]
}

as_factor <- function(x){
    if(inherits(x,"item.vector")){
        x <- include.missings(x)
        x <- if(is.ordinal(x)) as.ordered(x)
             else if(is.nominal(x)) as.factor(x)
             else as.vector(x)
    }
    if(!is.factor(x)){
        lev <- sort(unique(x),na.last=TRUE)
        i <- match(x,lev)
        x <- factor(i,labels=as.character(lev))
        return(x)
    }
    else {
        # if(any(is.na(x))){
        #     lev <- c(levels(x),"<NA>")
        #     nlev <- length(lev)
        #     x <- as.integer(x)
        #     x[is.na(x)] <- nlev
        #     x <- factor(x,labels=lev)
        # }
        x <- addNA(factor(x),ifany=TRUE)
        return(x)
    }
}

get_all_vars <- function(formula,data,inherits=TRUE){
    formula.vars <- all.vars(formula)
    data.vars <- names(data)
    if(all(formula.vars %in% data.vars))
        return(data[formula.vars])
    else {
        other.vars <- setdiff(formula.vars,data.vars)
        data.vars <- intersect(formula.vars,data.vars)
        res <- data[data.vars]
        other.vars <- mget(other.vars,inherits=inherits)
        res[names(other.vars)] <- other.vars
        return(res)
    }
}

Groups <- function(data,by,...) UseMethod("Groups")
    
Groups.data.frame <- function(data,by,...){
    by.vars <- get_all_vars(by,data)
    factors <- lapply(by.vars,as_factor)
    ii <- 1:nrow(data)
    ii <- tapply(ii,factors,I)
    n.ii <- sapply(ii,length)
    dim(n.ii) <- dim(ii)
    extra.attr <- list(groups=ii,
                       sizes=n.ii,
                       spec=by.vars[0,,drop=FALSE],
                       class=c(paste("grouped",class(data)[1],sep="."),"grouped.data",class(data)))
    attributes(data) <- c(attributes(data),extra.attr)
    data
}


Groups.data.set <- function(data,by,...){
    by.vars <- get_all_vars(by,data)
    factors <- lapply(by.vars,as_factor)
    data <- structure(data@.Data,
                      names=names(data),
                      row.names=data@row_names,
                      class="data.frame")
    ii <- 1:nrow(data)
    ii <- tapply(ii,factors,I)
    n.ii <- sapply(ii,length)
    dim(n.ii) <- dim(ii)
    extra.attr <- list(groups=ii,
                       sizes=n.ii,
                       spec=by.vars[0,,drop=FALSE],
                       class=c("grouped.data.set","grouped.data",class(data)))
    attributes(data) <- c(attributes(data),extra.attr)
    data
}

Groups.grouped.data <- function(data,by,...)Groups(recombine(data),by,...)

ngroups <- function(x){
    sum(attr(x,"sizes") > 0)
}  

eval1call <- function(data,call,envir){
    call$data <- data
    eval(call,envir=envir)
}

mklen <- function(x,n){
    y <- vector(length=n,mode=mode(x))
    y[] <- x
    return(y)
}

first <- function(x) x[[1]]
last <- function(x) x[[length(x)]]
set_first <- function(x,value){
    x[[1]] <- value
    x
}
set_last <- function(x,value){
    x[[length(x)]] <- value
    x
}

fill_dimnames2 <- function(dn,d){
    isn <- sapply(dn,length) == 0
    dn[isn] <- lapply(d[isn],seq.int)
    dn
}

within1.grouped.data <- function(ii,data,expr,N_,parent=parent.frame(),...){
    if(!length(ii)) return(NULL)
    encl <- new.env(parent=parent)
    n <- length(ii)
    assign("n_",n,envir=encl)
    assign("N_",N_,envir=encl)
    assign("i_",ii,envir=encl)
    e <- evalq(environment(),data[ii,,drop=FALSE],encl)
    eval(expr,e)
    l <- as.list(e,all.names=TRUE)
    # l[!vapply(l, is.null, NA, USE.NAMES = FALSE)]
    ll <- vapply(l,length,0,USE.NAMES=FALSE)
    ll1 <- which(ll==1)
    l[ll1] <- lapply(l[ll1],mklen,n)
    wrongl <- which(ll != n)
    l[wrongl] <- NULL
    return(l)                
}

rbind1col <- function(i,mat){
    do.call(c,mat[,i])
}

reorder1 <- function(x,ii){
    x[ii] <- x
    return(x)
}

within.grouped.data <- function(data,expr,recombine=FALSE,...){

    parent <- parent.frame()
    mc <- match.call()

    groups <- attr(data,"groups")
    sizes <- attr(data,"sizes")
    non_empty <- as.vector(sizes) > 0
    ii <- groups[non_empty]

    av <- all.vars(mc$expr)
    av <- intersect(av,names(data))

    res <- lapply(ii,within1.grouped.data,data[av],mc$expr,N_=nrow(data),parent=parent.frame())

    n <- length(res)
    m <- length(res[[1]])
    nms <- names(res[[1]])
    res <- unlist(res,recursive=FALSE)

    dim(res) <- c(m,n)

    res <- t(res)
    res <- lapply(1:m,FUN=rbind1col,mat=res)

    names(res) <- nms

    ii <- unlist(ii)

    res <- lapply(res,FUN=reorder1,ii=ii)
    res <- rev(res)

    data[names(res)] <- res
    
    if(recombine)
        return(recombine(data))
    else 
        return(data)
}

with1 <- function(ii,data,expr,N_,parent=parent.frame(),...){
    encl <- new.env(parent=parent)
    assign("n_",length(ii),envir=encl)
    assign("N_",N_,envir=encl)
    assign("i_",ii,envir=encl)
    eval(expr, data[ii,,drop=FALSE], enclos = encl)
}

add_tags <- function(expr,taggables=c("c","list","cbind","rbind")){
    is_taggable <- as.character(expr[[1]]) %in% taggables
    if(is_taggable){
        expr_1 <- expr[-1]
        n <- length(expr_1)
        if(!length(names(expr_1))){
            untagged <- rep(TRUE,n)
            names(expr) <- c("",letters[seq.int(n)])
        }
        else
            untagged <- !nzchar(names(expr_1))
        if(any(untagged)){
            names(expr)[-1][untagged] <- sapply(expr_1[untagged],deparse)
        }
    }
    expr
}

with.grouped.data <- function(data,expr,...){
    mc <- match.call()
    expr <- mc$expr
    groups <- attr(data,"groups")
    sizes <- attr(data,"sizes")
    non_empty <- as.vector(sizes) > 0
    av <- all.vars(mc$expr)
    av <- intersect(av,names(data))
    #with1. <- function(ii) eval(mc$expr,data[ii,av,drop=FALSE],enclos=parent.frame())
    expr <- add_tags(expr)
    
    ii <- groups[non_empty]
    res_ <- lapply(ii,with1,data[av],expr,N_=nrow(data),parent=parent.frame())
    #system.time(res_ <- lapply(ii,with1.))#,data[av],mc$expr)
    spec_ <- attr(data,"spec")

    res_len <- sapply(res_,length)

    if(length(unique(res_len))==1) {
        # All results have the same length
        res_len <- res_len[1]
        if(length(dim(res_[[1]]))){
            dim_res <- dim(res_[[1]])
            newdim <- c(dim_res,dim(groups))
            if(length(res_dimnames <- dimnames(res_[[1]]))){
                newdimnames <- c(res_dimnames,dimnames(groups))
            }
            else
                newdimnames <- c(rep(list(NULL),length(dim_res)),
                                 dimnames(groups))
        } else if(length(res_[[1]]) > 1){
            newdim <- c(length(res_[[1]]),dim(groups))
            newdimnames <- c(list(names(res_[[1]])),dimnames(groups))
        } else {
            newdim <- dim(groups)
            newdimnames <- dimnames(groups)
        }
        if(all(sapply(res_,is.atomic))) {
            if(all(non_empty)){
                res <- unlist(res_)
            }
            else {
                res <- rep(NA,length(groups)*res_len)
                res[rep(non_empty,each=res_len)] <- unlist(res_)
            }
            
        } else {
            res <- vector(mode="list",length=length(groups)*res_len)
            res[rep(non_empty,each=res_len)] <- unlist(res_,recursive=FALSE)
        }
        dim(res) <- newdim
        dimnames(res) <- fill_dimnames2(newdimnames,newdim)
    }
    else {
        res <- vector(mode="list",length=length(groups))
        res[non_empty] <- unlist(res_,recursive=FALSE)
        dim(res) <- dim(groups)
        dimnames(res) <- dimnames(groups)
    }

    if(length(dim(res)) >= 2){
        if(is.null(rownames(res))){
            if(nrow(res)==1) rownames(res) <- deparse(expr)
            else if( as.character(expr[[1]]) %in% c("c","cbind","rbind")
                    && length(expr[-1]) == nrow(res))
                rownames(res) <- paste(expr[-1])
            else if(as.character(expr[[1]]) %in% c("range"))
                rownames(res) <- c("Min","Max")
        }
    }
    
    class(res) <- "grouped.result"
    attr(res,"spec") <- spec_
    attr(res,"sizes") <- sizes
    if(is.atomic(res))
        class(res) <- c(class(res),"table")
    res
}

print.grouped.result <- function(x,...){
    attr(x,"spec") <- NULL
    attr(x,"sizes") <- NULL
    class(x) <- NULL #setdiff(class(x),"grouped.result")
    print.default(x)
}

withGroups <- function(data,by,expr,...) {
    data <- Groups(data=data,by=by)
    call_ <- match.call()
    call_[[1]] <- with.grouped.data
    call_$data <- data
    eval(call_,envir=parent.frame())
}
withinGroups <- function(data,by,expr,recombine=TRUE,...) {
    data <- Groups(data=data,by=by)
    call_ <- match.call()
    call_[[1]] <- within
    call_$data <- data
    call_$recombine <- recombine
    eval(call_,envir=parent.frame())
}

recombine <- function(x,...) UseMethod("recombine")

recombine.grouped.data.frame <- function(x,...) {
    attributes(x)[c("spec","sizes","groups")] <- NULL
    class(x) <- "data.frame"
    return(x)
}

recombine.grouped.data.set <- function(x,...) {
    attributes(x)[c("spec","sizes","groups")] <- NULL
    class(x) <- NULL
    return(new("data.set",x))
}


print.grouped.data.frame <- function(x,...){
  ngrps <- ngroups(x)  
  cat("\nGrouped data frame with",nrow(x), "observations in",ngrps,"groups of",ncol(x),"variables\n\n")
  print_frame_internal(x,max.obs=getOption("show.max.obs"),width=getOption("width"),...)
}

print.grouped.data.set <- function(x,...){
  ngrps <- ngroups(x)  
  cat("\nGrouped data frame with",nrow(x), "observations in",ngrps,"groups of",ncol(x),"variables\n\n")
  print_frame_internal(x,max.obs=getOption("show.max.obs"),width=getOption("width"),...)
}


as.data.set.grouped.data <- function(x,...){
    as.data.set(recombine(x),...)
}

as.data.frame.grouped.data <- function(x,...){
    as.data.frame(recombine(x),...)
}

setOldClass("grouped.data.set")
setOldClass("grouped.data.frame")
setMethod("as.data.set","grouped.data.set",as.data.set.grouped.data)
setMethod("as.data.set","grouped.data.frame",as.data.set.grouped.data)

muq <- function(x,sort=FALSE,drop.na=TRUE){
    u <- unique(x)
    if(drop.na)
        u <- na.omit(u)
    if(sort)
        u <- sort(u)
    structure(match(x,u),
              unique=u)
}

contract <- function(x,...) UseMethod("contract")

contract.data.frame <- function(x,
                                by=NULL,
                                weights=NULL,
                                name="Freq",
                                force.name=FALSE,
                                sort=FALSE,
                                drop.na=TRUE,
                                ...){
    w <- NULL
    if(!missing(by)){
        tmp <- as.list(seq_along(x))
        names(tmp) <- names(x)
        by <- eval(substitute(by),tmp,parent.frame())
        if(inherits(by,"formula")){
            if(length(by)>2){
                w <- eval(by[[2]],x,parent.frame())
                by <- by[-2]
            }
            by <- all.vars(by)
        }
    }
    if(!missing(weights)) {
        weights <- eval(substitute(weights),x,parent.frame())
        if(!is.numeric(weights)) stop("'weights' must be numeric")
        if(length(w) && is.numeric(w)) warning("Ignoring extraneous 'weights' argument")
        if(!length(w))
            w <- weights
    }
    if(!missing(by))
        x <- x[by]
        
    i <- lapply(x,muq,sort=sort,drop.na=drop.na)
    u <- lapply(i,attr,"unique")
    l <- sapply(u,length)
    n <- length(i)
    j <- i[[1]]
    if(n > 1){
        r <- l[1]
        for(k in 2:n){
            i.k <- i[[k]]
            j <- j + r*(i.k-1)
            j <- muq(j,sort=sort,drop.na=drop.na)
            r <- length(attr(j,"unique"))
        }
    }
    if(!length(w)){
        f <- tabulate(j)
    }
    else {
        if(length(w) != nrow(x))
            stop("'weights' argument has wrong length")
        if(any(is.na(w)))
            warning("NA weights treated as zero")
        if(is.factor(w)){
            w <- dummyMatrix(w)
            if(!missing(weights))
                w <- w*weights
        }
        if(drop.na && any(is.na(j))){
            keep <- !is.na(j)
            f <- rowsum.default(w[keep],j[keep],na.rm=TRUE)
        }
        else
            f <- rowsum.default(w,j,na.rm=TRUE)
    }
    if(drop.na)
        y <- x[!duplicated(j) & !is.na(j),,drop=FALSE]
    else
        y <- x[!duplicated(j),,drop=FALSE]
    if(sort){
        uj <- unique(j)
        y <- y[order(uj),,drop=FALSE]
    }
    rownames(y) <- 1:nrow(y)
    if(NCOL(f) > 1){
        if(force.name)
            colnames(f) <- paste(name,colnames(f),sep=".")
        nms <- c(colnames(y),colnames(f))
        y <- cbind(y,f)
        names(y) <- nms
    }
    else
        y[[name]] <- drop(f)
    y
}

contract.data.set <- contract.data.frame

dummyMatrix <- function(f){
    stopifnot(is.factor(f))
    l <- levels(f)
    res <- matrix(nrow=length(f),ncol=length(l))
    for(i in seq_along(l)){
        res[,i] <- as.integer(f==l[i])
    }
    colnames(res) <- l
    res
}

Mean <- function(x,na.rm=TRUE,...) mean(x,na.rm=na.rm,...)
Median <- function(x,na.rm=TRUE,...) median(x,na.rm=na.rm,...)
Min <- function(x,na.rm=TRUE,...) min(x,na.rm=na.rm,...)
Max <- function(x,na.rm=TRUE,...) max(x,na.rm=na.rm,...)
Weighted.Mean <- function(x, w, ..., na.rm = TRUE) weighted.mean(x,w,...,na.rm=na.rm)
Var <- function(x,na.rm=TRUE,...) var(x,na.rm=na.rm,...)
StdDev <- function(x,na.rm=TRUE,...) sd(x,na.rm=na.rm,...)

Cov <- function(x, y = NULL, use = "pairwise.complete.obs", ...) cov(x,y,use=use,...)
Cor <- function(x, y = NULL, use = "pairwise.complete.obs", ...) cor(x,y,use=use,...)
Range <- function(..., na.rm = TRUE, finite = FALSE) range(..., na.rm = FALSE, finite = FALSE)

Sapply <- function (X, FUN, ..., simplify = TRUE, USE.NAMES = TRUE){
    FUN <- match.fun(FUN)
    if(length(dim(X))){
        d.ans <- dim(X)
        dn.ans <- if(length(dimnames(X))) dimnames(X) else list(NULL) 
    } else {
        d.ans <- length(X)
        dn.ans <- if(USE.NAMES) list(names(X)) else list(NULL) 
    }
    if (!is.vector(X) || is.object(X))
        X <- as.list(X)
    answer <- lapply(X,FUN,...)
    if (USE.NAMES && is.character(X) && length(d.ans) == 1 && is.null(names(answer))) 
            dn.ans <- X
    if(simplify){
        dd.ans <- NULL
        ddn.ans <- list(NULL)
        DIMS <- lapply(answer,dim)
        ulDIMS <- unique(unlist(lapply(DIMS,length)))
        if(length(ulDIMS)==1 && ulDIMS > 0){
            DIMS <- array(unlist(DIMS),dim=c(ulDIMS,length(X)))
            common.dims <- rep(NA,ulDIMS)
            for(i in seq(nrow(DIMS))){
                uDIMS.i <- unique(DIMS[i,])
                if(length(uDIMS.i) == 1){
                    common.dims[i] <- uDIMS.i
                }
            }
            if(!any(is.na(common.dims))){
            dd.ans <- common.dims
            ddn.ans <- dimnames(answer[[1]])
            }
        }
        else {
            LEN <- unique(unlist(lapply(answer,length)))
            if(length(LEN)==1 && LEN > 1){
                dd.ans <- LEN
                ddn.ans <- list(names(answer[[1]]))
                }
        }
        if(!is.null(dd.ans)){
            if(is.null(ddn.ans)) ddn.ans <- rep(list(NULL),length(dd.ans))
            return(array(unlist(answer,recursive=FALSE),dim=c(dd.ans,d.ans),dimnames=c(ddn.ans,dn.ans)))
        }
        else
            return(array(unlist(answer,recursive=FALSE),dim=c(d.ans),dimnames=c(dn.ans)))
    }
    return(array(answer,dim=d.ans,dimnames=dn.ans))
}


Lapply <- function(X, FUN, ...)
     Sapply(X, FUN, ..., simplify = FALSE, USE.NAMES = FALSE)

queryOne <- function(x,
                pattern,
                fuzzy=TRUE,
                perl=TRUE,
                fixed=TRUE,
                ignore.case=TRUE,
                insertions=0.999999999,
                deletions=0,
                substitutions=0
                ){
    max.distance <- list(insertions=insertions,deletions=deletions,substitutions=substitutions)
    found <- if(fuzzy)
                agrep(pattern,x,
                  ignore.case=ignore.case,
                  max.distance=max.distance)
             else
                suppressWarnings(grep(pattern,x,
                  perl=perl,
                  fixed=fixed,
                  ignore.case=ignore.case))
    if(length(found)) x else NULL
}

queryList <- function(x,
                pattern,
                fuzzy=TRUE,
                perl=TRUE,
                fixed=TRUE,
                ignore.case=TRUE,
                insertions=0.999999999,
                deletions=0,
                substitutions=0
                ){
   res <- lapply(x,query,
    pattern=pattern,
    fuzzy=fuzzy,
    perl=perl,
    fixed=fixed,
    ignore.case=ignore.case,
    insertions=insertions,
    deletions=deletions,
    substitutions=substitutions
   )
   res <- res[sapply(res,length)>0]
   if(length(res)) res else NULL
}


setMethod("query","data.set",function(x,pattern,...)queryList(x,pattern,...))
setMethod("query","importer",function(x,pattern,...)queryList(x,pattern,...))

setMethod("query","item",function(x,pattern,...){
  annot <- queryOne(annotation(x),pattern,...)
  labs <- queryOne(labels(x),pattern,...)
  if(length(annot) && length(labs)) list(annotation=annot, labels=labs)
  else if (length(annot)) annot
  else if (length(labs)) labs
  else NULL
})

numericIfPossible <- function(x){
    if(is.atomic(x)) return(.numericIfPossible(x))
    else {
        res <- lapply(x,.numericIfPossible)
        attributes(res) <- attributes(x)
        return(res)
    }
}

.numericIfPossible <- function(x){
    if(is.numeric(x)) return(x)
    else if(is.character(x)) return(.Call("numeric_if_possible", x))
    else if(is.factor(x)) {
        levels <- .Call("numeric_if_possible",levels(x))
        if(is.numeric(levels)){
            return(levels[as.numeric(x)])
        } else return(x)
    }
    else return(x)
}

charTrans <- function(x, old = "", new = "", ...) UseMethod("charTrans")

charTrans.character <- function(x, old = "", new = "", ...) 
                          chartr(old = old, new = new, x = x)

charTrans.importer <- function(x, old = "", new = "", ...){
  x@.Data <- lapply(x@.Data, charTrans.item, old = old, new = new, ...)
  return(x)
}

charTrans.item <- function(x, old = "", new = "", ...){
  if(length(ann <- annotation(x)))
    annotation(x) <- charTrans.annotation(ann, old = old, new = new, ...)
  if(length(vl <- labels(x)))
    labels(x) <- charTrans.value.labels(vl, old = old, new = new, ...)
  return(x)
}

charTrans.value.labels <- function(x, old = "", new = "", ...){
  x@.Data <- chartr(x = x@.Data, old = old, new = new, ...)
  return(x)
}

charTrans.annotation <- function(x,old="",new="",...){
  x@.Data <- chartr(x = x@.Data, old = old, new = new, ...)
  return(x)
}

charTrans.data.set <- function(x,old="",new="",...){
  x@.Data <- lapply(x@.Data, charTrans.item, old = old, new = new, ...)
  return(x)
}

.memiscEnv <- new.env()
.SummaryTemplates <- list()
.SummaryStats <- list()

.CoefTemplates <- list()
.CoefTemplates$default <- c(est="($est:#)($p:*)",
                                          se="(($se:#))")
.CoefTemplates$stat <- c(est="($est:#)($p:*)",
                                      stat="(($stat:#))")
.CoefTemplates$all <- c(est="($est:#)($p:*)",
                                      se="(($se:#))",
                                      stat="(($stat:#))",
                                      p="(($p:#))"
                                      )
.CoefTemplates$all.nostar <- c(est="($est:#)",
                                      se="(($se:#))",
                                      stat="(($stat:#))",
                                      p="(($p:#))"
                                      )
.CoefTemplates$horizontal <- t(c(est="($est:#)($p:*)",
                                          se="(($se:#))"))
# .CoefTemplates$ci.se <- c(est="($est:3)",
#                                       se="(($se:#))",
#                                       ci="[($lwr:#);($upr:#)]")

.CoefTemplates$ci <- c(est="($est:#)",
                                        lwr="[($lwr:#)",
                                        upr="($upr:#)]"
                                        )


.CoefTemplates$ci.se <- c(est="($est:#)",
                                        se="(($se:#))",
                                        lwr="[($lwr:#)",
                                        upr="($upr:#)]"
                                        )

.CoefTemplates$ci.se.horizontal<- matrix(c(est="($est:#)",
                                        se="(($se:#))",
                                        lwr="[($lwr:#)",
                                        upr="($upr:#)]"
                                        ),ncol=2,nrow=2,byrow=TRUE,
                                        dimnames=list(
                                          c("est","ci"),
                                          c("est","se")
                                          ))

.CoefTemplates$ci.horizontal<- matrix(c(est="($est:#)",
                                           lwr="[($lwr:#)",
                                           upr="($upr:#)]"),
                                      ncol=3,nrow=1,byrow=TRUE,
                                      dimnames=list(
                                        c("est"),
                                        c("est","lwr","upr")
                                      ))


.CoefTemplates$ci.p <- c(est="($est:#)",
                                        p="(($p:#))",
                                        lwr="[($lwr:#)",
                                        upr="($upr:#)]"
                                        )

.CoefTemplates$ci.p.horizontal<- matrix(c(est="($est:#)",
                                        p="(($p:#))",
                                        lwr="[($lwr:#)",
                                        upr="($upr:#)]"
                                        ),ncol=2,nrow=2,byrow=TRUE,
                                        dimnames=list(
                                          c("est","ci"),
                                          c("est","se")
                                          ))


.SummaryTemplates$lm <-
  c(
          "R-squared"     = "($r.squared:f#)",
          "adj. R-squared" = "($adj.r.squared:f#)",
          sigma         = "($sigma:#)",
          F             = "($F:f#)",
          p             = "($p:f#)",
          "Log-likelihood"  = "($logLik:f#)",
          Deviance      = "($deviance:f#)",
          AIC           = "($AIC:f#)",
          BIC           = "($BIC:f#)",
          N             = "($N:d)"
  )
.SummaryStats$lm <- c("adj. R-squared","N")

.SummaryTemplates$glm <-
  c(
          "Aldrich-Nelson R-sq." = "($Aldrich.Nelson:f#)",
          "McFadden R-sq." = "($McFadden:f#)",
          "Cox-Snell R-sq." = "($Cox.Snell:f#)",
          "Nagelkerke R-sq."  = "($Nagelkerke:f#)",
          phi         = "($phi:#)",
          "Likelihood-ratio" = "($LR:f#)",
          p             = "($p:#)",
          "Log-likelihood" = "($logLik:f#)",
          Deviance      = "($deviance:f#)",
          AIC           = "($AIC:f#)",
          BIC           = "($BIC:f#)",
          N             = "($N:d)"
  )
.SummaryStats$glm <- c("Log-likelihood","N")

.SummaryTemplates$default <-
  c(
          "Aldrich-Nelson R-sq." = "($Aldrich.Nelson:f#)",
          "McFadden R-sq." = "($McFadden:f#)",
          "Cox-Snell R-sq." = "($Cox.Snell:f#)",
          "Nagelkerke R-sq."  = "($Nagelkerke:f#)",
          "Likelihood-ratio" = "($LR:f#)",
          p             = "($p:#)",
          "Log-likelihood" = "($logLik:f#)",
          Deviance      = "($deviance:f#)",
          AIC           = "($AIC:f#)",
          BIC           = "($BIC:f#)",
          N             = "($N:d)"
  )

.SummaryStats$default <- c("Log-likelihood","N")

.SummaryTemplates$mer <- .SummaryTemplates$lmer
  c(
          "Log-likelihood" = "($logLik:f#)",
          Deviance      = "($deviance:f#)",
          AIC           = "($AIC:f#)",
          BIC           = "($BIC:f#)",
          N             = "($N:d)"
  )


assign("SummaryTemplates",.SummaryTemplates, envir=.memiscEnv)
assign("CoefTemplates",.CoefTemplates, envir=.memiscEnv)
 

getSummary.clm <- function(obj,
                           alpha=.05,
                           ...){
  
  smry <- summary(obj)
  N <- if(length(weights(obj))) sum(weights(obj))
  else smry$nobs
  
  prmtable <- coef(smry)

  ## Move threshold parameters to end
  coefnames <- names(smry$beta)
  parnames <- rownames(prmtable)
  threshnames <- setdiff(parnames,coefnames)
  
  lower <- qnorm(p=alpha/2,mean=prmtable[,1],sd=prmtable[,2])
  upper <- qnorm(p=1-alpha/2,mean=prmtable[,1],sd=prmtable[,2])
  
  prmtable <- cbind(prmtable,lower,upper)
  
  colnames(prmtable) <- c("est","se","stat","p","lwr","upr")

  dn <- list(
    rownames(prmtable),
    c("est","se","stat","p","lwr","upr"),
    names(obj$model)[1]
  )
  dim(prmtable) <- c(dim(prmtable)[1],dim(prmtable)[2],1)
  dimnames(prmtable) <- dn  

  coef <- prmtable[coefnames,,,drop=FALSE]
  thresh <- prmtable[threshnames,,,drop=FALSE]

  ans <- list(coef = coef,
              Thresholds = thresh)
  
  null.model <- update(obj, .~1)
  
  ll <- logLik(obj)
  ll0 <- logLik(null.model)
  
  LR <- 2*(ll-ll0)
  df <- null.model$df.residual - smry$df.residual
  
  dev <- -2*ll
  
  if(df > 0){
    p <- pchisq(LR,df,lower.tail=FALSE)
    L0.pwr <- exp(2*ll0/N)
    
    Aldrich.Nelson <- LR/(LR+N)
    McFadden <- 1 - dev/(-2*ll0)
    Cox.Snell <- 1 - exp(-LR/N)
    Nagelkerke <- Cox.Snell/(1-L0.pwr)
  }
  else {
    LR <- NA
    df <- NA
    p <- NA
    Aldrich.Nelson <- NA
    McFadden <- NA
    Cox.Snell <- NA
    Nagelkerke <- NA
  }
  
  AIC <- AIC(obj)
  BIC <- AIC(obj,k=log(N))
  sumstat <- c(
    LR             = LR,
    df         = df,
    p             = p,
    logLik        = ll,
    deviance      = dev,
    Aldrich.Nelson = Aldrich.Nelson,
    McFadden      = McFadden,
    Cox.Snell       = Cox.Snell,
    Nagelkerke    = Nagelkerke,
    AIC           = AIC,
    BIC           = BIC,
    N             = N
  )

  ans <- c(ans,
           list(sumstat=sumstat,
                contrasts=obj$contrasts,
                xlevels=smry$xlevels,
                call=obj$call))
  return(ans)
}

getSummary.clmm <- function(obj,
                           alpha=.05,
                           ...){
  
  smry <- summary(obj)
  N <- if(length(weights(obj))) sum(weights(obj))
       else nrow(obj$model)
  
  prmtable <- coef(smry)

  ## Move threshold parameters to end
  coefnames <- names(smry$beta)
  parnames <- rownames(prmtable)
  threshnames <- setdiff(parnames,coefnames)
  
  lower <- qnorm(p=alpha/2,mean=prmtable[,1],sd=prmtable[,2])
  upper <- qnorm(p=1-alpha/2,mean=prmtable[,1],sd=prmtable[,2])
  
  prmtable <- cbind(prmtable,lower,upper)
  
  colnames(prmtable) <- c("est","se","stat","p","lwr","upr")

  dn <- list(
    rownames(prmtable),
    c("est","se","stat","p","lwr","upr"),
    names(obj$model)[1]
  )
  dim(prmtable) <- c(dim(prmtable)[1],dim(prmtable)[2],1)
  dimnames(prmtable) <- dn  

  coef <- prmtable[coefnames,,,drop=FALSE]
  thresh <- prmtable[threshnames,,,drop=FALSE]

  ans <- list(coef = coef,
              Thresholds = thresh)

  varcor <- lapply(obj$ST,tcrossprod)

  VarPar <- NULL
    
  for(i in seq_along(varcor)){
    vc.i <- varcor[[i]]
    lv.i <- names(varcor)[i]
    vr.i <- diag(vc.i)
    cv.i <- vc.i[lower.tri(vc.i)]
    nms.i <- rownames(vc.i)
    nms.i <- gsub("(Intercept)","1",nms.i,fixed=TRUE)
    vrnames.i <- paste0("Var(~",nms.i,"|",lv.i,")")
    cvnames.i <- t(outer(nms.i,nms.i,FUN=paste,sep=":"))
    cvnames.i <- cvnames.i[lower.tri(cvnames.i)]
    if(length(cvnames.i))
      cvnames.i <- paste0("Cov(~",cvnames.i,"|",lv.i,")")
    vp.i <- matrix(NA,nrow=length(vr.i)+length(cv.i),ncol=6)
    vp.i[,1] <- c(vr.i,cv.i)
    dim(vp.i) <- c(dim(vp.i),1)
    dimnames(vp.i) <- list(c(vrnames.i,cvnames.i),
                           c("est","se","stat","p","lwr","upr"),
                           names(obj$model)[1])
    VarPar <- c(VarPar,structure(
                           list(vp.i),
                           names=lv.i))
 }
  ans <- c(ans,list(Variances=VarPar))
  
  # null.model <- update(obj, .~1)
  
  ll <- logLik(obj)
  # ll0 <- logLik(null.model)
  
  # LR <- 2*(ll-ll0)
  # df <- null.model$df.residual - smry$df.residual
  
  dev <- -2*ll
  
  # if(df > 0){
  #   p <- pchisq(LR,df,lower.tail=FALSE)
  #   L0.pwr <- exp(2*ll0/N)
  #   Aldrich.Nelson <- LR/(LR+N)
  #   McFadden <- 1 - dev/(-2*ll0)
  #   Cox.Snell <- 1 - exp(-LR/N)
  #   Nagelkerke <- Cox.Snell/(1-L0.pwr)
  # }
  # else {
    df <- NA
    LR <- NA
    p <- NA
    Aldrich.Nelson <- NA
    McFadden <- NA
    Cox.Snell <- NA
    Nagelkerke <- NA
  # }
  
  AIC <- AIC(obj)
  # BIC <- AIC(obj,k=log(N))
  sumstat <- c(
    # LR            = LR,
    # df            = df,
    # p             = p,
    logLik        = ll,
    deviance      = dev,
    # Aldrich.Nelson = Aldrich.Nelson,
    # McFadden      = McFadden,
    # Cox.Snell       = Cox.Snell,
    # Nagelkerke    = Nagelkerke,
    AIC           = AIC,
  #   BIC           = BIC,
    N             = N
  )

  ans <- c(ans,
           list(sumstat=sumstat,
                contrasts=obj$contrasts,
                xlevels=smry$xlevels,
                call=obj$call))
  return(ans)
}

## Not exported yet ...

inflate <- function(df,w){
  w <- eval(substitute(w),df,parent.frame())
  i <- seq(nrow(df))
  i <- rep(i,as.integer(w))
  df[i,]
}


# test.inflate <- function(){
#   testdata <- data.frame(x=c(1,2,3),y=c(4,5,6),weights=c(4,8,12))
#   print(testdata)
#   print(
#     inflate(testdata,weights)
#     )
# }
# 
# test.expand()

collect <- function(...,names=NULL,inclusive=TRUE) {
  if(!length(match.call(expand.dots=FALSE)$...)) return(NULL)
  else UseMethod("collect",..1)
}

collect.default <- function(...,names=NULL,inclusive=TRUE){
  if(!is.atomic(..1)) stop("cannot handle arguments of mode ",mode(..1))
  args <- list(...)
  subst <- substitute(list(...))
  if(length(names)) {
    if(length(names)!=length(args)) stop("names argument has wrong length")
  }
  else {
    if(length(names(args))) names <- names(args)
    else {
      names <- sapply(lapply(subst[-1],deparse),paste,collapse=" ")
    }
  }
  modes <- sapply(args,mode)
  if(!inclusive){
    common.names <- reduce(lapply(args,names),intersect)
    args <- lapply(args,function(x)x[common.names])
  }
  if(any(modes!=mode(..1))){
    for(i in seq_along(args))
      mode(args[[i]]) <- mode(..1)
  }
  res <- clct.vectors(args)
  colnames(res) <- names
  res
}

collect.matrix <- collect.array <- function(...,names=NULL,inclusive=TRUE){
  if(!is.atomic(..1)) stop("cannot handle arguments of mode ",mode(..1))
  args <- list(...)
  subst <- substitute(list(...))
  if(length(names)) {
    if(length(names)!=length(args)) stop("names argument has wrong length")
  }
  else {
    if(length(names(args))) names <- names(args)
    else {
      names <- sapply(lapply(subst[-1],deparse),paste,collapse=" ")
    }
  }
  modes <- sapply(args,mode)
  if(any(modes!=mode(..1))){
    for(i in seq_along(args))
      mode(args[[i]]) <- mode(..1)
  }
  dims <- unique(sapply(args,function(x)length(dim(x))))
  if(length(dims)>1) stop("array dims do not match")
  
  if(!inclusive){
    name.select.args <- vector(mode="list",length=dims)
    for(i in 1:dims){
      common.names <- reduce(lapply(args,function(x)dimnames(x)[[i]]),intersect)
      name.select.args[[i]] <- common.names
    }
    for(i in seq_along(args))
      args[[i]] <- do.call("[",c(args[i],name.select.args))
  }
  
  res <- clct.arrays(args)
  dimnames(res)[[length(dim(res))]] <- names
  res
}

collect.table <- function(...,names=NULL,sourcename=".origin",fill=0){
  if(!is.atomic(..1)) stop("cannot handle arguments of mode ",mode(..1))
  args <- list(...)
  subst <- substitute(list(...))
  if(length(names)) {
    if(length(names)!=length(args)) stop("names argument has wrong length")
  }
  else {
    if(length(names(args))) names <- names(args)
    else {
      names <- sapply(lapply(subst[-1],deparse),paste,collapse=" ")
    }
  }
  one.dim <- all(sapply(args,function(x)length(dim(x)))==1)
  if(one.dim){
    args <- lapply(args,c)
    res <- clct.vectors(args)
    common.names <- "Freq"
  }
  else {
    common.names <- reduce(lapply(args,function(x)names(dimnames(x))),intersect)
    if(!length(common.names)) return(NULL)
    args <- lapply(args,function(x){
      ndn <- names(dimnames(x))
      mdims <- which(ndn %in% common.names)
      margin.table(x,mdims)
    })
    res <- clct.arrays(args)
  }
  res[is.na(res)] <- fill
  dimnames(res)[[length(dim(res))]] <- names
  names(dimnames(res)) <- c(common.names,sourcename)
  as.table(res)
}

collect.data.frame <- function(...,
  names=NULL,inclusive=TRUE,fussy=FALSE,warn=TRUE,
  detailed.warnings=FALSE,use.last=FALSE,
  sourcename=".origin"){
  args <- list(...)
  subst <- substitute(list(...))
  if(length(names)) {
    if(length(names)!=length(args)) stop("names argument has wrong length")
  }
  else {
    if(length(names(args))) names <- names(args)
    else {
      names <- sapply(lapply(subst[-1],deparse),paste,collapse=" ")
    }
  }
  all.vars <- lapply(args,names)
  common.vars <- reduce(all.vars,intersect)
  all.vars <- reduce(all.vars,union)
  other.vars <- setdiff(all.vars,common.vars)
  source <- rep(seq_along(args),sapply(args,nrow))
  nrow.items <- sapply(args,nrow)
  nrow.total <- sum(nrow.items)
  ix <- split(seq_len(nrow.total),source)
  res <- lapply(common.vars,function(var){
                vecs <- lapply(args,function(x)x[[var]])
                collOne(vecs,source=source,nrow.items=nrow.items,varname=var,fussy=fussy,
                        detailed.warnings=detailed.warnings,use.last=use.last)
                })
  names(res) <- common.vars
  if(inclusive){
    res1 <- lapply(other.vars,function(var){
                  vecs <- lapply(args,function(x)x[[var]])
                  collOne(vecs,source=source,nrow.items=nrow.items,varname=var,fussy=fussy,
                          detailed.warnings=detailed.warnings,use.last=use.last)
                  })
    names(res1) <- other.vars
    res <- c(res,res1)
  }
  res[[sourcename]] <- factor(source,labels=names)
  as.data.frame(res)
}

collect.data.set <- function(...,
  names=NULL,inclusive=TRUE,fussy=FALSE,warn=TRUE,
  detailed.warnings=FALSE,use.last=FALSE,
  sourcename=".origin"){
  args <- list(...)
  subst <- substitute(list(...))
  if(length(names)) {
    if(length(names)!=length(args)) stop("names argument has wrong length")
  }
  else {
    if(length(names(args))) names <- names(args)
    else {
      names <- sapply(lapply(subst[-1],deparse),paste,collapse=" ")
    }
  }
  all.vars <- lapply(args,names)
  common.vars <- reduce(all.vars,intersect)
  all.vars <- reduce(all.vars,union)
  other.vars <- setdiff(all.vars,common.vars)
  source <- rep(seq_along(args),sapply(args,nrow))
  nrow.items <- sapply(args,nrow)
  nrow.total <- sum(nrow.items)
  ix <- split(seq_len(nrow.total),source)
  res <- lapply(common.vars,function(var){
                vecs <- lapply(args,function(x)x[[var]])
                collOne(vecs,source=source,nrow.items=nrow.items,varname=var,fussy=fussy,
                        detailed.warnings=detailed.warnings,use.last=use.last)
                })
  names(res) <- common.vars
  if(inclusive){
    res1 <- lapply(other.vars,function(var){
                  vecs <- lapply(args,function(x)x[[var]])
                  collOne(vecs,source=source,nrow.items=nrow.items,varname=var,fussy=fussy,
                          detailed.warnings=detailed.warnings,use.last=use.last)
                  })
    names(res1) <- other.vars
    res <- c(res,res1)
  }
  res[[sourcename]] <- factor(source,labels=names)
  as.data.set(res)
}


toArray <- function(x){
  if(!length(dim(x))) as.matrix(x)
  else x
}

collect.by <- function(...,inclusive=TRUE,names=NULL){
  dn <- dimnames(..1)
  dd <- dim(..1)
  res <- if(any(sapply(..1,is.array)))
          clct.arrays(lapply(..1,toArray))
         else
          clct.vectors(..1)
  dd <- c(dim(res)[-length(dim(res))],dd)
  dn <- c(dimnames(res)[-length(dim(res))],dn)
  dim(res) <- dd
  dimnames(res) <- dn
  res
}


reduce <- function(x,FUN,...){
  lx <- length(x)
  if(lx<2) return(FUN(x[[1]],...))
  FUN <- match.fun(FUN)
  y <- x[[1]]
  for(i in 2:lx)
    y <- FUN(y,x[[i]],...)
  return(y)
}

collOne <- function(vecs,source,nrow.items,varname,fussy=FALSE,warn=TRUE,
                    detailed.warnings=FALSE,use.last=FALSE){
  lens <- sapply(vecs,length)
  has.els <- lens > 0
  checkAttribs(vecs[which(has.els)],varname=varname,fussy=fussy,
                    detailed.warnings=FALSE)
#   browser()
  if(use.last)
      use.attribs <- last(which(has.els))
  else
      use.attribs <- first(which(has.els))
  res <- lapply(seq_along(nrow.items),
                  function(i)
                    if(has.els[i]) vecs[[i]]
                    else rep(NA,nrow.items[i])
                  )
  res <- unsplit(res,source)
  attributes(res) <- attributes(vecs[[use.attribs]])
  if(isS4(vecs[[use.attribs]]))
      res <- asS4(res)
  res
}


checkAttribs <- function(x,varname,fussy=FALSE,warn=TRUE,
                         detailed.warnings=FALSE){
  modes <- sapply(x,mode)
  if(length(unique(modes))>1) stop("modes for ",sQuote(varname)," do not match",call.=FALSE)
  attribs <- lapply(x,attributes)
  attribs.identical <- namedlists.pairwise.identical(attribs)
  if(!attribs.identical && detailed.warnings && (warn || fussy)){
      cat("\n==========================================")
      cat("\nVariable: ",varname,)
      cat("\n------------------------------------------")
      cat("\n")
      print(attribs)
  }
  if(!attribs.identical && fussy) stop("attributes for ",sQuote(varname)," do not match",call.=FALSE)
  if(!attribs.identical && warn) warning("attributes for ",sQuote(varname)," do not match",call.=FALSE)
}

namedlists.pairwise.identical <- function(x){
  if(length(x) == 1) return(TRUE)
  x[[1]] <- x[[1]][suppressWarnings(sort(names(x[[1]])))]
  res <- TRUE
  if(length(x)>1){
    for(i in 2:length(x))
      res <- res && identical(x[[1]],x[[i]][suppressWarnings(sort(names(x[[i]])))])
    }
  res
}

fill_dimnames <- function(x){
  d <- length(dim(x))
  dd <- dim(x)
  dn <- dimnames(x)
  if(length(dn)){
    for(i in 1:d)
      if(!length(dn[[i]])) dn[[i]] <- seq_len(dd[i])
  }
  else {
    dn <- list()
    for(i in 1:d)
      dn[[i]] <- seq_len(dd[i])
  }
  dimnames(x) <- dn
  x
}

clct.arrays <- function(x){
  dims <- Sapply(x,dim)
  n.x <- length(x)
  if(is.list(dims)) stop("Dimensions do not match")
  x <- lapply(x,fill_dimnames)
  cdimnames <- vector(mode="list",length=nrow(dims))
  for(i in 1:nrow(dims)){
    for(j in 1:n.x){
      if(j==1) cdimnames[[i]] <- dimnames(x[[1]])[[i]]
      else cdimnames[[i]] <- union(cdimnames[[i]], dimnames(x[[j]])[[i]])
    }
  }
  cdims <- sapply(cdimnames,length)
  cdims <- c(cdims,n.x)
  res <- array(NA,dim=cdims,dimnames=c(cdimnames,list(NULL)))
#   str(res)
  for(j in 1:n.x){
    rhs <- list(as.symbol("["),as.call(list(as.symbol("[["),as.symbol("x"),j)))
    rhs <- as.call(rhs)
    lhs <- c(as.symbol("[<-"),as.symbol("res"),dimnames(x[[j]]),list(j))
    ex <- as.call(c(lhs,rhs))
    res <- eval(ex)
  }
  res
}

clct.vectors <- function(x){
  n.x <- length(x)
  for(j in 1:n.x){
    if(j==1) cnames <- names(x[[1]])
    else cnames <- union(cnames,names(x[[j]]))
    }
  if(length(cnames)){
    res <- matrix(NA,nrow=length(cnames),ncol=n.x)
    rownames(res) <- cnames
    for(j in 1:n.x){
      res[names(x[[j]]),j] <- x[[j]]
      }
    }
  else{
    lenx <- sapply(x,length)
    if(length(unique(lenx))>1){
      maxlen <- max(lenx)
      x <- lapply(x,"length<-",maxlen)
    }
    res <- do.call(cbind,x)
  }
  if(length(names(x))) colnames(res) <- names(x)
  res
}


clct.lists <- function(x){
  ncol <- length(x)
  lnames <- lapply(x,names)
  unames <- unique(unlist(lnames))
  nrow <- length(unames)
  res <- matrix(list(),nrow,ncol)
  dimnames(res) <- list(unames,names(x))
  for(i in seq_along(x))
    res[names(x[[i]]),i] <- x[[i]]
  res
}

ftable_format_stdstyle <- c(
  "padding-top"="3px",
  "padding-bottom"="3px",
  "padding-left"="0.5ex",
  "padding-right"="0.5ex",
  "margin-top"="0px",
  "margin-bottom"="0px",
  "border-style"="none",
  "border-width"="0px"
)


format_html.ftable <- function(x,
                               show.titles=TRUE,
                               digits=0,
                               format="f",
                               toprule=2,midrule=1,bottomrule=2,
                               split.dec=TRUE,
                               style=ftable_format_stdstyle,
                               margin="2ex auto",
                               ...){
  
  first.col <- c("padding-left"="0.3em")
  toprule <- c("border-top"=paste0(midrule,"px solid"))
  bottomrule <- c("border-bottom"=paste0(midrule,"px solid"))
  midrule_above <- c("border-top"=paste0(midrule,"px solid"))
  midrule <- c("border-bottom"=paste0(midrule,"px solid"))
  align.right <- c("text-align"="right")  
  align.left <- c("text-align"="left")  
  align.center <- c("text-align"="center")
  lrpad <- c("padding-left"="0.3em","padding-right"="0.3em")
  row_style <- c("border-style"="none")
  table_style <- c("border-collapse"="collapse" ,"border-style"="none")
  
  row.vars <- attr(x,"row.vars")
  col.vars <- attr(x,"col.vars")
  n.row.vars <- length(row.vars)
  n.col.vars <- length(col.vars)
  n <- nrow(x)
  m <- ncol(x)
  d <- digits
  digits <- integer(m)
  digits[] <- d
  
  fo <- format
  format <- integer(m)
  format[] <- fo
  
  body <- array("",dim=dim(x))
  for(i in seq(along=digits)) {
    #print(digits[i])
    body[,i] <- formatC(x[,i],digits=digits[i],format=format[i])
  }
  
  body <- array(trimws(body),dim=dim(x))
  if(getOption("html.use.ampersand",FALSE))
    body[] <- gsub("-","&minus;",body[],fixed=TRUE)
  else
    body[] <- gsub("-","\u2212",body[],fixed=TRUE)
  
  if(split.dec){
    tmp <- spltDec(body)
    body <- html_td_spltDec(tmp,style=css(style))
    dim(body) <- dim(x)
    colspan <- 3L
  }
  else {
    body <- html_td(body,vectorize=TRUE,style=css(style))
    dim(body) <- dim(x)
    colspan <- 1L
  }
  
  leaders <- array(list(),dim=c(n,n.row.vars))
  if(show.titles)
    leaders <- cbind(leaders,"")

  mm <- 1
  for(j in rev(1:n.row.vars)){
    rv <-row.vars[[j]]
    nrv <- length(rv)
    nn <- n/mm
    i <- (1:nn)*mm - mm + 1
    leaders[i,j] <- rv
    mm <- mm*nrv
  }
  for(i in 1:n){
    lstyle <- style
    lstyle1 <- upd_vect(lstyle,first.col)
    lstyle2 <- lstyle
    
    if(ncol(leaders)>1)
      lstyle <- c(lstyle1,rep(lstyle2,ncol(leaders)-1))
    else
      lstyle <- lstyle1
    
    leaders[i,] <- html_td(leaders[i,],style=css(lstyle),
                           vectorize=TRUE) 
  }
  
  body <- cbind(leaders,body)
  nn <- nrow(body)
  body[nn,] <- lapply(body[nn,],setStyle,bottomrule)

  body <- as.html_group(apply(body,1,html_tr,style=as.css(row_style)))
  
  header <- list()
  mm <- 1
  for(i in rev(1:n.col.vars)){
    cv <- col.vars[[i]]
    ncv <- length(cv)
    if(split.dec)
      attribs <- list(colspan=mm*3)
    else
      attribs <- list(colspan=mm)
    mm <- mm*ncv
    cv <- rep(cv,m%/%mm)
    
    hstyle <- upd_vect(style,align.center,lrpad)

    if(show.titles){
      if(n.col.vars == 1){
        htmp1 <- html_td(c(names(row.vars),""),
                         style=css(upd_vect(hstyle,align.left)),
                         vectorize=TRUE)
      }
      else {
        if(i == n.col.vars){
          htmp1 <- html_td(c(names(row.vars),paste0(names(col.vars)[i],":")),
                         style=css(upd_vect(hstyle,align.left)),
                         vectorize=TRUE)
        }
        else
          htmp1 <- html_td(c(rep("",n.row.vars),paste0(names(col.vars)[i],":")),
                         style=css(upd_vect(hstyle,align.left)),
                         vectorize=TRUE)
      }      
    }
    else 
      htmp1 <- html_td(rep("",ncol(leaders)),style=css(hstyle),
                       vectorize=TRUE)
    
    if(i==n.col.vars)
      attribs$style <- css(hstyle)
    else
      attribs$style <- css(upd_vect(hstyle,midrule))
      
    htmp2 <- setAttribs(html_td(cv,vectorize=TRUE),attribs)
    header <- c(list(c(htmp1,htmp2)),header)
  }
  if(show.titles && n.col.vars == 1){
    if(nzchar(names(col.vars))){
      hstyle <- upd_vect(style,lrpad)
      htmp1 <- html_td(rep("",ncol(leaders)),
                       style=css(hstyle),
                       vectorize=TRUE)
      colspan <- ncol(x)
      if(split.dec) 
        colspan <- colspan*3
      attribs <- list(colspan=colspan,
                      style=css(upd_vect(hstyle,align.center,midrule)))
      htmp2 <- setAttribs(html_td(names(col.vars),vectorize=TRUE),attribs)
      header <- c(list(c(htmp1,htmp2)),header)
    }
  }
  header[[1]] <- lapply(header[[1]],setStyle,toprule)
  lh <- length(header)
  header[[lh]] <- lapply(header[[lh]],setStyle,midrule)
  
  header <- html_tr(header,vectorize=TRUE,style=as.css(row_style))

  if(length(margin))
    table_style <- c(table_style,margin=margin)
  ans <- html_table(c(header,body),class="ftable",
                                   style=as.css(table_style))
  
  ans <- as.character(ans)
  return(ans)
}

setMethod("sample", "data.frame", function (x, size, replace = FALSE, prob = NULL){
    if (missing(size))
        size <- nrow(x)
    ii <- sample.int(nrow(x), size, replace, prob)
    x[ii,,drop=FALSE]
})

setMethod("sample", "data.set", function (x, size, replace = FALSE, prob = NULL){
    if (missing(size))
        size <- nrow(x)
    ii <- sample.int(nrow(x), size, replace, prob)
    x[ii,,drop=FALSE]
})

setMethod("sample", "importer", function (x, size, replace = FALSE, prob = NULL){
    if (missing(size))
        size <- nrow(x)
    ii <- sample.int(nrow(x), size, replace, prob)
    x[ii,,drop=FALSE]
})

allequal <- function(x) length(unique(x)) == 1

get.substr <- function(pattern, text, perl = FALSE,
        fixed = FALSE, useBytes = FALSE){
        if(fixed)
          rr <- gregexpr(pattern,text,fixed=TRUE,perl=FALSE,useBytes=useBytes)
        else
          rr <- gregexpr(pattern,text,perl=perl,useBytes=useBytes)
        positions <- lapply(rr,function(rr){
          if(rr[1]<0) return(NULL)
          len <- attr(rr,"match.length")
          sapply(1:length(rr),function(i){
          start <- rr[i]
          stop <- rr[i] + len[i] - 1
          c(start=start,stop=stop)
          })})
        ans <- lapply(1:length(text),function(i){
          txt <- text[i]
          pos <- positions[[i]]
          if(!length(pos)) return(NULL)
          sapply(1:NCOL(pos),function(j){
            substr(txt,start=pos["start",j],stop=pos["stop",j])
          })
        })
        structure(ans,positions=positions)
        }

get.oneSubstr <- function(pattern,text){
  if(length(text)>1) {
    warning("only first element used")
    text <- text[1]
    }
  start <- regexpr(pattern,text)
  if(start < 0) return(NULL)
  stop <- start + attr(start,"match.length") - 1
  substr(text,start=start,stop=stop)
}


formatSigSymbols <- function(x,signif.symbols=getOption("signif.symbols")){
  if(length(x)>1) {
    warning("only first element used")
    x <- x[1]
    }
  signif.symbols <- sort(signif.symbols)
  sel <- x <= signif.symbols
  if(!any(sel)) return("")
  sel <- which(signif.symbols == min(signif.symbols[sel]))
  return(names(signif.symbols)[sel])
}

formatOne <- function(x,spec,format="f",
                      digits=min(3,getOption("digits")),
                      signif.symbols=getOption("signif.symbols")
                      ){
  if(length(x)>1) {
    warning("only first element used")
    x <- x[1]
    }
  if(is.na(x)) return("")
  if(length(spec)>1) {
    warning("only first spec used")
    spec <- spec[1]
    }
  if(is.character(x)) return(x)
  if(!is.numeric(x)) {
    x <- paste(x,collapse=" ")
    if(nchar(x)>20)
      x <- paste(substr(x,1,20),"...")
    return(x)
    }
  format.arg <- format
  digits.arg <- digits
  format <- get.oneSubstr("[d-gEG]+",spec)
  digits <- as.numeric(get.oneSubstr("[0-9]+",spec))
  hash <- length(get.oneSubstr("[#]",spec)) > 0
  star <- length(get.oneSubstr("[*]",spec)) > 0
  if(star) return(formatSigSymbols(x,signif.symbols))
  if(hash){
    if(length(digits))
      digits <- min(digits,digits.arg)
    else
      digits <- digits.arg
  }
  if(!length(digits)) digits <- digits.arg
  if(!length(format)) format <- format.arg
  if(!(format %in% c("d","e","f","g","fg","E","G","s"))) stop("illegal format specifier")
  return(formatC(x,digits=digits,format=format,width=-1))
}

formatVec <- function(x,formats,
                      default="f",
                      digits=min(3,getOption("digits")),
                      signif.symbols=getOption("signif.symbols")){
  i <- 1:length(formats)
  
  res <- sapply(i,function(i) if(length(formats[[i]]))
    sapply(formats[[i]],formatOne,
           x=x,
           format=default,
           digits=digits,
           signif.symbols = signif.symbols)
      else NULL
      )
}

get.format <- function(x){
  if(!length(x)) return(NULL)
  x <- strsplit(x,":",fixed=TRUE)
  sapply(x,function(x)sub(")","",x[2],fixed=TRUE))
}


applyTemplate <- function(x,template,
                            float.style=getOption("float.style"),
                            digits=min(3,getOption("digits")),
                            signif.symbols=getOption("signif.symbols")){
    if(is.list(x)){
        y <- list()
        nms <- names(x)
        for(i in seq_along(x)){
            n_i <- nms[i]
            if(n_i==""){
                y[[i]] <- applyTemplate(x[[i]],
                                        template=template,
                                        float.style=float.style,
                                        digits=digits,
                                        signif.symbols=signif.symbols)
            }
            else {
                if(n_i %in% names(template)){
                    tmpl <- template[n_i]
                    tmpl <- sub(n_i,"1",tmpl)
                    y[[i]] <-as.matrix(unlist(lapply(x[[i]],
                                              applyTemplate,
                                              template=unname(tmpl),
                                              float.style=float.style,  
                                              digits=digits,
                                              signif.symbols=signif.symbols)))
                }
            }
        }
        do.call(rbind,y)
    }
    else {
        applyTemplate1(x, template=template,
                          float.style=float.style,
                          digits=digits,
                          signif.symbols=signif.symbols)
    }
}

applyTemplate1 <- function(x,template,float.style=getOption("float.style"),
                      digits=min(3,getOption("digits")),
                      signif.symbols=getOption("signif.symbols")){
   template <- as.matrix(template)
   if(is.numeric(x))
    patterns  <- paste("\\(\\$",1:length(x),":[d-gEG]*[0-9]*[#]?[*]?\\)",sep="")
   else
    patterns  <- paste("\\(\\$",1:length(x),"\\)",sep="")
   names(patterns) <- as.character(1:length(x))
   if(length(names(x))){
        if(is.numeric(x))
          npatterns <- paste("\\(\\$",names(x),":[d-gEG]*[0-9]*[#]?[*]?\\)",sep="")
        else
          npatterns <- paste("\\(\\$",names(x),"\\)",sep="")
        names(npatterns) <- names(x)
        }
   else npatterns <- c()
   res <- template
   for(i in 1:length(patterns)){
      targets <- get.substr(patterns[i],template)
      positions <- attr(targets,"positions")
      formats <- lapply(targets,get.format)
      formatted <- formatVec(x[i],formats,
                             default=float.style,
                             digits=digits,
                             signif.symbols=signif.symbols)
      for(j in 1:length(template)){
        if(length(targets[[j]])){
          targets.j <- targets[[j]]
          formatted.j <- formatted[[j]]
          for(k in 1:length(targets.j))
            res <- gsub(targets.j[k],formatted.j[k],res,fixed=TRUE)
        }
      }
   }
   if(length(npatterns)){
      for(n in names(npatterns)){
          targets <- get.substr(npatterns[n],template)
          positions <- attr(targets,"positions")
          formats <- lapply(targets,get.format)
          formatted <- formatVec(x[n],
                                 formats,
                                 default=float.style,
                                 digits=digits,
                                 signif.symbols=signif.symbols)
          for(j in 1:length(template)){
            if(length(targets[[j]])){
              targets.j <- targets[[j]]
              formatted.j <- formatted[[j]]
              for(k in 1:length(targets.j))
                res <- gsub(targets.j[k],formatted.j[k],res,fixed=TRUE)
            }
          }
      }
   }
   unchanged <- res == template
   res[unchanged] <- ""
   res
}

str_has <- function(text,has,not=NULL,how=c("all","any")){
    how <- match.fun(match.arg(how))

    hasit <- sapply(has,function(pat)regexpr(pat,text,fixed=TRUE) > 0)
    if(is.matrix(hasit))
        hasit <- apply(hasit,1,how)
    else
        hasit <- all(hasit)


    if(!length(not)) return(hasit)
    # else
    hasnot <- sapply(not,function(pat)regexpr(pat,text,fixed=TRUE) > 0)
    if(is.matrix(hasnot))
        hasnot <- apply(hasnot,1,how)
    else
        hasnot <- all(hasnot)

    hasit & !hasnot
}



setCoefTemplate <- function(...){
  args <- list(...)
  argnames <- names(args)
  CoefTemplates <- get("CoefTemplates", envir=.memiscEnv)
  OldCoefTemplates <- CoefTemplates
    for(coef.style in argnames){
      CoefTemplates[[coef.style]] <- args[[coef.style]]
  }
  assign("CoefTemplates",CoefTemplates, envir=.memiscEnv)
  return(invisible(OldCoefTemplates))
}

getFirstMatch <- function(x,n){
  for(n. in n){
    if(n. %in% names(x)) return(x[[n.]])
  }
  return(x[["default"]])
}

getCoefTemplate <- function(style){
  CoefTemplates <- get("CoefTemplates", envir=.memiscEnv)
  if(missing(style)) return(CoefTemplates)
  else return(CoefTemplates[[style]])
}


getSummary <- function(obj,alpha=.05,...) UseMethod("getSummary")
# setGeneric("getSummary")

summaryTemplate <- function(x)
  UseMethod("summaryTemplate")

getFirstS3method <- function(mname,cls,optional){
    for(cls1 in cls){
        mfun <- getS3method(mname,cls1,optional)
        if(length(mfun)) return(mfun)
    }
    return(NULL)
}

getSummaryTemplate <- function(x){
  SummaryTemplates <- get("SummaryTemplates", envir=.memiscEnv)
  if(missing(x)) return(SummaryTemplates)
  if(is.character(x)) cls <- x
  else cls <- class(x)
  stf <- getFirstS3method("summaryTemplate",cls,optional=TRUE)
  if(length(stf))
    res <- stf(x)
  else
    res <- getFirstMatch(SummaryTemplates,cls)
  return(res)
}

setSummaryTemplate <- function(...){
  args <- list(...)
  argnames <- names(args)
  OldSummaryTemplates <- SummaryTemplates <- get("SummaryTemplates", envir=.memiscEnv)
  for(cls in argnames){
      SummaryTemplates[[cls]] <- args[[cls]]
  }
  assign("SummaryTemplates",SummaryTemplates,envir=.memiscEnv)
  return(invisible(OldSummaryTemplates))
}

selectSummaryStats <- function(x,n) {
    if(is.character(n)){
        n
    }
    else if(isTRUE(n)){
        cls <- class(x)
        sumstats.name <- paste0("summary.stats.",cls)
        sumstats <- lapply(sumstats.name,getOption)
        if(any(!vapply(sumstats, is.null, TRUE))){
            sumstats <- unlist(sumstats)
            sumstats[1]
        }
        else
            sumstats <- getOption("summary.stats.default")
        sumstats
    }
    else FALSE
}

prettyNames <- function(coefnames,
                        contrasts,
                        xlevels,
                        factor.style,
                        show.baselevel,
                        baselevel.sep
                        ){
    termorders <- sapply(strsplit(coefnames,":",fixed=TRUE),length)
    ordergroups <- split(coefnames,termorders)
    ordergroups <- lapply(ordergroups,prettyNames1,
                        contrasts=contrasts,
                        xlevels=xlevels,
                        factor.style=factor.style,
                        show.baselevel=show.baselevel,
                        baselevel.sep=baselevel.sep
                        )
    unsplit(ordergroups,termorders)
}

prettyNames1 <- function(str,
                        contrasts,
                        xlevels,
                        factor.style,
                        show.baselevel,
                        baselevel.sep
                        ){
   str <- gsub(":"," x ",str,fixed=TRUE)
   for(f in names(contrasts)){
      contrast.f <- contrasts[[f]]
      levels <- xlevels[[f]]
      #if(!length(levels)) levels <- c("FALSE","TRUE")
      if(!length(levels)) {
        str <- gsub(paste(f,"TRUE",sep=""),f,str,fixed=TRUE)
        next
      }
      if(is.character(contrast.f))
        contrast.matrix <- do.call(contrast.f,list(n=levels))
      else if(is.matrix(contrast.f))
        contrast.matrix <- contrast.f
      levels.present <- sapply(levels,function(level)
            any(str_has(str,c(f,level)))
            )
      if(all(levels.present))
        oldlabels <- newlabels <- levels
      else if(!length(colnames(contrast.matrix))){
        oldlabels <- newlabels <- as.character(1:ncol(contrast.matrix))
        }
      else if(is.character(contrast.f) &&
          contrast.f %in% c(
              "contr.treatment",
              "contr.SAS"
              )){
         baselevel <- setdiff(rownames(contrast.matrix),colnames(contrast.matrix))
         if(show.baselevel)
           newlabels <- paste(colnames(contrast.matrix),baselevel,sep=baselevel.sep)
         else
           newlabels <- colnames(contrast.matrix)
         oldlabels <- colnames(contrast.matrix)
      }
      else if(is.character(contrast.f) &&
          contrast.f %in% c(
              "contr.sum",
              "contr.helmert"
              )){
         newlabels <- apply(contrast.matrix,2,
                                          function(x)rownames(contrast.matrix)[x>=1])
         oldlabels <- colnames(contrast.matrix)
      }
      else if(
        all(colnames(contrast.matrix) %in% rownames(contrast.matrix))
        ){
         baselevel <- setdiff(rownames(contrast.matrix),colnames(contrast.matrix))
         if(show.baselevel)
           newlabels <- paste(colnames(contrast.matrix),baselevel,sep=baselevel.sep)
         else
           newlabels <- colnames(contrast.matrix)
         oldlabels <- colnames(contrast.matrix)
      }
      else {
        oldlabels <- newlabels <- colnames(contrast.matrix)
      }
      from <- paste(f,oldlabels,sep="")
      to <- sapply(newlabels,
        function(l)applyTemplate(c(f=f,l=l),template=factor.style))
      for(i in 1:length(from))
        str <- gsub(from[i],to[i],str,fixed=TRUE)
   }
   str
}

bind_arrays <- function(args,along=1){
  along.dn <- unlist(lapply(args,function(x)dimnames(x)[[along]]))
  groups <- sapply(args,function(x)dim(x)[along])
  dn <- dimnames(args[[1]])
  keep.dn <- dn[-along]
  dim1 <- dim(args[[1]])
  keep.dim <- dim1[-along]
  ldim <- length(dim1)
  dimseq <- seq_len(ldim)
  perm.to <- dimseq
  perm.to[ldim] <- along
  perm.to[along] <- ldim
  res <- lapply(args,function(x){
    x <- aperm(x,perm.to)
    dim(x) <- c(prod(dim(x)[-ldim]),dim(x)[ldim])
    x
    })
  res <- do.call(cbind,res)
  dim(res) <- c(keep.dim,ncol(res))
  dimnames(res) <- c(keep.dn,list(along.dn))
  structure(aperm(res,perm.to),groups=groups)
}

names.or.rownames <- function(x){
    if(is.array(x)) rownames(x)
    else names(x)
}

mtable <- function(...,
                   coef.style=getOption("coef.style"),
                   summary.stats=TRUE,
                   signif.symbols=getOption("signif.symbols"),
                   factor.style=getOption("factor.style"),
                   show.baselevel=getOption("show.baselevel"),
                   baselevel.sep=getOption("baselevel.sep"),
                   getSummary=eval.parent(quote(getSummary)),
                   float.style=getOption("float.style"),
                   digits=min(3,getOption("digits")),
                   sdigits=digits,
                   show.eqnames=getOption("mtable.show.eqnames",NA),
                   gs.options=NULL,
                   controls=NULL,
                   collapse.controls=FALSE,
                   control.var.indicator=getOption("control.var.indicator",c("Yes","No"))
                   ){
  args <- list(...)
  if(length(args)==1 && inherits(args[[1]],"by"))
    args <- args[[1]]
  argnames <- names(args)
  if(!length(argnames)) {
    m <- match.call(expand.dots=FALSE)
    argnames <- sapply(m$...,paste)
  }
  n.args <- length(args)

  arg.classes <- lapply(args,class)
  if(any(sapply(arg.classes,length))==0) stop("don\'t know how to handle these arguments")
  
  if(length(gs.options)){
    summaries.call <- as.call(
      c(list(as.name("lapply"),
             as.name("args"),
             FUN=as.name("getSummary")),
        gs.options
      ))
    summaries <- eval(summaries.call)
  }
  else
    summaries <- lapply(args,getSummary)
  
  # parameter.types <- unique(unlist(lapply(summaries,names)))
  # parameter.types <- parameter.types[parameter.types %nin% c("sumstat","contrasts","call","xlevels")]
  parameter.types <- unique(unlist(lapply(summaries,`[[`,"parameter.types")))
  parameter.types <- c("coef",parameter.types)
  parmnames <- list()
  for(pt in parameter.types){

      tmp.pn <- lapply(summaries,`[[`,pt)
      tmp.pn <- lapply(tmp.pn,names.or.rownames)
      parmnames[[pt]] <- unique(unlist(tmp.pn))
  }
  parameter.names <- unique(unlist(parmnames))
  
  stemplates <- lapply(args,getSummaryTemplate)
  if(isTRUE(summary.stats))
      summary.stats <- lapply(args,selectSummaryStats,TRUE)
  else if(is.character(summary.stats))
      summary.stats <- lapply(args,selectSummaryStats,summary.stats)
  else if(is.list(summary.stats)){
      tmp.summary.stats <- summary.stats
      summary.stats <- vector(mode="list",length=length(args))
      summary.stats[] <- tmp.summary.stats
  } else {
      summary.stats <- vector(mode="list",length=length(args))
      summary.stats[] <- list(FALSE)
  }
      
  if(length(controls)){
      if(is.character(controls))
          controls <- asOneSidedFormula(controls)
      if(inherits(controls,"formula")){
          control.coefs <- lapply(args,formula2coefs,fo=controls)
          control.terms <- lapply(args,formula2termlabs,fo=controls)
      }
      else
          stop("'controls=' must be a formula or a character vector.")
      controls <- list(coefs=control.coefs,terms=control.terms)
  } 
  
  structure(summaries,
            names=argnames,
            class="memisc_mtable",
            parameter.names=parameter.names,
            coef.style=coef.style,
            summary.stats=summary.stats,
            signif.symbols=signif.symbols,
            factor.style=factor.style,
            show.baselevel=show.baselevel,
            baselevel.sep=baselevel.sep,
            float.style=float.style,
            digits=digits,
            stemplates=stemplates,
            sdigits=sdigits,
            show.eqnames=show.eqnames,
            controls=controls,
            collapse.controls=collapse.controls,
            control.var.indicator=control.var.indicator
            )
  
}

prefmt1 <- function(parm,template,float.style,digits,signif.symbols,controls){
    
    rn <- rownames(parm)
    if(length(intersect(rn,controls))){
        controls <- intersect(rn,controls)
        rn <- setdiff(rn,controls)
        if(length(dim(parm))==2)
            parm <- parm[rn,,drop=FALSE]
        else
            parm <- parm[rn,,,drop=FALSE]
    }
    else controls <- NULL
    adims <- if(length(dim(parm))==2) 1 else c(1,3)
    if(length(parm)){
        
        if(is.array(parm)){
            ans <- apply(parm,adims,applyTemplate,
                         template=template,
                         float.style=float.style,
                         digits=digits,
                         signif.symbols=signif.symbols)
        }
        else {
            ans <- array(formatC(parm,
                                 digits=digits,
                                 ifelse(is.integer(parm),
                                        "d","f"),
                                 width=1),
                         dim=c(1,1,length(parm),1),
                         dimnames=list(NULL,NULL,names(parm),NULL))
            return(ans)
        }
    }
    else {
        ans <- array(character(0),
                     dim=c(0,dim(parm)[adims]),
                     dimnames=c(list(NULL),dimnames(parm)[adims]))
    }
        
    if(length(dim(template))){
        newdims <- c(dim(template),dim(ans)[-1])
        newdimnames <- c(dimnames(template),dimnames(ans)[-1])

        # for(i in 1:length(newdims)){
        #     if(!length(newdimnames[[i]])){
        #         if(newdims[i]==0)
        #             newdimnames[[i]] <- character(0)
        #         else
        #             newdimnames[[i]] <- as.character(1:newdims[i])
        #     }
        # }
        
        dim(ans) <- newdims
        dimnames(ans) <- newdimnames
    } else rownames(ans) <- names(template)

    ans[ans=="()"] <- ""
    attr(ans,"controls") <- controls
    return(ans)
}

prefmt2 <- function(parm){
    
    if(length(dim(parm))<4)
        dim(parm)[4] <- 1

    parm <- aperm(parm,c(1,3,2,4))
    dim(parm) <- c(prod(dim(parm)[1:2]),prod(dim(parm)[3:4]))

    parm
}

colexpand <- function(x,nc){
    x.nr <- nrow(x)
    x.nc <- ncol(x)
    y <- matrix("",nrow=x.nr,ncol=max(nc,1))
    if(length(x))
        y[,1:x.nc] <- x
    y
}

rowexpand <- function(x,nr){
    x.nr <- nrow(x)
    x.nc <- ncol(x)
    y <- matrix("",nrow=nr,ncol=x.nc)
    if(length(x))
        y[1:x.nr,] <- x
    y
}


dimnames3 <- function(x)dimnames(x)[[3]]

getRows <- function(x,r){
    if(is.character(r))
        r <- intersect(r,rownames(x))
    x[r,,drop=FALSE]
}
get_rows <- function(x,i)try(x[i,,drop=FALSE])

relabel.memisc_mtable <- function(x,...,gsub=FALSE,fixed=!gsub,warn=FALSE){

    relab.req <- list(...,
                      gsub=gsub,fixed=fixed,warn=warn)
    
    relab.attr <- attr(x,"relabel")
    if(!length(relab.attr))
        relab.attr <-list(relab.req)
    else
        relab.attr <-c(relab.attr,
                       list(relab.req))

    attr(x,"relabel") <- relab.attr

    x
}

pt_getrow <- function(x,i){
    y <- x[i,]
    isn <- sapply(y,is.null)
    if(any(isn)) return(y[!isn])
    else return(y)
}

do_subs <- function(x,r){
    for(rr in r)
        x <- do_1sub(x,rr)
    return(x)
}

do_1sub <- function(x,r){

    r.gsub <- r$gsub
    r.fixed <- r$fixed

    r <- r[names(r)%nin%c("gsub","fixed","warn")]

    y <- x
    for(i in seq_along(r)){
        from <- names(r)[i]
        to <- r[[i]]
        if(r.gsub)
            y <- gsub(from,to,y,fixed=r.fixed)
        else {
            y[y==from] <- to
        }
    }
    return(y)
}

do_prettyfy <- function(pn,
                        contrasts,     
                        xlevels,         
                        factor.style,    
                        show.baselevel,
                        baselevel.sep){

    if(!length(contrasts)) return(pn)
    
    res <- pn

    done <- res != pn

    for(m in names(contrasts)){
        contrasts.m <- contrasts[[m]]
        xlevels.m <- xlevels[[m]]
        if(all(done)) break
        pn.tmp <- pn[!done]
        pn.tmp  <- prettyNames(pn.tmp,
                               contrasts=contrasts.m,
                               xlevels=xlevels.m,
                               factor.style=factor.style,
                               show.baselevel=show.baselevel,
                               baselevel.sep=baselevel.sep)
        res[!done] <- pn.tmp
        done <- res != pn
    }
    
    return(res)
}    

nzchar_row <- function(x){
    nzch <- array(nzchar(x),dim=dim(x))
    apply(nzch,1,any)
}

dropnull <- function(x) {
    ii <- sapply(x,is.null)
    x[!ii]
}
ni <- function(tab,x) x%in%tab
drop_zchrow <- function(x)x[nzchar_row(x),,drop=FALSE]

preformat_mtable <- function(x){

    x <- unclass(x)
    
    coef.style <- attr(x,"coef.style")
    summary.stats <- attr(x,"summary.stats")
    other.stats <- attr(x,"other.stats")
    signif.symbols <- attr(x,"signif.symbols")
    factor.style <- attr(x,"factor.style")
    show.baselevel <- attr(x,"show.baselevel")
    baselevel.sep <- attr(x,"baselevel.sep")
    float.style <- attr(x,"float.style")
    digits <- attr(x,"digits")
    stemplates <- attr(x,"stemplates")
    sdigits <- attr(x,"sdigits")
    
    # allcompo <- unique(unlist(lapply(x,names)))
    # nonparnames <- c("sumstat","contrasts","xlevels","call")
    # partypes <- setdiff(allcompo,nonparnames)

    sumstats <- lapply(x,`[[`,"sumstat")
    contrasts <- lapply(x,`[[`,"contrasts")
    xlevels <- lapply(x,`[[`,"xlevels")
    calls <- lapply(x,`[[`,"call")

    partypes <- unique(c("coef",unlist(lapply(x,`[[`,"parameter.types"))))
    parms <- lapply(x,`[`,partypes)
    parms <- lapply(parms,dropnull)

    descriptives <- lapply(x,`[[`,"descriptives")

    ctemplate <- getCoefTemplate(coef.style)
    if(!length(ctemplate)) stop("invalid coef.style argument")
    ctemplate <- as.matrix(ctemplate)
    ctdims <- dim(ctemplate)
    lctdims <- length(ctdims)
    if(lctdims>2) stop("can\'t handle templates with dim>2")

    relab.attr <- attr(x,"relabel")
    
    modelnames <- names(x)
    modelgroups <- attr(x,"model.groups")

    force.header <- isTRUE(attr(x,"force.header")) # Document that later ...
    show.eqnames <- attr(x,"show.eqnames")

    all.control.terms <- NULL
    control.terms <- NULL
    control.coefs <- NULL
    controls <- attr(x,"controls")
    collapse.controls <- attr(x,"collapse.controls")
    if(length(controls)){
        control.terms <- controls$terms
        control.coefs <- controls$coefs
        control.coefs <- unique(unlist(control.coefs))

        all.control.terms <- unique(unlist(control.terms))
    }
    
    parmtab <- NULL

    ct.indicator <- attr(x,"control.var.indicator")
    if(!length(ct.indicator)) ct.indicator <- c("X","")
    
    if(length(partypes)){
        for(n in 1:length(parms)){

            parms.n <- parms[[n]]
            parms.n<- lapply(parms.n,
                             prefmt1,
                             template=ctemplate,
                             float.style=float.style,
                             digits=digits,
                             signif.symbols=signif.symbols,
                             controls=control.coefs)
            if(length(control.terms)){
                ct <- control.terms[[n]]
                ct <- all.control.terms %in% ct
                if(collapse.controls) {
                    if(all(ct))
                        ct <- ct.indicator[1]
                    else if(!any(ct))
                        ct <- ct.indicator[2]
                    else
                        ct <- as.character(NA)
                    dim(ct) <- c(1,1,1,1)
                    dimnames(ct) <- list(1,2,"Controls",3)
                }
                else {
                    ct <- ifelse(ct,ct.indicator[1],ct.indicator[2])
                    dim(ct) <- c(1,1,length(ct),1)
                    dimnames(ct) <- list(1,2,all.control.terms,3)
                }
                parms.n <- append(parms.n,list(Controls=ct),after=1)
            }
            parms[[n]] <- parms.n
        }
        if(length(control.terms))
            partypes <- append(partypes,"Controls",after=1)
        parmtab <- array(list(),
                  dim=c(length(partypes),length(parms)),
                  dimnames=list(partypes,names(parms)))

        for(n in 1:length(parms)){
            mod <- parms[[n]]
            modnames <- names(mod)
            for(m in modnames){
                mod.m <- mod[[m]]
                parmtab[[m,n]] <- mod.m
            }
        }

        parameter.names <- attr(x,"parameter.names")
        parmnames <- list()
        
        for(m in rownames(parmtab)){
            tmp.pn <- lapply(parmtab[m,],dimnames3)
            tmp.pn <- unique(unlist(tmp.pn))
            tmp.pn <- parameter.names[parameter.names %in% tmp.pn]
            parmnames[[m]] <- tmp.pn
        }
        if(length(all.control.terms)){
            if(collapse.controls)
                parmnames$Controls <- "Controls"
            else
                parmnames$Controls <- all.control.terms
        }
        # Make sure that columns and rows match across models
        for(n in 1:ncol(parmtab)){
            mod <- parms[[n]]
            for(m in rownames(parmtab)){
                parmtab.mn <- parmtab[[m,n]]
                if(length(parmnames[[m]])){
                    parmtab.mn <- coefxpand(parmtab.mn,parmnames[[m]])
                    parmtab.mn <- prefmt2(parmtab.mn)
                    parmtab[[m,n]] <- parmtab.mn
                }
                modm <- mod[[m]]
            }
            maxncol <- max(unlist(lapply(parmtab[,n],ncol)) )
            parmtab[,n] <- lapply(parmtab[,n],colexpand,maxncol)
        }
        # Drop empty rows
        for(n in 1:nrow(parmtab)){
            maxnrow <- max(unlist(lapply(parmtab[n,],nrow)) )
            parmtab[n,] <- lapply(parmtab[n,],rowexpand,maxnrow)
            nz <- lapply(parmtab[n,],nzchar_row)
            if(length(nz)>1)
                nz <- reduce(nz,`|`)
            else
                nz <- nz[[1]]
            parmtab[n,] <- lapply(parmtab[n,],get_rows,i=nz)
        }
    }
    headers <- list()
    if(length(modelnames) > 1 || length(modelnames) == 1 && force.header) {
        modelnames <- do_subs(modelnames,relab.attr)
        headers[[1]] <- Map(structure,modelnames,span=lapply(parmtab[1,],ncol))
        if(length(modelgroups)){
            ncols <- sapply(parmtab[1,],ncol)
            sp <- lapply(modelgroups,function(mg)sum(ncols[mg]))
            h <- Map(structure,names(modelgroups),span=sp)
            headers <- c(list(h),headers)
        }
    }
    # show.eqnames <- show.eqnames || has.multieq(x)

    get_eq.headers <- function(x){
        cf <- x$coef
        dn.cf <- dimnames(cf)
        if(length(dn.cf)>2)
            eq.names <- dimnames(cf)[[3]]
        else
            eq.names <- NULL
    }
    eq.headers <- lapply(x,get_eq.headers)
    all.eq.names <- unique(unlist(eq.headers))
    if(is.na(show.eqnames))
        show.eqnames <- length(all.eq.names) > 1
    if(!show.eqnames)
        eq.headers <- NULL
    
    leaders <- vector(mode="list",length=nrow(parmtab))
    names(leaders) <- rownames(parmtab)
    if(length(partypes)){
      i <- 0 
      for(m in rownames(parmtab)){
        i <- i + 1
        pn <- parmnames[[m]]
        pn <- do_prettyfy(pn,
                          contrasts=contrasts,     
                          xlevels=xlevels,         
                          factor.style=factor.style,    
                          show.baselevel=show.baselevel,
                          baselevel.sep=baselevel.sep)  
        pn <- do_subs(pn,relab.attr)
        span <- nrow(parmtab[[m,1]])/length(pn)
        if(span < 1)
          leaders[[i]] <- NULL
        else
          leaders[[i]] <- lapply(pn,structure,span=span)
      }
      pt <- rownames(parmtab)
      pt <- do_subs(pt,relab.attr)
      rownames(parmtab) <- pt
      names(leaders) <- pt
    }

    if(length(summary.stats)){
        stemplates_ <- Map(select_by_names,stemplates,summary.stats)
        sst <- Map(applyTemplate,sumstats,stemplates_,digits=sdigits)
        sst <- lapply(sst,drop_zchrow)
        snames <- unique(unlist(lapply(sst,rownames)))
        nc <- lapply(parmtab[1,],ncol)
        summary.stats <- Map(smryxpand,sst,list(snames))

        snames <- do_subs(snames,relab.attr)
        snames <- lapply(snames,structure,span=1)
        leaders <- c(leaders,summary.stats=list(snames))
    }
    else summary.stats <- NULL

    if(length(descriptives)){
        dnames <- unique(unlist(lapply(descriptives,rownames)))
        nc <- lapply(parmtab[1,],ncol)
        descriptives <- Map(smryxpand,descriptives,list(dnames))
    }
    else descriptives <- NULL

    needs.signif <- any(grepl("$p",ctemplate,fixed=TRUE))
    if(needs.signif){
        signif.symbols <- signif.symbols
    }
    else
        signif.symbols <- NULL

    outtypes <- array("num",
                      dim=dim(parmtab),
                      dimnames=dimnames(parmtab))
    if(length(controls)){
        outtypes["Controls",] <- "text"
    }
    
    structure(list(parmtab=parmtab,
                   leaders=leaders,
                   headers=headers,
                   eq.headers=eq.headers,
                   summary.stats = summary.stats,
                   other.stats = other.stats,
                   signif.symbols=signif.symbols,
                   controls=controls,
                   outtypes=outtypes),
              class="preformatted.memisc_mtable")
    }

# Avoid NA's while subsetting
select_by_names <- function(x,nms){
    nms <- unique(nms)
    nms <- intersect(names(x),nms)
    x[nms]
}


format_signif <- function(syms,tmpl){
    title <- tmpl[1]
    clps <- tmpl[3]
    tmpl <- tmpl[2]
    res <- c()
    for(i in seq_along(syms)){
        sym <- names(syms)[i]
        thrsh <- unname(syms[i])
        res.i <- sub("$sym",sym,tmpl,fixed=TRUE)
        res.i <- sub("$val",thrsh,res.i,fixed=TRUE)
        res <- c(res,res.i)
    }
    res <- paste(res,collapse=clps)
    paste0(title,res)
}


format.memisc_mtable <- function(x,
                          target=c("print","LaTeX","HTML","delim"),
                          ...){
    target <- match.arg(target)
    x <- preformat_mtable(x)
    switch(target,
           print=pf_mtable_format_print(x,...),
           LaTeX=pf_mtable_format_latex(x,...),
           HTML=pf_mtable_format_html(x,...),
           delim=pf_mtable_format_delim(x,...)
           )
}

print.memisc_mtable <- function(x,center.at=getOption("OutDec"),
      topsep="=",bottomsep="=",sectionsep="-",...){

    calls <- sapply(x,"[[","call")
    cat("\nCalls:\n")
    for(i in seq(calls)){
        cat(names(calls)[i],": ",sep="")
        print(calls[[i]])
    }
    cat("\n")
    cat(format.memisc_mtable(x,target="print",
                      center.at=center.at,
                      topsep=topsep,
                      bottomsep=bottomsep,
                      sectionsep=sectionsep,...),
        sep="")
}

toLatex.memisc_mtable <- function(object,...){
  structure(format.memisc_mtable(x=object,target="LaTeX",...),
  class="Latex")
}

write.mtable <- function(object,file="",
                         format=c("delim","LaTeX","HTML"),
                         ...){
  l <- list(...)
  if(isTRUE(l[["forLaTeX"]])) # Avoid breaking old code
    target <- "LaTeX"
  else
    target <- match.arg(format)
    
  f <- format.memisc_mtable(object,target=target,...)
  if(target %in% c("LaTeX","HTML"))
    f <- paste(f,"\n",sep="")
  cat(f,file=file,sep="")
}

setValidity("named.list",function(object){
  if(all(is.na(object@names))) "list is unnamed"
  else if(length(object@.Data)!=length(object@names[nzchar(object@names)])){
    znames <- which(!nzchar(object@names))
    paste(
      if(length(znames) == 1) "element" else "elements",
      paste(znames,collapse=", "),
      if(length(znames) == 1) "is" else "are",
      "unnamed"
      )
    }
  else if(length(unique(object@names)) != length(object@names)) paste(
    "list has duplicate names:",
    paste(dQuote(object@names[duplicated(object@names)]),collapse=", ")
    )
  else TRUE
})

setValidity("data.set",function(object){

  isItemVector <- sapply(object,is,"item.vector")
  if(!all(isItemVector)) {
    wrong.els <- object[!isItemVector]
    wrong.classes <- sapply(wrong.els,class)
    wrong.names <- object@names[!isItemVector]
    paste(
      "object has elements of wrong class:",
      paste(
        paste("class(",wrong.names,") = ",wrong.classes,sep=""),
        collapse=", "
      )
    )
  }
  else if(any(length(object@row_names) != sapply(object,length))){
    wrong.els <- object[!isItemVector]
    wrong.names <- object@names[!isItemVector]
    wront.lengths <- sapply(object,length)
    paste(
    if(length(which(wrong.lengths)) > 1) "elements have" else "element has",
    "wrong length: ",
      paste(
        paste("class(",wrong.names,") = ",wrong.classes,sep=""),
        collapse=", "
      ),
    "where",
    length(object@row_names),
    "is required"
    )
  }
  else TRUE
})

setMethod("initialize","named.list",function(.Object,...){
  args <- list(...)
  if(is.list(args[[1]])) args <- unclass(args[[1]])
  .Object@.Data <- unname(args)
  .Object@names <- as.character(names(args))
  if(validObject(.Object)) .Object
})

setMethod("initialize","item.list",function(.Object,...){
  args <- list(...)
  if(is.list(args[[1]])) args <- unclass(args[[1]])
  .Object@.Data <- unname(lapply(args,as.item))
  .Object@names <- as.character(names(args))
  if(validObject(.Object)) .Object
})


setMethod("show","named.list",function(object)
  print.default(unclass(object))
)

setLength <- function(x,n){
  tmp <- unname(x)
  length(x) <- n
  x[] <- tmp
  attributes(x) <- attributes(tmp)
  x
}

setMethod("initialize","data.set",function(.Object,...,row.names=NULL,document=character()){

  args <- list(...)
  if(inherits(args[[1]],"data.frame") && missing(row.names)){
      row.names <- rownames(args[[1]])
  }
  if(is.list(args[[1]])) {
      args <- unclass(args[[1]])
  }
  nr <- max(sapply(args,length))
  args <- lapply(args,setLength,n=nr)
  args <- lapply(args,as.item)

  .Object@.Data <- unname(args)
  .Object@names <- as.character(names(args))

  if (is.null(row.names))
      row.names <- seq_len(nr)
  else {
      if (is.object(row.names) || !is.integer(row.names))
          row.names <- as.character(row.names)
      if (any(is.na(row.names)))
          stop("row names contain missing values")
      if (any(duplicated(row.names)))
          stop("duplicate row.names: ", paste(unique(row.names[duplicated(row.names)]),
              collapse = ", "))
  }
  .Object@row_names <- row.names
  .Object@document <- document

  if(validObject(.Object)) .Object
})

setAs("data.set","named.list",function(from,to){
  new(to,structure(from@.Data,names=from@names))
  })

# dim.data.set <- dim.data.frame
setMethod("dim","data.set",function(x)
  c( length(x@row_names),
     length(x@.Data)
  )
)

setMethod("row.names","data.set",function(x){
  x@row_names
})

setReplaceMethod("row.names","data.set",function(x,value){
  nr <- length(x@.Data[[1]])
  if(is.null(value)){
    value <- seq_len(nr)
  }
  else if(length(value) != nr)
    stop("invalid 'row.names' given for data set")
  x@row_names <- value
  x
})


setMethod("dimnames","data.set",function(x)
  list(x@row_names,x@names))

setReplaceMethod("dimnames","data.set",function(x,value) {
    d <- dim(x)
    if (!is.list(value) || length(value) != 2L)
        stop("invalid 'dimnames' given for data set")
    value[[1L]] <- as.character(value[[1L]])
    value[[2L]] <- as.character(value[[2L]])
    if (d[[1L]] != length(value[[1L]]) || d[[2L]] != length(value[[2L]]))
        stop("invalid 'dimnames' given for data set")
    row.names(x) <- value[[1L]]
    names(x) <- value[[2L]]
    x
})


setMethod("[",signature(x="data.set",i="atomic",j="atomic",drop="ANY"),
  function(x,i,j,...,drop=FALSE){
    frame <- structure(x@.Data,row.names=x@row_names,names=x@names,class="data.frame")
    frame <- frame[i,j,drop=drop]
    if(is.data.frame(frame))
      new("data.set",
        unclass(frame),
        document=x@document
        )
    else
      frame
})

setMethod("[",signature(x="data.set",i="atomic",j="missing",drop="ANY"),
  function(x,i,j,...,drop=FALSE){
#     cat("\ndata.set,atomic,missing\n")
    Narg <- nargs()-!missing(drop)
    frame <- structure(x@.Data,row.names=x@row_names,names=x@names,class="data.frame")
    if(Narg > 2){
      frame <- frame[i,,drop=drop]
      if(!is.data.frame(frame))
        frame
      else
        new("data.set",
          unclass(frame),
          document=x@document
          )
    }
    else {
      frame <- frame[i]
      if(!is.data.frame(frame))
        frame
      else
        new("data.set",
          unclass(frame),
          document=x@document
          )
    }
})

setMethod("[",signature(x="data.set",i="missing",j="atomic",drop="ANY"),
  function(x,i,j,...,drop=FALSE){
#     cat("\ndata.set,missing,atomic\n")
    frame <- structure(x@.Data,row.names=x@row_names,names=x@names,class="data.frame")
    frame <- frame[,j,drop=drop]
    if(is.data.frame(frame))
      new("data.set",
        unclass(frame),
        document=x@document
        )
    else
      frame
})

setMethod("[",signature(x="data.set",i="missing",j="missing",drop="ANY"),
  function(x,i,j,...,drop=FALSE){
    frame <- structure(x@.Data,row.names=x@row_names,names=x@names,class="data.frame")
    frame <- frame[,,drop=drop]
    if(is.data.frame(frame))
      new("data.set",
        unclass(frame),
        document=x@document
        )
    else
      frame
})


setReplaceMethod("[",signature(x="data.set",i="ANY",j="ANY",value="ANY"),
  function(x,i,j,value){
    frame <- structure(x@.Data,row.names=x@row_names,names=x@names,class="data.frame")
    frame[i,j] <- value
    new("data.set",
      unclass(frame),
      document=x@document
      )
})



"[[<-.data.set" <- function(x,...,value){
  frame <- structure(x@.Data,row.names=x@row_names,names=x@names,class="data.frame")
  frame[[...]] <- value
  new("data.set",
    unclass(frame),
    document=x@document
    )
}

as.list.data.set <- function(x,...)structure(x@.Data,names=x@names)

as.data.frame.data.set <- function(x, row.names = NULL, optional = FALSE, ...){
  as.data.frame(as.list(x),
          row.names=if(length(row.names)) rownames
                    else x@row_names,
          optional=optional)
}



data.set <- function(..., row.names = NULL, check.rows = FALSE, check.names = TRUE,
    stringsAsFactors = FALSE,
    document = NULL){

  args <- list(...)
  if(!length(names(args))){

    subst <- substitute(list(...))
    names(args) <- as.character(subst[-1])
  }
  argn <- names(args)
  args <- lapply(seq_along(args),function(i){
      x <- args[[i]]
      n <- names(args)[[i]]
      if(is(x,"item.vector"))
        structure(list(x),class="data.frame",row.names=seq_len(length(x)),names=n)
      else if(is(x,"data.set"))
        structure(as.list(x),class="data.frame",row.names=x@row_names)
      else x
    })
  names(args) <- argn
  frame <- do.call(data.frame,
    c(args,
      row.names=row.names,
      check.rows=check.rows,
      check.names=check.names,
      stringsAsFactors=stringsAsFactors
    ))
  new("data.set",
    frame,
    document=as.character(document)
    )
}


setMethod("annotation","data.set",function(x){
  d <- lapply(x,annotation)
  if(length(d))
    structure(d,names=x@names,class="annotation.list")
  else NULL
})


format.data.set <- function(x,...){
    y <- structure(x@.Data,
                   row.names=x@row_names,
                   names=x@names,
                   class="data.frame")
    y[] <- lapply(y[],format,justify="right")
    y
}
#setMethod("format","data.set",format.data.set)

print.data.set <- function(x,max.obs=Inf,width=Inf,...){
  frame <- structure(x@.Data,row.names=x@row_names,names=x@names,class="data.frame")
  print_frame_internal(frame,max.obs=max.obs,width=width,...)
}

print_frame_internal <- function(x,max.obs=Inf,width=Inf,...){
  if(is.finite(max.obs)){
    if(nrow(x)<=max.obs)
      {
        max.obs <- Inf
        res <- x
      }
    else
      res <- x[seq_len(max.obs),,drop=FALSE]
  }
  else
    res <- x

  varn <- names(res)
  rown <- rownames(res)

  res <- lapply(res,format)
  res <- mapply(c,varn,res)
  res <- apply(res,2,format,justify="right")
  res <- apply(cbind(c("",rown),res),2,format,justify="right")
  
  if(is.finite(width) && ncol(res)){

    ww <- cumsum(nchar(res[1,])+1)-1
    if(any(ww > width)){

      keep <- which(ww < width - 3)
      res <- cbind(res[,keep],"...")
    }
  }

  if(is.finite(max.obs) && nrow(res)){

    mkdots <- function(n) paste(rep(".",n),collapse="")
    ww <- nchar(res[1,])
    res <- rbind(res,sapply(ww,mkdots))
    res <- apply(res,1,paste,collapse=" ")
    res <- c(res,paste("(",length(res)-2," of ",nrow(x)," observations shown)",sep=""))
    }
  else
    res <- apply(res,1,paste,collapse=" ")

  writeLines(res)
}

setMethod("show","data.set",function(object){
  cat("\nData set with",nrow(object),"observations and",ncol(object),"variables\n\n")
  print.data.set(object,max.obs=getOption("show.max.obs"),width=getOption("width"))
})

setMethod("print","data.set",function(x,...)print.data.set(x,...))

# summary.data.set <- summary.data.frame

is.data.set <- function(x) is(x,"data.set")

str.data.set <- function (object, ...)
{
    cat("Data set ","with ", nrow(object), " obs. of ", (p <- ncol(object)),
        " variable", if (p != 1)
            "s", if (p > 0)
            ":", "\n", sep = "")
    object <- structure(as.list(object),class="data.frame")
    if (length(l <- list(...)) && any("give.length" == names(l)))
      invisible(NextMethod("str", ...))
    else invisible(NextMethod("str", give.length = FALSE, ...))
}

subset.data.set <- function (x, subset, select, drop = FALSE, ...) 
{
    r <- if (missing(subset)) 
        rep_len(TRUE, nrow(x))
    else {
        e <- substitute(subset)
        r <- eval(e, x, parent.frame())
        if (!is.logical(r)) 
            stop("'subset' must be logical")
        r & !is.na(r)
    }
    vars <- if (missing(select)) 
        rep_len(TRUE, ncol(x))
    else {
        nl <- as.list(seq_along(x))
        names(nl) <- names(x)
        eval(substitute(select), nl, parent.frame())
    }
    x[r, vars, drop = drop]
}



setMethod("within","data.set",function (data, expr, ...)
{
    parent <- parent.frame()
    encl <- new.env(parent=parent)  
    frame <- structure(data@.Data,row.names=data@row_names,names=data@names,class="data.frame")
    nr <- nrow(frame)
    rn <- row.names(frame)
    assign("N_",nr,envir=encl)
    e <- evalq(environment(), frame, encl)
    ret <- eval(substitute(expr), e)
    l <- rev(as.list(e))

    length1 <- sapply(l,length) == 1
    if(any(length1)){
      ii <- which(length1)
      for(i in ii){
        l[[i]] <- rep(l[[i]],nr)
      }
    }
    
    wrong.length <- sapply(l,length) != nr
    if(any(wrong.length)){
      warning("Variables ",paste(sQuote(names(l)[wrong.length]),collapse=","),
                " have wrong length, removing them.")
      l[wrong.length] <- NULL
    }
    coercable <- sapply(l,is.atomic) | sapply(l,is.factor)
    items <- sapply(l,is,"item")
    if(any(!items & coercable))
      l[!items & coercable] <- lapply(l[!items & coercable],as.item)
    if(any(!items & !coercable)){
      warning("Cannot change variables ",paste(sQuote(names(l)[!items & !coercable]),collapse=","),
            " into items, removing them.")
      l[!items & !coercable] <- NULL
    }
    frame[names(l)] <- l
    use <- names(frame) %in% names(l)
    frame <- frame[use]
    row.names(frame) <- rn
    new("data.set",
      frame,
      document=data@document)
})

cbind.data.set <- function (..., deparse.level = 1)
  data.set(..., check.names = FALSE)

setMethod("description","data.set",function(x){
  res <- lapply(x,description)
  structure(res,class="descriptions")
})
print.descriptions <- function(x,quote=FALSE,...){
  Write.descriptions(x,file=stdout())
}

Write.descriptions <- function(x,file=stdout(),...){
  x <- sapply(x,function(des){
          if(length(des)) sQuote(des)
          else " (none) "
          })
  out <- c(
    "",
    paste("",format(names(x),justify="left"),format(x,justify="left")),
    ""
  )
  writeLines(out,con=file)
}

as.data.frame.descriptions <- function(x,...){
  data.frame(variable=names(x),
             description=as.character(x))
}

setMethod("unique","data.set",function(x, incomparables = FALSE, ...){
  frame <- structure(x@.Data,row.names=x@row_names,names=x@names,class="data.frame")
  new("data.set",
      unique(frame,incomparables=incomparables,...),
      document=x@document
      )
})


fapply.data.set <- function(formula,data,...)
  fapply.default(formula,data=as.data.frame(data,optional=TRUE),...)
  
setMethod("as.data.set","list",function(x,row.names=NULL,...){
  n_row <- unique(sapply(x,length))
  if(length(n_row)>1)
      stop("list elements must have same length")
  class(x) <- "data.frame"
  if(length(row.names)){
    if(length(row.names)!=nrow(x)) stop("row.names argument has wrong length")
    attr(x,"row.names") <- row.names
  }
  else
    attr(x,"row.names") <- 1:n_row
  new("data.set",x)
})



setMethod("merge",signature(x="data.set","data.set"),function(x,y,...){
  x <- new("data.frame",as.list(x),row.names=x@row_names)
  y <- new("data.frame",as.list(y),row.names=y@row_names)
  z <- merge(x,y,...)
  new("data.set",z)
})

setMethod("merge",signature(x="data.set","data.frame"),function(x,y,...){
  x <- new("data.frame",as.list(x),row.names=x@row_names)
  z <- merge(x,y,...)
  new("data.set",z)
})

setMethod("merge",signature(x="data.frame","data.set"),function(x,y,...){
  y <- new("data.frame",as.list(y),row.names=y@row_names)
  z <- merge(x,y,...)
  new("data.set",z)
})

setMethod("rbind2",signature(x="data.set",y="data.set"),function(x,y){
  x <- asS4(new("data.frame",as.list(x),row.names=x@row_names),FALSE)
  y <- asS4(new("data.frame",as.list(y),row.names=y@row_names),FALSE)
  z <- rbind(x,y)
  new("data.set",z)
})

setMethod("rbind2",signature(x="data.set",y="data.frame"),function(x,y){
  x <- asS4(new("data.frame",as.list(x),row.names=x@row_names),FALSE)
  z <- cbind(x,y)
  new("data.set",z)
})

setMethod("cbind2",signature(x="data.set",y="data.set"),function(x,y){
  x <- asS4(new("data.frame",as.list(x),row.names=x@row_names),FALSE)
  y <- asS4(new("data.frame",as.list(y),row.names=y@row_names),FALSE)
  z <- cbind(x,y)
  new("data.set",z)
})

setMethod("cbind2",signature(x="data.frame",y="data.set"),function(x,y){
  y <- asS4(new("data.frame",as.list(y),row.names=y@row_names),FALSE)
  z <- cbind(x,y)
  new("data.set",z)
})

setMethod("cbind2",signature(x="data.set",y="data.frame"),function(x,y){
  x <- asS4(new("data.frame",as.list(x),row.names=x@row_names),FALSE)
  z <- cbind(x,y)
  new("data.set",z)
})

rbind.data.set <- function(...,deparse.level=1){
  args <- list(...)
  to.data.frame <- function(x){
    if(inherits(x,"data.set"))
      structure(
        x@.Data,
        names=x@names,
        row.names=x@row_names,
        class="data.frame"
      )
    else as.data.frame(x)
  }
  args <- lapply(args,to.data.frame)
  res <- do.call("rbind",c(args,list(deparse.level=deparse.level)))
  new("data.set",res,row.names=row.names(res))
}

dsView <- function(x){
  
    title <- paste("Data set:", deparse(substitute(x))[1])

    Data <- lapply(x@.Data,format,justify="left")
    
    document <- x@document
    row.names <- x@row_names
    .names <- x@names
    frame <- structure(Data,row.names=row.names,names=x@names,
                       class="data.frame")
    for(n in names(frame)){
        d <- description(x[[n]])
        if(length(d))
            attr(frame[[n]],"label") <- d
    }
    
    View.call <- call("View",x=frame,title=title)
    eval(View.call,globalenv())
  #View(x=frame,title=title)
  # do.call("View",list(x=frame,title=title))
}


## Copied and modified from 'summary.R' of the package 'base'.
## The original copyright notice says:
##  """
##  File src/library/base/R/summary.R
##  Part of the R package, https://www.R-project.org
##
##  Copyright (C) 1995-2022 The R Core Team
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##  """

setMethod("summary","data.set",
  function(object, maxsum = 7, digits = max(3, getOption("digits") -3), ...){
    z <- lapply(as.list(object), summary, maxsum = maxsum, digits = 12,
        ...)
    nv <- length(object)
    nm <- names(object)
    lw <- numeric(nv)
    nr <- max(unlist(lapply(z, NROW)))
    for (i in 1:nv) {
        sms <- z[[i]]
        if (is.matrix(sms)) {
            cn <- paste(nm[i], gsub("^ +", "", colnames(sms)),
                sep = ".")
            tmp <- format(sms)
            if (nrow(sms) < nr)
                tmp <- rbind(tmp, matrix("", nr - nrow(sms),
                  ncol(sms)))
            sms <- apply(tmp, 1, function(x) paste(x, collapse = "  "))
            wid <- sapply(tmp[1, ], nchar, type = "w")
            blanks <- paste(character(max(wid)), collapse = " ")
            pad0 <- floor((wid - nchar(cn, type = "w"))/2)
            pad1 <- wid - nchar(cn, type = "w") - pad0
            cn <- paste(substring(blanks, 1, pad0), cn, substring(blanks,
                1, pad1), sep = "")
            nm[i] <- paste(cn, collapse = "  ")
            z[[i]] <- sms
        }
        else {
            lbs <- format(names(sms))
            sms <- paste(lbs, ":", format(sms, digits = digits),
                "  ", sep = "")
            lw[i] <- nchar(lbs[1], type = "w")
            length(sms) <- nr
            z[[i]] <- sms
        }
    }
    z <- unlist(z, use.names = TRUE)
    dim(z) <- c(nr, nv)
    blanks <- paste(character(max(lw) + 2), collapse = " ")
    pad <- floor(lw - nchar(nm, type = "w")/2)
    nm <- paste(substring(blanks, 1, pad), nm, sep = "")
    dimnames(z) <- list(rep.int("", nr), nm)
    attr(z, "class") <- c("table")
    z
})

## as.list() method
as.list.item.list <- function(x,...)structure(x@.Data,names=x@names)

setMethod("head",signature(x="data.set"),
          function(x,n=20,...){
              y <- utils::head.matrix(x,n=n,...)
              rownames(y) <- rownames(x)[1:n]
              return(y)
          })
setMethod("tail",signature(x="data.set"),
          function(x,n=20,...){
              y <- utils::tail.matrix(x,n=n,...)
              rownames(y) <- rownames(x)[seq.int(to=nrow(x),length.out=n)]
              return(y)
          })


as.data.table.data.set <- function(x, ...){
  dataf <- as.data.frame(as.list(x),
                        row.names=if(length(row.names)) rownames
                                  else x@row_names)
  as.data.table(dataf,...)
}

# setMethod("as.POSIXct","numeric.item",function(x,...){
#    as.POSIXct(x@.Data,...)
# })


as.POSIXct.datetime.item <- function(x, tz = "", ...){
   origin <- if(is.null(x@origin)) "1970-01-01" else x@origin
   if(missing(tz)) {
      
      tz <- if(is.character(x@tzone) && !nzchar(x@tzone)) x@tzone else "UTC"
   }
   as.POSIXct(x@.Data,
      tz=tz,
      origin=origin)
}

setMethod("as.item","POSIXct",function(x,...){

   annotation <-new("annotation",NULL)
   new("datetime.item",
       as.numeric(x),
       tzone=attr(x,"tzone"),
       origin=NULL,
       value.labels=NULL,
       value.filter=NULL,
       measurement="date/time",
       annotation=annotation)
})

setMethod("as.item","datetime.item",as_item_item)


setMethod("[",signature(x="datetime.item",i="numeric",j="missing",drop="missing"),function(x,i){
    new("datetime.item",x@.Data[i],
        origin=x@origin,
        tzone=x@tzone,
        value.labels=labels(x),
        value.filter=value.filter(x),
        measurement=measurement(x),
        annotation=annotation(x)
        )
})

setMethod("[",signature(x="datetime.item",i="logical",j="missing",drop="missing"),function(x,i){
    new("datetime.item",x@.Data[i],
        origin=x@origin,
        tzone=x@tzone,
        value.labels=labels(x),
        value.filter=value.filter(x),
        measurement=measurement(x),
        annotation=annotation(x)
        )
})



as.data.frame.datetime.item <- function (x, row.names = NULL, optional = FALSE, ...)
{
    nrows <- length(x)
    nm <- paste(deparse(substitute(x), width.cutoff = 500), collapse = " ")
    if (is.null(row.names)) {
        if (nrows == 0L)
            row.names <- character(0L)
        else row.names <- .set_row_names(nrows)
    }

    x <- as.POSIXct.datetime.item(x)

    value <- list(x)
    if (!optional)
        names(value) <- nm
    attr(value, "row.names") <- row.names
    class(value) <- "data.frame"
    value
}

str.datetime.item <- function(object,give.head=TRUE,width=getOption("width"),...){
  if(give.head){
    hdr <- " Date-time item"
    cat(hdr,"")
  }
  str(as.POSIXct.datetime.item(object),give.head=FALSE,width=width,...)
}




format.datetime.item <- function(x,justify="right",format="",tz="",usetz=FALSE,...){
  if (missing(tz) && !is.null(tzone <- x@tzone)) tz <- tzone
  if(!nzchar(tz)) tz <- "UTC"
  format(as.POSIXct.datetime.item(x),format=format,tz=tz,usetz=usetz,...)
}
#setMethod("format","datetime.item",format.datetime.item)

print.datetime.item <- function(x,
    width=getOption("width"),
    compress=FALSE,
    usetz=TRUE,...,print.gap=NULL){
    if(length(x)){
      mkdots <- function(n) paste(rep(".",n),collapse="")
      pg <- if(is.null(print.gap) || compress) 1 else print.gap
      l <- length(x)
      if(compress) {
        x <- x[seq_len(min(width,l))]
        x <- format.datetime.item(x,usetz=usetz,...)
        x <- trimws(x)
        xw <- cumsum(nchar(x,"w")+1)
        hdr <- paste(" [","1:",length(x),"]",sep="")
        width <- width - nchar(hdr)
        if(any(xw > width)){
          dots <- mkdots(3)
          width <- width - nchar(dots,"w") - 1
          upto <- max(which(xw <= width))
          x <- trimws(format(c(x[seq_len(upto)],dots)))
        }
        cat(hdr,x)
        cat("\n")
      }
      else
        print.default(
              format.datetime.item(x,usetz=usetz,...),
              quote=FALSE,print.gap=print.gap)
    }
    else
      print(as.POSIXct.datetime.item(x),print.gap=print.gap)
}

setMethod("show","datetime.item",function(object){
  tzone <- if(length(object@tzone)) object@tzone else "UTC"
  cat("\nDate/time item",
    if(length(description(object))) sQuote(description(object)) else NULL,
    paste("(",
          "length = ",length(object),
          ", time zone = ",tzone,
          ")",sep=""),
    "\n\n")
  print.datetime.item(object,width=getOption("width"),compress=TRUE,usetz=FALSE)
})
setMethod("print","datetime.item",print.datetime.item)


setMethod("summary","datetime.item",function(object,...,maxsum=100,digits=3){
  ism <- is.missing(object)
  xvalid <- as.POSIXct.datetime.item(object[!ism])
  summary(xvalid)
})

missNAtab <- function(ism,isna,weights=NULL){
  if(!length(weights))
      weights <- rep(1,length(isna))
  isvalid <- !isna & !ism
  isomiss <- ism & !isna
  counts <- c(
      Valid = sum(weights*isvalid),
      "NA" = sum(weights*isna),
      "Other missing" = sum(weights*isomiss),
      Total = sum(weights)
  )
  perc <- 100*counts/counts[length(counts)]
  perc[length(counts)] <- NA
  cbind(N=counts,
        Percent=perc)
}


setMethod("codebookEntry","datetime.item",function(x,weights,...){
  annotation <- annotation(x)
  filter <- x@value.filter
  spec <- c(
            "Storage mode:"=storage.mode(x),
            "Measurement:"="Date/time",
            "Origin:"=x@origin
            )
  if(length(filter)) spec <- c(spec,
                              switch(class(filter),
                                        missing.values = c("Missing values:" = format(filter)),
                                        valid.values   = c("Valid values:"   = format(filter)),
                                        valid.range    = c("Valid range:"    = format(filter))
                                        ))
  ism <- is.missing(x)
  isna <- is.na(x)

  if(any(ism | isna)){
      tab <- missNAtab(ism,isna)
    if(length(weights)){
      wtab <- missNAtab(ism,isna,weights)
      tab <- collect(unweighted=tab,
                     weighted=tab)
    }
    else
      tab <- array(tab,
                   dim=c(dim(tab),1),
                   dimnames=c(dimnames(tab),
                              list(NULL)))
    attr(tab,"title") <- "Valid and missing values"
  } else
    tab <- integer(0)

  x <- as.POSIXct(x)
  descr <- Descriptives(x)
  # if(length(weights)){
  #     wdescr <- Descriptives(x,weights)
  #     descr <- collect(unweighted=format(descr),
  #                      weighted=format(wdescr))
  # }
  # else 
  descr <- as.matrix(format(descr))

  stats <- list(tab=tab,
                descr=descr)
  new("codebookEntry",
    spec = spec,
    stats = stats,
    annotation = annotation
  )
})

## Methods for auxiliar functions

xtfrm.datetime.item <- function(x) x@.Data
mtfrm.datetime.item <- function(x) x@.Data

setMethod("as.character","datetime.item",function(x,...)
   format(as.POSIXct.datetime.item(x))
)

unique.datetime.item <- unique.item.vector

dummy_labels <- function(x){
  ux <- sort(unique(as.character(x)))
  new("value.labels",ux,values=ux)
}


contr <- function(type,...){
  call <- match.call()
  contr.fun <- as.name(paste("contr",type,sep="."))
  args <- list(n=quote(n),...,contrasts=quote(contrasts))
  fun <- function(n,contrasts=TRUE) NULL
  body(fun) <- as.call(c(contr.fun,args))
  fun
}

setMethod("contrasts","ANY",function(x,contrasts=TRUE,...)stats::contrasts(x,contrasts=contrasts,...))

## Copied from stats:contrasts and modified
## Original copyright (C) 1995-2013 The R Core Team

setMethod("contrasts","item",function(x,contrasts=TRUE,...)
{
    if(measurement(x) %nin% c("nominal","ordinal"))
      warning("contrasts(x,...) called with non-categorical x")
    if(!length(vl <- labels(x))) vl <- dummy_labels(x) # stop("cannot obtain contrasts for unlabelled item")
    vl <- vl[is.valid2(vl@values,x@value.filter)]
    nvl <- length(vl@values)
    labs <- vl@.Data
    if (!contrasts)
        return(structure(diag(nvl), dimnames = list(labs,
                                                    labs)))
    ctr <- attr(x, "contrasts")
    if (is.null(ctr)) {
        ctrname <- getOption("contrasts")[[if (is.nominal(x)) 1 else 2]]
        ctr <- get(ctrname, mode = "function", envir = parent.frame())(labs,
                            contrasts = contrasts)
    }
    else if (is.character(ctr)){
        ctr <- get(ctr, mode = "function", envir = parent.frame())(labs,
                            contrasts = contrasts)
    }
    else if (is.function(ctr)){
        ctr <- ctr(labs,contrasts = contrasts)
    }
    else if (is.matrix(ctr) && nrow(ctr) != nvl){
        warning("contrast matrix has wrong rows, deleting it")
        ctr <- NULL
    }
    ctr
})

## Copied and mobified from file 'contrasts.R' in the R package 'stats'.
## The original copyright notice says:
## """
##  File src/library/stats/R/contrast.R
##  Part of the R package, https://www.R-project.org
##
##  Copyright (C) 1995-2021 The R Core Team
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
## """
setMethod("contrasts<-","item",function(x,how.many,value){
    if(measurement(x) %nin% c("nominal","ordinal"))
      warning("contrasts(x,...) called with non-categorical x")
    if(!length(vl <- labels(x))) vl <- dummy_labels(x) #stop("cannot obtain contrasts for unlabelled item")
    vl <- vl[is.valid2(vl@values,x@value.filter)]
    nvl <- length(vl@values)
    labs <- vl@.Data

    if (nvl < 2)
        stop("contrasts can be applied only to factors with 2 or more levels")
    if (is.numeric(value)) {
        value <- as.matrix(value)
        if (nrow(value) != nvl)
            stop("wrong number of contrast matrix rows")
        n1 <- if (missing(how.many)) nvl - 1
              else how.many
        nc <- ncol(value)
        rownames(value) <- labs
        if (nc < n1) {
            cm <- qr(cbind(1, value))
            if (cm$rank != nc + 1)
                stop("singular contrast matrix")
            cm <- qr.qy(cm, diag(nvl))[, 2:nvl]
            cm[, 1:nc] <- value
            dimnames(cm) <- list(levels(x), NULL)
            if (!is.null(nmcol <- dimnames(value)[[2]]))
                dimnames(cm)[[2]] <- c(nmcol,
                              rep.int("", n1 - nc))
        }
        else cm <- value[, 1:n1, drop = FALSE]
    }
    else if (is.function(value))
        cm <- value
    else if (is.character(value))
        cm <- value
    else if (is.null(value))
        cm <- NULL
    else stop("numeric contrasts or contrast name expected")
    attr(x, "contrasts") <- cm
    x
})

## Copied and mobified from file 'contrasts.R' in the R package 'stats'.
## The original copyright notice says:
## """
##  File src/library/stats/R/contrast.R
##  Part of the R package, https://www.R-project.org
##
##  Copyright (C) 1995-2021 The R Core Team
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
## """
contr.treatment <- function (n, base = 1, contrasts = TRUE)
{
    if (is.numeric(n) && length(n) == 1) {
        if (n > 1)
            levs <- 1:n
        else stop("not enough degrees of freedom to define contrasts")
    }
    else {
        levs <- n
        n <- length(n)
    }
    contr <- array(0, c(n, n), list(levs, levs))
    diag(contr) <- 1
    if (contrasts) {
        if (n < 2)
            stop(gettextf("contrasts not defined for %d degrees of freedom",
                n - 1), domain = NA)
        if (is.character(base)){
            base <- match(base,rownames(contr))
            if(is.na(base)) stop("Undefined baseline category")
        }
        if (base < 1 | base > n)
            stop("baseline group number out of range")
        contr <- contr[, -base, drop = FALSE]
    }
    contr
}

## Copied and mobified from file 'contrasts.R' in the R package 'stats'.
## The original copyright notice says:
## """
##  File src/library/stats/R/contrast.R
##  Part of the R package, https://www.R-project.org
##
##  Copyright (C) 1995-2021 The R Core Team
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
## """
contr.sum <- function (n, base = NULL, contrasts = TRUE)
{
    if (is.numeric(n) && length(n) == 1)
        levs <- 1:n
    else {
        levs <- n
        n <- length(n)
    }
    contr <- array(0, c(n, n), list(levs, levs))
    diag(contr) <- 1
    if (contrasts) {
        if (n < 2)
            stop(paste("Contrasts not defined for", n - 1, "degrees of freedom"))
        if (is.null(base)) base <- n
        if (is.character(base)){
            base <- match(base,rownames(contr))
            if(is.na(base)) stop("Undefined baseline category")
        }
        if (base < 1 | base > n)
            stop("Baseline group number out of range")
        contr <- contr[, -base, drop = FALSE]
        contr[base,] <- -1
    }
    contr
}

## Copied and mobified from file 'contr.sdif.R' in the R package 'stats'.
## The original copyright notice says:
## """
## file MASS/R/contr.sdif.R
## copyright (C) 1994-2015 W. N. Venables and B. D. Ripley
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 or 3 of the License
##  (at your option).
## """
contr.sdif <- function (n, contrasts = TRUE)
{
    if (is.numeric(n) && length(n) == 1) {
        if (n%%1 || n < 2)
            stop("invalid number of levels")
        lab <- as.character(seq(n))
    }
    else {
        lab <- as.character(n)
        n <- length(n)
        if (n < 2)
            stop("invalid number of levels")
    }
    if (contrasts) {
        contr <- col(matrix(nrow = n, ncol = n - 1))
        upper.tri <- !lower.tri(contr)
        contr[upper.tri] <- contr[upper.tri] - n
        structure(contr/n, dimnames = list(lab, paste(lab[-1],
            lab[-n], sep = "/")))
    }
    else structure(diag(n), dimnames = list(lab, lab))
}

ftable_format_stdstyle <- c(
  "padding-top"="3px",
  "padding-bottom"="3px",
  "padding-left"="0.5ex",
  "padding-right"="0.5ex",
  "margin-top"="0px",
  "margin-bottom"="0px",
  "border-style"="none",
  "border-width"="0"
)

style_ftab_global <- function(id=NULL,class=NULL,style=ftable_format_stdstyle,margin="2ex auto") {

    table_tmpl <- "
      table<<class-or-id>> {
         border-collapse: collapse; border-style: none; margin: <<margin>>;
      }"
    table_tr_tmpl <- "
      table<<class-or-id>> tr {
          border-style: none;
      }"
    table_td_tmpl <- "
      table<<class-or-id>> td {
        padding-top: <<padding-top>>; 
        padding-bottom: <<padding-bottom>>; 
        padding-left: <<padding-left>>; 
        padding-right: <<padding-right>>; 
        margin: 0px; 
        margin-top: <<margin-top>>;
        margin-bottom: <<margin-bottom>>;
        border-style: <<border-style>>; 
        border-width: <<border-width>>; 
      }"

    class_or_id <- class_id_selector(id=id,class=class)

    table_style <- fillin(table_tmpl,
                          c("class-or-id"=class_or_id,
                            margin=margin))
    table_tr_style <- fillin(table_tr_tmpl,
                             c("class-or-id"=class_or_id))
    table_td_style <- fillin(table_td_tmpl,
                             c("class-or-id"=class_or_id,
                               style))
    paste(table_style,table_tr_style,table_td_style)
}

style_ftab_rule <- function(id=NULL,class=NULL,top=FALSE,bottom=FALSE,rulewidth=1,rows=NULL,firstcol=NULL){
    if(length(rows)){
        class_or_id <- class_id_selector(id=id,class=class)
        rulewidth <- paste0(rulewidth,"px")
        selector <- character(0)
        if(length(firstcol)) {
            selector_tmpl <- "table<<class-or-id>> tr:nth-child(<<row>>) td:nth-child(n+<<firstcol>>)"
            for(row in rows){
                selector <- c(selector,
                              fillin(selector_tmpl,c("class-or-id"=class_or_id,row=row,firstcol=firstcol)))
            }
        }
        else {
            selector_tmpl <- "table<<class-or-id>> tr:nth-child(<<row>>)"
            for(row in rows){
                selector <- c(selector,
                              fillin(selector_tmpl,c("class-or-id"=class_or_id,row=row)))
            }
        }
        selector <- paste(selector,collapse=",\n")
        style_tmpl <- "<<selector>>{"
        if(top){
            style_tmpl <- paste(style_tmpl,
                                "     border-top: <<rulewidth>> solid;",
                                sep="\n")
        }
        if(bottom){
            style_tmpl <- paste(style_tmpl,
                                "     border-bottom: <<rulewidth>> solid;",
                                sep="\n")
        }
        style_tmpl <- paste(style_tmpl,"}",sep="\n")

        fillin(style_tmpl,c(selector=selector,rulewidth=rulewidth))
    } else ""
}

style_ftab_cols <- function(id=NULL,class=NULL,cols=NULL,style=""){
    if(length(cols)){
        class_or_id <- class_id_selector(id=id,class=class)
        selector_tmpl <- "table<<class-or-id>> td:nth-child(<<col>>)"
        selector <- character(0)
        for(col in cols){
            selector <- c(selector,
                          fillin(selector_tmpl,c("class-or-id"=class_or_id,col=col)))
        }
        selector <- paste(selector,collapse=",\n")
        paste0(selector,"{",style,"}")
    }
}

style_ftab_header <- function(id=NULL,class=NULL,style=""){
    class_or_id <- class_id_selector(id=id,class=class)
    selector_tmpl <- "table<<class-or-id>> td:nth-child(n of .header)"
    selector <- fillin(selector_tmpl,c("class-or-id"=class_or_id,col=col))
    paste0("\n",selector,"{",style,"}")
}

ftab_html_env <- new.env()
ftab_html_env$counter <- 1

format_html.ftable <- function(x,
                               show.titles=TRUE,
                               digits=0,
                               format="f",
                               toprule=2,midrule=1,bottomrule=2,
                               split.dec=TRUE,
                               style=ftable_format_stdstyle,
                               margin="2ex auto",
                               ...){
    
    row.vars <- attr(x,"row.vars")
    col.vars <- attr(x,"col.vars")
    n.row.vars <- length(row.vars)
    n.col.vars <- length(col.vars)
    n <- nrow(x)
    m <- ncol(x)
    d <- digits
    digits <- integer(m)
    digits[] <- d
    
    fo <- format
    format <- integer(m)
    format[] <- fo
    
    body <- array("",dim=dim(x))
    for(i in seq(along=digits)) {
    #print(digits[i])
        body[,i] <- formatC(x[,i],digits=digits[i],format=format[i])
    }
    
    body <- array(trimws(body),dim=dim(x))
    if(getOption("html.use.ampersand",FALSE))
        body[] <- gsub("-","&minus;",body[],fixed=TRUE)
    else
        body[] <- gsub("-","\u2212",body[],fixed=TRUE)
    
    dot_cols <- integer(0)

    if(split.dec){
        tmp <- spltDec(body)
        body <- html_td_spltDec(tmp)
        dim(body) <- dim(x)
        colspan <- 3L
        dot_cols <- (1:m-1)*3 + 2
    }
    else {
        body <- html_td(body,vectorize=TRUE)
        dim(body) <- dim(x)
        colspan <- 1L
    }
    
    leaders <- array(list(),dim=c(n,n.row.vars))
    if(show.titles)
        leaders <- cbind(leaders,"")

    mm <- 1
    for(j in rev(1:n.row.vars)){
        rv <-row.vars[[j]]
        nrv <- length(rv)
        nn <- n/mm
        i <- (1:nn)*mm - mm + 1
        leaders[i,j] <- rv
        mm <- mm*nrv
    }
    for(i in 1:n){
        leaders[i,] <- html_td(leaders[i,],
                               vectorize=TRUE) 
    }
    nl <- ncol(leaders)
    body <- cbind(leaders,body)
    nr <- nrow(body)

    body <- as.html_group(apply(body,1,html_tr))
    
    header <- list()
    mm <- 1
    for(i in rev(1:n.col.vars)){
        cv <- col.vars[[i]]
        ncv <- length(cv)
        if(split.dec)
            attribs <- list(colspan=mm*3)
        else
            attribs <- list(colspan=mm)
        mm <- mm*ncv
        cv <- rep(cv,m%/%mm)
        
        if(show.titles){
            if(n.col.vars == 1){
                htmp1 <- html_td(c(names(row.vars),""),
                                 vectorize=TRUE)
            }
            else {
                if(i == n.col.vars){
                    htmp1 <- html_td(c(names(row.vars),paste0(names(col.vars)[i],":")),
                                     vectorize=TRUE)
                }
                else
                    htmp1 <- html_td(c(rep("",n.row.vars),paste0(names(col.vars)[i],":")),
                                     vectorize=TRUE)
            }      
        }
        else 
            htmp1 <- html_td(rep("",ncol(leaders)),
                             vectorize=TRUE)
        attribs$class <- "header"
        htmp2 <- setAttribs(html_td(cv,vectorize=TRUE),attribs)
        header <- c(list(c(htmp1,htmp2)),header)
    }
    if(show.titles && n.col.vars == 1){
        if(nzchar(names(col.vars))){
            htmp1 <- html_td(rep("",ncol(leaders)),
                             vectorize=TRUE)
            colspan <- ncol(x)
            if(split.dec) 
                colspan <- colspan*3
            attribs <- list(colspan=colspan)
            attribs$class <- "header"
            htmp2 <- setAttribs(html_td(names(col.vars),vectorize=TRUE),attribs)
            header <- c(list(c(htmp1,htmp2)),header)
        }
    }
    header <- html_tr(header,vectorize=TRUE)
    nh <- length(header)

    ftab_id <- paste0("ftable-",ftab_html_env$counter)
    res <- html_table(c(header,body),class="ftable",id=ftab_id)

    ftab_html_env$counter <- ftab_html_env$counter + 1

    style_global <- style_ftab_global(id=ftab_id,style=style,margin=margin)
    style_toprule <- style_ftab_rule(id=ftab_id,rulewidth=toprule,top=TRUE,
                                       rows=1)
    style_bottomrule <- style_ftab_rule(id=ftab_id,rulewidth=bottomrule,bottom=TRUE,
                                          rows=nr+nh)
    style_midrule <- style_ftab_rule(id=ftab_id,rulewidth=midrule,bottom=TRUE,
                                          rows=nh)

    style_content <- paste(
        style_global,
        style_toprule,
        style_midrule,
        style_bottomrule,
        sep="\n"
    )

    if(nh > 1){
        style_grouprule <- style_ftab_rule(id=ftab_id,rulewidth=midrule,bottom=TRUE,
                                           firstcol=nl+1,
                                           rows=1:(nh-1))
        style_content <- paste(
            style_content,
            style_grouprule,
            sep="\n"
        )
    }

    if(split.dec){
        dot_style <- "
         padding-left: 0px;
         padding-right: 0px;
         text-align: center; 
         width: .2ex;
         /*background-color: blue;*/
        "
        before_dot_style <- "
         padding-left: 0.5ex;
         padding-right: 0px;
         text-align: right; 
         /*background-color: red;*/
        "
        behind_dot_style <- "
         padding-left: 0px;
         padding-right: 0.5ex;
         text-align: left; 
         /*background-color: green;*/
        "
        dot_cols <- dot_cols + nl - 1
        style_dots <- style_df_cols(id=ftab_id,cols=dot_cols+1,style=dot_style)
        style_before_dots <- style_df_cols(id=ftab_id,cols=dot_cols,style=before_dot_style)
        style_behind_dots <- style_df_cols(id=ftab_id,cols=dot_cols+2,style=behind_dot_style)
        style_content <- paste(style_content,
                               style_dots,
                               style_before_dots,
                               style_behind_dots,sep="\n")
    }
    header_style <- "
         padding-left: 0.5ex;
         padding-right: 0.5ex;
         text-align: center; 
         /*background-color: gray;*/
        "
    style_header <- style_ftab_header(id=ftab_id,style=header_style)
    style_content <- paste(style_content,
                           style_header,
                           "\n")

    style_element <- html("style",style_content,linebreak=TRUE)
    res <- html_group(style_element,res)
    res <- as.character(res)
    return(res)
}

setGeneric("prep_for_tibble",function(x,...)standardGeneric("prep_for_tibble"))
setMethod("prep_for_tibble",signature(x="item.vector"),function(x,...)
    prep_for_tibble_item_vector(x))

prep_for_tibble_item_vector <- function(x,...){
    y <- if(inherits(x,"character.item")) as.character(x)
         else if(is.ordinal(x)) as.ordered(x)
         else if(is.nominal(x)) as.factor(x)
         else as.vector(x)
    d <- description(x)
    if(length(d))
        attr(y,"label") <- d
    return(y)
}

as_tibble.data.set <- function(x, ...){
    y <- lapply(x@.Data,
                prep_for_tibble_item_vector)
    names(y) <- names(x)
    attr(y,"row.names") <- x@row_names
    class(y) <- c("tbl_df","tbl","data.frame")
    return(y)
}

sanitize_labels <- function(labels){
    labn <- names(labels)
    if(length(unique(labels)) < length(labn)){
        dup <- duplicated(labels)
        warning(sprintf("Dropped non-unique label(s) %s",
                        paste(paste(labn[dup],"=",labels[dup]),collapse=", ")),
                call.=FALSE,immediate.=TRUE)
        labels <- labels[!dup]
    }
    labels
}

setOldClass("labelled")
setMethod("as.item",signature(x="labelled"),function(x,...){
    annotation <- c(description=attr(x,"label",exact=TRUE))
    labels <- attr(x,"labels",exact=TRUE)
    labels <- sanitize_labels(labels)
    attributes(x) <- NULL
    as.item(x,labels=labels,
            annotation=annotation)
})
setMethod("codebookEntry","labelled",function(x){
    x <- as.item(x)
    annotation <- annotation(x)
    spec <- c(
        "Storage mode:"=storage.mode(x),
        "Measurement:"="undefined"
    )
    new("codebookEntry",
        spec = spec,
        stats = codebookStatsCateg(x),
        annotation = annotation
        )
})

# if(!requireNamespace("haven",quietly = TRUE))
    setOldClass("haven_labelled")
setMethod("as.item",signature(x="haven_labelled"),function(x,...){
    annotation <- c(description=attr(x,"label",exact=TRUE))
    labels <- attr(x,"labels",exact=TRUE)
    labels <- sanitize_labels(labels)
    attributes(x) <- NULL
    as.item(x,labels=labels,
            annotation=annotation)
})
setMethod("codebookEntry","haven_labelled",function(x){
    x <- as.item(x)
    annotation <- annotation(x)
    spec <- c(
        "Storage mode:"=storage.mode(x),
        "Measurement:"="undefined"
    )
    new("codebookEntry",
        spec = spec,
        stats = codebookStatsCateg(x),
        annotation = annotation
        )
})



# if(!requireNamespace("haven",quietly = TRUE))
    setOldClass("haven_labelled_spss")
setMethod("as.item",signature(x="haven_labelled_spss"),function(x,...){
    annotation <- c(description=attr(x,"label",exact=TRUE))
    labels <- attr(x,"labels",exact=TRUE)
    labels <- sanitize_labels(labels)
    mis_range <- attr(x,"na_range")
    mis_values <- attr(x,"na_values")
    attributes(x) <- NULL
    value_filter <- new("missing.values",
                        filter=mis_values,
                        range=mis_range)
    as.item(x,labels=labels,
            annotation=annotation,
            value.filter=value_filter)
})
setMethod("codebookEntry","haven_labelled_spss",function(x){
    x <- as.item(x)
    annotation <- annotation(x)
    filter <- x@value.filter
    spec <- c(
        "Storage mode:"=storage.mode(x),
        "Measurement:"="undefined"
    )
    if(length(filter)) spec <- c(spec,
                                 switch(class(filter),
                                        missing.values = c("Missing values:" = format(filter)),
                                        valid.values   = c("Valid values:"   = format(filter)),
                                        valid.range    = c("Valid range:"    = format(filter))
                                        ))
    new("codebookEntry",
        spec = spec,
        stats = codebookStatsCateg(x),
        annotation = annotation
        )
})

setGeneric("as_haven",function(x,...)standardGeneric("as_haven"))
setMethod("as_haven",signature(x="data.set"),function(x,user_na=FALSE,...){
    y <- lapply(x@.Data,as_haven,user_na=user_na,...)
    names(y) <- names(x)
    names(y) <- gsub(".","_",names(y),fixed=TRUE)
    attr(y,"row.names") <- x@row_names
    class(y) <- c("tbl_df","tbl","data.frame")
    return(y)
})
setMethod("as_haven",signature(x="item.vector"),function(x,user_na=FALSE,...){
    y <- x@.Data
    if(length(description(x)))
        attr(y,"label") <- description(x)
    if(is.character(x))
        labels(x) <- NULL
    if(length(labels(x))){
        l <- as.vector(labels(x))
        storage.mode(y) <- "integer"
        storage.mode(l) <- "integer"
        attr(y,"labels") <- l
        ms <- missing.values(x)
        if(user_na && length(ms)){
            attr(y,"na_values") <- as.integer(ms@filter)
            attr(y,"na_range") <- as.integer(ms@range)
            class(y) <- "haven_labelled_spss"
        } else {
            ism <- is.missing(x)
            y[ism] <- NA
            class(y) <- "haven_labelled"
        }
    }
    return(y)
})

any_dup <- function(x) length(x) && any(duplicated(x@.Data))

which_dup_lab <- function(x){
    ii <- duplicated(x@.Data)
    dup_lab <- unique(x@.Data[ii])
    structure(lapply(dup_lab,get_labs,labels=x@.Data,value=x@values),
              names=dup_lab)
}

get_labs <- function(which,labels,values){
    i <- labels == which
    values[i]
}
 
duplicated_labels <- function(x) UseMethod("duplicated_labels")

duplicated_labels.item <- function(x){
    l <- labels(x)
    ii <- duplicated(l@.Data)
    if(any(ii)){
        dup_lab <- unique(l@.Data[ii])
        structure(lapply(dup_lab,
                         get_labs,
                         labels=l@.Data,
                         value=l@values),
                  names=dup_lab,
                  class="dupLabelsReport1")
    }
    else NULL
}

print.dupLabelsReport1 <- function(x,...){
    n <- paste0(names(x),":")
    l <- sapply(x,paste,collapse=", ")
    r <- cbind(format(n),format(l))
    r <- apply(r,1,paste,collapse=" ")
    writeLines(r)
}

duplicated_labels.item.list <- function(x){
    ll <- lapply(as.list(x),labels)
    ii <- sapply(ll,any_dup)
    ll <- ll[ii]
    d <- description(x)
    d <- d[ii]
    if(length(ll))
        structure(lapply(ll,which_dup_lab),
                  description=d,
                  class="dupLabelsReport")
    else NULL
}

print.dupLabelsReport <- function(x,...){
    width <- getOption("width",80)
    toprule <- paste(rep("=",width),collapse="")
    midrule <- paste(rep("-",width),collapse="")
    n <- names(x)
    d <- attr(x,"description")
    for(i in seq_along(x)){
        cat("\n",toprule,sep="")
        cat("\n ",n[i],": ",sQuote(d[i]),sep="")
        cat("\n",midrule,"\n",sep="")
        x.i <- x[[i]]
        l.i <- sapply(x.i,paste,collapse=", ")
        w <- width - max(nchar(l.i)) - 5
        n.i <- names(x.i)
        cutit <- nchar(n.i) > w
        n.i[cutit] <- paste0(substr(n.i[cutit],start=1,stop=w-3),"...")
        n.i <- paste0(n.i,":")
        r.i <- cbind(" ",format(n.i),format(l.i))
        r.i <- apply(r.i,1,paste,collapse=" ")
        writeLines(r.i)
    }
}

warn_if_duplicate_labels <- function(variables){
    ll <- lapply(variables,labels)
    ii <- sapply(ll,any_dup)
    if(any(ii)){
        n <- names(variables)[ii]
        nn <- paste(n,collapse=", ")
        nn <- strwrap(nn,prefix="  ")
        nn <- paste(nn,collapse="\n")
        warning(sprintf("%d variables have duplicated labels:\n%s",
                        length(n),
                        nn),
             call.=FALSE)
    }
}

spss.file <- function(file,...){
    if(grepl("\\.por$",file) || grepl("\\.POR$",file))
        spss.portable.file(file,...)
    else
        spss.system.file(file,...)
}

spss.system.file <- function(
    file,
    varlab.file=NULL,
    codes.file=NULL,
    missval.file=NULL,
    count.cases=TRUE,
    to.lower=getOption("spss.sav.to.lower",FALSE),
    iconv=TRUE,
    encoded=getOption("spss.sav.encoding","cp1252"),
    ignore.scale.info = FALSE,
    negative2missing = FALSE
    ){
    file <- path.expand(file)
    check.file(file,error=TRUE)
    ptr <- .Call("NewSysFile",file)
    
    data.spec <- parseSysHeader(ptr)
    types <- data.spec$types

    varnames <- names(data.spec$variables)
    
    varlabs <- lapply(data.spec$variables,"[[",i="label")
    varlabs <- unlist(varlabs[sapply(varlabs,length)>0])  
    vallabs <- data.spec$value.labels
    vallabs.vars <- lapply(vallabs,"[[",i="variables")
    vallabs.vars <- lapply(vallabs.vars,function(i)names(types)[i])
    vallab.tmp <- lapply(seq_along(vallabs),function(i){
                        ans <- list()
                        ans[vallabs.vars[[i]]] <- list(vallabs[[i]]$labels)
                        ans
                      })
    vallabs <- unlist(vallab.tmp,recursive=FALSE)
    names.vallabs <- names(vallabs)
    string.vallabs <- types[names.vallabs] > 0
    vallabs[string.vallabs] <- lapply(vallabs[string.vallabs],spss.string.vallabs)
    
    missings <- mapply(function(v,r)list(values=v,range=r),
                          data.spec$missings$values,
                          data.spec$missings$ranges,
                          SIMPLIFY=FALSE
                          )
    

                          
    missings <- lapply(missings,function(x)if(!length(x$values) & !length(x$range))NULL else x)
    missings <- missings[sapply(missings,length)>0]

    varprintfmt <- lapply(data.spec$variables,"[[",i="print")
    varprintfmt <- sapply(varprintfmt,"[",i=3)
    vardatetime <- varprintfmt %in% c(20,22:24,26:30,38,39)
    
    variables <- vector(length(types),mode="list")
    names(variables) <- names(types)
    variables[types==0] <- list(new("double.item"))
    variables[types>0] <- list(new("character.item"))
    variables[types<0] <- NULL

    variables[vardatetime] <- list(new("datetime.item",
                                       tzone="GMT",
                                       origin="1582-10-14"))

    if(length(varlab.file) && check.file(varlab.file,error=TRUE)){
      message("using ",varlab.file)
      varlabs <- spss.parse.variable.labels(varlab.file,
                                            iconv=iconv,
                                            encoded=encoded)
      }
    if(length(codes.file) && check.file(codes.file,error=TRUE)){
      message("using ",codes.file)
      vallabs <- spss.parse.value.labels(codes.file,
                                         iconv=iconv,
                                         encoded=encoded)
      }
    if(length(missval.file) && check.file(missval.file,error=TRUE)){
      message("using ",missval.file)
      missings <- spss.parse.missing.values(missval.file)
      }

    if(length(varlabs))
      variables[names(varlabs)] <- mapply("description<-",variables[names(varlabs)],varlabs)
    if(length(vallabs)){
        unique_ <- function(x){
            dup <- duplicated(x)
            x[!dup]
        }
        vallabs <- lapply(vallabs,unique_)
        suppressWarnings(variables[names(vallabs)] <- mapply("labels<-",variables[names(vallabs)],vallabs))
    }
    if(length(missings))
      variables[names(missings)] <- mapply("missing.values<-",variables[names(missings)],missings)
    if(negative2missing)
      variables <- lapply(variables,`valid.range<-`,c(0,Inf))

    ncases <- data.spec$header$ncases
    if(ncases < 0){
      if(count.cases){
          .Call("rewind_sysfile",ptr)
          ncases <- .Call("count_cases_sysfile",ptr)
          .Call("rewind_sysfile",ptr)
      }
      else
          ncases <- NA
    }
    attr(ptr,"ncases") <- ncases
    document <- data.spec$document
    data.spec$document <- NULL

    aux <- data.spec$auxiliaries
    if(length(aux$aux_var) && !ignore.scale.info){
      aux_var <- do.call("rbind",aux$aux_var)
      mlev <- aux_var[,1]
      mlev[mlev==0] <- 1
      mlev <- c("nominal","ordinal","interval")[mlev]
      variables <- mapply("measurement<-",variables,mlev)
    }
    
    if(to.lower){
      names(variables) <- tolower(names(variables))
    }

    if(length(aux$aux_enc)){
        encoding2 <- toupper(aux$aux_enc)
        if(encoding2 %in% iconvlist()){
            encoding <- encoding2
            message(sprintf("File character set is '%s'.",encoding))
        }
        else {
            warning(sprintf("File contains unsupported character set '%s', using fallback.",encoding2))
            encoding <- encoded
        }
    }
    else if(length(aux$info_int32)){
        encoding <- aux$info_int32["character_code"]
        enc_table <- c("EBCDICUS"=1,
                   "ASCII"=2,
                   "ASCII"=3,
                   "DEC Kanji"=4,
                   "CP1250"=1250,
                   "CP1252"=1252,
                   "ISO8859-1"=28591,
                   "UTF-8"=65001)
        encoding1 <- names(enc_table)[match(encoding,enc_table)]
        if(encoding1 %in% iconvlist()){
            encoding <- encoding1
            message(sprintf("File character set is '%s'.",encoding))
        }
        else {
            warning(sprintf("File contains unsupported character set '%s', using fallback.",encoding1))
            encoding <- encoded
        }
    }
    else {
        encoding <- encoded
        message("File does not contain character set encoding information, using fallback.")
    }

    if(iconv){
        message(sprintf("Converting character set to UTF-8."))
        variables <- lapply(variables,Iconv,from=encoding,to="UTF-8")
    }
    
    warn_if_duplicate_labels(variables)
    
    new("spss.system.importer",
      variables,
      ptr=ptr,
      varlab.file=varlab.file,
      codes.file=codes.file,
      missval.file=missval.file,
      document=document,
      data.spec=data.spec,
      encoded=encoding
      )
}
setMethod("initialize","spss.system.importer",function(.Object,
                                                       variables,
                                                       ptr,
                                                       varlab.file=character(),
                                                       codes.file=character(),
                                                       missval.file=character(),
                                                       document=character(),
                                                       data.spec,
                                                       encoded=encoded
                                                       ){
     .Object@.Data <- variables
     .Object@ptr <- ptr
     .Object@varlab.file <- as.character(varlab.file)
     .Object@codes.file <- as.character(codes.file)
     .Object@missval.file <- as.character(missval.file)
     .Object@document <- as.character(document)
     .Object@data.spec <- data.spec
     .Object@encoded <- encoded
     .Object
})


setMethod("getNobs","spss.system.importer",function(x){
  ncases <- attr(x@ptr,"ncases")
  if(!length(ncases)) {
    .Call("rewind_sysfile",x@ptr)
    attr(x@ptr,"ncases") <- ncases <- .Call("count_cases_sysfile",x@ptr)
    .Call("rewind_sysfile",x@ptr)
  }
  ncases
})

setMethod("seekData","spss.system.importer",function(x){
  if(!.Call("check_pointer",x@ptr)) stop("pointer is NULL, you need to recreate the object")
  .Call("rewind_sysfile",x@ptr)
})

setMethod("readData","spss.system.importer",
  function(x,n){
    if(!.Call("check_pointer",x@ptr)) stop("pointer is NULL, you need to recreate the object")
    iconv_list(.Call("read_sysfile_data",
      x@ptr,
      what=x,
      ncases=n,
      types=x@data.spec$types),
      encoded=x@encoded)
  })

setMethod("readSlice","spss.system.importer",
  function(x,rows,cols){
    if(!.Call("check_pointer",x@ptr)) stop("pointer is NULL, you need to recreate the object")
    iconv_list(.Call("read_sysfile_slice",x@ptr,
      what=x,
      j=cols,i=rows,
      types=x@data.spec$types),
      encoded=x@encoded)
  })

setMethod("readChunk","spss.system.importer",
  function(x,nrows,cols){
    if(!.Call("check_pointer",x@ptr)) stop("pointer is NULL, you need to recreate the object")
    iconv_list(.Call("read_sysfile_chunk",x@ptr,
      what=x,
      vars=cols,n=nrows,
      types=x@data.spec$types),
      encoded=x@encoded)
  })


setMethod("show","spss.system.importer",
  function(object){
    file.name <- attr(object@ptr,"file.name")
    nobs <- nrow(object)
    nvar <- ncol(object)
    varlab.file <- object@varlab.file
    codes.file <- object@codes.file
    missval.file <- object@missval.file
    cat("\nSPSS system file",sQuote(file.name),"\n\twith ")
    cat(nvar,"variables and ")
    cat(nobs,"observations\n")
    if(length(varlab.file)) cat("\twith variable labels from file",sQuote(varlab.file),"\n")
    if(length(codes.file)) cat("\twith value labels from file",sQuote(codes.file),"\n")
    if(length(missval.file)) cat("\twith missing value definitions from file",sQuote(missval.file),"\n")
})

subset.spss.system.importer <- subset.importer

LaTeXcape <- function(x){
    x <- gsub("$","\\$",x,fixed=TRUE)
    x <- gsub("_","\\_",x,fixed=TRUE)
    x <- gsub("^","\\^",x,fixed=TRUE)
}

checkLaTeXcape <- function(x,where){
    do.warn <- any(grepl("$",x,fixed=TRUE))
    do.warn <- do.warn || any(grepl("_",x,fixed=TRUE))
    do.warn <- do.warn || any(grepl("^",x,fixed=TRUE))
    if(do.warn)
        warning("Unescaped TeX special in ",where)
}

## A variant of 'list' that names its elements just like
# 'data.frame' does:

List <- function(...){
    args <- list(...)
    argnames <- sapply(substitute(c(...))[-1],as.character)
    argtags <- names(args)
    if(length(argtags)){
        nz <- nzchar(argtags)
        argnames[nz] <- argtags[nz]
    }
    names(args) <- argnames
    args
}

html_grid <- function(x,title="",max_width=80,...){
    x <- as.matrix(x)
    n <- nchar(x)
    x[n>max_width] <- paste0(substr(x[n>max_width],start=1,stop=max_width),"...")
    tbody <- paste0("<td>",x,"</td>")
    dim(tbody) <- dim(x)
    tbody <- apply(tbody,1,paste0,collapse="")
    tbody <- paste0("<tr>",tbody,"</tr>")
    tbody <- c("<tbody>",tbody,"</tbody>")
    thead <- paste0("<th>",colnames(x),"</th>")
    thead <- paste0(thead,collapse="")
    thead <- paste0("<tr>",thead,"</tr>")
    thead <- c("<thead>",thead,"</thead>")
    table <- c(thead,tbody)
    table <- c("<table>",table,"</table>")
    style <- c("table { border: 0.5px solid #555; border-collapse:collapse; position: relative; }",
               "thead { background-color: #ddd; border: 0.5px solid #555;  }",
               "td { border: 0.5px solid #555; text-align: right; padding: 3px; white-space: nowrap; }",
               "th { border: 0.5px solid #555; text-align: center; padding: 3px;
                background-color: #ddd; position: sticky; top: 0;}",
               "body { margin: 0; }")
    style <- paste(style,collapse="\n")
    style <- c("<style>",style,"</style>")
    title <- paste0("<title>",title,"</title>")
    head <- c("<head>",title,style,"</head>")
    body <- c("<body>",table,"</body>")
    html <- c("<html>",head,body,"</html>")
    structure(html,class="raw_html")
}



view_html <- function(x,title=deparse(substitute(x)),output,...){
  
  ht <- html_grid(x,title,...)
  
  if(interactive()){
    # Test whether running under RStudio 
    isRStudio <- Sys.getenv("RSTUDIO") == "1"
    if(isRStudio)
      deflt.output <- "file-show"
    else
      deflt.output <- "browser"
  }
  else
    deflt.output <- "stdout"
  
  if(missing(output))
    output <- getOption("html_viewer",deflt.output)
  
  if(mode(output)=="character")
      output <- match.arg(output,c("stdout","browser","file-show"))
  else if(!is.function(output))
      stop("'output' should be either a character string of a function")
  
  if(is.function(output)){
    
    tf <- file.path(tempdir(),title)
    tf <- paste0(tf,".html")
    cat(ht,file=tf)
    
    output(tf)
  }
  else if(nzchar(Sys.getenv("JPY_PARENT_PID"))){
      ## Inside Jupyter 
      return(html_div(ht))
  }     
  else if(output=="stdout") cat(ht)
  else {
    
    tf <- file.path(tempdir(),title)
    tf <- paste0(tf,".html")
    cat(ht,file=tf)
    
    if(output=="file-show")
      file.show(tf,title=deparse(substitute(x)))
    else 
      browseURL(tf)
  }
  
}

percentages <- function(obj,...) UseMethod("percentages")

percentages.table <- function(obj,by=NULL,which=NULL,se=FALSE,ci=FALSE,ci.level=.95,...){
  
  dn <- dimnames(obj)
  ndn <- names(dn)
  
  by <- unique(by)
  which <- unique(which)
  
  lby <- length(by)
  lwi <- length(which)
  
  if(!(lby||lwi)){
    margin <- NULL
  } else if (lby && lwi) {
    
    if(length(intersect(by,which))>0)
      stop("duplicate variables")
    
    all <- union(which,by)
    ii <- match(all,ndn)
    if(any(is.na(ii))) stop("undefined variables")
    if(length(ii)<length(dn)){
      obj <- margin.table(obj,ii)
      return(Recall(obj,by=by,which=which,se=se,ci=ci,ci.level=ci.level,...))
    }
    else {
      margin <- match(by,ndn)
    }
  }
  else if(lby){
    margin <- match(by,ndn)
    if(any(is.na(margin))) stop("undefined variables")
  }
  else if(lwi){
    margin <- match(setdiff(ndn,which),ndn)
    if(any(is.na(margin))) stop("undefined variables")
  }
  
  tab <- obj  # Just another name ...
  ptab <- prop.table(tab,margin=margin)
  if(!ci && !se){
    structure(100*ptab,class=c("percentage.table",class(obj)))
  } else {
    mtab <- margin.table(tab,margin=margin)
    
    dd <- seq_along(dim(ptab))
    
    perm <- c(margin,setdiff(dd,margin))
    rev.perm <- seq_along(perm)
    rev.perm[perm] <- rev.perm
    
    mtab <- aperm(array(mtab,dim=dim(ptab)[perm]),
                  rev.perm)
    dimnames(mtab) <- dimnames(ptab)
    if(se){
      var.tab <- ptab*(1-ptab)/mtab
      se.tab <- sqrt(var.tab)
    }
    if(ci){
      alpha <- (1-ci.level)/2
      lower <- upper <- array(NA,dim=dim(ptab))
      isnull <- ptab == 0 | is.na(ptab)
      isfull <- ptab == 1 | is.na(ptab)
      lower[!isnull] <- qbeta(alpha,tab[!isnull],mtab[!isnull]-tab[!isnull]+1)
      lower[isnull] <- 0
      upper[!isfull] <- qbeta(1-alpha,tab[!isfull]+1,mtab[!isfull]-tab[!isfull])
      upper[isfull] <- 1
    }
    if(!ci){
      res <- 100*cbind(as.vector(ptab),as.vector(se.tab))
      res <- array(res,dim=c(dim(ptab),2),
                   dimnames=c(
                     dimnames(ptab),
                     list(Result=c("Percentage","SE"))
                     ))
    }
    else if(!se){
      res <- 100*cbind(as.vector(ptab),as.vector(lower),as.vector(upper))
      res <- array(res,dim=c(dim(ptab),3),
                   dimnames=c(
                     dimnames(ptab),
                     list(Result=c("Percentage","Lower bound","Upper bound"))
                   ))
    }
    else {
      res <- 100*cbind(as.vector(ptab),as.vector(se.tab),
                       as.vector(lower),as.vector(upper))
      res <- array(res,dim=c(dim(ptab),4),
                   dimnames=c(
                     dimnames(ptab),
                     list(Result=c("Percentage","SE","Lower bound","Upper bound"))
                   ))
    }
    structure(res,class=c("xpercentage.table","percentage.table",class(obj)))
  }
}

percentages.formula <- function(obj,data=parent.frame(),weights=NULL,...){
  
  if(is.table(data))
    tab <- data
  else{
    
    allv <- all.vars(obj)
    allv.formula <- obj
    if(length(obj)==3)
      allv.formula <- reformulate(allv)    
    else 
      allv.formula <- obj
    
    if(!missing(weights)){
          weights <- deparse(substitute(weights))
          allv.formula <- reformulate(all.vars(allv.formula),
                                      response=weights)
    }
    my.call <- match.call()
    xtc <- my.call
    ii <- match(names(formals(stats::xtabs)),names(xtc),0L)
    xtc <- xtc[c(1L,ii)]
    xtc[[1L]] <- quote(stats::xtabs)
    xtc$formula <- allv.formula
    if(is.environment(data)){
        mf <- model.frame(allv.formula,data=data)
        xtc$data <- mf
    }
    tab <- eval(xtc,parent.frame())
  }
  
  if(length(obj)==3){
    which <- all.vars(obj[-3])
    by <- all.vars(obj[-2])
  }
  else {
    which <- all.vars(obj)
    by <- NULL
  }
  percentages.table(tab,by=by,which=which,...)
}

percentages.default <- function(obj,weights=NULL,...){
    lab <- deparse(substitute(obj))
    percentages1(obj,weights=weights,name=lab,...)
}

percentages.list <- function(obj,weights=NULL,...){
    nms <- names(obj)
    if(!length(nms))
        nms <- rep("",length(obj))
    mapply(percentages1,obj,nms,MoreArgs=list(weights=weights,...))
}

percentages.data.frame <- percentages.list

percentages1 <- function(x,weights=NULL,name,...){
    if(length(weights))
    {
        tab <- rowsum(weights,x)
        dnm <- dimnames(tab)
        dim(tab) <- dim(tab)[1]
        dimnames(tab) <- dnm[1]
        class(tab) <- "table"
    }
    else
        tab <- table(x)
    tab <- percentages.table(tab,...)
    names(dimnames(tab))[1] <- name
    tab
}


as.data.frame.percentage.table <- function(x,...){
  res <- NextMethod("as.data.frame")
  rename(res,Freq="Percentage")
}

as.data.frame.xpercentage.table <- function(x,...){
  dx <- dim(x)
  dn <- dimnames(x)
  ld <- length(dx)
  lst.d <- dx[ld]
  xm <- array(x,dim=c(prod(dx[-ld]),dx[ld]))
  tabs <- lapply(1:dx[ld],mcol2df,
                 x=xm,
                 dim=dx[-ld],
                 dimnames=dn[-ld],
                 slicenames=dn[[ld]]
                 )
  Freq.col <- which("Freq"==names(tabs[[1]]))
  result.names <- dimnames(x)$Result
  for(i in seq_along(tabs))
      names(tabs[[i]])[Freq.col] <- result.names[i]
  Reduce(merge,tabs)
}

mcol2df <- function(i,x,dim,dimnames,slicenames){
  y <- structure(x[,i],
                 class="table",
                 dim=dim,
                 dimnames=dimnames)
  y <- as.data.frame(y)
  rename(y,Freq=slicenames[i])
}
  

rename <- function(x,...,gsub=FALSE,fixed=TRUE,warn=TRUE){
    m <- match.call(expand.dots=FALSE)
    subst <- sapply(m$...,as.character)
    if(gsub){
        names.x <- names(x)
        for(i in 1:length(subst)){
            names.x <- gsub(names(subst[i]),subst[i],names.x,fixed=fixed)
        }
        names(x) <- names.x
    }
    else {
        i <- match(names(subst),names(x))
        if(any(is.na(i))) {
            if(warn) warning("unused name(s) selected")
            if(any(!is.na(i)))
                subst <- subst[!is.na(i)]
            i <- i[!is.na(i)]
        }
        if(length(i))
            names(x)[i] <- subst
    }
    return(x)
}

dimrename <- function(x,dim=1,...,gsub=FALSE,fixed=TRUE,warn=TRUE){
    m <- match.call(expand.dots=FALSE)
    subst <- sapply(m$...,as.character)
    if(0 %in% dim){
        dimnames(x) <- rename(dimnames(x),...,gsub=gsub,fixed=fixed,warn=warn)
        dim <- dim[dim!=0]
    }
    if(length(dim)){
        for(i in 1:length(subst)){
            for(j in dim){
                if(gsub)
                    dimnames(x)[[j]] <- gsub(names(subst[i]),subst[i],dimnames(x)[[j]],fixed=fixed)
                else{
                    ii <- match(names(subst[i]),dimnames(x)[[j]])
                    if(any(is.na(ii))) {
                        if(warn) warning("unused dimname(s) selected")
                        if(any(!is.na(ii)))
                            subst[i] <- subst[!is.na(ii)]
                        ii <- ii[!is.na(ii)]
                    }
                    if(length(ii))
                        dimnames(x)[[j]][ii] <- subst[i]
                }
            }
        }
    }
    return(x)
}

colrename <- function(x,...,gsub=FALSE,fixed=TRUE,warn=TRUE){
    m <- as.list(match.call())
    m[[1]] <- as.name("dimrename")
    m <- as.call(c(m[1:2],list(dim=2),m[-(1:2)]))
    eval(m,envir=parent.frame())
}

rowrename <- function(x,...,gsub=FALSE,fixed=TRUE,warn=TRUE){
    m <- as.list(match.call())
    m[[1]] <- as.name("dimrename")
    m <- as.call(c(m[1:2],list(dim=1),m[-(1:2)]))
    eval(m,envir=parent.frame())
}

format_md.codebook <- function(x, ...) {
  out <- mapply(format_md, x = x@.Data, name = names(x),
                   MoreArgs = list(...))
  unlist(out)
}

format_md.codebookEntry <- function(x, name = "", add_rules = TRUE, ...) {
  blank_line <- "\n"
  width <- getOption("width")
  hrule <- if (add_rules) paste(rep("-",width),collapse="")
                     else NULL


  annot <- x@annotation
  description <- annot["description"]
  wording <- annot["wording"]
  if(length(annot)) annot <- annot[names(annot) %nin% c("description","wording")]

  title <- strwrap(if(length(description) && !is.na(description))
                  paste0(name[1]," --- ",sQuote(description))
                  else name[1] ,width=width,prefix="   ")
  
  wording <- if(length(wording) && !is.na(wording))
                strwrap(dQuote(wording),width=width,prefix="   ")
             else NULL

  spec <- paste("  ",names(x@spec),x@spec,"   ")
  tab <- unclass(x@stats$tab)
  descr <- unclass(x@stats$descr)
  
  if(length(tab)){
      
    tab.title <- attr(tab,"title")
    tab.d <- dim(tab)
    tab.dn <- dimnames(tab)
    tab_md <- apply(tab,3,format_cb_table_md)

    tab_rown <- trimws(rownames(tab))
    
    tab_lab <- strsplit(tab_rown," ")

    tab_lab <- lapply(tab_lab,trimws_drpz)
    
    tab_has_lab <- sapply(tab_lab,length) > 1
    tab_has_m <- sapply(sapply(tab_lab,"[",2) == "M",isTRUE)
    
    tab_val <- ifelse(tab_has_lab,sapply(tab_lab,"[",1),"")
    tab_lab <- ifelse(tab_has_lab,lapply(tab_lab,"[",-1,drop=FALSE),tab_lab)
    
    tab_m <- ifelse(tab_has_m,"M","")
    tab_lab <- ifelse(tab_has_m,lapply(tab_lab,"[",-1,drop=FALSE),tab_lab)
    
    tab_lab <- sapply(tab_lab,paste,collapse=" ")

    tab_val <- format_col_md(tab_val,
                             align="r",
                             colname="Values")
    tab_m <- format_col_md(tab_m,
                           align="r",
                           colname="")
    tab_lab <- format_col_md(tab_lab,
                             align="l",
                             colname="Labels")
    tab_ldr <- paste0(tab_val,"|",tab_m,"|",tab_lab)
    tab <- paste0("   |",tab_ldr,tab_md,"|")
  }
  if(!is.matrix(descr)) descr <- NULL
  if(length(descr)){
    descr.rn <- format(paste(rownames(descr),":",sep=""),justify="left")
    if(is.numeric(descr[]))  
       descr[] <- formatC(descr[],format="f",digits=3)
    descr[] <- gsub("NA","",descr[])
    if(!length(ncol(descr))) browser()
    if(ncol(descr) > 1){
      descr.rn <- c("","",descr.rn)
      descr <- rbind(colnames(descr),"",descr)
    }
    descr <- cbind(descr.rn,descr)
    descr <- apply(descr,2,format,justify="right")
    descr <- paste0("   ",apply(descr,1,paste,collapse=" "),"   ")
  }
  if(length(tab) && length(descr)){
    statstab <- format(c(tab,"",descr),justify="left")
  }
  else if(length(tab)){
    statstab <- tab
  }
  else if(length(descr)){
    statstab <- descr
  }
  else
    statstab <- NULL
  
  annot.out <- character()
  if(length(annot)){
    for(i in seq_len(length(annot))){
      annot.i <- annot[i]
      nm.i <- trimws(names(annot.i))
      annot.i <- strwrap(annot.i,width=getOption("width")-8-4)
      annot.i <- c(paste("      ",annot.i),"")
      if(nzchar(nm.i)){
        annot.i <- c(
          paste("   ",nm.i,":",sep=""),
          annot.i
          )
      }
      annot.out <- c(annot.out,annot.i)
    }
  }
  c(
    hrule,
    "",
    title,
    if(length(wording)) c(
        "",
        wording
        ),
    "",
    hrule,
    "",
    spec,
    "",
    statstab,
    "",
    if(length(annot.out)) annot.out
    )

  
}

format_cb_table_md <- function(tab){
    cn <- colnames(tab)
    if(ncol(tab)>2){
        if(all(trunc(tab[,1])==tab[,1])){
            tab <- cbind(
                formatC(tab[,1,drop=FALSE],format="d"),
                formatC(tab[,2],format="f",digits=1),
                formatC(tab[,3],format="f",digits=1)
            )
        } else {
            tab <- cbind(
                formatC(tab[,1,drop=FALSE],format="f",digits=1),
                formatC(tab[,2],format="f",digits=1),
                formatC(tab[,3],format="f",digits=1)
            )
        }
    }
    else {
        if(all(trunc(tab[,1])==tab[,1])){
            tab <- cbind(
                formatC(tab[,1,drop=FALSE],format="d"),
                formatC(tab[,2],format="f",digits=1)
            )
        } else {
            tab <- cbind(
                formatC(tab[,1,drop=FALSE],format="f",digits=1),
                formatC(tab[,2],format="f",digits=1)
            )
        }
        
    }
    tab[tab=="NA"] <- ""
    tab <- rbind(" "=cn,"",tab)
    tab <- format(tab,justify="right")
    tab[] <- paste0(" ",tab," ")
    specline <- sapply(nchar(tab[2,]),md_tabspec,align="r")
    tab[2,] <- specline
    tab <- apply(tab,1,paste,collapse="|")
    tab <- paste0("|",tab)
    tab
}

# This assumes that the contents are already formatted!
format_col_md <- function(x,align,colname){
    if(align=="r")
        x <- format(x,justify="right")
    else if(align=="l")
        x <- format(x,justify="left")
    x <- c(colname,x)
    x <- paste0(" ",format(x)," ")
    n <- nchar(x[1])
    spec <- md_tabspec(n,align=align)
    c(x[1],spec,x[-1])
}

md_tabspec <- function(n,align="l"){
    spec <- rep("-",n)
    if(align=="l")
        spec[1] <- ":"
    else if(align=="r")
        spec[n] <- ":"
    paste(spec,collapse="")
}

formula2coefs <- function(obj,fo,warn=FALSE){
    if(length(fo)>2)
        rhs <- fo[-2]
    else
        rhs <- fo
    xlevels <- get_xlevels(obj)
    tm <- terms(rhs)
    termlabs <- attr(tm,"term.labels")
    o.tm <- terms(obj)
    o.termlabs <- attr(o.tm,"term.labels")
    if(!all(termlabs%in%o.termlabs)){
        sdf <- setdiff(termlabs,o.termlabs)
        if(warn)
            warning(paste(paste(sdf,collapse=", "),"not in model"))
        termlabs <- intersect(termlabs,o.termlabs)
    }
    names <- lapply(termlabs,tl2coefs,xlevels=xlevels)
    names <- unlist(names)
    names <- unique(names)
    return(names)
}

formula2termlabs <- function(obj,fo,warn=FALSE){
    if(length(fo)>2)
        rhs <- fo[-2]
    else
        rhs <- fo
    tm <- terms(rhs)
    termlabs <- attr(tm,"term.labels")
    o.tm <- terms(obj)
    o.termlabs <- attr(o.tm,"term.labels")
    if(!all(termlabs%in%o.termlabs)){
        sdf <- setdiff(termlabs,o.termlabs)
        if(warn)
            warning(paste(paste(sdf,collapse=", "),"not in model"))
        termlabs <- intersect(termlabs,o.termlabs)
    }
    return(termlabs)
}


tl2coefs <- function(tl,xlevels){
    tl <- unlist(strsplit(tl,":"))
    nms <- lapply(tl,tl2coefs_,xlevels=xlevels)
    if(length(tl)>1)
        nms <- Reduce(intercoefs,nms)
    return(nms)
}

tl2coefs_ <- function(tl,xlevels){
    if(!(tl %in% names(xlevels)))
        return(tl)
    else {
        levs <- xlevels[[tl]]
        return(paste0(tl,levs))
    }
}

intercoefs <- function(x,y){
    outer(x,y,paste,sep=":")
}

get_xlevels <- function(obj) {
    if("xlevels" %in% names(obj))
        obj$xlevels
    else {
        xlevels <- list()
        Contr <- names(attr(model.matrix(obj), "contrasts"))
        for (c in Contr) xlevels[[c]] <- levels(obj@frame[,c])
        xlevels
    }
}

simplifyTypes <- function(x,verbose=TRUE,...){
  if(!is.data.frame(x) && !is.data.set(x)) stop("first arg. should be a data.frame or data.set")
  attr.x <- attributes(x)
  y <- list()
  for(i in seq_along(x)){
    y[[i]] <- simplifyType(x[[i]])
    if(verbose && typeof(y[[i]])!=typeof(x[[i]])) message("Changed type of variable ",
                                  names(x)[i],
                                  " from ",
                                  sQuote(typeof(x[[i]])),
                                  " to ",
                                  sQuote(typeof(y[[i]])))
    }
  attributes(y) <- attr.x
  y
}

simplifyType <- function(x) UseMethod("simplifyType")
simplifyType.default <- function(x) x
simplifyType.double <- function(x){
  ok <- is.finite(x)
  #y <- as.integer(x)
  if(all(x[ok]==trunc(x[ok]))) as.integer(x) else x
}
simplifyType.labelled <- function(x){
  if(typeof(x) %in% c("integer","character")) return(x)
  ok <- is.finite(x)
  if(any(x[ok]!=trunc(x[ok]))) return(x)
  y <- x
  attributes(y) <- NULL
  y <- as.integer(y)
  attributes(y) <- attributes(x)
  if(length(cds <- attr(y,"labels"))){
    clab <- names(cds)
    cds <- as.integer(cds)
    names(cds) <- clab
    attr(y,"labels") <- cds
    }
  if(length(msg <- attr(y,"missing.values"))){
    if(length(msg$values)) msg$values <- as.integer(msg$values)
    #if(length(msg$range)) msg$range[is.finite(msg$range)] <- as.integer(msg$range)
    attr(y,"missing.values") <- msg
    }
  y
}

getSummary.lm <- function(obj,
            alpha=.05,
            ...
            ){
  smry <- summary(obj)
  coef <- smry$coef

  dendf <- smry$df[2]

  lower <- coef[,1] + coef[,2]*qt(p=alpha/2,df=dendf)
  upper <- coef[,1] + coef[,2]*qt(p=1-alpha/2,df=dendf)

  coef <- cbind(coef,lower,upper)

  dn <- list(
      rownames(coef),
      c("est","se","stat","p","lwr","upr"),
      names(obj$model)[1]
  )
  dim(coef) <- c(dim(coef)[1],dim(coef)[2],1)
  dimnames(coef) <- dn
  
  sigma <- smry$sigma
  r.squared <- smry$r.squared
  adj.r.squared <- smry$adj.r.squared

  if(length(smry$fstatistic)){
      F <- unname(smry$fstatistic[1])
      numdf <- unname(smry$fstatistic[2])
      p <- pf(F,numdf,dendf,lower.tail=FALSE)
  }
  else {
      F <- 0
      numdf <- 0
      p <- 1
  }

  N <- sum(smry$df[1:2])
  ll <- logLik(obj)
  deviance <- deviance(obj)
  AIC <- AIC(obj)
  BIC <- AIC(obj,k=log(N))
  sumstat <- c(
          sigma         = sigma,
          r.squared     = r.squared,
          adj.r.squared = adj.r.squared,
          F             = F,
          numdf         = numdf,
          dendf         = dendf,
          p             = p,
          logLik        = ll,
          deviance      = deviance,
          AIC           = AIC,
          BIC           = BIC,
          N             = N
          )

  list(coef=coef,
       sumstat=sumstat,
       contrasts=obj$contrasts,
       xlevels=obj$xlevels,
       call=obj$call)
}

.SummaryStats$lm <- c("R-squared","N")

getSummary.glm <- function(obj,alpha=.05,...){

    smry <- summary(obj)
    N <- if(length(weights(obj)))
             sum(weights(obj),na.rm=TRUE)
         else sum(smry$df[1:2])

    coef <- smry$coef

    lower <- qnorm(p=alpha/2,mean=coef[,1],sd=coef[,2])
    upper <- qnorm(p=1-alpha/2,mean=coef[,1],sd=coef[,2])

    coef <- cbind(coef,lower,upper)

    dn <- list(
        rownames(coef),
        c("est","se","stat","p","lwr","upr"),
        names(obj$model)[1]
    )
    dim(coef) <- c(dim(coef)[1],dim(coef)[2],1)
    dimnames(coef) <- dn

    phi <- smry$dispersion
    LR <- smry$null.deviance - smry$deviance
    df <- smry$df.null - smry$df.residual

    ll <- logLik(obj)
    deviance <- deviance(obj)

    if(df > 0){
        p <- pchisq(LR,df,lower.tail=FALSE)
        L0.pwr <- exp(-smry$null.deviance/N)
        #LM.pwr <- exp(-smry$deviance/N)
        Aldrich.Nelson <- LR/(LR+N)
        McFadden <- 1- smry$deviance/smry$null.deviance
        Cox.Snell <- 1 - exp(-LR/N)
        Nagelkerke <- Cox.Snell/(1-L0.pwr)
    }
    else {
        LR <- NA
        df <- NA
        p <- NA
        Aldrich.Nelson <- NA
        McFadden <- NA
        Cox.Snell <- NA
        Nagelkerke <- NA
    }

    AIC <- AIC(obj)
    BIC <- AIC(obj,k=log(N))
    sumstat <- c(
        phi            = phi,
        LR             = LR,
        df             = df,
        p              = p,
        logLik         = ll,
        deviance       = deviance,
        Aldrich.Nelson = Aldrich.Nelson,
        McFadden       = McFadden,
        Cox.Snell      = Cox.Snell,
        Nagelkerke     = Nagelkerke,
        AIC            = AIC,
        BIC            = BIC,
        N              = N
    )

    list(coef=coef,
         sumstat=sumstat,
         contrasts=obj$contrasts,
         xlevels=obj$xlevels,
         call=obj$call)
}

setGeneric("as.item",function(x,...)standardGeneric("as.item"))

setGeneric("annotation",function(x)standardGeneric("annotation"))
setGeneric("annotation<-",function(x,value)standardGeneric("annotation<-"))

# setGeneric("description",function(x)standardGeneric("description"))
# setGeneric("description<-",function(x,value)standardGeneric("description<-"))

setGeneric("labels",function(object,...)standardGeneric("labels"))
setGeneric("labels<-",function(x,value)standardGeneric("labels<-"))

setGeneric("has.value.labels",function(x)standardGeneric("has.value.labels"))

setGeneric("is.labelled",function(x)standardGeneric("is.labelled"))

setGeneric("value.filter",function(x)standardGeneric("value.filter"))
setGeneric("value.filter<-",function(x,value)standardGeneric("value.filter<-"))

setGeneric("valid.values",function(x)standardGeneric("valid.values"))
setGeneric("missing.values",function(x)standardGeneric("missing.values"))
setGeneric("valid.range",function(x)standardGeneric("valid.range"))
setGeneric("valid.values<-",function(x,value)standardGeneric("valid.values<-"))
setGeneric("valid.range<-",function(x,value)standardGeneric("valid.range<-"))
setGeneric("missing.values<-",function(x,value)standardGeneric("missing.values<-"))
setGeneric("is.missing",function(x)standardGeneric("is.missing"))
setGeneric("include.missings",function(x,mark="*")standardGeneric("include.missings"))

setGeneric("measurement",function(x)standardGeneric("measurement"))
setGeneric("measurement<-",function(x,value)
  standardGeneric("measurement<-"))

setGeneric("as.measurement.level",function(x)standardGeneric("as.measurement.level"))


setGeneric("as.data.set",function(x,row.names=NULL,...)standardGeneric("as.data.set"))

setGeneric("codebook",function(x,weights=NULL,unweighted=TRUE,...)standardGeneric("codebook"))
setGeneric("wild.codes",function(x)standardGeneric("wild.codes"))

setGeneric("Table",function(x,...) standardGeneric("Table"))
setGeneric("Descriptives",function(x,...) standardGeneric("Descriptives"))
# setGeneric("unique",function(x,incomparables,...)standardGeneric("unique"))

setGeneric("query",function(x,pattern,...) standardGeneric("query"))

as.array <- function(x,...)base::as.array(x)
setGeneric("as.array",
  function(x,...)standardGeneric("as.array")
  )

#contrasts <- function(x,...) stats::contrasts(x,...)
setGeneric("contrasts",
  function(x,...) standardGeneric("contrasts")
)

# "contrasts<-" <- function(x,how.many,value) stats::`contrasts<-`(x,how.many,value)
# setGeneric("contrasts<-",
#   function(x,how.many,value) standardGeneric("contrasts<-")
# )

Write <- function(x,...) UseMethod("Write")

# setGeneric("Write",
#            function(object,...)
#              standardGeneric("Write"))

setGeneric("codeplan",
           function(x) standardGeneric("codeplan"))

setGeneric("setCodeplan",function(x,value)
  standardGeneric("setCodeplan"))

format_md <- function(x, ...) {
  UseMethod("format_md")
}

porStream <- function(file) .Call("NewPorStream",file)
closePorStream <- function(pstream) .Call("closePorStream",pstream)
readOnePushbackPorStream <- function(pstream) .Call("readOnePushbackPorStream",pstream)
readOnePorStream <- function(pstream) .Call("readOnePorStream",pstream)
seekPorStream <- function(pstream,pos) .Call("seekPorStream",pstream,pos)
tellPorStream <- function(pstream) .Call("tellPorStream",pstream)
lineTellPorStream <- function(pstream) .Call("lineTellPorStream",pstream)
offsetTellPorStream <- function(pstream) .Call("offsetTellPorStream",pstream)
readToSlashPorStream <- function(pstream,maxn) .Call("readToSlashPorStream",pstream,maxn)
readIntPorStream <- function(pstream) .Call("readIntPorStream",pstream)
readDoublePorStream <- function(pstream) .Call("readDoublePorStream",pstream)
readStringPorStream <- function(pstream) .Call("readStringPorStream",pstream)
setTranslationPorStream <- function(pstream,translation) .Call("setTranslationPorStream",pstream,translation)

readPorStream <- function(pstream,n=1,start=NULL){
  if(missing(start))
    .Call("readPorStream",pstream,n)
  else{
    seekPorStream(pstream,pos=start)
    .Call("readPorStream",pstream,n)
  }
}



readRangePorStream <- function(file,range)
  readPorStream(file,n=diff(range[1:2])+1,start=range[1])


readVarPorStream <- function(pstream,start){
  variable <- list()
  variable$width <- readIntPorStream(pstream)
  variable$name <- readStringPorStream(pstream)
  #print(variable$name)
  #cat("\nVariable ",variable$name," width: ",variable$width)
  variable$printformat <- c(readIntPorStream(pstream), readIntPorStream(pstream), readIntPorStream(pstream))
  variable$writeformat <- c(readIntPorStream(pstream), readIntPorStream(pstream), readIntPorStream(pstream))
  variable$missing$values <- c()
  valid.tags <- c("8","9","A","B","C")
  repeat{
    tag.code <- readOnePushbackPorStream(pstream)
    #cat("=========================================================================\n")
    #cat("=========================================================================\n")
    #cat("Tag code is --",dQuote(tag.code),"--\n")
    if(tag.code %in% valid.tags){
      tag.code <- readOnePorStream(pstream)
    if(tag.code=="8") {
         #cat("Found single missing value tag\n")
         if(variable$width)
            variable$missing$values <- c(variable$missing$values,readStringPorStream(pstream))
          else
            variable$missing$values <- c(variable$missing$values,readDoublePorStream(pstream))
          #if(variable$width) cat("String") else cat("numeric")
          #cat("Values: ",variable$missing$values,"\n")
          }
    if(tag.code=="9") {
         #cat("Found LO THRU x tag\n")
          if(variable$width)
            variable$missing$range <- c("-Inf",readStringPorStream(pstream))
          else
            variable$missing$range <- c(-Inf,readDoublePorStream(pstream))
          #if(variable$width) cat("String") else cat("numeric")
          #cat("Range: ",variable$missing$range,"\n")
          }
    if(tag.code=="A") {
         #cat("Found missing (x THRU HI) tag\n")
          if(variable$width)
            variable$missing$range <- c(readStringPorStream(pstream),"Inf")
          else
            variable$missing$range <- c(readDoublePorStream(pstream),Inf)
          #if(variable$width) cat("String") else cat("numeric")
          #cat("Range: ",variable$missing$range,"\n")
          }
    if(tag.code=="B") {
          #cat("Found missing (x THRU y) tag\n")
          if(variable$width)
            variable$missing$range <- c(readStringPorStream(pstream),readStringPorStream(pstream))
          else
            variable$missing$range <- c(readDoublePorStream(pstream),readDoublePorStream(pstream))
          #if(variable$width) cat("String") else cat("numeric")
          #cat("range: ",variable$missing$range,"\n")
          }
    if(tag.code=="C"){
          #cat("Found variable label tag\n")
          variable$label <- trimws(withEncoding(readStringPorStream(pstream),"bytes"))
          #cat("Variable label",variable$label,"\n")
          }
    }
    else {
      #cat("Probably end of variable record")
      #cat("---- remainder:\n")
      #print(readOnePushbackPorStream(pstream))
      break
      }
  }
  return(variable)
}

readLabPorStream <- function(pstream,types){
  nvar <- readIntPorStream(pstream)
  vars <- character(0)
  for(i in 1:nvar)
    vars <- c(vars,readStringPorStream(pstream))
  #cat("\nVariables: ",paste(vars,collapse=" "))
  nlabs <- readIntPorStream(pstream)
  #cat("\nNumber of labels:",nlabs)
  #cat("\nvars[1]=",vars[1])
  type <- types[[vars[1]]]
  #cat("\nType: ",type," i.e. ",c("integer","character")[type+1])
  #browser()
  if(type==0){
    labs <- numeric(nlabs)
    namlabs <- character(nlabs)
    for(j in 1:nlabs){
      labs[j] <- readDoublePorStream(pstream)
      nlabs[j] <- readStringPorStream(pstream)
      #cat("\n",labs[j]," ",sQuote(nlabs[j]))
      }
    }
  else {
    labs <- character(nlabs)
    for(j in 1:nlabs){
      labs[j] <- readStringPorStream(pstream)
      nlabs[j] <- readStringPorStream(pstream)
      #cat("\n",labs[j]," ",sQuote(nlabs[j]))
      }
    }
  names(labs) <- nlabs
  return(list(vars=vars,value.labels=labs))
}

readDocumentPorStream <- function(stream){
  nlines <- readIntPorStream(stream)
  doclines <- character(0)
  for(i in 1:nlines){
    doclines[i] <- readStringPorStream(stream)
  }
  doclines
}

parseHeaderPorStream <- function(stream,iconv=FALSE,encoded=getOption("spss.por.encoding","cp1252")){
  header <- list()
  header$control <- readRangePorStream(stream,200 + c(0,60))
  header$digits <- readRangePorStream(stream,200 + c(64,73))
  header$cletters <- readRangePorStream(stream,200 + c(74,99))
  header$lletters <- readRangePorStream(stream,200 + c(100,125))
  header$symbols <- readRangePorStream(stream,200 + c(126,188))
  header$slash <- readRangePorStream(stream,200 + c(142,142))
  header$translation <- readRangePorStream(stream,200 + c(0,255))
  transl <- setTranslationPorStream(stream,header$translation)
  if(
      !identical(header$digits,"0123456789") ||
      !identical(header$cletters,"ABCDEFGHIJKLMNOPQRSTUVWXYZ") ||
      !identical(header$lletters,"abcdefghijklmnopqrstuvwxyz") ||
      !identical(header$slash,"/")
  ) stop("import of files of this encoding not yet supported")

  seekPorStream(stream,pos=464)
  header$version <- readOnePorStream(stream)
  header$creation.date <- readStringPorStream(stream)
  header$creation.time <- readStringPorStream(stream)
  dictionary <- list()
  valid.tags <- c("1","2","3","4","6","7","D","E")
  nvlab <- 1
  value.labels <- list()
  document <- NULL
  types <- NULL
  repeat{
    #cat("##########################################################################\n")
    #cat("##########################################################################\n")
    #cat("Reading tag\t")
    tag.code <- readOnePorStream(stream)
    #cat("tag is:",dQuote(tag.code)," -- ")
    #browser()
    if(!(tag.code %in% valid.tags)) break
    if(tag.code=="1") {
            #cat("Found product tag\n")
            header$product <- readStringPorStream(stream)
            }
    if(tag.code=="2") {
            #cat("Found author tag\n")
            header$author <- readStringPorStream(stream)
            }
    if(tag.code=="3") {
            #cat("Found subproduct tag\n")
            header$subproduct <- readStringPorStream(stream)
            }
    if(tag.code=="4") {
            #cat("Found number of variables tag\n")
            header$nvar[1] <- readIntPorStream(stream)
            header$nvar[2] <- readIntPorStream(stream)
            types <- integer()
            }
    if(tag.code=="6") {
            #cat("Found weighting variable tag\n")
            header$weight <- readStringPorStream(stream)
            #cat("Weighting variable is:", header$weight)
            }
    if(tag.code=="7") {
            #cat("Found variable record tag\n")
            variable <- readVarPorStream(stream)
            nameidx <- match("name",names(variable))
            #print(variable)
            # Can't rely on the information in the print formats, so always "real" 5.9.07
            #types[variable$name] <- if(variable$width) 2 else 1#if (variable$printformat[3] > 0) 1 else 0
            if(iconv){
              variable$label <- iconv(variable$label,from=encoded)
            }
            types[variable$name] <- variable$width
            dictionary[[variable$name]] <- variable[-nameidx]
            }
    if(tag.code=="D") {
            #cat("Found value labels tag in line ",lineTellPorStream(stream),"\n")
            value.labels <- c(value.labels,list(readLabPorStream(stream,types)))
          }
    if(tag.code=="E") {
             #cat("Found document tag\n")
             document <- paste(readDocumentPorStream(stream),collapse="\n")
           }

  }
  if(length(value.labels)){
    names(value.labels) <- paste("label",seq(1,length(value.labels)),sep="")
  }
  if(iconv){
      if(length(value.labels)){
          value.labels <- lapply(value.labels,
                                 function(x){
                                     vl <- x$value.labels
                                     names(vl) <- iconv(names(vl),from=encoded)
                                     x$value.labels <- vl
                                     x
                                 })
      }
      document <- iconv(document,from=encoded)
  }


  p <- list()
  p$header <- header
  p$dictionary <- dictionary
  p$value.labels <- value.labels
  p$types <- types
  p$document <- document
  if(tag.code == "F"){
    #message("Found data tag, good ...\n")
    p$start.data <- tellPorStream(stream)
  }
  else {
    currpos <- tellPorStream(stream)
    currline <- lineTellPorStream(stream)
    offset <- offsetTellPorStream(stream)
    seekPorStream(stream,pos=currpos-80)
    context <- character(2)
    context[1] <- readPorStream(stream,n=80)
    context[2] <- readPorStream(stream,n=80)
    print(context)
    stop("unknown tag ",dQuote(tag.code)," found in line ",currline," offset ",offset)
  }
  return(p)
}



seekDataPorStream <- function(p){
  start <- p$start.data
  seekPorStream(p$stream,start)
}

mtable_format_stdstyle <- c(
  "padding-top"="3px",
  "padding-bottom"="3px",
  "padding-left"="0.5ex",
  "padding-right"="0.5ex",
  "margin-top"="0px",
  "margin-bottom"="0px",
  "border-style"="none",
  "border-width"="0px"
)

format_html.memisc_mtable <- function(x,
                               interaction.sep = NULL,
                               toprule=2,midrule=1,bottomrule=2,
                               split.dec=TRUE,
                               style=mtable_format_stdstyle,
                               margin="2ex auto",
                               sig.notes.style=c(width="inherit"),
                               ...){
  x <- preformat_mtable(x)
  res <- pf_mtable_format_html(x,interaction.sep=interaction.sep,
                        toprule=toprule,midrule=midrule,bottomrule=bottomrule,
                        split.dec=split.dec,style=style,margin=margin,
                        sig.notes.style=sig.notes.style,...)
  as.character(res)
}

mtable_format_html <- function(x,
                               interaction.sep = NULL,
                               toprule=2,midrule=1,bottomrule=2,
                               split.dec=TRUE,
                               style=mtable_format_stdstyle,
                               margin="2ex auto",
                               sig.notes.style=c(width="inherit"),
                               ...)
    pf_mtable_format_html(preformat_mtable(x),
                          interaction.sep=interaction.sep,
                          toprule=toprule,
                          midrule=midrule,
                          bottomrule=bottomrule,
                          split.dec=split.dec,
                          style=style,
                          margin=margin,
                          sig.notes.style=sig.notes.style,
                          ...)

pf_mtable_format_html <- function(x,
                               interaction.sep = NULL,
                               toprule=2,midrule=1,bottomrule=2,
                               split.dec=TRUE,
                               style=mtable_format_stdstyle,
                               margin="2ex auto",
                               sig.notes.style=c(width="inherit"),
                               ...
){
  
    firstcol <- c("padding-left"="0.3em")
    toprule <- c("border-top"=paste0(midrule,"px solid"))
    bottomrule <- c("border-bottom"=paste0(midrule,"px solid"))
    midrule_above <- c("border-top"=paste0(midrule,"px solid"))
    midrule <- c("border-bottom"=paste0(midrule,"px solid"))
    align.right <- c("text-align"="right")  
    align.left <- c("text-align"="left")  
    align.center <- c("text-align"="center")
    lrpad <- c("padding-left"="0.3em","padding-right"="0.3em")
    row_style <- c("border-style"="none")
    table_style <- c("border-collapse"="collapse" ,"border-style"="none")
    
    if(!length(interaction.sep)){
        if(getOption("html.use.ampersand",FALSE))
            interaction.sep <- " &times; "
        else 
            interaction.sep <- " \u00d7 "
    }

    colsep <- ""
    rowsep <- "\n"

    pt <- x$parmtab
    sst <- x$summary.stats
    leaders <- x$leaders
    headers <- x$headers
    eq.headers <- x$eq.headers
    outtypes <- x$outtypes
    l.headers <- length(headers)
    l.leaders <- length(leaders)

    ncols <- sapply(pt[1,,drop=FALSE],ncol)

    res <- NULL
    has.eq.headers <- length(eq.headers) > 0

    for(j in 1:ncol(pt)){
        
        name.j <- colnames(pt)[j]
        pt.j <- pt[,j]

        ncol.j <- unique(sapply(pt.j,ncol))
        stopifnot(length(ncol.j)==1)
        
        for(i in 1:length(pt.j)){

            pt.ij <- pt.j[[i]]
            ot.ij <- outtypes[i,j]
            dm.ij <- dim(pt.ij)
            
            if(ot.ij == "num"){
                if(getOption("html.use.ampersand",FALSE))
                    pt.ij[] <- gsub("-","&minus;",pt.ij[],fixed=TRUE)
                else
                    pt.ij[] <- gsub("-","\u2212",pt.ij[],fixed=TRUE)

                if(split.dec){
                    pt.ij <- spltDec(pt.ij)
                    pt.ij <- gsub("([*]+)","<span class=\"signif.symbol\">\\1</span>",pt.ij)
                    pt.ij <- html_td_spltDec(pt.ij, style=css(style))
                }
                else
                    pt.ij[] <- html_td(pt.ij,style=css(style),vectorize=TRUE)
            } else {
                tstyle <- upd_vect(style,align.center)
                if(split.dec)
                    pt.ij[] <- html_td(pt.ij,colspan=3,style=css(tstyle),vectorize=TRUE)
                else 
                    pt.ij[] <- html_td(pt.ij,style=css(tstyle),vectorize=TRUE)
            }
            dim(pt.ij) <- dm.ij
            
            pt.j[[i]] <- pt.ij
        }
        pt.j <- do.call(rbind,pt.j)
        if(has.eq.headers){
            eq.header.j <- eq.headers[[name.j]]
            n.eq.j <- length(eq.header.j)
            eq.span <- ncol.j/n.eq.j
            if(split.dec)
                eq.span <- eq.span*3
            ehstyle <- upd_vect(style,align.center)
            if(l.headers > 0)
                ehstyle <- upd_vect(ehstyle,midrule_above)
            eq.header.j <- html_td(eq.header.j,colspan=eq.span,style=css(ehstyle),vectorize=TRUE)
            pt.j <- rbind(eq.header.j,pt.j)
        }

        if(length(sst)){
            sst.j <- sst[[j]]
            if(getOption("html.use.ampersand",FALSE))
                sst.j <- gsub("-","&minus;",sst.j,fixed=TRUE)
            else
                sst.j <- gsub("-","\u2212",sst.j,fixed=TRUE)
            sst.j <- colexpand(sst.j,ncol.j)
            dm.ij <- dim(sst.j)
            if(split.dec){
                sst.j <- spltDec(sst.j)
                sst.j <- html_td_spltDec(sst.j,style=css(style))
            }
            else
                sst.j <- html_td(sst.j,style=css(style),vectorize=TRUE)
            dim(sst.j) <- dm.ij
            pt.j <- rbind(pt.j,sst.j)
        }

        pt.j <- apply(pt.j,1,as.html_group)
        
        res <- cbind(res,pt.j)
    }

    if(l.leaders){

        leaders <- lapply(leaders,ldxp)
        leaders <- do.call(rbind,leaders)
        if(has.eq.headers)
            leaders <- rbind("",leaders)
        
        lstyle <- upd_vect(style,align.left,firstcol)
        leaders <- html_td(leaders,vectorize=TRUE,style=css(lstyle))
        
        res <- cbind(leaders,res)
    }

    res <- apply(res,1,as.html_group)

    if(l.headers){
        for(k in 1:l.headers){
            headers.k <- headers[[k]]
            hspan.k <- sapply(headers.k,attr,"span")
            if(k == l.headers)
                hstyle <- upd_vect(style,align.center)
            else
                hstyle <- upd_vect(style,align.center,midrule)
            if(split.dec)
                hspan.k <- hspan.k*3
            headers.k <- Map(html_td,headers.k,colspan=hspan.k,MoreArgs=list(style=css(hstyle)))
            if(l.leaders){
                hlstyle <- upd_vect(style,align.left)
                lheader.k <- html_td("",colspan=1,style=css(hlstyle))
                headers.k <- c(list(lheader.k),headers.k)
            }
            headers[[k]] <- headers.k
        }
        headers <- lapply(headers,as.html_group)
        res <- c(headers,res)
    }

    
    res[[1]] <- lapply(res[[1]],setStyle,toprule)
    n <- length(res)
    res[[n]] <- lapply(res[[n]],setStyle,bottomrule)


    sect.at <- integer()
    csum <- 1
    for(i in 1:nrow(pt)){
        sect.at <- c(sect.at,csum)
        csum <- csum + nrow(pt[[i,1]])
    }
    if(length(sst) && any(sapply(sst,length)>0))
        sect.at <- c(sect.at,csum)
    if(l.headers)
        sect.at <- c(sect.at + l.headers)
    if(has.eq.headers)
        sect.at <- sect.at + 1
    #browser()
    for(i in sect.at)
        res[[i]] <- lapply(res[[i]],setStyle,midrule_above)

    signif.symbols <- x$signif.symbols
    if(length(signif.symbols)){
        signif.template <- getOption("signif.symbol.print.template",
                                     signif.symbol.print.default.template)
        signif.symbols <- format_signif_print(signif.symbols,
                                              signif.template,
                                              width=72)

        if(split.dec)
            totspan <- sum(ncols) * 3
        else
            totspan <- sum(ncols)
        if(l.leaders)
            totspan <- totspan + 1
        
        signif.symbols <- html_p(signif.symbols,style=css(sig.notes.style))
        signif.symbols <- html_td(signif.symbols,style=css(style),colspan=totspan)
        res <- c(res,list(signif.symbols))
    }
    
    res <- html_tr(res,vectorize=TRUE,style=as.css(row_style))
    
    if(length(margin))
        table_style <- c(table_style,margin=margin)
    res <- html_table(res,class="mtable",style=as.css(table_style))

    return(res)

}

get_colspan <- function(x)x$attributes$colspan

Iconv <- function(x,from="",to="",...) UseMethod("Iconv")

Iconv.character <- function(x,from="",to="",...){
  iconv(x,from=from,to=to,...)  
}

Iconv.importer <- function(x,from="",to="",...){
  x@.Data <- lapply(x@.Data,Iconv.item,from,to,...)
  return(x)
}

Iconv.item <- function(x,from="",to="",...){
  if(length(ann <- annotation(x)))
    annotation(x) <- Iconv.annotation(ann,from=from,to=to,...)
  if(length(vl <- labels(x)))
    labels(x) <- Iconv.value.labels(vl,from=from,to=to,...)
  return(x)
}

Iconv.value.labels <- function(x,from="",to="",...){
  x@.Data <- iconv(x@.Data,from=from,to=to,...)
  return(x)
}

Iconv.annotation <- function(x,from="",to="",...){
  x@.Data <- iconv(x@.Data,from=from,to=to,...)  
  return(x)
}

Iconv.data.set <- function(x,from="",to="",...){
  x@.Data <- lapply(x@.Data,Iconv.item,from,to,...)
  return(x)
}

relabel <- function(x,...,gsub=FALSE,fixed=TRUE,warn=TRUE){
    if(isS4(x)) {
        m <- as.list(match.call(expand.dots=FALSE))
        dots <- lapply(m$...,as.character)
        m <- c(m[1:2],dots,m[-(1:3)])
        m[[1]] <- as.name("relabel4")
        #relabel4(x,...,gsub=gsub,fixed=fixed,warn=warn)
        m <- as.call(m)
        eval(m,parent.frame())
    }
    else UseMethod("relabel")
}

relabel.default <- function(x,...,gsub=FALSE,fixed=TRUE,warn=TRUE){
    if(!is.null(attr(x,"labels"))) labels <- attr(x,"labels")
    else labels <- names(x)
    m <- match.call(expand.dots=FALSE)
    subst <- sapply(m$...,as.character)
    if(gsub){
        for(i in 1:length(subst)){
            labels <- gsub(names(subst[i]),subst[i],labels,fixed=fixed)
        }
    }
    else {
        i <- match(names(subst),labels)
        if(any(is.na(i))) {
            if(warn) warning("undefined label(s) selected")
            if(any(!is.na(i)))
                subst <- subst[!is.na(i)]
            i <- i[!is.na(i)]
        }
        if(length(i))
            labels[i] <- subst
    }
    if(!is.null(attr(x,"labels"))) attr(x,"labels") <- labels
    else names(x) <- labels
    return(x)
}

relabel.factor <- function(x,...,gsub=FALSE,fixed=TRUE,warn=TRUE){
    m <- match.call(expand.dots=FALSE)
    subst <- sapply(m$...,as.character)
    labels <- levels(x)
    if(gsub){
        for(i in 1:length(subst)){
            labels <- gsub(names(subst[i]),subst[i],labels,fixed=fixed)
        }
    }
    else {
        i <- match(names(subst),labels)
        if(any(is.na(i))) {
            if(warn) warning("undefined label(s) selected")
            if(any(!is.na(i)))
                subst <- subst[!is.na(i)]
            i <- i[!is.na(i)]
        }
        if(length(i))
            labels[i] <- subst
    }
    if(any(duplicated(labels)))
        warning("Duplicate labels")
    levels(x) <- labels
    return(x)
}

relabel1 <- function(x,...,gsub=FALSE,fixed=TRUE,warn=TRUE){
    m <- match.call(expand.dots=FALSE)
    subst <- sapply(m$...,as.character)
    if(gsub){
        for(i in 1:length(subst)){
            x <- gsub(names(subst[i]),subst[i],x,fixed=fixed)
        }
    }
    else {
        i <- match(names(subst),x)
        if(any(is.na(i))) {
            if(warn) warning("unused name(s) selected")
            if(any(!is.na(i)))
                subst <- subst[!is.na(i)]
            i <- i[!is.na(i)]
        }
        if(length(i))
            x[i] <- subst
    }
    return(x)
}


relabel.table <- function(x,...,gsub=FALSE,fixed=TRUE,warn=FALSE){
  
  dn <- dimnames(x)
  ndn <- names(dn)
  dn <- lapply(dn,relabel1,...,gsub=gsub,fixed=fixed,warn=warn)
  ndn <- relabel1(ndn,...,gsub=gsub,fixed=fixed,warn=warn)
  names(dn) <- ndn
  dimnames(x) <- dn
  return(x)
}


relabel.ftable <- function(x,...,gsub=FALSE,fixed=TRUE,warn=FALSE){
  attr(x,"row.vars") <- relabel1ft(attr(x,"row.vars"),...,gsub=gsub,fixed=fixed,warn=warn)
  attr(x,"col.vars") <- relabel1ft(attr(x,"col.vars"),...,gsub=gsub,fixed=fixed,warn=warn)
  return(x)
}

relabel1ft <- function(x,...,gsub=FALSE,fixed=TRUE,warn=TRUE){
  n.x <- names(x)
  if(length(n.x))
    n.x <- relabel1(n.x,...,gsub=gsub,fixed=fixed,warn=warn)
  x <- relabel1(x,...,gsub=gsub,fixed=fixed,warn=warn)
  names(x) <- n.x
  return(x)
}

relabel.ftable_matrix <- function(x,...,gsub=FALSE,fixed=TRUE,warn=FALSE){
  attr(x,"row.vars") <- lapply(attr(x,"row.vars"),relabel1ft,...,gsub=gsub,fixed=fixed,warn=warn)
  attr(x,"col.vars") <- lapply(attr(x,"col.vars"),relabel1ft,...,gsub=gsub,fixed=fixed,warn=warn)
  return(x)
}

## AER::ivreg

getSummary.ivreg <- function(obj, alpha = 0.05, ...) {
  ## extract summary object
  s <- summary(obj)
  
  ## coefficient matrix and confidence interval
  cf <- cbind(s$coefficients, confint(obj, level = 1 - alpha))
  colnames(cf) <- c("est", "se", "stat", "p", "lwr", "upr")

  # Patch ME 20.08.2017
  dn <- c(
      dimnames(cf),
      names(obj$model)[1]
  )
  dim(cf) <- c(dim(cf)[1],dim(cf)[2],1)
  dimnames(cf) <- dn
  
  ## further summary statistics
  sstat <- c("sigma" = s$sigma,
    "r.squared" = s$r.squared, "adj.r.squared" = s$adj.r.squared,
    "Wald" = s$waldtest[1], "numdf" = s$waldtest[3], "dendf" = s$waldtest[4],
    "p" = s$waldtest[2], "N" = nobs(obj))

  ## return everything
  return(list(
    coef = cf,
    sumstat = sstat,
    contrasts = obj$contrasts,
    xlevels = obj$xlevels,
    call = obj$call
  ))
}

setSummaryTemplate("ivreg" = c(
  "R-squared" = "($r.squared:f#)",
  "adj. R-squared" = "($adj.r.squared:f#)",
  "sigma" = "($sigma:#)",
  "Wald" = "($Wald:f#)",
  "p" = "($p:f#)",
  "N" = "($N:d)"
))
.SummaryStats$ivreg <- c("adj. R-squared","N")

## AER::tobit

getSummary.tobit <- function(obj, alpha = 0.05, ...) {
  ## extract summary object
  s <- summary(obj)
  
  ## coefficient matrix and confidence interval
  ## compute confidence interval manually to include Log(scale)
  cf <- cbind(s$coefficients,
    s$coefficients[, 1] + qnorm(alpha/2) * s$coefficients[, 2],
    s$coefficients[, 1] + qnorm(1 - alpha/2) * s$coefficients[, 2])
  colnames(cf) <- c("est", "se", "stat", "p", "lwr", "upr")

  ## Improvement by ME: deal with log-scale parameter
  sp.row <- match("Log(scale)",rownames(cf))
  sp <- cf[sp.row,,drop=FALSE]
  cf <- cf[-sp.row,,drop=FALSE]

  # Patch ME 20.08.2017
  dn <- c(
      dimnames(cf),
      rownames(attr(obj$terms,"factors"))[1]
  )
  dim(cf) <- c(dim(cf)[1],dim(cf)[2],1)
  dimnames(cf) <- dn

  ## further summary statistics
  sstat <- c(
    "scale" = s$scale,
    "Wald" = s$wald,
    "numdf" = sum(s$df) - s$idf,
    "p" = pchisq(s$wald, sum(s$df) - s$idf, lower.tail = FALSE),
    "N" = nobs(obj),
    "logLik" = as.vector(logLik(obj)),
    "AIC" = AIC(obj),
    "BIC" = BIC(obj))

  ## return everything
  return(list(
      coef=cf,
      scale=sp,
      sumstat = sstat,
      contrasts = obj$contrasts,
      xlevels = obj$xlevels,
      call = obj$call
  ))
}

setSummaryTemplate("tobit" = c(
  "Scale" = "($scale:#)",
  "Wald" = "($Wald:f#)",
  "p" = "($p:f#)",
  "Log-likelihood" = "($logLik:f#)",
  "AIC" = "($AIC:f#)",
  "BIC" = "($BIC:f#)",
  "N" = "($N:d)"
))

## pscl:::hurdle, pscl:::zeroinfl

getSummary.hurdle <- getSummary.zeroinfl <- function(obj, alpha = 0.05, ...) {
  ## extract coefficient summary
  cf <- summary(obj)$coefficients
  ## augment with confidence intervals
  cval <- qnorm(1 - alpha/2)
  for(i in seq_along(cf)) cf[[i]] <- cbind(cf[[i]],
    cf[[i]][, 1] - cval * cf[[i]][, 2],
    cf[[i]][, 1] + cval * cf[[i]][, 2])
  ## collect in array
  nam <- unique(unlist(lapply(cf, rownames)))
  acf <- array(dim = c(length(nam), 6, length(cf)),
    dimnames = list(nam, c("est", "se", "stat", "p", "lwr", "upr"), names(cf)))
  for(i in seq_along(cf)) acf[rownames(cf[[i]]), , i] <- cf[[i]]

  ## return everything
  return(list(
    coef = acf,
    sumstat = c(
      "N" = obj$n,
      "logLik" = as.vector(logLik(obj)),
      "AIC" = AIC(obj),
      "BIC" = AIC(obj, k = log(obj$n))
    ),
    contrasts = obj$contrasts,
    xlevels = obj$xlevels,
    call = obj$call
  ))
}

setSummaryTemplate("hurdle" = c(
  "Log-likelihood" = "($logLik:f#)",
  "AIC" = "($AIC:f#)",
  "BIC" = "($BIC:f#)",
  "N" = "($N:d)"
))

setSummaryTemplate("zeroinfl" = c(
  "Log-likelihood" = "($logLik:f#)",
  "AIC" = "($AIC:f#)",
  "BIC" = "($BIC:f#)",
  "N" = "($N:d)"
))

## betareg::betareg

getSummary.betareg <- function(obj, alpha = 0.05, ...) {
  ## extract coefficient summary
  s <- summary(obj)
  cf <- s$coefficients
  ## augment with confidence intervals
  cval <- qnorm(1 - alpha/2)
  for(i in seq_along(cf)) cf[[i]] <- cbind(cf[[i]],
    cf[[i]][, 1] - cval * cf[[i]][, 2],
    cf[[i]][, 1] + cval * cf[[i]][, 2])
  ## collect in array
  nam <- unique(unlist(lapply(cf, rownames)))
  acf <- array(dim = c(length(nam), 6, length(cf)),
    dimnames = list(nam, c("est", "se", "stat", "p", "lwr", "upr"), names(cf)))
  for(i in seq_along(cf)) acf[rownames(cf[[i]]), , i] <- cf[[i]]
  
  ## return everything
  return(list(
    coef = acf,
    sumstat = c(
      "N" = obj$n,
      "pseudo.r.squared" = s$pseudo.r.squared,
      "logLik" = as.vector(logLik(obj)),
      "AIC" = AIC(obj),
      "BIC" = AIC(obj, k = log(obj$n))
    ),
    contrasts = obj$contrasts,
    xlevels = obj$xlevels,
    call = obj$call
  ))
}

setSummaryTemplate("betareg" = c(
  "Pseudo R-sq." = "($pseudo.r.squared:f#)",
  "Log-likelihood" = "($logLik:f#)",
  "AIC" = "($AIC:f#)",
  "BIC" = "($BIC:f#)",
  "N" = "($N:d)"
))

getSummary.multinom <- function(obj, alpha = 0.05, ...) {
  ## extract coefficient summary
  s <- summary(obj)
  cf <- s$coefficients

  ## set up array
  acf <- array(dim = c(NCOL(cf), 6, NROW(cf)),
    dimnames = list(colnames(cf), c("est", "se", "stat", "p", "lwr", "upr"), rownames(cf)))
    
  ## coefficients and standard errors
  acf[, 1, ] <- t(cf)
  acf[, 2, ] <- t(s$standard.errors)

  ## compute z-statistic and asymptotic p-value
  acf[, 3, ] <- acf[, 1, ] / acf[, 2, ]
  acf[, 4, ] <- 2 * pnorm(abs(acf[, 3, ]), lower.tail = FALSE)
  
  ## asymptotic confidence intervals
  cval <- qnorm(1 - alpha/2)
  acf[, 5, ] <- acf[, 1, ] - cval * acf[, 2, ]
  acf[, 6, ] <- acf[, 1, ] + cval * acf[, 2, ]

  ## compute number of observations as sum of weights
  nobs <- sum(obj$weights)

  ## return everything
  return(list(
    coef = acf,
    sumstat = c(
      "N" = nobs,
      "deviance" = obj$deviance,
      "logLik" = as.vector(logLik(obj)),
      "AIC" = AIC(obj),
      "BIC" = AIC(obj, k = log(nobs))
    ),
    contrasts = obj$contrasts,
    xlevels = obj$xlevels,
    call = obj$call
  ))
}

setSummaryTemplate("multinom" = c(
  "Deviance" = "($deviance:f#)",
  "Log-likelihood" = "($logLik:f#)",
  "AIC" = "($AIC:f#)",
  "BIC" = "($BIC:f#)",
  "N" = "($N:d)"
))

each_term <- function(x){
   if(length(x) < 3)
       return(x)
   else
       c(Recall(x[[2]]),x[[3]])
}

dfTapply <- function (X, INDEX, FUN = NULL, ..., default = NA, simplify = TRUE) 
{
    FUN <- if (!is.null(FUN)) 
        match.fun(FUN)
    if (!is.list(INDEX)) 
        INDEX <- list(INDEX)
    INDEX <- lapply(INDEX, as.factor)
    nI <- length(INDEX)
    if (!nI) 
        stop("'INDEX' is of length zero")
    if (!all(lengths(INDEX) == NROW(X))) 
        stop("arguments must have same length")
    namelist <- lapply(INDEX, levels)
    extent <- lengths(namelist, use.names = FALSE)
    cumextent <- cumprod(extent)
    if (cumextent[nI] > .Machine$integer.max) 
        stop("total number of levels >= 2^31")
    storage.mode(cumextent) <- "integer"
    ngroup <- cumextent[nI]
    group <- as.integer(INDEX[[1L]])
    if (nI > 1L) 
        for (i in 2L:nI) group <- group + cumextent[i - 1L] * 
            (as.integer(INDEX[[i]]) - 1L)
    if (is.null(FUN)) 
        return(group)
    levels(group) <- as.character(seq_len(ngroup))
    class(group) <- "factor"
    ans <- split(X, group)
    names(ans) <- NULL
    index <- as.logical(lengths(ans))
    ans <- lapply(X = ans[index], FUN = FUN, ...)
    ansmat <- array(if (simplify && all(lengths(ans) == 1L)) {
        ans <- unlist(ans, recursive = FALSE, use.names = FALSE)
        if (!is.null(ans) && is.na(default) && is.atomic(ans)) 
            vector(typeof(ans))
        else default
    }
    else vector("list", prod(extent)), dim = extent, dimnames = namelist)
    if (length(ans)) {
        ansmat[index] <- ans
    }
    ansmat
}

mean_se <- function(x,na.rm=FALSE){
    if(na.rm)
        x <- na.omit(x)
    n <- length(x)
    sqrt(var(x)/n)
}

nvalid <- function(x) length(na.omit(x))

WMean <- function(x,na.rm=TRUE){
    weighted.mean(x=x[,1],w=x[,2],na.rm=TRUE)
}

WMean_se <- function(x,w,na.rm=FALSE){
    if(na.rm){
        isn <- is.na(x) || is.na(w)
        x <- x[!isn]
        w <- w[!isn]
    }
    n <- sum(w)
    nn1 <- n/(n-1)
    wm.x <- weighted.mean(x,w)
    wvar.x <- weighted.mean((x-wm.x)^2)*nn1
    sqrt(wvar.x/n)
}

is.formula <- function(x) typeof(x) == "language" && class(x) == "formula"

Means.data.frame <-
    function(data,
             by,
             weights=NULL,
             subset = NULL,
             default = NA,
             se = FALSE,
             ci = FALSE,
             ci.level = 0.95,
             counts = FALSE,
             ...){
    if(is.formula(by)){
        formula <- by
        lhs <- formula[[2]]
        rhs <- formula[[3]]
        if(typeof(lhs) == "symbol"){
            nms <- deparse(lhs)
        }
        else if(deparse(lhs[[1]]) != "cbind"){
          lhs <- each_term(lhs)
          nms <- sapply(lhs,deparse)
          lhs <- as.call(c(quote(cbind),lhs))
          formula[[2]] <- lhs
        }
        else {
            if(length(names(lhs)))
                nms <- names(lhs)[-1]
            else
                nms <- NULL
        }
        mf <- match.call(expand.dots=FALSE)
        m <- match(c("data", "subset", "weights"), names(mf), 0L)
        mf <- mf[c(1L, m)]
        mf$drop.unused.levels <- TRUE
        mf$na.action <- quote(na.pass)
        mf$formula <- formula
        mf[[1L]] <- quote(stats::model.frame)

        mf <- eval(mf, parent.frame())
        if(!missing(weights)){
            weights <- model.weights(mf)
            mf["(weights)"] <- NULL
        }
        rsp <- mf[[1]]
        rsp <- as.data.frame(rsp)
        names(rsp) <- nms
        if(identical(rhs,1)){
            fcts <- rep(1,nrow(data))
            rhs_const <- TRUE
        }
        else{
            fcts <- mf[-1]
            rhs_const <- FALSE
        }
    }
    else if(is.character(by)){
        fcts <- data[by]
        rsp <- !(names(data) %in% by)
        rsp <- data[rsp]
        rhs_const <- FALSE
    }
    else if(is.data.frame(by)){
        rsp <- !(names(data) %in% names(by))
        fcts <- by
        rsp <- data[rsp]
        rhs_const <- FALSE
    }
    else if(is.numeric(by) && length(by)==1){
        fcts <- rep(1,nrow(data))
        rsp <- data
        rhs_const <- TRUE
    } else {
        fcts <- by
        rsp <- data
        rhs_const <- FALSE
    }
    if(!rhs_const && !all(sapply(fcts,is.factor))) stop("all grouping variables must be factors")
    nms <- names(rsp)
    res <- vector(mode="list",length=ncol(rsp))
    means <- res
    if(missing(weights) || !length(weights)){
        for(i in 1:ncol(rsp)){
            means[[i]] <- tapply(rsp[[i]],fcts,mean,na.rm=TRUE,default=default)
        }
    }
    else{
        for(i in 1:ncol(rsp)){
            rsp_i <- data.frame(rsp[[i]],weights)
            means[[i]] <- dfTapply(rsp_i,fcts,WMean,na.rm=TRUE,default=default)
        }
    }
    names(means) <- nms
    means <- simplify2array(means)
    if(rhs_const) names(means) <- nms
    do_se <- se
    do_se <- do_se || ci
    do_counts <- counts || ci
    if(do_counts){
        n <- res
        for(i in 1:ncol(rsp)){
            n[[i]] <- tapply(rsp[[i]],fcts,nvalid,default=default)
        }
        names(n) <- nms
        n <- simplify2array(n)
    }
    if(do_se){
        mean_se <- res
        for(i in 1:ncol(rsp)){
            mean_se[[i]] <- tapply(rsp[[i]],fcts,mean_se,na.rm=TRUE,default=default)
        }
        names(mean_se) <- nms
        mean_se <- simplify2array(mean_se)
    }
    if(ci){
        alpha <- 1-ci.level
        tval <- qt(1-alpha/2,df=n-1)
        se_tval <- mean_se*tval
        Lower <- means - se_tval
        Upper <- means + se_tval
        ConfInt <- list(Lower = Lower,
                        Upper = Upper)
    }
    res <- list(Mean = means)
    if(se){
        res <- c(res,list(SE = mean_se))
    }
    if(ci){
        res <- c(res,ConfInt)
    }
    if(counts){
        res <- c(res,list(N = n))
    }
    if(se || ci || counts){
        if(rhs_const)
                res <- do.call(cbind,res)
            else
                res <- simplify2array(res)
        ldims <- length(dim(res))
        if(ldims > 1){
            names(dimnames(res))[ldims-1] <- "Variable"
            names(dimnames(res))[ldims] <- "Statistic"
        }
        structure(res,class=c("xmeans.table","means.table","table"),
                  rhs_const=rhs_const)
    }
    else{
        res <- res$Mean
        if(!is.array(res)){
            res <- as.matrix(res)
            colnames(res) <- "Mean"
        }
        structure(res,class=c("means.table","table"),
                  rhs_const=rhs_const)
    }
}

Means.numeric <- function(data,...){
    data <- eval(substitute(data.frame(data)),parent.frame())
    cl <- match.call()
    cl$data <- data
    cl[[1]] <- as.symbol("Means")
    eval(cl,parent.frame())
}

Means.data.set <- function(data,...){
    cl <- match.call()
    data <- as.data.frame(data)
    cl$data <- data
    cl[[1]] <- as.symbol("Means")
    eval(cl,parent.frame())
}

Means.formula <- function(data,subset,weights,
                          ...){
    formula <- data
    cl <- mf <- match.call(expand.dots=FALSE)
    m <- match(c("subset", "weights"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf$formula <- formula
    mf[[1L]] <- quote(stats::model.frame)
    data <- eval(mf, parent.frame())
    by <- formula
    cl$data <- data
    cl$by <- by
    cl[[1]] <- as.symbol("Means")
    eval(cl,parent.frame())
}

Means <- function(data,...) UseMethod("Means")

as.data.frame.means.table <- function(x, row.names=NULL, optional=TRUE, drop=TRUE, ...){
    rhs_const <- attr(x,"rhs_const")
    ldims <- length(dim(x))
    if(rhs_const)
        res <- as.data.frame.matrix(x)
    else
        res <- to.data.frame(x,as.vars=ldims)
    if(drop)
        na.omit(res)
    else res
}

as.data.frame.xmeans.table <- function(x, row.names=NULL, optional=TRUE, drop=TRUE, ...){
    rhs_const <- attr(x,"rhs_const")
    ldims <- length(dim(x))
    if(rhs_const)
        res <- to.data.frame(x,as.vars=2)
    else
        res <- to.data.frame(x,as.vars=ldims)
    if(drop)
        na.omit(res)
    else res
}

mat_format_stdstyle <- c(
  "padding-top"="3px",
  "padding-bottom"="3px",
  "padding-left"="0.5ex",
  "padding-right"="0.5ex",
  "margin-top"="0px",
  "margin-bottom"="0px",
  "border-style"="none",
  "border-width"="0px"
)


# Don't use this!
format_html.matrix <- function(x,
                               toprule=2,midrule=1,bottomrule=2,
                               split.dec=TRUE,
                               formatC=FALSE,
                               digits=getOption("digits"),
                               format="f",
                               style=mat_format_stdstyle,
                               margin="2ex auto",
                               ...){

  
  firstcol <- c("padding-left"="0.3em")
  lastcol <- c("padding-right"="0.3em")
  toprule <- c("border-top"=paste0(midrule,"px solid"))
  bottomrule <- c("border-bottom"=paste0(midrule,"px solid"))
  midrule_above <- c("border-top"=paste0(midrule,"px solid"))
  midrule <- c("border-bottom"=paste0(midrule,"px solid"))
  align.right <- c("text-align"="right")  
  align.left <- c("text-align"="left")  
  align.center <- c("text-align"="center")
  row_style <- c("border-style"="none")
  table_style <- c("border-collapse"="collapse" ,"border-style"="none")
  
  colsep <- ""
  rowsep <- "\n"
  
    n <- nrow(x)
    m <- ncol(x)
    dim.x <- dim(x)

  colspan <- integer(m)
  #body <- matrix(nrow=nrow(x),ncol=ncol(x))
  if(is.integer(x)){
    tmp <- formatC(x,format="d")
    body <- html_td(tmp,vectorize=TRUE,style=css(style))
    colspan <- 1L
    }
  else if(is.numeric(x)){
    if(formatC)
        tmp <- formatC(x,digits=digits,format=format)
    else
        tmp <- format(x)
    if(split.dec){
      tmp <- spltDec(tmp)
      body <- html_td_spltDec(tmp,style=css(style))
      colspan <- 3L
    }
    else{
      body <- html_td(tmp,vectorize=TRUE,style=css(style))
      colspan <- 1L
    }
  }
  else {
    tmp <- as.character(x)
    body <- html_td(tmp,vectorize=TRUE,style=css(style))
    colspan <- 1L
  }
  dim(body) <- dim.x
    
  if(length(rownames(x))){
    tmp <- rownames(x)
    ldr <- html_td(tmp,vectorize=TRUE,style=css(c(style,firstcol,align.right)))
    body <- cbind(ldr,body)
  }
    
  body[1,] <- lapply(body[1,],setStyle,toprule)
  body[n,] <- lapply(body[n,],setStyle,bottomrule)
  body <- apply(body,1,html_tr,style=as.css(row_style))
  
  if(length(colnames(x))){
    
    hdr <- colnames(x)
    if(length(rownames(x))){
      hdr <- c("",hdr)
      colspan <- c(1L,rep(colspan,m))
    }
    else
      colspan <- rep(colspan,m)
    hdr <- html_td(hdr,vectorize=TRUE,style=css(style))
    hdr[] <- mapply(setAttribs,hdr,colspan=colspan,SIMPLIFY=FALSE)
    hdr <- lapply(hdr,setStyle,align.center)
    hdr <- lapply(hdr,setStyle,toprule)
    hdr[[length(hdr)]] <- setStyle(hdr[[length(hdr)]],lastcol)
    hdr <- html_tr(hdr,style=as.css(row_style))
    
    ans <- html_table(c(list(hdr),body))
  }
  else
    ans <- html_table(body)
  
  if(length(margin))
    table_style <- c(table_style,margin=margin)
  style(ans) <- as.css(table_style)

  ans <- as.character(ans)
  return(ans)
}

each_term <- function(x){
   if(length(x) < 3)
       return(x)
   else
       c(Recall(x[[2]]),x[[3]])
}

dfTapply <- function (X, INDEX, FUN = NULL, ..., default = NA, simplify = TRUE) 
{
    FUN <- if (!is.null(FUN)) 
        match.fun(FUN)
    if (!is.list(INDEX)) 
        INDEX <- list(INDEX)
    INDEX <- lapply(INDEX, as.factor)
    nI <- length(INDEX)
    if (!nI) 
        stop("'INDEX' is of length zero")
    if (!all(lengths(INDEX) == NROW(X))) 
        stop("arguments must have same length")
    namelist <- lapply(INDEX, levels)
    extent <- lengths(namelist, use.names = FALSE)
    cumextent <- cumprod(extent)
    if (cumextent[nI] > .Machine$integer.max) 
        stop("total number of levels >= 2^31")
    storage.mode(cumextent) <- "integer"
    ngroup <- cumextent[nI]
    group <- as.integer(INDEX[[1L]])
    if (nI > 1L) 
        for (i in 2L:nI) group <- group + cumextent[i - 1L] * 
            (as.integer(INDEX[[i]]) - 1L)
    if (is.null(FUN)) 
        return(group)
    levels(group) <- as.character(seq_len(ngroup))
    class(group) <- "factor"
    ans <- split(X, group)
    names(ans) <- NULL
    index <- as.logical(lengths(ans))
    ans <- lapply(X = ans[index], FUN = FUN, ...)
    ansmat <- array(if (simplify && all(lengths(ans) == 1L)) {
        ans <- unlist(ans, recursive = FALSE, use.names = FALSE)
        if (!is.null(ans) && is.na(default) && is.atomic(ans)) 
            vector(typeof(ans))
        else default
    }
    else vector("list", prod(extent)), dim = extent, dimnames = namelist)
    if (length(ans)) {
        ansmat[index] <- ans
    }
    ansmat
}

mean_se <- function(x,na.rm=FALSE){
    if(na.rm)
        x <- na.omit(x)
    n <- length(x)
    sqrt(var(x)/n)
}

nvalid <- function(x) length(na.omit(x))

WMean <- function(x,na.rm=TRUE){
    weighted.mean(x=x[,1],w=x[,2],na.rm=TRUE)
}

WMean_se <- function(x,w,na.rm=FALSE){
    if(na.rm){
        isn <- is.na(x) || is.na(w)
        x <- x[!isn]
        w <- w[!isn]
    }
    n <- sum(w)
    nn1 <- n/(n-1)
    wm.x <- weighted.mean(x,w)
    wvar.x <- weighted.mean((x-wm.x)^2)*nn1
    sqrt(wvar.x/n)
}

is.formula <- function(x) typeof(x) == "language" && class(x) == "formula"

Means.data.frame <-
    function(data,
             by,
             weights=NULL,
             subset = NULL,
             default = NA,
             se = FALSE,
             ci = FALSE,
             ci.level = 0.95,
             counts = FALSE,
             ...){
    if(is.formula(by)){
        formula <- by
        lhs <- formula[[2]]
        rhs <- formula[[3]]
        if(typeof(lhs) == "symbol"){
            nms <- deparse(lhs)
        }
        else if(deparse(lhs[[1]]) != "cbind"){
          lhs <- each_term(lhs)
          nms <- sapply(lhs,deparse)
          lhs <- as.call(c(quote(cbind),lhs))
          formula[[2]] <- lhs
        }
        else {
            if(length(names(lhs)))
                nms <- names(lhs)[-1]
            else
                nms <- NULL
        }
        mf <- match.call(expand.dots=FALSE)
        m <- match(c("data", "subset", "weights"), names(mf), 0L)
        mf <- mf[c(1L, m)]
        mf$drop.unused.levels <- TRUE
        mf$na.action <- quote(na.pass)
        mf$formula <- formula
        mf[[1L]] <- quote(stats::model.frame)

        mf <- eval(mf, parent.frame())
        if(!missing(weights)){
            weights <- model.weights(mf)
            mf["(weights)"] <- NULL
        }
        rsp <- mf[[1]]
        rsp <- as.data.frame(rsp)
        names(rsp) <- nms
        if(identical(rhs,1)){
            fcts <- rep(1,nrow(data))
            rhs_const <- TRUE
        }
        else{
            fcts <- mf[-1]
            rhs_const <- FALSE
        }
    }
    else if(is.character(by)){
        fcts <- data[by]
        rsp <- !(names(data) %in% by)
        rsp <- data[rsp]
        rhs_const <- FALSE
    }
    else if(is.data.frame(by)){
        rsp <- !(names(data) %in% names(by))
        fcts <- by
        rsp <- data[rsp]
        rhs_const <- FALSE
    }
    else if(is.numeric(by) && length(by)==1){
        fcts <- rep(1,nrow(data))
        rsp <- data
        rhs_const <- TRUE
    } else {
        fcts <- by
        rsp <- data
        rhs_const <- FALSE
    }
    if(!rhs_const && !all(sapply(fcts,is.factor))) stop("all grouping variables must be factors")
    nms <- names(rsp)
    res <- vector(mode="list",length=ncol(rsp))
    means <- res
    if(missing(weights) || !length(weights)){
        for(i in 1:ncol(rsp)){
            means[[i]] <- tapply(rsp[[i]],fcts,mean,na.rm=TRUE,default=default)
        }
    }
    else{
        for(i in 1:ncol(rsp)){
            rsp_i <- data.frame(rsp[[i]],weights)
            means[[i]] <- dfTapply(rsp_i,fcts,WMean,na.rm=TRUE,default=default)
        }
    }
    names(means) <- nms
    means <- simplify2array(means)
    if(rhs_const) names(means) <- nms
    do_se <- se
    do_se <- do_se || ci
    do_counts <- counts || ci
    if(do_counts){
        n <- res
        for(i in 1:ncol(rsp)){
            n[[i]] <- tapply(rsp[[i]],fcts,nvalid,default=default)
        }
        names(n) <- nms
        n <- simplify2array(n)
    }
    if(do_se){
        mean_se <- res
        for(i in 1:ncol(rsp)){
            mean_se[[i]] <- tapply(rsp[[i]],fcts,mean_se,na.rm=TRUE,default=default)
        }
        names(mean_se) <- nms
        mean_se <- simplify2array(mean_se)
    }
    if(ci){
        alpha <- 1-ci.level
        tval <- qt(1-alpha/2,df=n-1)
        se_tval <- mean_se*tval
        Lower <- means - se_tval
        Upper <- means - se_tval
        ConfInt <- list(Lower = Lower,
                        Upper = Upper)
    }
    res <- list(Mean = means)
    if(se){
        res <- c(res,list(SE = mean_se))
    }
    if(ci){
        res <- c(res,ConfInt)
    }
    if(counts){
        res <- c(res,list(N = n))
    }
    if(se || ci || counts){
        if(rhs_const)
                res <- do.call(cbind,res)
            else
                res <- simplify2array(res)
        ldims <- length(dim(res))
        if(ldims > 1){
            names(dimnames(res))[ldims-1] <- "Variable"
            names(dimnames(res))[ldims] <- "Statistic"
        }
        structure(res,class=c("xmeans.table","means.table","table"),
                  rhs_const=rhs_const)
    }
    else{
        res <- res$Mean
        if(!is.array(res)){
            res <- as.matrix(res)
            colnames(res) <- "Mean"
        }
        structure(res,class=c("means.table","table"),
                  rhs_const=rhs_const)
    }
}

Means.numeric <- function(data,...){
    data <- eval(substitute(data.frame(data)),parent.frame())
    cl <- match.call()
    cl$data <- data
    cl[[1]] <- as.symbol("Means")
    eval(cl,parent.frame())
}

Means.data.set <- function(data,...){
    cl <- match.call()
    data <- as.data.frame(data)
    cl$data <- data
    cl[[1]] <- as.symbol("Means")
    eval(cl,parent.frame())
}

Means.formula <- function(data,subset,weights,
                          ...){
    formula <- data
    cl <- mf <- match.call(expand.dots=FALSE)
    m <- match(c("subset", "weights"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf$formula <- formula
    mf[[1L]] <- quote(stats::model.frame)
    data <- eval(mf, parent.frame())
    by <- formula
    cl$data <- data
    cl$by <- by
    cl[[1]] <- as.symbol("Means")
    eval(cl,parent.frame())
}

Means <- function(data,...) UseMethod("Means")

as.data.frame.means.table <- function(x, row.names=NULL, optional=TRUE, drop=TRUE, ...){
    rhs_const <- attr(x,"rhs_const")
    ldims <- length(dim(x))
    if(rhs_const)
        res <- as.data.frame.matrix(x)
    else
        res <- to.data.frame(x,as.vars=ldims)
    if(drop)
        na.omit(res)
    else res
}

as.data.frame.xmeans.table <- function(x, row.names=NULL, optional=TRUE, drop=TRUE, ...){
    rhs_const <- attr(x,"rhs_const")
    ldims <- length(dim(x))
    if(rhs_const)
        res <- to.data.frame(x,as.vars=2)
    else
        res <- to.data.frame(x,as.vars=ldims)
    if(drop)
        na.omit(res)
    else res
}

getSummary_expcoef.default <- function (obj, alpha = 0.05, ...){
    res <- getSummary(obj=obj, alpha=alpha, ...)
    coef <- res$coef
    exp_coef <- coef
    exp_coef[,1,] <- exp(coef[,1,])
    # Based on the delta-method
    exp_coef[,2,] <- exp_coef[,1,]*coef[,2,]
    if(ncol(coef) ==6)
        exp_coef[,5:6,] <- exp(coef[,5:6,])
    res$coef <- exp_coef
    return(res)
}

rofile <- function(filename) .Call("rofile",as.character(filename))

readfixed <- function(file,what,nlines,start,stop)
  .Call("readfixed",file,what,nlines,start,stop)

readfixedslice <- function(file,what,j,i,start,stop)
  .Call("readfixedslice",file,what,j,i,start,stop)

readfixedchunk <- function(file,what,j,n,start,stop)
  .Call("readfixedchunk",file,what,j,n,start,stop)

roreadline <- function(file)
  .Call("rofreadline",file)

# print.rofile <- function(x,...)
#   cat("Read-only file",dQuote(attr(x,"filename")),"\n")

roftell <- function(f) .Call("roftell",f)
rofseek <- function(f,pos=0,whence=0) .Call("rofseek",f,pos=pos,whence=whence)

spss.parse.data.spec <- function(file){
  
  text <- paste(readLines(file,n=-1,warn=FALSE),collapse=" ")
  
  keyword <- .pattern("data\\s+list\\s+",text,ignore.case=TRUE)
  if(!length(keyword)) return(NULL)
  
  header <- .pattern("/",text,before=TRUE,ignore.case=TRUE)
  text <- .remainder(header)
  
  skip <- .pattern("skip\\s*=\\s*[0-9]+",header,ignore.case = TRUE)
  if(!length(skip)) skip <- 0L
  else {
    
    skip <- .pattern("[0-9]+",skip)
    skip <- as.integer(skip)
  }
  
  varnames <- character()
  start <- integer()
  end <- integer()
  types <- integer()
  
  repeat {
    
    if(grepl("^[ \t\r\n]*[.]",text)){
      #message("found closing dot")
      break
    }
    vn <- .getvarnames(text)
    if(!length(vn)) {
      break
    }
    text <- .remainder(vn)
    
    start1 <- .pattern("^[0-9]+",text)
    text <- .remainder(start1)
    start1 <- as.integer(start1)
    text <- .remainder(.pattern("^\\s*-\\s*",text))
    end1 <- .pattern("^[0-9]+",text)
    text <- .remainder(end1)
    end1 <- as.integer(end1)
    type1 <- .pattern("^[(][aA0-9]+[])]",text) 
    if(!length(type1))
      type1 <- 1
    else {
      text <- .remainder(type1)
      type1 <- .pattern("[aA]",type1)
      if(length(type1))
        type1 <- 2
      else
        type1 <- 1
    }
    
    varnames <- c(varnames,vn)
    types <- c(types,type1)
    start <- c(start,start1)
    end <- c(end,end1)
  }
  names(types) <- varnames
  list(
    types=types,
    start=start,
    stop=end,
    skip=skip
  )
}

spss.parse.variable.labels <- function(file,iconv=TRUE,
    encoded=getOption("spss.syntax.encoding","cp1252")){
  text <- paste(readLines(file,n=-1,warn=FALSE),collapse=" ")
  if(iconv)
      text <- iconv(text,from=encoded,to="")
  text <- gsub("\"\"","'",text)
  text <- gsub("''","\"",text)
  res <- character()

  repeat{
    keyword <- .pattern("variable\\s+labels\\s+",text,ignore.case=TRUE)
    if(!length(keyword)) break
    text <- .remainder(keyword)
    vl <- spss.parse1.variable.labels(text)
    text <- .remainder(vl)
    .remainder(vl) <- NULL
    res <- c(res,vl)
  }
  res
}  

spss.parse1.variable.labels <- function(text){
  
  
  names <- character()
  labels <- character()
  
  # Remove leading slash stuff
  text <- sub("^\\s*/\\s*","",text)
  repeat {
    if(grepl("^[ \t\r\n]*[.]",text)){
      #message("found closing dot")
      break
    }
    varnames <- .getvarnames(text)
    text <- .remainder(varnames)
    lab <- .getquoted(text)
    text <- .remainder(lab)
    
    names <- c(names,varnames)
    labels <- c(labels,lab)
    #res <- c(res,list(list(varnames=varnames,vallabs=vallabs)))
    if(nchar(text)<1) break
  }
  
  names(labels) <- names
  structure(labels,
            remainder=text)
}

spss.parse.value.labels <- function(file,iconv=TRUE,
    encoded=getOption("spss.syntax.encoding","cp1252")){
  text <- paste(readLines(file,n=-1,warn=FALSE),collapse=" ")
  if(iconv)
      text <- iconv(text,from=encoded,to="")
  text <- gsub("\"\"","'",text)
  text <- gsub("''","\"",text)
  res <- list()
  repeat{
    keyword <- .pattern("value\\s+labels\\s+",text,ignore.case=TRUE)
    if(!length(keyword)) break
    text <- .remainder(keyword)
    vl <- spss.parse1.value.labels(text)
    text <- .remainder(vl)
    .remainder(vl) <- NULL
    res <- c(res,vl)
  }
  res
}  

spss.parse1.value.labels <- function(text){
  
  res <- list()
  repeat {
    
    if(grepl("^[ \t\r\n]*[.]",text)){
      #message("found closing dot")
      break
    }
    varnames <- .getvarnames(text)
    text <- .remainder(varnames)
    .remainder(varnames) <- NULL
    vallabs <- .getvallabs(text)
    text <- .remainder(vallabs)
    labs <- vallabs$labels
    vallabs <- vallabs$values
    names(vallabs) <- labs
    
    vallabs <- rep(list(vallabs),length(varnames))
    names(vallabs) <- varnames
    res <- c(res,vallabs)
    
    if(nchar(text)<1) break
  }
  structure(res,
            remainder=text)
}


spss.parse.missing.values <- function(file){
  
  text <- paste(readLines(file,n=-1,warn=FALSE),collapse=" ")
  res <- list()
  repeat{
    
    keyword <- .pattern("missing\\s+values\\s+",text,ignore.case=TRUE)
    if(!length(keyword)) break
    text <- .remainder(keyword)
    mv <- spss.parse1.missing.values(text)
    text <- .remainder(mv)
    .remainder(mv) <- NULL
    res <- c(res,mv)
  }
  res
}

spss.parse1.missing.values <- function(text){
  
  missvals <- list()
  
  repeat{
    
    if(grepl("^[ \t\r\n]*[.]",text)){
      #message("found closing dot")
      break
    }
    vn <- .getvarnames(text)
    text <- .remainder(vn)
    if(!length(vn)) break
    
    mvspec <- .pattern("^[(].*?[)]",text)
    if(!length(mvspec)) break
    text <- .remainder(mvspec)
    
    mvspec <- gsub("[()]","",mvspec)
    .remainder(mvspec) <- NULL
    
    if(grepl("thru",mvspec,ignore.case=TRUE)){
      lo <- .pattern("thru",mvspec,before=TRUE,ignore.case=TRUE)
      hi <- .remainder(lo)
      suppressWarnings(lo <- as.numeric(lo))
      if(!is.finite(lo)) lo <- -Inf
      
      if(grepl(",",hi)){
        hi <- .pattern(",",hi,before=TRUE)
        vals <- .remainder(hi)
        vals <- as.numeric(vals)
      }
      else vals <- NULL
      suppressWarnings(hi <- as.numeric(hi))
      if(!is.finite(hi)) hi <- Inf  
      range <- c(lo,hi)
    }
    else if(!grepl("\"",mvspec)) {
      range <- NULL
      vals <- numeric()
      for(ii in 1:3){
        v <- .pattern(",",mvspec,before=TRUE)
        mvspec <- .remainder(v)
        vals <- c(vals, as.numeric(v))
        if(!length(mvspec)) break
      }
    }
    else {
      range <- NULL
      vals <- character()
      for(ii in 1:3){
        v <- .pattern(",",mvspec,before=TRUE)
        mvspec <- .remainder(v)
        vals <- c(vals, gsub("\"","",v))
        if(!length(mvspec)) break
      }
    }
    mv <- list(values=vals,range=range)
    
    mv <- rep(list(mv),length(vn))
    names(mv) <- vn
    
    missvals <- c(missvals,mv)
  }
  return(structure(missvals,
                   remainder=text))
}


.getvarnames <- function(text){
  
  varnames <- character(0)
  text <- sub("^\\s*","",text)
  text <- sub("^[/;,]\\s*","",text)
  
  repeat {
    vn.pos <- regexpr("^[A-Za-z][A-Za-z0-9_]*",text)
    if(vn.pos < 0) return(structure(varnames,remainder=text))
    vn.length <- attr(vn.pos,"match.length")
    vn <- substring(text,first=1,last=vn.length)
    varnames <- c(varnames,vn)
    text <- substring(text,first=vn.length+1,last=nchar(text))
    text <- trimws(text,right=FALSE)
    text <- gsub("^[;,][ \t\n\r]*","",text)
  }
}

.getvallabs <- function(text){
  text <- trimws(text,right=FALSE)
  if(grepl("^[\"']",text)) .getvallabs1(text,strings=TRUE)
  else .getvallabs1(text)
}

.getvallabs1 <- function(text,strings=FALSE){
  
  if(strings)
    vals <- character(0)
  else
    vals <- numeric(0)
  labs <- character(0)
  repeat{
    text <- trimws(text,right=FALSE)
    if(grepl("^[/|.|;]",text)) return(structure(
      list(values=vals,labels=labs),
      remainder=text))
    
    if(strings){
      val <- .getquoted(text)
      text <- .remainder(val)
    }
    else{
      val <- .getnotquoted(text)
      text <- .remainder(val)
      val <- as.numeric(val)
    }
    .remainder(val) <- NULL
    lab <- .getquoted(text)
    text <- .remainder(lab)
    .remainder(lab) <- NULL
    
    vals <- c(vals,val)
    labs <- c(labs,lab)
  }
}

.getnotquoted <- function(text){
  text <- trimws(text,right=FALSE)
  if(!nzchar(text)) return(NULL)
  tok.pos <- regexpr("[^ \t\r\n]+",text)
  tok.len <- attr(tok.pos,"match.length")
  tok <- substring(text,first=1,last=tok.len)
  text <- substring(text,first=tok.len+1,last=nchar(text))
  structure(tok,remainder=text)
}

.getquoted <- function(text){
  text <- trimws(text,right=FALSE)
  if(!nzchar(text)) stop("empty string")
  if(!grepl("^[\"']",text)) stop("missing quotation mark")
  qm <- substring(text,first=1,last=1)
  text <- substring(text,first=2)
  qm.pos <- regexpr(qm,text)
  tok <- substring(text,first=1,last=qm.pos-1)
  text <- substring(text,first=qm.pos+1,last=nchar(text))
  structure(tok,remainder=text)
}

.pattern <- function(pattern,text,before=FALSE,...){
  attributes(text) <- NULL
  pat.pos <- regexpr(pattern,text,...)
  if(pat.pos < 1) {
    if(before) return(text)
    else return(NULL)
  }
  pat.len <- attr(pat.pos,"match.len")
  if(before)
    pat.found <- substring(text,first=1,last=pat.pos-1)
  else
    pat.found <- substring(text,first=pat.pos,last=pat.pos+pat.len-1)
  text <- substring(text,first=pat.pos+pat.len,last=nchar(text))
  structure(pat.found,remainder=text)
}

.remainder <- function(x){
  y <- attr(x,"remainder")
  attributes(y) <- NULL
  if(length(y)) sub('^[ \t\r\n]+', '',y)
  else y
}

".remainder<-" <- function(x,value){
  attr(x,"remainder") <- value
  x
}

mtable_format_stdstyle <- c(
  "padding-top"="1px",
  "padding-bottom"="1px",
  "padding-left"="0.5ex",
  "padding-right"="0.5ex",
  "margin-top"="0px",
  "margin-bottom"="0px",
  "border-style"="none",
  "border-width"="0px"
)

fillin <- function(template,substitutions){
    res <- template
    for(n in names(substitutions)){
        pat <- paste0("<<",n,">>")
        subst <- substitutions[n]
        res <- gsub(pat,subst,res)
    }
    res
}

class_id_selector <- function(id=NULL,class=NULL){
    class_or_id <- ""
    if(length(class)){
       class_or_id <- paste0(".",paste0(class,collapse="."))   
    } 
    if(length(id)){
        id <- paste0("#",id[1])
        class_or_id <- paste0(class_or_id,id)
    }
    class_or_id
}

style_mtable_global <- function(id=NULL,class=NULL,style=mtable_format_stdstyle,margin="2ex auto") {

    table_tmpl <- "
      table<<class-or-id>> {
         border-collapse: collapse; border-style: none; margin: <<margin>>;
      }"
    table_tr_tmpl <- "
      table<<class-or-id>> tr {
          border-style: none;
      }"
    table_td_tmpl <- "
      table<<class-or-id>> td {
        padding-top: <<padding-top>>; 
        padding-bottom: <<padding-bottom>>; 
        padding-left: <<padding-left>>; 
        padding-right: <<padding-right>>; 
        margin: 0px; 
        margin-top: <<margin-top>>;
        margin-bottom: <<margin-bottom>>;
        border-style: <<border-style>>; 
        border-width: <<border-width>>; 
      }"

    class_or_id <- class_id_selector(id=id,class=class)

    table_style <- fillin(table_tmpl,
                          c("class-or-id"=class_or_id,
                            margin=margin))
    table_tr_style <- fillin(table_tr_tmpl,
                             c("class-or-id"=class_or_id))
    table_td_style <- fillin(table_td_tmpl,
                             c("class-or-id"=class_or_id,
                               style))
    paste(table_style,table_tr_style,table_td_style)
}

style_mtable_rule <- function(id=NULL,class=NULL,top=FALSE,bottom=FALSE,rulewidth=1,rows=NULL){
    if(length(rows)){
        class_or_id <- class_id_selector(id=id,class=class)
        rulewidth <- paste0(rulewidth,"px")
        selector_tmpl <- "table<<class-or-id>> tr:nth-child(<<row>>)"
        selector <- character(0)
        for(row in rows){
            selector <- c(selector,
                          fillin(selector_tmpl,c("class-or-id"=class_or_id,row=row)))
        }
        selector <- paste(selector,collapse=",\n")
        style_tmpl <- "<<selector>>{"
        if(top){
            style_tmpl <- paste(style_tmpl,
                                "     border-top: <<rulewidth>> solid;",
                                sep="\n")
        }
        if(bottom){
            style_tmpl <- paste(style_tmpl,
                                "     border-bottom: <<rulewidth>> solid;",
                                sep="\n")
        }
        style_tmpl <- paste(style_tmpl,"}",sep="\n")

        fillin(style_tmpl,c(selector=selector,rulewidth=rulewidth))
    } else ""
}

style_mtable_cmidrule <- function(id=NULL,class=NULL,top=FALSE,bottom=FALSE,rulewidth=0.5,cols=NULL,rows=NULL){
    if(length(cols) && length(rows)){
        class_or_id <- class_id_selector(id=id,class=class)
        rulewidth <- paste0(rulewidth,"px")
        selector_tmpl <- "table<<class-or-id>> tr:nth-child(<<row>>) td:nth-child(<<col>>)"
        selector <- character(0)
        for(row in rows){
            for(col in cols){
                selector <- c(selector,
                              fillin(selector_tmpl,c("class-or-id"=class_or_id,row=row,col=col)))
            }
        }
        selector <- paste(selector,collapse=",\n")
        style_tmpl <- "<<selector>>{"
        if(top){
            style_tmpl <- paste(style_tmpl,
                                "     border-top: <<rulewidth>> solid;",
                                sep="\n")
        }
        if(bottom){
            style_tmpl <- paste(style_tmpl,
                                "     border-bottom: <<rulewidth>> solid;",
                                sep="\n")
        }
        style_tmpl <- paste(style_tmpl,"}",sep="\n")

        fillin(style_tmpl,c(selector=selector,rulewidth=rulewidth))
    } else ""
}



style_mtable_cols <- function(id=NULL,class=NULL,cols=NULL,style=""){
    if(length(cols)){
        class_or_id <- class_id_selector(id=id,class=class)
        selector_tmpl <- "table<<class-or-id>> td:nth-child(<<col>>)"
        selector <- character(0)
        for(col in cols){
            selector <- c(selector,
                          fillin(selector_tmpl,c("class-or-id"=class_or_id,col=col)))
        }
        selector <- paste(selector,collapse=",\n")
        paste0(selector,"{",style,"}")
    }
}

style_mtable_header <- function(id=NULL,class=NULL,style=""){
    class_or_id <- class_id_selector(id=id,class=class)
    selector_tmpl <- "table<<class-or-id>> td:nth-child(n of .header)"
    selector <- fillin(selector_tmpl,c("class-or-id"=class_or_id,col=col))
    paste0("\n",selector,"{",style,"}")
}



format_html.memisc_mtable <- function(x,
                               interaction.sep = NULL,
                               toprule=2,midrule=1,bottomrule=2,
                               split.dec=TRUE,
                               style=mtable_format_stdstyle,
                               margin="2ex auto",
                               sig.notes.style=c(width="inherit"),
                               ...){
  x <- preformat_mtable(x)
  res <- pf_mtable_format_html(x,interaction.sep=interaction.sep,
                        toprule=toprule,midrule=midrule,bottomrule=bottomrule,
                        split.dec=split.dec,style=style,margin=margin,
                        sig.notes.style=sig.notes.style,...)
  as.character(res)
}

mtable_format_html <- function(x,
                               interaction.sep = NULL,
                               toprule=2,midrule=1,bottomrule=2,
                               split.dec=TRUE,
                               style=mtable_format_stdstyle,
                               margin="2ex auto",
                               sig.notes.style=c(width="inherit"),
                               ...)
    pf_mtable_format_html(preformat_mtable(x),
                          interaction.sep=interaction.sep,
                          toprule=toprule,
                          midrule=midrule,
                          bottomrule=bottomrule,
                          split.dec=split.dec,
                          style=style,
                          margin=margin,
                          sig.notes.style=sig.notes.style,
                          ...)

mtable_html_env <- new.env()
mtable_html_env$counter <- 1

pf_mtable_format_html <- function(x,
                               interaction.sep = NULL,
                               toprule=2,midrule=1,bottomrule=2,
                               split.dec=TRUE,
                               style=mtable_format_stdstyle,
                               margin="2ex auto",
                               sig.notes.style=c(width="inherit"),
                               show.parmtypes = nrow(x$parmtab) > 1,
                               ...
){
  
    if(!length(interaction.sep)){
        if(getOption("html.use.ampersand",FALSE))
            interaction.sep <- " &times; "
        else 
            interaction.sep <- " \u00d7 "
    }

    colsep <- ""
    rowsep <- "\n"

    pt <- x$parmtab
    sst <- x$summary.stats
    leaders <- x$leaders
    headers <- x$headers
    eq.headers <- x$eq.headers
    outtypes <- x$outtypes
    l.headers <- length(headers)
    l.leaders <- length(leaders)

    ncols <- sapply(pt[1,,drop=FALSE],ncol)

    res <- NULL
    has.eq.headers <- length(eq.headers) > 0

    total_hdr_lines <- 1
    total_pt_lines <- 0
    total_sum_lines <- 0

    dot_cols <- integer(0)
    col_sum <- 0

    for(j in 1:ncol(pt)){
        
        name.j <- colnames(pt)[j]
        pt.j <- pt[,j]

        ncol.j <- unique(sapply(pt.j,ncol))
        stopifnot(length(ncol.j)==1)
        
        for(i in 1:length(pt.j)){

            pt.ij <- pt.j[[i]]
            ot.ij <- outtypes[i,j]
            dm.ij <- dim(pt.ij)
            
            if(ot.ij == "num"){
                if(getOption("html.use.ampersand",FALSE))
                    pt.ij[] <- gsub("-","&minus;",pt.ij[],fixed=TRUE)
                else
                    pt.ij[] <- gsub("-","\u2212",pt.ij[],fixed=TRUE)

                if(split.dec){
                    pt.ij <- spltDec(pt.ij)
                    pt.ij <- gsub("([*]+)","<span class=\"signif.symbol\">\\1</span>",pt.ij)
                    pt.ij <- html_td_spltDec(pt.ij,add_CSS=FALSE)
                }
                else
                    pt.ij[] <- html_td(pt.ij,vectorize=TRUE)
            } else {
                if(split.dec)
                    pt.ij[] <- html_td(pt.ij,colspan=3,vectorize=TRUE)
                else 
                    pt.ij[] <- html_td(pt.ij,vectorize=TRUE)
            }
            dim(pt.ij) <- dm.ij
            if(show.parmtypes){
                spaces <- rep(" ",ncol(pt.ij))
                spaces <- html_td(spaces,vectorize=TRUE)
                pt.ij <- rbind(spaces,pt.ij)
            }
            pt.j[[i]] <- pt.ij
        }
        pt.j <- do.call(rbind,pt.j)
        if(split.dec){
            dot_cols_cur <- (1:ncol(pt.j)-1)*3 + 2
            dot_cols <- c(dot_cols, col_sum + dot_cols_cur)
            col_sum <- col_sum + ncol(pt.j)*3
        }

        if(has.eq.headers){
           eq.header.j <- eq.headers[[name.j]]
            n.eq.j <- length(eq.header.j)
            eq.span <- ncol.j/n.eq.j
            if(split.dec)
                eq.span <- eq.span*3
            eq.header.j <- html_td(eq.header.j,colspan=eq.span,vectorize=TRUE,class="header")
            # pt.j <- rbind(eq.header.j,pt.j)
            #total_hdr_lines <- max(total_hdr_lines,1+n.eq.j)
           eq.headers[[name.j]] <- eq.header.j
        }
        total_pt_lines <- max(total_pt_lines,1+nrow(pt.j))

        if(length(sst)){
            sst.j <- sst[[j]]
            if(getOption("html.use.ampersand",FALSE))
                sst.j <- gsub("-","&minus;",sst.j,fixed=TRUE)
            else
                sst.j <- gsub("-","\u2212",sst.j,fixed=TRUE)
            sst.j <- colexpand(sst.j,ncol.j)
            dm.ij <- dim(sst.j)
            if(split.dec){
                sst.j <- spltDec(sst.j)
                sst.j <- html_td_spltDec(sst.j,add_CSS=FALSE)
            }
            else
                sst.j <- html_td(sst.j,vectorize=TRUE)
            dim(sst.j) <- dm.ij
            pt.j <- rbind(pt.j,sst.j)
            total_sum_lines <- max(total_sum_lines,nrow(sst.j))
        }

        pt.j <- apply(pt.j,1,as.html_group)
        
        res <- cbind(res,pt.j)
    }

    if(l.leaders){
        leaders <- lapply(leaders,ldxp)
        if(show.parmtypes){
            parmtypes <- rownames(x$parmtab)
            for(p in parmtypes){
                leaders[[p]] <- rbind(p,leaders[[p]])
            }
        }
        leaders <- do.call(rbind,leaders)

        leaders <- html_td(leaders,vectorize=TRUE)
        
        res <- cbind(leaders,res)
    }

    res <- apply(res,1,as.html_group)

    if(l.headers){
        for(k in 1:l.headers){
            headers.k <- headers[[k]]
            hspan.k <- sapply(headers.k,attr,"span")
            if(split.dec)
                hspan.k <- hspan.k*3
            headers.k <- Map(html_td,headers.k,colspan=hspan.k,class="header")
            if(l.leaders){
                lheader.k <- html_td("",colspan=1,class="header")
                headers.k <- c(list(lheader.k),headers.k)
            }
            headers[[k]] <- headers.k
        }
        headers <- lapply(headers,as.html_group)
        if(has.eq.headers) {
            if(l.leaders){
                lheader.e <- html_td("",colspan=1,class="header")
                tmp <- unlist(eq.headers,recursive=FALSE)
                eq.headers <- as.html_group(c(list(lheader.e),tmp))
            }
            headers[[l.headers + 1]] <- eq.headers
        }
        res <- c(headers,res)
    }
    else if(has.eq.headers) {
        if(l.leaders){
            lheader.e <- html_td("",colspan=1,class="header")
            tmp <- unlist(eq.headers,recursive=FALSE)
            eq.headers <- as.html_group(c(list(lheader.e),tmp))
        }
        res <- c(list(eq.headers), res)
    }

    csum <- l.headers

    sect.at <- integer()
    for(i in 1:nrow(pt)){
        sect.at <- c(sect.at,csum)
        csum <- csum + nrow(pt[[i,1]])
        if(nrow(pt) > 1)
            csum <- csum + 1
    }
    if(length(sst) && any(sapply(sst,length)>0))
        sect.at <- c(sect.at,csum)
    if(has.eq.headers)
        sect.at <- sect.at + 1
    lines_total <- length(res)

    signif.symbols <- x$signif.symbols
    if(length(signif.symbols)){
        signif.template <- getOption("signif.symbol.print.template",
                                     signif.symbol.print.default.template)
        signif.symbols <- format_signif_print(signif.symbols,
                                              signif.template,
                                              width=72)
        signif.symbols <- gsub("<","&lt;",signif.symbols)
        signif.symbols <- gsub(">","&gt;",signif.symbols)
        
        if(split.dec)
            totspan <- sum(ncols) * 3
        else
            totspan <- sum(ncols)
        if(l.leaders)
            totspan <- totspan + 1
        
        signif.symbols <- html_p(signif.symbols)
        signif.symbols <- html_td(signif.symbols,colspan=totspan)
        res <- c(res,list(signif.symbols))
    }
    
    res <- html_tr(res,vectorize=TRUE)

    mtable_id <- paste0("mtable-",mtable_html_env$counter)
    res <- html_table(res,id=mtable_id)

    mtable_html_env$counter <- mtable_html_env$counter + 1

    #midrule_lines <- total_hdr_lines
    #midrule_lines <- c(midrule_lines,total_hdr_lines + total_pt_lines)
    midrule_lines <- sect.at

    style_global <- style_mtable_global(id=mtable_id,style=style,margin=margin)
    style_toprule <- style_mtable_rule(id=mtable_id,rulewidth=toprule,top=TRUE,
                                       rows=1)
    style_bottomrule <- style_mtable_rule(id=mtable_id,rulewidth=bottomrule,bottom=TRUE,
                                          rows=lines_total)
    style_midrule <- style_mtable_rule(id=mtable_id,rulewidth=midrule,bottom=TRUE,
                                          rows=midrule_lines)


    if(l.headers > 0 && has.eq.headers) {
        if(l.leaders) cmod_fst <- 2 else cmod_fst <- 1
        last.header <- headers[[l.headers]]
        cmod_lst <- length((last.header))
        cmid_cols <- cmod_fst:cmod_lst
        style_mtable_cmidrule <- style_mtable_cmidrule(id=mtable_id,bottom=TRUE,
                                                       rows=l.headers,cols=cmid_cols)
    }
    else style_mtable_cmidrule <- NULL

    style_content <- paste(
        style_global,
        style_toprule,
        style_mtable_cmidrule,
        style_midrule,
        style_bottomrule,
        sep="\n"
    )
    if(split.dec){
        dot_style <- "
         padding-left: 0px;
         padding-right: 0px;
         text-align: center; 
         /*background-color: blue;*/
        "
        before_dot_style <- "
         padding-left: 0.5ex;
         padding-right: 0px;
         text-align: right; 
         /*background-color: red;*/
        "
        behind_dot_style <- "
         padding-left: 0px;
         padding-right: 0.5ex;
         text-align: left; 
         /*background-color: green;*/
        "
        
        style_dots <- style_mtable_cols(id=mtable_id,cols=dot_cols+1,style=dot_style)
        style_before_dots <- style_mtable_cols(id=mtable_id,cols=dot_cols,style=before_dot_style)
        style_behind_dots <- style_mtable_cols(id=mtable_id,cols=dot_cols+2,style=behind_dot_style)
        style_content <- paste(style_content,
                               style_dots,
                               style_before_dots,
                               style_behind_dots)
    }
    header_style <- "
         padding-left: 0.5ex;
         padding-right: 0.5ex;
         text-align: center; 
         /*background-color: gray;*/
        "
    style_header <- style_mtable_header(id=mtable_id,style=header_style)
    style_content <- paste(style_content,
                           style_header,
                           "\n")

    style_element <- html("style",style_content,linebreak=FALSE)

    res <- html_group(style_element,res)

    return(res)

}

get_colspan <- function(x)x$attributes$colspan

as_item_item <- function(x,
  labels=NULL,
  missing.values=NULL,
  valid.values=NULL,
  valid.range=NULL,
  value.filter=NULL,
  measurement=NULL,
  annotation=attr(x,"annotation"),
  ...
  ){
  if(!missing(labels))
    x@value.labels <- if(length(labels)) as(labels,"value.labels") else NULL
  if(!(missing(missing.values) && missing(valid.values) && missing(valid.range) && missing(value.filter))){
    mv <- length(missing.values) > 0
    vv <- length(valid.values) > 0
    vr <- length(valid.range) > 0
    vf <- length(value.filter) > 0
    if(mv + vv + vr + vf > 1)
      stop("to many value filter arguments")
    x@value.filter <- if(mv) as(missing.values,"missing.values")
                    else if(vv) as(valid.values,"valid.values")
                    else if(vr) as(valid.range,"valid.range")
                    else if(vf) {
                          if(!is(value.filter,"value.filter"))
                            stop("'value.filter' not of appropriate class")
                          value.filter
                          }
                  else NULL
  }
  if(!missing(measurement)){
    measurement <- if(length(measurement)) match.arg(measurement,c("nominal","ordinal","interval","ratio"))
                  else if(length(labels) && !length(measurement)) "nominal" else "interval"
    measurement(x) <- measurement
  }
  if(!missing(annotation)){
    annotation <- if(length(annotation))
                  new("annotation",structure(as.character(annotation),names=names(annotation)))
                  else x@annotation
    annotation(x) <- annotation
  }
  x
}

setMethod("as.item","double.item",as_item_item)
setMethod("as.item","integer.item",as_item_item)
setMethod("as.item","character.item",as_item_item)

setMethod("as.item","numeric",function(x,
  labels=NULL,
  missing.values=NULL,
  valid.values=NULL,
  valid.range=NULL,
  value.filter=NULL,
  measurement=NULL,
  annotation=attr(x,"annotation"),
  ...
  ){
  value.labels <- if(length(labels)) as(labels,"value.labels") else NULL
  mv <- length(missing.values) > 0
  vv <- length(valid.values) > 0
  vr <- length(valid.range) > 0
  vf <- length(value.filter) > 0
  if(mv + vv + vr + vf > 1)
    stop("to many value filter arguments")
  value.filter <- if(mv) as(missing.values,"missing.values")
                  else if(vv) as(valid.values,"valid.values")
                  else if(vr) as(valid.range,"valid.range")
                  else if(vf) {
                        if(!is(value.filter,"value.filter"))
                          stop("'value.filter' not of appropriate class")
                        value.filter
                        }
  measurement <- if(length(measurement)) match.arg(measurement,c("nominal","ordinal","interval","ratio"))
                 else if(length(value.labels) && !length(measurement)) "nominal" else "interval"
  annotation <- new("annotation",structure(as.character(annotation),names=names(annotation)))
  label <- attr(x,"label")
  if("description" %nin% names(annotation) && length(label))
      annotation["description"] <- label
  cl <- paste(storage.mode(x),"item",sep=".")
  new(cl,
    x,
    value.labels=value.labels,
    value.filter=value.filter,
    measurement=measurement,
    annotation=annotation
  )
})

setMethod("as.item","logical",function(x,...) {
  y <- as.integer(x)
  annotation <- attr(x,"annotation")
  label <- attr(x,"label")
  if("description" %nin% names(annotation) && length(label))
      annotation["description"] <- label
  attr(y,"annotation") <- annotation
  as.item(y,...)
})
setMethod("as.item","factor",function(x,...){
    y <- as.integer(x)
    annotation <- attr(x,"annotation")
    label <- attr(x,"label")
    if("description" %nin% names(annotation) && length(label))
        annotation["description"] <- label
    attr(y,"annotation") <- annotation
    y <- as.item(y,
            labels=new("value.labels",levels(x),values=seq_along(levels(x))),
            measurement="nominal",
            ...)
    attr(y,"contrasts") <- attr(x,"contrasts")
    y
})
setMethod("as.item","ordered",function(x,...){
    y <- as.integer(x)
    annotation <- attr(x,"annotation")
    label <- attr(x,"label")
    if("description" %nin% names(annotation) && length(label))
        annotation["description"] <- label
    attr(y,"annotation") <- annotation
    y <- as.item(y,
                labels=new("value.labels",levels(x),values=seq_along(levels(x))),
                measurement="ordinal",
                ...)
    attr(y,"contrasts") <- attr(x,"contrasts")
    y
})

setMethod("as.item","character",function(x,
  labels=NULL,
  missing.values=NULL,
  valid.values=NULL,
  valid.range=NULL,
  value.filter=NULL,
  measurement=NULL,
  annotation=attr(x,"annotation"),
  ...
  ){
  value.labels <- if(length(labels)) as(labels,"value.labels") else NULL
  mv <- length(missing.values) > 0
  vv <- length(valid.values) > 0
  vr <- length(valid.range) > 0
  vf <- length(value.filter) > 0
  if(mv + vv + vr + vf > 1)
    stop("to many value filter arguments")
  value.filter <- if(mv) as(missing.values,"missing.values")
                  else if(vv) as(valid.values,"valid.values")
                  else if(vr) as(valid.range,"valid.range")
                  else if(vf) {
                        if(!is(value.filter,"value.filter"))
                          stop("'value.filter' not of appropriate class")
                        value.filter
                        }
  measurement <- if(length(measurement)) match.arg(measurement,c("nominal","ordinal"))
                 else "nominal"
  annotation <- new("annotation",structure(as.character(annotation),names=names(annotation)))
  label <- attr(x,"label")
  if("description" %nin% names(annotation) && length(label))
      annotation["description"] <- label
  new("character.item",x,
    value.labels=value.labels,
    value.filter=value.filter,
    measurement=measurement,
    annotation=annotation
  )
})

setMethod("[",signature(x="item.vector",i="numeric",j="missing",drop="missing"),function(x,i){
  structure(new(class(x),x@.Data[i],
    value.labels=labels(x),
    value.filter=value.filter(x),
    measurement=measurement(x),
    annotation=annotation(x)
  ),contrasts=attr(x,"contrasts"))
})

setMethod("[",signature(x="item.vector",i="logical",j="missing",drop="missing"),function(x,i){
  structure(new(class(x),x@.Data[i],
    value.labels=labels(x),
    value.filter=value.filter(x),
    measurement=measurement(x),
    annotation=annotation(x)
  ),contrasts=attr(x,"contrasts"))
})


str.item.vector <- function(object,give.head=TRUE,width=getOption("width"),...){
  if(give.head){
    hdr <- switch(measurement(object),
                  nominal=" Nmnl. item",
                  ordinal=" Ordl. item",
                  interval=" Itvl. item",
                  ratio=" Rto. item"
                  )
    vl <- labels(object)
    if(length(vl)){
      vlab <- sQuote(vl@.Data)
      vlval <- vl@values
      vlab <- ifelse(nchar(vlab)>10,paste(substr(vlab,start=1,stop=7),"..'",sep=""),
                    vlab)
      lbl.header <- paste("w/",length(vl@.Data),"labels for")
      hdr <- paste(hdr,lbl.header)
      lbl.list <- paste(vlval)
      lbl.h.w <- nchar(hdr,"w") +1 + cumsum(nchar(lbl.list,"w")+1)
      use <- lbl.h.w < width*.6
      if(any(use)) {
        use <- min(3,max(which(use)))
        if(use < length(lbl.list)){
          lbl.list <- c(lbl.list[1:use],"...")
        }
        lbl.list <- paste(lbl.list,collapse=",")
        hdr <- paste(hdr,lbl.list)
      }
    }
    vf <- object@value.filter
    if(length(vf)){
      filt.hdr <- switch(class(vf),
          missing.values="ms.v.",
          valid.values="vd.v.",
          valid.range="vd.r."
          )
      hdr <- paste(hdr,filt.hdr,sep=" + ")
    }
    cat(hdr,"")
  }
  str(object@.Data,give.head=give.head,width=width,...)
}

str.double.item <- str.item.vector
str.integer.item <- str.item.vector
str.character.item <- str.item.vector

## unique ###############################################################################

unique.item.vector <- function(x, incomparables = FALSE, ...){
  y <- new(class(x),unique(x@.Data))
  attributes(y) <- attributes(x)
  y
}

unique.double.item <- unique.item.vector
unique.integer.item <- unique.item.vector
unique.character.item <- unique.item.vector


## unique ###############################################################################

setMethod("unique","item.vector",function(x, incomparables = FALSE, ...)
  new(class(x),
    .Data=callNextMethod(),
    annotation=x@annotation,
    value.labels=x@value.labels,
    value.filter=x@value.filter,
    measurement=x@measurement
  )
)

## coercion #############################################################################


as.data.frame.character.item <-
as.data.frame.double.item <-
as.data.frame.integer.item <- function (x, row.names = NULL, optional = FALSE, ...)
{
    nrows <- length(x)
    nm <- paste(deparse(substitute(x), width.cutoff = 500), collapse = " ")
    if (is.null(row.names)) {
        if (nrows == 0L)
            row.names <- character(0L)
        else row.names <- .set_row_names(nrows)
    }

    x <- if(is.ordinal(x)) as.ordered(x)
    else if(is.nominal(x)) as.factor(x)
    else as.vector(x)
    
    value <- list(x)
    if (!optional)
        names(value) <- nm
    attr(value, "row.names") <- row.names
    class(value) <- "data.frame"
    value
}

setMethod("as.vector","item",function(x,mode = "any"){
    ism <- is.missing(x)
    d <- description(x)
    x <- callNextMethod()
    if(mode=="any") mode <- storage.mode(x)
    x[ism] <- as.vector(NA,mode=mode)
    if(length(d))
        attr(x,"label") <- d
    x
})

setMethod("as.numeric","item",function(x,...) as.vector(x,mode="double"))
setMethod("as.integer","item",function(x,...) as.vector(x,mode="integer"))

setMethod("as.ordered","item.vector",function(x){
    labels <- x@value.labels
    if(length(labels)){
      values <- labels@values
      labels <- labels@.Data
    }
    else {
      values <- labels <- sort(unique(x@.Data))
    }
    filter <- x@value.filter
    use.levels <- if(length(filter)) is.valid2(values,filter) else TRUE
    values <- values[use.levels]
    labels <- labels[use.levels]
    if(any(duplicated(values))) stop("Duplicate values")
    if(any(duplicated(labels))) stop("Duplicate labels")
    f <- ordered(x@.Data,levels=values,labels=labels)
    if(length(attr(x,"contrasts")))
      attr(f,"contrasts") <- contrasts(x)
    d <- description(x)
    if(length(d))
        attr(f,"label") <- d
    f
})

setMethod("as.factor","item.vector",function(x){
    labels <- x@value.labels
    if(length(labels)){
      values <- labels@values
      labels <- labels@.Data
    }
    else {
      values <- labels <- sort(unique(x@.Data))
    }
    filter <- x@value.filter
    use.levels <- if(length(filter)) is.valid2(values,filter) else TRUE
    values <- values[use.levels]
    labels <- labels[use.levels]
    if(any(duplicated(values))) stop("Duplicate values")
    if(any(duplicated(labels))) stop("Duplicate labels")
    f <- factor(x@.Data,levels=values,labels=labels)
    if(length(attr(x,"contrasts")))
      contrasts(f) <- contrasts(x)
    d <- description(x)
    if(length(d))
        attr(f,"label") <- d
    f
})

setMethod("as.character","item.vector",function(x,use.labels=TRUE,include.missings=FALSE,...){
    d <- description(x)
    ism <- is.missing(x) & !isTRUE(include.missings)
    if(use.labels && length(vl <- labels(x))){
        i <- match(x@.Data,vl@values)
        y <- vl@.Data[i]
        y[is.na(y)] <- as.character(x@.Data[is.na(y)])
        if(length(d))
            attr(y,"label") <- d
    }
    else {
        y <- as.character(x@.Data)
        if(length(d))
            attr(y,"label") <- d
    }
    y[ism] <- as.vector(NA,mode="character")
    y
})


## utilities #############################################################################

relabel_item <- function(x,...,gsub=FALSE,fixed=TRUE,warn=TRUE){
  subst <- c(...)
  vl <- x@value.labels
  vll <- vl@.Data
  if(gsub){
    for(i in 1:length(subst)){
      vll <- gsub(names(subst[i]),subst[i],vll,fixed=fixed)
    }
  }
  else {
      i <- match(names(subst),vll)
      if(any(is.na(i))) {
          if(warn) warning("undefined label(s) selected")
          if(any(!is.na(i)))
              subst <- subst[!is.na(i)]
          i <- i[!is.na(i)]
      }
      vll[i] <- subst
  }
  vl@.Data <- vll
  if(validObject(vl))
      x@value.labels <- vl
  return(x)
}

setMethod("relabel4","item",function(x,...)relabel_item(x,...))


setMethod("summary","item.vector",function(object,...,maxsum=100,digits=3)
  switch(measurement(object),
    nominal = smry.nominal.vector(x=object,...,maxsum=maxsum),
    ordinal = smry.ordinal.vector(x=object,...,maxsum=maxsum),
    interval = smry.interval.vector(x=object,...,maxsum=maxsum,digits=digits),
    ratio = smry.ratio.vector(x=object,...,maxsum=maxsum,digits=digits)
    )
)

smry.nominal.vector <- smry.ordinal.vector <- function(x,...,maxsum=100,digits=3){
  ism <- is.missing(x)
  isna <- is.na(x)
  xvalid <- x@.Data[!ism]
  uvalid <- sort(unique(xvalid))
  tabvalid <- if(has.value.labels(x)) {
              vl <- x@value.labels
              vv <- match(uvalid,vl@values)
              nms <- vl@.Data[vv]
              nms[is.na(nms)] <- ""
              structure(tabulate(match(xvalid,uvalid)),names=nms)
              }
              else structure(tabulate(match(xvalid,uvalid)),names=uvalid)
  tabmis <- if(any(ism)) {
              xmis <- x@.Data[ism & !isna]
              umis <- sort(unique(xmis))
              if(has.value.labels(x)) {
                vl <- x@value.labels
                vv <- match(umis,vl@values)
                nms <- vl@.Data[vv]
                nms[is.na(nms)] <- ""
                structure(tabulate(match(xmis,umis)),names=paste("*",nms,sep=""))
              }
              else structure(tabulate(match(xmis,umis)),names=paste("*",umis,sep=""))
            } else integer(0)
  tabna <- if(any(isna)){
              c("NAs"=sum(isna)) #'
            } else integer(0)
  if(length(tabvalid)+length(tabmis)+length(tabna)>maxsum){
    if(length(tabmis)) tabmis <- c("missing values"=sum(tabmis))
    maxsum <- maxsum - length(tabmis) - length(tabna)
    if(any(seq_along(tabvalid) > maxsum)){
      maxsum <- maxsum - 1
      over <- seq_along(tabvalid) > maxsum
      tabvalid <- c(tabvalid[!over],"(Other)"=sum(tabvalid[over]))
    }
  }
  as.table(c(tabvalid,tabmis,tabna))
}

smry.interval.vector <- smry.ratio.vector <- function(x,...,maxsum=100,digits=3){
  ism <- is.missing(x)
  isna <- is.na(x)
  xvalid <- x@.Data[!ism]
  tabvalid <- summary.default(xvalid)
  tabmis <- if(any(ism)) {
              xmis <- x@.Data[ism & !isna]
              c("Missings"=length(xmis))
            } else integer(0)
  tabna <- if(any(isna)){
              c("NAs"=sum(isna))
            } else integer(0)
#   if(length(tabvalid)+length(tabmis)+length(tabna)>maxsum){
#     if(length(tabmis)) tabmis <- c("missing values"=sum(tabmis))
#     maxsum <- maxsum - length(tabmis) - length(tabna)
#     if(any(seq_along(tabvalid) > maxsum)){
#       over <- seq_along(tabvalid) > maxsum
#       tabvalid <- tabvalid[!over]
#     }
#   }
  as.table(c(tabvalid,tabmis,tabna))
}

format.item.vector <- function(x,use.labels=getOption("print.use.value.labels"),justify="right",...){
  ism <- is.missing(x) & !is.na(x)
  if(use.labels && has.value.labels(x))
    x <- as.character(x,use.labels=use.labels,include.missings=TRUE)
  x <- format.default(x,trim=TRUE,justify="none",...)
  x[ism] <- paste("*",x[ism],sep="")
  format(x,justify=justify,...)
}
#setMethod("format","item.vector",format.item.vector)

print.item.vector <- function(x,
    use.labels=isTRUE(getOption("print.use.value.labels")),
    width=getOption("width"),
    compress=FALSE,...,print.gap=NULL){
    if(length(x)){
      mkdots <- function(n) paste(rep(".",n),collapse="")
      pg <- if(is.null(print.gap) || compress) 1 else print.gap
      l <- length(x)
      if(compress) {
        x <- x[seq_len(min(width,l))]
        x <- format(x,use.labels=use.labels,...)
        x <- trimws(x)
        xw <- cumsum(nchar(x,"w")+1)
        hdr <- paste(" [","1:",l,"]",sep="")
        width <- width - nchar(hdr)
        if(any(xw > width)){
          dots <- mkdots(3)
          width <- width - nchar(dots,"w") - 1
          upto <- max(which(xw <= width))
          x <- trimws(format(c(x[seq_len(upto)],dots)))
        }
        cat(hdr,x)
        cat("\n")
      }
      else
        print.default(
              format(x,use.labels=use.labels,...),
              quote=FALSE,print.gap=print.gap)
    }
    else
      print(as.vector(x),print.gap=print.gap)
}

setMethod("show","item.vector",function(object){
  cat("\nItem",
    if(length(description(object))) sQuote(description(object)) else NULL,
    paste("(",
    "measurement: ",measurement(object),", ",
    "type: ",storage.mode(object),", ",
    "length = ",length(object),")",sep=""),
    "\n\n")
  print.item.vector(object,width=getOption("width"),compress=TRUE)
})
setMethod("print","item.vector",print.item.vector)

setMethod("Compare",signature(e1="numeric.item",e2="character"),
  function(e1,e2){
    if(length(e1@value.labels))
      switch(.Generic,
        "=="=,
        "!="=,
        ">"=,
        "<"=,
        ">="=,
        "<="=Compare_lvc1(.Generic,e1,e2),
        stop("Comparison not supported")
        )
     else callNextMethod()
})
setMethod("Compare",signature(e1="character",e2="numeric.item"),
  function(e1,e2){
    if(length(e2@value.labels))
      switch(.Generic,
        "=="=,
        "!="=Compare_lvc1(.Generic,e2,e1),
        ">"=Compare_lvc1("<",e2,e1),
        "<"=Compare_lvc1(">",e2,e1),
        ">="=Compare_lvc1("<=",e2,e1),
        "<="=Compare_lvc1(">=",e2,e1),
        callNextMethod()
        )
     else callNextMethod()
})
setMethod("Arith",signature(e1="numeric",e2="numeric.item"),
  function(e1,e2){
    e1 <- as.vector(e1)
    e2 <- as.vector(e2)
    callNextMethod()
})
setMethod("Arith",signature(e1="numeric.item",e2="numeric"),
  function(e1,e2){
    e1 <- as.vector(e1)
    e2 <- as.vector(e2)
    callNextMethod()
})
setMethod("Math","numeric.item",function(x){
  x <- as.vector(x)
  callNextMethod()
})
setMethod("Math2","numeric.item",function(x,digits){
  x <- as.vector(x)
  callNextMethod()
})
setMethod("Summary","numeric.item",function(x,...,na.rm=FALSE){
  x <- x[!is.missing(x)]
  callNextMethod()
})


Compare_lvc <- function(.Generic,e1,e2){
  if(is(e1,"character")) Compare_lvc1(.Generic,e2,e1)
  else Compare_lvc1(.Generic,e1,e2)
}

Compare_lvc1 <- function(.Generic,e1,e2){
  vl <- labels(e1)
  e1 <- vl@.Data[match(e1,vl@values)]
  ans <- callGeneric(e1,e2)
}

setMethod("%in%",signature(x="numeric.item",table="character"),function(x,table){
   vl <- labels(x)
   x <- vl@.Data[match(x,vl@values)]
   x %in% table
})

## Methods for the auxiliary function for 'sort' and 'order'

xtfrm.integer.item <- function(x) x@.Data
xtfrm.numeric.item <- function(x) x@.Data
xtfrm.double.item <- function(x) x@.Data
xtfrm.character.item <- function(x) as.integer(as.factor(x@.Data))

## Methods for the auxiliary function for 'match'

mtfrm.integer.item <- function(x) x@.Data
mtfrm.numeric.item <- function(x) x@.Data
mtfrm.double.item <- function(x) x@.Data
mtfrm.character.item <- function(x) as.integer(as.factor(x@.Data))

## rep

setMethod("rep","item.vector",function(x,...){

  x@.Data <- rep(x@.Data,...)
  x
})

hdxp0 <- function(x,span) {
    y <- matrix(rep("",span),nrow=1,ncol=span)
    y[,1] <- x
    y
}
hdxp1 <- function(x) {
    span <- attr(x,"span")
    y <- matrix(rep("",span),nrow=1,ncol=span)
    y[,1] <- x
    y
}
hdxp <- function(x)do.call(cbind,lapply(x,hdxp1))

ldxp1 <- function(x) {
    span <- attr(x,"span")
    y <- matrix(rep("",span),nrow=span,ncol=1)
    y[1,] <- x
    y
}
ldxp <- function(x)do.call(rbind,lapply(x,ldxp1))


mtable_format_delim <- function(x,
                                colsep="\t",
                                rowsep="\n",
                                interaction.sep = " x ",
                                ...)
    pf_mtable_format_delim(preformat_mtable(x),
                                colsep=colsep,
                                rowsep=rowsep,
                                interaction.sep = interaction.sep,
                                ...)

pf_mtable_format_delim <- function(x,
                                colsep="\t",
                                rowsep="\n",
                                interaction.sep = " x ",
                                show.parmtypes = nrow(x$parmtab) > 1,
                                ...
                                ){

    pt <- x$parmtab
    sst <- x$summary.stats
    sh <- x$sect.headers
    leaders <- x$leaders
    headers <- x$headers
    eq.headers <- x$eq.headers

    res <- NULL

    l.headers <- length(headers)
    l.leaders <- length(leaders)

    has.eq.headers <- length(eq.headers) > 0

    for(j in 1:ncol(pt)){
        
        name.j <- colnames(pt)[j]
        pt.j <- pt[,j]
        l.pt.j <- length(pt.j)

        ncol.j <- unique(sapply(pt.j,ncol))
        stopifnot(length(ncol.j)==1)

        for(i in 1:l.pt.j){
            pt.ij <- pt.j[[i]]
            if(show.parmtypes){
                pt.ij <- rbind(" ",pt.ij)
            }
            pt.j[[i]] <- pt.ij
        }

        pt.j <- do.call(rbind,pt.j)

        if(has.eq.headers){
            eq.header.j <- eq.headers[[name.j]]
            n.eq.j <- length(eq.header.j)
            eq.span <- ncol(pt.j)/n.eq.j
            tmp <- matrix("",ncol=n.eq.j,nrow=eq.span)
            tmp[1,] <- eq.header.j
            eq.header.j <- as.vector(tmp)
            pt.j <- rbind(eq.header.j,pt.j)
        }

        if(length(sst)){
            sst.j <- sst[[j]]
            sst.j <- colexpand(sst.j,ncol.j)
            pt.j <- rbind(pt.j,sst.j)
        }
        
        res <- cbind(res,pt.j)
    }

    if(l.headers){
        for(k in 1:l.headers){
            headers.k <- headers[[k]]
            headers.k <- lapply(headers.k,hdxp1)
            headers.k <- do.call(cbind,headers.k)
            headers[[k]] <- headers.k
        }
        headers <- do.call(rbind,headers)
        res <- rbind(headers,res)
    }
    

    if(l.leaders){
        lh <- l.headers + has.eq.headers      
        if(lh)
            leaders <- c(rep(list(list(structure("",span=1))),lh),
                         leaders)
        leaders <- lapply(leaders,ldxp)
        if(show.parmtypes){
            parmtypes <- rownames(x$parmtab)
            for(p in parmtypes){
                leaders[[p]] <- rbind(p,leaders[[p]])
            }
        }
        leaders <- do.call(rbind,leaders)
        
        res <- cbind(leaders,res)
    }

    res <- apply(res,1,paste,collapse=colsep)
    res <- paste0(res,rowsep)
    return(res)
}

cases <- function(...,check.xor=c("warn","stop","ignore"),
                  .default=NA,.complete=FALSE,
                  check.na=c("warn","stop","ignore"),
                  na.rm=TRUE){
  subst <- match.call(expand.dots=FALSE)$...
  
  if(!missing(check.xor))  
    if(is.logical(check.xor))
      check.xor <- ifelse(check.xor,"stop","ignore")  
    else 
      check.xor <- as.character(check.xor)
  if(!missing(check.na))  
    if(is.logical(check.na))
      check.na <- ifelse(check.na,"stop","ignore")  
    else 
      check.na <- as.character(check.na)
  
  check.xor <- match.arg(check.xor)
  check.na <- match.arg(check.na)
  
  deflabels <- sapply(subst,deparse)
  if(length(subst)<2) stop("need at least two conditions")

  have.arrows <- sapply(subst,length) > 1
  have.arrows[have.arrows] <- have.arrows[have.arrows] & sapply(sapply(subst[have.arrows],"[[",1),paste)=="<-"

  parent <- parent.frame()
  
  if(all(have.arrows)){
    cond.names <- names(subst)

    conditions <- lapply(subst,"[[",3)
    values <- lapply(subst,"[[",2)
    conditions <- do.call(cbind,lapply(conditions,eval,envir=parent))
    
    if(ncol(conditions)!=length(subst)) stop("at least one condition results in NULL")
    if(!is.logical(conditions)) stop("all conditions have to be logical")
    if(any(is.na(conditions)) && check.na!="ignore") {
      msg <- switch(check.na,warn=warning,stop=stop)
      msg("At least one logical condition results in missing values")
    }
    na.cond <- is.na(conditions)
    na.count <- rowSums(na.cond)
    any.na <- na.count > 0
    all.na <- na.count == ncol(conditions)

    done <- rowSums(conditions, na.rm=TRUE)
    if((any(done > 1) || any(done >= 1 & any.na)) && check.xor!="ignore") {
      msg <- switch(check.xor,warn=warning,stop=stop)
      msg("Conditions are not mutually exclusive")
    }
    never <- colSums(conditions, na.rm=TRUE) == 0
    if(any(never) && check.xor!="ignore"){
      neverlab <- deflabels[never]
      if(length(neverlab)==1)
        warning("condition ",neverlab," is never satisfied")
      else
        warning("conditions ",paste(neverlab,collapse=", ")," are never satisfied")
    }

    values <- lapply(values,eval,envir=parent.frame(),enclos=parent.frame())
    nrow <- unique(sapply(values,length))
    if(length(nrow) > 1 || nrow != nrow(conditions)){
      nrow <- nrow(conditions)
      values <- lapply(values,function(x){
        tmp <- x
        length(tmp) <- nrow
        tmp[] <- x
        tmp
        })
    }
    values <- do.call(cbind,values)
    na_ <- as.vector(NA,mode=storage.mode(values))
    res <- vector(nrow(conditions),mode=storage.mode(values))
    res[] <- na_
    for(i in rev(1:ncol(conditions))){
      cond.i <- which(conditions[,i])
      res[cond.i] <- values[cond.i,i]
    }
    if(any(done == 0) && !is.na(.default)){
        if(length(.default) > 1) warning("only first element of '.default' used")
        if(length(.default) < 1) stop("'.default' must have non-zero length")
      xvalue <- .default[1]
      xvalue <- as.vector(xvalue,mode=storage.mode(values))
      res[done == 0 &!any.na] <- xvalue
      done <- done + 1
    }
    if(!na.rm){
       res[any.na] <- na_
    }
    nNA <- sum(is.na(res))
    if(nNA > 0) warning(nNA," NAs created")
    if(length(cond.names) && all(nzchar(cond.names))){
        uq.values <- drop(unique(values))
        if(length(uq.values)==length(cond.names))
          labels(res) <- structure(unique(uq.values),names=cond.names)
      }
    res
  }
  else if(!any(have.arrows))
  {
    conditions <- cbind(...)
    if(ncol(conditions)!=length(subst)) stop("at least one condition results in NULL")
    if(!is.logical(conditions)) stop("all conditions have to be logical")
    if(any(is.na(conditions)) && check.na!="ignore") {
      msg <- switch(check.na,warn=warning,stop=stop)
      msg("At least one logical condition results in missing values")
    }
    na.cond <- is.na(conditions)
    na.count <- rowSums(na.cond)
    any.na <- na.count > 0
    all.na <- na.count == ncol(conditions)

    codes <- 1:ncol(conditions)
    labels <- colnames(conditions)
    if(length(labels))
      labels <- ifelse(nzchar(labels),labels,deflabels)
    else labels <- deflabels

    done <- rowSums(conditions,na.rm=TRUE)
    if((any(done > 1) || any(done >= 1 & any.na)) && check.xor!="ignore") {
      msg <- switch(check.xor,warn=warning,stop=stop)
      msg("conditions are not mutually exclusive")
    }

    never <- colSums(conditions, na.rm=TRUE) == 0
    if(any(never) && check.xor!="ignore"){
      neverlab <- deflabels[never]
      if(length(neverlab)==1)
        warning("condition ",neverlab," is never satisfied")
      else
        warning("conditions ",paste(neverlab,collapse=", ")," are never satisfied")
    }

    res <- integer(nrow(conditions))
    res[] <- NA_integer_

    for(i in rev(1:ncol(conditions))){
      cond.i <- which(conditions[,i])
      res[cond.i] <- i
    }
    if(any(done == 0 & !all.na) && .complete){
      nms <- names(subst)
      if(length(nms)){
        elevel <- paste(nms,collapse="|")
      }
      else {
        elevel <- sapply(subst,deparse)
        elevel <- paste(elevel,collapse=" | ")
      }
      elevel <- paste0("!(",elevel,")")
      xcode <- length(codes) + 1
      res[done == 0 &!any.na] <- xcode
      codes <- c(codes,xcode)
      labels <- c(labels,elevel)
      done <- done + 1
    }
    if(!na.rm)
        res[any.na] <- NA_integer_
    nNA <- sum(is.na(res))
    if(nNA > 0) warning(nNA," NAs created")
    factor(res,levels=codes,labels=labels)
  }
  else stop("inconsistent arguments to 'cases'")
}

mat_format_stdstyle <- c(
  "padding-top"="3px",
  "padding-bottom"="3px",
  "padding-left"="0.5ex",
  "padding-right"="0.5ex",
  "margin-top"="0px",
  "margin-bottom"="0px",
  "border-style"="none",
  "border-width"="0px"
)

style_mat_global <- function(id=NULL,class=NULL,style=mat_format_stdstyle,margin="2ex auto") {

    table_tmpl <- "
      table<<class-or-id>> {
         border-collapse: collapse; border-style: none; margin: <<margin>>;
      }"
    table_tr_tmpl <- "
      table<<class-or-id>> tr {
          border-style: none;
      }"
    table_td_tmpl <- "
      table<<class-or-id>> td {
        padding-top: <<padding-top>>; 
        padding-bottom: <<padding-bottom>>; 
        padding-left: <<padding-left>>; 
        padding-right: <<padding-right>>; 
        margin: 0px; 
        margin-top: <<margin-top>>;
        margin-bottom: <<margin-bottom>>;
        border-style: <<border-style>>; 
        border-width: <<border-width>>; 
      }"

    class_or_id <- class_id_selector(id=id,class=class)

    table_style <- fillin(table_tmpl,
                          c("class-or-id"=class_or_id,
                            margin=margin))
    table_tr_style <- fillin(table_tr_tmpl,
                             c("class-or-id"=class_or_id))
    table_td_style <- fillin(table_td_tmpl,
                             c("class-or-id"=class_or_id,
                               style))
    paste(table_style,table_tr_style,table_td_style)
}

style_mat_rule <- function(id=NULL,class=NULL,top=FALSE,bottom=FALSE,rulewidth=1,rows=NULL){
    if(length(rows)){
        class_or_id <- class_id_selector(id=id,class=class)
        rulewidth <- paste0(rulewidth,"px")
        selector_tmpl <- "table<<class-or-id>> tr:nth-child(<<row>>)"
        selector <- character(0)
        for(row in rows){
            selector <- c(selector,
                          fillin(selector_tmpl,c("class-or-id"=class_or_id,row=row)))
        }
        selector <- paste(selector,collapse=",\n")
        style_tmpl <- "<<selector>>{"
        if(top){
            style_tmpl <- paste(style_tmpl,
                                "     border-top: <<rulewidth>> solid;",
                                sep="\n")
        }
        if(bottom){
            style_tmpl <- paste(style_tmpl,
                                "     border-bottom: <<rulewidth>> solid;",
                                sep="\n")
        }
        style_tmpl <- paste(style_tmpl,"}",sep="\n")

        fillin(style_tmpl,c(selector=selector,rulewidth=rulewidth))
    } else ""
}

style_mat_cols <- function(id=NULL,class=NULL,cols=NULL,style=""){
    if(length(cols)){
        class_or_id <- class_id_selector(id=id,class=class)
        selector_tmpl <- "table<<class-or-id>> td:nth-child(<<col>>)"
        selector <- character(0)
        for(col in cols){
            selector <- c(selector,
                          fillin(selector_tmpl,c("class-or-id"=class_or_id,col=col)))
        }
        selector <- paste(selector,collapse=",\n")
        paste0(selector,"{",style,"}")
    }
}

style_mat_header <- function(id=NULL,class=NULL,style=""){
    class_or_id <- class_id_selector(id=id,class=class)
    selector_tmpl <- "table<<class-or-id>> td:nth-child(n of .header)"
    selector <- fillin(selector_tmpl,c("class-or-id"=class_or_id,col=col))
    paste0("\n",selector,"{",style,"}")
}

mat_html_env <- new.env()
mat_html_env$counter <- 1

format_html.matrix <- function(x,
                               toprule=2,midrule=1,bottomrule=2,
                               split.dec=TRUE,
                               formatC=FALSE,
                               digits=getOption("digits"),
                               format="f",
                               style=mat_format_stdstyle,
                               margin="2ex auto",
                               ...){

    
    colsep <- ""
    rowsep <- "\n"
    
    n <- nrow(x)
    m <- ncol(x)
    dim.x <- dim(x)

    dot_cols <- integer(0)
    col_sum <- 0

    colspan <- integer(m)
  
    if(is.integer(x)){
        tmp <- formatC(x,format="d")
        body <- html_td(tmp,vectorize=TRUE)
        colspan <- 1L
    }
    else if(is.numeric(x)){
        if(formatC)
            tmp <- formatC(x,digits=digits,format=format)
        else
            tmp <- format(x)
        if(split.dec){
            ncol.tmp <- ncol(tmp)
            tmp <- spltDec(tmp)
            body <- html_td_spltDec(tmp)
            colspan <- 3L
            dot_cols <- (1:ncol.tmp-1)*3 + 2
        }
        else{
            body <- html_td(tmp,vectorize=TRUE)
            colspan <- 1L
        }
    }
    else {
        tmp <- as.character(x)
        body <- html_td(tmp,vectorize=TRUE)
        colspan <- 1L
    }
    dim(body) <- dim.x
    
    if(length(rownames(x))){
        tmp <- rownames(x)
        ldr <- html_td(tmp,vectorize=TRUE)
        body <- cbind(ldr,body)
        if(length(dot_cols)>0)
            dot_cols <- dot_cols + 1
    }

    body <- apply(body,1,html_tr)

    if(length(colnames(x))){
        
        hdr <- colnames(x)
        if(length(rownames(x))){
            hdr <- c("",hdr)
            colspan <- c(1L,rep(colspan,m))
        }
        else
            colspan <- rep(colspan,m)
        hdr <- html_td(hdr,vectorize=TRUE,class="header")
        hdr[] <- mapply(setAttribs,hdr,colspan=colspan,SIMPLIFY=FALSE)
        hdr <- html_tr(hdr)
        
        res <- c(list(hdr),body)
    }
    else
        res <- body

    mat_id <- paste0("matrix-",mat_html_env$counter)
    res <- html_table(res,id=mat_id)

    mat_html_env$counter <- mat_html_env$counter + 1
    
    style_global <- style_mat_global(id=mat_id,style=style,margin=margin)
    style_toprule <- style_mat_rule(id=mat_id,rulewidth=toprule,top=TRUE,
                                       rows=1)
    if(length(colnames(x))){
        style_bottomrule <- style_mat_rule(id=mat_id,rulewidth=bottomrule,bottom=TRUE,
                                          rows=nrow(x)+1)
        style_midrule <- style_mat_rule(id=mat_id,rulewidth=midrule,bottom=TRUE,
                                          rows=1)
    }
    else{
        style_bottomrule <- style_mat_rule(id=mat_id,rulewidth=bottomrule,bottom=TRUE,
                                          rows=nrow(x))
        style_midrule <- NULL
    }


    style_content <- paste(
        style_global,
        style_toprule,
        style_midrule,
        style_bottomrule,
        sep="\n"
    )

    if(split.dec){
        dot_style <- "
         padding-left: 0px;
         padding-right: 0px;
         text-align: center; 
         width: .2ex;
         /*background-color: blue;*/
        "
        before_dot_style <- "
         padding-left: 0.5ex;
         padding-right: 0px;
         text-align: right; 
         /*background-color: red;*/
        "
        behind_dot_style <- "
         padding-left: 0px;
         padding-right: 0.5ex;
         text-align: left; 
         /*background-color: green;*/
        "
        style_dots <- style_mat_cols(id=mat_id,cols=dot_cols+1,style=dot_style)
        style_before_dots <- style_mat_cols(id=mat_id,cols=dot_cols,style=before_dot_style)
        style_behind_dots <- style_mat_cols(id=mat_id,cols=dot_cols+2,style=behind_dot_style)
        style_content <- paste(style_content,
                               style_dots,
                               style_before_dots,
                               style_behind_dots,sep="\n")
    }
    if(length(colnames(x))){
        header_style <- "
         padding-left: 0.5ex;
         padding-right: 0.5ex;
         text-align: center; 
         /*background-color: gray;*/
        "
        style_header <- style_mat_header(id=mat_id,style=header_style)
        style_content <- paste(style_content,
                               style_header,
                               "\n")
    }
    if(length(rownames(x))){
        leader_style <- "
         padding-left: 0.5ex;
         padding-right: 0px;
         text-align: right; 
         /*background-color: red;*/
        "
        style_leader <- style_mat_cols(id=mat_id,cols=1,style=leader_style)
        style_content <- paste(style_content,
                               style_leader,
                               "\n")
    }

    style_element <- html("style",style_content,linebreak=TRUE)
    res <- html_group(style_element,res)
    res <- as.character(res)
    return(res)
}

spss.fixed.file <- function(
    file,
    columns.file,
    varlab.file=NULL,
    codes.file=NULL,
    missval.file=NULL,
    count.cases=TRUE,
    to.lower=getOption("spss.fixed.to.lower",FALSE),
    iconv=TRUE,
    encoded=getOption("spss.fixed.encoding","cp1252"),
    negative2missing = FALSE
    ){
    file <- force(file)
    columns.file <- force(columns.file)
    file <- path.expand(file)
    columns.file <- path.expand(columns.file)
    check.file(file,error=TRUE)
    fptr <- rofile(file)
    check.file(columns.file,error=TRUE)
    data.spec <- spss.parse.data.spec(columns.file)
    types <- data.spec$types
#     browser()
    varlabs <- if(length(varlab.file) && check.file(varlab.file,error=TRUE))
                   spss.parse.variable.labels(varlab.file,
                                              iconv=iconv,
                                              encoded=encoded)
               else NULL #vector(length(types),mode="list")
    vallabs <- if(length(codes.file) && check.file(codes.file,error=TRUE))
                   spss.parse.value.labels(codes.file,
                                           iconv=iconv,
                                           encoded=encoded)
               else NULL #vector(length(types),mode="list")
    missings <- if(length(missval.file) && check.file(missval.file,error=TRUE)) spss.parse.missing.values(missval.file)
               else NULL #vector(length(types),mode="list")
    variables <- vector(length(types),mode="list")

    var.names <- names(types)
    names(variables) <- var.names
    variables[types==1] <- list(new("double.item"))
    variables[types==2] <- list(new("character.item"))

    if(length(varlabs)){
      nn <- names(varlabs)
      if(!all(nn %in% var.names)) stop("undefined variables in 'varlab.file'")
      for(n in nn)
        description(variables[[n]]) <- varlabs[n]
    }
    if(length(vallabs)){
      nn <- names(vallabs)
      if(!all(nn %in% var.names)) stop("undefined variables in 'codes.file'")
      for(n in nn)
        suppressWarnings(labels(variables[[n]]) <- vallabs[[n]])
    } 
    if(length(missings)){
      nn <- names(missings)
      if(!all(nn %in% var.names)) stop("undefined variables in 'missval.file'")
      for(n in nn)
        missing.values(variables[[n]]) <- missings[[n]]
    } 
    if(negative2missing)
      variables <- lapply(variables,`valid.range<-`,c(0,Inf))

    nlines <- if(count.cases) {
        rofseek(fptr,pos=0)
        for(i in seq_len(data.spec$skip)) roreadline(fptr)
        nlines <- .Call("countlines",fptr)
     } else NA_integer_
     attr(fptr,"nlines") <- nlines

    if(to.lower){
      names(variables) <- tolower(names(variables))
    }

    if(iconv)
        variables <- lapply(variables,Iconv,from=encoded,to="UTF-8")
    else
        encoded = ""

    warn_if_duplicate_labels(variables)
    
    new("spss.fixed.importer",
      variables,
      ptr=fptr,
      columns.file=columns.file,
      varlab.file=varlab.file,
      codes.file=codes.file,
      missval.file=missval.file,
      data.spec=data.spec
      )
}
setMethod("initialize","spss.fixed.importer",function(.Object,
                                                          variables,
                                                          ptr,
                                                          columns.file=character(),
                                                          varlab.file=character(),
                                                          codes.file=character(),
                                                          missval.file=character(),
                                                          document=character(),
                                                          data.spec
                                                          ){
     .Object@.Data <- variables
     .Object@ptr <- ptr
     .Object@columns.file <- as.character(columns.file)
     .Object@varlab.file <- as.character(varlab.file)
     .Object@codes.file <- as.character(codes.file)
     .Object@missval.file <- as.character(missval.file)
     .Object@document <- as.character(document)
     .Object@data.spec <- data.spec
     .Object
})


setMethod("getNobs","spss.fixed.importer",function(x){
  nlines <- attr(x@ptr,"nlines")
  if(!length(nlines)) {
        rofseek(x@ptr,pos=0)
        for(i in seq_len(x@data.spec$skip)) roreadline(x@ptr)
        attr(x@ptr,"nlines") <- nlines <- .Call("countlines",x@ptr)
  }
  nlines
})

setMethod("seekData","spss.fixed.importer",function(x){
  rofseek(x@ptr,pos=0)
  for(i in seq_len(x@data.spec$skip))
         roreadline(x@ptr)
})

setMethod("readData","spss.fixed.importer",
  function(x,n)
    readfixed(x@ptr,
      what=x,
      nlines=n,
      start=x@data.spec$start,
      stop=x@data.spec$stop
))

setMethod("readSlice","spss.fixed.importer",
  function(x,rows,cols)
    readfixedslice(x@ptr,
    what=x,
    j=cols,i=rows,
    start=x@data.spec$start,
    stop=x@data.spec$stop
))

setMethod("readChunk","spss.fixed.importer",
  function(x,nrows,cols)
    readfixedchunk(x@ptr,
    what=x,
    j=cols,n=nrows,
    start=x@data.spec$start,
    stop=x@data.spec$stop
))

setMethod("show","spss.fixed.importer",
  function(object){
    file.name <- attr(object@ptr,"file.name")
    nobs <- nrow(object)
    nvar <- ncol(object)
    varlab.file <- object@varlab.file
    codes.file <- object@codes.file
    missval.file <- object@missval.file
    cat("\nSPSS fixed column file",sQuote(file.name),"\n\twith ")
    cat(nvar,"variables and ")
    cat(nobs,"observations\n")
    if(length(varlab.file)) cat("\twith variable labels from file",sQuote(varlab.file),"\n")
    if(length(codes.file)) cat("\twith value labels from file",sQuote(codes.file),"\n")
    if(length(missval.file)) cat("\twith missing value definitions from file",sQuote(missval.file),"\n")
})

subset.spss.fixed.importer <- subset.importer

setMethod("as.data.set","importer",function(x,row.names=NULL,optional=NULL,
                    compress.storage.modes=FALSE,...){
  seekData(x)
  res <- readData(x,n=nrow(x))
  if(compress.storage.modes)
    res <- lapply(res,function(x){
      x@.Data <- compress.storage.mode(x@.Data)
      cl.x <- paste(storage.mode(x@.Data),"item",sep=".")
      attr(cl.x,"package") <- attr(class(x),"package")
      class(x) <- cl.x
      x
      })
  names(res) <- names(x)
  class(res) <- "data.frame"
  if(length(row.names)){
    if(length(row.names)!=nrow(x)) stop("row.names argument has wrong length")
    attr(res,"row.names") <- row.names
  }
  else
    attr(res,"row.names") <- seq_len(nrow(x))
  new("data.set",res)
})


setMethod("dim","importer",function(x){
  c(getNobs(x),length(x@.Data))
})

# setMethod("names","importer",function(x)names(x@frame))


bracket.importer <- function(x,Nargs,i,j,drop=TRUE,compress.storage.modes=FALSE,...){
  mycall <- match.call()
  mspl <- missing(compress.storage.modes)
  mdrp <- missing(drop)
  #Nargs <- Nargs - (!mdrp) - (!mspl) - length(list(...))
  
  ncases <- nrow(x)
  names <- names(x)
  nvar <- length(names)

#   browser()
#  message("Nargs = ",Nargs)

  if(Nargs == 1){
    if(missing(i)) return(as.data.set(x))
    if(is.array(i)) stop("index arrays not supported")
    j <- i
    i <- rep(TRUE,ncases)
  }
  else{
    if(missing(i)){
      i <- rep(TRUE,ncases)
    }
    if(missing(j)){
      j <- rep(TRUE,nvar)
    }
  }
  
  if(is.logical(i)){
    rows <- i
    ui <- which(i)
    row.nums <- ui
  } else if(is.numeric(i)){
    i <- as.integer(i)
    row.nums <- i
    ui <- sort(unique(i))
    rows <- logical(max(ui))
    rows[ui] <- TRUE
  }
  else stop("mode ",sQuote(mode(i))," not supported for row selection")

  if(is.logical(j)){
    cols <- j
  } else if(is.numeric(j)){
    if(max(j)>nvar) stop("undefined columns selected")
    cols <- logical(nvar)
    cols[j] <- TRUE
  } else if(is.character(j)){
    if(!all(j %in% names)) stop("undefined columns selected")
    cols <- names %in% j
  } else stop("mode ",sQuote(mode(i))," not supported for column selection")
  
  seekData(x)
  if(length(rows) > nrow(x)) length(rows) <- nrow(x)
  res <- readSlice(x,rows,cols)
  if(compress.storage.modes)
    res <- lapply(res,function(x){
      x@.Data <- compress.storage.mode(x@.Data)
      cl.x <- paste(storage.mode(x@.Data),"item",sep=".")
      attr(cl.x,"package") <- attr(class(x),"package")
      class(x) <- cl.x
      x
      })
  names(res) <- names[cols]
  class(res) <- "data.frame"
  if(is.numeric(i)){
    ii <- match(i,ui)
    res <- res[ii,,drop=FALSE]
  }
  attr(res,"row.names") <- as.integer(row.nums)
  if(drop && length(res)==1) res[[1]]
         new("data.set",res)
}

setMethod("[",signature(x="importer",i="atomic",j="atomic",drop="ANY"),
  function(x,i,j,...,drop=TRUE)
    bracket.importer(x,Nargs=nargs()-1-(!missing(drop))-length(list(...)),i,j,drop=drop,...)
)
setMethod("[",signature(x="importer",i="atomic",j="missing",drop="ANY"),
  function(x,i,j,...,drop=TRUE)
    bracket.importer(x,Nargs=nargs()-1-(!missing(drop))-length(list(...)),i,j,drop=drop,...)
)
setMethod("[",signature(x="importer",i="missing",j="atomic",drop="ANY"),
  function(x,i,j,...,drop=TRUE)
    bracket.importer(x,Nargs=nargs()-1-(!missing(drop))-length(list(...)),i,j,drop=drop,...)
)
setMethod("[",signature(x="importer",i="missing",j="missing",drop="ANY"),
  function(x,i,j,...,drop=TRUE)
    bracket.importer(x,Nargs=nargs()-1-(!missing(drop))-length(list(...)),i,j,drop=drop,...)
)

c_Data <- function(x1,x2) {
  if(is.null(x1)) return(x2) 
  else {
    y <- x1
    y@.Data <- c(x1@.Data,x2@.Data)
    return(y)
  }
}

subset.importer <- function (x, subset, select, drop = FALSE,
              ...){

    if(missing(subset) && missing(select))
        return(as.data.set(x))
              
    nobs <- nrow(x)
    names <- names(x)
    nvars <- length(names)
    cs <- getOption("subset.chunk.size",nobs)
    if(cs < 1) stop("'subset.chunk.size' option must be a positive integer or NULL")
    
    if(missing(select)){
        cols <- rep(TRUE,nvars)
        select.vars <- names(x)
    }
    else {
        nl <- as.list(1:nvars)
        names(nl) <- names
        cols <- logical(nvars)
        if (is.call(substitute(select))) {
            cols[eval(substitute(select), nl, parent.frame())] <- TRUE
            select.vars <- sapply(substitute(select)[-1],as.character)
        } else { # As suggested by Diogo Ferrari (https://dioferrari.com/)
            select.vars <- select
            cols[which(names(nl) %in% select.vars )] <- TRUE
        }

    }
    if(nobs < cs) cs <- nobs
    m <- nobs %/% cs
    r <- nobs %% cs

    seekData(x)
    if(missing(subset)){
        res <- readChunk(x,nrows=nobs,cols=cols)
        names(res) <- names(x)[cols]
        res <- res[select.vars]
        attr(res,"row.names") <- 1:nobs
    }
    else {
        select.cols <- cols
        e <- substitute(subset)
        subset.vars <- all.vars(e)
        subset.cols <- names(x) %in% subset.vars
        cols <- select.cols | subset.cols
        chunk.names <- names(x)[cols]
        chunk.cols <- chunk.names %in% select.vars
        res <- x@.Data[select.cols]
        res.nobs <- 0
        row.nums <- integer(0)
        nobs.visited <- 0
        for(i in 1:m){
            chunk <- readChunk(x,nrows=cs,cols=cols)
            names(chunk) <- chunk.names
            use.obs <- eval(e,chunk,parent.frame())
            if(!is.logical(use.obs)) stop("non-logical subset arg")
            if(any(use.obs)){
                res.nobs <- res.nobs + sum(use.obs)
                chunk <- chunk[chunk.cols]
                chunk <- lapply(chunk,"[",use.obs)
                res <- mapply(c_Data,res,chunk,SIMPLIFY=FALSE)
                row.nums.i <- seq.int(cs)[use.obs]
                row.nums.i <- nobs.visited + row.nums.i
                row.nums <- c(row.nums,row.nums.i)
            }
            nobs.visited <- nobs.visited + cs
        }
        if(r > 0){
            chunk <- readChunk(x,nrows=r,cols=cols)
            names(chunk) <- chunk.names
            use.obs <- eval(e,chunk,parent.frame())
            if(!is.logical(use.obs)) stop("non-logical subset arg")
            if(any(use.obs)){
                res.nobs <- res.nobs + sum(use.obs)
                chunk <- chunk[chunk.cols]
                chunk <- lapply(chunk,"[",use.obs)
                res <- mapply(c_Data,res,chunk,SIMPLIFY=FALSE)
                row.nums.i <- seq.int(r)[use.obs]
                row.nums.i <- nobs.visited + row.nums.i
                row.nums <- c(row.nums,row.nums.i)
            }
            nobs.visited <- nobs.visited + r
        }
        stopifnot(nobs.visited == nobs)
        for(j in 1:length(res)){
            attributes(res[[j]]) <- attributes(chunk[[j]])
        }
        names(res) <- names(x)[select.cols]
        res <- res[select.vars]
        attr(res,"row.names") <- as.integer(row.nums)
    }
    new.names <- names(select.vars)
    if(length(new.names) && any(nzchar(new.names))){
        names.res <- names(res)
        names.res[nzchar(new.names)] <- new.names[nzchar(new.names)]
        names(res) <- names.res
    }
    class(res) <- "data.frame"
    new("data.set",res)
}


setMethod("$",signature(x="importer"),
function(x,name){
  x[name]              
})

setMethod("[[",signature(x="importer"),
function(x,i,...){
  x[i]         
})


setMethod("description","importer",function(x){
    res <- lapply(structure(x@.Data,
                            names=x@names),
                  description)
    structure(res,class="descriptions")
})

setMethod("codebook","importer",function(x, weights=NULL, unweighted=TRUE, ...){
  if(!missing(weights))
    weights <- deparse(substitute(weights))
  if(length(weights)){
      cat("Weights are from ",weights,"\n")
      # warning("Weights for codebooks of 'importer' objects are not yet supported.")
  }
  
  cs <- getOption("codebook.chunk.size")
  nobs <- nrow(x)
  if(nobs < cs) cs <- nobs
  m <- nobs %/% cs
  r <- nobs %% cs
  nvar <- ncol(x)
  res <- lapply(structure(x@.Data,
                          names=x@names),
                initcodebookEntry)
  seekData(x)
  w <- NULL
  w.idx <- match(weights,names(x))
  for(i in 1:m){
    chunk <- readData(x,n=cs)
    if(length(weights)){
      w <- as.numeric(chunk[[w.idx]])
      w[is.na(w)] <- 0
    }
    res <- mapply(updatecodebookEntry,res,chunk,
                  MoreArgs=list(weights=w))
  }
  if(r > 0){
    chunk <- readData(x,n=r)
    if(length(weights)){
      w <- as.numeric(chunk[[w.idx]])
      w[is.na(w)] <- 0
    }
    res <- mapply(updatecodebookEntry,res,chunk,
                  MoreArgs=list(weights=w))
  }
  res <- lapply(res,fixupcodebookEntry,unweighted=unweighted)
  new("codebook",res)
})

initcodebookEntry <- function(x,weighted){
  annotation <- annotation(x)
  filter <- x@value.filter
  spec <- c(
    "Storage mode:"=storage.mode(x),
    "Measurement:"=measurement(x)
  )
  if(length(filter)) spec <- c(spec,
                               switch(class(filter),
                                      missing.values = c("Missing values:" = format(filter)),
                                      valid.values   = c("Valid values:"   = format(filter)),
                                      valid.range    = c("Valid range:"    = format(filter))
                               ))
  stats <- switch(spec["Measurement:"],
                  nominal=,ordinal=initcodebookStatsCateg(x,weights),
                  interval=,ratio=initcodebookStatsMetric(x,weights),
                  `Date/time`=initcodebookStatsDatetime(x)
  )
  new("codebookEntry",
      spec = spec,
      stats = stats,
      annotation = annotation
  )
}


updatecodebookEntry <- function(cbe,x,weights=NULL){
  switch(cbe@spec["Measurement:"],
         nominal=,ordinal=updatecodebookStatsCateg(cbe,x,weights),
         interval=,ratio=updatecodebookStatsMetric(cbe,x,weights),
         `Date/time`=updatecodebookStatsDatetime(cbe,x)
  )
}


initcodebookStatsCateg <- function(x,weights=NULL) {
    vl <- labels(x)
    ic <- inherits(x,"character")
    if(length(vl) || !ic){
        if(length(weights))
            list(tab=list(),wtab=list())
        else
            list(tab=list())
    } else {
        list()
    }
    
} 

updatecodebookStatsCateg <- function(cbe,x,weights=NULL){
    vl <- labels(x)
    ic <- inherits(x,"character")
    stats <- cbe@stats

    if(length(vl) || !ic){
        tab <- codebookTable_item(x,drop.empty=FALSE)
        tab.title <- attr(tab,"title")
        if(length(stats$tab)){
          stats$tab <- stats$tab + tab
        }
        else 
            stats$tab <- tab
        attr(stats$tab,"title") <- tab.title
        if(length(weights)){
            wtab <- codebookTable_item(x,weights=weights,drop.empty=FALSE)
            if(length(stats$wtab))
                stats$wtab <- stats$wtab + wtab
            else 
                stats$wtab <- wtab
        }
    } else {
        stats1 <- cbChar1(x)
        stats <- update_cbChar1(stats,stats1)
    }
    
    cbe@stats <- stats
    cbe
}

cbChar1 <- function(x){
    isna <- is.na(x)
    NAs <- sum(isna)
    descr <- structure(range(x),names=c("Min","Max"))
    list(descr=descr,
         NAs = NAs)
}

update_cbChar1 <- function(current,update){
    if(!length(current)) return(update)
    else {
        descr <- range(c(current$descr,update$descr))
        names(descr) <- c("Min","Max")
        list(
            descr = as.matrix(descr),
            NAs = current$NAs + update$NAs
        )
    }
}


initcodebookStatsMetric <- function(x,weights=NULL){
    if(length(labels(x))){
        if(length(weights))
            stats <- list(tab=list(),wtab=list())
        else
            stats <- list(tab=list())
    }
    else stats <- list()
    stats$descr <- numeric(0)
    stats$range <- numeric(0)
    stats$missings <- integer(0)
    stats
}

cb_descr <- function(x,w=NULL){
  m <- c(n     = length(x),
         sum.x = sum(x),
         sum.x2 = sum(x^2))
  if(length(w)){
    wm <- c(sum(w),
            sum(w*x),
            sum(w*x^2))
    m <- cbind(Unweighted=m,Weighted=wm)
  }
  else m <- as.matrix(m)
  return(m)
}

updatecodebookStatsMetric <- function(cbe,x,weights=NULL){
  stats <- cbe@stats
  if(length(x@value.labels)){
    tab <- codebookTable_item(x,drop.empty=FALSE)
    tab.title <- attr(tab,"title")
    if(length(stats$tab)){
      stats$tab <- stats$tab + tab
    }
    else 
      stats$tab <- tab
    attr(stats$tab,"title") <- tab.title
    if(length(weights)){
      wtab <- codebookTable_item(x,weights=weights,drop.empty=FALSE)
      if(length(stats$wtab))
        stats$wtab <- stats$wtab + wtab
      else 
        stats$wtab <- wtab
    }
  }
  ismiss <- is.missing(x)
  isna <- is.na(x@.Data)
  x <- x@.Data[!ismiss & !isna]
  NAs <- sum(isna)
  miss <- sum(ismiss,na.rm=TRUE)
  if(length(x)){
    if(length(weights))
      w <- weights[!ismiss & !isna]
    else
      w <- NULL
    descr <- cb_descr(x,w=w)
    stats$descr <- if(length(stats$descr)) stats$descr + descr
                     else descr
    
    stats$range <- if(length(stats$range)) range(stats$range,range(x))
                else range(x)
  }
  stats$missings <- if(length(stats$missings)) stats$missings + c(miss,NAs)
                    else c(miss,NAs)
  cbe@stats <- stats
  cbe
}

initcodebookStatsDatetime <- function(x){
    stats <- list()
    stats$moments <- numeric(0)
    stats$range <- numeric(0)
    stats$missings <- integer(0)
    stats
}


updatecodebookStatsDatetime <- function(cbe,x,weights=NULL){
  stats <- cbe@stats
  miss <- is.missing(x)
  NAs <- is.na(x@.Data)
  x <- x@.Data[!miss & !NAs]
  NAs <- sum(NAs)
  miss <- sum(miss,na.rm=TRUE)
  if(length(x)){
    stats$range <- if(length(stats$range)) range(stats$range,range(x))
                else range(x)
  }
  stats$missings <- if(length(stats$missings)) stats$missings + c(miss,NAs)
                    else c(miss,NAs)
  cbe@stats <- stats
  cbe
}


fixupcodebookEntry <- function(cbe,unweighted=TRUE){
    switch(cbe@spec["Measurement:"],
    nominal=,ordinal=fixupcodebookEntryCateg(cbe,unweighted=unweighted),
    interval=,ratio=fixupcodebookEntryMetric(cbe,unweighted=unweighted),
    `Date/time`=fixupcodebookEntryDatetime(cbe)
  )
}

prc <- function(x){
  s <- sum(x,na.rm=TRUE)
  100*x/s
}

fixupcodebookEntryCateg <- function(cbe,unweighted=TRUE){
    tab <- cbe@stats$tab
    wtab <- cbe@stats$wtab
    if(length(tab)){
        tab.title <- attr(tab,"title")
        if(length(wtab)){
            if(unweighted)
                tab <- collect(Unweighted=tab,
                               Weighted=wtab)
            else
                tab <- array(wtab,
                             dim=c(dim(tab),1),
                             dimnames=c(dimnames(tab),
                                        list(NULL)))
        } else {
            tab <- array(tab,
                         dim=c(dim(tab),1),
                         dimnames=c(dimnames(tab),
                                    list(NULL)))
        }
        rn <- rownames(tab)
        ii <- (grepl("(unlab.val.)",rn,fixed=TRUE) | grepl("(unlab.mss.)",rn,fixed=TRUE) |
               grepl("NA M ",rn,fixed=TRUE))
        ii <- ii & tab[,1,1] == 0
        tab <- tab[!ii,,,drop=FALSE]
        d <- dim(tab)
        dn <- dimnames(tab)
        tab[,-1,] <- apply(tab[,-1,,drop=FALSE],2:3,prc)
        dim(tab) <- d
        dimnames(tab) <- dn
        attr(tab,"title") <- tab.title
        cbe@stats <- list(tab=tab)
    }
    cbe
}

fixupcodebookEntryMetric <- function(cbe,unweighted=TRUE){
    stats <- cbe@stats
    tab <- stats$tab
    wtab <- stats$wtab
    if(length(tab)){
        tab.title <- attr(tab,"title")
        if(length(wtab)){
            if(unweighted)
                tab <- collect(Unweighted=tab,
                               Weighted=wtab)
            else
                tab <- array(wtab,
                             dim=c(dim(tab),1),
                             dimnames=c(dimnames(tab),
                                        list(NULL)))
        } else {
            tab <- array(tab,
                         dim=c(dim(tab),1),
                         dimnames=c(dimnames(tab),
                                    list(NULL)))
        }
        rn <- rownames(tab)
        ii <- (grepl("(unlab.val.)",rn,fixed=TRUE) | grepl("(unlab.mss.)",rn,fixed=TRUE) |
               grepl("NA M ",rn,fixed=TRUE))
        ii <- ii & tab[,1,1] == 0
        tab <- tab[!ii,,,drop=FALSE]
        d <- dim(tab)
        dn <- dimnames(tab)
        tab[,-1,] <- apply(tab[,-1,,drop=FALSE],2:3,prc)
        dim(tab) <- d
        dimnames(tab) <- dn
        attr(tab,"title") <- tab.title
    }
    descr <- stats$descr
    if(length(descr)){
        # if(length(dim(descr))!=2) browser()
        cn.d <- colnames(descr)
        descr <- t(t(descr[-1,,drop=FALSE])/descr[1,]) # From sums to (weighted) means ...
        descr[2,] <- sqrt(descr[2,] - descr[1,]^2)
        
        rnge <- stats$range
        if(ncol(descr)==2){
            if(unweighted)
                descr <- rbind(cbind(rnge,rnge),descr)
            else{
                descr <- as.matrix(c(rnge,descr[,2]))
                cn.d <- cn.d[2]
            }
        }
        else
            descr <- as.matrix(c(rnge,descr))
        rownames(descr) <- c("Min","Max","Mean","Std.Dev.")
        colnames(descr) <- cn.d
    }
    cbe@stats <- list(tab=tab,descr=descr)
    cbe
}

fixupcodebookEntryDatetime <- function(cbe){

  miss <- cbe@stats$missings[1]
  NAs <- cbe@stats$missings[2]
  Min <- cbe@stats$range[1]
  Max <- cbe@stats$range[2]

  origin <- unname(cbe@spec["Origin:"])
    
  descr <- c(
            Min=format(as.POSIXct(Min,origin=origin)),
            Max=format(as.POSIXct(Max,origin=origin))
  )
  descr[] <- format(descr[])
  descr <- as.matrix(descr)

  cbe@stats <- list(descr=descr)
  cbe
}

setMethod("head",signature(x="importer"),
          function(x,n=20,...){
              y <- utils::head.matrix(x,n=n,...)
              rownames(y) <- 1:n
              return(y)
          })
setMethod("tail",signature(x="importer"),
          function(x,n=20,...){
              y <- utils::tail.matrix(x,n=n,...)
              rownames(y) <- seq.int(to=nrow(x),length.out=n)
              return(y)
          })

as.data.frame.importer <- function(x, row.names = NULL, optional = FALSE, ...){
  stop("\"importer\" objects cannot be coerced into data frames directly.
First apply 'as.data.set()' or 'subset()'.")
}

as.data.frame.spss.system.importer <- as.data.frame.importer
as.data.frame.spss.portable.importer <- as.data.frame.importer
as.data.frame.spss.fixed.importer <- as.data.frame.importer
as.data.frame.Stata.importer <- as.data.frame.importer
as.data.frame.Stata_new.importer <- as.data.frame.importer

reversed.item <- function(x){
    y <- x
    if(length(labels(x))){
        l <- labels(x)
        vals <- vals.orig <- l@values
        labs <- l@.Data
        vf <- value.filter(x)
        from <- vals[is.valid2(vals,vf)]
        to <- rev(from)
        for(i in seq_along(from)){
            y[x==from[i]] <- to[i]
            vals[vals.orig==from[i]] <- to[i]
        }
        to.l <- structure(vals,names=labs)
        ii <- order(to.l)
        to.l <- to.l[ii]
        labels(y) <- to.l
    } else {
        v <- valid.values(x)
        if(length(v)){
            from <- v@filter
            to <- rev(from)
            for(i in seq_along(from)){
                y[x==from[i]] <- to[i]
            }
        } else stop("No way to determine how reverse the codes.")
    }
    return(y)
}




reversed.factor <- function(x){
    y <- factor(x,levels=rev(levels(x)))
    return(y)
}

setGeneric("reversed",
           function(x)standardGeneric("reversed"))
setMethod("reversed","item.vector",reversed.item)
setMethod("reversed","factor",reversed.factor)

---
title: Analysing the American National Election Study of 1948
output: rmarkdown::html_vignette
vignette: >
  % \VignetteIndexEntry{Analysing the American National Election Study of 1948}
  % \VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



```{r,echo=FALSE,message=FALSE}
knitr::opts_chunk$set(comment=NA,
               fig.align="center",
               results="markup")
```


# Introduction

This vignette gives an example for the analysis of a typical social science data set.
It is the data file of the *American National Election Study* of
1948[^1] available from the [American National Election Studies website](https://electionstudies.org). The data file contains data
from to USA-wide surveys conducted October and November 1948
by the Survey Research Centre, University Michigan
(principal investigators: Angus Campbell and Robert L. Kahn). The
total number of cases in the data set is 662 and the number of variables
is 65 (more details about this
data set can be found
at <https://electionstudies.org/studypages/1948prepost/1948prepost.htm>).

[^1]: National Election Studies, 1948: *Post-Election Study [dataset].* Ann Arbor, MI: University of Michigan, Center for Political Studies [producer and distributor], 1999. ANES Dataset ID: 1948.T; ICPSR Study Number: 7218. These materials are based on work supported by the National Science Foundation under Grant Nos.: SBR-9707741, SBR-9317631, SES-9209410, SES-9009379, SES-8808361, SES-8341310, SES-8207580, and SOC77-08885. Any opinions, findings and conclusions or recommendations expressed in these materials are those of the author(s) and do not necessarily reflect those of the National Science Foundation.

With 662 cases and 65 variables, the 1948 ANES data set is relatively
small as compared to current social science data sets. Such larger data
sets can be processed along the same lines as in this vignette.
Unlike the 1948 ANES data,
their size as well as, in some cases, legal restrictions prohibit the
inclusion of such a data set into the package, however.

This vignette starts with a demonstration how a data file can be
examined before loading it and how a subset of the data can be loaded
into memory.
After loading this subset into memory, some descriptive analyses are
conducted that showcase the construction of contingency tables and of
general tables of descriptive statistics using the
`genTable` function. In addition, a logit analysis
is demonstrated and the collection of several logit coefficients into
a comprehensive table by the `mtable` function.

It should be noted that the analyses reported in the following are conducted
only for purpose of demonstrating the features of the package and are
not to be considered of conclusive scientific evidence of any kind.

This vignette is run with the help of the [*knitr* package](https://cran.r-project.org/package=knitr). This
allows to showcase not only data management facilities provided
by *memisc*. The following code also demonstrates how output
created with some of the facilities of *memisc* can neatly
integrated in reports generated with *knitr*.
Before we start, we adjust *knitr*'s output (with which this
vignette is formatted) to produce HTML where possible.
```{r}
knit_print.codebook <-function(x,...) 
  knitr::asis_output(format_html(x,...))

knit_print.descriptions <-function(x,...) 
  knitr::asis_output(format_html(x,...))

knit_print.ftable <-function(x,options,...)
  knitr::asis_output(
    format_html(x,
                digits=if(length(options$ftable.digits))
                          options$ftable.digits
                       else 0,
                ...))
# We can now adjust the number of digits after the comma
# for each column e.g. by adding an `ftable.digits` option
# to an R chunk, as in ```{r,ftable=c(2,2,0)}

knit_print.mtable <-function(x,...)
  knitr::asis_output(format_html(x,...))
```

# Reading in a "portable" SPSS data file

We start with importing the data into R.
The following code extracts the SPSS portable file
`NES1948.POR` from
zip file `NES1948.ZIP` delivered with the *memisc* package.
```{r, message=FALSE}
library(memisc)
options(digits=3)
nes1948.por <- unzip(system.file("anes/NES1948.ZIP",package="memisc"),
                     "NES1948.POR",exdir=tempfile())
```
Now the portable file is in a temporary directory and the path
to the file is contained in the string variable `nes1948.por`.
In the next step, the file is declared as a SPSS/PSPP portable
file using the function `spss.portable.file`, which as first argument
takes the path to the file. `spss.portable.file`
reads in the information about the variables contained in the data
set and counts the
number of cases in the file. That is, standard
I/O operations are used on the file, but the data read in are just
thrown away without allocating core memory for the data.
This counting of cases can, of course, be suppressed if it would
take too long.
```{r}
nes1948 <- spss.portable.file(nes1948.por)
print(nes1948)
```
At this stage, the data are not loaded into the memory yet.
But we can see which variables exist inside
the data set:
```{r}
names(nes1948)
```
Note that the variable names are all changed from uppercase to lowercase (SPSS does not distinguish uppercase and lowercase variable names and uppercase looks like shouting). Casefolding could have been suppressed by the call `spsp.portable.file(nes1948.por,tolower=FALSE)`.  

We also can ask for a description ("variable label") for each variable:
```{r}
description(nes1948)
```
or even a code book using
```{r, eval=FALSE}
codebook(nes1948)
```
(this is not shown
here because the output would have taken more then thirty pages).
We can also get a codebook of the first few variabels instead, with
```{r}
codebook(nes1948[1:5])
```


### Reading in a subset of the data
After we have decided which variables to use we can read in a subset of
the data:
```{r}
vote.48 <- subset(nes1948,
              select=c(
                  V480018,
                  V480029,
                  V480030,
                  V480045,
                  V480046,
                  V480047,
                  V480048,
                  V480049,
                  V480050
                  ))
```
The subset of the ANES 1948 we read in is now contained in the
variable `vote.48`, which contains an object of class `data.set`.
A `data.set` is an "embellished" version of a `data.frame`,
a data structure intended to contained `labelled` vectors.
`labelled` vectors contain the all the special information
attached to the variables in the original data set, such as
variable labels, value labels, and general missing values.
A short summary of this special information shows up after a
call to `str`.
```{r}
str(vote.48)
```
This output shows, for example, that variable `V480018`
has the description (variable label) "DID R VOTE/FOR WHOM" is considered as
having nominal level of measurement, has seven value labels
and one defined missing value.

Since the variable names in the ANES data set are not very mnemonic,
we rename the variables:
```{r}
vote.48 <- rename(vote.48,
                  V480018 = "vote",
                  V480029 = "occupation.hh",
                  V480030 = "unionized.hh",
                  V480045 = "gender",
                  V480046 = "race",
                  V480047 = "age",
                  V480048 = "education",
                  V480049 = "total.income",
                  V480050 = "religious.pref"
        )
```
Since many data sets available from public repositories have
such non-mnemonic variable names as in this example, it might
be convenient to do the data loading and renaming in one step.
Indeed it is possible:
```{r,eval=FALSE}
vote.48 <- subset(nes1948,
                  select=c(
                    vote           = V480018,
                    occupation.hh  = V480029,
                    unionized.hh   = V480030,
                    gender         = V480045,
                    race           = V480046,
                    age            = V480047,
                    education      = V480048,
                    total.income   = V480049,
                    religious.pref = V480050
                  ))
```


Before we start with analyses, we take a closer look at
the data.
```{r}
codebook(vote.48)
```
We now have obtained a *codebook*, which contains information
of the class and type of the variables in the data set,
the value labels and defined missing values, and counts of
the distinct values of the variables.

# Analysis

## Some descriptive analyses

We start our analyses with a contingency table, but first
we make some preparations: We recode the variables of
interest into a smaller number of categories in order to
get results that are easier to read and interpret.
```{r}
vote.48 <- within(vote.48,{
  vote3 <- recode(vote,
    1 -> "Truman",
    2 -> "Dewey",
    3:4 -> "Other"
    )
  occup4 <- recode(occupation.hh,
    10:20 -> "Upper white collar",
    30 -> "Other white collar",
    40:70 -> "Blue collar",
    80 -> "Farmer"
    )
  relig3 <- recode(religious.pref,
    1 -> "Protestant",
    2 -> "Catholic",
    3:5 -> "Other,none"
    )
   race2 <- recode(race,
    1 -> "White",
    2 -> "Black"
    )
  })
```

Having constructed the unordered factors
`vote3`, `occup4`, `relig3`, and `race2` we can
proceed examining the association the vote, occupational class,
relgious denomination, and race. First, we look upon a simple
contingency table. 

```{r}
ftable(xtabs(~vote3+occup4,data=vote.48))
```

Tables of percentages may seem more informative about the impact
of various factors on the vote. So we use the function
`genTable` to obtain such tables of percentages:
```{r, ftable.digits=c(2,2,2,0)}
gt1 <- genTable(percent(vote3)~occup4,data=vote.48)
## For knitr-ing, we use ```{r, ftable.digits=c(2,2,2,0)} here.
ftable(gt1,row.vars=2)
```

Obviously, voters from farmer and blue collar worker households
were especially supportive of President Truman, while voters
of upper white collar background largely supported
the Republican Candidate Dewey.


```{r, ftable.digits=c(2,2,2,0)}
gt2 <- genTable(percent(vote3)~relig3,data=vote.48)
ftable(gt2,row.vars=2)
```


This table shows that Catholics and adherents of other denominations
were more supportive of Truman than of Dewey.


```{r, ftable.digits=c(2,2,2,0)}
gt3 <- genTable(percent(vote3)~race2,data=vote.48)
ftable(gt3,row.vars=2)
```


African Americans apparently supported Truman by a large majority. The
number of members of this group in the sample is very small, however,
so that such an inference would be very shaky.


```{r, ftable.digits=c(2,2,2,0)}
gt4 <- genTable(percent(vote3)~total.income,data=vote.48)
ftable(gt4,row.vars=2)
```
The table of percentage of vote by income suggests that income had
some considerable influence on the choice either of Truman or of Dewey,
but the unequal distribution of income categories warrants a more
refined analysis that takes into account the uncertainty about the
vote percentages. Therefore, the percentages of support for Truman
broken down by income shown with confidence intervals:
```{r, ftable.digits=c(2,2,2)}
## For knitr-ing, we use ```{r, ftable.digits=c(2,2,2)} here.
inc.tab <- genTable(percent(vote3,ci=TRUE)~total.income,data=vote.48)
ftable(inc.tab,row.vars=c(3,2))
```

Occupational class is more evenly distributed in the sample, thus it may be possible
to obtain more precise estimates of the percentages of support for Truman for occupational
classes:
```{r, ftable.digits=c(2,2,2)}
occup.tab <- genTable(percent(vote3,ci=TRUE)~occup4,data=vote.48)
ftable(occup.tab,row.vars=c(3,2))
```
The upper and lower white-collar and blue-collar classes are quite distinct
with regard to the percentages of support for Truman. The point estimates
of the percentages are outside the confidence intervals of the respective
other occupational classes, the confidence intervals do not even overlap.
However, it is not clear whether farmers are distinct from the blue-collar
and lower white-collar classes.

### Logit modelling of candidate choice

In the following we conduct a logit analysis of the vote for Truman.
First, we assign non-standard contrasts the categorical predictors.
Here, the function `contr` is used to assign treatment (dummy)
contrasts to `occup4` and `total.income` with
baseline category 3 and 4, respectively.
```{r}
vote.48 <- within(vote.48,{
  contrasts(occup4) <- contr("treatment",base = 3)
  contrasts(total.income) <- contr("treatment",base = 4)
  })
```
We now fit some logistic regression models of the
impact occupational class, income, and religious denomination
on the vote choice supporting Truman. The contrasts of
the occupational class and income factors are such that they
compare the choices of the members of the blue-collar class
with all other classes and the middle income group (\$ 2000-2999)
with the other income groups. The religious denomination factor
compares Protestants with Catholics and those with other or
no denominations.
```{r}
model1 <- glm((vote3=="Truman")~occup4,data=vote.48,
              family="binomial")
model2 <- glm((vote3=="Truman")~total.income,data=vote.48,
              family="binomial")
model3 <- glm((vote3=="Truman")~occup4+total.income,data=vote.48,
              family="binomial")
model4 <- glm((vote3=="Truman")~relig3,data=vote.48,
              family="binomial")
model5 <- glm((vote3=="Truman")~occup4+relig3,data=vote.48,
              family="binomial")
```
First, we use `mtable` to construct a comparative table
of the estimates of `model1`,
`model2`, and `model3`. We thus can compare
the impact of occupational class and income on the choice of
candidate Truman.
```{r}
mtable(model1,model2,model3,summary.stats=c("Nagelkerke R-sq.","Deviance","AIC","N"))
```
`mtable` returns an object of class `"mtable"`. When formatted
it looks close to the requirements of typical social science
publications. Yet at least we want to change the technical variable
names into non-technical ones, for which we can use `relabel`:

```{r}
relabel(mtable(
            "Model 1"=model1,
            "Model 2"=model2,
            "Model 3"=model3,
            summary.stats=c("Nagelkerke R-sq.","Deviance","AIC","N")),
          UNDER="under",
          "AND OVER"="and over",
          occup4="Occup. class",
          total.income="Income",
          gsub=TRUE
          )
```


The comparison of the pseudo-R-Square values of model 1 and 2 suggests that
occupational class has a stronger influence on a preference for Truman than
household income. Indeed, if occupational class is taken into account, the
effect of income is no longer statistically significant as the column
corresponding to model 3 indicates.

Second, we compare the effect of occupational class and religious denomination
on the preference for Truman along the same lines as above. We use
`mtable` to collect the estimates of `model1`, `model4`,
and `model5` into a common table.

```{r}
relabel(mtable(
              "Model 1"=model1,
              "Model 4"=model4,
              "Model 5"=model5,
              summary.stats=c("Nagelkerke R-sq.","Deviance","AIC","N")),
            occup4="Occup. class",
            relig3="Religion",
            gsub=TRUE
            )
```


A comparison of the pseudo-R-squared values suggests that also the effect of
religious denomination is weaker than that of occupational class. However,
as the third column in the above table indicates the effect of religious
denomination remains statistically significant.

```{r,echo=FALSE}
rm(knit_print.codebook,
   knit_print.descriptions,
   knit_print.ftable,
   knit_print.mtable)
```

---
title: Tables of model estimates
output: rmarkdown::html_vignette
vignette: >
  % \VignetteEngine{knitr::rmarkdown}
  % \VignetteIndexEntry{Tables of model estimates}
---

```{r,echo=FALSE,message=FALSE}
knitr::opts_chunk$set(comment=NA,
               fig.align="center",
               results="markup")
```

First, we load the package and estimate some regression models.

```{r, message=FALSE}
library(memisc)
lm0 <- lm(sr ~ pop15 + pop75,              data = LifeCycleSavings)
lm1 <- lm(sr ~                 dpi + ddpi, data = LifeCycleSavings)
lm2 <- lm(sr ~ pop15 + pop75 + dpi + ddpi, data = LifeCycleSavings)
```

Next, we put them together into a table with `mtable()`.

```{r}
mtable123 <- mtable("Model 1"=lm0,"Model 2"=lm1,"Model 3"=lm2,
                    summary.stats=c("sigma","R-squared","F","p","N"))
```
Third, we improve the appearance of the results by relabeling the `mtable` object.
```{r, comment=NA}
mtable123 <- relabel(mtable123,
                      "(Intercept)" = "Constant",
                      pop15 = "Percentage of population under 15",
                      pop75 = "Percentage of population over 75",
                      dpi = "Real per-capita disposable income",
                      ddpi = "Growth rate of real per-capita disp. income"
)
```
Finally we view the results in HTML format.
```{r, results='asis'}
show_html(mtable123)
```
The results can be written into an HTML file using
`write_html(mtable123,file="mtable123.html")` or the like. MS Word or LibreOffice can import such a file. 

[^1]: Unfortunately, LibreOffice currently seems to do a less than perfect job in
    importing HTML tables. The columns are a bit too wide and need to be
    adjusted by hand as do the skips after paragraphs. The example you find at
    the link mentioned above is thus manually adjusted.

---
title: Handling questionnaire items
output: rmarkdown::html_vignette
vignette: >
  % \VignetteIndexEntry{Handling questionnaire items}
  % \VignetteEngine{knitr::rmarkdown}
  % \VignetteEncoding{UTF-8}
---

```{r,echo=FALSE,message=FALSE}
knitr::opts_chunk$set(comment=NA,
               fig.align="center",
               results="markup")
```

# Motivation

R is a great tool to do data analysis and for data management tasks that arise
in the context of big data analytics. Nevertheless there is still room for
improvement in terms of the support for data management tasks that arise in the
social sciences, especially when it comes to handling data that come from social
surveys and opinion surveys. The main reason for this is that the way that
questionnaire item responses as they are usually coded in machine-readable
survey data sets do not directly and easily translate into R's data types for
numeric and categorical data, that is, numerical vectors and factors. As a
consequence, many social scientists exercise their everyday data management
tasks with commercial software packages such as SPSS or Stata, but there may be
social scientists who either cannot afford such commercial software or prefer to
use, out of principle, open-source software for all steps of data management and
analysis.

It is one of the aim of the "memisc" package to provide a bridge between social
science data sets of variables that contain coded responses to questionnaire
items, with their typical structures involving labelled numeric response codes
and numeric codes declared as "missing values". As an illustrative example,
suppose in a pre-election survey, respondents are asked about which party they
are going to vote for in their constituency in the framework of a
first-past-the-post electoral system. Suppose the response categories offered to
the respondents are "Conservative", "Labour", "Liberal Democrat", "Other
party".[^1] A survey agency that actually conducts the interviews with a sample
of voters may, according to common practice, use the following codes to collect
the responses to the question about the vote intention:

<div class="booktable">
<style>
.booktable table th {
    border-top: 0 none;
    border-bottom: 0 none;
    background-color: unset;
}
.booktable table td {
    border-top: 0 none;
    border-bottom: 0 none;
    background-color: unset;
}

.booktable table tr.even {
    background-color: unset;
}

.booktable table tr.header {
    border-top: 2px solid;
    border-bottom: 1px solid;
}

.booktable table tbody tr:last-child {
    border-bottom: 2px solid;
}
</style>

Response category     Code        
-------------------- ----- ----- 
Conservative            1      
Labour                  2      
Liberal Democrat        3      
Other Party             4      
Will not vote           9      
Don't know             97   (M)
Answer refused         98   (M)
Not applicable         99   (M)

</div>

In data sets that contain the results of such coding are essentially numeric
data -- with some additional information about the "value labels" (the labels
attached to the numeric values) and about the "missing values" (those numeric
values that indicate responses that one usually does not want to include into
statistical analysis). While this coding frame for responses to survey
questionnaires is far from uncommon in the social sciences, it is not
straightforward to retain this information in *R* objects. Here there are two
main alternatives, (1) one could store the responses as a numeric vector,
thereby losing the information about the labelled values, or (2) one could store
the responses as a factor, thereby losing the information contained in the
codes. Either way, one will lose the information about the "missing values". Of
course, one can filter out these missing values before data analysis by
replacing them with `NA`, but it would convenient to have facilities that do
that automatically.

[^1]: Those familiar with British politics will realise that this is a
    simplification of the menu of available choices that voters in England
    typically face in an election of the House of Commons.

# Standard attributes of survey items

The "memisc" package introduces a new data type (more correctly an `S4` class)
that allows to handle such data, that allows to adjust labels or missing values
definitions and to translate such data as needed either into numeric vectors of
factors, thereby automatically filtering out the missing values. This data time
(or `S4` class) is, for lack of a better term, called `"item"`. In general,
users do not bother with the construction of such item vectors. Usually they are
generated when data sets are imported from data files in SPSS or Stata
format. This page is mainly concerned with describing the structure of such item
vectors and how they can be manipulated in the data management step that usually
precedes data analysis. It is thus possible to do all the data management in *R*
from importing the pristine data obtained from data archives or other data
providers, such as the survey institutes to which a principal investigator has
delegated data collection. Of course, the facilities introduced by the `"item"`
data type also allow to create appropriate representations of survey item
responses if a principal investigator obtains only raw numeric codes. In the
following, the construction of `"item"` vectors from raw numeric data is mainly
used to highlight their structure.

## Value labels {#value_labels}

```{r,echo=FALSE}
set.seed(20)
vote.probs <- c(.361,.29,.23,1-.361-.29-.23)
decis.probs <- c(vote.probs*.651,1-.651)
all.probs <- c(.9*decis.probs,.1*rep(1/3,3))
voteint <- sample(c(1:4,9,97,98,99),
                  prob=all.probs,
                  replace=TRUE,
                  size=200)                
library(memisc)
```
Suppose a numeric vector of responses to the question about their vote intention coded using
the coding frame shown above looks as follows
```{r}
voteint
```

This numeric vector is transformed into an `"item"` vector by attaching labels
to the codes. The *R* code to attach labels that reflect the coding frame shown
above may look like follows (if formatted nicely):

```{r}
# This is to be run *after* memisc has been loaded.
labels(voteint) <- c(Conservative       =  1,
                     Labour             =  2,
                     "Liberal Democrat" =  3, # We have whitespace in the label, 
                     "Other Party"      =  4, # so we need quotation marks
                     "Will not vote"    =  9,
                     "Don't know"       = 97,
                     "Answer refused"   = 98,
                     "Not applicable"   = 99)
```
`voteint` is now an item vector, for which a particular `"show"` method is defined:
```{r}
class(voteint)
str(voteint)
voteint
```

Like with factors, if *R* shows the contents of the vector, the labels are shown
(instead of the codes). Since item vectors typically are quite long, because
they come from interviewing a survey sample and usual survey sample sizes are
about 2000, we usually do not want to see all the values in the
vector. `"memisc"` anticipates this and shows at most a single line of
output. (In the output, also the "level of measurement" is shown, which at this
point does not have a consquence. It will become clear later what the
implications of the "level of measurement" are.)

In line with the usual semantics `labels(voteint)` will now show us a
description of the labels and to which values they are assigned:

```{r}
labels(voteint)
```
Now if we rather want shorter labels, we can change them either by something like `labels(voteint) <- ...` or by changing the labels using `relabel()`:
```{r}
voteint <- relabel(voteint,
                   "Conservative"     = "Cons",
                   "Labour"           = "Lab",
                   "Liberal Democrat" = "LibDem",
                   "Other Party"      = "Other",
                   "Will not vote"    = "NoVote",
                   "Don't know"       = "DK",
                   "Answer refused"   = "Refused",
                   "Not applicable"   = "N.a.")
```
Let us take a look at the result:
```{r}
labels(voteint)
voteint
str(voteint)
```

## Missing values {#missing_values}

In the coding plan shown above, the values 97, 98, and 99 are marked as "missing
values", that is, while they represent coded responses, they are not to be
considered as valid in the sense of providing information about the respondent's
vote intention. For the statistical analysis of vote intention it is natural to
replace them by `NA`. Yet replacing codes 97, 98, and 99 already at the stage of
importing data into *R* memory would mean a loss of potentially precious
information since it precludes, e.g. the motivation to refuse responding to the
vote intention question or the antencedents of undecidedness. Hence it is better
to mark those values and to delay their replacement by `NA` to a later stage in
the analysis of vote intentions and to be able to undo or change the
"missingness" of these values. For example, not only may one be interested in
the antecedents of response refusals but also be interested to analyse vote
intention with non-voting excluded or included. The memisc package provides,
like SPSS and PSPP, facilities to mark particular values of an item vector as
"missing" and change such designations throughout the data preperation stage.

There are several ways with `"memisc"` to make distinctions between valid and
missing values. The first way that mirrors the way it is done in SPSS. To
illustrate this we return to the fictitious vote intention example. The values
97,98,99 of `voteint` are designated as "missing" by

```{r}
missing.values(voteint) <- c(97,98,99)
```

The missing values are reflected in the output of `voteint`, (labels of) missing
values are marked with `*` in the output:

```{r}
voteint
```

It is also possible to extend the set of missing values: We add another value to
the set of missing values.

```{r}
missing.values(voteint) <- missing.values(voteint) + 9
```
The missing values can be recalled as usual:
```{r}
missing.values(voteint)
```
The missing values are turned into `NA` if `voteint` is coerced into a numeric vector or a factor, which is what usually happens before the eventual statistical analysis:

```{r}
as.numeric(voteint)[1:30]
as.factor(voteint)[1:30]
```
It is also possible to drop all missing value designations:
```{r}
missing.values(voteint) <- NULL
missing.values(voteint)
as.numeric(voteint)[1:30]
```
In contrast to SPSS it is possible with `"memisc"` to designate the valid, i.e. non-missing values:
```{r}
valid.values(voteint) <- 1:4
valid.values(voteint)
missing.values(voteint)
```
Instead of individual valid or missing values it is also possible to define a *range* of values as valid:
```{r}
valid.range(voteint) <- c(1,9)
missing.values(voteint)
```

## Other attributes of survey items {#annotations}
Other software packages targeted at social scientists also allow to add
annotations to the variables in a data set, which are not subject to the
syntactic constraints of variable names. These annotations are usually called
"variable labels" in these software packages. In `"memisc"` the corresponding
term is "description". In continuation of the running example, we add a
description to the vote intention variable:

```{r}
description(voteint) <- "Vote intention"
description(voteint)
```

In contrast to other software, `"memisc"` allows to attach arbitrarily
annotation to survey items, such as the wording of a survey question:

```{r}
wording(voteint) <- "Which party are you going to vote for in the general election next Tuesday?"
wording(voteint)
annotation(voteint)
annotation(voteint)["wording"]
```

## Codebooks of survey items

It is common in survey research to describe a data set in the form of a
*codebook*. A codebook summarises each variable in the data set in terms of its
relevant attributes, that is, the label attached to the variable (in the context
of the `memisc` package this is called its "description"), the labels attached
to the values of the variable, which values of the variable are supposed to be
*missing* or *valid*, as well as univariate summary statistics of each variable,
usually without and with missing variables included. Such functionality is
provided in this package by the function `codebook()`. `codebook()` when applied
to an `"item"` object returns a `"codebook"` object, which when printed to the
console gives an overview of the variable usually required for the codebook of a
data set (the production of codebooks for whole data sets is described further
below). To illustrate the `codebook()` function we now produce a codebook of the
`voteint` item variable created above:

```{r}
codebook(voteint)
```

As can be seen in the output, the `codebook()` function reports the name of the
variable, the description (if defined for the variable), and the question
wording (again if defined). Further it reports the storage mode (which is use by
*R*), the level of measurement ("nominal", "ordinal", "interval", or "ratio")
and the range of valid values (or alternatively, individually defined valid
values, individually defined missing values, or ranges of missing values). For
item variables with value labels, it shows a table of frequencies of the
labelled values, and the percentages of valid values and all values with
missings included.

Codebooks are particularly useful to find "wild codes", that is codes that are
not labelled, and usually produced by coding errors. Such coding errors may be
less common in data sets produced by CAPI or CATI or online surveys, but they
may occur in older data sets from before the age of computer-assisted
interviewing and also during the course of data management. This use of
codebooks is demonstrated in the following by deliberatly adding some coding
errors into a copy of our `voteint` variable:

```{r}
voteint1 <- voteint
voteint1[sample(length(voteint),size=20)] <- c(rep(5,13),rep(7,7))
```

The presence of these "wild codes" can now be spotted using `codebook()`:

```{r}
codebook(voteint1)
```

The output shows that 20 observations contain wild codes in this variable. Why
don't we get a list of wild codes as part of the codebook? The reason is that
codebook is supposed also to work with continuous variables that have thousands
of unique, unlabelled values. Users certainly will not like to see them all as
part of a codebook.

In order to get a list of wild codes the development version of "memisc"
contains the function `wild.codes()`, which we apply to the variable `voteint1`

```{r}
wild.codes(voteint1)
```

We see that 6.5 and 3.5 percent of the observations have the wild codes 5 and 7.

To see how `codebook()` works with variables without value labels, we create an
unlabelled copy of our `voteint` variable:

```{r}
voteint2 <- voteint
labels(voteint2) <- NULL # This deletes all value labels
codebook(voteint2)
```

Usually, variables without labelled values represent measures on an interval or
ratio scale. In that case, we do not want to see how many unlabelled values
there are, but we want to get some other statistics, such as mean, variance,
etc. To this purpose, we decleare the variable `voteint2` to have an
interval-scale level of measurement.[^2]

```{r}
measurement(voteint2) <- "interval"
codebook(voteint2)
```
[^2]: Of course, substantially it does not make sense at all to form averages
    etc. of voting choices, so "do not try this at home". This example is merely
    to demonstrate codebooks and the setting of scale-levels.

For convenience of including them into word-processor documents, there is also
the possibility to export codebooks into HTML:

```{r,results='asis'}
show_html(codebook(voteint))
```

# Data sets: Containers of survey items

Usually one expects to be able handle data on responses to survey items not in
isolation, but as part of a data set, which contains a multitude of observations
on many variables. The usual data structure in *R* to contain
observation-on-variables data is the *data frame*. In principle it is possible
to put survey item vectors as described above into a data frame, nevertheless
the `"memisc"` package provides a special data structure to contain survey item
data called data sets or data set-objects, that is, objects of class
`"data.set"`. This opens up the possibility to automatically translate survey
items into regular vectors and factors, as expected by typical data analysis
functions, such as `lm()` or `glm()`.

## The structure of `"data.set"` objects

Data set objects have essentially the same row-by-column structure as data
frames: They are a set of vectors (however of class `"item"`) all with the same
length, so that in each row of the data set there are values in these vectors.
Observations can be addressed as rows of a `"data.set"` and variabels can be
addressed as columns, just as one may used to with regards to data frames. Most
data management operations that you can do with data frames can also be done
with data sets (such as merging them or using the functions `with()` or
`within()`). Yet in contrast to data frames, data sets are always expected to
contain objects of class `"item"`, and any vectors or factors from which a
`"data.set"` object is constructed are changed into `"item"` objects.

Another difference is the way that `"data.set"` objects are shown on the
console. As `S4` objects, if a user types in the name of a `"data.set"` objects,
the function `show()` (and not `print()`) is applied to it. The `show()`-method
for data set objects is defined in such a way that only the first few
observations of the first few variables are shown on the console -- in contrast
to `print()` as applied to a data frame, which shows *all* observations on *all*
variables. While it may be intuitive and convenient to be shown all observations
in a small data frame, this is not what you will want if your data set contains
more than 2000 observations on several hundred variables, the dimensions that
typical social science data sets have that you can download from data archives
such as that of ICPSR or GESIS.

The main facilitites of `"data.set"` objects are demonstrated in what follows.
First, we create a data set with fictional survey responses
```{r}
Data <- data.set(
          vote = sample(c(1,2,3,4,8,9,97,99),
                        size=300,replace=TRUE),
          region = sample(c(rep(1,3),rep(2,2),3,99),
                          size=300,replace=TRUE),
          income = round(exp(rnorm(300,sd=.7))*2000)
          )
```
Then, we take a look at this already sizeable `"data.set"`" object:
```{r}
Data
```

In this case, our data set has only three variables, all of which are shown, but
of the observations we see only the first 25. Actually the number of
observations shown can be determined by the option `"show.max.obs"` which
defaults to 25, but can be changed as convenient:

```{r}
options(show.max.obs=5)
Data
# Back to the default
options(show.max.obs=25)
```
If you *really* want to see the complete data on your console, then you can use `print()` instead:
```{r,eval=FALSE}
print(Data)
```
but you should not do this with large data sets, such as the Eurobarometer trend file ...

## Manipulating data in data sets

Typical data management tasks that you would otherwise have done in commercial packages like SPSS or
Stata can be conducted within data set objects. Actually to provide this possibility (to the author
of the package) was the main reason that the `"memisc"` package was created.
To demonstrate this, we continue with our fictional data which we prepare for further analysis:
```{r}
Data <- within(Data,{
  description(vote) <- "Vote intention"
  description(region) <- "Region of residence"
  description(income) <- "Household income"
  wording(vote) <- "If a general election would take place next Tuesday,
                    the candidate of which party would you vote for?"
  wording(income) <- "All things taken into account, how much do all
                    household members earn in sum?"
  foreach(x=c(vote,region),{
    measurement(x) <- "nominal"
    })
  measurement(income) <- "ratio"
  labels(vote) <- c(
                    Conservatives         =  1,
                    Labour                =  2,
                    "Liberal Democrats"   =  3,
                    "Other"               =  4,
                    "Don't know"          =  8,
                    "Answer refused"      =  9,
                    "Not applicable"      = 97,
                    "Not asked in survey" = 99)
  labels(region) <- c(
                    England               =  1,
                    Scotland              =  2,
                    Wales                 =  3,
                    "Not applicable"      = 97,
                    "Not asked in survey" = 99)
  foreach(x=c(vote,region,income),{
    annotation(x)["Remark"] <- "This is not a real survey item, of course ..."
    })
  missing.values(vote) <- c(8,9,97,99)
  missing.values(region) <- c(97,99)

  # These to variables do not appear in the
  # the resulting data set, since they have the wrong length.
  junk1 <- 1:5
  junk2 <- matrix(5,4,4)
  
})
```

Now that we have added information to the data set that reflects the code plan
of the variables, we take a look how the it looks like:

```{r}
Data
```

As you can see, labelled item look a bit like factors, but with a difference:
User-defined missing values are marked with an asterisk.

Subsetting a data set object works as expected:
```{r}
EnglandData <- subset(Data,region == "England")
EnglandData
```


## Codebooks of data sets {#codebooks}

Previouly, we created a code book for individual survey items. But it is also
possible to create a codebook for a whole data set (what one usually wants to
have a codebook of). Obtaining a codebook is simple, by applying the function
`codebook()` to the data frame:

```{r}
codebook(Data)
```
On a website, it looks better in HTML:
```{r,results='asis'}
show_html(codebook(Data))
```

## Translating data sets into data frames

The punchline of the existence of `"data.set"` objects however is that they can
be coerced into regular data frames, using `as.data.frame()`, which causes
survey items to be translated into regular numeric vectors or factors using
`as.numeric()`, `as.factor()` or `as.ordered()` as above, and pre-determined
missing values changed into `NA`. Whether a survey item is changed into a
numerical vector, an unordered or an ordered factor depends on the declared
measurement level (which can be manipulated by `measurement()` as shown above).

In the example developed so far, the variables `vote` and `region` are declared
to have a nominal level of measurement, while `income` is declared to have a
ratio scale. That is, in statistical analyses, we want the first two variables
to be handled as (unordered) factors, and the income variable as a numerical
vector. In addition, we want all the user-declared missing values to be changed
into `NA` so that observations where respondents stated to "don't know" what
they are goint go vote for are excluded from the analysis. So let's see whether
this works - we coerce our data set into a data frame:

```{r}
DataFr <- as.data.frame(Data)
## Looking a the data frame structure
str(DataFr)
## Looking at the first 25 observations
DataFr[1:25,]
```

Indeed the translation works as expected, so we can use it for statistical
analysis, here a simple cross tab:

```{r}
xtabs(~vote+region,data=DataFr)
```

In fact, since many functions such as `xtabs()`, `lm()`, `glm()`, etc. coerce
theire `data=` argument into a data frame, an explicit coercion with
`as.data.frame()` is not always needed:

```{r}
xtabs(~vote+region,data=Data)
```
Sometimes we do want missing values to be included, and this is possible too:
```{r}
xtabs(~vote+region,data=within(Data, 
                               vote <- include.missings(vote)))
```

For convenience, there is also a codebook method for data frames:
```{r,results='asis'}
show_html(codebook(DataFr))
```

# More tools for data preparation

When social scientists work with survey data, these are not always organised and
coded in a way that suits the intended data analysis. For this reasons, the
`"memisc"` package provides the two functions `recode()` and `cases()`. The
former is -- as the name suggests -- for recoding, while the second allows for
complex distinctions of cases and can be seen as a more general version of
`ifelse()`. These two functions are demonstrated with a "real-life" example.

## Recoding {#recode}

The function `recode()` is similar in semantics to the function of the same name
in package [`"car"`][CAR] and designed in such a way that it does not conflict
with this function.  In fact, if `recode()` is called in the way as expected in
package `"car"`, it will dispatch processing to this function. In other words,
users of this other package may use `recode()` as they are used to.  The version
of the `recode()` function provided by `"memisc"` differs from the `"car"`
version in so far as its syntax is more R-ish (or so I believe).

[CAR]: https://cran.r-project.org/package=car

Here we load an example data set -- a subset of the German Longitudinal Election
Study for 2013[^3] -- into R's memory.

```{r}
load(system.file("gles/gles2013work.RData",package="memisc"))
```

As a simple example for the use of `recode()` we use this function to recode
German Bundesländer into an item with two values or East and West Germany. But
first we create a codebook for the variable that contains the Bundesländer
codes:

```{r,results='asis'}
with(gles2013work,
     show_html(codebook(bula)))
```
We now recode the Bundesländer codes into a new variable:
```{r}
gles2013work <- within(gles2013work,
                       east.west <- recode(bula,
                                          East = 1 <- c(3,4,8,13,14,16),
                                          West = 2 <- c(1,2,5:7,9:12,15)
                                          ))
```
and check whether this was successful:
```{r}
xtabs(~bula+east.west,data=gles2013work)
```
as can be seen, `recode()` was called in such a way that not only old
codes are transferred into new ones, but also the new codes are labelled.


[^3]: The [German Longitudinal Election
Study](https://www.gesis.org/gles/about-gles) is funded by the German National
Science Foundation (DFG) and carried out outin close cooperation with the
[DGfW](https://www.dgfw.info/), German Society for Electoral Studies.  Principal
investigators are Hans Rattinger (University of Mannheim, until 2014), Sigrid
Roßteutscher (University of Frankfurt), Rüdiger Schmitt-Beck (University of
Mannheim), Harald Schoen (Mannheim Centre for European Social Research, from
2015), Bernhard Weßels (Social Science Research Center Berlin), and Christof
Wolf (GESIS – Leibniz Institute for the Social Sciences, since 2012). Neither
the funding organisation nor the principal investigators bear any responsibility
for the example code shown here.

## Case distinctions 

Recoding can be used to combine the codes of an item into a smaller set, but
sometimes one needs to do more complex data preparations, in which the values of
some variable are set conditional on values of another one, etc.  For such
tasks, the `"memisc"` package provides the function `cases()`. This function
takes several expressions that evaluate to logical vectors as arguments and
returns a numeric vector or a factor, the values or level of which indicate for
each observation which of the expressions evaluates to `TRUE` the respective
observation. The factor levels are named after the logical expressions. A simple
example looks thus:

```{r}
x <- 1:10
xc <- cases(x <= 3, 
            x > 3 & x <= 7, 
            x > 7)
data.frame(x,xc)
```
In this example `cases()` returns a factor. It can also be made to return a numeric value:
```{r}
xn <- cases(1 <- x <= 3, 
            2 <- x > 3 & x <= 7, 
            3 <- x > 7)
data.frame(x,xn)
```

This example shows the way `cases()` works in the abstract. How this can be made
used of in practical example is best demonstrated by a real-world example, again
using data from the German Longitudinal Election Study.

In the 2013 election module, the intention to vote during the pre-election of
respondents interviewed in the pre-election wave (`wave==1`) and the
participation in the election of respondents interviewed in the post-election
wave (`wave==2`) are recorded in different data set variables, named here
`intent.turnout` and `turnout`.  The variable `intent.voteint` has codes for
whether the respondents were sure to participate (1), were likely to participate
(2), were undecided (3), likely not to (4), sure not to participate (5), or
whether they have cast a postal vote (6). Variable `turnout` has codes for those
who participated in the election (1) or did not (2).

The intention for the candidate vote is recorded in variable `voteint.candidate`
and the intention for the list vote is recoded in variable `voteint.list` for
the pre-election wave.  A postal vote for party candidate is recorded in
variable `postal.vote.candidate` and for a party list is in variable
`postal.vote.list`. Recalled votes in the post-election wave are recorded in
variables `vote.candidate` and `vote.list`.

These various variables are combined into two variables that has valid values
for both waves, `candidate.vote` and `list.vote`. For this, several conditions
have to be handled: whether a respondent is in the pre-election or the
post-election wave, whether s/he is not likely or sure not to vote, or whether
she has cast a postal vote. Thus the variable `cases()` is helpful here:

```{r}
gles2013work <- within(gles2013work,{

  candidate.vote <- cases(
              wave == 1 & intent.turnout == 6 -> postal.vote.candidate,
              wave == 1 & intent.turnout %in% 4:5 -> 900,
              wave == 1 & intent.turnout %in% 1:3 -> voteint.candidate,
              wave == 2 & turnout == 1 -> vote.candidate,
              wave == 2 & turnout == 2 -> 900
            )

  list.vote <- cases(
              wave == 1 & intent.turnout == 6 -> postal.vote.list,
              wave == 1 & intent.turnout %in% 4:5 -> 900,
              wave == 1 & intent.turnout %in% 1:3 -> voteint.list,
              wave == 2 & turnout ==1 -> vote.list,
              wave == 2 & turnout ==2 -> 900
            )
})
```

The code shown above does the following: In the pre-election wave (`wave == 1`),
the `candidate.vote` variable receives the value of the postal vote variable
`postal.vote.candidate` if a postal vote was cast (`intent.turnout == 6`), it
receives the value `900` for those respondents who where likely or sure not to
vote (`intent.turnout %in% 4:5`), and the value of the variable
`voteint.candidate` for all others (`intent.turnout %in% 1:3`).  In the
post-election wave (`wave == 2`) variable `candidate.vote` receives the value of
variable `vote.candidate` if the respondent has voted (`turnout == 1`) or the
value `900` if s/he has not voted (`turnout == 2`).  The variable `list.vote` is
constructed in an analogous manner from the variables `wave`, `intent.turnout`,
`turnout`, `postal.vote.list`, `voteint.list` and `vote.list`.  After the
constructin, the resulting variables `candidate.vote` and `list.vote` are
labelled and missing values are declared:

```{r}
gles2013work <- within(gles2013work,{
  candidate.vote <- recode(as.item(candidate.vote),
                      "CDU/CSU"   =  1 <- 1,
                      "SPD"       =  2 <- 4,
                      "FDP"       =  3 <- 5,
                      "Grüne"     =  4 <- 6,
                      "Linke"     =  5 <- 7,
                      "NPD"       =  6 <- 206,
                      "Piraten"   =  7 <- 215,
                      "AfD"       =  8 <- 322,
                      "Other"     = 10 <- 801,
                      "No Vote"   = 90 <- 900,
                      "WN"        = 98 <- -98,
                      "KA"        = 99 <- -99
                  )
  list.vote <- recode(as.item(list.vote),
                      "CDU/CSU"   =  1 <- 1,
                      "SPD"       =  2 <- 4,
                      "FDP"       =  3 <- 5,
                      "Grüne"     =  4 <- 6,
                      "Linke"     =  5 <- 7,
                      "NPD"       =  6 <- 206,
                      "Piraten"   =  7 <- 215,
                      "AfD"       =  8 <- 322,
                      "Other"     = 10 <- 801,
                      "No Vote"   = 90 <- 900,
                      "WN"        = 98 <- -98,
                      "KA"        = 99 <- -99
                  )
  
   missing.values(candidate.vote) <- 98:99
   missing.values(list.vote) <- 98:99
   measurement(candidate.vote) <- "nominal"
   measurement(list.vote) <- "nominal"
})
```

Finally, we can get a cross-tabulation of list votes and the East-West factor and
a cross tabulation of candidate votes against list votes:
```{r,width=120}
xtabs(~list.vote+east.west,data=gles2013work)
xtabs(~list.vote+candidate.vote,data=gles2013work)
```

---
title: Combining flattened contingency tables (and other tables)
output: rmarkdown::html_vignette
vignette: >
  % \VignetteIndexEntry{Combining flattened contingency tables (and other tables)}
  % \VignetteEngine{knitr::rmarkdown}
  % \VignetteEncoding{UTF-8}
---

```{r,echo=FALSE,message=FALSE}
knitr::opts_chunk$set(comment=NA,
               fig.align="center",
               results="markup")
```

This vignette demonstrates how flattened contingency tables can be combined into more
complex tables. For starters, we create a few contingendy tables.

```{r}
tab.Class.Age <- xtabs(Freq~Class+Age,data=Titanic)
tab.Survived.Class.Age <- xtabs(Freq~Survived+Class+Age,data=Titanic)
tab.Survived.Class.Sex <- xtabs(Freq~Survived+Class+Sex,data=Titanic)
tab.Survived.Class <- xtabs(Freq~Survived+Class,data=Titanic)
tab.Survived.Sex <- xtabs(Freq~Survived+Sex,data=Titanic)
tab.Survived.Age <- xtabs(Freq~Survived+Age,data=Titanic)
tab.Survived <- xtabs(Freq~Survived,data=Titanic)
```
Then we load the *memisc* package:
```{r,message=FALSE}
library(memisc)
```

Next, we create a few flattened contingency tables:
```{r}
(ftab.Survived.Age <- ftable(tab.Survived.Age))
(ftab.Survived.Sex <- ftable(tab.Survived.Sex))
```
Since these tables have the same row variables, we can combine them columnwise:
```{r}
cbind(ftab.Survived.Age,
      ftab.Survived.Sex)
```
We can even add a simple table of counts:
```{r}
cbind(ftab.Survived.Age,
      ftab.Survived.Sex,
      Total=tab.Survived)
```
Of course, it is not enough to see such tables on screen, we also may need them in a presentable
format, such as LaTeX or HTML. For the latter format we again have `format_html` and `show_html`.
To see how this works, we first look at the individual `ftable`s:
```{r,results='asis'}
show_html(ftab.Survived.Age)
```
```{r,results='asis'}
show_html(ftab.Survived.Sex)
```
... and then at their combination
```{r,results='asis'}
show_html(
  cbind(ftab.Survived.Age,
      ftab.Survived.Sex,
      Total=tab.Survived)
)
```
To make, in "knitr", the creation of HTML-versions automatic one can use the following little trick:
```{r}
knit_print.ftable_matrix <-function(x,options,...)
  knitr::asis_output(
    format_html(x,
                digits=if(length(options$ftable.digits))
                          options$ftable.digits
                       else 0,
                ...))
```
Now we do not need to call `show_html` while using "knitr":
```{r}
cbind(ftab.Survived.Age,
      ftab.Survived.Sex,
      Total=tab.Survived)
```
This can be undone by removing the method function of `knit_print`:
```{r}
rm(knit_print.ftable_matrix)
```
For `show_html` and `toLatex` there are some variants in how the 
variable names are positioned, for example:
```{r,results='asis'}
show_html(
  cbind(ftab.Survived.Age,
      ftab.Survived.Sex,
      Total=tab.Survived),
  varinfront=FALSE
)
```

```{r,results='asis'}
show_html(
  cbind(ftab.Survived.Age,
      ftab.Survived.Sex,
      Total=tab.Survived),
  varontop=FALSE
)
```
Of course it is also possible to combine flat contingency tables rowwise:
```{r}
ftab.Age.Survived <- ftable(tab.Survived.Age,col.vars=1)
ftab.Sex.Survived <- ftable(tab.Survived.Sex,col.vars=1)
ftab.Class.Survived <- ftable(tab.Survived.Class,col.vars=1)

rbind(
  ftab.Age.Survived,
  ftab.Sex.Survived,
  ftab.Class.Survived,
  Total=tab.Survived
)
```
```{r,results='asis'}
show_html(
  rbind(
    ftab.Age.Survived,
    ftab.Sex.Survived,
    ftab.Class.Survived,
    Total=tab.Survived
  )
)
```
It is also possible to create the `ftable`s from tables of percentages or generic tables (created
with `genTable`) etc.
```{r,results='asis'}
ptab.Survived.Age<-percentages(Survived~Age,data=Titanic)
ptab.Survived.Sex<-percentages(Survived~Sex,data=Titanic)
ptab.Survived.Class<-percentages(Survived~Class,data=Titanic)

fptab.Age.Survived <- ftable(ptab.Survived.Age,col.vars=1)
fptab.Sex.Survived <- ftable(ptab.Survived.Sex,col.vars=1)
fptab.Class.Survived <- ftable(ptab.Survived.Class,col.vars=1)

show_html(
  rbind(
    fptab.Age.Survived,
    fptab.Sex.Survived,
    fptab.Class.Survived
  ),
  digits=1
)
```

It is also possible to combine `rbind` and `cbind`:
```{r,results='asis'}
tab.Age <- xtabs(Freq~Age,data=Titanic)
tab.Sex <- xtabs(Freq~Sex,data=Titanic)
tab.Class <- xtabs(Freq~Class,data=Titanic)


show_html(
  rbind(
    cbind(fptab.Age.Survived,Total=tab.Age),
    cbind(fptab.Sex.Survived,Total=tab.Sex),
    cbind(fptab.Class.Survived,Total=tab.Class)
  ),
  digits=c(1,0) # One digit after dot for percentages 
                # no digits for total counts.
)
```
The same construct as LaTeX code:
```{r}
toLatex(
  rbind(
    cbind(fptab.Age.Survived,Total=tab.Age),
    cbind(fptab.Sex.Survived,Total=tab.Sex),
    cbind(fptab.Class.Survived,Total=tab.Class)
  ),
  digits=c(1,0) # One digit after dot for percentages 
                # no digits for total counts.
)
```

---
title: Generating Tables of Descriptive Statistics
output: rmarkdown::html_vignette
vignette: >
  % \VignetteIndexEntry{Generating Tables of Descriptive Statistics}
  % \VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r,echo=FALSE,message=FALSE}
knitr::opts_chunk$set(comment=NA,eval=FALSE)
```


# Motivation

*R* is well suited for statistical graphics, the application of advanced data
analysis techniques, and Monte Carlo studies of estimators. However, it lacks
support for the typical data management tasks as they arise in the social
sciences as well as for the simple generation of desctiptive
statistics. "memisc" facilitates not only typical data management tasks of
survey researchers, but also the generation of descriptive statistics, as they
are often a first step in serious social science data analysis.  In particular
it facilitates the creation of tables of percentages of other descriptive
statistics broken down by subgroups in the data. This is mainly achieved by the
function `genTable`, which is described in the following section. The section
thereafter describes how tables thus created can be exported to LaTeX and HTML.


Note that these examples require data not included in the package 
(you need to register to [GESIS](https://www.gesis.org) to download the data). 
The vignette code cannot be run without this additional data.

# Creating Tables of Descriptive Statistics

General table of descriptive statistics can be created using the function
`genTable()`. The syntax of calls to this function is quite similar to that of
the function `xtabs()`: The first argument (tagged `formula`) is a formula that
determines the descriptive statistics used and by what groups they are
computed. The left-hand side of the formula determines the statistics being
computed. The right-hand side determines the grouping factor(s). The second
argument is an optional `data=` argument that determines from which data frame
or data set the descriptive statistics are to be computed. This is illustrated
by the following example, which uses (like the page on item
objects, see `?item`) the GLES 2013 election study[^1].  In this example we
first create a table of some descriptives of the age distribution of the
respondents per German federal state:

```r
library(memisc)
ZA5702 <- spss.system.file("Data/ZA5702_v2-0-0.sav",
                           ignore.scale.info=TRUE) # Because the measurement info in the file is wrong.
gles2013work <- subset(ZA5702,
                       select=c(
                         wave                  = survey,
                         gender                = vn1,
                         byear                 = vn2c,
                         bmonth                = vn2b,
                         intent.turnout        = v10,
                         turnout               = n10,
                         voteint.candidate     = v11aa,
                         voteint.list          = v11ba,
                         postal.vote.candidate = v12aa,
                         postal.vote.list      = v12ba,
                         vote.candidate        = n11aa,
                         vote.list             = n11ba,                 
                         bula                  = bl
                       ))
gles2013work <- within(gles2013work,{
  measurement(byear) <- "interval"
  measurement(bmonth) <- "interval"
  age <- 2013 - byear
  age[bmonth > 9] <- age[bmonth > 9] - 1
  
})
options(digits=3)
age.tab <- genTable(c(Mean=mean(age),
           `Std.dev`=sd(age),
           Median=median(age))~bula,
         data=gles2013work)
age.tab
```

```
         bula
          Baden-Wuerttemberg Bayern Berlin Brandenburg Bremen Hamburg Hessen
  Mean                    55     54     53          60     60      51     57
  Std.dev                 19     19     20          19     12      19     19
  Median                  57     56     57          62     63      53     60
         bula
          Mecklenburg-Vorpommern Niedersachsen Nordrhein-Westfalen
  Mean                        57            55                  54
  Std.dev                     19            18                  19
  Median                      60            56                  55
         bula
          Rheinland-Pfalz Saarland Sachsen Sachsen-Anhalt Schleswig-Holstein
  Mean                 57       62      58             55                 60
  Std.dev              18       17      17             17                 20
  Median               60       65      60             56                 65
         bula
          Thueringen
  Mean            58
  Std.dev         17
  Median          60
```
This table does not look good, so we transprose it:

```r
age.tab <- t(age.tab)
age.tab
```

```                      
bula                     Mean Std.dev Median
  Baden-Wuerttemberg     54.5    18.9   57.0
  Bayern                 54.4    18.9   56.0
  Berlin                 52.8    19.8   57.0
  Brandenburg            59.7    19.3   62.5
  Bremen                 60.4    11.5   63.0
  Hamburg                51.5    18.7   53.0
  Hessen                 56.9    18.5   60.0
  Mecklenburg-Vorpommern 57.0    19.2   60.5
  Niedersachsen          55.1    18.4   56.0
  Nordrhein-Westfalen    53.9    19.1   55.0
  Rheinland-Pfalz        57.2    18.2   60.5
  Saarland               61.9    17.3   65.0
  Sachsen                58.3    16.7   60.5
  Sachsen-Anhalt         54.7    17.1   56.0
  Schleswig-Holstein     60.0    19.9   65.0
  Thueringen             57.8    17.4   60.0
```
In the next example we create a table of percentages of the second votes per federal state. First we have to prepare the data, though:

```r
gles2013work <- within(gles2013work,{

  candidate.vote <- cases(
              wave == 1 & intent.turnout == 6 -> postal.vote.candidate,
              wave == 1 & intent.turnout %in% 4:5 -> 900,
              wave == 1 & intent.turnout %in% 1:3 -> voteint.candidate,
              wave == 2 & turnout == 1 -> vote.candidate,
              wave == 2 & turnout == 2 -> 900
            )

  list.vote <- cases(
              wave == 1 & intent.turnout == 6 -> postal.vote.list,
              wave == 1 & intent.turnout %in% 4:5 -> 900,
              wave == 1 & intent.turnout %in% 1:3 -> voteint.list,
              wave == 2 & turnout ==1 -> vote.list,
              wave == 2 & turnout ==2 -> 900
            )

  candidate.vote <- recode(as.item(candidate.vote),
                      "CDU/CSU"   =  1 <- 1,
                      "SPD"       =  2 <- 4,
                      "FDP"       =  3 <- 5,
                      "Grüne"     =  4 <- 6,
                      "Linke"     =  5 <- 7,
                      "NPD"       =  6 <- 206,
                      "Piraten"   =  7 <- 215,
                      "AfD"       =  8 <- 322,
                      "Other"     = 10 <- 801,
                      "No Vote"   = 90 <- 900,
                      "WN"        = 98 <- -98,
                      "KA"        = 99 <- -99
                  )
  list.vote <- recode(as.item(list.vote),
                      "CDU/CSU"   =  1 <- 1,
                      "SPD"       =  2 <- 4,
                      "FDP"       =  3 <- 5,
                      "Grüne"     =  4 <- 6,
                      "Linke"     =  5 <- 7,
                      "NPD"       =  6 <- 206,
                      "Piraten"   =  7 <- 215,
                      "AfD"       =  8 <- 322,
                      "Other"     = 10 <- 801,
                      "No Vote"   = 90 <- 900,
                      "WN"        = 98 <- -98,
                      "KA"        = 99 <- -99
                  )
  
   missing.values(candidate.vote) <- 98:99
   missing.values(list.vote) <- 98:99
   measurement(candidate.vote) <- "nominal"
   measurement(list.vote) <- "nominal"
})
```

```
Warning messages:
1: In cases(postal.vote.candidate <- wave == 1 & intent.turnout ==  :
  78 NAs created
2: In cases(postal.vote.list <- wave == 1 & intent.turnout == 6, 900 <- wave ==  :
  78 NAs created
3: In recode(as.item(candidate.vote), `CDU/CSU` = 1 <- 1, SPD = 2 <- 4,  :
  recoding created 18 NAs
4: In recode(as.item(list.vote), `CDU/CSU` = 1 <- 1, SPD = 2 <- 4,  :
  recoding created 19 NAs
```

(When the code is run, some warnings are issued, that indicate that the conditions are not exhaustive,
that is, there are some observations for which none of the conditions in the call `cases()`
are met. The corresponding elements of resulting vector will contain `NA` for these observations.
In the present case this occurs with observations that have missing values in both `intent.turnout` and `turnout`.)

After having set up the data, we get our table of percentages:

```r
vote.tab <- genTable(percent(list.vote)~bula,
                     data=gles2013work)
options(digits=1)
t(vote.tab)
```

```
bula                     CDU/CSU SPD FDP Grüne Linke NPD Piraten AfD Other No Vote   N
  Baden-Wuerttemberg          28  22   7    17     6 0.4     2.1 4.6   1.1      12 285
  Bayern                      36  18   6    11     5 0.0     2.4 4.0   2.0      16 451
  Berlin                      27  22   8    10    14 1.8     1.8 6.6   0.6       8 166
  Brandenburg                 20  23   2     6    19 0.6     0.6 2.5   1.2      25 162
  Bremen                      22  26   0    17    13 0.0     0.0 4.3   0.0      17  23
  Hamburg                     22  36   2     4     7 2.2     0.0 4.4   2.2      20  45
  Hessen                      42  26   3     8     4 0.0     0.5 3.0   0.0      12 200
  Mecklenburg-Vorpommern      33  20   2     4    18 1.4     2.7 1.4   0.0      18 146
  Niedersachsen               33  32   3    10     3 0.0     0.7 0.7   0.4      17 284
  Nordrhein-Westfalen         33  31   3    11     4 0.4     2.3 1.8   0.7      13 563
  Rheinland-Pfalz             39  21   2     6     9 1.6     0.8 3.9   1.6      15 127
  Saarland                    40  40   0     0     0 0.0     0.0 0.0   0.0      20  30
  Sachsen                     49  17   1     3    14 0.3     1.2 0.9   0.3      13 332
  Sachsen-Anhalt              27  29   1     8    19 0.4     0.8 0.4   0.0      13 241
  Schleswig-Holstein          28  26   4     9     4 0.0     0.0 5.2   0.9      22 116
  Thueringen                  35  16   2     3    22 1.2     0.0 2.4   0.8      18 245
```

It is of course also possible to create multi-dimensional tables, i.e. tables created by grouping by more than one factor:

```r
gles2013work <- within(gles2013work,{

  # We relabel the items, since they are originally in German
  labels(turnout) <- c("Yes, voted"=1, "No, did not vote"=2)   
  labels(gender) <- c("Male"=1,"Female"=2)
})
genTable(percent(turnout)~gender+bula,
         data=gles2013work)
```

```
, , bula = Baden-Wuerttemberg

                  gender
                   Male Female
  Yes, voted         88     85
  No, did not vote   12     15
  N                  90     61

, , bula = Bayern

                  gender
                   Male Female
  Yes, voted         85     80
  No, did not vote   15     20
  N                  89    129

, , bula = Berlin

                  gender
                   Male Female
  Yes, voted        100     85
  No, did not vote    0     15
  N                  38     52

, , bula = Brandenburg

                  gender
                   Male Female
  Yes, voted         83     77
  No, did not vote   17     23
  N                  36     62

, , bula = Bremen

                  gender
                   Male Female
  Yes, voted         91     80
  No, did not vote    9     20
  N                  11      5

, , bula = Hamburg

                  gender
                   Male Female
  Yes, voted         88     76
  No, did not vote   12     24
  N                  16     21

, , bula = Hessen

                  gender
                   Male Female
  Yes, voted         91     81
  No, did not vote    9     19
  N                  66     48

, , bula = Mecklenburg-Vorpommern

                  gender
                   Male Female
  Yes, voted         84     72
  No, did not vote   16     28
  N                  32     47

, , bula = Niedersachsen

                  gender
                   Male Female
  Yes, voted         88     83
  No, did not vote   12     17
  N                  75     70

, , bula = Nordrhein-Westfalen

                  gender
                   Male Female
  Yes, voted         90     82
  No, did not vote   10     18
  N                 148    158

, , bula = Rheinland-Pfalz

                  gender
                   Male Female
  Yes, voted         84     85
  No, did not vote   16     15
  N                  43     34

, , bula = Saarland

                  gender
                   Male Female
  Yes, voted         91     72
  No, did not vote    9     28
  N                  11     18

, , bula = Sachsen

                  gender
                   Male Female
  Yes, voted         88     88
  No, did not vote   12     12
  N                 103     73

, , bula = Sachsen-Anhalt

                  gender
                   Male Female
  Yes, voted         89     81
  No, did not vote   11     19
  N                  63     73

, , bula = Schleswig-Holstein

                  gender
                   Male Female
  Yes, voted         89     85
  No, did not vote   11     15
  N                  37     33

, , bula = Thueringen

                  gender
                   Male Female
  Yes, voted         91     71
  No, did not vote    9     29
  N                  70     73
```

# Formatting Tables of Descriptive Statistics

The results of `genTable()` are objects of class `"table"` so that they can be
re-arranged into a "flattened" table by the function `ftable`. To demonstrate
this, we continue the previous example:

```r
gt <- genTable(percent(turnout)~gender+bula,
         data=gles2013work)
# We beautify the table a bit ...
names(dimnames(gt)) <- c("Voted","Gender","State")
gt <- dimrename(gt,"Yes, voted"="Yes",
                "No, did not vote"="No")
ftable(gt,col.vars = c("Gender","Voted"))
```

```
                       Gender Male         Female        
                       Voted   Yes  No   N    Yes  No   N
State                                                    
Baden-Wuerttemberg              88  12  90     85  15  61
Bayern                          85  15  89     80  20 129
Berlin                         100   0  38     85  15  52
Brandenburg                     83  17  36     77  23  62
Bremen                          91   9  11     80  20   5
Hamburg                         88  12  16     76  24  21
Hessen                          91   9  66     81  19  48
Mecklenburg-Vorpommern          84  16  32     72  28  47
Niedersachsen                   88  12  75     83  17  70
Nordrhein-Westfalen             90  10 148     82  18 158
Rheinland-Pfalz                 84  16  43     85  15  34
Saarland                        91   9  11     72  28  18
Sachsen                         88  12 103     88  12  73
Sachsen-Anhalt                  89  11  63     81  19  73
Schleswig-Holstein              89  11  37     85  15  33
Thueringen                      91   9  70     71  29  73
```

Arranging the cells of a table using `ftable()` improves the appearance of the
results of `genTable()` on screen, but to include the results into a word
processor document or a LaTeX file, further facilities are needed and provided
by "memisc". To include the flattened table into a LaTeX document, one can
convert and store it in the appropriate format using `toLatex()` and
`writeLines()`

```r
ft <- ftable(gt,col.vars = c("Gender","Voted"))
lt <- toLatex(ft,digits=c(1,1,0,1,1,0))
writeLines(lt,con="Voted2013-GenderState.tex")
```


For HTML output, one can use `show_html()` (e.g. for inclusion in "knitr"
documents) and `write_html()`, both functions being based on
`format_html()`. Here we continue the example to demonstate this:

```r
show_html(ft,digits=c(1,1,0,1,1,0))
```

```{=html}
<style>
      table#ftable-3 {
         border-collapse: collapse; border-style: none; margin: 2ex auto;
      } 
      table#ftable-3 tr {
          border-style: none;
      } 
      table#ftable-3 td {
        padding-top: 3px; 
        padding-bottom: 3px; 
        padding-left: 0.5ex; 
        padding-right: 0.5ex; 
        margin: 0px; 
        margin-top: 0px;
        margin-bottom: 0px;
        border-style: none; 
        border-width: 0; 
      }
table#ftable-3 tr:nth-child(1){
     border-top: 2px solid;
}
table#ftable-3 tr:nth-child(2){
     border-bottom: 1px solid;
}
table#ftable-3 tr:nth-child(18){
     border-bottom: 2px solid;
}
table#ftable-3 tr:nth-child(1) td:nth-child(n+3){
     border-bottom: 1px solid;
}
table#ftable-3 td:nth-child(4),
table#ftable-3 td:nth-child(7),
table#ftable-3 td:nth-child(10),
table#ftable-3 td:nth-child(13),
table#ftable-3 td:nth-child(16),
table#ftable-3 td:nth-child(19){
         padding-left: 0px;
         padding-right: 0px;
         text-align: center; 
         width: .2ex;
         /*background-color: blue;*/
        }
table#ftable-3 td:nth-child(3),
table#ftable-3 td:nth-child(6),
table#ftable-3 td:nth-child(9),
table#ftable-3 td:nth-child(12),
table#ftable-3 td:nth-child(15),
table#ftable-3 td:nth-child(18){
         padding-left: 0.5ex;
         padding-right: 0px;
         text-align: right; 
         /*background-color: red;*/
        }
table#ftable-3 td:nth-child(5),
table#ftable-3 td:nth-child(8),
table#ftable-3 td:nth-child(11),
table#ftable-3 td:nth-child(14),
table#ftable-3 td:nth-child(17),
table#ftable-3 td:nth-child(20){
         padding-left: 0px;
         padding-right: 0.5ex;
         text-align: left; 
         /*background-color: green;*/
        } 
table#ftable-3 td:nth-child(n of .header){
         padding-left: 0.5ex;
         padding-right: 0.5ex;
         text-align: center; 
         /*background-color: gray;*/
        } 
</style>
<table class="ftable" id="ftable-3">
<tr><td></td><td>Gender:</td><td colspan="9" class="header">Male</td><td colspan="9" class="header">Female</td></tr>
<tr><td>State</td><td>Voted:</td><td colspan="3" class="header">Yes</td><td colspan="3" class="header">No</td><td colspan="3" class="header">N</td><td colspan="3" class="header">Yes</td><td colspan="3" class="header">No</td><td colspan="3" class="header">N</td></tr>
<tr><td>Baden-Wuerttemberg</td><td></td><td>87</td><td>.</td><td>8</td><td>12</td><td>.</td><td>2</td><td>90</td><td></td><td></td><td>85</td><td>.</td><td>2</td><td>14</td><td>.</td><td>8</td><td>61</td><td></td><td></td></tr>
<tr><td>Bayern</td><td></td><td>85</td><td>.</td><td>4</td><td>14</td><td>.</td><td>6</td><td>89</td><td></td><td></td><td>79</td><td>.</td><td>8</td><td>20</td><td>.</td><td>2</td><td>129</td><td></td><td></td></tr>
<tr><td>Berlin</td><td></td><td>100</td><td>.</td><td>0</td><td>0</td><td>.</td><td>0</td><td>38</td><td></td><td></td><td>84</td><td>.</td><td>6</td><td>15</td><td>.</td><td>4</td><td>52</td><td></td><td></td></tr>
<tr><td>Brandenburg</td><td></td><td>83</td><td>.</td><td>3</td><td>16</td><td>.</td><td>7</td><td>36</td><td></td><td></td><td>77</td><td>.</td><td>4</td><td>22</td><td>.</td><td>6</td><td>62</td><td></td><td></td></tr>
<tr><td>Bremen</td><td></td><td>90</td><td>.</td><td>9</td><td>9</td><td>.</td><td>1</td><td>11</td><td></td><td></td><td>80</td><td>.</td><td>0</td><td>20</td><td>.</td><td>0</td><td>5</td><td></td><td></td></tr>
<tr><td>Hamburg</td><td></td><td>87</td><td>.</td><td>5</td><td>12</td><td>.</td><td>5</td><td>16</td><td></td><td></td><td>76</td><td>.</td><td>2</td><td>23</td><td>.</td><td>8</td><td>21</td><td></td><td></td></tr>
<tr><td>Hessen</td><td></td><td>90</td><td>.</td><td>9</td><td>9</td><td>.</td><td>1</td><td>66</td><td></td><td></td><td>81</td><td>.</td><td>2</td><td>18</td><td>.</td><td>8</td><td>48</td><td></td><td></td></tr>
<tr><td>Mecklenburg-Vorpommern</td><td></td><td>84</td><td>.</td><td>4</td><td>15</td><td>.</td><td>6</td><td>32</td><td></td><td></td><td>72</td><td>.</td><td>3</td><td>27</td><td>.</td><td>7</td><td>47</td><td></td><td></td></tr>
<tr><td>Niedersachsen</td><td></td><td>88</td><td>.</td><td>0</td><td>12</td><td>.</td><td>0</td><td>75</td><td></td><td></td><td>82</td><td>.</td><td>9</td><td>17</td><td>.</td><td>1</td><td>70</td><td></td><td></td></tr>
<tr><td>Nordrhein-Westfalen</td><td></td><td>89</td><td>.</td><td>9</td><td>10</td><td>.</td><td>1</td><td>148</td><td></td><td></td><td>82</td><td>.</td><td>3</td><td>17</td><td>.</td><td>7</td><td>158</td><td></td><td></td></tr>
<tr><td>Rheinland-Pfalz</td><td></td><td>83</td><td>.</td><td>7</td><td>16</td><td>.</td><td>3</td><td>43</td><td></td><td></td><td>85</td><td>.</td><td>3</td><td>14</td><td>.</td><td>7</td><td>34</td><td></td><td></td></tr>
<tr><td>Saarland</td><td></td><td>90</td><td>.</td><td>9</td><td>9</td><td>.</td><td>1</td><td>11</td><td></td><td></td><td>72</td><td>.</td><td>2</td><td>27</td><td>.</td><td>8</td><td>18</td><td></td><td></td></tr>
<tr><td>Sachsen</td><td></td><td>88</td><td>.</td><td>3</td><td>11</td><td>.</td><td>7</td><td>103</td><td></td><td></td><td>87</td><td>.</td><td>7</td><td>12</td><td>.</td><td>3</td><td>73</td><td></td><td></td></tr>
<tr><td>Sachsen-Anhalt</td><td></td><td>88</td><td>.</td><td>9</td><td>11</td><td>.</td><td>1</td><td>63</td><td></td><td></td><td>80</td><td>.</td><td>8</td><td>19</td><td>.</td><td>2</td><td>73</td><td></td><td></td></tr>
<tr><td>Schleswig-Holstein</td><td></td><td>89</td><td>.</td><td>2</td><td>10</td><td>.</td><td>8</td><td>37</td><td></td><td></td><td>84</td><td>.</td><td>8</td><td>15</td><td>.</td><td>2</td><td>33</td><td></td><td></td></tr>
<tr><td>Thueringen</td><td></td><td>91</td><td>.</td><td>4</td><td>8</td><td>.</td><td>6</td><td>70</td><td></td><td></td><td>71</td><td>.</td><td>2</td><td>28</td><td>.</td><td>8</td><td>73</td><td></td><td></td></tr>
</table>
```

```r
show_html(ft,digits=c(1,1,0,1,1,0),show.titles=FALSE)
```

```{=html}
<style>
      table#ftable-4 {
         border-collapse: collapse; border-style: none; margin: 2ex auto;
      } 
      table#ftable-4 tr {
          border-style: none;
      } 
      table#ftable-4 td {
        padding-top: 3px; 
        padding-bottom: 3px; 
        padding-left: 0.5ex; 
        padding-right: 0.5ex; 
        margin: 0px; 
        margin-top: 0px;
        margin-bottom: 0px;
        border-style: none; 
        border-width: 0; 
      }
table#ftable-4 tr:nth-child(1){
     border-top: 2px solid;
}
table#ftable-4 tr:nth-child(2){
     border-bottom: 1px solid;
}
table#ftable-4 tr:nth-child(18){
     border-bottom: 2px solid;
}
table#ftable-4 tr:nth-child(1) td:nth-child(n+2){
     border-bottom: 1px solid;
}
table#ftable-4 td:nth-child(3),
table#ftable-4 td:nth-child(6),
table#ftable-4 td:nth-child(9),
table#ftable-4 td:nth-child(12),
table#ftable-4 td:nth-child(15),
table#ftable-4 td:nth-child(18){
         padding-left: 0px;
         padding-right: 0px;
         text-align: center; 
         width: .2ex;
         /*background-color: blue;*/
        }
table#ftable-4 td:nth-child(2),
table#ftable-4 td:nth-child(5),
table#ftable-4 td:nth-child(8),
table#ftable-4 td:nth-child(11),
table#ftable-4 td:nth-child(14),
table#ftable-4 td:nth-child(17){
         padding-left: 0.5ex;
         padding-right: 0px;
         text-align: right; 
         /*background-color: red;*/
        }
table#ftable-4 td:nth-child(4),
table#ftable-4 td:nth-child(7),
table#ftable-4 td:nth-child(10),
table#ftable-4 td:nth-child(13),
table#ftable-4 td:nth-child(16),
table#ftable-4 td:nth-child(19){
         padding-left: 0px;
         padding-right: 0.5ex;
         text-align: left; 
         /*background-color: green;*/
        } 
table#ftable-4 td:nth-child(n of .header){
         padding-left: 0.5ex;
         padding-right: 0.5ex;
         text-align: center; 
         /*background-color: gray;*/
        } 
</style>
<table class="ftable" id="ftable-4">
<tr><td></td><td colspan="9" class="header">Male</td><td colspan="9" class="header">Female</td></tr>
<tr><td></td><td colspan="3" class="header">Yes</td><td colspan="3" class="header">No</td><td colspan="3" class="header">N</td><td colspan="3" class="header">Yes</td><td colspan="3" class="header">No</td><td colspan="3" class="header">N</td></tr>
<tr><td>Baden-Wuerttemberg</td><td>87</td><td>.</td><td>8</td><td>12</td><td>.</td><td>2</td><td>90</td><td></td><td></td><td>85</td><td>.</td><td>2</td><td>14</td><td>.</td><td>8</td><td>61</td><td></td><td></td></tr>
<tr><td>Bayern</td><td>85</td><td>.</td><td>4</td><td>14</td><td>.</td><td>6</td><td>89</td><td></td><td></td><td>79</td><td>.</td><td>8</td><td>20</td><td>.</td><td>2</td><td>129</td><td></td><td></td></tr>
<tr><td>Berlin</td><td>100</td><td>.</td><td>0</td><td>0</td><td>.</td><td>0</td><td>38</td><td></td><td></td><td>84</td><td>.</td><td>6</td><td>15</td><td>.</td><td>4</td><td>52</td><td></td><td></td></tr>
<tr><td>Brandenburg</td><td>83</td><td>.</td><td>3</td><td>16</td><td>.</td><td>7</td><td>36</td><td></td><td></td><td>77</td><td>.</td><td>4</td><td>22</td><td>.</td><td>6</td><td>62</td><td></td><td></td></tr>
<tr><td>Bremen</td><td>90</td><td>.</td><td>9</td><td>9</td><td>.</td><td>1</td><td>11</td><td></td><td></td><td>80</td><td>.</td><td>0</td><td>20</td><td>.</td><td>0</td><td>5</td><td></td><td></td></tr>
<tr><td>Hamburg</td><td>87</td><td>.</td><td>5</td><td>12</td><td>.</td><td>5</td><td>16</td><td></td><td></td><td>76</td><td>.</td><td>2</td><td>23</td><td>.</td><td>8</td><td>21</td><td></td><td></td></tr>
<tr><td>Hessen</td><td>90</td><td>.</td><td>9</td><td>9</td><td>.</td><td>1</td><td>66</td><td></td><td></td><td>81</td><td>.</td><td>2</td><td>18</td><td>.</td><td>8</td><td>48</td><td></td><td></td></tr>
<tr><td>Mecklenburg-Vorpommern</td><td>84</td><td>.</td><td>4</td><td>15</td><td>.</td><td>6</td><td>32</td><td></td><td></td><td>72</td><td>.</td><td>3</td><td>27</td><td>.</td><td>7</td><td>47</td><td></td><td></td></tr>
<tr><td>Niedersachsen</td><td>88</td><td>.</td><td>0</td><td>12</td><td>.</td><td>0</td><td>75</td><td></td><td></td><td>82</td><td>.</td><td>9</td><td>17</td><td>.</td><td>1</td><td>70</td><td></td><td></td></tr>
<tr><td>Nordrhein-Westfalen</td><td>89</td><td>.</td><td>9</td><td>10</td><td>.</td><td>1</td><td>148</td><td></td><td></td><td>82</td><td>.</td><td>3</td><td>17</td><td>.</td><td>7</td><td>158</td><td></td><td></td></tr>
<tr><td>Rheinland-Pfalz</td><td>83</td><td>.</td><td>7</td><td>16</td><td>.</td><td>3</td><td>43</td><td></td><td></td><td>85</td><td>.</td><td>3</td><td>14</td><td>.</td><td>7</td><td>34</td><td></td><td></td></tr>
<tr><td>Saarland</td><td>90</td><td>.</td><td>9</td><td>9</td><td>.</td><td>1</td><td>11</td><td></td><td></td><td>72</td><td>.</td><td>2</td><td>27</td><td>.</td><td>8</td><td>18</td><td></td><td></td></tr>
<tr><td>Sachsen</td><td>88</td><td>.</td><td>3</td><td>11</td><td>.</td><td>7</td><td>103</td><td></td><td></td><td>87</td><td>.</td><td>7</td><td>12</td><td>.</td><td>3</td><td>73</td><td></td><td></td></tr>
<tr><td>Sachsen-Anhalt</td><td>88</td><td>.</td><td>9</td><td>11</td><td>.</td><td>1</td><td>63</td><td></td><td></td><td>80</td><td>.</td><td>8</td><td>19</td><td>.</td><td>2</td><td>73</td><td></td><td></td></tr>
<tr><td>Schleswig-Holstein</td><td>89</td><td>.</td><td>2</td><td>10</td><td>.</td><td>8</td><td>37</td><td></td><td></td><td>84</td><td>.</td><td>8</td><td>15</td><td>.</td><td>2</td><td>33</td><td></td><td></td></tr>
<tr><td>Thueringen</td><td>91</td><td>.</td><td>4</td><td>8</td><td>.</td><td>6</td><td>70</td><td></td><td></td><td>71</td><td>.</td><td>2</td><td>28</td><td>.</td><td>8</td><td>73</td><td></td><td></td></tr>
</table>
```

```r
# Writing into a HTML file ...
write_html(ft,digits=c(1,1,0,1,1,0),show.titles=FALSE,
           file="Voted2013-GenderState.html")
```
Continuing another example:

```r
# age.tab was created earlier
age.ftab <- ftable(age.tab,row.vars=2)
show_html(age.ftab,digits=1,show.titles=FALSE)
```

```{=html}
<style>
      table#ftable-6 {
         border-collapse: collapse; border-style: none; margin: 2ex auto;
      } 
      table#ftable-6 tr {
          border-style: none;
      } 
      table#ftable-6 td {
        padding-top: 3px; 
        padding-bottom: 3px; 
        padding-left: 0.5ex; 
        padding-right: 0.5ex; 
        margin: 0px; 
        margin-top: 0px;
        margin-bottom: 0px;
        border-style: none; 
        border-width: 0; 
      }
table#ftable-6 tr:nth-child(1){
     border-top: 2px solid;
}
table#ftable-6 tr:nth-child(1){
     border-bottom: 1px solid;
}
table#ftable-6 tr:nth-child(17){
     border-bottom: 2px solid;
}
table#ftable-6 td:nth-child(3),
table#ftable-6 td:nth-child(6),
table#ftable-6 td:nth-child(9){
         padding-left: 0px;
         padding-right: 0px;
         text-align: center; 
         width: .2ex;
         /*background-color: blue;*/
        }
table#ftable-6 td:nth-child(2),
table#ftable-6 td:nth-child(5),
table#ftable-6 td:nth-child(8){
         padding-left: 0.5ex;
         padding-right: 0px;
         text-align: right; 
         /*background-color: red;*/
        }
table#ftable-6 td:nth-child(4),
table#ftable-6 td:nth-child(7),
table#ftable-6 td:nth-child(10){
         padding-left: 0px;
         padding-right: 0.5ex;
         text-align: left; 
         /*background-color: green;*/
        } 
table#ftable-6 td:nth-child(n of .header){
         padding-left: 0.5ex;
         padding-right: 0.5ex;
         text-align: center; 
         /*background-color: gray;*/
        } 
</style>
<table class="ftable" id="ftable-6">
<tr><td></td><td colspan="3" class="header">Mean</td><td colspan="3" class="header">Std.dev</td><td colspan="3" class="header">Median</td></tr>
<tr><td>Baden-Wuerttemberg</td><td>54</td><td>.</td><td>5</td><td>18</td><td>.</td><td>9</td><td>57</td><td>.</td><td>0</td></tr>
<tr><td>Bayern</td><td>54</td><td>.</td><td>4</td><td>18</td><td>.</td><td>9</td><td>56</td><td>.</td><td>0</td></tr>
<tr><td>Berlin</td><td>52</td><td>.</td><td>8</td><td>19</td><td>.</td><td>8</td><td>57</td><td>.</td><td>0</td></tr>
<tr><td>Brandenburg</td><td>59</td><td>.</td><td>7</td><td>19</td><td>.</td><td>3</td><td>62</td><td>.</td><td>5</td></tr>
<tr><td>Bremen</td><td>60</td><td>.</td><td>4</td><td>11</td><td>.</td><td>5</td><td>63</td><td>.</td><td>0</td></tr>
<tr><td>Hamburg</td><td>51</td><td>.</td><td>5</td><td>18</td><td>.</td><td>7</td><td>53</td><td>.</td><td>0</td></tr>
<tr><td>Hessen</td><td>56</td><td>.</td><td>9</td><td>18</td><td>.</td><td>5</td><td>60</td><td>.</td><td>0</td></tr>
<tr><td>Mecklenburg-Vorpommern</td><td>57</td><td>.</td><td>0</td><td>19</td><td>.</td><td>2</td><td>60</td><td>.</td><td>5</td></tr>
<tr><td>Niedersachsen</td><td>55</td><td>.</td><td>1</td><td>18</td><td>.</td><td>4</td><td>56</td><td>.</td><td>0</td></tr>
<tr><td>Nordrhein-Westfalen</td><td>53</td><td>.</td><td>9</td><td>19</td><td>.</td><td>1</td><td>55</td><td>.</td><td>0</td></tr>
<tr><td>Rheinland-Pfalz</td><td>57</td><td>.</td><td>2</td><td>18</td><td>.</td><td>2</td><td>60</td><td>.</td><td>5</td></tr>
<tr><td>Saarland</td><td>61</td><td>.</td><td>9</td><td>17</td><td>.</td><td>3</td><td>65</td><td>.</td><td>0</td></tr>
<tr><td>Sachsen</td><td>58</td><td>.</td><td>3</td><td>16</td><td>.</td><td>7</td><td>60</td><td>.</td><td>5</td></tr>
<tr><td>Sachsen-Anhalt</td><td>54</td><td>.</td><td>7</td><td>17</td><td>.</td><td>1</td><td>56</td><td>.</td><td>0</td></tr>
<tr><td>Schleswig-Holstein</td><td>60</td><td>.</td><td>0</td><td>19</td><td>.</td><td>9</td><td>65</td><td>.</td><td>0</td></tr>
<tr><td>Thueringen</td><td>57</td><td>.</td><td>8</td><td>17</td><td>.</td><td>4</td><td>60</td><td>.</td><td>0</td></tr>
</table>
```

Of course we can also export to LaTeX:

```r
toLatex(age.ftab,digits=1,show.titles=FALSE)
```

```
\begin{tabular}{llD{.}{.}{1}D{.}{.}{1}D{.}{.}{1}}
\toprule
 && \multicolumn{1}{c}{Mean}&\multicolumn{1}{c}{Std.dev}&\multicolumn{1}{c}{Median}\\
\midrule
Baden-Wuerttemberg     && 54.5 & 18.9 & 57.0\\
Bayern                 && 54.4 & 18.9 & 56.0\\
Berlin                 && 52.8 & 19.8 & 57.0\\
Brandenburg            && 59.7 & 19.3 & 62.5\\
Bremen                 && 60.4 & 11.5 & 63.0\\
Hamburg                && 51.5 & 18.7 & 53.0\\
Hessen                 && 56.9 & 18.5 & 60.0\\
Mecklenburg-Vorpommern && 57.0 & 19.2 & 60.5\\
Niedersachsen          && 55.1 & 18.4 & 56.0\\
Nordrhein-Westfalen    && 53.9 & 19.1 & 55.0\\
Rheinland-Pfalz        && 57.2 & 18.2 & 60.5\\
Saarland               && 61.9 & 17.3 & 65.0\\
Sachsen                && 58.3 & 16.7 & 60.5\\
Sachsen-Anhalt         && 54.7 & 17.1 & 56.0\\
Schleswig-Holstein     && 60.0 & 19.9 & 65.0\\
Thueringen             && 57.8 & 17.4 & 60.0\\
\bottomrule
\end{tabular}
```



[^1]: The [German Longitudinal Election
Study](https://www.gesis.org/gles/about-gles) is funded by the German National
Science Foundation (DFG) and carried out outin close cooperation with the
[DGfW](https://www.dgfw.info/), German Society for Electoral Studies.  Principal
investigators are Hans Rattinger (University of Mannheim, until 2014), Sigrid
Roßteutscher (University of Frankfurt), Rüdiger Schmitt-Beck (University of
Mannheim), Harald Schoen (Mannheim Centre for European Social Research, from
2015), Bernhard Weßels (Social Science Research Center Berlin), and Christof
Wolf (GESIS – Leibniz Institute for the Social Sciences, since 2012). Neither
the funding organisation nor the principal investigators bear any responsibility
for the example code shown here.

---
title: Importing data from SPSS and Stata
output: rmarkdown::html_vignette
vignette: >
  % \VignetteIndexEntry{Importing data from SPSS and Stata}
  % \VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r,echo=FALSE,message=FALSE}
knitr::opts_chunk$set(comment=NA,eval=FALSE)
```

# Motivation

The \"foreign\" package for *R* already provides facilities to import
data from other statistical software packages such as SPSS or Stata, but
they are limited by the way survey data are generally represented in
*R*. That is, since variables in an R data frame can only be numerical
vectors or factors, any direct translation of SPSS or Stata data sets
into data frames will lead to the loss of information of information,
such as variable labels, variable labels, or user-specified missing
values. (Value labels can be preserved by translating them into factor
levels, but this means losing information about the original codes. It
will also lead to undesired missing values, if variables in the original
data sets are only partially labelled.) The \"memisc\" package for this
reason provides functions that allow to import SPSS or Stata data sets
into objects of the class `"data.set"` defined in it.

# The role of \"importer\" objects

Importing data using the facilities provided by the \"memisc\" package
consists of two steps. In the first step, a description of the data in
the file is collected in an object of class \"importer\". In the second
step, data are imported into \"data.set\" objects with the help of these
\"importer\" objects. These \"importer\" objects contain only meta-data
e.g. about variable labels, value labels, and user-defined missing
values. This allows to get an overview of the structure of the file
without the need of loading the complete data, which is advantageous
esp. if the data set is large. For example, with the help of an
\"importer object\" it is possible to see what the labels of the
variables are so that one can select those variables from the data file
that are actually needed. The data set object in R memory can then
created by \-- if `imprtr` is an importer object \-- by calls like
`subset(imprtr,...)`, `imprtr[...]` or `as.data.set(imprtr)`. Some
examples are given in the following.

Note that these examples require data not included in the package 
(you need to register to [GESIS](https://www.gesis.org) to download the data). 
The vignette code cannot be run without this additional data.

# Importing data from SPSS files

## Importing from a SPSS \"system\" file

In order to import data from an SPSS \"system\" file, the usual binary
format in which SPSS data now is usually saved and often distributed,
one needs to first make the file that contains the data known to R, as
in the following example:

``` r
library(memisc)
ZA5702 <- spss.system.file("Data/ZA5702_v2-0-0.sav")
ZA5702
```

:

    SPSS system file 'Data/ZA5702_v2-0-0.sav' 
        with 979 variables and 3911 observations

Once the \"system file\" is declared using the function
`spss.system.file()`, metadata becomes available, such as the number of
cases and variables (as just seen), the names and labels of the
variables (as seen below):

``` r
description(ZA5702)
```

:

    study    'Studiennummer'                       
    version  'GESIS Archiv Version'                
    year     'Erhebungsjahr'                       
    field    'Erhebungszeitraum'                   
    glescomp 'GLES-Komponente'                     
    survey   'Erhebung/Welle'                      
    lfdn     'Laufende Nummer (Kumulation)'        
    vlfdn    'Laufende Nummer (Vorwahl)'           
    nlfdn    'Laufende Nummer (Nachwahl)'          
    datum    'Datum der Befragung (Monat/Tag/Jahr)'

(Here only an extract of the full output was shown, since the data set
contains as many as 979 variables.)

An \"importer\" object, such as `ZA5702` in this example, would also
allow to obtain a full codebook with

``` r
codebook(ZA5702)
```

but we refrain from showing such a codebook for the obvious reason of
not creating too much output. As the inspection of the data in the file
shows, most variable names have a standardised, yet non-mnemonic
structure. Variables referring to questions asked in the pre-election
wave of the GLES 2013 study have names starting with \"`v`\", those
referring to questions asked in the post-election wave have names
starting with \"`v`\", while those referring to question asked in both
waves have names starting \"`nv`\". For a specific analysis, such
variable names are not very useful. For this reason we want to rename
them. We could do this after loading the data, but it is more convenient
to do the data import and the renaming in one step as in the example
below:

``` r
gles2013work <- subset(ZA5702,
                       select=c(
                         wave                  = survey,
                         intent.turnout        = v10,
                         turnout               = n10,
                         voteint.candidate     = v11aa,
                         voteint.list          = v11ba,
                         postal.vote.candidate = v12aa,
                         postal.vote.list      = v12ba,
                         vote.candidate        = n11aa,
                         vote.list             = n11ba,                 
                         bula                  = bl
                       ))
```

The variable names to the left of the equality sign are the variable
names as they will appear in the data set after import, while the
variable names to the right of the equality aign are the variable names
as they exist in the data file.

As a demonstration of what information can be extracted from the data
file, we create a codebook for one of the items in the data set:

``` r
codebook(gles2013work$turnout)
```

```
================================================================================

   gles2013work$turnout 'Wahlbeteiligung'

--------------------------------------------------------------------------------

   Storage mode: double
   Measurement: interval
   Missing values: -Inf - -1

   Values and labels                      N Valid Total
                                                       
   -99 M 'keine Angabe'                   3         0.1
   -97 M 'trifft nicht zu'               20         0.5
   -94 M 'nicht in Auswahlgesamtheit'  2003        51.2
     1   'ja, habe gewaehlt'           1596  84.7  40.8
     2   'nein, habe nicht gewaehlt'    289  15.3   7.4
                                                       
        Min: 1.000                                     
        Max: 2.000                                     
       Mean: 1.153                                     
   Std.Dev.: 0.360   
```
## Import from a SPSS \"portable\" file

Data from SPSS \"portable\" files are imported in essentially the same
way as data from SPSS \"system\" files: The first step again is to make
the data set known to *R*:

``` r
ZA3861 <- spss.portable.file("Data/ZA3861.por",iconv=FALSE)
ZA3861
```

:

    SPSS portable file 'Data/ZA3861.por' 
        with 331 variables and 3263 observations

Since this file contains German umlauts (in contrast to the previous
example), we need to convert the character coding of the value labels
etc. from \"Latin-1\" (the original coding of the data) into the native
encoding of the system (unless the computer is using natively
\"Latin-1\" encoding and not - as must Mac and most Linux System - a
variant of UTF8).

``` r
ZA3861 <- Iconv(ZA3861,from="latin1")
```

Importer objects created from \"portable\" files can be examined in the
same way as importer objects created from \"system\" files. For example,
we get a description of the variables in the data set (the variable
labels) and a codebook.

``` r
description(ZA3861)
```

:

    vvpnid   'Fallnummer'                                 
    vsplitwo 'West-Ost-Kennung'                           
    vvornach 'Vor-/Nachwahl'                              
    vland    'Bundesland'                                 
    v10      'Wirtschaftl. Lage allgemein'                
    v20      'Wirtschaftl. Lage retrospektiv'             
    v30      'Wirtschaftl. Lage prospektiv'               
    v31      'Wichtigkeit Erst/Zweitstimme BTW (nicht 94)'
    v40      'Demokratiezufriedenheit'                    
    v50      'Staerke Politikinteresse'                   

To actually import the data and make them accessible for analysis we can
(as above), use `as.data.set()`, or `subset()` as in this example:

``` r
work2002 <- subset(ZA3861,
    select=c(
          respid            = VVPNID,
          split.wo          = VSPLITWO,
          split.vor.nach    = VVORNACH,
          Bundesland        = VLAND,
          Erststimme        = V69,
          Zweitstimme       = V70,
          Geschlecht        = VSEX,
          GebMonat          = VMONAT,
          GebJahr           = VJAHR,
          Konfession        = VRELIG,
          Kirchgang         = VKIRCHG,
          Erwerbst          = VBERUFTG,
          FrErwerbst        = VFRBERTG,
          Beruf             = VBERUF,
          Famstand          = VFAMSTDN,
          Partner           = VPARTNER,
          BildungP          = VPBILDGA,
          BerufstP          = VPBERUFT,
          FrBerufstP        = VPFBERTG,
          BerufP            = VPBERUF,
          ReprGewicht       = VGVWNW
        )
    )
```

## Import from a fixed-width file accompanied by SPSS syntax

Data from more recent study components of the American National Elecion
Study comes in fixed-width format, with some additional SPSS syntax
files that define columns, variable labels, value labels, and missing
values. `memisc` also provides an importer function such data. Naturally
this requires a little bit more information. In addition to the actual
data file, we also need a file with SPSS syntax specifying the data
columns. Optionally, Syntax files that define variable labels, value
lables, and missing values can also be specified.

``` r
anes2008TS <- spss.fixed.file("Data/anes2008/anes2008TS_dat.txt",
                              columns.file="Data/anes2008/anes2008TS_col.sps",
                              varlab.file="Data/anes2008/anes2008TS_lab.sps",
                              codes.file="Data/anes2008/anes2008TS_cod.sps",
                              missval.file="Data/anes2008/anes2008TS_md.sps")
anes2008TS
```

:

    SPSS fixed column file 'issues/anes2008/anes2008TS_dat.txt' 
        with 1954 variables and 2322 observations
        with variable labels from file 'issues/anes2008/anes2008TS_lab.sps' 
        with value labels from file 'issues/anes2008/anes2008TS_cod.sps' 
        with missing value definitions from file 'issues/anes2008/anes2008TS_md.sps' 

Further information about the data can now be obtained from the returned
importer object in the same way as from importer objects that describe
SPSS \"system\" or SPSS \"portable\" files. That is, we can use
`names()`, `description()`, and `codebook()`. To get the data in to the
memory of *R* we can use (as above) the functions `as.data.set()` and
`subset()`.

## Importing data from a Stata file

Data from Stata files (up to Stata Version 12) can be imported in the
same way as data from SPSS files. The main difference is the function
used for it, and the fact that user-defined missing values do not exists
in Stata. For this, see the following example:

``` r
library(memisc)
ZA5702.dta <- Stata.file("Data/ZA5702_v2-0-0.dta")
ZA5702.dta
```

:

    Stata file 'Data/ZA5702_v2-0-0.dta' 
        with 874 variables and 3911 observations

``` r
gles2013work.dta <- subset(ZA5702.dta,
                       select=c(
                         wave                  = survey,
                         intent.turnout        = v10,
                         turnout               = n10,
                         voteint.candidate     = v11aa,
                         voteint.list          = v11ba,
                         postal.vote.candidate = v12aa,
                         postal.vote.list      = v12ba,
                         vote.candidate        = n11aa,
                         vote.list             = n11ba,                 
                         bula                  = bl
                       ))
codebook(gles2013work.dta$turnout)
```

```
================================================================================

   gles2013work.dta$turnout 'Wahlbeteiligung'

--------------------------------------------------------------------------------

   Storage mode: integer
   Measurement: nominal
   Missing values: 100 - 127

   Values and labels                               N Percent
                                                            
   -99   'keine Angabe'                            3     0.1
   -98   'weiss nicht'                             0     0.0
   -97   'trifft nicht zu'                        20     0.5
   -96   'Split'                                   0     0.0
   -95   'nicht teilgenommen'                      0     0.0
   -94   'nicht in Auswahlgesamtheit'           2003    51.2
   -93   'Interview abgebrochen'                   0     0.0
   -92   'Fehler in Daten'                         0     0.0
   -86   'nicht wahlberechtigt'                    0     0.0
   -85   'nicht waehlen'                           0     0.0
   -84   'keine Erst-/Zweitstimme abgegeben'       0     0.0
   -83   'ungueltig waehlen'                       0     0.0
   -82   'keine andere Partei waehlen'             0     0.0
   -81   'noch nicht entschieden'                  0     0.0
   -72   'nicht einzuschaetzen'                    0     0.0
   -71   'nicht bekannt'                           0     0.0
     1   'ja, habe gewaehlt'                    1596    40.8
     2   'nein, habe nicht gewaehlt'             289     7.4
```

% \VignetteIndexEntry{Handling questionnaire items with "memisc"}
% \VignetteEngine{knitr::rmarkdown}
% \VignetteEncoding{UTF-8}

```{r,echo=FALSE,message=FALSE}
knitr::opts_chunk$set(comment=NA,
               fig.align="center",
               results="markup")
```
--- 
title:  Handling questionnaire items with 'memisc'
---

# Handling questionnaire items with *memisc*

## Motivation

R is a great tool to do data analysis and for data management tasks that arise in the context of big data analytics. Nevertheless there is still room for improvement in terms of the support for data management tasks that arise in the social sciences, especially when it comes to handling data that come from social surveys and opinion surveys. The main reason for this is that the way that questionnaire item responses as they are usually coded in machine-readable survey data sets do not directly and easily translate into R's data types for numeric and categorical data, that is, numerical vectors and factors. As a consequence, many social scientists exercise their everyday data management tasks with commercial software packages such as SPSS or Stata, but there may be social scientists who either cannot afford such commercial software or prefer to use, out of principle, open-source software for all steps of data management and analysis.

It is one of the aim of the "memisc" package to provide a bridge between social science data sets of variables that contain coded responses to questionnaire items, with their typical structures involving labelled numeric response codes and numeric codes declared as "missing values". As an illustrative example, suppose in a pre-election survey, respondents are asked about which party they are going to vote for in their constituency in the framework of a first-past-the-post electoral system. Suppose the response categories offered to the respondents are "Conservative", "Labour", "Liberal Democrat", "Other party".[^1] A survey agency that actually conducts the interviews with a sample of voters may, according to common practice, use the following codes to collect the responses to the question about the vote intention:

<div class="booktable">

Response category     Code        
-------------------- ----- ----- 
Conservative            1      
Labour                  2      
Liberal Democrat        3      
Other Party             4      
Will not vote           9      
Don't know             97   (M)
Answer refused         98   (M)
Not applicable         99   (M)

</div>

In data sets that contain the results of such coding are essentially numeric data -- with some additional information about the "value labels" (the labels attached to the numeric
values) and about the "missing values" (those numeric values that indicate responses that one usually does not want to include into statistical analysis). While this coding frame for responses to survey questionnaires is far from uncommon in the social sciences, it is not straightforward to retain this information in *R* objects. Here there are two main alternatives, (1) one could store the responses as a numeric vector, thereby losing the information about the labelled values, or (2) one could store the responses as a factor, thereby losing the information contained in the codes. Either way, one will lose the information about the "missing values". Of course, one can filter out these missing values before 
data analysis by replacing them with `NA`, but it would convenient to have facilities that do that automatically.

[^1]: Those familiar with British politics will realise that this is a simplification of the menu of available choices that voters in England typically face in an election of the House of Commons.

## Standard attributes of survey items

The "memisc" package introduces a new data type (more correctly an `S4` class) that allows to handle such data, that allows to adjust labels or missing values definitions and to translate such data as needed either into numeric vectors 
of factors, thereby automatically filtering out the missing values. This data time (or `S4` class) is, for lack of a better term, called `"item"`. In general, users do not bother with the construction of such item vectors. Usually they are  generated when data sets are imported from data files in SPSS or Stata format. This page is mainly concerned with describing the structure of such item vectors and how they can be manipulated in the data management step that usually precedes data analysis. It is thus possible to do all the data management in *R* from importing the pristine data obtained from data archives or other data providers, such as the survey institutes to which a principal investigator has delegated data collection. Of course, the facilities introduced by the `"item"` data type also allow to create appropriate representations of survey item responses if a principal investigator obtains only raw numeric codes. In the following, the construction of `"item"` vectors from raw numeric data is mainly used to highlight their structure.

### Value labels {#value_labels}

```{r,echo=FALSE}
set.seed(20)
vote.probs <- c(.361,.29,.23,1-.361-.29-.23)
decis.probs <- c(vote.probs*.651,1-.651)
all.probs <- c(.9*decis.probs,.1*rep(1/3,3))
voteint <- sample(c(1:4,9,97,98,99),
                  prob=all.probs,
                  replace=TRUE,
                  size=200)                
library(memisc)
```
Suppose a numeric vector of responses to the question about their vote intention coded using
the coding frame shown above looks as follows
```{r}
voteint
```

This numeric vector is transformed into an `"item"` vector by attaching labels to the codes. The *R* code to attach labels that reflect the coding frame shown above may look like follows (if formatted nicely):
```{r}
# This is to be run *after* memisc has been loaded.
labels(voteint) <- c(Conservative       =  1,
                     Labour             =  2,
                     "Liberal Democrat" =  3, # We have whitespace in the label, 
                     "Other Party"      =  4, # so we need quotation marks
                     "Will not vote"    =  9,
                     "Don't know"       = 97,
                     "Answer refused"   = 98,
                     "Not applicable"   = 99)
```
`voteint` is now an item vector, for which a particular `"show"` method is defined:
```{r}
class(voteint)
str(voteint)
voteint
```
Like with factors, if *R* shows the contents of the vector, the labels are shown (instead of the codes). Since item vectors typically are quite long, because they come from interviewing a survey sample and usual survey sample sizes are about 2000, we usually do not want to see all the values in the vector. `"memisc"` anticipates this and shows at most a single line of output. (In the output, also the "level of measurement" is shown, which at this point does not have a consquence. It will become clear later what the implications of the "level of measurement" are.)

In line with the usual semantics `labels(voteint)` will now show us a description of the labels and to which values they are assigned:
```{r}
labels(voteint)
```
Now if we rather want shorter labels, we can change them either by something like `labels(voteint) <- ...` or by changing the labels using `relabel()`:
```{r}
voteint <- relabel(voteint,
                   "Conservative"     = "Cons",
                   "Labour"           = "Lab",
                   "Liberal Democrat" = "LibDem",
                   "Other Party"      = "Other",
                   "Will not vote"    = "NoVote",
                   "Don't know"       = "DK",
                   "Answer refused"   = "Refused",
                   "Not applicable"   = "N.a.")
```
Let us take a look at the result:
```{r}
labels(voteint)
voteint
str(voteint)
```

### Missing values {#missing_values}

In the coding plan shown above, the values 97, 98, and 99 are marked as "missing values", that is, while they represent coded responses, they are not to be considered as valid in the sense of providing information about the respondent's vote intention. For the statistical analysis of vote intention it is natural to replace them by `NA`. Yet replacing codes 97, 98, and 99 already at the stage of importing data into *R* memory would mean a loss of potentially precious information since it precludes, e.g. the motivation to refuse responding to the vote intention question or the antencedents of undecidedness. Hence it is better to mark those values and to delay their replacement by `NA` to a later stage in the analysis of vote intentions and to be able to undo or change the "missingness" of these values. For example, not only may one be interested in the antecedents of response refusals but also be interested to analyse vote intention with non-voting excluded or included. The memisc package provides, like SPSS and PSPP, facilities to mark particular values of an item vector as "missing" and change such designations throughout the data preperation stage.

There are several ways with `"memisc"` to make distinctions between valid and missing values. The first way that mirrors the way it is done in SPSS. To illustrate this we return to the fictitious vote intention example. The values 97,98,99 of `voteint` are designated as "missing" by
```{r}
missing.values(voteint) <- c(97,98,99)
```
The missing values are reflected in the output of `voteint`, (labels of) missing values are marked with `*` in the output:
```{r}
voteint
```
It is also possible to extend the set of missing values: We add another value to the set of missing values.
```{r}
missing.values(voteint) <- missing.values(voteint) + 9
```
The missing values can be recalled as usual:
```{r}
missing.values(voteint)
```
The missing values are turned into `NA` if `voteint` is coerced into a numeric vector or a factor, which is what usually happens before the eventual statistical analysis:
```{r}
as.numeric(voteint)[1:30]
as.factor(voteint)[1:30]
```
It is also possible to drop all missing value designations:
```{r}
missing.values(voteint) <- NULL
missing.values(voteint)
as.numeric(voteint)[1:30]
```
In contrast to SPSS it is possible with `"memisc"` to designate the valid, i.e. non-missing values:
```{r}
valid.values(voteint) <- 1:4
valid.values(voteint)
missing.values(voteint)
```
Instead of individual valid or missing values it is also possible to define a *range* of values as valid:
```{r}
valid.range(voteint) <- c(1,9)
missing.values(voteint)
```

### Other attributes of survey items {#annotations}
Other software packages targeted at social scientists also allow to add annotations to the variables in a data set, which are not subject to the syntactic constraints of variable names. These annotations are usually called "variable labels" in these software packages. In `"memisc"` the corresponding term is "description". In continuation of the running example, we add a description to the vote intention variable:
```{r}
description(voteint) <- "Vote intention"
description(voteint)
```
In contrast to other software, `"memisc"` allows to attach arbitrarily annotation to survey items, such as the wording of a survey question:
```{r}
wording(voteint) <- "Which party are you going to vote for in the general election next Tuesday?"
wording(voteint)
annotation(voteint)
annotation(voteint)["wording"]
```

### Codebooks of survey items

It is common in survey research to describe a data set in the form of a *codebook*. A codebook summarises each variable in the data set in terms of its relevant attributes, that is, the label attached to the variable (in the context of the `memisc` package this is called its "description"), the labels attached to the values of the variable, which values of the variable are supposed to be *missing* or *valid*, as well as univariate summary statistics of each variable, usually without and with missing variables included. Such functionality is provided in this package by the function `codebook()`. `codebook()` when applied to an `"item"` object returns a `"codebook"` object, which when printed to the console gives an overview of the variable usually required for the codebook of a data set (the production of codebooks for whole data sets is described further below). To illustrate the `codebook()` function we now produce a codebook of the `voteint` item variable created above:
```{r}
codebook(voteint)
```
As can be seen in the output, the `codebook()` function reports the name of the variable, the description (if defined for the variable), and the question wording (again if defined). Further it reports the storage mode (which is use by *R*), the level of measurement ("nominal", "ordinal", "interval", or "ratio") and the range of valid values (or alternatively, individually defined valid values, individually defined missing values, or ranges of missing values). For item variables with value labels, it shows a table of frequencies of the labelled values, and the percentages of valid values and all values with missings included. 

Codebooks are particularly useful to find "wild codes", that is codes that are not labelled, and usually produced by coding errors. Such coding errors may be less common in data sets produced by CAPI or CATI or online surveys, but they may occur in older data sets from before the age of computer-assisted interviewing and also during the course of data management. This use of codebooks is demonstrated in the following by deliberatly adding some coding errors into a copy of our `voteint` variable:
```{r}
voteint1 <- voteint
voteint1[sample(length(voteint),size=20)] <- c(rep(5,13),rep(7,7))
```
The presence of these "wild codes" can now be spotted using `codebook()`:
```{r}
codebook(voteint1)
```
The output shows that 20 observations contain wild codes in this variable. Why don't we get a list of wild codes as part of the codebook? The reason is that codebook is supposed also to work with continuous variables that have thousands of unique, unlabelled values. Users certainly will not like to see them all as part of a codebook. 

In order to get a list of wild codes the development version of "memisc" contains the function `wild.codes()`, which we apply to the variable `voteint1`
```{r}
wild.codes(voteint1)
```
We see that 6.5 and 3.5 percent of the observations have the wild codes 5 and 7.


To see how `codebook()` works with variables without value labels, we create an unlabelled copy of our `voteint` variable:
```{r}
voteint2 <- voteint
labels(voteint2) <- NULL # This deletes all value labels
codebook(voteint2)
```
Usually, variables without labelled values represent measures on an interval or ratio scale. In that case, we do not want to see how many unlabelled values there are, but we want to get some other statistics, such as mean, variance, etc. To this purpose, we decleare the variable `voteint2` to have an interval-scale level of measurement.[^2]
```{r}
measurement(voteint2) <- "interval"
codebook(voteint2)
```
[^2]: Of course, substantially it does not make sense at all to form averages etc. of voting choices, so "do not try this at home". This example is merely to demonstrate codebooks and the setting of scale-levels.

For convenience of including them into word-processor documents, there is also the possibility to export codebooks
into HTML:
```{r,results='asis'}
show_html(codebook(voteint))
```

## Data sets: Containers of survey items

Usually one expects to be able handle data on responses to survey items not in isolation, but as part of a data set,
which contains a multitude of observations on many variables. The usual data structure in *R* to contain observation-on-variables
data is the *data frame*. In principle it is possible to put survey item vectors as described above into a data frame, nevertheless
the `"memisc"` package provides a special data structure to contain survey item data called data sets or data set-objects, that is, objects of class `"data.set"`. This opens up the possibility to automatically translate survey items into regular vectors
and factors, as expected by typical data analysis functions, such as `lm()` or `glm()`.

### The structure of `"data.set"` objects

Data set objects have essentially the same row-by-column structure as data frames: 
They are a set of vectors (however of class `"item"`)
all with the same length, so that in each row of the data set there are values in these vectors.
Observations can be addressed as rows of a `"data.set"` and variabels can be addressed as columns,
just as one may used to with regards to data frames. Most data management operations that you
can do with data frames can also be done with data sets (such as merging them or using the 
functions `with()` or `within()`). Yet in contrast to data frames, data sets are always expected
to contain objects of class `"item"`, and any vectors or factors from which a `"data.set"` object
is constructed are changed into `"item"` objects. 

Another difference is the way that `"data.set"` objects are shown on the console. As `S4` objects,
if a user types in the name of a `"data.set"` objects, the function `show()` (and not `print()`) is
applied to it. The `show()`-method for data set objects is defined in such a way that only the first few
observations of the first few variables are shown on the console -- in contrast to `print()` as applied
to a data frame, which shows *all* observations on *all* variables. While it may be intuitive and convenient
to be shown all observations in a small data frame, this is not what you will want if your data set contains
more than 2000 observations on several hundred variables, the dimensions that typical social science data
sets have that you can download from data archives such as that of ICPSR or GESIS.

The main facilitites of `"data.set"` objects are demonstrated in what follows.
First, we create a data set with fictional survey responses
```{r}
Data <- data.set(
          vote = sample(c(1,2,3,4,8,9,97,99),
                        size=300,replace=TRUE),
          region = sample(c(rep(1,3),rep(2,2),3,99),
                          size=300,replace=TRUE),
          income = round(exp(rnorm(300,sd=.7))*2000)
          )
```
Then, we take a look at this already sizeable `"data.set"`" object:
```{r}
Data
```
In this case, our data set has only three variables, all of which are shown, but of the observations we
see only the first 25. Actually the number of observations shown can be determined by the option `"show.max.obs"` which defaults to 25, but can be changed as convenient:
```{r}
options(show.max.obs=5)
Data
# Back to the default
options(show.max.obs=25)
```
If you *really* want to see the complete data on your console, then you can use `print()` instead:
```{r,eval=FALSE}
print(Data)
```
but you should not do this with large data sets, such as the Eurobarometer trend file ...

### Manipulating data in data sets

Typical data management tasks that you would otherwise have done in commercial packages like SPSS or
Stata can be conducted within data set objects. Actually to provide this possibility (to the author
of the package) was the main reason that the `"memisc"` package was created.
To demonstrate this, we continue with our fictional data which we prepare for further analysis:
```{r}
Data <- within(Data,{
  description(vote) <- "Vote intention"
  description(region) <- "Region of residence"
  description(income) <- "Household income"
  wording(vote) <- "If a general election would take place next Tuesday,
                    the candidate of which party would you vote for?"
  wording(income) <- "All things taken into account, how much do all
                    household members earn in sum?"
  foreach(x=c(vote,region),{
    measurement(x) <- "nominal"
    })
  measurement(income) <- "ratio"
  labels(vote) <- c(
                    Conservatives         =  1,
                    Labour                =  2,
                    "Liberal Democrats"   =  3,
                    "Other"               =  4,
                    "Don't know"          =  8,
                    "Answer refused"      =  9,
                    "Not applicable"      = 97,
                    "Not asked in survey" = 99)
  labels(region) <- c(
                    England               =  1,
                    Scotland              =  2,
                    Wales                 =  3,
                    "Not applicable"      = 97,
                    "Not asked in survey" = 99)
  foreach(x=c(vote,region,income),{
    annotation(x)["Remark"] <- "This is not a real survey item, of course ..."
    })
  missing.values(vote) <- c(8,9,97,99)
  missing.values(region) <- c(97,99)

  # These to variables do not appear in the
  # the resulting data set, since they have the wrong length.
  junk1 <- 1:5
  junk2 <- matrix(5,4,4)
  
})
```
Now that we have added information to the data set that reflects the code plan of 
the variables, we take a look how the it looks like:
```{r}
Data
```
As you can see, labelled item look a bit like factors, but with a difference: User-defined missing values
are marked with an asterisk.

Subsetting a data set object works as expected:
```{r}
EnglandData <- subset(Data,region == "England")
EnglandData
```


### Codebooks of data sets {#codebooks}

Previouly, we created a code book for individual survey items. But it is also possible to create a codebook for a whole
data set (what one usually wants to have a codebook of). Obtaining a codebook is simple, by applying the function `codebook()`
to the data frame:
```{r}
codebook(Data)
```
On a website, it looks better in HTML:
```{r,results='asis'}
show_html(codebook(Data))
```

### Translating data sets into data frames

The punchline of the existence of `"data.set"` objects however
is that they can be coerced into regular data frames, using `as.data.frame()`, which causes
survey items to be translated into regular numeric vectors or factors
using `as.numeric()`, `as.factor()` or `as.ordered()` as above, and pre-determined missing values
changed into `NA`. Whether a survey item is changed into a numerical vector, an unordered or an 
ordered factor depends on the declared measurement level (which can be manipulated by
`measurement()` as shown above).

In the example developed so far, the variables `vote` and `region` are declared to have a nominal level
of measurement, while `income` is declared to have a ratio scale. That is, in statistical analyses,
we want the first two variables to be handled as (unordered) factors, and the income variable as a numerical
vector. In addition, we want all the user-declared missing values to be changed into `NA` so that observations
where respondents stated to "don't know" what they are goint go vote for are excluded from the analysis. So let's see 
whether this works - we coerce our data set into a data frame:
```{r}
DataFr <- as.data.frame(Data)
## Looking a the data frame structure
str(DataFr)
## Looking at the first 25 observations
DataFr[1:25,]
```
Indeed the translation works as expected, so we can use it for statistical analysis, here a simple
cross tab:
```{r}
xtabs(~vote+region,data=DataFr)
```
In fact, since many functions such as `xtabs()`, `lm()`, `glm()`, etc. coerce theire `data=` argument into
a data frame, an explicit coercion with `as.data.frame()` is not always needed:
```{r}
xtabs(~vote+region,data=Data)
```
Sometimes we do want missing values to be included, and this is possible too:
```{r}
xtabs(~vote+region,data=within(Data, 
                               vote <- include.missings(vote)))
```

For convenience, there is also a codebook method for data frames:
```{r,results='asis'}
show_html(codebook(DataFr))
```

## More tools for data preparation

When social scientists work with survey data, these are not always organised and coded in a way that 
suits the intended data analysis. For this reasons, the `"memisc"` package provides the two functions
`recode()` and `cases()`. The former is -- as the name suggests -- for recoding, while the second allows
for complex distinctions of cases and can be seen as a more general version of `ifelse()`. These two
functions are demonstrated with a "real-life" example.

### Recoding {#recode}

The function `recode()` is similar in semantics to the function of the same name in 
package [`"car"`][CAR] and designed in such a way that it does not conflict with this function.
In fact, if `recode()` is called in the way as expected in package `"car"`, it will dispatch processing
to this function. In other words, users of this other package may use `recode()` as they are used to.
The version of the `recode()` function provided by `"memisc"` differs from the `"car"` version in so far
as its syntax is more R-ish (or so I believe).

[CAR]: https://cran.r-project.org/package=car

Here we load an example data set -- a subset of the German Longitudinal Election Study for 2013[^3] -- into
R's memory.
```{r}
load(system.file("gles/gles2013work.RData",package="memisc"))
```
As a simple example for the use of `recode()` we use this function to recode German Bundesländer into
an item with two values or East and West Germany. But first we create a codebook for the variable that
contains the Bundesländer codes:
```{r,results='asis'}
with(gles2013work,
     show_html(codebook(bula)))
```
We now recode the Bundesländer codes into a new variable:
```{r}
gles2013work <- within(gles2013work,
                       east.west <- recode(bula,
                                          East = 1 <- c(3,4,8,13,14,16),
                                          West = 2 <- c(1,2,5:7,9:12,15)
                                          ))
```
and check whether this was successful:
```{r}
xtabs(~bula+east.west,data=gles2013work)
```
as can be seen, `recode()` was called in such a way that not only old
codes are transferred into new ones, but also the new codes are labelled.


[^3]: The [German Longitudinal Election Study](https://www.gesis.org/gles/about-gles) is funded by the German National Science Foundation (DFG) and carried out outin close cooperation with the [DGfW](https://www.dgfw.info/), German Society for Electoral Studies.
Principal investigators are Hans Rattinger (University of Mannheim, until 2014), Sigrid Roßteutscher (University of Frankfurt), Rüdiger Schmitt-Beck (University of Mannheim), Harald Schoen (Mannheim Centre for European Social Research, from 2015), Bernhard Weßels (Social Science Research Center Berlin), and Christof Wolf (GESIS – Leibniz Institute for the Social Sciences, since 2012). Neither the funding organisation nor the principal investigators bear any responsibility for the example code shown here.

### Case distinctions 

Recoding can be used to combine the codes of an item into a smaller set, but sometimes one needs to do more
complex data preparations, in which the values of some variable are set conditional on values of another one, etc.
For such tasks, the `"memisc"` package provides the function `cases()`. This function takes several expressions
that evaluate to logical vectors as arguments and returns a numeric vector or a factor, the values or level of which indicate for each observation which of the expressions evaluates to `TRUE` the respective observation. The factor levels are named after the logical expressions. A simple example looks thus:
```{r}
x <- 1:10
xc <- cases(x <= 3, 
            x > 3 & x <= 7, 
            x > 7)
data.frame(x,xc)
```
In this example `cases()` returns a factor. It can also be made to return a numeric value:
```{r}
xn <- cases(1 <- x <= 3, 
            2 <- x > 3 & x <= 7, 
            3 <- x > 7)
data.frame(x,xn)
```
This example shows the way `cases()` works in the abstract. How this can be made used of in practical example is best
demonstrated by a real-world example, again using data from the German Longitudinal Election Study.

In the 2013 election module, the intention to vote during the pre-election of respondents interviewed in the pre-election
wave (`wave==1`) and the participation in the election of respondents interviewed in the post-election wave
(`wave==2`) are recorded in
different data set variables, named here `intent.turnout` and `turnout`. 
The variable `intent.voteint` has codes for whether the respondents were sure to participate (1),
were likely to participate (2), were undecided (3), likely not to (4), sure not to participate (5), 
or whether they have cast a postal vote (6). Variable `turnout` has codes for those who participated
in the election (1) or did not (2).

The intention for the candidate vote is recorded in variable `voteint.candidate` and the 
intention for the list vote is recoded in variable `voteint.list` for the pre-election wave.
A postal vote for party candidate is recorded in variable `postal.vote.candidate`
and for a party list is in variable `postal.vote.list`. Recalled votes in the post-election
wave are recorded in variables `vote.candidate` and `vote.list`. 

These various variables
are combined into two variables that has valid values for both waves, `candidate.vote` and
`list.vote`. For this, several conditions have to be handled: whether a respondent is in 
the pre-election or the post-election wave, whether s/he is not likely or sure not to vote, or
whether she has cast a postal vote. Thus the variable `cases()` is helpful here:
```{r}
gles2013work <- within(gles2013work,{

  candidate.vote <- cases(
              wave == 1 & intent.turnout == 6 -> postal.vote.candidate,
              wave == 1 & intent.turnout %in% 4:5 -> 900,
              wave == 1 & intent.turnout %in% 1:3 -> voteint.candidate,
              wave == 2 & turnout == 1 -> vote.candidate,
              wave == 2 & turnout == 2 -> 900
            )

  list.vote <- cases(
              wave == 1 & intent.turnout == 6 -> postal.vote.list,
              wave == 1 & intent.turnout %in% 4:5 -> 900,
              wave == 1 & intent.turnout %in% 1:3 -> voteint.list,
              wave == 2 & turnout ==1 -> vote.list,
              wave == 2 & turnout ==2 -> 900
            )
})
```
The code shown above does the following: In the pre-election wave (`wave == 1`),
the `candidate.vote` variable receives the value of the postal vote variable `postal.vote.candidate`
if a postal vote was cast (`intent.turnout == 6`),
it receives the value `900` for those respondents who where likely or sure not to vote 
(`intent.turnout %in% 4:5`),
and the value of the variable `voteint.candidate` for all others (`intent.turnout %in% 1:3`).
In the post-election wave (`wave == 2`) variable `candidate.vote` receives the value of
variable `vote.candidate` if the respondent has voted (`turnout == 1`) or the value `900`
if s/he has not voted (`turnout == 2`).
The variable `list.vote` is constructed in an analogous manner from the variables
`wave`, `intent.turnout`, `turnout`, `postal.vote.list`, `voteint.list` and `vote.list`.
After the constructin, the resulting variables `candidate.vote` and `list.vote` are labelled
and missing values are declared:
```{r}
gles2013work <- within(gles2013work,{
  candidate.vote <- recode(as.item(candidate.vote),
                      "CDU/CSU"   =  1 <- 1,
                      "SPD"       =  2 <- 4,
                      "FDP"       =  3 <- 5,
                      "Grüne"     =  4 <- 6,
                      "Linke"     =  5 <- 7,
                      "NPD"       =  6 <- 206,
                      "Piraten"   =  7 <- 215,
                      "AfD"       =  8 <- 322,
                      "Other"     = 10 <- 801,
                      "No Vote"   = 90 <- 900,
                      "WN"        = 98 <- -98,
                      "KA"        = 99 <- -99
                  )
  list.vote <- recode(as.item(list.vote),
                      "CDU/CSU"   =  1 <- 1,
                      "SPD"       =  2 <- 4,
                      "FDP"       =  3 <- 5,
                      "Grüne"     =  4 <- 6,
                      "Linke"     =  5 <- 7,
                      "NPD"       =  6 <- 206,
                      "Piraten"   =  7 <- 215,
                      "AfD"       =  8 <- 322,
                      "Other"     = 10 <- 801,
                      "No Vote"   = 90 <- 900,
                      "WN"        = 98 <- -98,
                      "KA"        = 99 <- -99
                  )
  
   missing.values(candidate.vote) <- 98:99
   missing.values(list.vote) <- 98:99
   measurement(candidate.vote) <- "nominal"
   measurement(list.vote) <- "nominal"
})
```

Finally, we can get a cross-tabulation of list votes and the East-West factor and
a cross tabulation of candidate votes against list votes:
```{r,width=120}
xtabs(~list.vote+east.west,data=gles2013work)
xtabs(~list.vote+candidate.vote,data=gles2013work)
```

\name{percent}
\alias{percent}
\alias{percent.default}
\alias{percent.logical}
\title{Table of Percentages with Percentage Base}
\description{
  \code{percent} returns a table of percentages along with
  the percentage base. It will be useful
  in conjunction with \code{\link{Aggregate}} or \code{\link{genTable}}.
}
\usage{
  percent(x,\dots)
  \method{percent}{default}(x,weights=NULL,total=!(se || ci),
      se=FALSE,ci=FALSE,ci.level=.95,
      total.name="N",perc.label="Percentage",\dots)
  \method{percent}{logical}(x,weights=NULL,total=!(se || ci),
      se=FALSE,ci=FALSE,ci.level=.95,
      total.name="N",perc.label="Percentage",\dots)
}
\arguments{
  \item{x}{a numeric vector or factor.}
  \item{weights}{a optional numeric vector of weights of the same length as \code{x}.}
  \item{total}{logical; should the total sum of counts from which the percentages are
    computed be included into the output?}
  \item{se}{logical; should standard errors of the percentages be included?}
  \item{ci}{logical; should confidence intervals of the percentages be included?}
  \item{ci.level}{numeric; nominal coverage of confidence intervals}
  \item{total.name}{character; name given for the total sum of counts}
  \item{perc.label}{character; label given for the percentages if the
    table has more than one dimensions, e.g. if \code{se} or \code{ci} is TRUE.}
  \item{\dots}{for \code{percent.mresp}: one or several 1-0 vectors or matrices
  otherwise, further arguments, currently ignored.}
}

\value{
  A table of percentages.
}

\examples{

x <- rnorm(100)
y <- rnorm(100)
z <- rnorm(100)
f <- sample(1:3,100,replace=TRUE)
f <- factor(f,labels=c("a","b","c"))


percent(x>0)
percent(f)

genTable(
  cbind(percent(x>0),
        percent(y>0),
        percent(z>0)) ~ f
  )

gt <- genTable(
  cbind("x > 0" = percent(x>0,ci=TRUE),
        "y > 0" = percent(y>0,ci=TRUE),
        "z > 0" = percent(z>0,ci=TRUE)) ~ f
  )

ftable(gt,row.vars=3:2,col.vars=1)

ex.data <- expand.grid(mean=c(0,25,50),sd=c(1,10,100))[rep(1:9,rep(250,9)),]
ex.data <- within(ex.data,x <- rnorm(n=nrow(ex.data),mean=ex.data$mean,sd=ex.data$sd))
ex.data <- within(ex.data,x.grp <- cases( x < 0,
                                            x >= 0 & x < 50,
                                            x >= 50 & x < 100,
                                            x >= 100
                                          ))
genTable(percent(x.grp)~mean+sd,data=ex.data)

Aggregate(percent(Admit,weight=Freq)~Gender+Dept,data=UCBAdmissions)
}
\keyword{univar}

\name{reversed}
\alias{reversed}
\alias{reversed,item.vector-method}
\alias{reversed,factor-method}
\title{Reverse the codes of a survey item or the levels of a factor}
\description{
The function \code{reversed()} returns a copy of its argument with codes
or levels in reverse order.
}
\usage{
reversed(x)
\S4method{reversed}{item.vector}(x)
\S4method{reversed}{factor}(x)
}
\arguments{
  \item{x}{An object -- an "item" object or a "data.set" object}
}
\value{
  If the argument of the function \code{reversed()} than either the
  unique valid values or the labelled valid values recoded into the
  reverse order.

  If th argument is a factor than the function returns the factor with
  levels in reverse order.
}
\examples{
ds <- data.set(
    x = as.item(sample(c(1:3,9),100,replace=TRUE),
                labels=c("One"=1,
                         "Two"=2,
                         "Three"=3,
                         "Missing"=9)))
df <- as.data.frame(ds)
ds <- within(ds,{
    xr <- reversed(x)
})
codebook(ds)
df <- within(df,{
    xr <- reversed(x)
})
codebook(df)
}

\name{tibbles}
\alias{as_tibble.data.set}
\alias{as.item,labelled-method}
\alias{as.item,haven_labelled-method}
\alias{as.item,haven_labelled_spss-method}
\alias{as_haven}
\alias{as_haven,data.set-method}
\alias{as_haven,item.vector-method}
\alias{as.data.set,tbl_df-method}
\title{Interface to Packages 'tibble' and 'haven'}
\description{
  A \code{as_tibble} method (\code{as_table.data.set}) allows to transform \code{"data.set"} objects
  into objects of class \code{"tbl_df"} as defined by the package
  "tibble".

  \code{as.item} methods for objects of classes \code{"haven_labelled"}
  and \code{"have_labelled_spss"} allow to transform a "tibble" imported
  using \code{read_dta}, \code{read_spss}, etc. from the package "haven"
  into an object of class \code{"data.set"}.

  \code{as_haven} can be used to transform \code{"data.set"} objects
  into objects of class \code{"tbl_df"} with that additional information
  that objects imported using the "haven" package usually have, i.e.
  variable labels and value labels (as the \code{"label"} and
  \code{"labels"} attributes of the columns).
}
\usage{
as_tibble.data.set(x,\dots)
\S4method{as.item}{haven_labelled}(x,\dots)
\S4method{as.item}{haven_labelled_spss}(x,\dots)
as_haven(x,\dots)
\S4method{as_haven}{data.set}(x,user_na=FALSE,\dots)
\S4method{as_haven}{item.vector}(x,user_na=FALSE,\dots)
\S4method{as.data.set}{tbl_df}(x,row.names=NULL,\dots)
}
\arguments{
  \item{x}{for \code{as_tibble.data.set} and \code{as_haven}, an object
    of class \code{"data.set"}; for \code{as.item}, an object of class
    \code{"haven_labelled"} or \code{"haven_labelled_spss"};
    an object of class \code{"tbl_df"} for \code{as.data.set}.
  }
  \item{user_na}{logical; if \code{TRUE} then the resulting vectors
    have an \code{"na_values"} and/or \code{"na_range"} attribute.}
  \item{row.names}{\code{NULL} or an optional character vector of row names.}
  \item{\dots}{further arguments, passed through to other the the
    \code{as_tibble} method for lists, or ignored.}
}
\value{
  \code{as_tibble.data.set} and the \code{"data.set"}-method of
  \code{as_haven} return a "tibble". The \code{"item.vector"}-method
  (which is for internal use only) returns a vector with S3 class either
  \code{"haven_labelled"} or \code{"haven_labelled_spss"}. 
}

\name{memisc-deprecated}
\alias{memisc-deprecated}
\alias{fapply}
\alias{fapply.default}
\title{Deprecated Functions in Package \pkg{memisc}}
\description{
  These functions are provided for compatibility with older versions of
  \pkg{memisc} only, and may be defunct as soon as the next release.
}
\usage{
fapply(formula,data,\dots) # calls UseMethod("fapply",data)
\method{fapply}{default}(formula, data, subset=NULL,
      names=NULL, addFreq=TRUE,\dots)
}
\arguments{
  \item{formula}{a formula. The right hand side includes one or more
  grouping variables separated by '+'. These may be factors, numeric,
  or character vectors. The left hand side may be empty,
  a numerical variable, a factor, or an expression.
  See details below.}
  \item{data}{an environment or data frame or an object coercable into a data frame.}
  \item{subset}{an optional vector specifying a subset of observations
    to be used.}
  \item{names}{an optional character vector giving names to the
    result(s) yielded by the expression on the left hand side of \code{formula}.
    This argument may be redundant if the left hand side results in is a named vector.
    (See the example below.)}
  \item{addFreq}{a logical value. If TRUE and
      \code{data} is a table or a data frame with a variable
      named "Freq", a call to
      \code{table}, \code{\link{Table}}, \code{\link{percent}}, or \code{\link{nvalid}}
      is supplied by an additional argument \code{Freq}
      and a call to \code{table} is translated into
      a call to \code{Table}.
    }
  \item{\dots}{further arguments, passed to methods or ignored.}
}

\name{Substitute}
\alias{Substitute}
\title{Substitutions in Language Objects}
\description{
  \code{Substitute} differs from \code{\link{substitute}}
  in so far as its first argument can be a variable that
  contains an object of mode "language". In that case,
  substitutions take place inside this object.
}
\usage{
Substitute(lang,with)
}
\arguments{
  \item{lang}{any object, unevaluated expression, or
    unevaluated language construct, such as a sequence
    of calls inside braces
    }
  \item{with}{
    a named list, environment, data frame or data set.
  }
}
\details{
  The function body is just
  \code{do.call("substitute",list(lang,with))}.
}
\value{
  An object of storage mode "language" or "symbol".
}
\examples{
lang <- quote(sin(x)+z)
substitute(lang,list(x=1,z=2))
Substitute(lang,list(x=1,z=2))
}
\keyword{programming}

\name{contr}
\alias{contr.treatment}
\alias{contr.sum}
\alias{contr}
\alias{contrasts}
\alias{contrasts,item-method}
\alias{contrasts,ANY-method}
\alias{contrasts<-}
\alias{contrasts<-,item-method}
\alias{contrasts<-,ANY-method}
\title{Convenience Methods for Setting Contrasts}
\description{
  This package provides modified versions of
  \code{\link[stats:contrast]{contr.treatment}} and
  \code{\link[stats:contrast]{contr.sum}}. \code{contr.sum}
  gains an optional \code{base} argument, analog to the
  one of \code{contr.treatment}, furthermore,
  the \code{base} argument may be the name of a
  factor level.

  \code{contr} returns a function that calls either
  \code{contr.treatment}, \code{contr.sum}, etc.,
  according to the value given to its first argument.

  The \code{contrasts} method for \code{"item"} objects
  returns a contrast matrix or a function to produce
  a contrast matrix for the factor into which
  the item would be coerced via \code{as.factor} or \code{as.ordered}.
  This matrix or function can be specified by
  using \code{contrasts(x)<-value}
}
\usage{
contr(type,\dots)
contr.treatment(n, base=1,contrasts=TRUE)
contr.sum(n,base=NULL,contrasts=TRUE)
\S4method{contrasts}{item}(x,contrasts=TRUE,\dots)
\S4method{contrasts}{item}(x,how.many) <- value
# These methods are defined implicitely by making 'contrasts' generic.
\S4method{contrasts}{ANY}(x,contrasts=TRUE,\dots)
\S4method{contrasts}{ANY}(x,how.many) <- value
}
\arguments{
  \item{type}{a character vector, specifying the type of the contrasts.
    This argument should have a value such that, if e.g. \code{type="something"},
    then there is a function \code{contr.something} that produces
    a contrast matrix.
    }
  \item{\dots}{further arguments, passed to \code{contr.treatment}, etc. }
  \item{n}{a number of factor levels or a vector of factor levels names, see e.g. \code{\link[stats:contrast]{contr.treatment}}.}
  \item{base}{a number of a factor level or the names of a factor level,
    which specifies the baseline category,
    see e.g. \code{\link[stats:contrast]{contr.treatment}} or NULL.
    }
  \item{contrasts}{a logical value, see  \code{\link[stats]{contrasts}}}
  \item{how.many}{the number of contrasts to generate, see \code{\link[stats]{contrasts}}}
  \item{x}{a factor or an object of class "item"}
  \item{value}{a matrix, a function or the name of a function}
}
\value{
\code{contr} returns a funtion that calls one of \code{contr.treatment},
\code{contr.sum,\dots}.
\code{contr.treatment} and \code{contr.sum} return contrast matrices.
\code{contrasts(x)} returns the "contrasts" attribute of an
object, which may be a function name, a function, a contrast matrix or NULL.
}
\examples{
ctr.t <- contr("treatment",base="c")
ctr.t
ctr.s <- contr("sum",base="c")
ctr.h <- contr("helmert")
ctr.t(letters[1:7])
ctr.s(letters[1:7])
ctr.h(letters[1:7])

x <- factor(rep(letters[1:5],3))
contrasts(x)
x <- as.item(x)
contrasts(x)
contrasts(x) <- contr.sum(letters[1:5],base="c")
contrasts(x)
missing.values(x) <- 5
contrasts(x)
contrasts(as.factor(x))

# Obviously setting missing values after specifying
# contrast matrix breaks the contrasts.
# Using the 'contr' function, however, prevents this:

missing.values(x) <- NULL
contrasts(x) <- contr("sum",base="c")
contrasts(x)
missing.values(x) <- 5
contrasts(x)
contrasts(as.factor(x))
}

\name{getSummary}
\alias{getSummary}
\alias{getSummary_expcoef}
\alias{getSummary.lm}
\alias{getSummary.glm}
\alias{getSummary.clm}
\alias{getSummary.polr}
\alias{getSummary.simex}
\alias{getSummary_expcoef.default}
\alias{getSummary.aftreg}
\alias{getSummary.coxph}
\alias{getSummary.phreg}
\alias{getSummary.survreg}
\alias{getSummary.weibreg}
\alias{getSummary.merMod}
\alias{getSummary.ivreg}
\alias{getSummary.tobit}
\alias{getSummary.hurdle}
\alias{getSummary.zeroinfl}
\alias{getSummary.betareg}
\alias{getSummary.multinom}


\title{Get Model Summaries for Use with "mtable"}
\description{A generic function and methods to collect coefficients
  and summary statistics from a model object. It is used in \code{\link{mtable}}
  }
\usage{
  
  \method{getSummary}{lm}(obj, alpha=.05,\dots)
  \method{getSummary}{glm}(obj, alpha=.05,\dots)
  \method{getSummary}{merMod}(obj, alpha=.05, \dots)

# These are contributed by Christopher N. Lawrence
  \method{getSummary}{clm}(obj, alpha=.05,\dots)
  \method{getSummary}{polr}(obj, alpha=.05,\dots)
  \method{getSummary}{simex}(obj, alpha=.05,\dots)

# These are contributed by Jason W. Morgan
  \method{getSummary}{aftreg}(obj, alpha=.05,\dots)
  \method{getSummary}{coxph}(obj, alpha=.05,\dots)
  \method{getSummary}{phreg}(obj, alpha=.05,\dots)
  \method{getSummary}{survreg}(obj, alpha=.05,\dots)
  \method{getSummary}{weibreg}(obj, alpha=.05,\dots)

# These are contributed by Achim Zeileis
  \method{getSummary}{ivreg}(obj, alpha=.05,\dots)
  \method{getSummary}{tobit}(obj, alpha=.05,\dots)
  \method{getSummary}{hurdle}(obj, alpha=.05,\dots)
  \method{getSummary}{zeroinfl}(obj, alpha=.05,\dots)
  \method{getSummary}{betareg}(obj, alpha=.05,\dots)
  \method{getSummary}{multinom}(obj, alpha=.05,\dots)
  
# A variant that reports exponentiated coefficients.
# The default method calls 'getSummary()' internally and should
# be applicable to all classes for which 'getSummary()' methods exist.
getSummary_expcoef(obj, alpha=.05,\dots)
  \method{getSummary_expcoef}{default}(obj, alpha=.05,\dots)
}
\arguments{
  \item{obj}{a model object, e.g. of class \code{lm} or \code{glm}}
  \item{alpha}{level of the confidence intervals; their coverage should
    be 1-alpha/2 }
  \item{\dots}{further arguments; ignored.}
}
\details{
  The generic function \code{getSummary} is called by \code{\link{mtable}}
  in order to obtain the coefficients and summaries of model objects.
  In order to adapt \code{\link{mtable}} to models of classes other
  than \code{\link{lm}} or \code{\link{glm}} one needs to
  define \code{getSummary} methods for these classes and
  to set a summary template via \code{\link{setSummaryTemplate}}
}
\value{
  Any method of \code{getSummary} must return a list with the following
  components:

  \item{coef}{an array with coefficient estimates;
    the lowest dimension \emph{must} have the following
    names and meanings:
    \tabular{rcl}{
    \code{est} \tab  \tab the coefficient estimates,\cr
    \code{se}  \tab  \tab the estimated standard errors,\cr
    \code{stat}\tab  \tab t- or Wald-z statistics,\cr
    \code{p}   \tab  \tab significance levels of the statistics,\cr
    \code{lwr} \tab  \tab lower confidence limits, \cr
    \code{upr} \tab  \tab upper confidence limits. \cr
    }
    The higher dimensions of the array correspond to
    the individual coefficients and, in multi-equation models,
    to the model equations.

    }
  \item{sumstat}{a vector containing the model summary statistics;
    the components may have arbitrary names.}
}


\keyword{misc}
\keyword{utilities}

\name{view_html}
\alias{view_html}
\title{HTML Output for 'view.}
\description{An alternative to 'View' for use with 'view'.}
\usage{
view_html(x,title=deparse(substitute(x)),output,\dots)
}
\arguments{
  \item{x}{the result of \code{viewPrep}, a matrix of character strings.
  }
  \item{title}{an optional character string; shown as the title of the
    display.}
  \item{output}{a function or the name of a function. It determines how
    where the HTML code is directed to.

    If the working environment is RStudio, the default value is
    \code{"file.show"}. In other interactive environments it is
    \code{"browser"}. In non-interactive sessions it is 
    \code{"stdout"}.

    If \code{output} equals \code{"browser"} the generated HTML code
    is shown using \code{\link[utils]{browseURL}}. If \code{output}
    equals \code{"stdout"} the HTML code is written to the console
    output window. If \code{output} equals \code{"file.show"}, the
    function \code{file.show} is used.

    If \code{view_html} is called within a \emph{Jupyter} session,
    the HTML code created is envelopped in a pair of \code{<div>} tags
    and included into the Jupyter output.}
  \item{\dots}{other arguments; ignored.}
}
\examples{
\dontrun{
    example(data.set)
    view(Data,vfunc=view_html)
}
}

\name{ftable-matrix}
\alias{ftable_matrix}
\alias{cbind.ftable}
\alias{rbind.ftable}
\alias{cbind.ftable_matrix}
\alias{rbind.ftable_matrix}
\alias{format.ftable_matrix}
\alias{print.ftable_matrix}
\alias{Write.ftable_matrix}

\title{
 Combining flattened tables.
}
\description{
 With the method functions described here, flattened (contingency) tables can be combined
 into more complex objects, of class \code{"ftable_matrix"}. For objects of these class 
 \code{format} and \code{print} methods are provided
}
\usage{
\method{cbind}{ftable}(\dots, deparse.level=1)

\method{rbind}{ftable}(\dots, deparse.level=1)

\method{cbind}{ftable_matrix}(\dots, deparse.level=1)

\method{rbind}{ftable_matrix}(\dots, deparse.level=1)

\method{format}{ftable_matrix}(x,quote=TRUE,digits=0,format="f",\dots)

\method{Write}{ftable_matrix}(x,
                            file = "",
                            quote = TRUE,
                            append = FALSE,
                            digits = 0,
                            \dots)
                            
\method{print}{ftable_matrix}(x,quote=FALSE,\dots)
}
\arguments{
  \item{\dots}{for \code{cbind} and \code{rbind} methods, two or more objects
  of class \code{"ftable"} or \code{"ftable_matrix"}; for the other methods: further 
  arguments, ignored.}
  \item{deparse.level}{ignored, retained for compatibility reasons only.}
  \item{x}{an object used to select a method.}
  \item{quote}{logical, indicating whether or not strings should be printed with 
        surrounding quotes.} 
  \item{digits}{numeric or integer, number of significant digits to be shown.}
  \item{format}{a format string as in \code{\link{formatC}}} 
  \item{file}{character string, containing a file path.}
  \item{append}{logical, should the output appended to the file?}
}
\value{
  \code{cbind} and \code{rbind}, when used with \code{"ftable"} or \code{"ftable_matrix"}
  objects, return objects of class \code{"ftable_matrix"}.
}
\examples{
ft1 <- ftable(Sex~Survived,Titanic)
ft2 <- ftable(Age+Class~Survived,Titanic)
ft3 <- ftable(Survived~Class,Titanic)
ft4 <- ftable(Survived~Age,Titanic)
ft5 <- ftable(Survived~Sex,Titanic)

tab10 <- xtabs(Freq~Survived,Titanic)

(c12.10 <- cbind(ft1,ft2,Total=tab10))
(r345.10 <- rbind(ft3,ft4,ft5,Total=tab10))

\dontrun{
tf <- tempfile()
Write(c12.10,file=tf)
file.show(tf)
}
}

\name{sort-methods}
\alias{sort.data.frame}
\alias{sort.data.set}
\title{Convenience Methods to Sort Data Frames and Data Sets}
\description{
The methods below return a sorted version of
the data frame or data set, given as first argument.
}
\usage{
\method{sort}{data.frame}(x,decreasing=FALSE,by=NULL,na.last=NA,\dots)
\method{sort}{data.set}(x,decreasing=FALSE,by=NULL,na.last=NA,\dots)
}
\arguments{
  \item{x}{a data frame or data set.}
  \item{decreasing}{a logical value, should sorting
    be in increasing or decreasing order?}
  \item{by}{
    a character name of variable names, by which to sort;
    a formula giving the variables, by which to sort;
    NULL, in which case, the data frame / data set
    is sorted by all of its variables.
    }
  \item{na.last}{for controlling the treatment of 'NA's. If 'TRUE', missing
          values in the data are put last; if 'FALSE', they are put
          first; if 'NA', they are removed}
  \item{\dots}{other arguments, currently ignored.}
}
\value{
A sorted copy of \code{x}.
}
\examples{
DF <- data.frame(
        a = sample(1:2,size=20,replace=TRUE),
        b = sample(1:4,size=20,replace=TRUE))
sort(DF)
sort(DF,by=~a+b)
sort(DF,by=~b+a)
sort(DF,by=c("b","a"))
sort(DF,by=c("a","b"))
}

\name{withSE}
\alias{withSE}
\alias{withVCov}
\alias{withVCov.lm}
\alias{vcov.withVCov}
\alias{summary.withVCov}
\alias{summary.withVCov.lm}
\title{Add Alternative Variance Estimates to Models Estimates}
\description{
  A simple object-orientation infrastructure to add alternative standard
  errors, e.g. sandwich estimates or New-West standard errors to 
  fitted regression-type models, such as fitted by \code{lm()} or \code{glm()}.  
}
\usage{

withSE(object, vcov, \dots) 

withVCov(object, vcov, \dots)

\method{withVCov}{lm}(object, vcov, \dots)

\method{summary}{withVCov}(object, \dots)
\method{summary}{withVCov.lm}(object, \dots)
}
\arguments{
  \item{object}{a fitted model object}
  \item{vcov}{a function that returns a variance matrix estimate, a
    given matrix that is such an estimate, or a character string that
    identifies a function that returns a variance matrix estimate
    (e.g. \code{"HAC"} for \code{vcovHAC}).
  }
  \item{\dots}{further arguments, passed to \code{vcov()} or, respectively,
    to the parent method of \code{summary()}  }
}
\details{
Using \code{withVCov()} an alternative variance-covariance matrix is
attributed to a fitted model object. Such a matrix may be produced by
any of the variance estimators provided by the "sandwich" package or
any package that extends it.

\code{withVCov()} has no consequences on how a fitted model itself is
printed or represented, but it does have consequences what standard
errors are reported, when the function \code{summary()} or the function
\code{mtable()} is applied.

\code{withSE()} is a convenience front-end to \code{withVCov()}. It can
be called in the same way as \code{withVCov}, but also allows to specify
the type of variance estimate by a character string that identifies
the function that gives the covariance matrix (e.g. \code{"OPG"} for
\code{vcovOPG}).
}
\value{
  \code{withVCov} returns a slightly modified model object: It adds an
  attribute named ".VCov" that contains the alternate covaraince matrix
  and modifies the class attribute. If e.g. the original model object has class
  "lm" then the model object modified by \code{withVCov} has the class
  attribute \code{c("withVCov.lm", "withVCov", "lm")}.  
}
\examples{
## Generate poisson regression relationship
x <- sin(1:100)
y <- rpois(100, exp(1 + x))
## compute usual covariance matrix of coefficient estimates
fm <- glm(y ~ x, family = poisson)

library(sandwich)
fmo <- withVCov(fm,vcovOPG)
vcov(fm)
vcov(fmo)

summary(fm)
summary(fmo)

mtable(Default=fm,
       OPG=withSE(fm,"OPG"),
       summary.stats=c("Deviance","N")
       )

vo <- vcovOPG(fm)

mtable(Default=fm,
       OPG=withSE(fm,vo),
       summary.stats=c("Deviance","N")
       )
}

\name{Groups}
\alias{Groups}
\alias{Groups.data.set}
\alias{Groups.data.frame}
\alias{grouped.data}
\alias{Groups.grouped.data}
\alias{with.grouped.data}
\alias{within.grouped.data}
\alias{recombine}
\alias{recombine.grouped.data.frame}
\alias{recombine.grouped.data.set}
\alias{as.data.frame.grouped.data}
\alias{as.data.set,grouped.data.frame-method}
\alias{as.data.set,grouped.data.set-method}
\alias{withGroups}
\alias{withinGroups}
\title{Operate on grouped data in data frames and data sets}
\description{\code{Group} creates a grouped variant of an object of
  class "data.frame" or of class "data.set", for which methods for
  \code{with} and \code{within} are defined, so that these well-known
  functions can be applied "groupwise".
}
\usage{
# Create an object of class "grouped.data" from a
# data frame or a data set.
Groups(data,by,\dots)
\method{Groups}{data.frame}(data,by,\dots)
\method{Groups}{data.set}(data,by,\dots)
\method{Groups}{grouped.data}(data,by,\dots)

# Recombine grouped data into a data fame or a data set
recombine(x,\dots)
\method{recombine}{grouped.data.frame}(x,\dots)
\method{recombine}{grouped.data.set}(x,\dots)

# Recombine grouped data and coerce the result appropriately:
\method{as.data.frame}{grouped.data}(x,\dots)
\S4method{as.data.set}{grouped.data.frame}(x,row.names=NULL,\dots)
\S4method{as.data.set}{grouped.data.set}(x,row.names=NULL,\dots)

# Methods of the generics "with" and "within" for grouped data
\method{with}{grouped.data}(data,expr,\dots)
\method{within}{grouped.data}(data,expr,recombine=FALSE,\dots)

# This is equivalent to with(Groups(data,by),expr,...)
withGroups(data,by,expr,\dots)
# This is equivalent to within(Groups(data,by),expr,recombine,...)
withinGroups(data,by,expr,recombine=TRUE,\dots)
}

\arguments{
  \item{data}{an object of the classes "data.frame", "data.set" if an
    argument to \code{Groups}, \code{withGroups}, \code{withinGroups},
  }
  \item{by}{a formula with the factors the levels of which define the
    groups.}
  \item{expr}{an expression, or several expressions enclosed in curly
    braces.}
  \item{recombine}{a logical vector; should the resulting grouped
    data be recombined?}
  \item{x}{an object of class "grouped.data".}
  \item{row.names}{an optional character vector with row names.}
  \item{\dots}{other arguments, ignored.}
}

\details{
    When applied to a data frame \code{Groups} returns an object with class attributes
    "grouped.data.frame", "grouped.data", and "data.frame", when applied do an object with class
    "data.set", it returns an object with class attributes "grouped.data.set",
    "grouped.data", and "data.set".

    When applied to objects with class attributed
    "grouped.data", both the functions \code{with()} amd \code{within()}
    evaluate \code{expr} separately for each group defined by
    \code{Groups}. \code{with()} returns an array composed of the results
    of \code{expr}, while \code{within()} returns a modified copy of its
    \code{data} argument, which will be a "grouped.data" object
    ("grouped.data.frame" or "grouped.data.set"), unless the argument
    \code{recombine=TRUE} is set.

    The expression \code{expr} may contain references to the variables
    \code{n_}, \code{N_}, and \code{i_}. \code{n_} is equal to the size of
    the respective group (the number of rows belonging to it), while
    \code{N_} is equal to the total number of observations in all
    groups. The variable \code{i_} equals to the indices of the rows
    belonging to the respective group of observations.
}

\examples{
some.data <- data.frame(x=rnorm(n=100))
some.data <- within(some.data,{
    f <- factor(rep(1:4,each=25),labels=letters[1:4])
    g <- factor(rep(1:5,each=4,5),labels=LETTERS[1:5])
    y <- x + rep(1:4,each=25) +  0.75*rep(1:5,each=4,5)
})

# For demonstration purposes, we create an
# 'empty' group:
some.data <- subset(some.data,
                       f!="a" | g!="C")

some.grouped.data <- Groups(some.data,
                           ~f+g)    

# Computing the means of y for each combination f and g
group.means <- with(some.grouped.data,
                    mean(y))
group.means

# Obtaining a groupwise centered variant of y
some.grouped.data <- within(some.grouped.data,{
    y.cent <- y - mean(y)
},recombine=FALSE)

# The groupwise centered variable should have zero mean
# whithin each group
group.means <- with(some.grouped.data,
                    round(mean(y.cent),15))
group.means

# The following demonstrates the use of n_, N_, and i_
# An external copy of y
y1 <- some.data$y
group.means.n <- with(some.grouped.data,
                      c(mean(y),  # Group means for y
                        n_,       # Group sizes
                        sum(y)/n_,# Group means for y
                        n_/N_,    # Relative group sizes
                        sum(y1)/N_,# NOT the grand mean
                        sum(y1[i_])/n_)) # Group mean for y1
group.means.n

# Names can be attached to the groupwise results
with(some.grouped.data,
     c(Centered=round(mean(y.cent),15),
       Uncentered=mean(y)))

some.data.ungrouped <- recombine(some.grouped.data)
str(some.data.ungrouped)

# It all works with "data.set" objects
some.dataset <- as.data.set(some.data)
some.grouped.dataset <- Groups(some.dataset,~f+g)

with(some.grouped.dataset,
     c(Mean=mean(y),
       Variance=var(y)))

# The following two expressions are equivalent:
with(Groups(some.data,~f+g),mean(y))
withGroups(some.data,~f+g,mean(y))

# The following two expressions are equivalent:
some.data <- within(Groups(some.data,~f+g),{
    y.cent <- y - mean(y)
    y.cent.1 <- y - sum(y)/n_
})

some.data <- withinGroups(some.data,~f+g,{
    y.cent <- y - mean(y)
    y.cent.1 <- y - sum(y)/n_
})

# Both variants of groupwise centred varaibles should
# have zero groupwise means:
withGroups(some.data,~f+g,{
    c(round(mean(y.cent),15),
      round(mean(y.cent.1),15))
})

}

\name{Memisc}
\docType{package}
\alias{memisc-package}
\alias{memisc}
\title{Introduction to the 'memisc' Package}
\description{This package collects an assortment of tools that are intended to make
work with \code{R} easier for the author of this package
and are submitted to the public in the hope that they will be also be useful to others.

The tools in this package can be grouped into four major categories:
\itemize{
  \item Data preparation and management
  \item Data analysis
  \item Presentation of analysis results
  \item Programming
}
}
\section{Data preparation and management}{
\subsection{Survey Items}{
  \code{memisc} provides facilities to work with what users from other
  packages like SPSS, SAS, or Stata know as `variable labels', `value labels'
  and `user-defined missing values'. In the context of this package these
  aspects of the data are represented by the \code{"description"},
  \code{"labels"}, and \code{"missing.values"} attributes of a
  data vector. 
  These facilities are useful, for example, if you work with
  survey data that contain coded items like vote intention that
  may have the following structure:

  Question: ``If there was a parliamentary election next tuesday, which party would you vote for?''
  \tabular{rl}{
    1 \tab Conservative Party \cr
    2 \tab Labour Party \cr
    3 \tab Liberal Democrat Party \cr
    4 \tab Scottish Nation Party \cr
    5 \tab Plaid Cymru \cr
    6 \tab Green Party \cr
    7 \tab British National Party \cr
    8 \tab Other party \cr
    96 \tab Not allowed to vote \cr
    97 \tab Would not vote \cr
    98 \tab Would vote, do not know yet for which party \cr
    99 \tab No answer 
  }
  A statistical package like SPSS allows to
  attach labels like `Conservative Party', `Labour Party', etc.
  to the codes 1,2,3, etc. and to mark
  mark the codes 96, 97, 98, 99
  as `missing' and thus to exclude these variables from statistical
  analyses. \code{memisc} provides similar facilities.
  Labels can be attached to codes by calls like \code{\link{labels}(x) <- something}
  and expendanded by calls like \code{\link{labels}(x) <- \link{labels}(x) + something},
  codes can be marked as `missing' by
  calls like \code{\link{missing.values}(x) <- something} and
  \code{\link{missing.values}(x) <- \link{missing.values}(x) + something}.

  \code{memisc} defines a class called "data.set", which is similar to the class "data.frame".
  The main difference is that it is especially geared toward containing survey item data.
  Transformations of and within "data.set" objects retain the information about
  value labels, missing values etc. Using \code{as.data.frame} sets the data up for
  \emph{R}'s statistical functions, but doing this explicitely is seldom necessary.
  See \code{\link{data.set}}.
  }
  \subsection{More Convenient Import of External Data}{
  Survey data sets are often relative large and contain up to a few thousand variables.
  For specific analyses one needs however only a relatively small subset of these variables.
  Although modern computers have enough RAM to load such data sets completely into an R session,
  this is not very efficient having to drop most of the variables after loading. Also, loading
  such a large data set completely can be time-consuming, because R has to allocate space for
  each of the many variables. Loading just the subset of variables really needed for an analysis
  is more efficient and convenient - it tends to be much quicker. Thus this package provides
  facilities to load such subsets of variables, without the need to load a complete data set.
  Further, the loading of data from SPSS files is organized in such a way that all informations
  about variable labels, value labels, and user-defined missing values are retained.
  This is made possible by the definition of \code{\link{importer}} objects, for which
  a \code{\link{subset}} method exists. \code{\link{importer}} objects contain only
  the information about the variables in the external data set but not the data.
  The data itself is loaded into memory when the functions \code{subset} or \code{\link{as.data.set}}
  are used.
  }
  \subsection{Recoding}{
  \code{memisc} also contains facilities for recoding
  survey items. Simple recodings, for example collapsing answer
  categories, can be done using the function \code{\link{recode}}. More
  complex recodings, for example the construction of indices from
  multiple items, and complex case distinctions, can be done
  using the function \code{\link{cases}}. This function may also
  be useful for programming, in so far as it is a generalization of
  \code{\link{ifelse}}.
  }
  \subsection{Code Books}{
  There is a function \code{\link{codebook}} which produces a code book of an
  external data set or an internal "data.set" object. A codebook contains in a
  conveniently formatted way concise information about every variable in a data set,
  such as which value labels and missing values are defined and some univariate statistics.

  An extended example of all these facilities is contained in the vignette "anes48",
  and in \code{demo(anes48)}
  }
}

\section{Data Analysis}{
\subsection{Tables and Data Frames of Descriptive Statistics}{
  \code{\link{genTable}} is a generalization of \code{\link{xtabs}}:
  Instead of counts, also descriptive statistics like means or variances
  can be reported conditional on levels of factors. Also conditional
  percentages of a factor can be obtained using this function.

  In addition an \code{Aggregate} 
  function is provided, which has the same syntax as \code{genTable}, but
  gives a data frame of descriptive statistics instead of a \code{table}
  object. 
  }
  \subsection{Per-Subset Analysis}{
  \code{\link{By}} is a variant of the
  standard function \code{\link[base]{by}}: Conditioning factors
  are specified by a formula and are
  obtained from the data frame the subsets of which are to be analysed.
  Therefore there is no need to \code{\link{attach}} the data frame
  or to use the dollar operator.
  }
}
\section{Presentation of Results of Statistical Analysis}{
  \subsection{Publication-Ready Tables of Coefficients}{

Journals of the Political and Social Sciences usually require
that estimates of regression models are presented in the following
form:
\preformatted{
    ==================================================
                    Model 1     Model 2     Model 3
    --------------------------------------------------
    Coefficients
    (Intercept)     30.628***    6.360***   28.566***
                    (7.409)     (1.252)     (7.355)
    pop15           -0.471**                -0.461**
                    (0.147)                 (0.145)
    pop75           -1.934                  -1.691
                    (1.041)                 (1.084)
    dpi                          0.001      -0.000
                                (0.001)     (0.001)
    ddpi                         0.529*      0.410*
                                (0.210)     (0.196)
    --------------------------------------------------
    Summaries
    R-squared         0.262       0.162       0.338
    adj. R-squared    0.230       0.126       0.280
    N                50          50          50
    ==================================================
}

Such tables of coefficient estimates can be produced
by \code{\link{mtable}}. To see some of the possibilities of
this function, use \code{example(mtable)}.
}
\subsection{LaTeX Representation of R Objects}{
Output produced by \code{\link{mtable}} can be transformed into
LaTeX tables by an appropriate method of the generic function
\code{\link[utils]{toLatex}} which is defined in the package
\code{utils}. In addition, \code{memisc} defines \code{toLatex} methods
for matrices and \code{\link[stats]{ftable}} objects. Note that
results produced by \code{\link{genTable}} can be coerced into
\code{\link[stats]{ftable}} objects. Also, a default method
for the \code{toLatex} function is defined which coerces its
argument to a matrix and applies the matrix method of \code{toLatex}.
}
}
\section{Programming}{
\subsection{Looping over Variables}{
  Sometimes users want to contruct loops that run over variables rather than values.
  For example, if one wants to set the missing values of a battery of items.
  For this purpose, the package contains the function \code{\link{foreach}}.
  To set 8 and 9 as missing values for the items \code{knowledge1},
  \code{knowledge2}, \code{knowledge3}, one can use
  \preformatted{
    foreach(x=c(knowledge1,knowledge2,knowledge3),
        missing.values(x) <- 8:9)
  }
}
\subsection{Changing Names of Objects and Labels of Factors}{
  \code{R} already makes it possible to change the names of an object.
  Substituting the \code{\link[base]{names}} or \code{\link[base]{dimnames}}
  can be done with some programming tricks. This package defines
  the function \code{\link{rename}},
  \code{\link{dimrename}}, \code{\link{colrename}}, and \code{\link{rowrename}}
  that implement these tricks in a convenient way, so that programmers
  (like the author of this package) need not reinvent the weel in
  every instance of changing names of an object.
}
\subsection{Dimension-Preserving Versions of \code{lapply} and \code{sapply}}{
  If a function that is involved in a call to 
  \code{\link[base:lapply]{sapply}} returns a result an array or a matrix, the
  dimensional information gets lost. Also, if a list object to which
  \code{\link[base]{lapply}} or \code{\link[base:lapply]{sapply}} are applied
  have a dimension attribute, the result looses this information.
  The functions \code{\link{Lapply}} and
  \code{\link{Sapply}} defined in this package preserve such
  dimensional information.
}
\subsection{Combining Vectors and Arrays by Names}{
  The generic function \code{\link{collect}} collects several objects of the
  same mode into one object, using their names, \code{rownames},
  \code{colnames} and/or \code{dimnames}. There are methods for
  atomic vectors, arrays (including matrices), and data frames.
  For example
  \preformatted{
  a <- c(a=1,b=2)
  b <- c(a=10,c=30)
  collect(a,b)
  }
  leads to
  \preformatted{
     x  y
  a  1 10
  b  2 NA
  c NA 30
  }
}
\subsection{Reordering of Matrices and Arrays}{
  The \code{memisc} package includes a \code{\link{reorder}}
  method for arrays and matrices. For example, the matrix
  method by default reorders the rows of a matrix according the results
  of a function.
  }
}

\keyword{misc}

\name{data.set}
\alias{data.set}
\alias{data.set-class}
\alias{initialize,data.set-method}
\alias{is.data.set}
\alias{within,data.set-method}
\alias{print,data.set-method}
\alias{format,data.set-method}
\alias{show,data.set-method}
\alias{summary,data.set-method}
\alias{as.data.frame.data.set}
\alias{str.data.set}
\alias{as.data.set}
\alias{as.data.set,list-method}
\alias{[<-,data.set-method}
\alias{[,data.set,atomic,atomic,ANY-method}
\alias{[,data.set,atomic,missing,ANY-method}
\alias{[,data.set,missing,atomic,ANY-method}
\alias{[,data.set,missing,missing,ANY-method}
\alias{dim,data.set-method}
\alias{dimnames,data.set-method}
\alias{dimnames<-,data.set-method}
\alias{row.names,data.set-method}
\alias{head,data.set-method}
\alias{tail,data.set-method}
\alias{tail,data.set-method}
\alias{as.data.table.data.set}
\alias{dsView}
\title{Data Set Objects}
\description{
  \code{"data.set"} objects are collections of \code{"item"} objects,
  with similar semantics as data frames. They are distinguished
  from data frames so that coercion by \code{as.data.fame}
  leads to a data frame that contains only vectors and factors.
  Nevertheless most methods for data frames are inherited by
  data sets, except for the method for the \code{within} generic
  function. For the \code{within} method for data sets, see the details section.

  Thus data preparation using data sets retains all informations
  about item annotations, labels, missing values etc.
  While (mostly automatic) conversion of data sets into data
  frames makes the data amenable for the use of R's statistical
  functions.
  
  \code{dsView} is a function that displays data sets in a similar
  manner as \code{View} displays data frames. (\code{View} works
  with data sets as well, but changes them first into data frames.)
}
\usage{
data.set(\dots,row.names = NULL, check.rows = FALSE, check.names = TRUE,
    stringsAsFactors = FALSE, document = NULL)
as.data.set(x, row.names=NULL, \dots)
\S4method{as.data.set}{list}(x,row.names=NULL,\dots)
is.data.set(x)
\method{as.data.frame}{data.set}(x, row.names = NULL, optional = FALSE, \dots)
\S4method{within}{data.set}(data, expr, \dots)

dsView(x)

\S4method{head}{data.set}(x,n=20,\dots)
\S4method{tail}{data.set}(x,n=20,\dots)
}
\arguments{
  \item{\dots}{
    For the \code{data.set} function several vectors or items,
    for \code{within} further, ignored arguments.
  }
  \item{row.names, check.rows, check.names, stringsAsFactors, optional}{arguments
    as in \code{\link[base]{data.frame}} or \code{\link[base]{as.data.frame}},
    respectively.
    }
  \item{document}{NULL or an optional character vector that contains
    documenation of the data.}
  \item{x}{for \code{is.data.set(x)}, any object; for
    \code{as.data.frame(x,...)} and \code{dsView(x)} a "data.set" object.}
  \item{data}{a data set, that is, an object of class "data.set". }
  \item{expr}{an expression, or several expressions enclosed in curly braces.}
  \item{n}{integer; the number of rows to be shown by \code{head} or \code{tail}}
}
\details{
  The \code{as.data.frame} method for data sets is just a copy
  of the method for list. Consequently, all items in the data set
  are coerced in accordance to their \code{\link{measurement}} setting,
  see \code{\link{as.vector,item-method}} and \code{\link{measurement}}.

  The \code{within} method for data sets has the same effect as
  the \code{\link[base:with]{within}} method for data frames, apart from two differences:
  all results of the computations are coerced into items if
  they have the appropriate length, otherwise, they are automatically
  dropped.

  Currently only one method for the generic function \code{as.data.set}
  is defined: a method for "importer" objects.
}
\value{
\code{data.set} and the \code{within} method for
data sets returns a "data.set" object, \code{is.data.set}
returns a logical value, and \code{as.data.frame} returns
a data frame.
}
\examples{
Data <- data.set(
          vote = sample(c(1,2,3,8,9,97,99),size=300,replace=TRUE),
          region = sample(c(rep(1,3),rep(2,2),3,99),size=300,replace=TRUE),
          income = exp(rnorm(300,sd=.7))*2000
          )

Data <- within(Data,{
  description(vote) <- "Vote intention"
  description(region) <- "Region of residence"
  description(income) <- "Household income"
  wording(vote) <- "If a general election would take place next tuesday,
                    the candidate of which party would you vote for?"
  wording(income) <- "All things taken into account, how much do all
                    household members earn in sum?"
  foreach(x=c(vote,region),{
    measurement(x) <- "nominal"
    })
  measurement(income) <- "ratio"
  labels(vote) <- c(
                    Conservatives         =  1,
                    Labour                =  2,
                    "Liberal Democrats"   =  3,
                    "Don't know"          =  8,
                    "Answer refused"      =  9,
                    "Not applicable"      = 97,
                    "Not asked in survey" = 99)
  labels(region) <- c(
                    England               =  1,
                    Scotland              =  2,
                    Wales                 =  3,
                    "Not applicable"      = 97,
                    "Not asked in survey" = 99)
  foreach(x=c(vote,region,income),{
    annotation(x)["Remark"] <- "This is not a real survey item, of course ..."
    })
  missing.values(vote) <- c(8,9,97,99)
  missing.values(region) <- c(97,99)

  # These to variables do not appear in the
  # the resulting data set, since they have the wrong length.
  junk1 <- 1:5
  junk2 <- matrix(5,4,4)
  
})
# Since data sets may be huge, only a
# part of them are 'show'n
Data

\dontrun{

# If we insist on seeing all, we can use 'print' instead
print(Data)
}

str(Data)
summary(Data)

\dontrun{
# If we want to 'View' a data set we can use 'dsView'
dsView(Data)
# Works also, but changes the data set into a data frame first:
View(Data)
}

Data[[1]]
Data[1,]
head(as.data.frame(Data))

EnglandData <- subset(Data,region == "England")
EnglandData

xtabs(~vote+region,data=Data)
xtabs(~vote+region,data=within(Data, vote <- include.missings(vote)))
}

\name{Sapply}
\alias{Lapply}
\alias{Sapply}
\title{A Dimension Preserving Variant of "sapply" and "lapply"}
\description{
  \code{Sapply} is equivalent to \code{\link{sapply}}, except
  that it preserves the dimension and dimension names of the
  argument \code{X}. It also preserves the dimension of
  results of the function \code{FUN}.
  It is intended for application to results e.g.
  of a call to \code{\link{by}}. \code{Lapply} is an analog
  to \code{lapply} insofar as it does not try to simplify
  the resulting \code{list} of results of \code{FUN}.
}
\usage{
Sapply(X, FUN, ..., simplify = TRUE, USE.NAMES = TRUE)
Lapply(X, FUN, ...)
  }
\arguments{
  \item{X}{a vector or list appropriate to a call to \code{sapply}.}
  \item{FUN}{a function.}
  \item{...}{optional arguments to \code{FUN}.}
  \item{simplify}{a logical value; should the result be simplified to a vector or matrix if possible?}
  \item{USE.NAMES}{logical; if \code{TRUE} and if \code{X} is character, use \code{X} as names for the result unless it had names already.}
}

\value{
  If \code{FUN} returns a scalar, then the result has the same dimension
  as \code{X}, otherwise the dimension of the result is enhanced relative
  to \code{X}.
}

\examples{
berkeley <- Aggregate(Table(Admit,Freq)~.,data=UCBAdmissions)
berktest1 <- By(~Dept+Gender,
                glm(cbind(Admitted,Rejected)~1,family="binomial"),
                data=berkeley)
berktest2 <- By(~Dept,
                glm(cbind(Admitted,Rejected)~Gender,family="binomial"),
                data=berkeley)

sapply(berktest1,coef)
Sapply(berktest1,coef)

sapply(berktest1,function(x)drop(coef(summary(x))))
Sapply(berktest1,function(x)drop(coef(summary(x))))

sapply(berktest2,coef)
Sapply(berktest2,coef)
sapply(berktest2,function(x)coef(summary(x)))
Sapply(berktest2,function(x)coef(summary(x)))
}
\keyword{utilities}
\keyword{misc}

\name{relabel}
\alias{relabel}
\alias{relabel.default}
\alias{relabel.factor}
\alias{relabel4}
\alias{relabel4,item-method}
\title{Change labels of factors or labelled objects}
\description{
  Function \code{relabel} changes the labels of a factor or any object
  that has a \code{names}, \code{labels}, \code{value.labels}, or \code{variable.labels} attribute.
  Function \code{relabel4} is an (internal) generic which is called by \code{relabel}
  to handle S4 objects.
}
\usage{
\method{relabel}{default}(x, \dots, gsub = FALSE, fixed = TRUE, warn = TRUE)
\method{relabel}{factor}(x, \dots, gsub = FALSE, fixed = TRUE, warn = TRUE)


\S4method{relabel4}{item}(x, \dots)
# This is an internal method, see details.
# Use relabel(x, \dots) for 'item' objects
}
\arguments{
  \item{x}{An object with a \code{names}, \code{labels}, \code{value.labels}, or \code{variable.labels} attribute}
  \item{\dots}{A sequence of named arguments, all of type character}
  \item{gsub}{a logical value; if TRUE, \code{\link{gsub}} is used to change the
      labels of the object. That is, instead of substituting whole labels, substrings of the
      labels of the object can changed.}
  \item{fixed}{a logical value, passed to \code{\link{gsub}}. If TRUE,
    substitutions are by fixed strings and not by regular expressions.}
  \item{warn}{a logical value; if TRUE, a warning is issues if a
      a change of labels was unsuccessful.}
  }
\details{
  This function changes the names or labels of \code{x} according to the
  remaining arguments.
  If \code{gsub} is FALSE, argument tags are the \emph{old}
  labels, the values are the new labels.
  If \code{gsub} is TRUE, arguments are substrings of the labels
  that are substituted by the argument values.

  Function \code{relabel} is S3 generic. If its first argument is an S4 object,
  it calls the (internal) \code{relabel4} generic function.
}
\value{
  The object \code{x} with new labels defined by the \dots arguments.
}
\examples{
  f <- as.factor(rep(letters[1:4],5))
  levels(f)
  F <- relabel(f,
    a="A",
    b="B",
    c="C",
    d="D"
    )
  levels(F)
  
  f <- as.item(f)
  labels(f)
  F <- relabel(f,
    a="A",
    b="B",
    c="C",
    d="D"
    )
  labels(F)

  # Since version 0.99.22 - the following also works:

  f <- as.factor(rep(letters[1:4],5))
  levels(f)
  F <- relabel(f,
    a=A,
    b=B,
    c=C,
    d=D
    )
  levels(F)
  
  f <- as.item(f)
  labels(f)
  F <- relabel(f,
    a=A,
    b=B,
    c=C,
    d=D
    )
  labels(F)
  
}


\keyword{manip}

\name{importers}
\alias{importer}
\alias{importer-class}
\alias{spss.file}
\alias{spss.portable.file}
\alias{spss.portable.importer-class}
\alias{show,spss.portable.importer-method}
\alias{spss.system.file}
\alias{spss.system.importer-class}
\alias{show,spss.system.importer-method}
\alias{spss.fixed.file}
\alias{spss.fixed.importer-class}
\alias{show,spss.fixed.importer-method}
\alias{Stata.file}
\alias{Stata.importer-class}
\alias{show,Stata.importer-method}
\alias{Stata_new.importer-class}
\alias{show,Stata_new.importer-method}
\alias{as.data.set,importer-method}
\alias{dim,importer-method}
\alias{names,importer-method}
\alias{[,importer,atomic,atomic,ANY-method}
\alias{[,importer,atomic,missing,ANY-method}
\alias{[,importer,missing,atomic,ANY-method}
\alias{[,importer,missing,missing,ANY-method}
\alias{[[,importer-method}
\alias{$,importer-method}
\alias{head,importer-method}
\alias{tail,importer-method}
\alias{initialize,spss.portable.importer-method}
\alias{initialize,spss.system.importer-method}
\alias{initialize,spss.fixed.importer-method}
\alias{initialize,Stata.importer-method}
\alias{initialize,Stata_new.importer-method}
\alias{subset.spss.portable.importer}
\alias{subset.spss.system.importer}
\alias{subset.spss.fixed.importer}
\alias{subset.Stata.importer}
\alias{subset.Stata_new.importer}

\title{Object Oriented Interface to Foreign Files}
\description{
Importer objects are objects that refer to an external
data file. Currently only Stata files,
SPSS system, portable, and fixed-column files are supported.

Data are actually imported by `translating' an
importer file into a \code{\link{data.set}} using
\code{as.data.set} or \code{subset}.

The \code{importer} mechanism is more flexible and extensible
than \code{\link[foreign]{read.spss}} and  \code{\link[foreign]{read.dta}}
of package "foreign", as most of the parsing of the file headers is done in R.
It is also adapted to efficiently load large data sets.
Most importantly, importer objects support the
\code{\link{labels}}, \code{\link{missing.values}},
and \code{\link{description}}s, provided by this package.
}
\usage{
spss.file(file,\dots)

spss.fixed.file(file,
  columns.file,
  varlab.file=NULL,
  codes.file=NULL,
  missval.file=NULL,
  count.cases=TRUE,
  to.lower=getOption("spss.fixed.to.lower",FALSE),
  iconv=TRUE,
  encoded=getOption("spss.fixed.encoding","cp1252"),
  negative2missing = FALSE)

spss.portable.file(file,
  varlab.file=NULL,
  codes.file=NULL,
  missval.file=NULL,
  count.cases=TRUE,
  to.lower=getOption("spss.por.to.lower",FALSE),
  iconv=TRUE,
  encoded=getOption("spss.por.encoding","cp1252"),
  negative2missing = FALSE)

spss.system.file(file,
  varlab.file=NULL,
  codes.file=NULL,
  missval.file=NULL,
  count.cases=TRUE,
  to.lower=getOption("spss.sav.to.lower",FALSE),
  iconv=TRUE,
  encoded=getOption("spss.sav.encoding","cp1252"),
  ignore.scale.info = FALSE,
  negative2missing = FALSE)

Stata.file(file,
           iconv=TRUE,
           encoded=if(new_format)
                        getOption("Stata.new.encoding","utf-8")
                   else getOption("Stata.old.encoding","cp1252"),
           negative2missing = FALSE)

## The most important methods for "importer" objects are:
\method{subset}{spss.system.importer}(x, subset, select, drop = FALSE, \dots)
\method{subset}{spss.portable.importer}(x, subset, select, drop = FALSE, \dots)
\method{subset}{spss.fixed.importer}(x, subset, select, drop = FALSE, \dots)
\method{subset}{Stata.importer}(x, subset, select, drop = FALSE, \dots)
\method{subset}{Stata_new.importer}(x, subset, select, drop = FALSE, \dots)

\S4method{as.data.set}{importer}(x,row.names=NULL,optional=NULL,
                    compress.storage.modes=FALSE,\dots)

\S4method{head}{importer}(x,n=20,\dots)
\S4method{tail}{importer}(x,n=20,\dots)
}
\arguments{
    \item{file}{character string; the path to the file containing
      the data}
    \item{\dots}{Other arguments. \code{spss.file()} passes them on to
        \code{spss.portable.file()} of  \code{spss.system.file()}. Other
        function ignore further arguments.}
    \item{columns.file}{character string; the path to an
        SPSS/PSPP syntax file with a \code{DATA LIST FIXED} statement}
    \item{varlab.file}{character string; the path to an
        SPSS/PSPP syntax file with a \code{VARIABLE LABELS} statement}
    \item{codes.file}{character string; the path to an
        SPSS/PSPP syntax file with a \code{VALUE LABELS} statement}
    \item{missval.file}{character string; the path to an
        SPSS/PSPP syntax file with a \code{MISSING VALUES} statement}
    \item{count.cases}{logical; should cases in file be counted? This
      takes effect only if the data file does not already contain information
      about the number of cases.}
    \item{to.lower}{logical; should variable names changed to lower
      case?}
    \item{iconv}{logical; should strings (in labels and
      variables) changed into encoding of the platform?}
    \item{encoded}{a cacharacter string; the way characters are encoded
      in the improrted file. For the available encoding options
      see \code{?iconvlist}. Using this argument for
      \code{spss.system.file} this is only a fallback, as the function
      uses the encoding information present in the file if it is
      present.}
    \item{negative2missing}{logical; should negative values be marked
      as missing values? This is the convention of some newer data sets that
      are available e.g. from the GESIS data archive.}
    \item{ignore.scale.info}{logical; should information about measuremnt
      scale levels provided in the file be ignored?}
    \item{x}{an object that inherits from class \code{"importer"}.}
    \item{subset}{a logical vector or an expression containing variables
        from the external data file that evaluates to logical. }
    \item{select}{a vector of variable names from the external data file.
          This may also be a named vector, where the names give
          the names into which the variables from the external data
          file are renamed.}
    \item{drop}{a logical value, that determines what happens if
      only one column is selected. If TRUE and only one column
      is selected, \code{subset} returns only a single \code{item}
      object and not a \code{data.set}.}
    \item{row.names}{ignored, present only for compatibility.}
    \item{optional}{ignored, present only for compatibility.}
    \item{compress.storage.modes}{logical value; if TRUE floating point values
      are converted to integers if possible without loss of information.}
    \item{n}{integer; the number of rows to be shown by \code{head} or \code{tail}}
}
\value{
\code{spss.fixed.file}, \code{spss.portable.file},
\code{spss.system.file}, and \code{Stata.file}
return, respectively, objects of class
\code{"spss.fixed.importer"}, \code{"spss.portable.importer"},
\code{"spss.system.importer"}, \code{"Stata.importer"}, or \code{"Stata_new.importer"},
which, by inheritance, are also objects of class \code{"importer"}.
\code{"Stata.importer"} is for files in the format of Stata versions up
    to 12, while \code{"Stata_new.importer"} is for files in the newer
    format of Stata versions from 13.

Objects of class \code{"importer"} have at least the following two slots:

  \item{ptr}{an external pointer}
  \item{variables}{a list of objects of class \code{"item.vector"} which
    provides a `prototype' for the \code{"data.set"} set objects returned
    by the \code{as.data.set} and \code{subset} methods for objects of
    class \code{"importer"} }


The \code{as.data.frame} for \code{importer} objects does
the actual data import and returns a data frame. Note that in contrast
to \code{\link[foreign]{read.spss}}, the variable names of the
resulting data frame will be lower case, unless the importer function
is called with \code{to.lower=FALSE}. If long variable names
are defined (in case of a PSPP/SPSS system file), they take
precedence and are \emph{not} coerced to lower case.
}
\seealso{ \code{\link{codebook}}, \code{\link{description}},
\code{\link[foreign]{read.spss}}
}
\details{
    A call to a `constructor' for an importer object, that is,
    \code{spss.fixed.file}, \code{spss.portable.file}, \code{spss.sysntax.file},
    or \code{Stata.file},
    causes R to read in the header of the data file and/or
    the syntax files that contain information about
    the variables, such as the columns that they occupy
    (in case of \code{spss.fixed.file}), variable labels,
    value labels and missing values.

    The information in the file header and/or the accompagnying
    files is then processed to prepare the file for importing.
    Thus the inner structure of an \code{importer} object may
    well vary according to what type of file is to imported and
    what additional information is given.

    The \code{as.data.set} and \code{subset} methods
    for \code{"importer"} objects internally use the
    generic functions \code{seekData}, \code{readData}, \code{readSlice},
    and \code{readChunk}, which have methods for the
    subclasses of \code{"importer"}.
    These functions are not callable
    from outside the package, however.

    The \code{subset} method for \code{"importer"} objects reads in
    the data `chunk-wise' to create the subset of observations if
    the option \code{"subset.chunk.size"} is set to a non-\code{NULL}
    value, e.g. by \code{options(subset.chunk.size=1000)}. This may be
    useful in case of very large data sets from which only a tiny subset
    of observations is needed for analysis.
    
    Since the functions described here are more or less complete rewrite
    based on the description of the file structure provided
    by the documenation for PSPP, they are perhaps not as thorougly tested as the 
    functions in the \code{foreign} package, apart from the frequent use
    by the author of this package.
}
\examples{
# Extract American National Election Study of 1948
nes1948.por <- unzip(system.file("anes/NES1948.ZIP",package="memisc"),
                     "NES1948.POR",exdir=tempfile())

# Get information about the variables contained.
nes1948 <- spss.portable.file(nes1948.por)

# The data are not yet loaded:
show(nes1948)

# ... but one can see what variables are present:
description(nes1948)

# Now a subset of the data is loaded:
vote.socdem.48 <- subset(nes1948,
              select=c(
                  V480018,
                  V480029,
                  V480030,
                  V480045,
                  V480046,
                  V480047,
                  V480048,
                  V480049,
                  V480050
                  ))

# Let's make the names more descriptive:
vote.socdem.48 <- rename(vote.socdem.48,
                  V480018 = "vote",
                  V480029 = "occupation.hh",
                  V480030 = "unionized.hh",
                  V480045 = "gender",
                  V480046 = "race",
                  V480047 = "age",
                  V480048 = "education",
                  V480049 = "total.income",
                  V480050 = "religious.pref"
        )

# It is also possible to do both
# in one step:
# vote.socdem.48 <- subset(nes1948,
#              select=c(
#                  vote           = V480018,
#                  occupation.hh  = V480029,
#                  unionized.hh   = V480030,
#                  gender         = V480045,
#                  race           = V480046,
#                  age            = V480047,
#                  education      = V480048,
#                  total.income   = V480049,
#                  religious.pref = V480050
#                  ))



# We examine the data more closely:
codebook(vote.socdem.48)

# ... and conduct some analyses.
#
t(genTable(percent(vote)~occupation.hh,data=vote.socdem.48))

# We consider only the two main candidates.
vote.socdem.48 <- within(vote.socdem.48,{
  truman.dewey <- vote
  valid.values(truman.dewey) <- 1:2
  truman.dewey <- relabel(truman.dewey,
              "VOTED - FOR TRUMAN" = "Truman",
              "VOTED - FOR DEWEY"  = "Dewey")
  })

summary(truman.relig.glm <- glm((truman.dewey=="Truman")~religious.pref,
    data=vote.socdem.48,
    family="binomial",
))
}
\keyword{file}

\name{By}
\alias{By}
\title{Conditional Evaluation of an Expression}
\description{
  The function \code{By} evaluates an expression within subsets of
  a data frame, where the subsets are defined by a formula.
  }
\usage{
By(formula,expr,data=parent.frame())
}
\arguments{
  \item{formula}{an expression or (preferably) a formula
      containing the names of conditioning variables or factors.}
  \item{expr}{an expression that is evaluated for any unique combination
    of values of the variables contained in \code{formula}.}
  \item{data}{a data frame, an object that can be coerced into
    a data frame (for example, a \link{table}), or an environment,
    from which values for the variables in \code{formula}
    or \code{expr} are taken.
    }
}
\value{
  A list of class "by", giving the results for each combination of values
  of variables in \code{formula}.
}
\examples{

berkeley <- Aggregate(Table(Admit,Freq)~.,data=UCBAdmissions)
(Bres <- By(~Dept,glm(cbind(Admitted,Rejected)~Gender,family="binomial"),data=berkeley))
# The results all have 'data' components
str(Bres[[1]]$data)

attach(berkeley)
(Bres <- By(~Dept,glm(cbind(Admitted,Rejected)~Gender,family="binomial")))
detach(berkeley)

}
\keyword{manip}
\keyword{univar}

\name{mtable_format_print}
\alias{mtable_format_print}
\title{Print Format for 'mtable' Objects}
\description{
  \code{mtable_format_print} formats 'mtable' in a way suitable for screen output
  with 'print'.
  }
\usage{
mtable_format_print(x,
  topsep="=",
  bottomsep="=",
  sectionsep="-",
  interaction.sep = " x ",
  center.at=getOption("OutDec"),
  align.integers=c("dot","right","left"),
  padding = "  ",
  \dots
  )
}
\arguments{
     \item{x}{an object of class \code{mtable}}
     \item{topsep}{a character string that is recycled to a top rule.}
     \item{bottomsep}{a character string that is recycled to a bottom rule.}
     \item{sectionsep}{a character string that is recycled to seperate coefficients
        from summary statistics.}
     \item{interaction.sep}{a character string that separates factors that are involved
        in an interaction effect}
     \item{center.at}{a character string on which resulting values are centered.
        Typically equal to ".". This is the default when \code{forLaTeX==TRUE}.
        If \code{NULL}, reported values are not centered.}
    \item{align.integers}{how to align integer values.}
    \item{padding}{a character string, usually whitespace, used to insert left- and right-padding of 
        table contents.}
    \item{\dots}{further arguments, ignored. }
}
\value{
    A character string.
}
\keyword{misc}

\name{measurement_autolevel}
\alias{measurement_autolevel}
\alias{measurement_autolevel,ANY-method}
\alias{measurement_autolevel,item.vector-method}
\alias{measurement_autolevel,data.set-method}
\title{Automatically Adapt Measurement Levels}
\description{
  The generic function \code{measurement_autolevel} changes the measurement
  levels of "item" objects to "nominal" or "ordinal", if
  the proportion of its values that have labels is above a certain
  threshold.
}
\usage{
measurement_autolevel(x, \dots)
\S4method{measurement_autolevel}{ANY}(x, \dots) # Returns its argument as is
\S4method{measurement_autolevel}{item.vector}(x, 
                to=getOption("measurement.adapt.default","nominal"),
                threshold=getOption("measurement.adapt.threshold",.75),
                \dots)
\S4method{measurement_autolevel}{data.set}(x, 
                to=getOption("measurement.adapt.default","nominal"),
                threshold=getOption("measurement.adapt.threshold",.75),
                except=NULL,
                only=NULL,
                \dots)
}
\arguments{
  \item{x}{an object from class "item.vector" or "data.set".}
  \item{to}{a character vector, the target measurement level}
  \item{threshold}{the proportion of values, if reached the target
    measurement level is set}
  \item{except}{a vector with variable names, either as symbols
    (without quotation marks) or character strings (with quotation
    markes), the variables in the data set that are not to be
    changed by \code{measurement_autolevel()}.}
  \item{only}{a vector with variable names, either as symbols
    (without quotation marks) or character strings (with quotation
    markes), the variables in the data set that are to be
    changed by \code{measurement_autolevel()}.}
  \item{\dots}{other arguments, currently ignored.}
}
\examples{

 exvect <- as.item(rep(1:2,5))
 labels(exvect) <- c(a=1,b=2)
 codebook(exvect)
 codebook(measurement_autolevel(exvect))

 avect <- as.item(sample(1:3,16,replace=TRUE))
 labels(avect) <- c(a=1,b=2,c=3)
 bvect <- as.item(sample(1:4,16,replace=TRUE))
 labels(bvect) <- c(A=1,B=2,C=3,D=4)
 ds <- data.set(a=avect,b=bvect)
 codebook(ds)
 codebook(measurement_autolevel(ds))
 codebook(measurement_autolevel(ds,except=c(a,b)))
 codebook(measurement_autolevel(ds,only=a))
}

\name{query}
\alias{query}
\alias{query,data.set-method}
\alias{query,importer-method}
\alias{query,item-method}

\title{Query an Object for Information}

\description{
  The function \code{query} can be used to search an object
  for a keyword.

  The \code{\link{data.set}} and \code{\link{importer}} methods perform such a search
  through the annotations and value labels of
  the items in the data set.
  }
\usage{
query(x,pattern,\dots)
\S4method{query}{data.set}(x,pattern,\dots)
\S4method{query}{importer}(x,pattern,\dots)

\S4method{query}{item}(x,pattern,\dots)
# (Called by the methods above.)
}
\arguments{
  \item{x}{an object}
  \item{pattern}{a character string that gives the pattern to be searched
    for}
  \item{\dots}{optional arguments such as
    \describe{
      \item{\code{fuzzy}}{logical, TRUE by default; use fuzzy search via \code{\link{agrep}} or regexp search
         via \code{\link{grep}} }
      \item{\code{extended}}{logical, defaults to FALSE; passed to \code{\link{grep}} }
      \item{\code{perl}}{logical, defaults to TRUE; passed to \code{\link{grep}} }
      \item{\code{fixed}}{logical, defaults to TRUE; passed to \code{\link{grep}} }
      \item{\code{ignore.case}}{logical, defaults to TRUE; passed to \code{\link{grep}} or \code{\link{agrep}}}
      \item{\code{insertions}}{numerical value, defaults
                    to 0.999999999; passed to \code{\link{agrep}} }
      \item{\code{deletions}}{numerical value, defaults to 0; passed to \code{\link{agrep}} }
      \item{\code{substitutions}}{numerical value, defaults to 0; passed to \code{\link{agrep}} }
    }
  }
}
\value{
  If both the annotation and the value labels of an item match the pattern
  the \code{query} method for 'item' objects returns a list containing the annotation
  and the value labels, otherwise if only the annotation or the value labels
  match the pattern, either the annotation or the value labels are returned,
  otherwise if neither matches the pattern, \code{query} returns \code{NULL}.

  The methods of \code{query} for 'data.set' and 'importer' objects return
  a list of all non-\code{NULL} query results of all items contained by these
  objects, or \code{NULL}.
}
\examples{
nes1948.por <- unzip(system.file("anes/NES1948.ZIP",package="memisc"),
                     "NES1948.POR",exdir=tempfile())
nes1948 <- spss.portable.file(nes1948.por)
query(nes1948,"TRUMAN")
}

\keyword{manip}

\name{format_html}
\alias{show_html}
\alias{format_html}
\alias{write_html}
\alias{df_format_stdstyle}
\alias{mat_format_stdstyle}
\alias{format_html.data.frame}
\alias{format_html.matrix}
\title{
 Format Objects in HTML, show the HTML Format or Write it to a File
}
\description{
 \code{show_html} is for showing objects in a convenient way in HTML format. 
 \code{write_html} writes them in HTML format into a file.
 Both functions call the generic \code{format_html} for the format conversion.
}
\usage{
show_html(x, output = NULL, \dots)
write_html(x, file, \dots, standalone = TRUE)

format_html(x, \dots)

\method{format_html}{data.frame}(x,
    toprule=2,midrule=1,bottomrule=2,
    split.dec=TRUE,
    row.names=TRUE,
    digits=getOption("digits"),
    format="f",
    style=df_format_stdstyle,
    margin="2ex auto", 
    \dots)
    
\method{format_html}{matrix}(x,
    toprule=2,midrule=1,bottomrule=2,
    split.dec=TRUE,
    formatC=FALSE,
    digits=getOption("digits"),
    format="f",
    style=mat_format_stdstyle,
    margin="2ex auto", 
    \dots)
}
\arguments{
  \item{x}{an object.}
  \item{output}{character string or a function
    that determines how the HTML formatted object is shown.
  
  If \code{output} is a function, it is called with the path
  of a (temporary) file with HTML code, e.g. \emph{RStudio}'s \code{viewer}
  function (which is available in the package \code{rstudioapi}.
  
  If \code{output} equals "stdout", the HTML code is written to the 
  standard output stream (for use e.g. in output produced with \code{knitr}),
  if "file-show", the contents of a file with the HTML code is shown
  via \code{\link{file.show}}, and
  if "browser", the contents of a file with the HTML code is shown
  by the standard browser (via \code{\link{browseURL}}).
  
  This arguments has different defaults, depending of the type of
  the session. In non-interactive sessions, the default is
  "console", in interactive sessions other than RStudio,
  it is "browser", in interactive sessions with RStudio
  it is "file-show".
  
  These default settings can be overriden by the option "html_viewer"
  (see \code{\link{options}}).
  }
  \item{file}{character string; name or path of the file where to
  write the HTML code to.}
  \item{toprule}{integer;
  thickness in pixels of rule at the top of the table.}
  \item{midrule}{integer;
  thickness in pixels of rules within the table.}
  \item{bottomrule}{integer;
  thickness in pixels of rule at the bottom of the table.}
  \item{split.dec}{logical; whether numbers should be centered
  at the decimal point by splitting the table cells.}
  \item{row.names}{logical; whether row names should be shown/exported.}
  \item{digits}{number of digits to be shown after the decimal dot. This is only useful, if
  the "ftable" object was created from a table created with \code{\link{genTable}} or the like.}
  \item{formatC}{logical; whether to use \code{\link{formatC}} instead
  of \code{\link{format}} to format cell contents.}
  \item{format}{a format string for \code{\link{formatC}}}
  \item{style}{string containing the stanard CSS styling of table cells.}
  \item{margin}{character string, determines the margin and thus
    the position of the HTML table.}
  \item{\dots}{other arguments, passed on to formatter functions.}
  \item{standalone}{logical; should HTML file contain a "!DOCTYPE" header?}
}
\value{
   \code{format_html} character string with code suitable for inclusion into a HTML-file.
}

\name{applyTemplate}
\alias{applyTemplate}
\alias{template}
\title{Apply a Formatting Template to a Numeric or Character Vector}
\description{
  \code{applyTemplate} is called internally by \code{\link{mtable}}
  to format coefficients and summary statistics.
  }
\usage{
applyTemplate(x,template,float.style=getOption("float.style"),
                      digits=min(3,getOption("digits")),
                      signif.symbols=getOption("signif.symbols"))
}
\arguments{
 \item{x}{a numeric or character vector to be formatted, or a list of such vectors.}
 \item{template}{a character vector that defines the template, see details.}
 \item{float.style}{A character string that is passed to \code{\link{formatC}}
        by \code{applyTemplate}; valid values
        are \code{"e"}, \code{"f"}, \code{"g"}, \code{"fg"},
        \code{"E"}, and \code{"G"}. By default, the \code{float.style} 
        setting of \code{\link{options}} is used. The `factory fresh' setting is
        \code{options(float.style="f")}
        }
 \item{digits}{number of significant digits to use if not specified in
      the template.}
 \item{signif.symbols}{a named vector that specifies how significance levels
      are symbolically indicated, values of the vector specify
      significance levels and names specify the symbols. By default, the 
      \code{signif.symbols} setting of \code{\link{options}} is used. The "factory-fresh" setting is
        \code{options(signif.symbols=c("***"=.001,"**"=.01,"*"=.05))}.
      }
}
\details{
  Character vectors that are used as templates may be arbitrary. However,
  certain character sequences may form \emph{template expressions}.
  A template expression is of the form \code{($<POS>:<Format spec>)},
  where "\code{($}" indicates the start of a template expression,
  "\code{<POS>}" stands for either an index or name that selects an
  element from \code{x} and "\code{<Format spec>}" stands for a
  \emph{format specifier}. It may contain an letter indicating the
  style in which the vector element selected by \code{<POS>}
  will be formatted by \code{\link{formatC}}, it may contain
  a number as the number of significant digits, a "\code{#}"
  indicating that the number of signifcant digits will be at most that given
  by \code{getOption("digits")}, or \code{*} that means that
  the value will be formatted as a significance symbol.
}
\value{
  \code{applyTemplate} returns a character vector in which template
  expressions in \code{template} are substituted by formatted values from \code{x}.
  If \code{template} is an array then the return value is also an array of
  the same shape.
}
\examples{
applyTemplate(c(a=.0000000000000304,b=3),template=c("($1:g7#)($a:*)"," (($1:f2)) "))
applyTemplate(c(a=.0000000000000304,b=3),template=c("($a:g7#)($a:*)"," (($b:f2)) "))
}
\keyword{misc}
\keyword{utilities}

\name{deduplicate_labels}
\alias{deduplicate_labels}
\alias{deduplicate_labels.item}
\alias{deduplicate_labels.item.list}
\title{Handle duplicated labels}
\description{
  The function \code{deduplicate_labels} can be used with "item" objects,
  "importer" objects or "data.set" objects to deal with 
  duplicate labels,
  i.e. labels that are attached to more than
  one code. There are several ways to de-duplicate labels: by combining
  values that share their label or by making labels duplicate labels distinct.
}
\usage{
deduplicate_labels(x,\dots)
\S3method{deduplicate_labels}{item}(x,
    method=c("combine codes",
             "prefix values",
             "postfix values"),\dots)
# Applicable to 'importer' objects and 'data.set' objects
\S3method{deduplicate_labels}{item.list}(x,\dots)
}
\arguments{
  \item{x}{an item with value labels or that contains items with
    value labels}
  \item{method}{a character string that determines the method to
    make value labels unique.}
  \item{\dots}{other arguments, passed to specific methods of the
  generic function.}
}
\value{The function \code{deduplicate_labels} a copy of \code{x}
  that has unqiue value labels.
}
\examples{
x1 <- as.item(rep(1:5,4),
              labels=c(
                  A = 1,
                  A = 2,
                  B = 3,
                  B = 4,
                  C = 5
              ),
              annotation = c(
                  description="Yet another test"
))
              
x2 <- as.item(rep(1:4,5),
              labels=c(
                  i   = 1,
                  ii  = 2,
                  iii = 3,
                  iii = 4
                  ),
              annotation = c(
                  description="Still another test"
))

x3 <- as.item(rep(1:2,10),
              labels=c(
                  a = 1,
                  b = 2
                  ),
              annotation = c(
                  description="Still another test"
))
                            
codebook(deduplicate_labels(x1))
codebook(deduplicate_labels(x1,method="prefix"))
codebook(deduplicate_labels(x1,method="postfix"))

ds <- data.set(x1,x2,x3)
codebook(deduplicate_labels(ds))
codebook(deduplicate_labels(ds,method="prefix"))
codebook(deduplicate_labels(ds,method="postfix"))

}

\name{Mean}
\alias{Mean}
\alias{Median}
\alias{Min}
\alias{Max}
\alias{Weighted.Mean}
\alias{Var}
\alias{StdDev}
\alias{Cov}
\alias{Cor}
\alias{Range}
\title{Convenience wrappers for common statistical functions}
\description{\code{Mean()}, \code{Median()}, etc. are mere wrappers of
 the functions \code{mean()}, \code{median()}, etc. with the
 \code{na.rm=} optional argument set \code{TRUE} by default.}
\usage{
Mean(x, na.rm=TRUE, \dots)
Median(x, na.rm=TRUE, \dots)
Min(x, na.rm=TRUE, \dots)
Max(x, na.rm=TRUE, \dots)
Weighted.Mean(x, w, \dots, na.rm = TRUE)
Var(x, na.rm=TRUE, \dots)
StdDev(x, na.rm=TRUE, \dots)

Cov(x, y = NULL, use = "pairwise.complete.obs", \dots)
Cor(x, y = NULL, use = "pairwise.complete.obs", \dots)
Range(\dots, na.rm = TRUE, finite = FALSE)
}
\arguments{
  \item{x}{a (numeric) vector.}
  \item{y}{a (numeric) vector or \code{NULL}.}
  \item{w}{a (numeric) vector of weights.}
  \item{na.rm}{a logical value, see \code{\link[base]{mean}}.}
  \item{use}{a character string, see \code{\link[stats]{cor}}.}
  \item{\dots}{other arguments, passed to the wrapped functions.}
  \item{finite}{a logical value, see \code{\link[base]{range}}.}
}

\name{format_html.ftable}
\alias{format_html.ftable}
\alias{format_html.ftable_matrix}
\alias{ftable_format_stdstyle}
\title{
  Format "Flattened Tables" as HTML
}
\description{
  This is the method of \code{\link{format_html}} for "ftable" objects (i.e. flattened
  contingency tables)
}
\usage{
\method{format_html}{ftable}(x, 
                   show.titles = TRUE, 
                   digits = 0, 
                   format = "f", 
                   toprule = 2, midrule = 1, bottomrule = 2, 
                   split.dec = TRUE, 
                   style = ftable_format_stdstyle,
                   margin="2ex auto", 
                   \dots)
\method{format_html}{ftable_matrix}(x,
                   show.titles=TRUE,
                   digits=0,
                   format="f",
                   toprule=2,midrule=1,bottomrule=2,
                   split.dec=TRUE,
                   style = ftable_format_stdstyle,
                   margin="2ex auto", 
                   varontop,
                   varinfront,
                   grouprules=1,
                   multi_digits=NULL,
                   \dots)
}
\arguments{
  \item{x}{an object of class \code{\link{ftable}}.}
  \item{show.titles}{logical; should the names of the cross-classified variables be shown?}
  \item{digits}{number of digits to be shown after the decimal dot. This is only useful, if
  the "ftable" object was created from a table created with \code{\link{genTable}} or the like.}
  \item{format}{a format string for \code{\link{formatC}}}
     \item{toprule}{integer;
     thickness in pixels of rule at the top of the table.}
     \item{midrule}{integer;
     thickness in pixels of rules within the table.}
     \item{bottomrule}{integer;
     thickness in pixels of rule at the bottom of the table.}
    \item{split.dec}{logical; whether numbers should be centered
    at the decimal point by splitting the table cells.}
    \item{style}{string containing the standard CSS styling of table cells.}
    \item{margin}{character string, determines the margin and thus
      the position of the HTML table.}
    \item{varontop}{logical; whether names of column variables should appear on top of factor levels}
    \item{varinfront}{logical; whether names of row variables should
      appear in front of factor levels}
    \item{grouprules}{integer, should be either 1 or 2; whether one or
      two rules should drawn to distinguish groups of rows.}
     \item{multi_digits}{NULL, a numeric vector, or a list. If it is a
          list it should have as many elements as
           the "ftable_matrix" contains columns, where each vector has
           as many columns as the respective "ftable". If it is a
           vector, it is put into a list with replicated elements
           according to the "ftable" components.
           The elements of these vectors can be used to specify a separate
           number of digits for each column of the respective "ftable".}
    \item{\dots}{further arguments, ignored.}}
\seealso{
  See Also as \code{\link{format_html}}, \code{\link{show_html}}, \code{\link{write_html}}.
}

\name{mtable}
\alias{mtable}
\alias{relabel.memisc_mtable}
\alias{format.memisc_mtable}
\alias{print.memisc_mtable}
\alias{write.mtable}
\alias{toLatex.memisc_mtable}
\alias{dim.memisc_mtable}
\alias{dimnames.memisc_mtable}
\alias{[.memisc_mtable}
\alias{c.memisc_mtable}

\title{Comparative Table of Model Estimates}
\description{
  \code{mtable} produces a table of estimates for several models.
  }
\usage{
mtable(\dots,coef.style=getOption("coef.style"),
    summary.stats=TRUE,
    signif.symbols=getOption("signif.symbols"),
    factor.style=getOption("factor.style"),
    show.baselevel=getOption("show.baselevel"),
    baselevel.sep=getOption("baselevel.sep"),
    getSummary=eval.parent(quote(getSummary)),
    float.style=getOption("float.style"),
    digits=min(3,getOption("digits")),
    sdigits=digits,
    show.eqnames=getOption("mtable.show.eqnames",NA),
    gs.options=NULL,
    controls=NULL,
    collapse.controls=FALSE,
    control.var.indicator=getOption("control.var.indicator",c("Yes","No"))
  )
\method{relabel}{memisc_mtable}(x, \dots, gsub = FALSE, fixed = !gsub, warn = FALSE)

\method{format}{memisc_mtable}(x,target=c("print","LaTeX","HTML","delim"),
    \dots
    )

\method{print}{memisc_mtable}(x,
    center.at=getOption("OutDec"),
    topsep="=",bottomsep="=",sectionsep="-",\dots)

write.mtable(object,file="",
             format=c("delim","LaTeX","HTML"),\dots)

\method{toLatex}{memisc_mtable}(object,...)
}
\arguments{
     \item{\dots}{as argument to \code{mtable}: several model objects, e.g. of class 
     \code{\link{lm}}; as argument to \code{print.memisc_mtable},
     \code{toLatex.memisc_mtable}, \code{write.memisc_mtable}: further arguments
     passed to \code{format.memisc_mtable}; as argument to \code{format.memisc_mtable}:
     further arguments passed to \code{format.default};
     as argument to \code{relabel.memisc_mtable}: further arguments
     passed to \code{\link{dimrename}}.
     }
     \item{coef.style}{a character string which specifies the style of
        coefficient values, whether standard errors, Wald/t-statistics,
         or significance levels are reported, etc. See \code{\link{coef.style}}.}
     \item{summary.stats}{if \code{FALSE}, no summary statistics
       are repored. If \code{TRUE} then for each object in \code{\dots}
       either all summary statistics are reported, or those specified by
       the option \code{"summary.stats.<cls>"}, where \code{<cls>} is
       the class of the respective object.
       
       This argument may also contain a character vector with
       the names of the summary statistics to report, or a list of
       character vectors with names of summary statistics for each
       object passed as argument in \code{\dots}.
     }
     \item{signif.symbols}{a named numeric vector to specify
          the "significance levels" and corresponding symbols. 
          The numeric elements define the significance levels,
          the attached names define the associated symbols.}
     \item{factor.style}{a character string that specifies the style in
        which factor contrasts are labled. See \code{\link{factor.style}}.}
     \item{show.baselevel}{logical; determines whether base levels of factors are indicated
      for dummy coefficients}
     \item{baselevel.sep}{character that is used to separate the base level from the 
           level that a dummy variable represents}
     \item{getSummary}{a function that computes model-related statistics that
        appear in the table. See \code{\link{getSummary}}.}
     \item{float.style}{default format for floating point numbers if
        no format is specified by \code{coef.style}}.
     \item{digits}{number of significant digits if not specified by
        the template returned from \code{\link{getCoefTemplate}}
        \code{\link{getSummaryTemplate}}
        }
     \item{sdigits}{integer; number of digits after decimal dot for
			 summary statistics.}
		 \item{show.eqnames}{logical; if \code{TRUE}, left-hand sides
		   of equations are (always) shown in the table header;
			 if \code{FALSE}, left-hand sides
		   of equations are not shown;
			 if \code{NA}, left-hand sides
		   of equations are shown only if left-hand sides differ
			 among models or one of the models has multiple equations.
		 }
     \item{gs.options}{an optional list of arguments passed on to
       \code{getSummary}}
     \item{controls}{an optional formula or character vector that
       designates "control variables" for which no coefficients are
       reported, but only whether they are present in the model.
     }
     \item{collapse.controls}{a logical values; should the report about
       inclusion of control variables collapsed to a single value? If
       yes, models should either contain none or all of the control
       variables.
     }
     \item{control.var.indicator}{
       a character vector with to elements; the first element being used
       to indicate the presence of a control variable or all
       control variables (if \code{collapse.controls=TRUE}), the second
       element being used otherwise. By default these elements are
       \code{"Yes"} and \code{"No"}. 
     }
     \item{x, object}{an object of class \code{mtable}}
     \item{gsub, warn, fixed}{logical values, see \code{\link{relabel}}}
     \item{target}{a character string which indicates the target format.
     Currenlty the targets
     "print" (see \code{\link{mtable_format_print}}),
     "LaTeX" (see \code{\link{mtable_format_latex}}),
     "HTML" (see \code{\link{mtable_format_html}}), and
     "delim" (see \code{\link{mtable_format_delim}})
     are supported.
     }
     \item{center.at}{a character string on which resulting values are centered.
        Typically equal to ".". This is the default when \code{forLaTeX==TRUE}.
        If \code{NULL}, reported values are not centered.}
     \item{topsep}{a character string that is recycled to a top rule.}
     \item{bottomsep}{a character string that is recycled to a bottom rule.}
     \item{sectionsep}{a character string that is recycled to seperate coefficients
        from summary statistics.}
     \item{file}{name of the file where to write to; defaults to console output.}
     \item{format}{character string that specifies the desired format.}
}
\details{
  \code{mtable} constructs a table of estimates for regression-type models.
  \code{format.memisc_mtable} formats suitable for use with output or conversion functions
  such as \code{print.memisc_mtable}, \code{toLatex.memisc_mtable}, or
  \code{write.memisc_mtable}.
}
\value{
    A call to \code{mtable} results in an object of class \code{"mtable"}
    with the following components:

    \item{coefficients}{a list that contains the model coefficients,}
    \item{summaries}{a matrix that contains the model summaries,}
    \item{calls}{a list of calls that created the model estimates
          being summarised.}

}
\examples{
#### Basic workflow

lm0 <- lm(sr ~ pop15 + pop75,              data = LifeCycleSavings)
lm1 <- lm(sr ~                 dpi + ddpi, data = LifeCycleSavings)
lm2 <- lm(sr ~ pop15 + pop75 + dpi + ddpi, data = LifeCycleSavings)

options(summary.stats.lm=c("R-squared","N"))
mtable("Model 1"=lm0,"Model 2"=lm1,"Model 3"=lm2)

options(summary.stats.lm=c("sigma","R-squared","N"))
mtable("Model 1"=lm0,"Model 2"=lm1,"Model 3"=lm2)

options(summary.stats.lm=NULL)

mtable123 <- mtable("Model 1"=lm0,"Model 2"=lm1,"Model 3"=lm2,
    summary.stats=c("sigma","R-squared","F","p","N"))

(mtable123 <- relabel(mtable123,
  "(Intercept)" = "Constant",
          pop15 = "Percentage of population under 15",
          pop75 = "Percentage of population over 75",
            dpi = "Real per-capita disposable income",
           ddpi = "Growth rate of real per-capita disp. income"
  ))

# This produces output in tab-delimited format:
write.mtable(mtable123)

\dontrun{
# This produces output in tab-delimited format:
file123 <- "mtable123.txt"
write.mtable(mtable123,file=file123)
file.show(file123)
# The contents of this file can be pasted into Word
# and converted into a Word table.
}
\dontshow{toLatex(mtable123)}
\dontrun{texfile123 <- "mtable123.tex"
write.mtable(mtable123,format="LaTeX",file=texfile123)
file.show(texfile123)}

#### Examples with UC Berkeley data

berkeley <- Aggregate(Table(Admit,Freq)~.,data=UCBAdmissions)

berk0 <- glm(cbind(Admitted,Rejected)~1,data=berkeley,family="binomial")
berk1 <- glm(cbind(Admitted,Rejected)~Gender,data=berkeley,family="binomial")
berk2 <- glm(cbind(Admitted,Rejected)~Gender+Dept,data=berkeley,family="binomial")

mtable(berk0,summary.stats=c("Deviance","N"))
mtable(berk1,summary.stats=c("Deviance","N"))

mtable(berk0,berk1,berk2,summary.stats=c("Deviance","N"))

mtable(berk0,berk1,berk2,
          coef.style="horizontal",
          summary.stats=c("Deviance","AIC","N"))
mtable(berk0,berk1,berk2,
          coef.style="stat",
          summary.stats=c("Deviance","AIC","N"))
mtable(berk0,berk1,berk2,
          coef.style="ci",
          summary.stats=c("Deviance","AIC","N"))
mtable(berk0,berk1,berk2,
          coef.style="ci.se",
          summary.stats=c("Deviance","AIC","N"))
mtable(berk0,berk1,berk2,
          coef.style="ci.se.horizontal",
          summary.stats=c("Deviance","AIC","N"))
mtable(berk0,berk1,berk2,
          coef.style="ci.p.horizontal",
          summary.stats=c("Deviance","AIC","N"))
mtable(berk0,berk1,berk2,
          coef.style="ci.horizontal",
          summary.stats=c("Deviance","AIC","N"))
mtable(berk0,berk1,berk2,
          coef.style="all",
          summary.stats=c("Deviance","AIC","N"))
mtable(berk0,berk1,berk2,
          coef.style="all.nostar",
          summary.stats=c("Deviance","AIC","N"))

mtable(by(berkeley,berkeley$Dept,
  function(x)glm(cbind(Admitted,Rejected)~Gender,
        data=x,family="binomial")),
      summary.stats=c("Likelihood-ratio","N"))


mtable(By(~Gender,
  glm(cbind(Admitted,Rejected)~Dept,
        family="binomial"),
        data=berkeley),
      summary.stats=c("Likelihood-ratio","N"))

berkfull <- glm(cbind(Admitted,Rejected)~Dept/Gender - 1,
                      data=berkeley,family="binomial")
relabel(mtable(berkfull),Dept="Department",gsub=TRUE)

#### Array-like semantics

mtable123 <- mtable("Model 1"=lm0,"Model 2"=lm1,"Model 3"=lm2,
    summary.stats=c("sigma","R-squared","F","p","N"))

dim(mtable123)
dimnames(mtable123)
mtable123[c("dpi","ddpi"),
          c("Model 2","Model 3")]

#### Concatention
mt01 <- mtable(lm0,lm1,summary.stats=c("R-squared","N"))
mt12 <- mtable(lm1,lm2,summary.stats=c("R-squared","F","N"))
c(mt01,mt12) # not that this makes sense, but ...
c("Group 1"=mt01,
  "Group 2"=mt12)
}
\keyword{misc}

\name{contract}
\alias{contract}
\alias{contract.data.frame}
\alias{contract.data.set}
\title{Contract data into pattern-frequency format}
\description{
  \code{contract()} contracts data into pattern-frequency format, similar
  to a contatenation of \code{table()} (or \code{xtabs}) and
  \code{as.data.frame()}. Yet it uses much less memory if patterns
  are sparse, because it does not create rows for patterns that do not occur.
}
\usage{
contract(x,\dots)
\method{contract}{data.frame}(x,by=NULL, weights=NULL,name="Freq",
    force.name=FALSE,sort=FALSE,drop.na=TRUE,\dots)
\method{contract}{data.set}(x,by=NULL, weights=NULL,name="Freq",
    force.name=FALSE,sort=FALSE,drop.na=TRUE,\dots)
}
\arguments{
  \item{x}{
    an object of class \code{"data.frame"} or \code{"data.set"}.
  }
  \item{by}{the formula or a vector of variable names (quoted or not quoted).
    Specifies the patterns (and optionally weights).
    If \code{by} is a formula, then the right-hand side specifies the
    variables the value patterns of which are counted.
    If the left-hand side of the formula is (the name of) a numeric
    vector, its values are used as weights (in which case the
    \code{weights} argument will be ignored.) If the left-hand side of
    the formula is (the name of) a factor, counts are computed  in
    separate columns for each of its levels.
  }
  \item{weights}{a numeric vector of weights or \code{NULL}.}
  \item{name}{a character string, the name of the variable that
    containts the frequency counts of the value patterns.}
  \item{force.name}{a logical value, defaults to \code{FALSE}. If
    \code{TRUE} and the left-hand side of \code{by} formula is a factor,
    the names of the columns with the counts are combinations of the
    labels of the factor levels and the argument of \code{name}; if
    \code{FALSE}, the column names are created from the labels of the
    factor levels only.
  }
  \item{sort}{a logical value, defaults to \code{FALSE}. If \code{TRUE},
    the resulting data set is sorted by the variables that define the
    patterns. If \code{FALSE}, the row of the resulting data frame or
    data set are ordered according to the occurrence of the patterns. 
  }
  \item{drop.na}{a logical value, defaults to \code{TRUE}. If
    \code{FALSE}, patterns that involve \code{NA} are included in the
    resulting data frame or data set.}
  \item{\dots}{further arguments, passed to methods or ignored.}
}
\value{
  If \code{x} is a data fame, the value of \code{contract()} is also a
  data frame. If it is a \code{"data.set"} object, the result is also a
  \code{"data.set"} object. 
}
\examples{
iris_ <- sample(iris,size=nrow(iris),replace=TRUE)
w <- rep(1,nrow(iris_))
contract(iris[4:5])
contract(iris[4:5],sort=TRUE)
contract(iris[4:5],weights=w,sort=TRUE)
contract(iris,by=c(Petal.Width,Species),sort=TRUE)
contract(iris,by=~Petal.Width+Species)
contract(iris,by=w~Species)

library(MASS)
contract(housing,
         by=Sat~Infl+Type+Cont,
         weights=Freq)

contract(housing,
         by=Sat~Infl+Type+Cont,
         weights=Freq,
         name="housing",force.name=TRUE
         )
}

\name{items-to-vectors}
\alias{as.data.frame.character.item}
\alias{as.data.frame.double.item}
\alias{as.data.frame.integer.item}
\alias{as.data.frame.datetime.item}
\alias{as.data.frame.Date.item}
\alias{as.vector,item-method}
\alias{as.numeric,item-method}
\alias{as.integer,item-method}
\alias{as.factor,item.vector-method}
\alias{as.ordered,item.vector-method}
\alias{as.character,item.vector-method}
\alias{as.character,datetime.item-method}
\alias{as.character,Date.item-method}

\title{How Survey Items Are Converted into "Ordinary" Data Vectors}
\description{
    Survey item objects in are numeric or character vectors with some extra information
    that may helpful for for managing and documenting survey data, but they are not suitable
    for statistical data analysis. To run regressions etc. one should convert 
    \code{\link{item}} objects into "ordinary" numeric vectors or factors.
    This means that codes or values declared as "missing" (if present) are translated into
    the generial missing value \code{NA}, while value labels (if defined) are translated into
    factor levels.
  }
\usage{
# The following methods can be used to covert items into
# vectors with a given mode or into factors. 
\S4method{as.vector}{item}(x, mode = "any")
\S4method{as.numeric}{item}(x, \dots)
\S4method{as.integer}{item}(x, \dots)
\S4method{as.factor}{item.vector}(x)
\S4method{as.ordered}{item.vector}(x)
\S4method{as.character}{item.vector}(x, use.labels = TRUE, include.missings = FALSE, \dots)
\S4method{as.character}{datetime.item.vector}()
\S4method{as.character}{Date.item.vector}()
# The following methods are unlikely to be useful in practice, other than
# that they are called internally by the 'as.data.frame()' method for "data.set"
# objects.
\method{as.data.frame}{character.item}(x, row.names = NULL, optional = FALSE, \dots)
\method{as.data.frame}{double.item}(x, row.names = NULL, optional = FALSE, \dots)
\method{as.data.frame}{integer.item}(x, row.names = NULL, optional = FALSE, \dots)
\method{as.data.frame}{Date.item}(x, row.names = NULL, optional = FALSE, \dots)
\method{as.data.frame}{datetime.item}(x, row.names = NULL, optional = FALSE, \dots)
}

\arguments{
  \item{x}{an object in class "item","item.vector", etc., as relevant
    for the respective conversion method.}
  \item{mode}{the mode of the vector to be returned, usually \code{"numeric"},
    \code{"integer"}, or \code{"charcater"}}
  \item{use.labels}{logical,should value labels be used for creating
    the character vector?}
  \item{include.missings}{logical; if \code{TRUE}, declared missing values are
    not converted into \code{NA}, but into character strings with \code{"*"} as the "missingness marker"
    added at the beginning.}
  \item{row.names}{optional row names, see \code{\link[base]{as.data.frame}}}
  \item{optional}{a logical value, see \code{\link[base]{as.data.frame}}}
  \item{\dots}{other arguments, ignored.}
}

\value{
The function \code{as.vector()} returns a logical, numeric, or character
depending on the \code{mode=} argument. If \code{mode="any"}, the vector
has the mode that corresponds to the (internal) mode of the item
vector, that is, an item in class "integer.item" will become an integer
vector, an item in class "double.item" will become a double-precision
numeric vector, an item in class "character.item" will become a
character vector; since the internal mode of a "dateitem.item" or a
"Date.item" vector is numeric, a numeric vector will be returned.

The functions \code{as.integer()}, \code{as.numeric()}, \code{as.character()}, 
\code{as.factor()}, and \code{as.ordered()} return an integer, numeric,
or character vector, or an ordered or unordered factor, respectively.

When \code{as.data.frame()} is applied to an survey item object, the
result is a single-column data frame, where the single column is a
numeric vector or character vector or factor depending on the
\code{\link{measurement}} attribute of the item. In particular, if the
\code{\link{measurement}} attribute equals \code{"ratio"} or
\code{"interval"} this column will be the result of \code{as.vector()},
if the \code{\link{measurement}} attribute equals \code{"ordinal"} this
column will be an ordered factor (see \code{\link{ordered}}), and if
the \code{\link{measurement}} attribute equals \code{"nominal"} this
column will be an unordered factor (see \code{\link{factor}}).

All these functions have in common that values declared as "missing" by
virtue of the \code{\link{value.filter}} attribute will be turned into \code{NA}. 
}

\seealso{
  \code{\link{items}}
  \code{\link{annotation}}
  \code{\link{labels}}
  \code{\link{value.filter}}
}
\examples{
  x <- as.item(rep(1:5,4),
      labels=c(
          "First"      = 1,
          "Second"     = 2,
          "Third"      = 3,
          "Fourth"     = 4,
          "Don't know" = 5
        ),
      missing.values=5,
      annotation = c(
        description="test"
      ))
  str(x)
  summary(x)
  as.numeric(x)

  test <- as.item(rep(1:6,2),labels=structure(1:6,
                                      names=letters[1:6]))

  as.factor(test)
  as.numeric(test)
  as.character(test)
  as.character(test,include.missings=TRUE)

  as.data.frame(test)[[1]]
}
\keyword{manip}

\name{items-to-vectors}
\alias{as.data.frame.character.item}
\alias{as.data.frame.double.item}
\alias{as.data.frame.integer.item}
\alias{as.data.frame.datetime.item}
\alias{as.data.frame.Date.item}
\alias{as.vector,item-method}
\alias{as.numeric,item-method}
\alias{as.integer,item-method}
\alias{as.factor,item.vector-method}
\alias{as.ordered,item.vector-method}
\alias{as.character,item.vector-method}
\alias{as.character,datetime.item-method}
\alias{as.character,Date.item-method}
\alias{as.character,datetime.item.vector-method}
\alias{as.character,Date.item.vector-method}

\title{How Survey Items Are Converted into "Ordinary" Data Vectors}
\description{
    Survey item objects in are numeric or character vectors with some extra information
    that may helpful for for managing and documenting survey data, but they are not suitable
    for statistical data analysis. To run regressions etc. one should convert 
    \code{\link{item}} objects into "ordinary" numeric vectors or factors.
    This means that codes or values declared as "missing" (if present) are translated into
    the generial missing value \code{NA}, while value labels (if defined) are translated into
    factor levels.
  }
\usage{
# The following methods can be used to covert items into
# vectors with a given mode or into factors. 
\S4method{as.vector}{item}(x, mode = "any")
\S4method{as.numeric}{item}(x, \dots)
\S4method{as.integer}{item}(x, \dots)
\S4method{as.factor}{item.vector}(x)
\S4method{as.ordered}{item.vector}(x)
\S4method{as.character}{item.vector}(x, use.labels = TRUE, include.missings = FALSE, \dots)
\S4method{as.character}{datetime.item.vector}()
\S4method{as.character}{Date.item.vector}()
# The following methods are unlikely to be useful in practice, other than
# that they are called internally by the 'as.data.frame()' method for "data.set"
# objects.
\method{as.data.frame}{character.item}(x, row.names = NULL, optional = FALSE, \dots)
\method{as.data.frame}{double.item}(x, row.names = NULL, optional = FALSE, \dots)
\method{as.data.frame}{integer.item}(x, row.names = NULL, optional = FALSE, \dots)
\method{as.data.frame}{Date.item}(x, row.names = NULL, optional = FALSE, \dots)
\method{as.data.frame}{datetime.item}(x, row.names = NULL, optional = FALSE, \dots)
}

\arguments{
  \item{x}{an object in class "item","item.vector", etc., as relevant
    for the respective conversion method.}
  \item{mode}{the mode of the vector to be returned, usually \code{"numeric"},
    \code{"integer"}, or \code{"charcater"}}
  \item{use.labels}{logical,should value labels be used for creating
    the character vector?}
  \item{include.missings}{logical; if \code{TRUE}, declared missing values are
    not converted into \code{NA}, but into character strings with \code{"*"} as the "missingness marker"
    added at the beginning.}
  \item{row.names}{optional row names, see \code{\link[base]{as.data.frame}}}
  \item{optional}{a logical value, see \code{\link[base]{as.data.frame}}}
  \item{\dots}{other arguments, ignored.}
}

\value{
The function \code{as.vector()} returns a logical, numeric, or character
depending on the \code{mode=} argument. If \code{mode="any"}, the vector
has the mode that corresponds to the (internal) mode of the item
vector, that is, an item in class "integer.item" will become an integer
vector, an item in class "double.item" will become a double-precision
numeric vector, an item in class "character.item" will become a
character vector; since the internal mode of a "dateitem.item" or a
"Date.item" vector is numeric, a numeric vector will be returned.

The functions \code{as.integer()}, \code{as.numeric()}, \code{as.character()}, 
\code{as.factor()}, and \code{as.ordered()} return an integer, numeric,
or character vector, or an ordered or unordered factor, respectively.

When \code{as.data.frame()} is applied to an survey item object, the
result is a single-column data frame, where the single column is a
numeric vector or character vector or factor depending on the
\code{\link{measurement}} attribute of the item. In particular, if the
\code{\link{measurement}} attribute equals \code{"ratio"} or
\code{"interval"} this column will be the result of \code{as.vector()},
if the \code{\link{measurement}} attribute equals \code{"ordinal"} this
column will be an ordered factor (see \code{\link{ordered}}), and if
the \code{\link{measurement}} attribute equals \code{"nominal"} this
column will be an unordered factor (see \code{\link{factor}}).

All these functions have in common that values declared as "missing" by
virtue of the \code{\link{value.filter}} attribute will be turned into \code{NA}. 
}

\seealso{
  \code{\link{items}}
  \code{\link{annotation}}
  \code{\link{labels}}
  \code{\link{value.filter}}
}
\examples{
  x <- as.item(rep(1:5,4),
      labels=c(
          "First"      = 1,
          "Second"     = 2,
          "Third"      = 3,
          "Fourth"     = 4,
          "Don't know" = 5
        ),
      missing.values=5,
      annotation = c(
        description="test"
      ))
  str(x)
  summary(x)
  as.numeric(x)

  test <- as.item(rep(1:6,2),labels=structure(1:6,
                                      names=letters[1:6]))

  as.factor(test)
  as.numeric(test)
  as.character(test)
  as.character(test,include.missings=TRUE)

  as.data.frame(test)[[1]]
}
\keyword{manip}

\name{foreach}
\alias{foreach}
\title{Loop over Variables in a Data Frame or Environment}
\description{
  \code{foreach} evaluates an expression given as untagged argument by substituting
  in variables. The expression may also contain assignments, which take effect in
  the caller's environment.
}
\usage{
  foreach(\dots,.sorted,.outer=FALSE)
}
\arguments{
  \item{\dots}{tagged and untagged arguments.
    The tagged arguments define the 'variables' that are looped over,
    the first untagged argument defines the expression wich is
    evaluated.
  }
  \item{.sorted}{an optional logical value; relevant only
    when a range of variable is specified using the column operator
    "\code{:}". Decises whether variable names should be sorted
    alphabetically before the range of variables are created.

    If this argument missing, its default value is TRUE, if \code{foreach()} is called
    in the global environment, otherwise it is FALSE.
  }
  \item{.outer}{an optional logical value; if TRUE, each combination of
    the variables is used to evaluate the expression,
    if FALSE (the default) then the variables all need to have
    the same length and the corresponding values of the
    variables are used in the evaluation of the expression.
  }
}
\examples{
x <- 1:3
y <- -(1:3)
z <- c("Uri","Schwyz","Unterwalden")
print(x)
print(y)
print(z)
foreach(var=c(x,y,z),          # assigns names
  names(var) <- letters[1:3]   # to the elements of x, y, and z
  )
print(x)
print(y)
print(z)

ds <- data.set(
        a = c(1,2,3,2,3,8,9),
        b = c(2,8,3,2,1,8,9),
        c = c(1,3,2,1,2,8,8)
      )
print(ds)
ds <- within(ds,{ 
      description(a) <- "First item in questionnaire"
      description(b) <- "Second item in questionnaire"
      description(c) <- "Third item in questionnaire"
      
      wording(a) <- "What number do you like first?"
      wording(b) <- "What number do you like second?"
      wording(c) <- "What number do you like third?"

      foreach(x=a:c,{ # Lazy data documentation:
        labels(x) <- c(    # a,b,c get value labels in one statement
                         one = 1,
                         two = 2,
                       three = 3,
                "don't know" = 8,
         "refused to answer" = 9)
        missing.values(x) <- c(8,9)
        })
      })
      
codebook(ds)

# The colon-operator respects the order of the variables
# in the data set, if .sorted=FALSE
with(ds[c(3,1,2)],
     foreach(x=a:c,
             print(description(x))
            ))

# Since .sorted=TRUE, the colon operator creates a range 
# of alphabetically sorted variables.
with(ds[c(3,1,2)],
     foreach(x=a:c,
             print(description(x)),
             .sorted=TRUE
            ))

# The variables in reverse order
with(ds,
     foreach(x=c:a,
             print(description(x))
            ))

# The colon operator can be combined with the 
# concatenation function
with(ds,
     foreach(x=c(a:b,c,c,b:a),
             print(description(x))
            ))

# Variables can also be selected by regular expressions.
with(ds,
     foreach(x=rx("[a-b]"),
             print(description(x))
            ))

# A demonstration for '.outer=TRUE'
foreach(l=letters[1:2],
        i=1:3,
        cat(paste0(l,i,"\n")),
        .outer=TRUE)

}
\keyword{programming}

\name{Descriptives}
\alias{Descriptives}
\alias{Descriptives,ANY-method}
\alias{Descriptives,atomic-method}
\alias{Descriptives,item.vector-method}
\title{Vectors of Univariate Sample Statistics}
\description{
  \code{Descriptives(x)} gives a vector of sample statistics
  for use in \code{\link{codebook}}.
}
\usage{
Descriptives(x,...)
\S4method{Descriptives}{atomic}(x, weights = NULL, ...)
\S4method{Descriptives}{item.vector}(x, weights = NULL, ...)
}
\arguments{
  \item{x}{an atomic vector or \code{"item.vector"} object. }
  \item{weights}{an optional vector of weights.}
  \item{\dots}{further arguments, to be passed to future methods.}
}
\value{
A numeric vector of sample statistics, containing the range, the
mean, the standard deviation, the skewness and the (excess) kurtosis.
}
\examples{
x <- rnorm(100)
Descriptives(x)
}

\name{duplicated_labels}
\alias{duplicated_labels}
\alias{duplicated_labels.item}
\alias{duplicated_labels.item.list}
\title{Check for and report duplicated labels}
\description{
  The function \code{duplicated_labels} can be used with "item" objects,
  "importer" objects or "data.set" objects to check whether items
  contain duplicate labels, i.e. labels that are attached to more than
  one code.
}
\usage{
duplicated_labels(x)
\S3method{duplicated_labels}{item}(x)
# Applicable to 'importer' objects and 'data.set' objects
\S3method{duplicated_labels}{item.list}(x)
}
\arguments{
  \item{x}{an item with value labels or that contains items with
    value labels}
}
\value{The function \code{duplicate.labels} returns a list with a class
  attribute, which allows pretty printing of duplicated value labels}
\examples{
x1 <- as.item(rep(1:5,4),
              labels=c(
                  A = 1,
                  A = 2,
                  B = 3,
                  B = 4,
                  C = 5
              ),
              annotation = c(
                  description="Yet another test"
))
              
x2 <- as.item(rep(1:4,5),
              labels=c(
                  i   = 1,
                  ii  = 2,
                  iii = 3,
                  iii = 4
                  ),
              annotation = c(
                  description="Still another test"
))

x3 <- as.item(rep(1:2,10),
              labels=c(
                  a = 1,
                  b = 2
                  ),
              annotation = c(
                  description="Still another test"
))
                            
duplicated_labels(x1)
ds <- data.set(x1,x2,x3)
duplicated_labels(ds)
codebook(ds)

nes1948.por <- unzip(system.file("anes/NES1948.ZIP",package="memisc"),
                     "NES1948.POR",exdir=tempfile())
nes1948 <- spss.portable.file(nes1948.por)
duplicated_labels(nes1948)             
}

\name{coarsen}
\alias{coarsen}
\alias{coarsen.numeric}
\title{Coarsen a vector into a factor with a lower number of levels}
\description{
  \code{coarsen} can be used to obtain a factor from a vector, similar
  to \code{\link[base]{cut}}, but with less technical and more "aesthetic"
  labels of the factor levels.
}
\usage{
coarsen(x,\dots)
\method{coarsen}{numeric}(x,
        n=5,
        pretty=TRUE,
        quantiles=!pretty,
        breaks=NULL,
        brackets=FALSE,
        sep=if(brackets)";"else if(quantiles) "-" else " - ",
        left="[",
        right="]",
        range=FALSE,
        labels=NULL,
        \dots)
}
\arguments{
  \item{x}{a vector, usually a numeric vector}
  \item{n}{number of categories of the resulting factor}
  \item{pretty}{a logical value, whether \code{\link[base]{pretty}}
    should be used to compute the breaks.}
  \item{quantiles}{a logical value, whether \code{\link[stats]{quantile}}
    should be used to compute the breaks.}
  \item{breaks}{a vector of break points or \code{NULL}.}
  \item{brackets}{a logical value, whether the labels should include brackets.}
  \item{sep}{a character string, used as a separator between upper and
    lower boundaries in the labels.}
  \item{left}{a character string, to be used as the left bracket}
  \item{right}{a character string, to be used as the right bracket}
  \item{range}{a logical value, whether the minimum and maximum of
    \code{x} should be included into \code{breaks}.}
  \item{labels}{an optional character vector of labels.}
  \item{\dots}{further arguments, passed on to \code{\link[base]{pretty}}
    or \code{\link[stats]{quantile}} if applicable.
  }
}


\examples{
x <- rnorm(200)

table(coarsen(x))

table(coarsen(x,quantiles=TRUE))

table(coarsen(x,brackets=TRUE))

table(coarsen(x,breaks=c(-1,0,1)))

table(coarsen(x,breaks=c(-1,0,1),
              range=TRUE,labels=letters[1:4]))

}
\keyword{manip}

\name{neg2mis}
\alias{neg2mis}
\title{Mark Negative Values as Missing}
\description{In many newer survey data sets available from social
  science data archives non-valid responses (such as "don't know" or
  "answer refused") are given negative codes. The function
  \code{neg2miss} allows to mark them as missing values.)
}
\usage{
neg2mis(x,all=FALSE,exclude=NULL,select=NULL,zero=FALSE)
}
\arguments{
  \item{x}{an object that inherits from class "item.list", e.g. a
    "data.set" or an "importer" object.}
  \item{all}{logical; should the marking of negative values as missing
    applied to all variables?}
  \item{exclude}{an optional vector of variable naems to which
    the marking of negative values as missing should \emph{not} be
    applied.}
  \item{select}{an optional vector of variable names to which
    the marking of negative values as missing should be applied.}
  \item{zero}{logical; should zeroes also be marked as missing?}
}
\examples{
ds <- data.set(
          var1 = c(0,1,-1,2,3),
          var2 = c(-1,-1,1,1,1),
          var3 = c(1,2,3,4,5)
          )
neg2mis(ds,all=TRUE)
neg2mis(ds,all=TRUE,zero=TRUE)
neg2mis(ds,exclude=var1)
neg2mis(ds,select=var1)
}

\name{attr-operators}
\alias{\%#\%}
\alias{\%##\%}
\alias{\%@\%}
\alias{\%@\%<-}
\title{Operators for Setting Annotations and Attributes}
\description{The operator \code{\%#\%} can be used to attach a
  \code{\link{description}} annotation to an object. \code{\%##\%} can be
  used to attach a character vector of annotations to an object.
  \code{\%@\%} returns the attribute with the name given as second
  argument. With \code{\%@\%} it is also possible to assign attributes.
}
\usage{
  x \%#\% descr
  x \%##\% annot
  x \%@\% nm
  x \%@\% nm <- value
}
\arguments{
  \item{x}{an object, usually and \code{\link{item}} or a vector.}
  \item{descr}{a character string}
  \item{annot}{a named character vector; its contents are added to the
    "annotation" attribute of \code{x}. Existing elements are kept.}
  \item{nm}{a character string, the name of the attribute being set or
    requested.}
  \item{value}{any kind of object that can be attached as an attribute.}
}
\examples{
test1 <- 1 \%#\% "One"
# This is equivalent to:
# test <- 1
# description(test) <- "One"
description(test1)
# Results in "One"

# Not that it makes sense, but ...
test2 <- 2 \%##\% c(
                    Precedessor = 0,
                    Successor   = 2
                 )
# This is equivalent to:
# test2 <- 2
# annotation(test2) <- c(
#                    Precedessor = 0,
#                    Successor   = 2
#                 )
annotation(test2)

# The following examples are equivalent to
# attr(test2,"annotation")
test2 \%@\% annotation

test2 \%@\% "annotation"

test2 \%@\% another.attribute <- 42
# This is equivalent to attr(test2,"another.attribute") <- 42

attributes(test2)
}

\name{data.set manipulation}
\alias{subset.data.set}
\alias{unique,data.set-method}
\alias{merge,data.set,data.set-method}
\alias{merge,data.set,data.frame-method}
\alias{merge,data.frame,data.set-method}
\alias{cbind.data.set}
\alias{rbind.data.set}
\title{Manipulation of Data Sets}
\description{
  Like data frames, \code{data.set} objects have
  \code{\link[base]{subset}}, \code{\link[base]{unique}},
  \code{\link[base]{cbind}}, \code{\link[base:cbind]{rbind}},
  \code{\link[base]{merge}} methods defined for them.

  The semantics are basically the same as the methods defined
  for data frames in the \code{base} package, with the only difference
  that the return values are \code{data.set} objects.
  In fact, the methods described here are front-ends to the
  corresponding methods for data frames, which are constructed
  such that the "extra" information attached to variables within
  \code{data.set} objects, that is, to \code{item} objects.
}
\usage{
\method{subset}{data.set}(x, subset, select, drop = FALSE, \dots)

\S4method{unique}{data.set}(x, incomparables = FALSE, \dots)

\S3method{cbind}{data.set}(\dots, deparse.level = 1)

\S3method{rbind}{data.set}(\dots, deparse.level = 1)

\S4method{merge}{data.set,data.set}(x,y, \dots)

\S4method{merge}{data.set,data.frame}(x,y, \dots)

\S4method{merge}{data.frame,data.set}(x,y, \dots)
}
\arguments{
  \item{x,y}{\code{data.set} objects. On of the arguments to
    \code{merge} may also be an object coercable into a data frame
    and the result still is a \code{data.set} object.
  }
  \item{subset}{a logical expression, used to select observations from
    the data set.}
  \item{select}{a vector with variablen names, which are retained in the
    data subset.}
  \item{drop}{logical; if \code{TRUE} and the result has only one
    column, the result is an item and not a data set.}
  \item{\dots}{for \code{subset}: a logical vector
    of the same length as the number of rows of the \code{data.set}
    and, optionally, a vector of variable names (tagged as \code{select});
    for \code{unique}: further arguments, ignored;
    for \code{cbind}, \code{rbind}: objects coercable
    into data frames, with at least one being a \code{data.set}
    object;
    for \code{merge}: further arguments
    such as arguments tagged with \code{by}, \code{by.x}, \code{by.y},
    etc. that specify the variables by which to merge
    the data sets of data frames \code{x} and \code{y}.
  }
  \item{incomparables}{a vector of values that cannot be compared. See
    \code{\link[base]{unique}}.
  }
  \item{deparse.level}{an argument retained for
    reasons of compatibility of the default methods
    of \code{\link[base]{cbind}} and \code{\link[base:cbind]{rbind}}.
  }
}
\examples{
ds1 <- data.set(
      a = rep(1:3,5),
      b = rep(1:5,each=3)
  )
ds2 <- data.set(
      a = c(3:1,3,3),
      b = 1:5
  )

ds1 <- within(ds1,{
      description(a) <- "Example variable 'a'"
      description(b) <- "Example variable 'b'"
  })

ds2 <- within(ds2,{
      description(a) <- "Example variable 'a'"
      description(b) <- "Example variable 'b'"
  })

str(ds3 <- rbind(ds1,ds2))
description(ds3)

ds3 <- within(ds1,{
        c <- a
        d <- b
        description(c) <- "Copy of variable 'a'"
        description(d) <- "Copy of variable 'b'"
        rm(a,b)
    })
str(ds4 <- cbind(ds1,ds3))
description(ds4)

ds5 <- data.set(
        c = 1:3,
        d = c(1,1,2)
        )
ds5 <- within(ds5,{
      description(c) <- "Example variable 'c'"
      description(d) <- "Example variable 'd'"
  })
str(ds6 <- merge(ds1,ds5,by.x="a",by.y="c"))

# Note that the attributes of the left-hand variables
# have priority.
description(ds6)
}

\name{percentages}
\alias{percentages}
\alias{percentages.table}
\alias{percentages.formula}
\alias{percentages.default}
\alias{percentages.data.frame}
\alias{percentages.list}
\alias{as.data.frame.percentage.table}
\alias{as.data.frame.xpercentage.table}
\title{
  Easy Creation of Tables of Percentages
}
\description{
  The generic function \code{percentages} and its methods
  create one- or multidimensional tables of percentages. As such,
  the function \code{percentages} can be viewed as a convenience
  interface to \code{\link{prop.table}}. However, it also
  allows to obtain standard errors and confidence intervals.
}
\usage{
percentages(obj, \dots)
\method{percentages}{table}(obj,
      by=NULL, which=NULL, se=FALSE, ci=FALSE, ci.level=.95, \dots)
\method{percentages}{formula}(obj,
      data=parent.frame(), weights=NULL, \dots)
\method{percentages}{default}(obj,
      weights=NULL, \dots)
\method{percentages}{data.frame}(obj,
      weights=NULL, \dots)
\method{percentages}{list}(obj,
      weights=NULL, \dots)
\method{as.data.frame}{percentage.table}(x, \dots)
\method{as.data.frame}{xpercentage.table}(x, \dots)
}
\arguments{
  \item{obj}{an object; a contingency table or a formula. If
  it is a formula, its left-hand side determines the factor 
  or combination of factors for which percentages are 
  computed while its right-hand side determines the 
  factor or combination of factors that define the 
  groups within which percentages are computed.}
  \item{by}{a character vector with the names of the
  factor variables that define the groups within which percentages
  are computed. Percentages sum to 100 within combination
  of levels of these factors.}
  \item{which}{a character vector with the names of the
  factor variables for which percentages 
  are computed.}
  \item{se}{a logical value; determines whether standard
  errors are computed.}
  \item{ci}{a logical value; determines whether confidence
  intervals are computed. Note that the confidence intervals
  are for infinite (or very large) populations.}
  \item{ci.level}{a numerical value, the required confidence level of
  the confidence intervals.}
  \item{data}{a contingency table (an object that inherits from "table")
   or a data frame or an object coercable into a data frame.}
  \item{weights}{an optional vector of weights. Should be NULL or a
    numeric vector.}
  \item{\dots}{Further arguments passed on to the
    "table" method of \code{percentages} or ignored in case of
     a call to \code{as.data.frame}.}
  \item{x}{an object coerced into a data frame.}
}
\value{
  An array that inherits classes "percentage.table" and "table". If
  \code{percentages} was called with \code{se=TRUE} or \code{ci=TRUE}
  then the result additionally inherits class "xpercentage.table".
}
\examples{
percentages(UCBAdmissions)

# Three equivalent ways to create the same table of conditional
# percentages
percentages(Admit~Gender+Dept,data=UCBAdmissions)
percentages(UCBAdmissions,by=c("Gender","Dept"))
percentages(UCBAdmissions,which="Admit")
# Percentage table as data frame
as.data.frame(percentages(Admit~Gender+Dept,data=UCBAdmissions))

# Standard errors and confidence intervals
percentages(Admit~Dept,data=UCBAdmissions,se=TRUE)
percentages(Admit~Dept,data=UCBAdmissions,ci=TRUE)
(p<- percentages(Admit~Dept,data=UCBAdmissions,ci=TRUE,se=TRUE))

# An extended table of percentages as data frame
as.data.frame(p)

# A table of percentages of a factor
percentages(iris$Species)

UCBA <- as.data.frame(UCBAdmissions)
percentages(UCBA$Admit,weights=UCBA$Freq)

percentages(UCBA,weights=UCBA$Freq)

}

\name{dimrename}
\alias{dimrename}
\alias{colrename}
\alias{rowrename}
\title{Change dimnames, rownames, or colnames}
\description{
  These functions provide an easy way to change the \code{dimnames}, \code{rownames} or \code{colnames} of
  an array.
}
\usage{
dimrename(x, dim = 1, \dots, gsub = FALSE, fixed = TRUE, warn = TRUE)
rowrename(x, \dots, gsub = FALSE, fixed = TRUE, warn = TRUE)
colrename(x, \dots, gsub = FALSE, fixed = TRUE, warn = TRUE)
}
\arguments{
  \item{x}{An array with dimnames}
  \item{dim}{A vector that indicates the dimensions}
  \item{\dots}{A sequence of named arguments}
  \item{gsub}{a logical value; if TRUE, \code{\link{gsub}} is used to change the
      \code{dimnames} of the object.
      That is, instead of substituting whole names, substrings of the
      \code{dimnames} of the object can changed.
      }
  \item{fixed}{a logical value, passed to \code{\link{gsub}}. If TRUE,
    substitutions are by fixed strings and not by regular expressions.}
  \item{warn}{logical; should a warning be issued if the pattern is not found?}
}
\details{
  \code{dimrename} changes the dimnames of \code{x} along dimension(s) \code{dim} according to the
  remaining arguments. The argument names are the \emph{old}
  names, the values are the new names.
  \code{rowrename} is a shorthand for changing the rownames,
  \code{colrename} is a shorthand for changing the colnames of a matrix
  or matrix-like object.

  If \code{gsub} is FALSE, argument tags are the \emph{old}
  \code{dimnames}, the values are the new \code{dimnames}.
  If \code{gsub} is TRUE, arguments are substrings of the \code{dimnames}
  that are substituted by the argument values.

}
\value{
  Object \code{x} with changed dimnames.
}

\examples{
m <- matrix(1,2,2)
rownames(m) <- letters[1:2]
colnames(m) <- LETTERS[1:2]
m
dimrename(m,1,a="first",b="second")
dimrename(m,1,A="first",B="second")
dimrename(m,2,"A"="first",B="second")

rowrename(m,a="first",b="second")
colrename(m,"A"="first",B="second")

# Since version 0.99.22 - the following also works:

dimrename(m,1,a=first,b=second)
dimrename(m,1,A=first,B=second)
dimrename(m,2,A=first,B=second)
}

\keyword{manip}

\name{labels}
\alias{labels}
\alias{labels,NULL-method}
\alias{labels,item-method}
\alias{labels<-}
\alias{labels<-,item,ANY-method}
\alias{labels<-,ANY,NULL-method}
\alias{labels<-,item,NULL-method}
\alias{labels<-,vector,ANY-method}
\alias{labels<-,vector,NULL-method}
\alias{value.labels-class}
\alias{initialize,value.labels-method}
\alias{Arith,value.labels,ANY-method}
\alias{[,value.labels,numeric,missing,missing-method}
\alias{[,value.labels,logical,missing,missing-method}
\alias{coerce,numeric,value.labels-method}
\alias{coerce,character,value.labels-method}
\alias{coerce,value.labels,numeric-method}
\alias{coerce,value.labels,character-method}
\alias{as.vector,value.labels-method}
\alias{show,value.labels-method}
\title{Value Labels}
\description{
Value labels associate character labels to possible values
of an encoded survey item. Value labels are represented
as objects of class "value.labels".

Value labels of an item can be obtained
using \code{labels(x)} and
can be associated to items and to vectors
using labels(x) <- value

Value labels also can be updated using the  \code{+}
and \code{-} operators.
}
\usage{
labels(object,\dots)
labels(x) <- value
}
\arguments{
  \item{object}{any object.}
  \item{\dots}{further arguments for other methods.}
  \item{x}{a vector or "item" object.}
  \item{value}{an object of class "value.labels" or
    a vector that can be coerced into an "value.labels" object or NULL}
}
\examples{
  x <- as.item(rep(1:5,4),
      labels=c(
          "First"      = 1,
          "Second"     = 2,
          "Third"      = 3,
          "Fourth"     = 4,
          "Don't know" = 5
        ),
      missing.values=5,
      annotation = c(
        description="test"
      ))
  labels(x)
  labels(x) <- labels(x) - c("Second"=2)
  labels(x)
  labels(x) <- labels(x) + c("Second"=2)
  labels(x)

  puvl <- getOption("print.use.value.labels")
  options(print.use.value.labels=FALSE)
  x
  options(print.use.value.labels=TRUE)
  x
  options(print.use.value.labels=puvl)
}

\name{Reshape}
\alias{Reshape}
\title{Reshape data frames or data sets}
\description{\code{Reshape} is a convenience
  wrapper around \code{\link[stats]{reshape}} with a somewhat simpler
  syntax.}
\usage{
Reshape(data,\dots,id,within_id,drop,keep,direction)
}
\arguments{
  \item{data}{a data frame or data set to be reshaped.}
  \item{\dots}{
    Further arguments that specify the variables in
    long and in wide format as well as the time variable.
    The name tags of the arguments given here specify 
    variable names in long format,
    the arguments themselves specify the variables in wide format
    (or observations in long vormat)
    and the variable of the "time" variable.
    The time variable is usually the last of these arguments.
    An "automatic" time variable can be specified if only
    a single argument in \code{\dots} is given.
  }
  \item{id}{a variable name or a concatenation of variable names
      (either as character strings or as unquoted symbols), that identify
      individual units. Defaults to \code{"id"} or the id variable
      specified in the \code{"reshapeLong"} attribute of the \code{data}
      argument. Needed only if the data are reshaped from long to wide
      format.
  }
  \item{within_id}{an optional variable name 
      (either as character string or as unquoted symbol), that identifies
      individual observations on units.
      Relevant only if the data are reshaped from long to wide
      format.}
  \item{drop}{a variable name or a concatenation of variable names
      (either as character strings or as unquoted symbols), thast specifies
      the variables to be dropped before reshaping.
  }
  \item{keep}{a variable name or a concatenation of variable names
      (either as character strings or as unquoted symbols), thast specifies
      the variables to be kept after reshaping (including the ones
      used to define the reshaping).
  }
  \item{direction}{a character string, should be either equal "long"
    or "wide".}
}
\examples{
example.data.wide <- data.frame(
    v  = c(35,42),
    x1 = c(1.1,2.1),
    x2 = c(1.2,2.2),
    x3 = c(1.3,2.3),
    x4 = c(1.4,2.4),
    y1 = c(2.5,3.5),
    y2 = c(2.7,3.7),
    y3 = c(2.9,3.9))
example.data.wide

# The following two calls are equivalent:
example.data.long <- Reshape(data=example.data.wide,
                             x=c(x1,x2,x3,x4),
                             # N.B. it is possible to
                             # specify 'empty' i.e. missing
                             # measurements
                             y=c(y1,y2,y3,),
                             t=1:4,
                             direction="long")

example.data.long <- Reshape(data=example.data.wide,
                             list(
                                 x=c(x1,x2,x3,x4),
                                 # N.B. it is possible to
                                 # specify 'empty' i.e. missing
                                 # measurements
                                 y=c(y1,y2,y3,)
                             ),
                             t=1:4,
                             direction="long")

example.data.long

# Since the data frame contains an "reshapeLong" attribute
# an id variable is already specified and part of the data
# frame.
example.data.wide <- Reshape(data=example.data.long,
                             x=c(x1,x2,x3,x4),
                             y=c(y1,y2,y3,),
                             t=1:4,
                             direction="wide")
example.data.wide

# Here we examine the case where no "reshapeLong" attribute
# is present:
example.data.wide <- Reshape(data=example.data.long,
                             x=c(x1,x2,x3,x4),
                             y=c(y1,y2,y3,),
                             t=1:4,
                             id=v,
                             direction="wide")
example.data.wide


# Here, an "automatic" time variable is created. This works
# only if there is a single argument other than the data=
# and direction= arguments
example.data.long <- Reshape(data=example.data.wide,
                             list(
                                 x=c(x1,x2,x3,x4),
                                 y=c(y1,y2,y3,)
                             ),
                             direction="long")

example.data.long

example.data.wide <- Reshape(data=example.data.long,
                             list(
                                 x=c(x1,x2,x3,x4),
                                 y=c(y1,y2,y3,)
                             ),
                             direction="wide")
example.data.wide
}

\name{charTrans}
\alias{charTrans}
\alias{charTrans.character}
\alias{charTrans.annotation}
\alias{charTrans.data.set}
\alias{charTrans.importer}
\alias{charTrans.item}
\alias{charTrans.value.labels}
\encoding{UTF-8}

\title{Character Translation of Aspects of Objects}
\description{
  This function uses the base package function \code{\link[base]{chartr}}
  to translate characters in variable descriptions (a.k.a variable labels) and
  value labels of \code{\link{item}}, \code{\link{data.set}},
  and \code{\link{importer}} objects.
  
  It will be useful when the encoding of an important data set 
  cannot be fully identified or if the encoding in the data file
  is incorrect or unknown.
}
\usage{

charTrans(x, old = "", new = "", \dots)
\method{charTrans}{annotation}(x, old = "", new = "", \dots)
\method{charTrans}{data.set}(x, old = "", new = "", \dots)
\method{charTrans}{importer}(x, old = "", new = "", \dots)
\method{charTrans}{item}(x, old = "", new = "", \dots)
\method{charTrans}{value.labels}(x, old = "", new = "", \dots)

}
\arguments{
  \item{x}{a character vector or an object of which character data or attributes
   character-translated.}
  \item{old}{a string with the characters to be translated.}
  \item{new}{a string with the translated characters.}
  \item{\dots}{further arguments, currently ignored.}
}
\value{
  \code{charTrans} returns a copy of its first argument with character-translated
  character data or attributes.
}
\seealso{
  \code{\link[base]{chartr}}
}
\examples{
\dontrun{
# Locate an SPSS 'portable' file and get info on variables, their labels 
S2601.POR <- spss.portable.file("POR-Files/S2601.POR", encoded = "cp850")
# 'ß' appears to be correctly coded, but 'ä', 'ö', 'ü' are not, so we need to
# to some fine-tuning
S2601.POR <- charTrans(S2601.POR, old="{|}\r", new="äöü ")
# Now labels etc. are correctly encoded.
codebook(S2601.POR)
}
}

\name{format_html.codebook}
\alias{format_html.codebook}
\alias{codebook_format_stdstyle}
\title{
  Format Codebooks as HTML
}
\description{
  This is the method of \code{\link{format_html}} for "codebook" objects as created
  by the eponymous function (see \code{\link{codebook}})
}
\usage{
\method{format_html}{codebook}(x, 
    toprule = 2,
    midrule = 1,
    indent = "3ex",
    style = codebook_format_stdstyle,
    var_tag = "code", 
    varid_prefix = "", title_tag = "p",\dots)
}
\arguments{
  \item{x}{a "codebook" object}
  \item{toprule}{a non-negative integer; thickness of the line (in pixels) at the top of each
  codebook entry}
  \item{midrule}{a non-negative integer; thickness of the line (in pixels) that separates the
  header of an codebook entry from its body}
  \item{indent}{character string; indentation (by padding) of the 
    codebook entry contents}
  \item{style}{string containing the standard CSS styling of codebook table cells.}
  \item{var_tag}{character string; the HTML tag that contains the 
  name of the variable}
  \item{varid_prefix}{character string; a prefix added to 
  the anchor IDs of the code entry titles (to facilitate the 
  creation of tables of contents etc.)}
  \item{title_tag}{character string; the HTML tag that contains 
  the title of the codebook entry (the variable name and its description)}
 \item{\dots}{further arguments, ignored.}}
\seealso{
  See Also as \code{\link{format_html}}, \code{\link{show_html}}, \code{\link{write_html}}.
}

\name{List}
\alias{List}
\title{Create a list and conveniently supply names to its elements}
\description{\code{List} creates a list and names its elements after the
  arguments given, in a manner analogously to \code{\link{data.frame}}}
\usage{
  List(\dots)
}
\arguments{
  \item{\dots}{tagged or untagged arguments from which the list is
    formed. If the untagged arguments are variables from the englosing
  environment, their names become the names of the list elements.}
}
\examples{
  num <- 1:3
  strng <- c("a","b","A","B")
  logi <- rep(FALSE,7)
  List(num,strng,logi)
}

\name{html}
\alias{html}
\alias{print.html_elem}
\alias{as.character.html_elem}
\alias{c.html_elem}
\alias{html_group}
\alias{as.html_group}
\alias{as.character.html_group}
\alias{print.html_group}
\alias{[.html_group}
\alias{[<-.html_group}
\alias{c.html_group}

\alias{content}
\alias{content<-}
\alias{setContent}
\alias{attribs}
\alias{attribs<-}
\alias{setAttribs}
\alias{setAttribs.character}
\alias{setAttribs.html_elem}
\alias{setAttribs.html_group}
\alias{[<-.html_attributes}

\alias{css}
\alias{as.css}
\alias{style}
\alias{style<-}
\alias{setStyle}
\alias{setStyle.character}
\alias{setStyle.html_elem}
\alias{setStyle.html_group}
\alias{as.character.css}
\alias{print.css}
\alias{[<-.css}
\title{
Building Blocks for HTML Code
}
\description{
The functions described here form building blocks for
the \code{\link{format_html}} methods functions for \code{codebook},
\code{ftable}, \code{ftable_matrix}, and \code{mtable} objects, etc.

The most basic of these functions is \code{html}, which constructs an
object that represents a minimal piece of HTML code and is member of the
class \code{"html_elem"}. Unlike a character string containing HTML
code, the resulting code element can relatively easily modified using
other functions presented here. The actual code is created when the
function \code{as.character} is applied to these objects.

Longer sequences of HTML code can be prepared by 
concatenating them with \code{c}, or by \code{html_group},
or by applying \code{as.html_group} to a list of 
\code{"html_elem"} objects. All these result in objects
of class \code{"html_group"}.

Attributes (such as class, id etc.) of HTML elements can be added to the
call to \code{html}, but can also later recalled or modified with
\code{attribs} or \code{setAttribs}. An important attribute
is the style attribute, which can contain CSS styling. It can
be recalled or modified with \code{style} or \code{setStyle}. Styling
strings can also be created with \code{hmtl_style} or \code{as.css}

}
\usage{
html(tag, \dots, .content = NULL, linebreak = FALSE)
html_group(\dots)
as.html_group(x)

content(x)
content(x)<-value
setContent(x,value)

attribs(x)
attribs(x)<-value
setAttribs(x,\dots)
\method{setAttribs}{character}(x,\dots)
\method{setAttribs}{html_elem}(x,\dots)
\method{setAttribs}{html_group}(x,\dots)

css(\dots)
as.css(x)
style(x)
style(x) <- value
setStyle(x,\dots)
\method{setStyle}{character}(x,\dots)
\method{setStyle}{html_elem}(x,\dots)
\method{setStyle}{html_group}(x,\dots)
}
\arguments{
  \item{tag}{a character string that determines the opening and closing
      tags of the HTML element. (The closing tag is relevant only if
      the element has a content.)}
  \item{\dots}{optional further arguments, named or not.
      
      For \code{html}: named arguments create the attributes of the HTML element,
      unnamed arguments define the content of the HTML element, i.e. whatever 
      appears between opening and closing tags (e.g. \code{<p>} and \code{</p>}).
      Character strings, \code{"html_elem"}, or \code{"html_group"} objects can appear
      as content of a HTML element.
      
      For \code{setAttribs}: named arguments create the attributes of the HTML element,
      unnamed arguments are ignored.
      
      For \code{setStyle}: named arguments create the styling of the HTML element,
      unnamed arguments are ignored.
      
      For \code{html_group}: several objects of class \code{"html_elem"}
      or \code{"html_group"}.
      
      For \code{css}: named arguments (character strings!) 
      become components of a styling in CSS format.
      }
  \item{.content}{an optional character string, \code{"html_elem"}, or 
      \code{"html_group"} object}
  \item{linebreak}{a logical value or vector of length 2, determines
    whether linebreaks are inserted after the HTML tags.}
  \item{x}{an object. For \code{as.html_group}, this should be a list
      of objects of class \code{"html_elem"} or \code{"html_group"}. 
      For \code{content}, \code{setContent}, \code{attribs}, 
      \code{setAttribs}, \code{style}, \code{setStyle},
      this should be an object of class \code{"html_elem"} or \code{"html_group"}.  
  }
  \item{value}{an object of appropriate class. 
  
      For \code{content<-}: a character string, \code{"html_elem"}, or 
      \code{"html_group"} object, or a concatenation thereof.
      
      For \code{attribs<-} or \code{style<-}: a named character vector.
      }
}
\details{
  Objects created with \code{html} are lists with class attribute
  \code{"html_elem"} and components
  \describe{
      \item{tag}{a character string}
      \item{attributes}{a named character vector}
      \item{content}{a character vector, an \code{"html_elem"} or \code{"html_group"}
      object, or a list of such.
        }
      \item{linebreak}{a logical value or vector of length 2.}
  }
  Objects created with \code{html_group} or by concatenation
  of \code{"html_elem"} or \code{"html_group"}
      object
  are lists of such objects, with class attribute \code{"html_group"}.
}
\examples{

html("img")
html("img",src="test.png")
html("div",class="element",id="first","Sisyphus")
html("div",class="element",id="first",.content="Sisyphus")

div <- html("div",class="element",id="first",linebreak=c(TRUE,TRUE))
content(div) <- "Sisyphus"
div

tag <- html("tag",linebreak=TRUE)
attribs(tag)["class"] <- "something"
attribs(tag)["class"]
tag

style(tag) <- c(color="#342334")
style(tag)
tag

style(tag)["bg"] <- "white"
tag

setStyle(tag,bg="black")
setStyle(tag,c(bg="black"))

c(div,tag,tag)

c(
  c(div,tag),
  c(div,tag,tag)
)

c(
  c(div,tag),
  div,tag,tag
)

c(
  div,tag,
  c(div,tag,tag)
)

content(div) <- c(tag,tag,tag)
div


css("background-color"="black",
                  color="white")

as.css(c("background-color"="black",
                  color="white"))



Hello <- "Hello World!"
Hello <- html("p",Hello,linebreak=c(TRUE,TRUE))
style(Hello) <- c(color="white",
                  "font-size"="40px",
                  "text-align"="center")
     
Link <- html("a","More examples here ...",
             href="http://elff.eu/software/memisc",
             title="More examples here ...",
             style=css(color="white"),
             linebreak=c(TRUE,FALSE))
Link <- html("p"," (",Link,")",linebreak=c(TRUE,TRUE))
style(Link) <- c(color="white",
                 "font-size"="15px",
                 "text-align"="center")

Hello <- html("div",c(Hello,Link),linebreak=c(TRUE,TRUE))
style(Hello) <- c("background-color"="#160666",
                  padding="20px")
Hello

show_html(Hello)

}

\name{negative match}
\alias{\%nin\%}
\title{Negative Match}
\description{
  \code{\%nin\%}
  is a convenience operator:
  \code{x \%nin\% table}
  is equivalent to
  \code{!(x \%in\% table).}
}
\usage{
  x \%nin\% table
}
\arguments{
  \item{x}{the values to be matched}
  \item{table}{a values to be match against}
}
\value{
  A logical vector
}
\examples{
x <- sample(1:6,12,replace=TRUE)
x \%in\% 1:3
x \%nin\% 1:3
}

\name{Means}
\alias{Means}
\alias{Means.data.frame}
\alias{Means.numeric}
\alias{Means.formula}
\alias{as.data.frame.means.table}
\alias{as.data.frame.xmeans.table}
\title{Means for groups of observations}
\description{The function \code{Means()} creates a table of group
  means, optionally with standard errors, confidence intervals, and
  numbers of valid observations.}
\usage{
Means(data, \dots)
\method{Means}{data.frame}(data,
    by, weights=NULL, subset=NULL,
    default=NA,
    se=FALSE, ci=FALSE, ci.level=.95,
    counts=FALSE, \dots)
\method{Means}{formula}(data, subset, weights, \dots)
\method{Means}{numeric}(data, \dots)
\method{as.data.frame}{means.table}(x, row.names=NULL, optional=TRUE, drop=TRUE, \dots)
\method{as.data.frame}{xmeans.table}(x, row.names=NULL, optional=TRUE, drop=TRUE, \dots)
}
\arguments{
  \item{data}{an object usually containing data, or a formula.

    If \code{data} is a numeric vector or an object that can be coerced
    into a data frame, it is changed into a data frame and the data
    frame method of \code{Means()} is applied to it.

    If \code{data} is a formula, then a data frame is constructed from
    the variables in the formula and \code{Means} is applied to this
    data frame, while the formula is passed on as a \code{by=} argument.
  }
  \item{by}{a formula, a vector of variable names or a data frame or
    list of factors.

    If \code{by} is a vector of variable names,
    they are extracted from \code{data} to define the groups for which
    means are computed, while the variables for which the means are
    computed are those not named in \code{by}.

    If \code{by} is a data frame or a list of factors,
    these are used to defined the groups for which means are computed,
    while the variables for which the means are
    computed are those not in \code{by}.

    If \code{by} is a formula, its left-hand side determines the
    variables of which means are computed, while its right-hand side
    determines the factors that define the groups.
  }
  \item{weights}{an optional vector of weights, usually a variable in \code{data}.}
  \item{subset}{an optional logical vector to select observations,
    usually the result of an expression in variables from \code{data}.}
  \item{default}{a default value used for empty cells without
    observations.}
  \item{se}{a logical value, indicates whether standard errors should be
    computed.}
  \item{ci}{a logical value, indicates whether limits of confidence
    intervals should be computed.}
  \item{ci.level}{a number, the confidence level of the confidence interval}
  \item{counts}{a logical value, indicates whether numbers of valid
    observations should be reported.}
  \item{x}{for \code{as.data.frame()}, a result of \code{Means()}.}
  \item{row.names}{an optional character vector. This argmument presently is
    inconsequential and only included for reasons of compatiblity
    with the standard methods of \code{\link[base]{as.data.frame}}.}
  \item{optional}{an optional logical value. This argmument presently is
    inconsequential and only included for reasons of compatiblity
    with the standard methods of \code{\link[base]{as.data.frame}}.}
  \item{drop}{a logical value, determines whether "empty cells" should
    be dropped from the resulting data frame.}
  \item{\dots}{other arguments, either ignored or passed on to other
    methods where applicable.}
}
\value{
  An array that inherits classes "means.table" and "table". If
  \code{Means} was called with \code{se=TRUE} or \code{ci=TRUE}
  then the result additionally inherits class "xmeans.table".
}
\examples{
# Preparing example data
USstates <- as.data.frame(state.x77)
USstates <- within(USstates,{
    region <- state.region
    name <- state.name
    abb <- state.abb
    division <- state.division
})
USstates$w <- sample(runif(n=6),size=nrow(USstates),replace=TRUE)

# Using the data frame method
Means(USstates[c("Murder","division","region")],by=c("division","region"))
Means(USstates[c("Murder","division","region")],by=USstates[c("division","region")])
Means(USstates[c("Murder")],1)
Means(USstates[c("Murder","region")],by=c("region"))

# Using the formula method
# One 'dependent' variable
Means(Murder~1, data=USstates)
Means(Murder~division, data=USstates)
Means(Murder~division, data=USstates,weights=w)
Means(Murder~division+region, data=USstates)
as.data.frame(Means(Murder~division+region, data=USstates))

# Standard errors and counts
Means(Murder~division, data=USstates, se=TRUE, counts=TRUE)
drop(Means(Murder~division, data=USstates, se=TRUE, counts=TRUE))
as.data.frame(Means(Murder~division, data=USstates, se=TRUE, counts=TRUE))

# Confidence intervals
Means(Murder~division, data=USstates, ci=TRUE)
drop(Means(Murder~division, data=USstates, ci=TRUE))
as.data.frame(Means(Murder~division, data=USstates, ci=TRUE))

# More than one dependent variable
Means(Murder+Illiteracy~division, data=USstates)
as.data.frame(Means(Murder+Illiteracy~division, data=USstates))

# Confidence intervals
Means(Murder+Illiteracy~division, data=USstates, ci=TRUE)
as.data.frame(Means(Murder+Illiteracy~division, data=USstates, ci=TRUE))

# Some 'non-standard' but still valid usages:
with(USstates,
     Means(Murder~division+region,subset=region!="Northeast"))

with(USstates,
     Means(Murder,by=list(division,region)))
}

\name{trim_labels}
\alias{trim_labels}
\alias{trim_labels,item.vector-method}
\alias{trim_labels,data.set-method}
\title{Trim Codes from the Labels of an Item}
\description{
  Occasionally, labels of codes in a survey data sets (e.g. from the
  2016 American National Election Study) include a
  character representation of the codes being labelled. While there may
  be technical reasons for this, it is often inconvenient (e.g. if one
  wants to reorder the labelled codes). The function \code{trim_labels}
  trims the code representations (if they are present.) 
}
\usage{
trim_labels(x,\dots)
\S4method{trim_labels}{item.vector}(x,\dots)
\S4method{trim_labels}{data.set}(x,\dots)
}
\arguments{
  \item{x}{An object -- an "item" object or a "data.set" object}
  \item{\dots}{Further arguments, currently ignored}
}
\details{
  The "data.set" method applies the "item.vector" method to all the
  labelled items in the data set.

  The "item.vector" returns a copy of its argument with modified labels,
  where a label such as "1. First alternative" is changed into "First alternative".
}
\examples{
x <- as.item(sample(1:3,10,replace=TRUE),
             labels=c("1. One"=1,
                      "2. Two"=2,
                      "2. Three"=3))
y <- as.item(sample(1:2,10,replace=TRUE),
             labels=c("1. First category"=1,
                      "2. Second category"=2))
ds <- data.set(x,y)
x <- trim_labels(x)
codebook(x)
ds <- trim_labels(ds)
codebook(ds)
}

\name{Table}
\alias{Table}
\alias{Table,atomic-method}
\alias{Table,factor-method}
\alias{Table,item.vector-method}
\title{One-Dimensional Table of Frequences and/or Percentages}
\description{
  \code{Table} is a generic function that
  produces a table of counts or weighted counts
  and/or the corresponding percentages of an atomic vector,
  factor or \code{"item.vector"} object.
  This function is intended for use with
  \code{\link{Aggregate}} or \code{\link{genTable}}.
  The \code{"item.vector"} method is the workhorse
  of \code{\link{codebook}}.
  }
\usage{
\S4method{Table}{atomic}(x,weights=NULL,counts=TRUE,percentage=FALSE,\dots)
\S4method{Table}{factor}(x,weights=NULL,counts=TRUE,percentage=FALSE,\dots)
\S4method{Table}{item.vector}(x,weights=NULL,counts=TRUE,percentage=(style=="codebook"),
              style=c("table","codebook","nolabels"),
              include.missings=(style=="codebook"),
              missing.marker=if(style=="codebook") "M" else "*",\dots)
}
\arguments{
  \item{x}{an atomic vector, factor or \code{"item.vector"} object}
  \item{counts}{logical value, should the table contain counts?}
  \item{percentage}{logical value, should the table contain percentages?
  Either the \code{counts} or the \code{percentage} arguments or both
  should be \code{TRUE}. }
  \item{style}{character string, the style of the names or rownames of the table.}
  \item{weights}{a numeric vector of weights of the same length as \code{x}.}
  \item{include.missings}{a logical value; should missing values included into the table?}
  \item{missing.marker}{a character string, used to mark missing values
    in the table (row)names.}
  \item{\dots}{other, currently ignored arguments.}
}
\value{
  The atomic vector and factor methods return either a vector
  of counts or vector of percentages or a matrix of counts and percentages.
  The same applies to the \code{"item.vector"} vector method unless
  \code{include.missing=TRUE} and \code{percentage=TRUE},
  in which case total percentages and percentages of valid values
  are given.
}
\examples{
  with(as.data.frame(UCBAdmissions),Table(Admit,Freq))
  Aggregate(Table(Admit,Freq)~.,data=UCBAdmissions)

  A <- sample(c(1:5,9),size=100,replace=TRUE)
  labels(A) <- c(a=1,b=2,c=3,d=4,e=5,dk=9)
  missing.values(A) <- 9
  Table(A,percentage=TRUE)
}
\keyword{univar}

\name{annotations}
\alias{annotation}
\alias{annotation,ANY-method}
\alias{annotation,item-method}
\alias{annotation,data.set-method}
\alias{annotation<-}
\alias{annotation<-,ANY,NULL-method}
\alias{annotation<-,ANY,character-method}
\alias{annotation<-,ANY,annotation-method}
\alias{annotation<-,item,annotation-method}
\alias{annotation<-,vector,annotation-method}
\alias{description}
\alias{description,data.set-method}
\alias{description,importer-method}
\alias{description,data.frame-method}
\alias{description,tbl_df-method}
\alias{description<-}
\alias{wording}
\alias{wording<-}
\alias{annotation-class}
\alias{show,annotation-method}
\title{Adding Annotations to Objects}
\description{
  Annotations, that is, objects of class \code{"annotation"},
  are character vectors with all their elements named.
  Only one method is defined for this subclass of character vectors,
  a method for \code{\link{show}}, that shows the annotation in
  a nicely formatted way. Annotations of an object can be obtained
  via the function \code{annotation(x)} and can be set via
  \code{annotation(x)<-value}.

  Elements of an annotation with names \code{"description"}
  and \code{"wording"} have a special meaning.
  The first kind can be obtained and set via
  \code{description(x)} and \code{description(x)<-value},
  the second kind can be obtained via
  \code{wording(x)} and \code{wording(x)<-value}.
  \code{"description"} elements are used in way the "variable labels"
  are used in SPSS and Stata. \code{"wording"} elements of annotation
  objects are meant to contain the question wording of a questionnaire
  item represented by an \code{"item"} objects.
  These elements of annotations are treated in a special way
  in the output of the \code{coodbook} function.
}
\usage{
annotation(x)
\S4method{annotation}{ANY}(x)
\S4method{annotation}{item}(x)
\S4method{annotation}{data.set}(x)
annotation(x)<-value
\S4method{annotation}{ANY,character}(x)<-value
\S4method{annotation}{ANY,annotation}(x)<-value
\S4method{annotation}{item,annotation}(x)<-value
\S4method{annotation}{vector,annotation}(x)<-value

description(x)
description(x)<-value

wording(x)
wording(x)<-value

\S4method{description}{data.set}(x)
\S4method{description}{importer}(x)
\S4method{description}{data.frame}(x)
\S4method{description}{tbl_df}(x)
}
\arguments{
  \item{x}{an object}
  \item{value}{a character or annotation object}
}
\value{
\code{annotation(x)} returns an object of class \code{"annotation"},
which is a named character.
\code{description(x)} and \code{wording(x)} each usually return a character string.
If \code{description(x)} is applied to a \code{\link{data.set}} or an \code{\link{importer}} object,
however, a character vector is returned, which is named after the
variables in the data set or the external file.
}
\examples{
vote <- sample(c(1,2,3,8,9,97,99),size=30,replace=TRUE)
labels(vote) <- c(Conservatives         =  1,
                    Labour                =  2,
                    "Liberal Democrats"   =  3,
                    "Don't know"          =  8,
                    "Answer refused"      =  9,
                    "Not applicable"      = 97,
                    "Not asked in survey" = 99
                    )
missing.values(vote) <- c(97,99)
description(vote) <- "Vote intention"
wording(vote) <- "If a general election would take place next tuesday,
                    the candidate of which party would you vote for?"
annotation(vote)
annotation(vote)["Remark"] <- "This is not a real questionnaire item, of course ..."
codebook(vote)
}

\name{Write}
\alias{Write}
\alias{Write.codebook}
\alias{Write.descriptions}

\title{Write Codebooks and Variable Descriptions into a Text File}
\description{
   This is a convenience function to facilitate the creation of data set documents
   in text files.
  }
\usage{
Write(x,\dots)
\S3method{Write}{codebook}(x,file=stdout(),\dots)
\S3method{Write}{descriptions}(x,file=stdout(),\dots)
}
\arguments{
  \item{x}{a "codebook" or "descriptions" object.}
  \item{file}{ a connection, see \link{connections}.}
  \item{\dots}{ further arguments, ignored or passed on to particular methods.}
}
\keyword{misc}

\name{genTable}
\alias{Aggregate}
\alias{genTable}
\title{Generic Tables and Data Frames of Descriptive Statistics}
\description{
  \code{genTable} creates a table of arbitrary summaries conditional on
  given values of independent variables given by a formula.

  \code{Aggregate} does the same, but returns a \code{data.frame} instead.

  \code{fapply} is a generic function that dispatches on its \code{data}
  argument. It is called internally by \code{Aggregate} and \code{genTable}.
  Methods for this function can be used to adapt \code{Aggregate} and
  \code{genTable} to data sources other than data frames.
  }
\usage{
Aggregate(formula, data=parent.frame(), subset=NULL,
          names=NULL, addFreq=TRUE, drop = TRUE, as.vars=1,
          \dots)

genTable(formula, data=parent.frame(), subset=NULL,
         names=NULL, addFreq=TRUE,\dots)

}
\arguments{
  \item{formula}{a formula. The right hand side includes one or more
  grouping variables separated by '+'. These may be factors, numeric,
  or character vectors. The left hand side may be empty,
  a numerical variable, a factor, or an expression.
  See details below.}
  \item{data}{an environment or data frame or an object coercable into a data frame.}
  \item{subset}{an optional vector specifying a subset of observations
    to be used.}
  \item{names}{an optional character vector giving names to the
    result(s) yielded by the expression on the left hand side of \code{formula}.
    This argument may be redundant if the left hand side results in is a named vector.
    (See the example below.)}
  \item{addFreq}{a logical value. If \code{TRUE} and
      \code{data} is a table or a data frame with a variable
      named "Freq", a call to
      \code{table}, \code{\link{Table}}, \code{\link{percent}}, or \code{\link{nvalid}}
      is supplied by an additional argument \code{Freq}
      and a call to \code{table} is translated into
      a call to \code{Table}.
    }
  \item{drop}{a logical value. If \code{TRUE}, empty groups (i.e. when
    there are no observations in the aggregated data frame that contain
    the defining combination of values or factor levels of the
    conditioning variables in \code{by}) are dropped from the result of
    \code{Aggregate}. Otherwise, result are filled with \code{NA}, where appropriate.
  }
  \item{as.vars}{an integer; relevant only if the left hand side of the formula returns
    an array or a matrix - which dimension (rows, columns, or layers etc.) will transformed to
    variables? Defaults to columns in case of matrices and to the highest dimensional extend
    in case of arrays.}
  \item{\dots}{further arguments, passed to methods or ignored.}
}
\details{
    If an expression is given as left hand side of the formula, its
    value is computed for any combination of values of the values on the
    right hand side. If the right hand side is a dot, then all
    variables in \code{data} are added to the right hand side of the
    formula.

    If no expression is given as left hand side,
    then the frequency counts for the respective
    value combinations of the right hand variables are computed.

    If a single factor is on the left hand side, then the left hand side is
    translated into an appropriate
    call to \code{table()}. Note that also in this case \code{addFreq} takes effect.

    If a single numeric variable is on the left hand side, frequency
    counts weighted by this variable are computed. In these cases,
    \code{genTable} is equivalent to \code{\link[stats]{xtabs}} and
    \code{Aggregate} is equivalent to \code{as.data.frame(xtabs(\dots))}.
}
\value{
    \code{Aggregate}
    results in a data frame with conditional summaries and unique value combinations
    of conditioning variables.

    \code{genTable} returns a \link{table}, that is, an array with class \code{"table"}.
}
\seealso{\link{aggregate.data.frame}, \link{xtabs}}
\examples{
ex.data <- expand.grid(mu=c(0,100),sigma=c(1,10))[rep(1:4,rep(100,4)),]
ex.data <- within(ex.data,
                  x<-rnorm(
                    n=nrow(ex.data),
                    mean=mu,
                    sd=sigma
                    )
                  )

Aggregate(~mu+sigma,data=ex.data)
Aggregate(mean(x)~mu+sigma,data=ex.data)
Aggregate(mean(x)~mu+sigma,data=ex.data,name="Average")
Aggregate(c(mean(x),sd(x))~mu+sigma,data=ex.data)
Aggregate(c(Mean=mean(x),StDev=sd(x),N=length(x))~mu+sigma,data=ex.data)
genTable(c(Mean=mean(x),StDev=sd(x),N=length(x))~mu+sigma,data=ex.data)

Aggregate(table(Admit)~.,data=UCBAdmissions)
Aggregate(Table(Admit,Freq)~.,data=UCBAdmissions)
Aggregate(Admit~.,data=UCBAdmissions)
Aggregate(percent(Admit)~.,data=UCBAdmissions)
Aggregate(percent(Admit)~Gender,data=UCBAdmissions)
Aggregate(percent(Admit)~Dept,data=UCBAdmissions)
Aggregate(percent(Gender)~Dept,data=UCBAdmissions)
Aggregate(percent(Admit)~Dept,data=UCBAdmissions,Gender=="Female")
genTable(percent(Admit)~Dept,data=UCBAdmissions,Gender=="Female")
}
\keyword{misc}

\name{as.array}
\alias{as.array}
\alias{as.array,data.frame-method}
\title{Converting Data Frames into Arrays}
\description{
  The \code{as.array} for data frames
  takes all factors in a data frame and uses them
  to define the dimensions of the resulting array,
  and fills the array with the values of
  the remaining numeric variables.

  Currently, the data frame must contain all
  combinations of factor levels.
}
\usage{
\S4method{as.array}{data.frame}(x,data.name=NULL,\dots)
}
\arguments{
  \item{x}{a data frame}
  \item{data.name}{
    a character string, giving the name
    attached to the dimension
    that corresponds to the
    numerical variables in the data frame
    (that is, the \code{name} attached to
    the corresponding element of the
    \code{dimnames} list).
    }
  \item{\dots}{other arguments, ignored.}
}
\value{
  An array
}
\examples{
BerkeleyAdmissions <- to.data.frame(UCBAdmissions)
BerkeleyAdmissions
as.array(BerkeleyAdmissions,data.name="Admit")
try(as.array(BerkeleyAdmissions[-1,],data.name="Admit"))
}

\name{recode}
\alias{recode}
\alias{recode,vector-method}
\alias{recode,item-method}
\alias{recode,factor-method}
\title{Recode Items, Factors and Numeric Vectors}
\description{
  \code{recode} substitutes old values of a factor or a numeric
  vector by new ones, just like the recoding facilities in some
  commercial statistical packages.
}
\usage{
recode(x,\dots,
       copy=getOption("recode_copy",identical(otherwise,"copy")),
       otherwise=NA)
\S4method{recode}{vector}(x,\dots,
    copy=getOption("recode_copy",identical(otherwise,"copy")),
    otherwise=NA)
\S4method{recode}{factor}(x,\dots,
    copy=getOption("recode_copy",identical(otherwise,"copy")),
    otherwise=NA)
\S4method{recode}{item}(x,\dots,
    copy=getOption("recode_copy",identical(otherwise,"copy")),
    otherwise=NA)
}
\arguments{
  \item{x}{An object}
  \item{\dots}{One or more assignment expressions, each 
    of the form \code{new.value <- old.values}.
    \code{new.value} should be a scalar numeric value
    or character string. If one of the \code{new.value}s
    is a character string, the return value
    of \code{recode} will be a factor and each \code{new.value}
    will be coerced to a character string that labels a level of the factor.

    Each \code{old.value} in an assignment expression may be a
    (numeric or character) vector. If \code{x} is numeric such an
    assignment expression may have the form \code{new.value <- range(lower,upper)}
    In that case, values between \code{lower} and \code{upper} are exchanged by
    \code{new.value}. If one of the arguments to \code{range} is \code{min},
    it is substituted by the minimum of \code{x}.
    If one of the arguments to \code{range} is \code{max},
    it is substituted by the maximum of \code{x}.

    In case of the method for \code{labelled} vectors, the \emph{tags} of
    arguments of the form \code{tag = new.value <- old.values}
    will define the labels of the new codes.

    If the \code{old.values} of different assignment expressions overlap,
    an error will be raised because the recoding is ambigous.
  }
  \item{copy}{logical; should those values of \code{x} not given an
    explicit new code copied into the resulting vector?}
  \item{otherwise}{a character string or some other value
    that the result may obtain. If equal to \code{NA} or \code{"NA"},
    original codes not given an explicit new code are recoded into
    \code{NA}. If equal to \code{"copy"},
    original codes not given an explicit new code are copied.
    }
}

\value{
  A numerical vector, factor or an \code{item} object.
}

\details{
\code{recode} relies on the lazy evaluation mechanism of \emph{R}:
Arguments are not evaluated until required by the function they are given to.
\code{recode} does not cause arguments that appear in \code{\dots} to be evaluated.
Instead, \code{recode} parses the \code{\dots} arguments. Therefore, although
expressions like \code{1 <- 1:4} would cause an error action, if evaluated
at any place elsewhere in \emph{R}, they will not cause an error action,
if given to \code{recode} as an argument. However, a call of the
form \code{recode(x,1=1:4)}, would be a syntax error.

If John Fox' package "car" is installed, \code{recode} will also be callable
with the syntax of the \code{recode} function of that package.
}

\seealso{
  \code{recode} of package "car".
}

\examples{
x <- as.item(sample(1:6,20,replace=TRUE),
        labels=c( a=1,
                  b=2,
                  c=3,
                  d=4,
                  e=5,
                  f=6))
print(x)


codebook(
    recode(x,
           a = 1 <- 1:2,
           b = 2 <- 4:6))


codebook(
    recode(x,
           a = 1 <- 1:2,
           b = 2 <- 4:6,
           copy = TRUE))

# Note the handling of labels if the recoding rules are bijective
codebook(
    recode(x,
           1 <- 2,
           2 <- 1,
           copy=TRUE))

codebook(
    recode(x,
           a = 1 <- 2,
           b = 2 <- 1,
           copy=TRUE))


# A recoded version of x is returned
# containing the values 1, 2, 3, which are
# labelled as "A", "B", "C".
recode(x,
  A = 1 <- range(min,2),
  B = 2 <- 3:4,
  C = 3 <- range(5,max), # this last comma is ignored
  )

# This causes an error action: the sets
# of original values overlap.
try(recode(x,
  A = 1 <- range(min,2),
  B = 2 <- 2:4,
  C = 3 <- range(5,max)
  ))

recode(x,
  A = 1 <- range(min,2),
  B = 2 <- 3:4,
  C = 3 <- range(5,6),
  D = 4 <- 7
  )
  
# This results in an all-missing vector:
recode(x,
  D = 4 <- 7,
  E = 5 <- 8
  )

f <- as.factor(x)
x <- as.integer(x)

recode(x,
  1 <- range(min,2),
  2 <- 3:4,
  3 <- range(5,max)
  )

# This causes another error action:
# the third argument is an invalid
# expression for a recoding.
try(recode(x,
  1 <- range(min,2),
  3:4,
  3 <- range(5,max)
  ))

# The new values are character strings,
# therefore a factor is returned.
recode(x,
  "a" <- range(min,2),
  "b" <- 3:4,
  "c" <- range(5,6)
  )
  
recode(x,
  1 <- 1:3,
  2 <- 4:6
  )
  
recode(x,
  4 <- 7,
  5 <- 8,
  otherwise = "copy"
  )

recode(f,
  "A" <- c("a","b"),
  "B" <- c("c","d"),
  otherwise="copy"
  )

recode(f,
  "A" <- c("a","b"),
  "B" <- c("c","d"),
  otherwise="C"
  )
 
recode(f,
  "A" <- c("a","b"),
  "B" <- c("c","d")
  )

DS <- data.set(x=as.item(sample(1:6,20,replace=TRUE),
        labels=c( a=1,
                  b=2,
                  c=3,
                  d=4,
                  e=5,
                  f=6)))
print(DS)

DS <- within(DS,{
    xf <- recode(x,
                 "a" <- range(min,2),
                 "b" <- 3:4,
                 "c" <- range(5,6)
                 )
    xn <- x@.Data
    xc <- recode(xn,
                 "a" <- range(min,2),
                 "b" <- 3:4,
                 "c" <- range(5,6)
                 )
    xc <- as.character(x)
    xcc <- recode(xc,
                  1 <- letters[1:2],
                  2 <- letters[3:4],
                  3 <- letters[5:6]
                  )
})

DS

DS <- within(DS,{
    xf <- recode(x,
                 "a" <- range(min,2),
                 "b" <- 3:4,
                 "c" <- range(5,6)
                 )
    x1 <- recode(x,
                 1 <- range(1,2),
                 2 <- range(3,4),
                 copy=TRUE
                 )
    xf1 <- recode(x,
                 "A" <- range(1,2),
                 "B" <- range(3,4),
                 copy=TRUE
                 )
})
DS
codebook(DS)

DF <- data.frame(x=rep(1:6,4,replace=TRUE))
DF <- within(DF,{
    xf <- recode(x,
                 "a" <- range(min,2),
                 "b" <- 3:4,
                 "c" <- range(5,6)
                 )
    x1 <- recode(x,
                 1 <- range(1,2),
                 2 <- range(3,4),
                 copy=TRUE
                 )
    xf1 <- recode(x,
                 "A" <- range(1,2),
                 "B" <- range(3,4),
                 copy=TRUE
                 )
    xf2 <- recode(x,
                 "B" <- range(3,4),
                 "A" <- range(1,2),
                 copy=TRUE
                 )
})
DF
codebook(DF)

}
\keyword{manip}

\name{mtable_format_delim}
\alias{mtable_format_delim}
\title{Format for 'mtable' Objects for Writing into File}
\description{
  \code{mtable_mtable_print} formats 'mtable' in a way suitable for output into a file
  with \code{write.table}
  }
\usage{
mtable_format_delim(x,
          colsep="\t",
          rowsep="\n",
          interaction.sep = " x ",
          \dots
          )
}
\arguments{
     \item{x}{an object of class \code{mtable}}
     \item{colsep}{a character string which seperates the columns in the output.}
     \item{rowsep}{a character string which seperates the rows in the output.}
     \item{interaction.sep}{a character string that separates factors that are involved
        in an interaction effect}
    \item{\dots}{further arguments, ignored. }
}
\value{
    A character string.
}
\keyword{misc}

\name{styles}
\alias{coef.style}
\alias{factor.style}
\alias{setCoefTemplate}
\alias{getCoefTemplate}
\alias{getSummaryTemplate}
\alias{setSummaryTemplate}
\alias{summaryTemplate}
\title{Formatting Styles for Coefficients, Factor Contrasts, and Summary Statistics}
\description{
  Methods for setting and getting templates for formatting
  model coefficients and summaries for use in \code{\link{mtable}}.
}
\usage{
setCoefTemplate(...)
getCoefTemplate(style)
getSummaryTemplate(x)
setSummaryTemplate(...)
summaryTemplate(x)
  }
\arguments{
  \item{...}{sevaral tagged arguments; in case of \code{setCoefTemplate} the
    tags specify the \code{coef.style}s, in case of \code{setSummaryTemplate}
    they specify model classes.
    The associated values are \link{template}s.}
  \item{style}{a character string with the name of a coefficient style,
      if left empty, all coefficient templates are returned.  }
  \item{x}{a model or a name of a model class, for example \code{"lm"} or \code{"glm"};
    if left empty, all summary templates are returned.}
}
\details{
  The style in which model coefficients are formatted by \code{\link{mtable}}
  is by default selected from the \code{coef.style} setting of \code{\link{options}},
  "factory-fresh" setting being \code{options(coef.style="default")}.

  The appearance of factor levels in an \code{\link{mtable}}
  can be influenced by the \code{factor.style} setting of \code{\link{options}}.
  The "factory-fresh" setting is \code{options(factor.style="($f): ($l)")},
  where \code{($f)} stands for the factor name and \code{($l)} stands
  for the factor level. In case of treatment contrasts, the baseline level
  will also appear in an \code{\link{mtable}} separated from the current
  factor level by the \code{baselevel.sep} setting of \code{\link{options}}.
  The "factory-fresh" setting is \code{options(baselevel.sep="-")},

  Users may specify additional coefficient styles by a call to \code{setCoefTemplate}.
  
  In order to adapt the display of summary statistics of other model classes, users need to
  set a template for model summaries via a call to \code{setSummaryTemplate}
  or to define a method of the generic function \code{summaryTemplate}.
}

\keyword{misc}

\name{view}
\alias{view}
\alias{viewPrep}
\alias{viewPrep.default}
\alias{viewPrep.data.set}
\alias{viewPrep.data.frame}
\alias{viewPrep.descriptions}
\alias{viewPrep.codeplan}
\alias{viewPrep.importer}
\title{A Generic Viewing Function}
\description{
  The function \code{view} provides generic interface to the non-generic
  function \code{View}.

  In contrast to the implementation of \code{View} provided by either
  basic \emph{R} or \emph{RStudio}, this function can be extended to
  handle new kinds of objects by defining \code{viewPrep} methods for
  them. Further, \code{view} can be adapted to other GUIs by specifying
  the \code{"vfunc"} option or the \code{vfunc=} optional argument.
    
  Internally, \code{view} usues the generic function \code{viewPrep}
  to prepare data so it can be passed on to the (non-generic) function
  \code{View} or (optionally) a different graphical user interface
  function that can be used to display matrix- or data frame-like
  objects.  

  The \code{vfunc} argument determines how the result of \code{viewPrep}
  is displayed. Its default is the function \code{View}, but an
  alternative is \code{\link{view_html}} which creates and displays an HTML grid.
}
\usage{
view(x,
     title=deparse(substitute(x)),
     vfunc=getOption("vfunc","View"),
     \dots)

# The internal generic, not intended to be used by the end-user.
viewPrep(x,title,\dots)
\method{viewPrep}{data.set}(x,title,\dots)
\method{viewPrep}{data.frame}(x,title,\dots)
\method{viewPrep}{descriptions}(x,title,\dots)
\method{viewPrep}{codeplan}(x,title,compact=FALSE,\dots)
\method{viewPrep}{importer}(x,title,compact=TRUE,\dots)
}
\arguments{
  \item{x}{an object, e.g. a data frame, data.set, or importer.
  }
  \item{title}{an optional character string; shown as the title of the
    display.}
  \item{vfunc}{a character string; a name of a GUI function to call
    with the results of \code{viewPrep()}}
  \item{compact}{a logical value; should the codeplan be shown in a
    compact form - one line per variable - or in a more expanive form -
    one line per labelled value?}
  \item{\dots}{further arguments; \code{view()} passes them on to
    \code{viewPrep}.}
}
\examples{
\dontrun{
    example(data.set)
    view(Data)
    view(description(Data))
    view(codeplan(Data))
    # Note that this file is *not* included in the package
    # and has to be obtained from GESIS in order to run the 
    # following
    ZA7500sav <- spss.file("ZA7500_v2-0-0.sav")
    view(ZA7500sav)
}
}

\name{assign_if}
\alias{assign_if}
\alias{\%if\%}
\title{Assign a values to a variable for instances where a condition is
  met}
\description{
  The \code{\%if\%} operator allows to assign values to a variable only if
    a condition is met i.e. results in \code{TRUE}. It is supposed to
    be used similar to the \code{replace ... if} construct in Stata.
}
\usage{
expr \%if\% condition
# For example
# (variable <- value) \%if\% (other_variable == 0)
}
\arguments{
  \item{expr}{An expression that assigns a value to variable}
  \item{condition}{A logical vector or a an expression that evaluates
  to a logical vector}
}
\details{
  The 'value' that is assigned to the variable in \code{expr}
  should either be a scalar, a vector with as many elements as the
  condition vector has, or as many elements as the number of elements
  in the condition vector that are equal (or evaluate to) \code{TRUE}. 
}
\examples{
(test_var <- 1) \%if\% (1:7 > 3)

test_var

(test_var <- 2) \%if\% (1:7 <= 3)
test_var

(test_var <- 100*test_var) \%if\% (1:7\%\%2==0)
test_var

# This creates a warning about non-matching lengths.
(test_var <- 500:501) \%if\% (1:7 <= 3)
test_var

(test_var <- 501:503) \%if\% (1:7 <= 3)
test_var

(test_var <- 401:407) \%if\% (1:7 <= 3)
test_var
}

\name{items}
\alias{item}
\alias{items}
\alias{item-class}
\alias{item.vector-class}
\alias{integer.item-class}
\alias{double.item-class}
\alias{numeric.item-class}
\alias{character.item-class}
\alias{datetime.item-class}
\alias{Date.item-class}
\alias{as.item}
\alias{as.item,numeric-method}
\alias{as.item,logical-method}
\alias{as.item,character-method}
\alias{as.item,factor-method}
\alias{as.item,ordered-method}
\alias{as.item,POSIXct-method}
\alias{as.item,Date-method}
\alias{as.item,double.item-method}
\alias{as.item,integer.item-method}
\alias{as.item,character.item-method}
\alias{as.item,datetime.item-method}
\alias{as.item,Date.item-method}
\alias{[,item.vector,numeric,missing,missing-method}
\alias{[,item.vector,logical,missing,missing-method}
\alias{[,datetime.item,numeric,missing,missing-method}
\alias{[,datetime.item,logical,missing,missing-method}
\alias{str.double.item}
\alias{str.integer.item}
\alias{str.character.item}
\alias{str.datetime.item}
\alias{unique,item.vector-method}
\alias{summary,item.vector-method}
\alias{summary,datetime.item-method}
\alias{summary,Date.item-method}
\alias{format,item.vector-method}
\alias{show,item.vector-method}
\alias{format,datetime.item-method}
\alias{format,Date.item-method}
\alias{show,datetime.item-method}
\alias{show,Date.item-method}
\alias{print,item.vector-method}
\alias{print,datetime.item-method}
\alias{print,Date.item-method}
\alias{Compare,numeric.item,character-method}
\alias{Compare,character,numeric.item-method}
\alias{Arith,numeric.item,numeric.item-method}
\alias{Arith,numeric.item,numeric-method}
\alias{Arith,numeric,numeric.item-method}
\alias{Math,numeric.item-method}
\alias{Math2,numeric.item-method}
\alias{Summary,numeric.item-method}
\alias{\%in\%,numeric.item,character-method}
\alias{rep,item.vector-method}
\alias{weighted.mean,item.vector-method}

\title{Survey Items}
\description{
  Objects of class \code{item} are data vectors with additional information
  attached to them like ``value labels'' and ``user-defined missing values''
  known from software packages like SPSS or Stata.

  The class \code{item} is intended to facilitate data management of
  survey data. Objects in this class should \emph{not} directly used
  in data analysis. Instead they should changed into "ordinary" vectors
  or factors before. For this see the documentation for \code{\link{as.vector,item-method}}.
  }
\usage{
## The constructor for objects of class "item"
## more convenient than new("item",...)
\S4method{as.item}{numeric}(x,
  labels=NULL,  missing.values=NULL,
  valid.values=NULL,  valid.range=NULL,
  value.filter=NULL,  measurement=NULL,
  annotation=attr(x,"annotation"), \dots
  )
\S4method{as.item}{character}(x,
  labels=NULL,  missing.values=NULL,
  valid.values=NULL,  valid.range=NULL,
  value.filter=NULL,  measurement=NULL,
  annotation=attr(x,"annotation"), \dots
  )

\S4method{as.item}{logical}(x,\dots)
# x is first coerced to integer,
# arguments in ... are then passed to the "numeric"
# method.

\S4method{as.item}{factor}(x,\dots)
\S4method{as.item}{ordered}(x,\dots)
\S4method{as.item}{POSIXct}(x,\dots)

\S4method{as.item}{double.item}(x,
  labels=NULL,  missing.values=NULL,
  valid.values=NULL,  valid.range=NULL,
  value.filter=NULL,  measurement=NULL,
  annotation=attr(x,"annotation"), \dots
  )

\S4method{as.item}{integer.item}(x,
  labels=NULL,  missing.values=NULL,
  valid.values=NULL,  valid.range=NULL,
  value.filter=NULL,  measurement=NULL,
  annotation=attr(x,"annotation"), \dots
  )

\S4method{as.item}{character.item}(x,
  labels=NULL,  missing.values=NULL,
  valid.values=NULL,  valid.range=NULL,
  value.filter=NULL,  measurement=NULL,
  annotation=attr(x,"annotation"), \dots
  )

\S4method{as.item}{datetime.item}(x,
  labels=NULL,  missing.values=NULL,
  valid.values=NULL,  valid.range=NULL,
  value.filter=NULL,  measurement=NULL,
  annotation=attr(x,"annotation"), \dots
  )

}

\arguments{
  \item{x}{for \code{as.item} methods,
  any atomic vector; 
  for the \code{unique}, 
  \code{summary}, \code{str}, \code{print}, \code{[}, and \code{<-}
  methods, a vector with class \code{labelled}.
  }
  \item{labels}{a named vector of the same mode as \code{x}.}
  \item{missing.values}{either a vector of the same mode as \code{x},
    or a list with components \code{"values"},
    vector of the same mode as \code{x} (which defines individual missing values)
    and \code{"range"} a matrix with two rows with
    the same mode as \code{x} (which defines a range of missing values),
    or an object of class \code{"missing.values"}.
    }
  \item{valid.values}{either a vector of the same mode as \code{x},
    defining those values of \code{x} that are to be considered as valid,
    or an object of class \code{"valid.values"}.}
  \item{valid.range}{either a vector of the same mode as \code{x} and length 2,
    defining a range of valid values of \code{x},
    or an object of class \code{"valid.range"}.}
  \item{value.filter}{an object of class \code{"value.filter"}, that is, of
     classes \code{"missing.values"}, \code{"valid.values"}, or \code{"valid.range"}.}
  \item{measurement}{level of measurement; one of "nominal", "ordinal", "interval", or "ratio".}
  \item{annotation}{a named character vector,
  or an object of class \code{"annotation"} }
  \item{\dots}{further arguments, ignored.}
}

\seealso{
  \code{\link{annotation}}
  \code{\link{labels}}
  \code{\link{value.filter}}
}
\examples{
  x <- as.item(rep(1:5,4),
      labels=c(
          "First"      = 1,
          "Second"     = 2,
          "Third"      = 3,
          "Fourth"     = 4,
          "Don't know" = 5
        ),
      missing.values=5,
      annotation = c(
        description="test"
      ))
  str(x)
  summary(x)
  as.numeric(x)

  test <- as.item(rep(1:6,2),labels=structure(1:6,
                                      names=letters[1:6]))
  test
  test == 1
  test != 1
  test == "a"
  test != "a"
  test == c("a","z")
  test != c("a","z")
  test %in% 1:3
  test %in% c("a","c")

  codebook(test)

  Test <- as.item(rep(letters[1:6],2),
                    labels=structure(letters[1:6],
                                     names=LETTERS[1:6]))
  Test
  Test == "a"
  Test != "a"
  Test == "A"
  Test != "A"
  Test == c("a","z")
  Test != c("a","z")
  Test %in% c("A","C")
  Test %in% c("a","c")

  as.factor(test)
  as.factor(Test)
  as.numeric(test)
  as.character(test)
  as.character(Test)

  as.data.frame(test)[[1]]
}
\keyword{manip}

\name{Utility classes}
\alias{named.list}
\alias{item.list}
\alias{named.list-class}
\alias{item.list-class}
\alias{initialize,item.list-method}
\alias{initialize,named.list-method}
\alias{coerce,data.set,named.list-method}
\alias{show,named.list-method}
\alias{atomic-class}
\alias{double-class}
\title{Named Lists, Lists of Items, and Atomic Vectors}
\description{
  The classes "named.list" and "item.list" are merely some
  'helper classes' for the construction of the classes "data.set"
  and "importer".

  Class "named.list" extends the basic class "list" by an additional
  slot "names". Its \code{initialize} method assures that the names
  of the list are unique.
  
  Class "item.list" extends the class "named.list", but does not
  add any slots. From "named.list" it differs only by the 
  \code{initialize} method, which calls that for "named.list"
  and makes sure that all elements of the list belong to 
  class "\link{item}".
  
  Classes "atomic" and "double" are merely used for
  method selection.
}
\examples{
new("named.list",a=1,b=2)

# This should generate an error, since the names
# are not unique.
try(new("named.list",a=1,a=2))

# Another error, one name is missing.
try(new("named.list",a=1,2))

# Also an error, the resulting list would be unnamed.
try(new("named.list",1,2))


new("item.list",a=1,b=2)

# Also an error: "item.list"s are "named.lists",
# and here the names would be non-unique.
try(new("item.list",a=1,a=2))
}

\name{toLatex}
\alias{toLatex.default}
\alias{toLatex.ftable}
\alias{toLatex.matrix}
\alias{toLatex.data.frame}
\alias{toLatex.ftable_matrix}
\title{Additional Methods for LaTeX Representations for R objects}
\description{
  Methods for the generic function \code{\link[utils]{toLatex}} of package ``\link{utils}''
  are provided for generating LaTeX representations
  of matrices and flat contingency tables (see \code{\link{ftable}}). Also a default method is defined
  that coerces its first argument into a matrix and applies
  the matrix method.
}
\usage{
\method{toLatex}{default}(object,\dots)

\method{toLatex}{matrix}(object,
    show.titles=TRUE,
    show.vars=FALSE,
    show.xvar=show.vars,
    show.yvar=show.vars,
    digits=if(is.table(object)) 0 else getOption("digits"),
    format="f",
    useDcolumn=getOption("useDcolumn",TRUE),
    colspec=if(useDcolumn)
                paste("D{.}{",LaTeXdec,"}{",ddigits,"}",sep="")
             else "r",
    LaTeXdec=".",
    ddigits=digits,
    useBooktabs=getOption("useBooktabs",TRUE),
    toprule=if(useBooktabs) "\\\\toprule" else "\\\\hline\\\\hline",
    midrule=if(useBooktabs) "\\\\midrule" else "\\\\hline",
    cmidrule=if(useBooktabs) "\\\\cmidrule" else "\\\\cline",
    bottomrule=if(useBooktabs) "\\\\bottomrule" else "\\\\hline\\\\hline",
    toLatex.escape.tex=getOption("toLatex.escape.tex",FALSE),
    \dots)


\method{toLatex}{data.frame}(object,
    digits=getOption("digits"),
    format="f",
    useDcolumn=getOption("useDcolumn",TRUE),
    numeric.colspec=if(useDcolumn)
                       paste("D{.}{",LaTeXdec,"}{",ddigits,"}",sep="")
                    else "r",
    factor.colspec="l",
    LaTeXdec=".",
    ddigits=digits,
    useBooktabs=getOption("useBooktabs",TRUE),
    toprule=if(useBooktabs) "\\\\toprule" else "\\\\hline\\\\hline",
    midrule=if(useBooktabs) "\\\\midrule" else "\\\\hline",
    cmidrule=if(useBooktabs) "\\\\cmidrule" else "\\\\cline",
    bottomrule=if(useBooktabs) "\\\\bottomrule" else "\\\\hline\\\\hline",
    row.names=is.character(attr(object,"row.names")),
    NAas="",
    toLatex.escape.tex=getOption("toLatex.escape.tex",FALSE),
    \dots)

\method{toLatex}{ftable}(object,
    show.titles=TRUE,
    digits=if(is.integer(object)) 0 else getOption("digits"),
    format=if(is.integer(object)) "d" else "f",
    useDcolumn=getOption("useDcolumn",TRUE),
    colspec=if(useDcolumn)
                paste("D{.}{",LaTeXdec,"}{",ddigits,"}",sep="")
             else "r",
    LaTeXdec=".",
    ddigits=digits,
    useBooktabs=getOption("useBooktabs",TRUE),
    toprule=if(useBooktabs) "\\\\toprule" else "\\\\hline\\\\hline",
    midrule=if(useBooktabs) "\\\\midrule" else "\\\\hline\n",
    cmidrule=if(useBooktabs) "\\\\cmidrule" else "\\\\cline",
    bottomrule=if(useBooktabs) "\\\\bottomrule" else "\\\\hline\\\\hline",
    extrarowsep = NULL,
    toLatex.escape.tex=getOption("toLatex.escape.tex",FALSE),
    fold.leaders=FALSE,
    \dots)

\method{toLatex}{ftable_matrix}(object,
    show.titles=TRUE,
    digits=getOption("digits"),
    format="f",
    useDcolumn=getOption("useDcolumn",TRUE),
    colspec=if(useDcolumn)
                paste("D{.}{",LaTeXdec,"}{",ddigits,"}",sep="")
             else "r",
    LaTeXdec=".",
    ddigits=digits,
    useBooktabs=getOption("useBooktabs",TRUE),
    toprule=if(useBooktabs) "\\\\toprule" else "\\\\hline\\\\hline",
    midrule=if(useBooktabs) "\\\\midrule" else "\\\\hline",
    cmidrule=if(useBooktabs) "\\\\cmidrule" else "\\\\cline",
    bottomrule=if(useBooktabs) "\\\\bottomrule" else "\\\\hline\\\\hline",
    compact=FALSE,
    varontop,varinfront,
    groupsep="3pt",
    grouprule=midrule,
    toLatex.escape.tex=getOption("toLatex.escape.tex",FALSE),
    multi_digits=NULL,
    \dots)
}
\arguments{
     \item{object}{an \code{\link{ftable}}, a matrix or an object coercable into 
     a matrix.  }
     \item{show.titles}{logical, should variable names (in case of the 
     \code{ftable} and \code{table} methods)
      or row and column names (in case of the \code{matrix} method) be appear
      in the \code{LaTeX} code? }
     \item{show.vars,show.xvar,show.yvar}{logical, should the names of the dimnames of \code{object}
      be shown in the margins of the LaTeX tabular? Such names usually represent the
      row and/or column variables of a two-dimensional \code{\link{table}}.
     }
     \item{digits}{number of significant digits.
        }
     \item{format}{character containing a format specifier, see \code{\link{format}}. }
     \item{useDcolumn}{logical, should the facilities of the \code{dcolumn} LaTeX package be used?
          Note that, if TRUE, you will need to include \code{\\usepackage{dcolumn}}
          in the preamble of your LaTeX document.}
     \item{colspec}{character, LaTeX table column format specifyer(s).}
     \item{numeric.colspec}{character, LaTeX table column format
          specifyer(s) for numeric vectors in the data frame.}
     \item{factor.colspec}{character, LaTeX table column format
          specifyer(s) for factors in the data frame.}
     \item{LaTeXdec}{character, the decimal point in the final LaTeX output.}
     \item{ddigits}{integer, digits after the decimal point.}
     \item{useBooktabs}{logical, should the facilities of the \code{booktabs} LaTeX package be used?
          Note that, if TRUE, you will need to include \code{\\usepackage{booktabs}}
          in the preamble of your LaTeX document.}
     \item{toprule}{character string, TeX code that determines the appearance of the top border of the LaTeX \code{tabular} environment.}
     \item{midrule}{character string, TeX code that determines how  coefficients and summary statistics are
        separated in the LaTeX \code{tabular} environment.}
     \item{cmidrule}{character string, TeX code that determines the appearance of rules under section headings.}
     \item{bottomrule}{character string, TeX code that determines the appearance of the bottom border of the LaTeX \code{tabular} environment.}
     \item{extrarowsep}{character string, extra code to be inserted between the column titles and the
     table body produced by \code{toLatex}. }
     \item{compact}{logical, if \code{TRUE}, extra column space between sub-tables 
        is suppressed. Defaults to \code{FALSE}}
     \item{varontop}{logical, whether names of column variables should appear on top of factor levels}
     \item{varinfront}{logical, whether names of row variables should appear in front of factor levels}
     \item{groupsep}{character string, containing a TeX length; extra 
      vertical space inserted between sub-tables, unless \code{compact} is \code{TRUE}.}
     \item{grouprule}{character string, TeX code that determines how
       sub-table headings are embellished.}
     \item{row.names}{logical, whether row names should be included in
			 exported LaTeX code.}
     \item{NAas}{character string, how missing values should be represented.}
     \item{toLatex.escape.tex}{logical, should symbols "$", "_", and "^" be
	   escaped with backslashes?}
     \item{fold.leaders}{logical, if \code{TRUE}, factor levels of row
       variables are not distributed into different columns, but
       'folded' into a single column.}
     \item{multi_digits}{NULL, a numeric vector, or a list. If it is a
          list it should have as many elements as
           the "ftable_matrix" contains columns, where each vector has
           as many columns as the respective "ftable". If it is a
           vector, it is put into a list with replicated elements
           according to the "ftable" components.
           The elements of these vectors can be used to specify a separate
           number of digits for each column of the respective "ftable".}
     \item{\dots}{further argument, currently ignored.}
}
\examples{
toLatex(diag(5))

toLatex(ftable(UCBAdmissions))

toLatex(rbind(
  ftable(margin.table(UCBAdmissions,c(2,1))),
  ftable(margin.table(UCBAdmissions,c(3,1)))
))
}
\keyword{misc}

\name{Iconv}
\alias{Iconv}
\alias{Iconv.character}
\alias{Iconv.annotation}
\alias{Iconv.data.set}
\alias{Iconv.importer}
\alias{Iconv.item}
\alias{Iconv.value.labels}

\title{Convert Annotations, and Value Labels between Encodings}
\description{
  This function uses the base package function \code{\link[base]{iconv}}
  to translate variable descriptions (a.k.a variable labels) and
  value labels of \code{\link{item}}, \code{\link{data.set}},
  and \code{\link{importer}} objects into a specified encoding.
  
  It will be useful in UTF-8 systems when data file come in some ancient
  encoding like 'Latin-1' as long used by Windows systems.
  }
\usage{

Iconv(x,from="",to="",\dots) 
\method{Iconv}{character}(x,from="",to="",\dots) 
\method{Iconv}{annotation}(x,from="",to="",\dots) 
\method{Iconv}{data.set}(x,from="",to="",\dots) 
\method{Iconv}{importer}(x,from="",to="",\dots) 
\method{Iconv}{item}(x,from="",to="",\dots) 
\method{Iconv}{value.labels}(x,from="",to="",\dots) 

}
\arguments{
  \item{x}{a character vector or an object of which character data or attributes 
    are to be re-encoded.}
  \item{from}{a character string describing the original encoding}
  \item{to}{a character string describing the target encoding}
  \item{\dots}{further arguments, passed to \code{\link[base]{iconv}}}
}
\value{
  \code{Iconv} returns a copy of its first argument with re-encoded 
  character data or attributes.
}
\seealso{
  \code{\link[base]{iconv}}, \code{\link[base:iconv]{iconvlist}}
}
\examples{
\dontrun{
# Locate an SPSS 'system' file and get info on variables, their labels etc.
ZA5302 <- spss.system.file("Daten/ZA5302_v6-0-0.sav",to.lower=FALSE)

# Convert labels etc. from 'latin1' to the encoding of the current locale.
ZA5302 <- Iconv(ZA5302,from="latin1")

# Write out the codebook
writeLines(as.character(codebook(ZA5302)),
           con="ZA5302-cdbk.txt")

# Write out the description of the varialbes (their 'variable labels')
writeLines(as.character(description(ZA5302)),
            con="ZA5302-description.txt")
}
}
\keyword{misc}

\name{as.symbols}
\alias{as.symbols}
\alias{syms}
\title{Construction of Lists of Symbols}
\description{
  \code{as.symbols} and \code{syms} are functions potentially useful
  in connection with \code{foreach} and \code{xapply}.
  \code{as.symbols} produces a list of symbols from a character vector,
  while \code{syms} returns a list of symbols from symbols given as arguments,
  but it can be used to construct patterns of symbols.
}
\usage{
as.symbols(x)
syms(\dots,paste=FALSE,sep="")
}
\arguments{
  \item{x}{a character vector}
  \item{\dots}{character strings or (unquoted) variable names}
  \item{paste}{logical value; should the character strings
    \code{\link{paste}}d into one string?}
  \item{sep}{a separator string, passed to \code{\link{paste}}. }
}
\value{
A list of language symbols (results of \code{\link{as.symbol}} - not graphical
symbols!).
}
\examples{
  as.symbols(letters[1:8])
  syms("a",1:3,paste=TRUE)

  sapply(syms("a",1:3,paste=TRUE),typeof)
}
\keyword{programming}

\name{rename}
\alias{rename}
\title{Change Names of a Named Object}
\description{
  \code{rename} changes the names of a named object.
}
\usage{
rename(x, \dots, gsub = FALSE, fixed = TRUE, warn = TRUE)
}
\arguments{
  \item{x}{Any named object}
  \item{\dots}{A sequence of named arguments, all of type character}
  \item{gsub}{a logical value; if TRUE, \code{\link{gsub}} is used to change the
      row and column labels of the resulting table.
      That is, instead of substituting whole names, substrings of the
      names of the object can changed.
      }
  \item{fixed}{a logical value, passed to \code{\link{gsub}}. If TRUE,
    substitutions are by fixed strings and not by regular expressions.}
  \item{warn}{a logical value; should a warning be issued if
    those names to change are not found?}
}
\details{
  This function changes the names of \code{x} according to the
  remaining arguments.
  If \code{gsub} is FALSE, argument tags are the \emph{old}
  names, the values are the new names.
  If \code{gsub} is TRUE, arguments are substrings of the names
  that are substituted by the argument values.
}

\value{
  The object \code{x} with new names defined by the \dots arguments.
}

\examples{
  x <- c(a=1, b=2)
  rename(x,a="A",b="B")
  # Since version 0.99.22 - the following also works:
  rename(x,a=A,b=B)
  
  str(rename(iris,
                  Sepal.Length="Sepal_Length",
                  Sepal.Width ="Sepal_Width",
                  Petal.Length="Petal_Length",
                  Petal.Width ="Petal_Width"
                  ))
  str(rename(iris,
                  .="_"
                  ,gsub=TRUE))

  # Since version 0.99.22 - the following also works:
  str(rename(iris,
                  Sepal.Length=Sepal_Length,
                  Sepal.Width =Sepal_Width,
                  Petal.Length=Petal_Length,
                  Petal.Width =Petal_Width
                  ))
}
\keyword{manip}

\name{format_md}
\alias{format_md}
\alias{format_md.codebook}
\alias{format_md.codebookEntry}
\title{
  Format Codebooks as Markdown
}
\description{
 \code{format_md} is for showing objects in a convenient way in Markdown
format. Can be included to Rmarkdown file with the \code{cat()} function and the
\code{results='asis'} code block option. The following example should be runned
in a Rmd file with different output formats.
}
\usage{
\method{format_md}{codebook}(x, \dots)
\method{format_md}{codebookEntry}(x, name = "", add_rules = TRUE, \dots)
}
\arguments{
  \item{x}{a "codebook" or "codebookEntry" object}
  \item{name}{a string; the variable name}
  \item{add_rules}{a boolean value; if TRUE adds a horizontal rules before and after the title}
  \item{\dots}{further arguments, passed to other functions}
  }
\value{
   \code{format_md} character string with code suitable for inclusion into a Markdown-file.
}
\examples{
library(memisc)

Data1 <- data.set(
  vote = sample(c(1,2,3,8,9,97,99),size=300,replace=TRUE),
  region = sample(c(rep(1,3),rep(2,2),3,99),size=300,replace=TRUE),
  income = exp(rnorm(300,sd=.7))*2000
)

Data1 <- within(Data1,{
  description(vote) <- "Vote intention"
  description(region) <- "Region of residence"
  description(income) <- "Household income"
  foreach(x=c(vote,region),{
    measurement(x) <- "nominal"
  })
  measurement(income) <- "ratio"
  labels(vote) <- c(
    Conservatives         =  1,
    Labour                =  2,
    "Liberal Democrats"   =  3,
    "Don't know"          =  8,
    "Answer refused"      =  9,
    "Not applicable"      = 97,
    "Not asked in survey" = 99)
  labels(region) <- c(
    England               =  1,
    Scotland              =  2,
    Wales                 =  3,
    "Not applicable"      = 97,
    "Not asked in survey" = 99)
  foreach(x=c(vote,region,income),{
    annotation(x)["Remark"] <- "This is not a real survey item, of course ..."
  })
  missing.values(vote) <- c(8,9,97,99)
  missing.values(region) <- c(97,99)
})

codebook_data <- codebook(Data1)

codebook_md <- format_md(codebook_data, digits = 2)

writeLines(codebook_md)

\dontrun{
writeLines(codebook_md,con="codebook-example.md")
}
 
}

\name{measurement}
\alias{measurement}
\alias{measurement,ANY-method}
\alias{measurement,item-method}
\alias{measurement,data.set-method}
\alias{measurement<-}
\alias{measurement<-,item-method}
\alias{measurement<-,data.set-method}
\alias{set_measurement}
\alias{is.nominal}
\alias{is.ordinal}
\alias{is.interval}
\alias{is.ratio}
\alias{as.nominal}
\alias{as.ordinal}
\alias{as.interval}
\alias{as.ratio}
\title{Levels of Measurement of Survey Items}
\description{
  The measurement level of a \code{"item"} object, which is one of "nominal", "ordinal", "interval", "ratio",
  determines what happens to it, if it or the \code{\link{data.set}}
  containing it is coerced into a \code{\link{data.frame}}.
  If the level of measurement level is "nominal", the it will be
  converted into an (unordered) \link{factor}, if the level of measurement is "ordinal",
  the item will be converted into an \link{ordered} vector. If the measurement
  is "interval" or "ratio", the item will be converted into a numerical vector.
}
\usage{
\S4method{measurement}{item}(x)
\S4method{measurement}{item}(x) <- value
\S4method{measurement}{data.set}(x)
\S4method{measurement}{data.set}(x) <- value
is.nominal(x)
is.ordinal(x)
is.interval(x)
is.ratio(x)
as.nominal(x)
as.ordinal(x)
as.interval(x)
as.ratio(x)
set_measurement(x,\dots)
}
\arguments{
  \item{x}{an object, usually of class \code{"item"}.}
  \item{value}{
    for the \code{item} method, 
    a character string; either "nominal", "ordinal", "interval", or
    "ratio";
    for the \code{data.set} method,
    a list of character vectors with variable names,
    where the names of the list corresponds to a measurement level and
    and the list elements indicates the variables to which the
    measurement levels are assigned.
  }
  \item{\dots}{vectors of variable names, either symbols or character
    strings, tagged with the intended measurement level.
  }
}
\value{
  The \code{item} method of \code{measurement(x)} returns a character
  string, the \code{data.set} method returns a named character vector,
  where the name of each element is a variable name and each.

  \code{as.nominal}, \code{as.ordinal}, \code{as.interval}, \code{as.ratio}
  return an item with the requested level of measurement setting.
  
  \code{is.nominal}, \code{is.ordinal}, \code{is.interval}, \code{is.ratio}
  return a logical value.
}
\references{
Stevens, Stanley S. 1946. "On the theory of scales of measurement." \emph{Science} 103: 677-680.
}
\seealso{\code{\link{data.set}}, \code{\link{item}}}
\examples{
vote <- sample(c(1,2,3,8,9),size=30,replace=TRUE)
labels(vote) <- c(Conservatives         =  1,
                  Labour                =  2,
                  "Liberal Democrats"   =  3,
                  "Don't know"          =  8,
                  "Answer refused"      =  9
                  )
missing.values(vote) <- c(8,9)
as.data.frame(vote)[[1]]
measurement(vote) <- "interval"
as.data.frame(vote)[[1]]
vote <- as.nominal(vote)
as.data.frame(vote)[[1]]
group <- sample(c(1,2),size=30,replace=TRUE)
labels(group) <- c(A=1,B=2)
DataS <- data.set(group,vote)
measurement(DataS)
measurement(DataS) <- list(interval=c("group","vote"))
head(as.data.frame(DataS))
DataS <- set_measurement(DataS,
                         nominal=c(group,vote))
head(as.data.frame(DataS))
}
\keyword{manip}

\name{retain}
\alias{retain}
\title{Retain Objects in an Environment}
\description{\code{retain} removes all objects from the environment
  except those mentioned as argument.}
\usage{
retain(\dots, list = character(0), envir = parent.frame(),force=FALSE)
}
\arguments{
  \item{\dots}{names of objects to be retained, as names (unquoted)
    or character strings(quoted).}
  \item{list}{a character vector naming the objects to be retained.}
  \item{envir}{the environment from which the objects are removed
    that are not to be retained.}
  \item{force}{logical value. As a measure of caution, this
    function removes objects only from local environments,
    unless \code{force} equals TRUE. In that case, \code{retain} can
    also be used to clear the global environment, the user's workspace.}
}
\examples{
local({
  foreach(x=c(a,b,c,d,e,f,g,h),x<-1)
  cat("Objects before call to 'retain':\n")
  print(ls())
  retain(a)
  cat("Objects after call to 'retain':\n")
  print(ls())
})
x <- 1
y <- 2
retain(x)
}
\keyword{manip}

\name{cases}
\alias{cases}
\title{Distinguish between Cases Specified by Logical Conditions}
\description{
  \code{cases} allows to distinguish several cases defined logical
  conditions. It can be used to code these cases into a vector. The
  function can be considered as a multi-condition generalization of
  \code{\link{ifelse}}.
  }
\usage{
cases(\dots,check.xor=c("warn","stop","ignore"),
      .default=NA,.complete=FALSE,
      check.na=c("warn","stop","ignore"),
      na.rm=TRUE)
}
\arguments{
  \item{\dots}{A sequence of logical expressions or assignment expressions containing
    logical expressions as "right hand side".}
  \item{check.xor}{character (either \code{"warn"}, \code{"stop"}, or  \code{"ignore"})
    or logical; if \code{TRUE} or equal to \code{"stop"} or 
    \code{"warn"}, 
    \code{cases} checks whether the case
    conditions are mutually exclusive. If this is not
    satisfied and \code{check.xor} equals \code{"warn"} (the default), a warning is shown,
    otherwise an error exception is raised.}
  \item{.default}{a value to be used for unsatisfied conditions.}
  \item{.complete}{logical, if \code{TRUE} an additional factor level is
    created for the unsatisfied conditions.}
  \item{check.na}{character (either \code{"warn"}, \code{"stop"}, or  \code{"ignore"})
    or logical; if \code{TRUE} or equal to \code{"stop"} or 
    \code{"warn"}, \code{cases} checks, whether any of the case
    conditions evaluates to \code{NA}.
    If that case, if \code{check.na} is \code{TRUE} or equals
    \code{"stop"} an error exception is raised, while if \code{check.na}
    equals \code{"warn"} (the default) a warning is shown.}
  \item{na.rm}{a logical value; how to handle \code{NA}s (if they do not
    already lead to an error exception). If \code{FALSE} if \emph{any} of the
    conditions evaluates to \code{NA}, the corresponding value of the
    result vector is \code{NA}. If \code{TRUE} (the default), the
    resulting vector or factor is \code{NA} only for instances where all
    conditions result in \code{NA}. 
  }
}
\details{
  There are two distinct ways to use this function. Either the
  function can be used to construct a factor that represents
  several logical cases or it can be used to conditionally
  evaluate an expression in a manner similar to \code{\link{ifelse}}.

  For the first use, the \code{\dots} arguments have to be a series of
  logical expressions. \code{cases} then returns a factor
  with as many levels as logical expressions given as
  \code{\dots} arguments. The resulting factor will attain its
  first level if the first condition is TRUE, otherwise it will attain its
  second level if the second condition is TRUE, etc.
  The levels will be named after the conditions or, if name tags are
  attached to the logical expressions, after the tags of the expressions.
  Not that the logical expressions all need to evaluate to logical vectors
  of the same length, otherwise an error condition is raised.
  If \code{.complete} is \code{TRUE} then an additional factor level is
  created for the conditions not satisfied for any of the cases.
  
  For the second use, the \code{\dots} arguments have to be a series
  of assignment expression of the type \code{<expression> <- <logical expression>}
  or \code{<logical expression> -> <expression>}. For cases
  in which the first logical expression is TRUE, the result of first expression that
  appears on the other side of the assignment operator become elements of the
  vector returned by \code{cases}, for cases in which the second logical expression is TRUE,
  the result of the second expression that appears on the other side
  of the assignment operator become elements of the
  vector returned by \code{cases}, etc.
  For cases that do not satisfy any of the given conditions the value of
  the \code{.default} argument is used. 
  Note that the logical expressions also here all need to evaluate to logical
  vectors of the same length. The expressions on the other side of the
  assignment operator should also be either vectors of the same length
  and mode or should scalars of the same mode, otherwise unpredictable
  results may occur.

}
\value{
  If it is called with logical expressions as \dots arguments,
  \code{cases} returns a factor, if it is called with
  assignment expressions the function returns a vector with the
  same mode as the results of the "assigned" expressions
  and with the same length as the logical conditions.
}

\examples{
# Examples of the first kind of usage of the function
#
df <- data.frame(x = rnorm(n=20), y = rnorm(n=20))
df <- df[do.call(order,df),]
(df <- within(df,{
  x1=cases(x>0,x<=0)
  y1=cases(y>0,y<=0)
  z1=cases(
    "Condition 1"=x<0,
    "Condition 2"=y<0,# only applies if x >= 0
    "Condition 3"=TRUE
    )
  z2=cases(x<0,(x>=0 & y <0), (x>=0 & y >=0))
  }))
xtabs(~x1+y1,data=df)
dd <- with(df,
  try(cases(x<0,
            x>=0,
            x>1,
            check.xor=TRUE)# let's be fussy
            )
  )
dd <- with(df,
  try(cases(x<0,x>=0,x>1))
  )
genTable(range(x)~dd,data=df)

# An example of the second kind of usage of the function:
# A construction of a non-smooth function
#
fun <- function(x)
  cases(
    x==0      -> 1,
    abs(x)> 1 -> abs(x),
    abs(x)<=1 -> x^2
  )
x <- seq(from=-2,to=2,length=101)
plot(fun(x)~x)

# Demo of the new .default and .complete arguments
x <- seq(from=-2,to=2)
cases(a = x < -1,
      b = x > 1,
      .complete = TRUE)
cases(x < -1,
      x > 1,
      .complete = TRUE)
cases(1 <- x < -1,
      3 <- x > 1,
      .default = 2)

threshhold <- 5
d <- c(1:10, NaN)

d1 <- cases(
  d > threshhold -> 1,
  d <= threshhold -> 2
)

d2 <- cases(
  is.na(d) -> 0,
  d > threshhold -> 1,
  d <= threshhold -> 2
)

# Leads to missing values because some of the conditions result in missing
# even though they could be 'captured'
d3 <- cases(
  is.na(d) -> 0,
  d > threshhold -> 1,
  d <= threshhold -> 2,
  na.rm=FALSE
)

d4 <- cases(
  is.na(d) -> 0,
  d > threshhold +2 -> 1,
  d <= threshhold -> 2,
  na.rm=FALSE
)

cbind(d,d1,d2,d3,d4)

cases(
  d > threshhold,
  d <= threshhold
)

cases(
  is.na(d),
  d > threshhold,
  d <= threshhold
)

cases(
  d > threshhold,
  d <= threshhold,
  .complete=TRUE
)

cases(
  d > threshhold + 2,
  d <= threshhold,
  .complete=TRUE
)
}

\keyword{manip}

\name{codebook}
\alias{codebook}
\alias{codebook,ANY-method}
\alias{codebook,item-method}
\alias{codebook,data.set-method}
\alias{codebook,importer-method}
\alias{codebook,NULL-method}
\alias{codebook,atomic-method}
\alias{codebook,data.frame-method}
\alias{codebook,tbl_df-method}
\alias{codebook,factor-method}
\alias{codebook-class}
\alias{format,codebookEntry-method}
\alias{show,codebook-method}
\alias{as.character,codebook-method}
\alias{$,codebook-method}
\alias{[,codebook,atomic,missing,ANY-method}
\alias{[[,codebook-method}
\title{Generate a Codebook of a Data Set}
\description{
  Function \code{codebook} collects documentation about an item,
  or the items in a data set or external data file. It returns
  an object that, when \code{show}n, print this documentation
  in a nicely formatted way. 
}
\usage{
codebook(x, weights = NULL, unweighted = TRUE, \dots)
\S4method{codebook}{item}(x, weights = NULL, unweighted = TRUE, \dots)
\S4method{codebook}{atomic}(x, weights = NULL, unweighted = TRUE, \dots)
\S4method{codebook}{factor}(x, weights = NULL, unweighted = TRUE, \dots)
\S4method{codebook}{data.set}(x, weights = NULL, unweighted = TRUE, \dots)
\S4method{codebook}{importer}(x, weights = NULL, unweighted = TRUE, \dots)
\S4method{codebook}{data.frame}(x, weights = NULL, unweighted = TRUE, \dots)
\S4method{codebook}{tbl_df}(x, weights = NULL, unweighted = TRUE, \dots)
}
\arguments{
  \item{x}{an \code{\link{item}}, numeric or character vector, factor,
    \code{\link{data.set}}, \code{\link{data.frame}} or \code{\link{importer}} object for \code{codebook()}}
  \item{weights}{an optional vector of weights.}
  \item{unweighted}{an optional logical vector; if weights are given, it
    determines of only summaries of weighted data are show or also summaries of
    unweighted data.}
  \item{\dots}{other arguments, currently ignored.}
}
\value{
  An object of class "codebook", for which a \code{\link{show}} method exists that
  produces a nicely formatted output.
}
\examples{

Data <- data.set(
          vote = sample(c(1,2,3,8,9,97,99),size=300,replace=TRUE),
          region = sample(c(rep(1,3),rep(2,2),3,99),size=300,replace=TRUE),
          income = exp(rnorm(300,sd=.7))*2000
          )

Data <- within(Data,{
  description(vote) <- "Vote intention"
  description(region) <- "Region of residence"
  description(income) <- "Household income"
  wording(vote) <- "If a general election would take place next tuesday,
                    the candidate of which party would you vote for?"
  wording(income) <- "All things taken into account, how much do all
                    household members earn in sum?"
  foreach(x=c(vote,region),{
    measurement(x) <- "nominal"
    })
  measurement(income) <- "ratio"
  labels(vote) <- c(
                    Conservatives         =  1,
                    Labour                =  2,
                    "Liberal Democrats"   =  3,
                    "Don't know"          =  8,
                    "Answer refused"      =  9,
                    "Not applicable"      = 97,
                    "Not asked in survey" = 99)
  labels(region) <- c(
                    England               =  1,
                    Scotland              =  2,
                    Wales                 =  3,
                    "Not applicable"      = 97,
                    "Not asked in survey" = 99)
  foreach(x=c(vote,region,income),{
    annotation(x)["Remark"] <- "This is not a real survey item, of course ..."
    })
  missing.values(vote) <- c(8,9,97,99)
  missing.values(region) <- c(97,99)
})

description(Data)

codebook(Data)

codebook(Data)$vote
codebook(Data)[2]

codebook(Data[2])
 
DataFr <- as.data.frame(Data)
DataHv <- as_haven(Data,user_na=TRUE)

codebook(DataFr)
codebook(DataHv)

   
\dontrun{
Write(description(Data),
           file="Data-desc.txt")
Write(codebook(Data),
           file="Data-cdbk.txt")
  }
}
\keyword{manip}

\name{reorder.array}
\alias{reorder}
\alias{reorder.array}
\alias{reorder.matrix}
\title{Reorder an Array or Matrix}
\description{
  \code{reorder.array} reorders an array along a specified
  dimension according given names, indices or results of
  a function applied.
}
\usage{
\method{reorder}{array}(x,dim=1,names=NULL,indices=NULL,FUN=mean,...)
\method{reorder}{matrix}(x,dim=1,names=NULL,indices=NULL,FUN=mean,...)
}
\arguments{
  \item{x}{An array}
  \item{dim}{An integer specifying the dimension along which \code{x} should be ordered.}
  \item{names}{A character vector}
  \item{indices}{A numeric vector}
  \item{FUN}{A function that can be used in \code{apply(x,dim,FUN)} }
  \item{...}{further arguments, ignored.}
}
\details{
  Typical usages are
  \preformatted{
  reorder(x,dim,names)
  reorder(x,dim,indices)
  reorder(x,dim,FUN)
  }

  The result of \code{rename(x,dim,names)} is \code{x}
  reordered such that \code{dimnames(x)[[dim]]} is equal to
  the concatenation of those elements of \code{names}
  that are in \code{dimnames(x)[[dim]]} and the remaining elements
  of \code{dimnames(x)[[dim]]}.

  The result of \code{rename(x,dim,indices)} is \code{x}
  reordered along \code{dim} according to \code{indices}.

  The result of \code{rename(x,dim,FUN)} is \code{x}
  reordered along \code{dim} according to \code{order(apply(x,dim,FUN))}.
  
}

\value{
  The reordered object \code{x}.
}
\seealso{
  The default method of \code{\link[stats:reorder.factor]{reorder}} in package \code{stats}.
}
\examples{
  (M <- matrix(rnorm(n=25),5,5,dimnames=list(LETTERS[1:5],letters[1:5])))
  reorder(M,dim=1,names=c("E","A"))
  reorder(M,dim=2,indices=3:1)
  reorder(M,dim=1)
  reorder(M,dim=2)
}
\keyword{manip}

\name{sample-methods}
\alias{sample,data.frame-method}
\alias{sample,data.set-method}
\alias{sample,importer-method}
\title{Take a Sample from a Data Frame-like Object}
\description{
  The methods below are convenience short-cuts to
  take samples from data frames and data sets.
  They result in a data frame or data set, respectively,
  the rows of which are a sample of the complete
  data frame/data set.
}
\usage{
\S4method{sample}{data.frame}(x, size, replace = FALSE, prob = NULL)
\S4method{sample}{data.set}(x, size, replace = FALSE, prob = NULL)
\S4method{sample}{importer}(x, size, replace = FALSE, prob = NULL)
}
\arguments{
  \item{x}{a data frame or data set.}
  \item{size}{an (optional) numerical value, the sample size,
    defaults to the total number of rows of \code{x}.}
  \item{replace}{a logical value, determines whether
    sampling takes place with or without replacement.}
  \item{prob}{a vector of sampling probabities or NULL.}
}
\value{A data frame or data set.}

\examples{
for(.i in 1:4)
  print(sample(iris,5))
}

\name{to.data.frame}
\alias{to.data.frame}
\title{Convert an Array into a Data Frame}
\description{
  \code{to.data.frame} converts an array into a data frame, in such a way
  that a chosen dimensional extent forms variables in the data frame.
  The elements of the array must be either atomic, data frames
  with matching variables, or coercable into such data frames.
}
\usage{
to.data.frame(X,as.vars=1,name="Freq")
}
\arguments{
  \item{X}{an array.}
  \item{as.vars}{a numeric value or a character string.
    If it is a numeric value then it indicates the dimensional extend
    which defines the variables. If it is a character string then it is
    matched against the names of the dimenstional extents. This is
    applicable e.g. if \code{X} is a contingency table and the
    dimensional extents are named after the cross-classified factors.
    Takes effect only if \code{X} is 
    an atomic array. If \code{as.vars} equals zero, a new variable
    is created that contains the values of the array, that is,
    \code{to.data.frame} acts on the array \code{X}
    like \code{as.data.frame(as.table(X))}
    }
  \item{name}{a character string; the name of the variable
    created if \code{X} is an atomic array and \code{as.vars} equals zero.
    }
}

\value{
  A data frame.
}

\examples{
berkeley <- Aggregate(Table(Admit,Freq)~.,data=UCBAdmissions)
berktest1 <- By(~Dept+Gender,
                glm(cbind(Admitted,Rejected)~1,family="binomial"),
                data=berkeley)
berktest2 <- By(~Dept,
                glm(cbind(Admitted,Rejected)~Gender,family="binomial"),
                data=berkeley)
Stest1 <- Lapply(berktest2,function(x)predict(x,,se.fit=TRUE)[c("fit","se.fit")])
Stest2 <- Sapply(berktest2,function(x)coef(summary(x)))
Stest2.1 <- Lapply(berktest1,function(x)predict(x,,se.fit=TRUE)[c("fit","se.fit")])
to.data.frame(Stest1)
to.data.frame(Stest2,as.vars=2)
to.data.frame(Stest2.1)
# Recasting a contingency table
to.data.frame(UCBAdmissions,as.vars="Admit")
}
\keyword{manip}

\name{mtable_format_html}
\alias{mtable_format_html}
\alias{format_html.memisc_mtable}
\alias{mtable_format_html}
\alias{mtable_format_stdstyle}
\title{HTML Formatting for 'mtable' Results}
\description{
  These functions formats 'mtable' objects into HTML format.
  }
\usage{
mtable_format_html(x,
                    interaction.sep = NULL,
                    toprule=2,midrule=1,bottomrule=2,
                    split.dec=TRUE,
                    style=mtable_format_stdstyle,
                    margin="2ex auto", 
                    sig.notes.style=c(width="inherit"),
                    \dots
  )
\method{format_html}{memisc_mtable}(x,
                    interaction.sep = NULL,
                    toprule=2,midrule=1,bottomrule=2,
                    split.dec=TRUE,
                    style=mtable_format_stdstyle,
                    margin="2ex auto", 
                    sig.notes.style=c(width="inherit"),
                    \dots
  )
}
\arguments{
     \item{x}{an object of class \code{mtable}}
     \item{toprule}{integer;
     thickness in pixels of rule at the top of the table.}
     \item{midrule}{integer;
     thickness in pixels of rules within the table.}
     \item{bottomrule}{integer;
     thickness in pixels of rule at the bottom of the table.}
     \item{interaction.sep}{a character string that separates factors that are involved
        in an interaction effect or NULL. If NULL then a reasonable default is used
        (either a unicode character or an ampersand encoded HTML entity).}
    \item{split.dec}{logical; whether numbers should be centered
    at the decimal point by splitting the table cells.}
    \item{style}{string containing default the CSS styling.}
    \item{margin}{character string, determines the margin and thus
      the position of the HTML table.}
    \item{sig.notes.style}{a character vector with named elements,
           allows extra styling of the p-values notes at the bottom of
           the table.}
    \item{\dots}{further arguments, ignored. }
}
\value{
    A character string with code suitable for inclusion into a HTML-file.
}
\examples{
lm0 <- lm(sr ~ pop15 + pop75,              data = LifeCycleSavings)
lm1 <- lm(sr ~                 dpi + ddpi, data = LifeCycleSavings)
lm2 <- lm(sr ~ pop15 + pop75 + dpi + ddpi, data = LifeCycleSavings)

mtable123 <- mtable("Model 1"=lm0,"Model 2"=lm1,"Model 3"=lm2,
                    summary.stats=c("sigma","R-squared","F","p","N"))

(mtable123 <- relabel(mtable123,
                      "(Intercept)" = "Constant",
                      pop15 = "Percentage of population under 15",
                      pop75 = "Percentage of population over 75",
                      dpi = "Real per-capita disposable income",
                      ddpi = "Growth rate of real per-capita disp. income"
))

# Use HTML entity '&minus;' for minus sign
options(html.use.ampersand=TRUE)
show_html(mtable123)
show_html(mtable123[1:2],
          sig.notes.style=c(width="30ex"))

# Use unicode for minus sign (default)
options(html.use.ampersand=FALSE)
show_html(mtable123)
}

\name{wild.codes}
\alias{wild.codes}
\alias{wild.codes,item-method}
\title{
 Table of frequencies for unlabelled codes
}
\description{
  The function \code{wild.codes} creates a table of frequencies of those codes
  of an item that do not have labelled attached to them. This way, it helps to identify
  coding errors.
}
\usage{
wild.codes(x)
\S4method{wild.codes}{item}(x)
}
\arguments{
  \item{x}{an object of class "item"}
}
\value{
  A table of frequencies (i.e. an array of class "table")
}

\name{codeplan}
\alias{codeplan}
\alias{codeplan,ANY-method}
\alias{codeplan,item.list-method}
\alias{codeplan,item-method}
\alias{codeplan<-}
\alias{setCodeplan}
\alias{setCodeplan,data.frame,codeplan-method}
\alias{setCodeplan,data.frame,NULL-method}
\alias{setCodeplan,data.set,codeplan-method}
\alias{setCodeplan,data.set,NULL-method}
\alias{setCodeplan,item,codeplan-method}
\alias{setCodeplan,item,NULL-method}
\alias{setCodeplan,atomic,codeplan-method}
\alias{setCodeplan,atomic,NULL-method}
\alias{read_codeplan}
\alias{write_codeplan}
\title{Describe structure of Data Sets and Importers}
\description{
  The function \code{codeplan()} creates a data frame that
  describes the structure of an item list (a \code{data.set} object or
  an \code{importer} object), so that this structure can be stored and
  and recovered. The resulting data frame has a particular print method
  that delimits the output to one line per variable.

  With \code{setCodeplan} an item list structure (as returned by \code{codeplan()})
  can be applied to a data frame or data set. It is also possible to use an
  assignment like \code{codeplan(x) <- value} to a similar effect.
}
\usage{
codeplan(x)
\S4method{codeplan}{item.list}(x)
\S4method{codeplan}{item}(x)
setCodeplan(x,value)
\S4method{setCodeplan}{data.frame,codeplan}(x,value)
\S4method{setCodeplan}{data.frame,NULL}(x,value)
\S4method{setCodeplan}{data.set,codeplan}(x,value)
\S4method{setCodeplan}{data.set,NULL}(x,value)
\S4method{setCodeplan}{item,codeplan}(x,value)
\S4method{setCodeplan}{item,NULL}(x,value)
\S4method{setCodeplan}{atomic,codeplan}(x,value)
\S4method{setCodeplan}{atomic,NULL}(x,value)
codeplan(x) <- value
read_codeplan(filename,type)
write_codeplan(x,filename,type,pretty)
}
\arguments{
  \item{x}{for \code{codeplan(x)} an object that inherits from class \code{"item.list"},
    i.e. can be a \code{"data.set"} object or an \code{"importer"}
  object, it can also be an object that inherits from class
  \code{"item"}.
   For \code{write_codeplan} an object from class \code{"codeplan"}.}
  \item{value}{an object as it would be returned by \code{codeplan(x)}
    or \code{NULL}.}
  \item{filename}{a character string, the name of the file that is to be
                  read or to be written.}
  \item{type}{a character string (either "yaml" or "json") oder NULL (the default), gives the type
              of the file into which the codeplan is written or from
              which it is read.
              If \code{type} is NULL then the file type is inferred from
              the file name ending (".yaml" or ",yml" for "yaml",
              ".json" for "json").
            }
  \item{pretty}{a logical value, whether the JSON output created by \code{write_codeplan(...)} should be prettified.}
}
\value{If applicable, \code{codeplan} returns a list with
  additional S3 class attribute \code{"codeplan"}. For arguments for
  which the relevant information does not exist, the function returns \code{NULL}.

  The list has at least one element or several elements, named after the
  variable in the "item.list" or "data.set" \code{x}. Each list element
  is a list itself with the following elements:

  \item{\code{annotation}}{a named character vector,}
  \item{\code{labels}}{a named list of labels and labelled values}
  \item{\code{value.filter}}{a list with at least two elements named
      "class" and "filter", and optionally another element named
      "range". The "class" element determines the class of the
      value filter and equals either "missing.values", "valid.values",
      or "valid.range". An element named "range" may only be needed
      if "class" is "missing.values", as it is possible (like in SPSS)
      to have \emph{both} individual missing values and a range of
      missing values.
  }
  \item{\code{mode}}{a character string that describes storage mode, such as \code{"character"},
        \code{"integer"}, or \code{"numeric"}.}
  \item{\code{measurement}}{a character string with the measurement level,
        \code{"nominal"}, \code{"ordinal"}, \code{"interval"}, or
        \code{"ratio"}.}
      
  If \code{codeplan(x)<-value} or \code{setCodeplan(x,value)} is used
  and \code{value} is \code{NULL}, all the special information about
  annotation, labels, value filters, etc. is removed from the resulting
  object, which then is usually a mere atomic vector or data frame.
}
\examples{
Data1 <- data.set(
          vote = sample(c(1,2,3,8,9,97,99),size=300,replace=TRUE),
          region = sample(c(rep(1,3),rep(2,2),3,99),size=300,replace=TRUE),
          income = exp(rnorm(300,sd=.7))*2000
          )

Data1 <- within(Data1,{
  description(vote) <- "Vote intention"
  description(region) <- "Region of residence"
  description(income) <- "Household income"
  foreach(x=c(vote,region),{
    measurement(x) <- "nominal"
    })
  measurement(income) <- "ratio"
  labels(vote) <- c(
                    Conservatives         =  1,
                    Labour                =  2,
                    "Liberal Democrats"   =  3,
                    "Don't know"          =  8,
                    "Answer refused"      =  9,
                    "Not applicable"      = 97,
                    "Not asked in survey" = 99)
  labels(region) <- c(
                    England               =  1,
                    Scotland              =  2,
                    Wales                 =  3,
                    "Not applicable"      = 97,
                    "Not asked in survey" = 99)
  foreach(x=c(vote,region,income),{
    annotation(x)["Remark"] <- "This is not a real survey item, of course ..."
    })
  missing.values(vote) <- c(8,9,97,99)
  missing.values(region) <- c(97,99)
})
cpData1 <- codeplan(Data1)

Data2 <- data.frame(
          vote = sample(c(1,2,3,8,9,97,99),size=300,replace=TRUE),
          region = sample(c(rep(1,3),rep(2,2),3,99),size=300,replace=TRUE),
          income = exp(rnorm(300,sd=.7))*2000
          )
codeplan(Data2) <- cpData1
codeplan(Data2)
codebook(Data2)

# Note the difference between 'as.data.frame' and setting
# the codeplan to NULL:
Data2df <- as.data.frame(Data2)
codeplan(Data2) <- NULL
str(Data2)
str(Data2df)
codeplan(Data2) <- NULL # Does not change anything

# Codeplans of survey items can also be inquired and manipulated:
vote <- Data1$vote
str(vote)
cp.vote <- codeplan(vote)
codeplan(vote) <- NULL
str(vote)
codeplan(vote) <- cp.vote
vote

fn.json <- paste0(tempfile(),".json")
write_codeplan(codeplan(Data1),filename=fn.json)
codeplan(Data2) <- read_codeplan(fn.json)
codeplan(Data2)
}

\name{collect}
\alias{collect}
\alias{collect.default}
\alias{collect.array}
\alias{collect.matrix}
\alias{collect.table}
\alias{collect.data.frame}
\alias{collect.data.set}
\title{Collect Objects}
\description{
  \code{collect} gathers several objects into one, matching the
  elements or subsets of the objects by \code{\link{names}} or \code{\link{dimnames}}.
}
\usage{
collect(\dots,names=NULL,inclusive=TRUE)
\method{collect}{default}(\dots,names=NULL,inclusive=TRUE)
\method{collect}{array}(\dots,names=NULL,inclusive=TRUE)
\method{collect}{matrix}(\dots,names=NULL,inclusive=TRUE)
\method{collect}{table}(\dots,names=NULL,sourcename=".origin",fill=0)
\method{collect}{data.frame}(\dots,names=NULL,inclusive=TRUE,
                                  fussy=FALSE,warn=TRUE,
                                  detailed.warnings=FALSE,use.last=FALSE,
                                  sourcename=".origin")
\method{collect}{data.set}(\dots,names=NULL,inclusive=TRUE,
                                  fussy=FALSE,warn=TRUE,
                                  detailed.warnings=FALSE,use.last=FALSE,
                                  sourcename=".origin")
}
\arguments{
  \item{\dots}{more atomic vectors, arrays, matrices, tables, data.frames or data.sets}
  \item{names}{optional character vector; in case of the default and array methods,
    giving \code{\link{dimnames}} for the new dimension that identifies the
    collected objects; in case of the data.frame and data.set methods,
    levels of a factor indentifying the collected objects.
    }
  \item{inclusive}{logical, defaults to TRUE; should unmatched elements included? See details below.}
  \item{fussy}{logical, defaults to FALSE; should it count as an error, if variables with same
    names of collected data.frames/data.sets have different attributes?}
  \item{warn}{logical, defaults to TRUE; should an warning be given, if variables with same
    names of collected data.frames/data.sets have different attributes?}
  \item{detailed.warnings}{logical, whether the attributes of each
    variable should be printed if they differ, and if \code{warn} or
    \code{fuzzy} is TRUE.}
  \item{use.last}{logical, defaults to FALSE. If the function is
    applied to data frames or similar objects, attributes of variables
    may differ between data frames (or other objects, respectively). If
    this argument is TRUE, then the attributes are harmonised based on
    the variables in the last data frame/object, otherwise the
    attributes of variables in the first data frame/object are used for harmonisation.
  }
  \item{sourcename}{name of the factor that identifies the collected data.frames or data.sets}
  \item{fill}{numeric; with what to fill empty table cells, defaults to zero, assuming
    the table contains counts}
}
\value{
  If \code{x} and all following \dots arguments are vectors of the same mode (numeric,character, or logical)
  the result is a matrix with as many columns as vectors. If argument \code{inclusive} is TRUE,
  then the number of rows equals the number of names that appear at least once in each of the
  vector names and the matrix is filled with \code{NA} where necessary,
  otherwise the number of rows equals the number of names that are present in \emph{all}
  vector names.

  If \code{x} and all \dots arguments are matrices or arrays of the same mode (numeric,character, or logical)
  and \eqn{n} dimension the result will be a \eqn{n+1} dimensional array or table. The extend of the
  \eqn{n+1}th dimension equals the number of matrix, array or table arguments,
  the extends of the lower dimension depends on the \code{inclusive} argument:
  either they equal to the number of dimnames that appear at least once for each given
  dimension and the array is filled with \code{NA} where necessary,
  or they equal to the number of dimnames that appear in all arguments
  for each given dimension.

  If \code{x} and all \dots arguments are data frames or data sets, the
  result is a data frame or data set.
  The number of variables of the resulting data frame or data set depends on
  the \code{inclusive} argument. If it is true, the number of variables
  equals the number of variables that appear in each of the arguments at least once
  and variables are filled with \code{NA} where necessary, otherwise the
  number of variables equals the number of variables that are present in
  all arguments.
}
\examples{
x <- c(a=1,b=2)
y <- c(a=10,c=30)

x
y

collect(x,y)
collect(x,y,inclusive=FALSE)

X <- matrix(1,nrow=2,ncol=2,dimnames=list(letters[1:2],LETTERS[1:2]))
Y <- matrix(2,nrow=3,ncol=2,dimnames=list(letters[1:3],LETTERS[1:2]))
Z <- matrix(3,nrow=2,ncol=3,dimnames=list(letters[1:2],LETTERS[1:3]))

X
Y
Z

collect(X,Y,Z)
collect(X,Y,Z,inclusive=FALSE)

X <- matrix(1,nrow=2,ncol=2,dimnames=list(a=letters[1:2],b=LETTERS[1:2]))
Y <- matrix(2,nrow=3,ncol=2,dimnames=list(a=letters[1:3],c=LETTERS[1:2]))
Z <- matrix(3,nrow=2,ncol=3,dimnames=list(a=letters[1:2],c=LETTERS[1:3]))

collect(X,Y,Z)
collect(X,Y,Z,inclusive=FALSE)

df1 <- data.frame(a=rep(1,5),b=rep(1,5))
df2 <- data.frame(a=rep(2,5),b=rep(2,5),c=rep(2,5))
collect(df1,df2)
collect(df1,df2,inclusive=FALSE)

data(UCBAdmissions)
Male <- as.table(UCBAdmissions[,1,])
Female <- as.table(UCBAdmissions[,2,])
collect(Male,Female,sourcename="Gender")
collect(unclass(Male),unclass(Female))

Male1 <- as.table(UCBAdmissions[,1,-1])
Female2 <- as.table(UCBAdmissions[,2,-2])
Female3 <- as.table(UCBAdmissions[,2,-3])
collect(Male=Male1,Female=Female2,sourcename="Gender")
collect(Male=Male1,Female=Female3,sourcename="Gender")
collect(Male=Male1,Female=Female3,sourcename="Gender",fill=NA)

f1 <- gl(3,5,labels=letters[1:3])
f2 <- gl(3,6,labels=letters[1:3])
collect(f1=table(f1),f2=table(f2))

ds1 <- data.set(x = 1:3)
ds2 <- data.set(x = 4:9,
                y = 1:6)
collect(ds1,ds2)

}
\keyword{utilities}
\keyword{manip}

\name{within-operators}
\alias{Within}
\alias{Within.data.frame}
\alias{\%$\%}
\alias{\%$$\%}
\title{Operators to abbreviate use of "with" and "within"}
\description{The operators \code{\%$\%} and \code{\%$$\%} provide
  abbrevitions for calls to \code{with()} and \code{within()}
  respectively.
  The function \code{Within()} is a variant of \code{with()} were
  the resulting data frame contains any newly created variables in the
  order in which they are created (and not in the reverse order).
}
\usage{
data \%$\% expr
data \%$$\% expr
Within(data,expr,\dots)
\method{Within}{data.frame}(data,expr,\dots)
}
\arguments{
  \item{data}{a data frame or similar object, see \code{\link{with}} and \code{\link{within}}}
  \item{expr}{a single or compound expression (i.e. several expressions
  enclosed in curly braces), see \code{\link{with}} and
  \code{\link{within}}}
  \item{\dots}{Further arguments, currently ignored}
}
\seealso{\code{\link{with}} and \code{\link{within}} in package "base".} 
\examples{
df <- data.frame(a = 1:7,
                 b = 7:1)

df

df <- within(df,{
  ab <- a + b
  a2b2 <- a^2 + b^2
})
df

df <- data.frame(a = 1:7,
                 b = 7:1)
df <- Within(df,{
  ab <- a + b
  a2b2 <- a^2 + b^2
})
df


df <- data.frame(a = 1:7,
                 b = 7:1)
df

ds <- as.data.set(df)
ds

df \%$$\% {
  ab <- a + b
  a2b2 <- a^2 + b^2
}
df

ds \%$$\% {
  ab <- a + b
  a2b2 <- a^2 + b^2
}
ds

df \%$\% c(a.ssq = sum(a^2),
         b.ssq = sum(b^2))
}

\name{value.filter}
\alias{value.filter}
\alias{value.filter-class}
\alias{value.filter,NULL-method}
\alias{value.filter,item-method}
\alias{missing.values-class}
\alias{valid.values-class}
\alias{valid.range-class}
\alias{missing.values}
\alias{missing.values,NULL-method}
\alias{missing.values,item.vector-method}
\alias{missing.values<-}
\alias{missing.values<-,ANY,NULL-method}
\alias{missing.values<-,ANY,atomic-method}
\alias{missing.values<-,ANY,list-method}
\alias{missing.values<-,item,NULL-method}
\alias{missing.values<-,item,ANY-method}
\alias{missing.values<-,item,missing.values-method}
\alias{missing.values<-,atomic,missing.values-method}
\alias{valid.values}
\alias{valid.values,NULL-method}
\alias{valid.values,item.vector-method}
\alias{valid.values<-}
\alias{valid.values<-,ANY,NULL-method}
\alias{valid.values<-,ANY,atomic-method}
\alias{valid.values<-,item,valid.values-method}
\alias{valid.values<-,atomic,valid.values-method}
\alias{valid.range}
\alias{valid.range,NULL-method}
\alias{valid.range,item.vector-method}
\alias{valid.range<-}
\alias{valid.range<-,ANY,NULL-method}
\alias{valid.range<-,ANY,atomic-method}
\alias{valid.range<-,item,valid.range-method}
\alias{valid.range<-,atomic,valid.range-method}
\alias{format,valid.values-method}
\alias{format,valid.range-method}
\alias{format,missing.values-method}
\alias{show,value.filter-method}
\alias{coerce,atomic,valid.values-method}
\alias{coerce,atomic,valid.range-method}
\alias{coerce,atomic,missing.values-method}
\alias{coerce,list,missing.values-method}
\alias{Arith,valid.values,valid.values-method}
\alias{Arith,missing.values,missing.values-method}
\alias{Arith,valid.range,valid.range-method}
\alias{Arith,value.filter,vector-method}
\alias{is.valid}
\alias{is.missing}
\alias{is.missing,item.vector-method}
\alias{is.missing,NULL-method}
\alias{is.missing,atomic-method}
\alias{is.missing,factor-method}
\alias{include.missings}
\alias{include.missings,item-method}
\alias{nvalid}
\title{Value Filters}
\description{
  Value filters, that is objects that inherit from class "value.filter",
  are a mechanism to distinguish between valid codes of a survey
  item and codes that are considered to be missing, such as
  the codes for answers like "don't know" or "answer refused".

  Value filters are optional slot values of "item" objects.
  They determine which codes of "item" objects are
  replaced by \code{NA} when they are coerced into
  a vector or a factor.

  There are three (sub)classes of value filters:
  "missing.values", which specify individual
  missing values and/or a range of missing values;
  "valid.values", which specify individual
  valid values (that is, all other values of the
  item are considered as missing);
  "valid.range", which specify a range of
  valid values (that is, all values outside the range
  are considered as missing).
  Value filters of class "missing.values" correspond
  to missing-values declarations in SPSS files,
    imported by \code{\link{spss.fixed.file}},
  \code{\link{spss.portable.file}}, or
  \code{\link{spss.system.file}}.

  Value filters also can be updated using the  \code{+}
  and \code{-} operators.
}
\usage{
value.filter(x)

missing.values(x)
missing.values(x)<-value

valid.values(x)
valid.values(x)<-value

valid.range(x)
valid.range(x)<-value

is.valid(x)
nvalid(x)
is.missing(x)
include.missings(x,mark="*")
}
\arguments{
  \item{x,value}{objects of the appropriate class.}
  \item{mark}{a character string, used to pasted
  to value labels of \code{x} (if present).}
}
\value{
  \code{value.filter(x)}, \code{missing.values(x)}, \code{valid.values(x)}, and \code{valid.range(x)},
  return the value filter associated with \code{x}, an
  object of class "value.filter", that is, of class
  "missing.values", "valid.values", or "valid.range", respectively.

  \code{is.missing(x)} returns a logical vector indicating for
  each element of \code{x} whether it is a missing value or not.
  \code{is.valid(x)} returns a logical vector indicating for
  each element of \code{x} whether it is a valid value or not.
  \code{nvalid(x)} returns the number of elements of \code{x}
  that are valid.
  
  For convenience, \code{is.missing(x)} and \code{is.valid(x)} also work
  for atomic vectors and factors, where they are equivalent to
  \code{is.na(x)} and \code{!is.na(x)}. For atomic vectors and factors,
  \code{nvalid(x)} returns the number of elements of \code{x} for
  which \code{!is.na(x)} is TRUE.

  \code{include.missings(x,...)} returns a copy of \code{x}
  that has all values declared as valid.
}
\examples{
x <- rep(c(1:4,8,9),2,length=60)
labels(x) <- c(
    a=1,
    b=2,
    c=3,
    d=4,
    dk=8,
    refused=9
    )
missing.values(x) <- 9
missing.values(x)
missing.values(x) <- missing.values(x) + 8
missing.values(x)
missing.values(x) <- NULL
missing.values(x)
missing.values(x) <- list(range=c(8,Inf))
missing.values(x)
valid.values(x)
print(x)
is.missing(x)
is.valid(x)
as.factor(x)
as.factor(include.missings(x))
as.integer(x)
as.integer(include.missings(x))
}

\name{mtable_format_latex}
\alias{mtable_format_latex}
\title{Format 'mtable' Results for LaTeX}
\description{
  This function formats objects created by \code{\link{mtable}} for inclusion
  into LaTeX files.
  }
\usage{
  mtable_format_latex(x,
            useDcolumn=getOption("useDcolumn",TRUE),
            colspec=if(useDcolumn) 
                      paste("D{.}{",LaTeXdec,"}{",ddigits,"}",sep="") 
                    else "l",
            LaTeXdec=".",
            ddigits=min(3,getOption("digits")),
            useBooktabs=getOption("useBooktabs",TRUE),
            toprule=if(useBooktabs) "\\\\toprule" else "\\\\hline\\\\hline",
            midrule=if(useBooktabs) "\\\\midrule" else "\\\\hline",
            cmidrule=if(useBooktabs) "\\\\cmidrule" else "\\\\cline",
            bottomrule=if(useBooktabs) "\\\\bottomrule" else "\\\\hline\\\\hline",
            interaction.sep = " $\\\\times$ ",
            sdigits=min(1,ddigits),
            compact=FALSE,
            sumry.multicol=FALSE,
            escape.tex=getOption("toLatex.escape.tex",FALSE),
            signif.notes.type=getOption("toLatex.signif.notes.type","include"),
            signif.notes.spec=getOption("toLatex.signif.notes.spec","p{.5\\\\linewidth}"),
            \dots
  )
}
\arguments{
     \item{x}{an object of class \code{mtable}}
     \item{useDcolumn}{should the \code{dcolumn} LaTeX package be used?
          If true, you will have to include \code{\\usepackage{dcolumn}} into
          the preamble of your LaTeX document.}
     \item{colspec}{LaTeX table column format specifyer(s).}
     \item{LaTeXdec}{the decimal point in the final LaTeX output.}
     \item{ddigits}{alignment specification or digits after the decimal point.}
     \item{useBooktabs}{should the \code{booktabs} LaTeX package be used?
          If true, you will have to include \code{\\usepackage{booktabs}} into
          the preamble of your LaTeX document.}
     \item{toprule}{appearance of the top border of the LaTeX \code{tabular} environment.}
     \item{midrule}{how are coefficients and summary statistics
        separated in the LaTeX \code{tabular} environment.}
     \item{cmidrule}{appearance of rules under section headings.}
     \item{bottomrule}{appearance of the bottom border of the LaTeX \code{tabular} environment.}
     \item{interaction.sep}{a character string that separates factors that are involved
        in an interaction effect}
     \item{sdigits}{integer; number of digits after decimal dot for summary statistics.}
    \item{compact}{logical; should the table be compact, without extra columns
    between multi-equation models?}
    \item{sumry.multicol}{logical, should summaries enclosed into
      \code{\\multicol} commands?}
		\item{escape.tex}{logical, should symbols \code{$}, \code{_}, and \code{^} be
			escaped with backslashes?}
    \item{signif.notes.type}{character string; should be either
      \code{"include"},\code{"append"}, \code{"drop"},  or \code{"tnotes"}. If
      \code{"append"}, (very simple) LaTeX code is appended that contains
      notes that relate significance symbols to p-values. If
      \code{"include"}, the LaTeX table will include a (multi-column)
      cell with these notes. If \code{"drop"}, notes will not be added.
      If \code{"tnotes"}, the exported LaTeX
      table is wrapped in a \code{threeparttable} environment and the
      p-value notes are wrapped in a \code{tablenotes} environment. This
      requires the LaTeX package \code{threeparttable} in order to work.
    }
    \item{signif.notes.spec}{character string; specifies format
      of cells that include notes about p-values; relevant only if
      \code{signif.notes.type="include"}
    }
    \item{\dots}{further arguments, ignored. }
}
\value{
    A character string with code suitable for inclusion into a LaTeX-file.
}
\keyword{misc}
